/* vi: set sw=4 ts=4: */
/*
 * Mini syslogd implementation for busybox
 *
 * Copyright (C) 1999,2000 by Lineo, inc. and Erik Andersen
 * Copyright (C) 1999,2000,2001 by Erik Andersen <andersee@debian.org>
 *
 * Copyright (C) 2000 by Karl M. Hegbloom <karlheg@debian.org>
 *
 * "circular buffer" Copyright (C) 2001 by Gennady Feldman <gfeldman@cachier.com>
 *
 * Maintainer: Gennady Feldman <gena01@cachier.com> as of Mar 12, 2001
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <paths.h>
#include <signal.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/un.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/sysinfo.h>
#include "sysklogd.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
/* SYSLOG_NAMES defined to pull some extra junk from syslog.h */
#define SYSLOG_NAMES
#include <sys/syslog.h>
#include <sys/uio.h>
#include "../nvram/nvram.h" 
#include "../time_zone/time_zone.h"
/* Path for the file where all log messages are written */
#define __LOG_FILE "/var/log/messages"
#define __CONF_FILE "/etc/syslog.conf"

#define __LOG_FULL 21504-1024 //21k is the size we will show on our UI page
/* Path to the unix socket */
static char lfile[MAXPATHLEN]="";

static char *logFilePath = __LOG_FILE;

#define dprintf(msg,...) 
struct syslog_conf conf;

#define ALERT_MAX_INTERVAL 3*60
static time_t last_send_mail=0;

/* interval between marks in seconds */
static int MarkInterval = 10 * 60;


//#define DEBUG
#ifdef DEBUG
#include <stdarg.h>
int mylog(const char *format, ...)
{
    char buff[1024]="";
    va_list arg;
    FILE *fp;
    va_start(arg, format);
    vsnprintf(buff,1024, format, arg);
    va_end(arg);
    fp=fopen("/tmp/startlogd.log","a+");
    if(fp==NULL)
        return 1;
    fwrite(buff,1024,1,fp);
    fclose(fp);
    usleep(1);
    return 0;
}
#else
int mylog(const char *format, ...)
{
	return 0;
}
#endif



#ifdef SHOW_HOSTNAME
/* localhost's name */
static char LocalHostName[64]="";
#endif

static char last_log[1024]="";

enum{
    NEEDNOT_DST,    	/*need not daylight saving time*/
    NA_DST,         	/*North America need DST*/
    EU_DST,         	/*Europe DST*/
    CHILE_DST,		/*Chile dst*/
    SA_DST,		/*sourth america,for example: Brazil*/
    IRAQ_DST,		/*raq and Iran */
    AU2_DST,		/*Australia - Tasmania*/
    AU3_DST,		/*New Zealand, Chatham */
    AF_DST,		/*Egypt*/
    AU_DST         	/*Australia DST*/
};

/* Add by Barry 7.24,2006 for debug */
//#define DEBUG_LOG
#ifdef DEBUG_LOG
void vr_print(const char *fmt, ...)
{	
	va_list arg;
	va_start (arg, fmt);
	vprintf (fmt, arg);
	va_end (arg);
}
#else
void vr_print(const char *fmt, ...){}
#endif

void logMessage (int pri, char *msg);
int daylight_saving(void);
int parse_config(char *conf_path);

char *config_file_path;

#ifdef BB_FEATURE_REMOTE_LOG
#include <netinet/in.h>
/* udp socket for logging to remote host */
static int remotefd = -1;
/* where do we log? */
static char *RemoteHost=NULL;
/* what port to log to? */
static int RemotePort = 514;
/* To remote log or not to remote log, that is the question. */
static int doRemoteLog = FALSE;
static int local_logging = FALSE;
#endif

static int email_send_threshold = 0;
#define MAXLINE         1024            /* maximum line length */

/*raul add this fot semaphore */
#if 1
#include <sys/sem.h>
#include <sys/shm.h>

/* our shared key */
static const long KEY_ID = 0x414e4547; /*"GENA"*/

// Semaphore operation structures
static struct shbuf_ds {
	int size;               // size of data written
	int head;               // start of message list
	int tail;               // end of message list
	/* can't use char *data */
	char data[1];           // data/messages 
} *buf = NULL;                  // shared memory pointer

static struct sembuf SMwup[1] = {{1, -1, IPC_NOWAIT}}; // set SMwup
static struct sembuf SMwdn[3] = {{0, 0}, {1, 0}, {1, +1}}; // set SMwdn

static int      shmid = -1;     // ipc shared memory id
static int      s_semid = -1;   // ipc semaphore id
int     data_size = 16000; // data size
int     shm_size = 16000 + sizeof(*buf); // our buffer size
#endif
/* circular buffer variables/structures */
#ifdef BB_FEATURE_IPC_SYSLOG
#if __GNU_LIBRARY__ < 5
#error Sorry.  Looks like you are using libc5.
#error libc5 shm support isnt good enough.
#error Please disable BB_FEATURE_IPC_SYSLOG
#endif

#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>

/* our shared key */
static const long KEY_ID = 0x414e4547; /*"GENA"*/

// Semaphore operation structures
static struct shbuf_ds {
	int size;               // size of data written
	int head;               // start of message list
	int tail;               // end of message list
	/* can't use char *data */
	char data[1];           // data/messages 
} *buf = NULL;                  // shared memory pointer

static struct sembuf SMwup[1] = {{1, -1, IPC_NOWAIT}}; // set SMwup
static struct sembuf SMwdn[3] = {{0, 0}, {1, 0}, {1, +1}}; // set SMwdn

static int      shmid = -1;     // ipc shared memory id
static int      s_semid = -1;   // ipc semaphore id
int     data_size = 16000; // data size
int     shm_size = 16000 + sizeof(*buf); // our buffer size
static int circular_logging = TRUE;

enum{
    NEEDNOTDST,
    NEEDDST    
};

/* Ron */
static void reload_signal(int sig)
{
	parse_config(config_file_path);
}

static void clear_signal(int sig)
{
	buf->head=0;
	buf->tail=0;
}

/*
 * sem_up - up()'s a semaphore.
 */
static inline void sem_up(int semid)
{
	if ( semop(semid, SMwup, 1) == -1 )
		perror_msg_and_die("semop[SMwup]");
}

/*
 * sem_down - down()'s a semaphore
 */
static inline void sem_down(int semid)
{
	if ( semop(semid, SMwdn, 3) == -1 )
		perror_msg_and_die("semop[SMwdn]");
}


void ipcsyslog_cleanup(void){
	dprintf("Exiting Syslogd!\n");
	if (shmid != -1)
		shmdt(buf);

	if (shmid != -1)
		shmctl(shmid, IPC_RMID, NULL);
	if (s_semid != -1)
		semctl(s_semid, 0, IPC_RMID, 0);
}

void ipcsyslog_init(void){
	if (buf == NULL){
	    if ((shmid = shmget(KEY_ID, shm_size, IPC_CREAT | 1023)) == -1)
			perror_msg_and_die("shmget");


	    if ((buf = shmat(shmid, NULL, 0)) == -1)
			perror_msg_and_die("shmat");


	    buf->size=data_size;
	    buf->head=buf->tail=0;

	    // we'll trust the OS to set initial semval to 0 (let's hope)
	    if ((s_semid = semget(KEY_ID, 2, IPC_CREAT | IPC_EXCL | 1023)) == -1){
		if (errno == EEXIST){
		   if ((s_semid = semget(KEY_ID, 2, 0)) == -1)
		    perror_msg_and_die("semget");
		}else
			perror_msg_and_die("semget");
	    }
	}else{
		dprintf("Buffer already allocated just grab the semaphore?");
	}
}

//raul change sendmail to support auth
static void send_mail_signal(int sig)
{
	sem_down(s_semid);
	
	if(conf.mail_enable==1){
		char cmd[1024];
		if(conf.mail_enable_auth==1)
		sprintf(cmd,"/usr/sbin/smtpc -m -h %s -r %s -s \"%s\" -U %s -P %s </var/log/messages "
				,conf.mail_server
				,conf.mail_receiver
				,conf.mail_subject
				,conf.mail_login
				,conf.mail_password);
		else
		sprintf(cmd,"/usr/sbin/smtpc -m -h %s -r %s -s \"%s\" </var/log/messages "
				,conf.mail_server
				,conf.mail_receiver
				,conf.mail_subject);

		if(system(cmd)==0){
			buf->head=0;
			buf->tail=0;
		}
	}

	sem_up(s_semid);
}
/* write message to buffer */
void circ_message(const char *msg){
	int l=strlen(msg); /* count the whole message w/ '\0' included */
	
	sem_down(s_semid);
	
	if ( (buf->tail + l) < buf->size ){
		if ( buf->tail < buf->head){
			if ( (buf->tail + l) >= buf->head ){
				int k= buf->tail + l - buf->head; 
				char *c=memchr(buf->data+buf->head + k,'\n',buf->size - (buf->head + k));
			   	buf->head=(c != NULL)?( c - buf->data + 1):0;
			     	
			}
		} 
		strncpy(buf->data + buf->tail,msg,l); /* append our message */
		buf->tail+=l; 	
	}else{
		char *c;
		int k=buf->tail + l - buf->size; 
		
		c=memchr(buf->data + k ,'\n', buf->size - k);

		if (c != NULL) {
				buf->head=c-buf->data+1;
				strncpy(buf->data + buf->tail, msg, l - k - 1);
				strcpy(buf->data, &msg[l-k-1]);
				buf->tail = k + 1;
		}else{
				buf->head = buf->tail = 0; 	
		}

	}
	sem_up(s_semid);
}
#endif  /* BB_FEATURE_IPC_SYSLOG */

/*
 * sem_up - up()'s a semaphore.
 */
static inline void sem_up(int semid)
{
	if ( semop(semid, SMwup, 1) == -1 )
		perror_msg_and_die("semop[SMwup]");
}

/*
 * sem_down - down()'s a semaphore
 */
static inline void sem_down(int semid)
{
	if ( semop(semid, SMwdn, 3) == -1 )
		perror_msg_and_die("semop[SMwdn]");
}

static void send_mail_signal(int sig)
{

	struct stat   stat_buf;

		if(stat("/var/log/messages_mail",&stat_buf)<0)
		{
    sem_down(s_semid);		
    if (conf.mail_enable == 1)
    {
					
    		system("/bin/cp -f /var/log/messages /var/log/messages_mail");

        char cmd[1024];
		if(conf.mail_enable_auth==1)
		sprintf(cmd,"/usr/sbin/smtpc -m -h %s -r %s -f %s -s \"%s\" -U %s -P %s </var/log/messages_mail &"
				,conf.mail_server
				,conf.mail_receiver
				,conf.mail_sender
				,conf.mail_subject
				,conf.mail_login
				,conf.mail_password);
		else		
        sprintf(cmd, "/usr/sbin/smtpc -m -h %s -r %s -f %s -s \"%s\" </var/log/messages_mail &"
                , conf.mail_server
                , conf.mail_receiver
                , conf.mail_sender
//                , conf.mail_server //we now use the mail server as the mail sender
                , conf.mail_subject);

        if (system(cmd) == 0)
        {
        	system("/bin/echo === Former logs mailed ! === > /var/log/messages");
        }
    	
		}
    sem_up(s_semid);
  } 
      		
}

/* try to open up the specified device */
int device_open(char *device, int mode)
{
	int m, f, fd = -1;

	m = mode | O_NONBLOCK;

	/* Retry up to 5 times */
	for (f = 0; f < 5; f++)
		if ((fd = open(device, m, 0600)) >= 0)
			break;
	if (fd < 0)
		return fd;
	/* Reset original flags. */
	if (m != mode)
		fcntl(fd, F_SETFL, mode);
	return fd;
}
int vdprintf(int d, const char *format, va_list ap)
{
	char buff[BUF_SIZE];
	int len;

	len = vsnprintf(buff, sizeof(buff), format, ap);
	return write(d, buff, len);
}

//change the return unit of time_adjust to minute tu support time zone such as (GMT+09:30)
int time_adjust(char *tz)
{
    int time=0;
    char a=tz[4];
    char b=tz[5];

    if('0' == a)
        return 0;
    if('0' <= b && '9' >= b) {
        time = (10*(a-'0')+(b-'0'))*60;
        if('0' == tz[7])
            time += 30;
    } else {
        if('0' <= a && '9' >= a)
            time = (a-'0')*60;
        if('0' == tz[6])
            time += 30;
    }

    if('+' == tz[3])
        return 0-time;
    else
        return time;
}

/* Note: There is also a function called "message()" in init.c */
/* Print a message to the log file. */

static int log_is_full=0;

void check_is_full()
{
		struct stat   stat_buf;
		if(stat("/var/log/messages",&stat_buf)<0)
			{
				return ;
				}		
		if(stat_buf.st_size > __LOG_FULL)
			{
				if(conf.mail_log_full==1)
					{
					send_mail_signal(0);
					log_is_full=0;
						}
				else
					log_is_full=1;	
				}
		else
					log_is_full=0;
		return 	;		
	}

static void message (char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
static void message (char *fmt, ...)
{
	int fd;
	struct flock fl;
	va_list arguments;

	fl.l_whence = SEEK_SET;
	fl.l_start  = 0;
	fl.l_len    = 1;

	/**** check whether the log is full****/
	check_is_full();
	if(log_is_full==1)
		return ;
#ifdef BB_FEATURE_IPC_SYSLOG
	if ((circular_logging == TRUE) && (buf != NULL)){
			char b[1024];
			va_start (arguments, fmt);
			vsnprintf (b, sizeof(b)-1, fmt, arguments);
			va_end (arguments);
			circ_message(b);
#ifdef DEBUG
			printf("head=%d tail=%d\n",buf->head,buf->tail);
#endif
/* print_circ_buf */			
			if((fd=open(logFilePath,O_WRONLY| O_CREAT|O_TRUNC|O_NONBLOCK))<0)
				return;	
			fl.l_type = F_WRLCK;
			fcntl(fd, F_SETLKW, &fl);
			if(buf->tail > buf->head){
				write(fd,buf->data,buf->tail);
				write(fd,"\0",1);
			}else {
					
				write(fd,buf->data+buf->head,buf->size-buf->head-1);
				write(fd,buf->data,buf->tail);
				write(fd,"\0",1);

				if(conf.mail_log_full==1)
				send_mail_signal(0);
#if 0
				char tmp[buf->size+1];
				char *pt=tmp;
				/* we don't +1 because last bit is '\0'*/
				memcpy(pt,buf->data+buf->head,buf->size-buf->head);			
				strncat(pt+buf->size-buf->head-1,buf->data,buf->tail);
				write(fd,tmp,data_size-buf->head+buf->tail);
				
				if(conf.mail_enable==1 && conf.mail_log_full==1){
					if(send_mail(conf.mail_server
							,conf.mail_sender
							,conf.mail_receiver
							,conf.mail_subject
							,tmp)==TRUE){
						buf->head=0;
						buf->tail=0;
					}
				}
#endif
			}
			fl.l_type = F_UNLCK;
			fcntl(fd, F_SETLKW, &fl);
			close(fd);
	}else
#endif
	if ((fd = device_open (logFilePath,
						   O_WRONLY | O_CREAT | O_NOCTTY | O_APPEND |
						   O_NONBLOCK)) >= 0) {
		fl.l_type = F_WRLCK;
		fcntl (fd, F_SETLKW, &fl);
		va_start (arguments, fmt);
		vdprintf (fd, fmt, arguments);
		va_end (arguments);
		fl.l_type = F_UNLCK;
		fcntl (fd, F_SETLKW, &fl);
		close (fd);
	} else {
		/* Always send console messages to /dev/console so people will see them. */
		if ((fd = device_open (_PATH_CONSOLE,
							   O_WRONLY | O_NOCTTY | O_NONBLOCK)) >= 0) {
			va_start (arguments, fmt);
			vdprintf (fd, fmt, arguments);
			va_end (arguments);
			close (fd);
		} else {
			fprintf (stderr, "Bummer, can't print: ");
			va_start (arguments, fmt);
			vfprintf (stderr, fmt, arguments);
			fflush (stderr);
			va_end (arguments);
		}
	}
}


int check_log(int fac)
{
	int i=0;
	while(conf.log_list[i]>=0 && i < 20){
	   if(conf.log_list[i++]==fac)
	   		return TRUE;
	}
	return FALSE;
}
#if 0	
void strccpy(char *dst, char *src,char c)
{
	char *pt=src;
	if(pt==NULL){
		printf("pt==NULL");
		dst[0]='\0';
		return ;
	}
	for(;*pt!=c && *pt!='\0';*dst++=*pt++);
	*dst='\0';

}
#endif
void strccpy2(char *dst, char *src,char *key,char c)
{
	char *pt=strstr(src,key);
	if(pt==NULL){
		dst[0]='\0';
		return ;
	}
	pt+=strlen(key);
	for(;*pt!=c && *pt!='\0';*dst++=*pt++);
	*dst='\0';
}
#if 1
static int someone_in_use=0;  /*1=someone in use, 0=nobody*/
static void strupper(char *str);
static void strupper(char *str)
{
    char *c = str;

    if (!str)
        return ;

    while (*c)
    {
        *c = ( *c >= 'a' && *c <= 'z' ) ? *c - 32 : *c;
        c++;
    }
}

void logMessage (int pri, char *msg)
{
		char res[20] = "";
    CODE *c_fac;
    int fac = 0;
    char *p, *p1, *p2, *pTmp = NULL;
    time_t now;
    char *wday[7] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
    struct tm *st;
		char timestamp[32]=""; 
    char name[20], inif[10], outif[10];
    char srcip[16], dstip[16], srcport[6], dstport[6], protocal[20];
    int sip[4], dip[4];
    int start = 0;
    int end = 0;
    char sendbuf[150];
    char dhcpip[16], dhcpmac[20], LeaOrReq[15];
    char logip[16], getip[16];
    char error_event[10];
    char *poe_mode, *ppp_mode;
    char ppp_ip[16];
		int  send_mail =0;
    if (msg != NULL)
    {
        p = msg;
        p1 = msg;
        p2 = msg;
    }
    else
        return ;

    if (pri != 0)
    {
        for (c_fac = facilitynames;
                c_fac->c_name && !(c_fac->c_val == LOG_FAC(pri) << 3); c_fac++)
            ;
        fac = c_fac->c_val;
    }

#if 0
    /* check this log  */
    if (check_log(fac) == FALSE)
        return ;
#endif /* end if 0 */

    /* if msg had time stamp ,remove it*/
    if (strlen(msg) > 16 && msg[3] == ' ' && msg[6] == ' ' &&
            msg[9] == ':' && msg[12] == ':' && msg[15] == ' ')
        msg += 16;

    /* get current time */
	time(&now);
	if( atoi(conf.daylight)== 1)
			now += daylight_saving_new()*60*60;
		//now += daylight_saving()*60*60;
    	now += time_adjust(conf.TZ)*60;
	st=localtime(&now);
	sprintf(timestamp,"%s, %d-%02d-%02d %02d:%02d:%02d"
			,wday[st->tm_wday]
			,st->tm_year+1900
			,st->tm_mon+1
			,st->tm_mday
			,st->tm_hour
			,st->tm_min
			,st->tm_sec);
			
			

#ifdef __SYSLOG_DEBUG__

    printf("%s,%s\n", msg, timestamp);
    mBUG("%s\n", msg);
#endif

    p1 = strstr(p1, "[");
    p2 = strstr(p2, "]");

    if (p1 == NULL || p2 == NULL)
        return ;

    snprintf(name, p2 - p1, "%s", p1 + 1);

#ifdef __SYSLOG_DEBUG__

    printf("name=%s\n", name);
#endif

		if(((conf.log_syslog!=1)&&strstr(msg,"NTP")) || ((conf.log_auth!=1)&&(strstr(msg,"login")||strstr(msg,"log Out"))))
		return;
/**** check need log ****/
		if((!strstr(conf.log_keyword,name))&&(!strcmp(name,"DOS")||!strcmp(name,"BLOCK")||!strcmp(name,"PORT SCAN")))
			return;
		if(strstr(conf.mail_keyword,name)){
			send_mail=1;
		}
    /* parse system log*/
    if (strcmp(name, "admin login")==0)
    	{
    		if(someone_in_use ==0)
    			{
    		someone_in_use = 1;		
        message("%s %s\r\n", p1,timestamp);  
        }
        return ; 		
    		}
    if (strcmp(name, "admin logout")==0)
    	{
        message("%s %s\r\n", p1,timestamp); 
        someone_in_use = 0; 
        return ; 		
    		}

    if (strcmp(name, "ntp")==0)
    	{
        message("%s %s\r\n", p1,timestamp);  
        return ; 		
    		}
    		    		
    if (strcmp(name, "Unauthorized") == 0)
    {
        //msg format: [Unauthorized] 192.168.2.2 Unauthorized login
        sscanf(p2 + 2, "%[^ ] ", logip);
        message("%s %s Unauthorized login\r\n", logip, timestamp);
        return ;

    }
    else if (strcmp(name, "Dhcp") == 0)
    { //DHCP server message
        //msg format: [DHCP] X.X.X.X XX:XX:XX:XX:XX:XX Request/Release
        sscanf(p2 + 2, "%s %s %s", dhcpip, dhcpmac, LeaOrReq);
        message("[%s] %s %s  %s %s\r\n", name, timestamp, dhcpip, dhcpmac, LeaOrReq);
        return ;

    }
    else if (strcmp(name, "Dhcpc") == 0)
    { // dhcp client message
        //[Dhcpc] X.X.X.X obtain
        sscanf(p2 + 2, "%[^ ] ", getip);
        message("[Wan]:WAN DHCP Client Connected IP %s :  %s \r\n", getip, timestamp);
        return ;
    }
    else if ( strcmp(name, "PPPoE") == 0)
    {
        poe_mode = nvram_get("wan_mode");
        if (strcmp(poe_mode, "pppoe") == 0)
            message("[Wan] %s WAN %s\r\n", p2 + 2, timestamp);

        free(poe_mode);
        return ;
    }
    else if ((strcmp(name, "PPTP") == 0) || (strcmp(name, "BPA") == 0) || (strcmp(name, "L2TP") == 0))
    {
        message("[Wan] %s WAN %s\r\n", timestamp, p2 + 2);
        return ;
    }
    else if (strcmp(name, "PPP") == 0)
    { // connected or disconnected
        ppp_mode = nvram_get("wan_mode");
        strupper(ppp_mode);

        if (strstr(p2, "disconnected") && strcmp(ppp_mode, "L2TP"))
            message("[Wan] %s WAN %s disconnected\r\n", timestamp, ppp_mode);
        else
            message("[Wan] %s WAN %s\r\n", timestamp, p2 + 2);
        
        free(ppp_mode);
        return ;
    }
    else if (strcmp(name, "Error") == 0)
    { // error event message
        message("%s %s\r\n", p2 + 2, timestamp);
        return ;
    }

    /*
     * parser the message dropped by firewall
     */
    //if ((strcmp(name, "BLOCK") == 0) || (strcmp(name , "DOS") == 0)) //raul change to Log not only BLOCK and DOS log
    else
    {
        /* msg format: [Block]IN=br0 OUT=eth1 SRC=192.168.1.100 DST=61.172.201.47
         *            					LEN=48 TOS=0x00 PREC=0x00 TTL=127 ID=4421 DF 
         *								PROTO=TCP SPT=1106 DPT=80 WINDOW=65535 RES=0x00 SYN URGP=0
         *
            * msg format: [Dos]IN=eth0 OUT= MAC=00:c0:02:77:82:32:00:0c:76:23:9e:57:08:00 
        	 *                     SRC=172.21.4.93 DST=172.21.4.87 LEN=48 
        	 *	             	  TOS=0x06 PREC=0x00 TTL=125 ID=54633 PROTO=UDP SPT=7553 DPT=65130 LEN=28 	
        	 * ICMP format:
        	 *              ICMP flood: [Dos] IN=eth0 OUT= MAC=00:c0:02:77:82:32:00:0c:76:23:9e:57:08:00 
        	 *                     SRC=172.21.4.93 DST=172.21.4.87 LEN=48 PROTO=ICMP
        	 *              PingOfDeath: [Dos] PingOfDeath SRC=x.x.x.x DST=x.x.x.x
        	 */
        p1 = strstr(p1, "SRC");
        if (p1)
            sscanf(p1 + 4, "%[^ ] ", srcip);
        else
            return ;

        p1 = strstr(p1, "DST");
        if (p1)
            sscanf(p1 + 4, "%[^ ] ", dstip);
        else
            return ;

        if (strstr(p2, "PingOfDeath"))
        {
            message("[PING OF DEATH] :Ping Of Death attack from %s to %s droped %s\r\n", srcip, dstip, timestamp);
            return ;
        }

        p1 = strstr(p1, "PROTO=");
        if (p1)
            sscanf(p1 + 6, "%[^ ] ", protocal);
        else
            return ;

        if (strcmp(protocal, "ICMP") == 0)
        {
            message("[FLOOD] %s flood From %s To %s droped %s \r\n", protocal, srcip, dstip, timestamp);
            return ;
        }

        p1 = strstr(p1, "SPT");
        if (p1)
            sscanf(p1 + 4, "%[^ ] ", srcport);
        else
            return ;

        p1 = strstr(p1, "DPT");
        if (p1)
            sscanf(p1 + 4, "%[^ ] ", dstport);
        else
            return ;

        if (strcmp(name, "Dos") == 0)
        {
            /* UDP or TCP flood */
            message("[DOS] :%s flood From %s port:%s To %s port:%s droped %s \r\n", protocal, srcip, srcport, dstip, dstport, timestamp);
        }
        else if(strcmp(name, "BLOCK") == 0)
            message("[BLOCK] %s From %s port:%s To %s port:%s  %s \r\n",protocal, srcip, srcport, dstip, dstport, timestamp);
        else /* Block */
            message("[%s] %s From %s port:%s To %s port:%s  %s \r\n",name,protocal, srcip, srcport, dstip, dstport, timestamp);

    }
	/* todo: supress duplicates */
#ifdef BB_FEATURE_REMOTE_LOG
	/* send message to remote logger */
	if ( (-1 != remotefd) && (doRemoteLog==TRUE)){
static const int IOV_COUNT = 2;
		struct iovec iov[IOV_COUNT];
		struct iovec *v = iov;

		memset(&res, 0, sizeof(res));
		snprintf(res, sizeof(res), "<%d>", pri);
		v->iov_base = res ;
		v->iov_len = strlen(res);
		v++;

		v->iov_base = msg;
		v->iov_len = strlen(msg);
writev_retry:
		if ( -1 == writev(remotefd,iov, IOV_COUNT)){
			if (errno == EINTR) goto writev_retry;
			error_msg_and_die("cannot write to remote file handle on"
					"%s:%d",RemoteHost,RemotePort);
		}
	}	
							
//	if (local_logging == TRUE)
#endif    
	if(send_mail == 1)
		{
			sleep(3);
			send_mail_signal(0);
			}

        return ;
}
#else
static int login=0;

void logMessage (int pri, char *msg)
{
	time_t now;
	char *wday[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	struct tm *st;
	char timestamp[32]="";
	char res[20] = "";
	CODE *c_fac;
	int fac=0;
	int send_mail=email_send_threshold;
	int do_log=1;
    	
    	
   mylog("raul debug .... in function logMessage() \n"); 	
    	vr_print("\nLOG = %s", msg);
	if (pri!=0){ 
		for (c_fac = facilitynames;
			c_fac->c_name && !(c_fac->c_val == LOG_FAC(pri) << 3); c_fac++);
		fac=c_fac->c_val;
	}
		
	/* if msg had time stamp ,remove it*/
	if (strlen(msg) > 16 && msg[3] == ' ' && msg[6] == ' ' &&
			msg[9] == ':' && msg[12] == ':' && msg[15] == ' ') 
		msg+=16;

	/* we don't need log from pppd */
	if(strstr(msg,"ppp0 -> "))
		return;
	
#ifdef DEBUG_CALL_TRACE	
	/* log Call Trace for debug */
	if(strstr(msg,"Call Trace")){
		system("/bin/dmesg > /var/log/messages");
		exit(0);
	}
#endif

	// add by john,  format pluto log message
	if (strstr(msg,"pluto")!=NULL){// filter chars before :, add header [VPN log]
		char *msgP=msg;
		while(*msgP!='\0' && *msgP++!=':');  // filter chars before :
		if (strncmp(msgP," |",2)==0)   //  if begin with |, it is debug message,skip it
			return;
		sprintf(msg,"[VPN Log]:%s",msgP);		
		
		goto add_timestamp;
	}
	// end of add by john
			
	while(*msg!='\0' && *msg++!=':');

    /* =======================================================
     * After record [Firewall Log-...] log msg,
     * the follow msg reveive from socket sometimes error,
     * so I drop it here -- Jeff -- Apr.4.2005 --
     * ======================================================= */
    if(strstr(last_log,"[Firewall Log-")!=NULL && strstr(msg,"[Firewall Log-")==NULL
        && ((strstr(msg,"ID=")!=NULL && strstr(msg,"SEQ=")!=NULL)||(strstr(msg,"URGP=")!=NULL)))
        return;

	/* ==============================================================================
	 * NOTICE: -- Jeff add here -- Apr.4.2005 --
	 * 
	 * Description:
	 *      In the previous --  " if(pri!=0){
	 * 		    for (c_fac = facilitynames;
	 *              ......} "
	 *      LOG_FAC(p) defined as (((p) & 0x03F8) >> 3),but we get "pri" there
	 *      is from LOG_EMERG(0) to LOG_DEBUG(7),then the value of LOG_FAC(pri)
	 *      is always ZERO,and "fac" equals ZERO too.
	 *      In the following part of "handle kern message",LOG_LOCAL0 defined
	 *      as (16<<3),so the "fac(0)" will never equals to "LOG_LOCAL0(128)",
	 * 
	 * Result:
	 *      It will never into if(){......}. So the msg which include log_keyword
	 *      or mail_keyword will never be handled to a correct format.
	 *
	 * Solution:
	 *      The msg which include log_keyword or mail_keyword always include '['
	 *      and ']', So we broaden the condition for accepting these msgs.
	 * ============================================================================== */    
	/* handle kern message*/
//	if(fac==LOG_LOCAL0){    
	if(fac==LOG_LOCAL0 || strchr(msg,'[')!=NULL || strchr(msg,']')!=NULL){	
		char proto[8]="",src[16]="",dst[128]="",spt[8]="",dpt[8]="";
		char prefix[32]="";
		char *pt;
		int i;
		
		/* Get prefix:e,g:[Firewall Log-Filter Cookies]*/
		if(((pt=strchr(msg,'['))==NULL) || (strchr(msg,']')==NULL))
			return;
		for(i=0,pt;*pt!=']';prefix[i++]=*pt++);
		prefix[i++]=']';
		prefix[i]='\0';
		vr_print("\nprefix = %s", prefix);
		
		strccpy2(proto,msg,"PROTO=",0x20);
		if(strcmp(proto,"47")==0){
			strcpy(proto,"GRE");
		}else if(strcmp(proto,"50")==0){
			strcpy(proto,"ESP");
		}else if(strcmp(proto,"51")==0){
			strcpy(proto,"AH");
		}else if(strcmp(proto,"115")==0){
			strcpy(proto,"L2TP");
		}
		
		strccpy2(src,msg,"SRC=",0x20);
		strccpy2(dst,msg,"DST=",0x20);
		strccpy2(spt,msg,"SPT=",0x20);
		strccpy2(dpt,msg,"DPT=",0x20);
	
		if(strstr(prefix,"Access Log-")!=NULL){
			struct hostent *host;
			struct in_addr addr;
			addr.s_addr=inet_addr(dst);
			host=gethostbyaddr((char *)&addr,4,AF_INET);
			if(host){
				strcpy(dst,host->h_name);
			}
		}
		pt=msg;
		if(spt[0]!='\0' && strcmp(prefix,"[Firewall Log-PORT SCAN]")!=0)
			sprintf(pt,"%s Packet - Source:%s,%s Destination:%s,%s - %s"
				,proto,src,spt,dst,dpt,prefix);
		else
			sprintf(pt,"%s Packet - Source:%s Destination:%s - %s"
				,proto,src,dst,prefix);
#if 0
		pt=msg;
		sprintf(pt,"%s Packet - Source:%s Destination:%s - %s",proto,src,dst,prefix);
#endif
		if(strstr(conf.mail_keyword,prefix)){
			email_send_threshold++;
		}
		if(strstr(conf.log_keyword,prefix)==NULL){
			do_log=0;
		}
		if(strstr(prefix,"match")){
			do_log=1;
		}
	}else
		msg++;
    
	if(strcmp(last_log,msg)==0){
        	//if mail_keyword match,do check whether send mail,but don't log it.
        	if(conf.mail_enable==1 && email_send_threshold>send_mail)
            		do_log=0;
        	else
		    	return ;
	}else{
		strcpy(last_log,msg);
	}

add_timestamp:
	/* time stamp */
	time(&now);
	if( atoi(conf.daylight)== 1)
		now += daylight_saving_new()*60*60;
		//now += daylight_saving()*60*60;
    	//now += time_adjust(conf.TZ)*60;
	st=localtime(&now);
	sprintf(timestamp,"%s, %d-%02d-%02d %02d:%02d:%02d"
			,wday[st->tm_wday]
			,st->tm_year+1900
			,st->tm_mon+1
			,st->tm_mday
			,st->tm_hour
			,st->tm_min
			,st->tm_sec);	

	if(conf.mail_enable==1){
		char cmd[1024];
		FILE *fp;
		//sem_down(s_semid);
		if(email_send_threshold<=atoi(conf.dos_thresholds)){//record msg,the max number of the msg is dos_thresholds.
            		if(email_send_threshold>send_mail){//if msg match mail_keyword,no match no record.
                		fp=fopen("/var/log/alert","a+");	
        			if(fp==NULL)
        				return ;
        			fprintf(fp,"No.%03d  %s - %s\n",email_send_threshold, timestamp, msg);
        			fclose(fp);
            		}
        	}
		if(email_send_threshold>=atoi(conf.dos_thresholds)){
    			if((now-last_send_mail)>ALERT_MAX_INTERVAL){
                		sprintf(cmd,"/usr/sbin/smtpc -m -h %s -r %s -f %s -s \"%s\" </var/log/alert "
    					,conf.mail_server
    					,conf.mail_receiver
    					,conf.mail_sender
    					,conf.mail_subject_alert);
            
    				system(cmd);
    				last_send_mail=now;
                		email_send_threshold=0;
                		send_mail=0;
    				unlink("/var/log/alert");
    			}
		}
		//sem_up(s_semid);
	}
    else
        email_send_threshold=0;
	
		
	mylog("log enable ==%d \n",conf.log_enable);
	if(do_log==0 || conf.log_enable ==0)
		return ;
	
	/* todo: supress duplicates */
#ifdef BB_FEATURE_REMOTE_LOG
	/* send message to remote logger */
	if ( (-1 != remotefd) && (doRemoteLog==TRUE)){
static const int IOV_COUNT = 2;
		struct iovec iov[IOV_COUNT];
		struct iovec *v = iov;

		memset(&res, 0, sizeof(res));
		snprintf(res, sizeof(res), "<%d>", pri);
		v->iov_base = res ;
		v->iov_len = strlen(res);
		v++;

		v->iov_base = msg;
		v->iov_len = strlen(msg);
writev_retry:
		if ( -1 == writev(remotefd,iov, IOV_COUNT)){
			if (errno == EINTR) goto writev_retry;
			error_msg_and_die("cannot write to remote file handle on"
					"%s:%d",RemoteHost,RemotePort);
		}
	}	
		
//	if (local_logging == TRUE)
#endif
if(strstr(msg,"login"))
	{
	if(login==0)
		login=1;
	else		
		return;
	}
if(strstr(msg,"log Out"))
	login=0;

if(((conf.log_syslog!=1)&&strstr(msg,"NTP")) || ((conf.log_auth!=1)&&(strstr(msg,"login")||strstr(msg,"log Out"))))
return;

#ifdef SHOW_HOSTNAME
		/* now spew out the message to wherever it is supposed to go */
		message("%s %s %s %s\n", LocalHostName, res, msg, timestamp);
#else
		message("%s - %s\n", msg, timestamp);
#endif
}

#endif
static void reload_signal(int sig)
{
	parse_config(config_file_path);
}

static void quit_signal(int sig)
{
	//logMessage(LOG_SYSLOG | LOG_INFO, "System log daemon exiting.");
	unlink(lfile);
#ifdef BB_FEATURE_IPC_SYSLOG
	ipcsyslog_cleanup();
#endif
	exit(TRUE);
}
static int log_start=0;

static inline void router_start()
{
	struct sysinfo info;
	time_t now;
	char *wday[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	struct tm *st;
	char timestamp[128];
	
	if(log_start==1)
		return ;
	
	sysinfo(&info);
	/* time stamp */
//	setenv("TZ",conf.TZ,1);
	time(&now);
    	now += atoi(conf.daylight)*60*60;
    	now += time_adjust(conf.TZ)*60;
	now-=info.uptime;    
	st=localtime(&now);
	sprintf(timestamp,"%s, %d-%02d-%02d %02d:%02d:%02d"
			,wday[st->tm_wday]
			,st->tm_year+1900
			,st->tm_mon+1
			,st->tm_mday
			,st->tm_hour+atoi(conf.daylight)
			,st->tm_min
			,st->tm_sec);

	if(st)
		free(st);

	log_start=1;
	if(conf.log_syslog==1)
	message("%s - %s\n", "Router start up", timestamp);
}

static void domark(int sig)
{
	if(log_start==0){
		router_start();
	}
	last_log[0]='\0';
	alarm(MarkInterval);
}

/* This must be a #define, since when DODEBUG and BUFFERS_GO_IN_BSS are
 * enabled, we otherwise get a "storage size isn't constant error. */
static int serveConnection (char* tmpbuf, int n_read)
{
	char *p = tmpbuf;

	while (p < tmpbuf + n_read) {

		int           pri = (LOG_USER | LOG_NOTICE);
		char          line[ MAXLINE + 1 ];
		unsigned char c;
		int find=0;

		char *q = line;

		while ( (c = *p) && q < &line[ sizeof (line) - 1 ]) {
			if (c == '<' && find==0) {
			/* Parse the magic priority number. */
				pri = 0;
				find= 1;
				while (isdigit (*(++p))) {
					pri = 10 * pri + (*p - '0');
				}
				if (pri & ~(LOG_FACMASK | LOG_PRIMASK)){
					pri = (LOG_USER | LOG_NOTICE);
				}
			} else if (c == '\n') {
				*q++ = ' ';
			} else if (iscntrl (c) && (c < 0177)) {
				*q++ = '^';
				*q++ = c ^ 0100;
			} else {
				*q++ = c;
			}
			p++;
		}
		*q = '\0';
		p++;
		/* Now log it */
	mylog("raul debug  dosyslog  ... Now log it \n");		
		logMessage (pri, line);
	}
	return n_read;
}


#ifdef BB_FEATURE_REMOTE_LOG
static void init_RemoteLog (void)
{

  struct sockaddr_in remoteaddr;
  struct hostent *hostinfo;
  int len = sizeof(remoteaddr);
  int so_bc=1;

  memset(&remoteaddr, 0, len);

  remotefd = socket(AF_INET, SOCK_DGRAM, 0);

  if (remotefd < 0) {
    error_msg_and_die("cannot create socket");
  }
  
  remoteaddr.sin_family = AF_INET;
  
  /* Ron */ 
  /* allow boardcast */
  setsockopt(remotefd,SOL_SOCKET,SO_BROADCAST,&so_bc,sizeof(so_bc));
  hostinfo = gethostbyname(RemoteHost);
  remoteaddr.sin_addr = *(struct in_addr *) *hostinfo->h_addr_list;
  remoteaddr.sin_port = htons(RemotePort);

  /*
     Since we are using UDP sockets, connect just sets the default host and port
     for future operations
  */
  if ( 0 != (connect(remotefd, (struct sockaddr *) &remoteaddr, len))){
     error_msg_and_die("cannot connect to remote host %s:%d", RemoteHost, RemotePort);
  }
}
#endif

static void doSyslogd (void) __attribute__ ((noreturn));
static void doSyslogd (void)
{
	struct sockaddr_un sunx;
	socklen_t addrLength;

	int sock_fd;
	fd_set fds;

	/* Set up signal handlers. */
	signal (SIGINT,  quit_signal);
	signal (SIGTERM, quit_signal);
	signal (SIGQUIT, quit_signal);
//#ifdef BB_FEATURE_IPC_SYSLOG //raul add signal support for email
	signal (SIGHUP,  send_mail_signal);
//	signal (SIGUSR1, clear_signal);
	signal (SIGUSR2, reload_signal);
//#endif
	signal (SIGCHLD,  SIG_IGN);
#ifdef SIGCLD
	signal (SIGCLD,  SIG_IGN);
#endif
	signal (SIGALRM, domark);
	//wait ntp get correct time
	alarm (MarkInterval);


	mylog("raul debug  dosyslog  ... 1 \n");
	/* Create the syslog file so realpath() can work. */
	if (realpath (_PATH_LOG, lfile) != NULL)
		unlink (lfile);

	memset (&sunx, 0, sizeof (sunx));
	sunx.sun_family = AF_UNIX;
	strncpy (sunx.sun_path, lfile, sizeof (sunx.sun_path));
	if ((sock_fd = socket (AF_UNIX, SOCK_DGRAM, 0)) < 0)
		perror_msg_and_die ("Couldn't get file descriptor for socket " _PATH_LOG);

	addrLength = sizeof (sunx.sun_family) + strlen (sunx.sun_path);
	if (bind(sock_fd, (struct sockaddr *) &sunx, addrLength) < 0)
		perror_msg_and_die ("Could not connect to socket " _PATH_LOG);

	if (chmod (lfile, 0666) < 0)
		perror_msg_and_die ("Could not set permission on " _PATH_LOG);

	mylog("raul debug  dosyslog  ... 2 \n");
#ifdef BB_FEATURE_IPC_SYSLOG
	if (circular_logging == TRUE ){
	   ipcsyslog_init();
	}
#endif
	
#if 0
#ifdef BB_FEATURE_REMOTE_LOG
	if (doRemoteLog == TRUE){
	  init_RemoteLog();
	}
#endif
#endif

	mylog("raul debug  dosyslog  ... 3 \n");
	for (;;) {

		FD_ZERO (&fds);
		FD_SET (sock_fd, &fds);

		if (select (sock_fd+1, &fds, NULL, NULL, NULL) < 0) {
			if (errno == EINTR) {
				/* alarm may have happened. */
				continue;
			}
			perror_msg_and_die ("select error");
		}

		if (FD_ISSET (sock_fd, &fds)) {
		       int   i;
		       RESERVE_BB_BUFFER(tmpbuf, BUFSIZ + 1);

		       memset(tmpbuf, '\0', BUFSIZ+1);
		       if ( (i = recv(sock_fd, tmpbuf, BUFSIZ, 0)) > 0) {
			       serveConnection(tmpbuf, i);
		       } else {
			       perror_msg_and_die ("UNIX socket error");
		       }
		       RELEASE_BB_BUFFER (tmpbuf);
		}/* FD_ISSET() */
	} /* for main loop */
}
    
/* Modify by Jeff -Feb.22.2005- */
int parse_config(char *conf_path)
{
		FILE *fp;
		char buff[1024];
		int i=0;
		char rhip[16]="";
#ifdef DEBUG
		printf("conf_path==%s\n",conf_path);	
#endif
		if(conf_path==NULL)
			fp=fopen(__CONF_FILE,"r");
		else
			fp=fopen(conf_path,"r");
		
		if(fp==NULL)
			return FALSE;
		
		fread(buff,1024,1,fp);	
		fclose(fp);
		
		/* initial conf */
		bzero(&conf,sizeof(conf));
		memset(&conf.log_list,-1,sizeof(conf.log_list));
		/* initial conf */

		if(strstr(buff,"mail_enable=1")) conf.mail_enable=1;
		if(strstr(buff,"mail_enable_auth=1")) conf.mail_enable_auth=1;	
#ifdef BB_FEATURE_REMOTE_LOG
		strccpy2(rhip,buff,"log_remote_ip=",'\n');

		if(strlen(rhip)>2){
				if(RemoteHost) free(RemoteHost);
				RemoteHost=strdup(rhip);
				doRemoteLog = TRUE;
				if(remotefd > 0) close(remotefd);
	  			init_RemoteLog();
		}else
				doRemoteLog = FALSE ;
#endif				
		/* if email is not enable ,we don't need to parser those config*/
		if(conf.mail_enable==1){
			if(strstr(buff,"mail_log_full=1")) conf.mail_log_full=1;

			strccpy2(conf.mail_server,buff,"mail_server=",'\n');
			strccpy2(conf.mail_receiver,buff,"mail_receiver=",'\n');
//			strccpy2(conf.mail_enable_auth,buff,"mail_enable_auth=",'\n');			
			strccpy2(conf.mail_login,buff,"mail_login=",'\n');
			strccpy2(conf.mail_password,buff,"mail_password=",'\n');						
            strccpy2(conf.mail_sender,buff,"mail_sender=",'\n');            
			strccpy2(conf.mail_subject,buff,"mail_subject=",'\n');
			strccpy2(conf.mail_subject_alert,buff,"mail_subject_alert=",'\n');
			strccpy2(conf.mail_keyword,buff,"mail_keyword=",'\n');
            strccpy2(conf.dos_thresholds,buff,"dos_thresholds=",'\n');
		}
			if(strstr(buff,"log_syslog=1")) conf.log_syslog=1;

			if(strstr(buff,"log_auth=1")) conf.log_auth=1;
		
		strccpy2(conf.TZ,buff,"TZ=",'\n');

		/*setenv("TZ",conf.TZ,1);*/
		
		strccpy2(conf.daylight,buff,"daylight=",'\n');
        
        if(strstr(buff,"log_enable=1")) conf.log_enable=1;

		strccpy2(conf.log_keyword,buff,"log_keyword=",'\n');
        
		return TRUE;
}


/***************************************************************************
 Description:
    Check whether need to implenment daylight saving time, if need ,auto change
    to daylight saving time
 return :
   1: need daylight saving time
   0: need not daylight saving time         
 ***************************************************************************/ 
 
 /* *******************************************************************
 Carl modified this function to support all timezone which need the daylight saving
 This algorithm should be accurate thru 2099.After 2099, we need adjust it
Description for daylight start and end time:
NA
	Start: First Sunday in April
	End: Last Sunday in October
CHILE
	Start: Second Saturday of October - at midnight
	End: Second Saturday of March - at midnight
SA
	Start: First Sunday in November
	End: Third Sunday in February
EU
	Start: Last Sunday in March at 1 am UTC
	End: Last Sunday in October at 1 am UTC
ISRARL(Israel will change it every year,so we dont support it as winXP's do)
	Start: First Friday in April
	End: First Friday in September
IRAQ
	Start: April 1
	End: October 1
AU
	Start: Last Sunday in October
	End: Last Sunday in March
AU2
	Start: First Sunday in October
	End: Last Sunday in March
AU3
	Start: First Sunday in October
	End: Third Sunday in March
AF
	Start: Last Friday in April
	End: Last Thursday in September
*/
int daylight_saving(void)
{
    	int dst_region = 0; 
    	int last_left_day; 
    	int current_year = 2000; //2000
 	int startDay = 0;
 	int endDay = 0;
 	int bSaveFlag = 0;
	time_t t;
	struct tm *st;
	char *pt=conf.TZ;
	
	t=time(NULL);
	st=localtime(&t);
	
    	current_year=1900+st->tm_year;
    	last_left_day=st->tm_yday;
    	
    	if(strcmp(pt+3, "-9")==0 || strcmp(pt+3, "-8")==0
    	   || strcmp(pt+3, "-7 2")==0 || strcmp(pt+3, "-6 2")==0
    	   || strcmp(pt+3, "-5 2")==0 || strcmp(pt+3, "-4 2")==0)
    	   	dst_region = NA_DST;
    	else if(strcmp(pt+3, "-3 2")==0 || strcmp(pt+3, "-2")==0
    	   || strcmp(pt+3, "-1")==0 || strcmp(pt+3, "+0 2")==0
    	   || strcmp(pt+3, "+1 1")==0 || strcmp(pt+3, "+1 2")==0
    	   || strcmp(pt+3, "+2 2")==0)
    		dst_region = EU_DST;
    	else if(strcmp(pt+3, "+2 1")==0)
    		dst_region = AF_DST;
    	else if(strcmp(pt+3, "+3")==0)
    		dst_region = IRAQ_DST;
    	else if(strcmp(pt+3, "+10 2")==0)
    		dst_region = AU_DST;
    	else if(strcmp(pt+3, "+12 2")==0)
    		dst_region = AU3_DST;

	switch(dst_region){
	        case NA_DST:
	       	    	startDay=97-((current_year-2002+((current_year-2000)/4))%7);
	       	    	endDay=304-(current_year-1999+((current_year-1996)/4))%7;            
	            	break;
	        case EU_DST:
	       	    	startDay=90-(current_year-2002+((current_year-2000)/4))%7;
	       	    	endDay=304-(current_year-1999+((current_year-1996)/4))%7;
	            	break;            
	        case AU_DST:
	       	    	startDay=304-(current_year-1999+((current_year-1996)/4))%7;
	       	    	endDay=90-(current_year-2002+((current_year-2000)/4))%7;
	            	break;
	        case CHILE_DST:
	            	startDay = 287 - (current_year-2000+((current_year-2000)/4))%7;
		    	endDay = 73 - (current_year-1998+((current_year-1996)/4))%7;
		    	break;  
	        case SA_DST:
	            	startDay = 311 - (current_year-2004+((current_year-2004)/4))%7;
		    	endDay = 52 - (current_year-1999+((current_year-1996)/4))%7;
		    	break;  
	        case AF_DST:
	            	startDay = 120 - (current_year-2004+((current_year-2004)/4))%7;
		    	endDay = 273 - (current_year-2004+((current_year-2004)/4))%7;
		    	break;  
	        case IRAQ_DST:
	        	if (current_year%4 != 0){
	            		startDay = 91;
		    		endDay = 274;
		    	}else{
	            		startDay = 92;
		    		endDay = 275;
		    	}
		    	break;  
	        case AU2_DST:
	            	startDay = 280 - (current_year-2001+((current_year-2000)/4))%7;
		    	endDay = 90 - (current_year-2002+((current_year-2000)/4))%7;
		    	break;  
	        case AU3_DST:
	            	startDay = 280 - (current_year-2001+((current_year-2000)/4))%7;
		    	endDay = 80 - (current_year-1999+((current_year-1996)/4))%7;
		    	break;  
	        default:
	            break;                    
	    }

       if(dst_region==AU_DST||dst_region==AU2_DST||dst_region==AU3_DST){    /*AU in south */            
	   if(current_year==2006)
	   {
	   	last_left_day++;
	   	if(dst_region==AU_DST)
		   	last_left_day+=2;	   			   	
	  }	
	  if(current_year==2007&&last_left_day+2>endDay)
	  {
	  	last_left_day++;	   	
	  }
	  if(last_left_day>=startDay || last_left_day<=endDay)
    		bSaveFlag = 1;
            else
    		bSaveFlag = 0;
	}else {
	    if(last_left_day>=startDay-1 && last_left_day<=endDay-1)
		bSaveFlag=1;
	    else
	        bSaveFlag=0;
	}
    
    return  bSaveFlag;               
}

int syslogd_main(int argc, char **argv)
//int main(int argc, char **argv)
{
	int opt;
#if ! defined(__uClinux__)
	int doFork = TRUE;
#endif

	char *p;

	
	/* do normal option parsing */
	while ((opt = getopt(argc, argv, "m:nO:R:f:LC")) > 0) {
		switch (opt) {
			case 'm':
				MarkInterval = atoi(optarg) * 60;
				break;
#if ! defined(__uClinux__)
			case 'n':
				doFork = FALSE;
				break;
#endif
			case 'O':
				logFilePath = strdup(optarg);
				break;
#ifdef BB_FEATURE_REMOTE_LOG
			case 'R':
				if(RemoteHost!=NULL) free(RemoteHost);
				RemoteHost = strdup(optarg);
				if ( (p = strchr(RemoteHost, ':'))){
					RemotePort = atoi(p+1);
					*p = '\0';
				}
				doRemoteLog = TRUE;
				break;
			case 'L':
				local_logging = TRUE;
				break;
#endif
#ifdef BB_FEATURE_IPC_SYSLOG
			case 'C':
				circular_logging = TRUE;
				break;
#endif
			case 'f':
				config_file_path=optarg;
				if(parse_config(optarg)==FALSE)
					show_usage();
				break;
				
			default:
				show_usage();
		}
	}
#if 1
#ifdef BB_FEATURE_REMOTE_LOG
	/* If they have not specified remote logging, then log locally */
	if (doRemoteLog == FALSE)
		local_logging = TRUE;
#endif
#endif
	
#ifdef SHOW_HOSTNAME
//	}
	/* Store away localhost's name before the fork */
	gethostname(LocalHostName, sizeof(LocalHostName));
	if ((p = strchr(LocalHostName, '.'))) {
		*p++ = '\0';
	}
#endif
	umask(0);

#if ! defined(__uClinux__)
	if (doFork == TRUE) {
		if (daemon(0, 1) < 0)
			perror_msg_and_die("daemon");
	}
#endif
	doSyslogd();

	return EXIT_SUCCESS;
}
#if 1
extern int klogd_main (int argc ,char **argv);

int main(int argc ,char **argv)
{
	int ret = 0;
	char *base = strrchr(argv[0], '/');

	if (strstr(base ? (base + 1) : argv[0], "syslogd"))
		ret = syslogd_main(argc,argv);
	else if (strstr(base ? (base + 1) : argv[0], "klogd")) 
		ret = klogd_main(argc,argv);
	else	
		show_usage();
	
	return ret;
}
#endif
/*
Local Variables
c-file-style: "linux"
c-basic-offset: 4
tab-width: 4
End:
*/
