#include <linux/init.h>
#include <linux/module.h>
#include <asm/io.h>
#include <linux/delay.h>
#include <asm/uaccess.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <asm-arm/system.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/timer.h>
#include <linux/netlink.h>
#ifdef VSSDEV_SUPPORT
   #include "sm_drv_vss.h"
#endif
#ifdef NETLINK_SUPPORT
#include "sm_drv_netlink.h"
extern int src_seq;
#endif
#include "sm_drv_ahb.h"
#include "driver_version.h"


MODULE_DESCRIPTION("WLC STUN DRIVER");


char driver_name[] = "sm_drv_stun_ahb" ;
extern unsigned int boot_mode;

extern unsigned long ssid_from_driver ;
extern unsigned int ssid_from_user;
extern char ssid_from_user_str[33];

/* PDA support */
uint8_t pda_data[PDA_INFO_SIZE + sizeof(struct wrm_block)];
/* For Proc debugging support */
int cbaddr_read(char *page, char **start, off_t offset, int count,int *eof, void *data );
int create_procfile(void);
int remove_procfile(void);

//added by subrat for device proc interface
int dev_create_procfile(struct net_device *dev);
//end


#ifdef VSSDEV_SUPPORT
inline int check_skb_for_vss(struct sk_buff *skb, uint8_t *vssid)
{
    /* If this skb came from a VSS device, the
     * skb->cb field will be filled with a VSS id.
     * Due to the reason the cb field is free to use, the VSS id
     * will be packed in the following keys "VSSL"vssid".
     */

    if(*(long*)&skb->cb[0] == cpu_to_le32(0x4c535356)) /* VSSL */
    {
        *(vssid) = skb->cb[4];
        /* Make the VSS data invalid */
        *(long*)&skb->cb[0] = 0;

        return(1);      /* VSS is available */
    }

    return(0);          /* VSS is unavailable */
}
#endif

void sm_drv_stun_initialize(struct net_device *dev)
{
    struct net_local *lp = dev->priv;

    struct stun_ahb_hif_local_data * hif_lp ;
    hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    spin_lock_bh(&lp->sm_lock);

    sm_drv_psm_fsm(dev, PSMEVENT_BOOT);

    lp->sm_descr.mtu = 1600;
    lp->sm_descr.rxoffset = 0;
    lp->sm_descr.header = 0;
    lp->sm_descr.trailer = 0;
    /* fill rx queues of control block */
    /* Memory leak fix*/
    while(fragq_drv_pspace(lp, CB_RX_CTRLQ) > 0) {
        if(sm_drv_stun_add_rx_queue(dev, CB_RX_CTRLQ) == -1)
	{
		break;
	}
    }
    /* Memory leak fix*/
    while(fragq_drv_pspace(lp, CB_RX_DATAQ) > 0) {
        if(sm_drv_stun_add_rx_queue(dev, CB_RX_DATAQ) == -1)
	{
		break;
	}
    }
    spin_unlock_bh(&lp->sm_lock);
}

#ifdef DEBUG_PROC
uint32_t cb_address;
uint32_t base_addr;
int cbaddr_read(char *page, char **start, off_t offset, int count,int *eof, void *data )
{
    int i =0,j=0;
    int cnt=0;
    uint32_t reg=0;
    char str[17];
    sprintf(page,"Control Block Address= %x\n",cb_address);
    strcpy(str,"\0");
    STUN_WRITE_REG( (base_addr+ 0xb0),
                   0x1ffe7000);
    reg =  STUN_READ_REG(base_addr + STUN_HIF_CSR);
    mdelay(10);
    printk("Dump ************************* \n");
    for(i=0 ; i<0x100 ; i=i+4)
    {  
       reg = STUN_READ_REG(base_addr + 0x1f00 + i);
       printk("%.8x ", reg);
       reg=cpu_to_be32(reg);
       memcpy(str+cnt,&reg,4);

       if(cnt>=12)
       {
        str[16]='\0';
        for(j=0;j<16;j++)
        {
           if(str[j]==0)
           printk(".");
           else
           printk("%c",str[j]);
        }
        printk("\n");
        cnt=0;
       }
       else
          cnt+=4;
      }
    printk("DUMP Over ******************************* \n");
    return 0;
}
#endif
static int sm_drv_stun_init(void)
{

    struct stun_ahb_hif_local_data * hif_lp = NULL; /* Local Data  */
    uint32_t u32_reg;
    struct net_device *dev=NULL;
    struct net_local *lp;
    int rvalue;
  
    printk("DRIVER VERSION: %s \n",version_no);
 
#ifdef NETLINK_SUPPORT
    mgt_initialize();
#endif
 
  
    if (dev = sm_drv_stun_ahb_probe(dev, 0, 0), dev == NULL)
    {
        DEBUG(DBG_ERR,"ERROR: %s could not configure network device \n", driver_name );
        goto error2;
    }
    /* Access the pointers setup in sm_drv_netdev_create() */
    lp = dev->priv;
    hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
 
    /* Setup the MemBase pointer */
    hif_lp->MemBase = STUN_PHY_TO_VIRT(STUN_REG_BASE);

    /* Just to check if the mapping is correct */
    u32_reg = STUN_READ_REG(hif_lp->MemBase + STUN_HIF_CSR);

#ifdef DEBUG_PROC
    base_addr = hif_lp->MemBase;
#endif

    DEBUG(DBG_CALLS,"CSR REG[%x] \n",u32_reg);
    lp->watch_dog=(WD_TIMEOUT_SEC*1000)/TIMEOUT_RES_MSEC;               /* Watch-Dog counter for time-out */
    hif_lp->stungSdRamAddr = MAS_HOST_MEM_BASE_REG;
    /* Initialise the MacMemBase register */
    STUN_WRITE_REG((hif_lp->MemBase + STUN_HIF_MAS_HOST_MEM_ADDR),
                   MAS_HOST_MEM_BASE_REG);
    mdelay(DELAY); 
 
    DEBUG(DBG_CALLS,"Reading the MAS_HOST_MEM_ADDR Reg [%x] \n",
	    STUN_READ_REG(hif_lp->MemBase + STUN_HIF_MAS_HOST_MEM_ADDR));

    /* Set up the Control Block Register here -- */
    hif_lp->control_block = (struct stun_controlblock *)kmalloc(sizeof(struct stun_controlblock) , GFP_ATOMIC);

    if( hif_lp->control_block == NULL)
    {
        DEBUG(DBG_ERR,"Could not allocate control buffer \n");
        goto error2;
    }

    if( ( ((uint32_t)(hif_lp->control_block)) & 0x3 ) != 0 )
    {
        DEBUG(DBG_ERR,"Control Buffer not aligned \n");
        goto error2;
    }	
    /* Map Control Block to Physical  Address */ 
    ahb_MapAddressToSlaveAddress((uint32_t)hif_lp->stungSdRamAddr,hif_lp->control_block,sizeof(struct stun_controlblock), 
                                 &(hif_lp->stun_ahbcontrolblockaddress) );

    memset(hif_lp->control_block,0,sizeof(struct stun_controlblock));

    lp->next_module = root_sm_drv_device;
    root_sm_drv_device = dev;
 
    /* Reset the Card */
    stun_ResetCard(hif_lp);
    /* Let the card reboot */
    mdelay(1500);

    if (rvalue = request_irq(STUN_AHB_IRQNO, &sm_drv_stun_interrupt, SA_INTERRUPT |SA_SHIRQ,
        driver_name, dev), rvalue != 0)
    {
        DEBUG(DBG_ERR,"Could not install IRQ-handler \n");
        goto error2;
    }
    /* Tell the Kernel About us */
    register_netdev(dev);
	
    //subrat added for device proc file support
#if 0
        dev_create_procfile(dev);
#endif
    //end


    /* Init our Bottom halve handler */
#ifdef TASK_LET
    tasklet_init(&lp->tl,sm_drv_stun_ahb_bh,dev);
#else
    INIT_WORK(&(lp->glbWQ), (void *)sm_drv_stun_ahb_bh,dev);
#endif

    STUN_WRITE_REG((hif_lp->MemBase + STUN_HIF_MAS_HOST_MEM_ADDR),
                   MAS_HOST_MEM_BASE_REG);
    /* Initially, we startup using the boot mode setting */
    if (sm_drv_up(dev, boot_mode) < 0)
    {
        DEBUG(DBG_ERR,"ERROR: %s could not start SHoCMAC on SOLOS-WLC \n", driver_name);
        goto error2;
    }
#ifdef NETLINK_SUPPORT
    mgt_indication_handler ( DEV_NETWORK, dev->ifindex, dev->name, 0, 
                             sm_drv_mgt_indication );
#endif

    /* According to the doxbox: 416843-D doc. */
    {
        uint32_t reg;
        reg = STUN_READ_REG( hif_lp->MemBase + STUN_HIF_CSR) ;
        DEBUG(DBG_CALLS,"23 = %08x,24 = %08x, 25 = %08x reg %08x\n", 
             (0x00800000&reg),
             (0x01000000&reg),
             (0x02000000&reg),
              reg);
        STUN_WRITE_REG( hif_lp->MemBase +  STUN_HIF_CSR, (0x00800000|reg));
        reg = STUN_READ_REG( hif_lp->MemBase + STUN_HIF_CSR);
        DEBUG(DBG_CALLS,"23 = %08x,24 = %08x, 25 = %08x\n", 
             (0x00800000&reg),
             (0x01000000&reg),
             (0x02000000&reg));
    }

    /* Tell the card about Control Block Address */
    u32_reg = STUN_READ_REG( hif_lp->MemBase + STUN_CONTROL_BASE );
    DEBUG(DBG_CALLS,"%s: About to set control block to 0x%08x. " \
                   "Old 0x%08x PCI Base 0x%08x \n",
                   __FUNCTION__,
                   (uint32_t)hif_lp->control_block,
                   u32_reg,
                   (uint32_t)hif_lp->stun_ahbcontrolblockaddress);
 
    /* Set up the Control Block Register here -- */
    STUN_WRITE_REG( hif_lp->MemBase + STUN_CONTROL_BASE,hif_lp->stun_ahbcontrolblockaddress);
    mdelay(DELAY);

    /* Map the Control Block to UNCACHED address */
    hif_lp->control_block = (struct stun_controlblock *)((uint32_t)hif_lp->control_block + 0x10000000); 
    DEBUG(DBG_CALLS,"CONTROL BLOCK UNCACHED ADDRESS [%p] \n",hif_lp->control_block); 
    if(stun_RunFirmware(hif_lp) < 0)
    {
        DEBUG(DBG_ERR,"Not able to run Firmware \n"); 
    }

#ifdef WDS_SUPPORT
    lp->wdsp = kmalloc(sizeof(struct wds_priv), GFP_KERNEL );
    if( !lp->wdsp )
    { 
        DEBUG(DBG_ERR,"%s: Error: Could not allocate WDS device memory \n", driver_name);
        goto error2;
    }
    memset(lp->wdsp, 0, sizeof(struct wds_priv));
#endif
#ifdef VSSDEV_SUPPORT
    lp->vssdev = kmalloc(sizeof(struct vss_priv), GFP_KERNEL);
    if ( lp->vssdev == NULL)
    {
        DEBUG(DBG_ERR,"%s: Error: Could not allocate VSS device memory \n", driver_name);
        goto error2;
        //kfree(lp);
        //kfree(netdev);
       // return NULL;
    }
    memset(lp->vssdev, 0, sizeof(struct vss_priv));
#endif

    /* Everything went fine */	  
    /* Start the WLAN Led Timer */
    led_timer_init(lp);
    /* Create the Netlink Socket for restarting PAED */
    lp->nl_sk = netlink_kernel_create(NETLINK_PAED_RESET,NULL);
    if( lp->nl_sk == NULL)
    {
        DEBUG(DBG_ERR ,"%s , Memory Allocation Failed \n" , __FUNCTION__);
        return -1;
    }
   
#ifdef DEBUG_PROC 
    create_procfile();
    cb_address=virt_to_phys(hif_lp->control_block)^(0x10000000);
    if(create_proc_read_entry("SolosW_CB",0,NULL,cbaddr_read,NULL)==NULL)
    {
  	DEBUG(DBG_ERR,"ERROR: %s could not create proc file \n", driver_name ); 
        return 0;
    }
#endif
   //subrat for acllist
   initialize_acllist();
   //end
    

    return 0;	
    error2:
    /* Error handling code to be added */
    return -1;
}

static void sm_drv_stun_cleanup(void)
{
    /* Clean-Up Code to be added */
#ifdef NETLINK_SUPPORT
    mgt_cleanup();
#endif
}

/********************** HELPER FUNCTIONS **************************/
int stun_ResetCard(struct stun_ahb_hif_local_data * ptr_str_lp)
{
    uint32_t u32_reg = 0;

    u32_reg = STUN_READ_REG(ptr_str_lp->MemBase + STUN_HIF_CSR);
    u32_reg &= ~( STUN_HIF_CSR_CPU_EN );

    STUN_WRITE_REG( (ptr_str_lp->MemBase + STUN_HIF_CSR), u32_reg );
    mdelay(5); 

    u32_reg = STUN_READ_REG(ptr_str_lp->MemBase + STUN_HIF_CSR);
 
    /* 1. Clearing the RAMBoot and Reset Bits */
    u32_reg &= ~(STUN_HIF_CSR_RESET + STUN_HIF_CSR_CLKRUN);
 
#ifdef STUN_BOOT_SRAM
    u32_reg |=(STUN_HIF_CSR_RAMBOOT + STUN_HIF_CSR_OVERRIDE +
               STUN_HIF_CSR_HALTED);
#endif 

#ifdef STUN_BOOT_SDRAM
    u32_reg |= ~STUN_HIF_CSR_RAMBOOT;
    u32_reg |= ( STUN_HIF_CSR_OVERRIDE + STUN_HIF_CSR_HALTED );
#endif
 
    STUN_WRITE_REG((ptr_str_lp->MemBase + STUN_HIF_CSR),u32_reg);
    mdelay(DELAY);

    /* 2. Set the Reset Bit to latch things done in step 1 */
    u32_reg |= STUN_HIF_CSR_RESET;
    STUN_WRITE_REG((ptr_str_lp->MemBase + STUN_HIF_CSR),u32_reg);
    mdelay(DELAY);
 /* 3. Clear the reset bit to start target reset */
    u32_reg &= ~STUN_HIF_CSR_RESET;
    STUN_WRITE_REG((ptr_str_lp->MemBase + STUN_HIF_CSR),u32_reg);
    mdelay(DELAY);

    return  SUCCESS; 
}

int stun_UploadFirmware(struct stun_ahb_hif_local_data * ptr_str_lp)
{
    uint8_t *data = NULL;
    mm_segment_t fs;
    struct file *ifp = NULL;
    uint32_t u32_reg,temp_length=0,temp_count = 0,ctr2=0,wait_counter=0;
    uint32_t window = 0;
    long bcount = 0;
    uint32_t length,Ramblockboot , max_wait = 0;
    int ready_int,i,cts_int;
    uint32_t ctr1;
 
    data = kmalloc(STUN_HIF_MEM_WINDOW_SIZE , GFP_KERNEL);
    if(NULL == data)
    {
        return ALLOC_ERR; 
    }
 
    /* Open firmware file */
    fs = get_fs();
    set_fs(KERNEL_DS);
    ifp = filp_open(STUN_IMAGE_FILE ,O_RDONLY ,0);
    if(IS_ERR(ifp))
    {
        DEBUG(DBG_ERR,"Cannot open firmware file \n");
        set_fs(fs);
        kfree(data);
        return FOPEN_ERR;
    } 
    /* Suspend ARM */
    u32_reg = STUN_READ_REG(ptr_str_lp->MemBase + STUN_HIF_CSR);
    u32_reg |= (STUN_HIF_CSR_OVERRIDE + STUN_HIF_CSR_HALTED);
    STUN_WRITE_REG(ptr_str_lp->MemBase + STUN_HIF_CSR , u32_reg);

#ifdef STUN_BOOT_SRAM
    window = STUN_FW_SRAMDEST;
#endif

#ifdef STUN_BOOT_SDRAM
    window = STUN_FW_SDRAMDEST;
#endif 

    do
    {
        length = 0;
        ready_int = 0;
        bcount = 0;	
        Ramblockboot =0;
        max_wait = 0;	

        bcount = ifp->f_op->read(ifp, data , sizeof(uint32_t)*2, &ifp->f_pos);
        Ramblockboot = le32_to_cpu( *(uint32_t *)data);
        length = le32_to_cpu( *(uint32_t *)(data + 4));
        max_wait = Ramblockboot & STUN_MAX_WAIT_BIT_MASK;
	
        DEBUG(DBG_CALLS,"Length [%08x],Ramboot [%08x],Max_Wait [%08x],Bcount [%08x] \n",
	        length, Ramblockboot, max_wait,(uint32_t)bcount);   
    
        if( (bcount >0) && (length >0) )
        {
	    if(Ramblockboot & STUN_RAM_BOOT_NEEDED)
            {
                DEBUG(DBG_CALLS,"Inside Rambootneeded \n");
        	for(i=0;i<length;i+=STUN_HIF_MEM_WINDOW_SIZE)
		{
                    STUN_WRITE_REG(ptr_str_lp->MemBase + STUN_HIF_MAC_MEM_BASE,
                                   window);
                    mdelay(DELAY);
                    if( (length-i) < STUN_HIF_MEM_WINDOW_SIZE)
	            {
			 temp_length = length -i;
		    }
                    else
		    {
		        temp_length = STUN_HIF_MEM_WINDOW_SIZE;	
		    }
				 
                    temp_count = ifp->f_op->read(ifp,data,temp_length,&ifp->f_pos);

                    if(temp_count != temp_length)
		    {
                       DEBUG(DBG_ERR,"File read error,temp_length[%d],\
					   temp_count[%d] \n",temp_length,temp_count);
                    }
		    for(ctr2=0; ctr2<temp_count; ctr2+=4)
                    {
#ifdef COPY_UH_DIRECTLY_SHSDRAM
                        STUN_WRITE_REG((STUN_FW_SDRAMDEST + ctr2), 
                                       *(uint32_t *)(&data[ctr2]));
#else
                        STUN_WRITE_MEM(ptr_str_lp->MemBase, window,
                                      (ptr_str_lp->MemBase + STUN_MEM_WINDOW + ctr2),
							  *(uint32_t *)(&data[ctr2]));
                        udelay(10);
#endif
                    } /* End for loop */
  	            window+= temp_count;

		}/* End for loop */
                u32_reg = STUN_READ_REG(ptr_str_lp->MemBase + STUN_HIF_CSR);
		u32_reg |= STUN_HIF_CSR_CPU_EN;
		STUN_WRITE_REG(ptr_str_lp->MemBase + STUN_HIF_CSR,u32_reg);
		mdelay(50);
		/*Wait for ready interrupt from device */
	        for( u32_reg=0,wait_counter=0,ready_int=0;
		   ((wait_counter < max_wait) && !ready_int);
		     wait_counter++)
                {
                    u32_reg = STUN_READ_REG(ptr_str_lp->MemBase + STUN_HIF_HOST_INT);
		 
                    if( u32_reg & STUN_HIF_HOST_INT_READY)
		    {
                        STUN_WRITE_REG(ptr_str_lp->MemBase+STUN_HIF_HOST_INT_ACK,u32_reg);
			STUN_WRITE_REG(ptr_str_lp->MemBase+STUN_HIF_HOST_INT_ACK,
                                       STUN_HIF_HOST_INT_READY);
			udelay(1);
		        ready_int = 1;
		     }
                 } /* End for loop */
                 if(!ready_int)
                 {
	             DEBUG(DBG_ERR,"Timeout waiting for ready int bit \n");
                     return ERROR;
                 }
                 else
	         {
                     DEBUG(DBG_CALLS,"Ready interrupt received \n");
	         }
			
                 if( stun_UploadPDA(ptr_str_lp) < 0)
                 {
                     DEBUG(DBG_ERR,"PDA not uploaded \n");
                     return ERROR;
                 }
            } /* End of if(Ramblockboot & STUN_RAM_BOOT_NEEDED) */
		/* Download the next blocks using DR/CTS using GPR1 */
	    else
            {
                for(ctr1=0; ctr1<length; (ctr1=ctr1+4))
                {
                     wait_counter = 0;
                     cts_int = 0;
                     u32_reg = 0;
			 
                     temp_count =ifp->f_op->read(ifp,&data[0],sizeof(uint32_t),
                                                &ifp->f_pos);
	 
        	     STUN_WRITE_REG(ptr_str_lp->MemBase+STUN_HIF_GPR1,
                                   *( (uint32_t *)&data[0]) );	
                     wmb();	 
                     udelay(1);		   
		     STUN_WRITE_REG((ptr_str_lp->MemBase+STUN_HIF_ARM_INT),
						   STUN_HIF_ARM_INT_UART_DR);	
                     wmb();	 
                     while(wait_counter <=max_wait)
		     {
                         u32_reg = STUN_READ_REG(ptr_str_lp->MemBase + STUN_HIF_HOST_INT);	 
                         rmb();	 
			 if( u32_reg & STUN_HIF_HOST_INT_PCIUART_CTS)
			 {
                             cts_int = 1;
                             break;
                         }
                         udelay(2);
                         wait_counter++; 
		     }/* End while */	
			  
                     if(!cts_int)
		     {
                          DEBUG(DBG_ERR,"Timeout waiting for CTS irq bit Ctr1: [%d]\n", ctr1);
                          DEBUG(DBG_CALLS,"Reading the MAS_HOST_MEM_ADDR Reg [%x] \n",
                          STUN_READ_REG(ptr_str_lp->MemBase + STUN_HIF_MAS_HOST_MEM_ADDR)); 		   
                          set_fs(fs);
		          return ERROR;  	   
                     }
                     STUN_WRITE_REG((ptr_str_lp->MemBase+STUN_HIF_HOST_INT_ACK),
                                    STUN_HIF_HOST_INT_PCIUART_CTS);
                     wmb();
                }/* End For loop*/
            }/* End else */
        } /* if( (bcount >0) && (length >0) ) */
    }while(bcount != 0);

    /* Close  File */
    filp_close(ifp, NULL);
    set_fs(fs);
    kfree(data); 
    return 0;
}


int stun_UploadPDA(struct stun_ahb_hif_local_data * ptr_str_lp)
{
    struct file *ifp = NULL;
    uint32_t pda_size = 0,temp_count = 0;
    struct wrm_block *wBlock;
    uint32_t PDA_DEST_ADDR = 0x0000,wait_counter,cts_int,max_wait=0x00002000 ;
    uint32_t u32_reg = 0;
    uint32_t ctr1=0;

    pda_size = PDA_INFO_SIZE;
    /* Open firmware file */

    ifp = filp_open(STUN_PDA_FILE ,O_RDONLY ,0);
 
    if(IS_ERR(ifp))
    {
        DEBUG(DBG_ERR,"Cannot open PDA file \n");
        return FOPEN_ERR;
    }

    temp_count =ifp->f_op->read(ifp, pda_data + (sizeof(struct wrm_block)) ,pda_size,
                                &ifp->f_pos);

    DEBUG(DBG_CALLS,"%s,Bytes read from PDA file [%d] \n", __FUNCTION__,temp_count);

    wBlock = (struct wrm_block *)&pda_data[0];
    wBlock->pda_dest_addr = PDA_DEST_ADDR;
    wBlock->pda_size = pda_size;
    wBlock->pda_src_addr = 0x0001;
	 
    for (ctr1 = 0; ctr1 <(PDA_INFO_SIZE+(sizeof(struct wrm_block))); ctr1=ctr1+4)
    {
        wait_counter = 0;
        cts_int = 0;
        u32_reg =0;	 
        STUN_WRITE_REG(ptr_str_lp->MemBase+STUN_HIF_GPR1, *((uint32_t *)&pda_data[ctr1]) );
        STUN_WRITE_REG((ptr_str_lp->MemBase+STUN_HIF_ARM_INT),
                       STUN_HIF_ARM_INT_UART_DR);
        udelay(10);
		 
        while(wait_counter <=max_wait)
        {
            memcpy_fromio((void*)&u32_reg,(void *)(ptr_str_lp->MemBase + STUN_HIF_HOST_INT),4);

            if( u32_reg & STUN_HIF_HOST_INT_PCIUART_CTS)
            {
                cts_int = 1;
                break;
            }
            udelay(2);
            wait_counter++;
			   
        }/* End while */	

        if(!cts_int)
        {
            DEBUG(DBG_ERR,"Timeout waiting for CTS irq bit Ctr1: [%d]\n", ctr1);
            filp_close(ifp, NULL);
	    return ERROR;
        }
        STUN_WRITE_REG((ptr_str_lp->MemBase+STUN_HIF_HOST_INT_ACK),
                        STUN_HIF_HOST_INT_PCIUART_CTS);

    }/* End For Loop */
    DEBUG(DBG_CALLS,"PDA Uploaded successfully \n");
    /* Close  File */
    filp_close(ifp, NULL); 
    return 0;
 
}

int stun_RunFirmware(struct stun_ahb_hif_local_data * ptr_str_lp)
{
    /* Clear all IRQs and don't enable any */
    uint32_t u32_reg = 0;
	
    u32_reg = STUN_READ_REG( ptr_str_lp->MemBase + STUN_HIF_HOST_INT);
    STUN_WRITE_REG( ptr_str_lp->MemBase + STUN_HIF_HOST_INT_ACK, u32_reg );
    STUN_WRITE_REG( ptr_str_lp->MemBase + STUN_HIF_HOST_INT_EN, 0 );
    mdelay(DELAY ); 

    /* Check init is off */
    if( u32_reg & STUN_HIF_HOST_INT_DEV_INIT )
    {
        DEBUG(DBG_CALLS,"%s: INIT bit is active before start %08x\n", 
                         __FUNCTION__, u32_reg );
    }
    
    mdelay(100);
    STUN_WRITE_REG(ptr_str_lp->MemBase + STUN_HIF_ARM_INT, 
                   STUN_HIF_ARM_INT_RESET );
   
    /* enable interrupts */
    STUN_WRITE_REG(ptr_str_lp->MemBase + STUN_HIF_HOST_INT_EN,
                   STUN_HIF_INT_ENABLES );
    udelay(100);
    return 0;	
}


struct net_device *sm_drv_stun_ahb_probe(struct net_device *dev, long ioaddr, int irq)
{
    struct net_local *lp;

    DEBUG(DBG_CALLS, "sm_drv_stun_ahb_probe\n");
 
    dev = sm_drv_netdev_create(dev, ioaddr, irq);
    if( !dev )
        return NULL;

    lp = dev->priv;
    lp->hif_up = &sm_drv_stun_ahb_hif_up;
    lp->hif_down = &sm_drv_stun_ahb_hif_down;
    lp->hif_cli = &sm_drv_stun_ahb_cli;
    lp->hif_local_data = kmalloc(sizeof(struct stun_ahb_hif_local_data), GFP_ATOMIC);

    if (!lp->hif_local_data) 
    {
        DEBUG(DBG_ERR,"ERROR: Could not allocate host interface specific data\n");
        kfree(lp);
        kfree(dev);
        return NULL;;
    }
 
    memset(lp->hif_local_data, 0, sizeof(struct stun_ahb_hif_local_data));
    return dev; 
}

int sm_drv_stun_ahb_hif_up(struct net_device *dev)
{
    struct net_local *lp;
    int queue;
    struct stun_ahb_hif_local_data * hif_lp;
    lp = dev->priv;
    hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    /* Do the Control Block Initialisation here */
    /* Initialize the fragment queues. They are filled with empty
     * fragments in the bottom halve after the init interrupt
     */
    for (queue = 0; queue < CB_QCOUNT; queue++) {
        fragq_init(dev, queue);
    }
    if (stun_UploadFirmware(hif_lp) < 0) 
    {
        DEBUG(DBG_ERR,"ERROR:failed to upload the firmware \n");
        return -1;
    }
    return 0;
}


int sm_drv_stun_ahb_hif_down(struct net_device *dev)
{
    struct net_local *lp = dev->priv;
    struct stun_ahb_hif_local_data *hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;

    /* stop the FIRMWARE now in order to prevent it writing to host memory anymore. */
    STUN_WRITE_REG(hif_lp->MemBase + STUN_HIF_ARM_INT,STUN_HIF_INT_ABORT);
    mdelay(50);
    /* After Firmware shutdown, we can free the queues */
    sm_drv_ahb_free_rx_queue(dev, CB_RX_CTRLQ);
    sm_drv_ahb_free_rx_queue(dev, CB_RX_DATAQ);
    return 0;
}


void sm_drv_stun_ahb_cli(struct net_device *dev)
{

}

/* NOT called under sm_lock. This function cleans frags which have */
/* tx-ed to DEVICE and which have accepted by DEVICE. We just release  */
/* those fragments if needed.                                      */
void sm_drv_stun_frag_txed(struct net_device *dev, struct stun_fragment *frag,
                           struct s_fragq_hdl *fragq_hdl)
{
    struct sk_buff *skb = fragq_hdl->skb;
    struct net_local *lp = dev->priv;
    struct stun_ahb_hif_local_data *hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    uint8_t * virt_address = NULL;
    DEBUG(DBG_IRQ, "sm_drv_stun_frag_txed\n");
    
    lp = dev->priv;
    hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    if(skb)
    {
       frame_skb_free(dev,skb);
    }
    else
    { 
       virt_address = (uint8_t *)phys_to_virt((uint32_t)(fragq_hdl->sdram_phy_address) - STUN_FW_SDRAMDEST);
       kfree(virt_address);    
    }
}


/* NOT called under sm_lock. This function checks if frames have   */
/* been tx-ed to DEVICE and which have been accepted by DEVICE. We     */
/* just release  those fragments if needed. This function is to    */
/* be called from interrupt handler in case op an update interrupt */
/* and before any target DMA is started to make sure right         */
/* fragment is sent with targetDMA function.                       */
void sm_drv_stun_check_txed(struct net_device *dev)
{
    struct net_local *lp = dev->priv;
    struct stun_fragment *frag;
    struct s_fragq_hdl *fragq_hdl;
    /* check for any transmitted frames, first CTRL queue */
    while (fragq_drv_cspace(lp, CB_TX_CTRLQ)) {
        DEBUG(DBG_BH, "Control Frame Sent to DEVICE\n");
        frag = fragq_consume(lp, CB_TX_CTRLQ, &fragq_hdl);
        sm_drv_stun_frag_txed(dev, frag, fragq_hdl);
    }
    /* now the data queue */
    while (fragq_drv_cspace(lp, CB_TX_DATAQ)) {
        DEBUG(DBG_BH, "Data Frame Sent to DEVICE\n");
        frag = fragq_consume(lp, CB_TX_DATAQ, &fragq_hdl);
        sm_drv_stun_frag_txed(dev, frag, fragq_hdl);
    }
}

void sm_drv_stun_ahb_bh(struct net_device *dev)
{
    struct net_local *lp = dev->priv;
    struct stun_ahb_hif_local_data *hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    struct stun_fragment *frag;
    struct s_fragq_hdl *fragq_hdl;
    /* Lets lock the SoftMAC since we cant have that one interferring */
    DEBUG(DBG_BH, "Bottom Halve\n");

    spin_lock_bh(&lp->sm_lock);
       
    sm_drv_stun_check_txed(dev); 
    /* check all queues in priority order */
    while(fragq_drv_cspace(lp, CB_RX_CTRLQ)) {
        frag = fragq_consume(lp, CB_RX_CTRLQ, &fragq_hdl);
        sm_drv_stun_frag_rx(dev, frag, fragq_hdl,CB_RX_CTRLQ);
        /* Memory leak fix*/
        while(fragq_drv_pspace(lp, CB_RX_CTRLQ) > 0) {
            if(sm_drv_stun_add_rx_queue(dev, CB_RX_CTRLQ)==-1)break;
        }
    }

    while (fragq_drv_cspace(lp, CB_RX_DATAQ)) {
        DEBUG(DBG_BH, "Received frame in Data Queue \n");
        frag = fragq_consume(lp, CB_RX_DATAQ, &fragq_hdl);
          sm_drv_stun_frag_rx(dev, frag, fragq_hdl,CB_RX_DATAQ);
        /* Memory leak fix*/
        while(fragq_drv_pspace(lp, CB_RX_DATAQ) > 0) {
            if(sm_drv_stun_add_rx_queue(dev, CB_RX_DATAQ)==-1)break;
        }
    }
    spin_unlock_bh(&lp->sm_lock);
    if( hif_lp->generate_update_interrupt )
    {
        int irq;

        hif_lp->generate_update_interrupt = 0;
        irq = STUN_HIF_ARM_INT_UPDATE;
        sm_drv_stun_trigger_device(hif_lp, irq);
    }
    DEBUG(DBG_BH, "EXIT Bottom Halve\n");
}

/* Memory leak fix*/
int sm_drv_stun_add_rx_queue(struct net_device *dev, int queue)
{
    struct net_local *lp = dev->priv;
    struct stun_ahb_hif_local_data *hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    struct s_fragq_hdl fragq_hdl;
    struct stun_fragment frag;
    struct sk_buff *skb; 


    if(queue == CB_RX_CTRLQ)
        skb = dev_alloc_skb(1600 + lp->sm_descr.header + lp->sm_descr.trailer + lp->sm_descr.rxoffset );
    else
        skb = dev_alloc_skb(lp->sm_descr.mtu + lp->sm_descr.header + lp->sm_descr.trailer + lp->sm_descr.rxoffset);
    if(!skb) {
        DEBUG(DBG_ERR ,"Cannot allocate skb for RX Queue %d\n", queue);
        return -1;
    }

    if(queue == CB_RX_CTRLQ)
      skb_put(skb,1600);
    else
      skb_put(skb,lp->sm_descr.mtu);

    if(queue == CB_RX_DATAQ)
       lp->frameGiven++;

    fragq_hdl.skb = skb;
    /* Map SKB to physical address */
    ahb_MapAddressToSlaveAddress((uint32_t)hif_lp->stungSdRamAddr,skb->data,skb->len,&(fragq_hdl.sdram_phy_address));

    frag.host_address = fragq_hdl.sdram_phy_address;
    frag.size = skb->len;
    frag.flags = 0;
    fragq_produce(lp, queue, &frag, &fragq_hdl);
    hif_lp->generate_update_interrupt = 1;
    return 0;
}

inline void sm_drv_stun_interrupt_device(struct net_device *dev)
{
    struct net_local *lp = dev->priv;
    struct stun_ahb_hif_local_data *hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    
    sm_drv_stun_trigger_device(hif_lp, STUN_HIF_ARM_INT_UPDATE);
    sm_drv_psm_fsm(dev, PSMEVENT_RUN_BH);
}
void sm_drv_add_tx_queue(struct net_device *dev, struct sk_buff *skb)
{
    struct net_local *lp = dev->priv;
    struct stun_ahb_hif_local_data *hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    struct s_fragq_hdl fragq_hdl;
    struct stun_fragment frag;
    struct page * page_start;
    struct page * page_end;
#ifdef WDS_SUPPORT
    struct sk_buff *temp;
    unsigned char wds_mac[6];
    int wds = 0;
#endif
#ifdef VSSDEV_SUPPORT
        int vss;
        uint8_t vssid;
#endif
#ifdef WDS_SUPPORT
    memset (wds_mac, 0, 6);
    wds = check_skb_for_wds( skb, &wds_mac[0] );
    if( wds )
    {
    temp = dev_alloc_skb(skb->len+6);     // TO_DO SOMETHING
    skb_put(temp,skb->len+6);
    memcpy(temp->data+6,skb->data,skb->len);
  	   memcpy( temp->data, &wds_mac[0], 6 );
      dev_kfree_skb(skb);              
             skb =temp;
	// mark the frame as WDS
	frag.flags = SM_FRAME_WDS;      // TO_DO SOMETHING
    }
    else
    {
      frag.flags=0;
    }
#endif 
    fragq_hdl.skb = skb;
    /* Map skb to physical address */
    ahb_MapAddressToSlaveAddress((uint32_t)hif_lp->stungSdRamAddr,skb->data,skb->len,
                                 &(fragq_hdl.sdram_phy_address));
    frag.size = skb->len ;
    frag.host_address = fragq_hdl.sdram_phy_address;
#ifdef VSSDEV_SUPPORT

#ifdef DEBUGVSS
printk(KERN_ERR "Sending packet AHB skb->cb : %d\n",skb->cb[4]);
#endif

    vss = check_skb_for_vss(skb, &vssid);

    if(vss) /* Valid Packet found from VSS Device */
    {
#ifdef DEBUGVSS
    	printk(KERN_ERR "skbuff destination %2x:%2x:%2x:%2x:%2x:%2x\n",skb->data[0],                                                        skb->data[1],
                                                        skb->data[2],
                                                        skb->data[3],
                                                        skb->data[4],
                                                        skb->data[5]);
#endif
        frag.ssnr = vssid;
        frag.flags = SM_FRAME_VSS;
#ifdef DEBUGVSS
        printk(KERN_ERR "Found vssid %d Flag %d\n",frag.ssnr,frag.flags);
#endif
    }
    else
        {
        frag.flags = 0;
        frag.ssnr = 0;
        }
#endif


    page_start = virt_to_page(skb->data);
    page_end = virt_to_page( (skb->data) + skb->len);
    while( page_start <=page_end)
    {
        flush_dcache_page(page_start);
        page_start++;
    }
    fragq_produce(lp, CB_TX_DATAQ, &frag, &fragq_hdl);
    sm_drv_stun_interrupt_device(dev);
}


void sm_drv_add_tx_ctrl_queue(struct net_device *dev, uint8_t *pMsg, int32_t length)
{
    struct net_local *lp = dev->priv;
    struct stun_ahb_hif_local_data *hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    struct s_fragq_hdl fragq_hdl;
    struct stun_fragment frag;
    struct page * page_start;
    struct page * page_end;
    
    fragq_hdl.skb = NULL;   
    ahb_MapAddressToSlaveAddress((uint32_t)hif_lp->stungSdRamAddr,pMsg,length,
                                 &(fragq_hdl.sdram_phy_address));
    frag.host_address =fragq_hdl.sdram_phy_address;
    frag.size = length;
    frag.flags = FRG_FLAG_CONTROL;
    page_start = virt_to_page(pMsg);
    page_end = virt_to_page( pMsg + length);
    while( page_start <=page_end)
    {
        flush_dcache_page(page_start);
        page_start++;
    }
    fragq_produce(lp, CB_TX_CTRLQ, &frag, &fragq_hdl);
    sm_drv_stun_interrupt_device(dev);
}

int sm_drv_stun_interrupt(int irq, void *config, struct pt_regs *regs)
{
    struct net_device  *dev = config;
    struct net_local *lp = dev->priv;
    struct stun_ahb_hif_local_data * hif_lp ;
    uint32_t reg;
#ifdef DEBUG_ON
    struct stun_controlblock *cb ;
#endif
  
    hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    reg =STUN_READ_REG(hif_lp->MemBase + STUN_HIF_HOST_INT);
    reg &= STUN_HIF_INT_ENABLES; 
    if(reg)
    {
        /* Ack the interrupt received */
        STUN_WRITE_REG( hif_lp->MemBase + STUN_HIF_HOST_INT_ACK, reg);
#ifdef STUN_UART
        if(reg & STUN_HIF_ARM_INT_UART_DR)
        {
           sm_drv_print_radio2host_block(hif_lp);
        }
#endif
        if(reg & STUN_HIF_HOST_INT_DEV_UPDATE)
        {
#ifdef DEBUG_ON
            cb = ((struct stun_ahb_hif_local_data*)lp->hif_local_data)->control_block;
            DEBUG(DBG_IRQ," update int received \n");
            DEBUG(DBG_IRQ,
            "CB drv Qs: [%i][%i][%i][%i][%i][%i]\n",
                 le32_to_cpu(cb->driver_curr_frag[0]),
                 le32_to_cpu(cb->driver_curr_frag[1]),
                 le32_to_cpu(cb->driver_curr_frag[2]),
                 le32_to_cpu(cb->driver_curr_frag[3]),
                 le32_to_cpu(cb->driver_curr_frag[4]),
                 le32_to_cpu(cb->driver_curr_frag[5])
                 );
            DEBUG(DBG_IRQ,
            "CB dev Qs: [%i][%i][%i][%i][%i][%i]\n",
                 le32_to_cpu(cb->device_curr_frag[0]),
                 le32_to_cpu(cb->device_curr_frag[1]),
                 le32_to_cpu(cb->device_curr_frag[2]),
                 le32_to_cpu(cb->device_curr_frag[3]),
                 le32_to_cpu(cb->device_curr_frag[4]),
                 le32_to_cpu(cb->device_curr_frag[5])
                 );
#endif
           
            sm_drv_psm_fsm(dev, PSMEVENT_RUN_BH);
        }

        /* INIT Interrupt */
        if(reg & STUN_HIF_HOST_INT_DEV_INIT)
        {
            sm_drv_stun_initialize(dev);
#ifdef STUN_UART 
            sm_drv_uart(hif_lp);
#endif
            /* Run the bottom halve to fill the queue */
            sm_drv_psm_fsm(dev, PSMEVENT_RUN_BH);
        }
    }
    return IRQ_RETVAL(1);
}

void sm_drv_ahb_free_rx_queue(struct net_device *dev, int queue)
{
    struct net_local *lp = dev->priv;

    DEBUG(DBG_CALLS, "sm_drv_ahb_free_rx_queue: q %d\n", queue);

    /* Free up any frames that are still in the RX queues */
    while (fragq_drv_pspace(lp, queue) < frag_queue_size[queue]) {
        struct stun_fragment *frag;
        struct sk_buff *skb;
        struct s_fragq_hdl *fragq_hdl;
        frag = fragq_consume(lp, queue, &fragq_hdl);
        skb = fragq_hdl->skb;
        frame_skb_free(dev, skb);
    }
}
/* Map local memory address to AHB-bus memory address */
inline int ahb_MapAddressToSlaveAddress(
    uint32_t           stungSdRamAddr,
    void*               buffer,
    uint32_t           length,
    uint32_t           *AHBAddr )
{

    *AHBAddr = virt_to_phys(buffer) + STUN_FW_SDRAMDEST;
    return SUCCESS;
}

void fragq_produce(struct net_local *lp, int queue,
                          struct stun_fragment *frag, struct s_fragq_hdl *fragq_hdl)
{
    struct stun_controlblock *cb = ((struct stun_ahb_hif_local_data*)lp->hif_local_data)->control_block;
    struct stun_ahb_hif_local_data *hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;

    struct s_fragq_hdl *handlep;
    int index;
    unsigned long flags;

    spin_lock_irqsave(&hif_lp->cb_lock, flags);

    index = le32_to_cpu(cb->driver_curr_frag[queue]) & (frag_queue_size[queue] - 1);
    switch(queue) {
        case CB_RX_DATAQ:
            handlep = &(hif_lp->local_cb_handle.rx_data_hdl[index]);
            cb->rx_data[index].host_address = frag->host_address;
            cb->rx_data[index].size = frag->size;
            cb->rx_data[index].flags = frag->flags;
            cb->rx_data[index].ssnr = frag->ssnr;
            break;
        case CB_TX_DATAQ:
            handlep = &(hif_lp->local_cb_handle.tx_data_hdl[index]);
            cb->tx_data[index].host_address = frag->host_address;
            cb->tx_data[index].size = frag->size;
            cb->tx_data[index].flags = frag->flags;
            cb->tx_data[index].ssnr = frag->ssnr;
            break;
        case CB_RX_CTRLQ:
            handlep = &(hif_lp->local_cb_handle.rx_control_hdl[index]);
            cb->rx_control[index].host_address = frag->host_address;
            cb->rx_control[index].size = frag->size;
            cb->rx_control[index].flags = frag->flags;
            cb->rx_control[index].ssnr = frag->ssnr;
            break;
        case CB_TX_CTRLQ:
            handlep = &(hif_lp->local_cb_handle.tx_control_hdl[index]);
            cb->tx_control[index].host_address = frag->host_address;
            cb->tx_control[index].size = frag->size;
            cb->tx_control[index].flags = frag->flags;
            cb->tx_control[index].ssnr = frag->ssnr;

            break;
        default:
            printk(KERN_ERR "frag_produce: Illegal queue %d\n", queue);
            spin_unlock_irqrestore(&hif_lp->cb_lock, flags);
            return;
    }
    memcpy(handlep, fragq_hdl, sizeof(struct s_fragq_hdl));
    cb->driver_curr_frag[queue] = cpu_to_le32(le32_to_cpu(cb->driver_curr_frag[queue]) + 1);

    spin_unlock_irqrestore(&hif_lp->cb_lock, flags);
}

/* Consume a fragment and associated handle from a queue */
struct stun_fragment *fragq_consume(struct net_local *lp, int queue,
                                      struct s_fragq_hdl **handlep)
{
    struct stun_controlblock *cb = ((struct stun_ahb_hif_local_data*)lp->hif_local_data)->control_block;
    struct stun_ahb_hif_local_data *hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;

    int index;
    unsigned long flags;
    struct stun_fragment *frag = NULL;
    void *handle;

    spin_lock_irqsave(&hif_lp->cb_lock, flags);

    index = hif_lp->fragment_c[queue] & (frag_queue_size[queue] - 1);

    switch(queue) {
        case CB_RX_DATAQ:
            frag = &(cb->rx_data[index]);
            handle = &(hif_lp->local_cb_handle.rx_data_hdl[index]);
            break;
        case CB_TX_DATAQ:
            frag = &(cb->tx_data[index]);
            handle = &(hif_lp->local_cb_handle.tx_data_hdl[index]);
            break;
        case CB_RX_CTRLQ:
            frag = &(cb->rx_control[index]);
            handle = &(hif_lp->local_cb_handle.rx_control_hdl[index]);
            break;
        case CB_TX_CTRLQ:
            frag = &(cb->tx_control[index]);
            handle = &(hif_lp->local_cb_handle.tx_control_hdl[index]);
            break;
        default:
            printk(KERN_ERR "frag_consume: Illegal queue %d\n", queue);
            goto err_out;
    }
    if(handlep)
        *handlep = handle;

    (hif_lp->fragment_c[queue])++;

err_out:
    spin_unlock_irqrestore(&hif_lp->cb_lock, flags);
    return frag;
}

void sm_drv_stun_frag_rxed(struct net_device *dev, struct stun_fragment *frag,
                           struct s_fragq_hdl *fragq_hdl)
{
    struct sk_buff *skb = fragq_hdl->skb;
    /* Free up any frames that are still in the RX queues */
    frame_skb_free(dev, skb);
}
/* called under sm_lock */
void sm_drv_stun_frag_rx(struct net_device *dev, struct stun_fragment *frag,
                           struct s_fragq_hdl *fragq_hdl , int queue)
{
    struct net_local *lp = dev->priv;
    struct sk_buff *skb = fragq_hdl->skb;
    static int check=0;
#ifdef VSSDEV_SUPPORT
        struct net_device *vss_dev = NULL;
        struct vss_dev_local *vss_lp = NULL;
        int vssid = 0;
        int isvss;
#endif

    DEBUG(DBG_IRQ, "sm_drv_stun_frag_rx\n");
     
    skb->len = le16_to_cpu(frag->size);
#ifdef VSSDEV_SUPPORT
     isvss = frag->flags;
     vssid = frag->ssnr;
#endif
#ifdef WDS_SUPPORT
    struct wds_priv *wdsp = lp->wdsp;
    struct net_device *wds_dev =NULL;
    struct wds_net_local *wnlp =NULL;
    int  iswds = frag->flags;
#endif
    if(queue == CB_RX_CTRLQ)
    {
        PimforHeader *header = (PimforHeader *)skb->data;
        void *pData;
        uint8_t op;
        uint32_t oid, length;
        PimforHeaderGet(header, &op, &oid, &length, &pData);
        if((header->version!=PIMFOR_VERSION)|| length < 0 )
        {
            DEBUG(DBG_ERR,"ERROR IN FRAME RECEIVED\n");
            goto error_exit;
        }

	/* If the MAC address is changed, then the dev_addr needs to be updated
	 * such that the modified MAC address is reflected in ifconfig
	 */

        if ((oid == DOT11_OID_BEACONTX) && (op == PIMFOR_OP_TRAP))
	{
            memcpy( dev->dev_addr, (uint8_t *)(pData), ETH_ALEN );
            // printk("Starting Timer \n");
            keepalive_timer_init(dev);
	}

        if ((oid == GEN_OID_LINKSTATE )&& (op ==PIMFOR_OP_TRAP))
        {
            uint32_t speed;
            speed = *((uint32_t*)pData);   
            printk("received link state trap\n");
            wlan_link_status(LINK_UP);
            control_wlan_led(22 , WLAN_SET);
            if(speed)
            {
                if(check == 0)
                {
                    mdelay(10);
                    struct s_sm_conf *smconf1;
                    smconf1 = kmalloc(sizeof(struct s_sm_conf),GFP_KERNEL);
                    if(NULL == smconf1)
			{
                        printk("%s:kmalloc returned NULL \n",__FUNCTION__);
                        goto error_exit;
                    }
                    memset(smconf1,0,sizeof(struct s_sm_conf));                                
                    smconf1->data = kmalloc(4, GFP_KERNEL);
                    if(NULL == smconf1->data)
			{
                        printk("%s:kmalloc returned NULL \n",__FUNCTION__);
                        kfree(smconf1);
                        goto error_exit;
                    }
                    smconf1->length =4;

                    smconf1->oid = 0x1b000008;
                    smconf1->flags = SM_CONF_OPSET;
                    *(uint32_t *)(smconf1->data) = 1500;
                    spin_unlock_bh(&lp->sm_lock);
                    sm_drv_conf(dev,smconf1);

                    kfree(smconf1->data); 
                    kfree(smconf1);
                    spin_lock_bh(&lp->sm_lock);
                    check=1;
                }
                netif_start_queue(dev);
            }
            else
                netif_stop_queue(dev);
        }
        if((oid == GEN_OID_MACADDRESS) && (op == PIMFOR_OP_TRAP))
        {
            /* Generic SM_CONF structure */
            struct s_sm_conf * ssid ;
            /* OBJ_SSID */
            struct obj_ssid *obj_ssid;

            printk("Received the MAC Address trap\n");
            memcpy( dev->dev_addr, (uint8_t *)(pData), ETH_ALEN );

            /* If we got the MAC address, this means the device has finished booting */

#ifdef NETLINK_SUPPORT
            mgt_response( 0, src_seq++, DEV_NETWORK_WLAN, dev->ifindex, PIMFOR_OP_TRAP, oid, pData, ETH_ALEN);
#endif

            wake_up(&lp->conf_waitq);
            sm_drv_psm_fsm(dev, PSMEVENT_IFUP);
            /* SET SSID Functionality start here */
            ssid = kmalloc(sizeof(struct s_sm_conf) , GFP_KERNEL);
            obj_ssid = kmalloc(sizeof(struct obj_ssid), GFP_KERNEL);
            if( (ssid== NULL) || (obj_ssid==NULL) )
            {
                printk("%s,Unable to allocate memory \n",__FUNCTION__);
            }

            memset(ssid, 0, sizeof(struct s_sm_conf));
            memset(obj_ssid , 0 , sizeof(struct obj_ssid));
            ssid->oid = 0x10000002;
            ssid->length =sizeof(struct obj_ssid) ;

            ssid->data = obj_ssid ;
           
            if( (ssid_from_driver == 0) && ( strcmp(ssid_from_user_str , "not_set") != 0  ) )
            {
                memcpy(obj_ssid->octets,ssid_from_user_str , 33);
            }
            else
            {
                sprintf(obj_ssid->octets,"PRISM_%02x_%02x_%02x",dev->dev_addr[3],dev->dev_addr[4],dev->dev_addr[5] );
            }
            ssid_from_driver = 1;
            obj_ssid->length = strlen(obj_ssid->octets);
            ssid->flags =SM_CONF_OPSET;

            spin_unlock_bh(&lp->sm_lock);
            sm_drv_conf(dev,ssid);

            kfree(ssid->data);
            kfree(ssid);

            spin_lock_bh(&lp->sm_lock);
        }
        else if((op == PIMFOR_OP_RESPONSE) || (op == PIMFOR_OP_ERROR))
        {
            struct s_sm_conf t;

            memset(&t, 0, sizeof(struct s_sm_conf));
            t.oid = oid;
            t.length = length;
            t.data = kmalloc(t.length, GFP_ATOMIC);
            if(op == PIMFOR_OP_ERROR)
                t.flags =PIMFOR_OP_ERROR;
            memcpy((uint8_t *)t.data, (uint8_t *)pData, t.length);
            /* This is a response to a GET operation */
            memcpy( &lp->getresp, &t, sizeof(struct s_sm_conf) );

            /* Wake up the app that requested this info.
             * This will also free t.data. */
            wake_up( &lp->getresp_waitq );
        }
#ifdef AUTO_ACCESS
        else if((oid == DOT11_OID_AUTHENTICATE ) && (op == PIMFOR_OP_TRAP)) 
	{
            struct s_sm_conf *smconf;
            smconf = kmalloc(sizeof(struct s_sm_conf),GFP_KERNEL);
            if(NULL == smconf)
			{
                printk("%s:kmalloc returned NULL \n",__FUNCTION__);
                goto error_exit;
            }
            memset(smconf,0,sizeof(struct s_sm_conf));                                
            smconf->data = kmalloc(length, GFP_KERNEL);
            if(NULL == smconf->data)
			{
                printk("%s:kmalloc returned NULL \n",__FUNCTION__);
                kfree(smconf);
                goto error_exit;
            }
            smconf->length =length;
            smconf->oid = oid;
            smconf->flags = SM_CONF_OPSET;
            memcpy((uint8_t *)smconf->data, (uint8_t *)pData, length);
            spin_unlock_bh(&lp->sm_lock);
            sm_drv_conf(dev,smconf);

            kfree(smconf->data); 
            kfree(smconf);
            spin_lock_bh(&lp->sm_lock);
        }
        else if((oid == DOT11_OID_ASSOCIATE) && (op == PIMFOR_OP_TRAP)) 
        {
            struct s_sm_conf *smconf;
            smconf = kmalloc(sizeof(struct s_sm_conf) ,GFP_KERNEL);
            if(NULL == smconf)
			{
                printk("%s:kmalloc returned NULL \n",__FUNCTION__);
                goto error_exit;
            }
            memset(smconf,0,sizeof(struct s_sm_conf));                 
            smconf->data = kmalloc(length, GFP_KERNEL);
            if(NULL == smconf->data)
			{
                printk("%s:kmalloc returned NULL \n",__FUNCTION__);
                kfree(smconf);
                goto error_exit;
            }
            smconf->length =length;
            smconf->oid = oid;
            smconf->flags = SM_CONF_OPSET;
            memcpy((uint8_t *)smconf->data, (uint8_t *)pData, length);
            spin_unlock_bh(&lp->sm_lock) ;
            sm_drv_conf(dev,smconf);  		
        
            kfree(smconf->data); 
            kfree(smconf);
            spin_lock_bh(&lp->sm_lock); 

        }
#endif

#ifdef NETLINK_SUPPORT
        if(op == PIMFOR_OP_TRAP) 
        {
            mgt_response( 0, src_seq++, DEV_NETWORK_WLAN, dev->ifindex, PIMFOR_OP_TRAP,oid, pData, length );
        }

#endif

        sm_drv_stun_frag_rxed(dev,frag,fragq_hdl);
    }
    else
    {
#ifdef VSSDEV_SUPPORT
               		 if(isvss == SM_FRAME_VSS)
                	{
                       
 	// find the VSS device from the table
                        	if( vss_dev = vss_find_device( vssid, lp->vssdev ), vss_dev == NULL ) {
#ifdef DEBUGVSS
                         	printk( KERN_ERR "Received a frame from an unknown VSS device\n" );
#endif
#ifdef MEMORY_TRACER
                        	//mu_dev_kfree_skb(skb, 10);
#else
                       		// dev_kfree_skb(skb); // This will crash the driver
#endif
                       	 	goto error_exit;//Decide here to free skb or not
                       // skb = NULL;
                    	}
                        else
                        {
#ifdef DEBUGVSS
                         	printk( KERN_ERR "Received a  valid frame with vssid %d\n",vssid );
#endif
                        	// remove the VSS device address from the buffer and update the statistics
                        	vss_lp = (struct vss_dev_local *) vss_dev->priv;
                       		vss_lp->stats.rx_packets++;
                        	vss_lp->stats.rx_bytes += skb->len;
                        	skb->dev = vss_dev;
                    	}
         		skb->priority = 0;
         		skb->protocol = eth_type_trans( skb, skb->dev );
         		skb->ip_summed = CHECKSUM_NONE ;
         		netif_rx(skb);
                        return ;
                }
#endif
#ifdef WDS_SUPPORT
               		 if(iswds == SM_FRAME_WDS)
                      {
                    // find the WDS device in the list of WDS links
                    if( wds_dev = wds_find_device( skb->data, wdsp ), wds_dev == NULL ) {
                        DEBUG( DBG_ERR ,"Received a frame from an unknown WDS links\n" );
                          goto error_exit;
                    } else {
                        // remove the WDS link address from the buffer and update the statistics
                        wnlp = (struct wds_net_local *) wds_dev->priv;
                        memmove( skb->data, skb->data+6, skb->len-6 );
                        skb_trim( skb, skb->len-6 );

                        wnlp->stats.rx_packets++;
                        wnlp->stats.rx_bytes += skb->len;
                        skb->dev = wds_dev;
                        DEBUG( DBG_WDS, "WDS Frame RX: length %d\n", skb->len );
	                  skb->protocol = eth_type_trans( skb, wds_dev );
                        netif_rx(skb);
                         return ;
                    }
                } 
#endif
       sm_drv_frame_rx(dev, skb);

    }
error_exit:
    return;
}
#ifdef STUN_UART
uint32_t  wait_for_dr_bit(struct stun_ahb_hif_local_data *hif_lp)
{
    uint32_t i, reg = 0;
    for(i=0; i<=STUN_MAX_STUNUART_TIMEOUT; i++)
    {
        reg = STUN_READ_REG(hif_lp->MemBase + STUN_HIF_ARM_INT);
        if( !(reg & STUN_HIF_ARM_INT_UART_DR) )
        {
            STUN_DEBUG_2("%s: Got the DR  bit disabled \n",__FUNCTION__);
            break;
        }
        mdelay(20);
    }

    if(( reg & STUN_HIF_ARM_INT_UART_DR ) )
    {
        STUN_DEBUG_1("%s: Timeout waiting for DR irq bit. ARM_INT %08x\n",
                         __FUNCTION__, reg );

        return( EIO );
    }
    return SUCCESS;
}

uint32_t sm_drv_tell_to_radio(struct stun_ahb_hif_local_data *hif_lp )
{
    uint32_t status = SUCCESS;
    uint32_t ahb_addr;

    /* Tell about Host to Radio location */
#ifdef SM_DRV_STUN_PCI
    status = pci_MapAddressToSlaveAddress( (void*)hif_lp->shared_dbg_htos,
                                           STUN_DBG_SIZE_H2S,
                                           &(ahb_addr));
#else
    DEBUG(DBG_UART,"%s,shared_dgb_htos=%x\n",__FUNCTION__,hif_lp->shared_dbg_htos);
    status = ahb_MapAddressToSlaveAddress( (uint32_t)hif_lp->stungSdRamAddr,
                                           (void*)hif_lp->shared_dbg_htos,
                                           STUN_DBG_SIZE_H2S,
                                           &(ahb_addr));
#endif /* SM_DRV_STUN_PCI */
    if( status !=SUCCESS )
    {
        DEBUG(DBG_ERR,"%s: Unable to map to AHB %d\n",
                      __FUNCTION__, status );
        return EIO;
    }
    STUN_WRITE_REG( hif_lp-> MemBase+STUN_HIF_GPR2, (uint32_t)ahb_addr);
    mdelay(10);

    STUN_WRITE_REG(  hif_lp->MemBase+STUN_HIF_ARM_INT, STUN_HIF_ARM_INT_UART_DR );

    status = wait_for_dr_bit(hif_lp);
    if( status != SUCCESS )
    {
        DEBUG (DBG_ERR,"Returning NON SUCCESS .\n");
        return status;
    }

    DEBUG(DBG_UART,"%s,shared_dgb_stoh=%x\n",__FUNCTION__,hif_lp->shared_dbg_stoh);
    status = ahb_MapAddressToSlaveAddress( (uint32_t)hif_lp->stungSdRamAddr,
                                           (void*)hif_lp->shared_dbg_stoh,
                                           STUN_DBG_SIZE_S2H,
                                           &(ahb_addr));

    if( status != SUCCESS )
    {
        STUN_DEBUG_1("%s: Unable to map to AHB %d\n",
                         __FUNCTION__, status );
        return EIO;
    }

    STUN_WRITE_REG( hif_lp-> MemBase+STUN_HIF_GPR2, (uint32_t)ahb_addr);
    STUN_WRITE_REG( hif_lp->MemBase+STUN_HIF_ARM_INT, STUN_HIF_ARM_INT_UART_DR );

    /* Waiting for DR bit to come */
    status = wait_for_dr_bit(hif_lp);
    if( status != SUCCESS )
    {
        DEBUG (DBG_ERR,"WAIT dr bit Failure: AFter AHB ADDR: [%x].\n", status);
        return status;
    }
        /* Tell the size of Radio2Host memory to Radio-Block */
    STUN_WRITE_REG(  hif_lp->MemBase+STUN_HIF_GPR2, STUN_DBG_SIZE_S2H);
    STUN_WRITE_REG( hif_lp-> MemBase+STUN_HIF_ARM_INT, STUN_HIF_ARM_INT_UART_DR );

    status = wait_for_dr_bit(hif_lp);
    if( status != SUCCESS )
    {
        DEBUG (DBG_ERR,"WAIT dr bit Failure: AFter Size S2H : [%x].\n", status);
        return status;
    }
    DEBUG (DBG_UART,"Returning Status: [%x].\n", status);
    return status;
}

uint32_t  sm_drv_uart(struct  stun_ahb_hif_local_data *hif_lp )
{
    uint8_t    *uartMemStart = NULL;
    uint32_t status = SUCCESS;
    uint16_t  reg1, reg2;


    /*
    ** The following  buffer will be used to excha
    */
    uartMemStart = kmalloc(STUN_DBG_POOL_SIZE,GFP_KERNEL);
    if(NULL == uartMemStart)
    {
        DEBUG(DBG_ERR,"Failure in getting memory for UART Block\n");
        status = ENOMEM;
        return status;
    }
    DEBUG(DBG_UART,"addressof start is %08x \n",uartMemStart);
    DEBUG(DBG_UART,"got cyan buf size_H2S %08x, size_S2H %08x\n",
                    STUN_DBG_SIZE_H2S,
                    STUN_DBG_SIZE_S2H );


    hif_lp->shared_dbg_htos = (int32_t *)(uartMemStart);
    hif_lp->shared_dbg_stoh = (int32_t *)(uartMemStart + STUN_DBG_SIZE_H2S + 
                                          STUN_SIZE_IN_PTR + STUN_SIZE_OUT_PTR);

    hif_lp->in = (int32_t *)(uartMemStart + STUN_DBG_SIZE_H2S );

    hif_lp->out = (int32_t *)(uartMemStart + STUN_DBG_SIZE_H2S + STUN_SIZE_IN_PTR);
    STUN_UART_WRITE32((uint32_t)hif_lp->in, 0x00);
    STUN_UART_WRITE32((uint32_t)hif_lp->out,(STUN_DBG_SIZE_S2H -1));
    /* tell to radio about these locations */
    status = sm_drv_tell_to_radio(hif_lp);
    if (status != SUCCESS)
    {
        DEBUG(DBG_UART,"%s: Unable to tell the shared location to card\n",
                        __FUNCTION__);

        if( status != SUCCESS )
        {
            DEBUG(DBG_UART, "%s: sm_drv_w_cmd_reset failed %d\n",
                             __FUNCTION__, status );
            return ENOENT;
        }
    }

    reg1=STUN_UART_READU32((uint32_t )hif_lp->in );
    reg2=STUN_UART_READU32((int32_t )hif_lp->out );

    return status;
}


int sm_drv_print_radio2host_block(struct stun_ahb_hif_local_data *hif_lp)
{
    uint32_t status = SUCCESS, in, out, counter;
    uint8_t  reg;
    uint8_t *temp = NULL;
 
    temp =  (int8_t *)hif_lp->shared_dbg_stoh;
    /* print dbg buffer only if debug lavel is immediate
     */
    out=STUN_UART_READU32((uint32_t)hif_lp->out);
    in=STUN_UART_READU32((uint32_t)hif_lp->in);
    if (out >= in)
    {
        for(counter = out+1; counter <(STUN_DBG_SIZE_S2H); counter++)
        {
            /* Do snooping for LF and CR char and if found replace them with
            * \n and "sturn-> " string
            */
                        
            reg= STUN_UART_READU8((uint32_t)(temp + counter));
            if( reg == '\n')
            {
              DEBUG(DBG_UART,"\nSTUN ->");
              continue;
            }
            else
              DEBUG(DBG_UART,"%c",reg);
        }

        for(counter = 0; counter < in ; counter++)
        {
            /* Do snooping for LF and CR char and if found replace them with
             * \n and "sturn-> " string
             */
                  
            reg= STUN_UART_READU8((uint32_t)(temp + counter));
            if( reg == '\n')
            {
            //     DEBUG(DBG_UART,"\nSTUN ->");
                continue;
            }
            else
                DEBUG(DBG_UART,"%c",reg);
        }
    }
    else if (out < in )
    {
        for(counter = out+1; counter < in ; (counter)++)
        {
            /* Do snooping for LF and CR char and if found replace them with
            * \n and "sturn-> " string
            */
                 
            reg= STUN_UART_READU8((uint32_t)(temp + counter));
            if( reg == '\n')
            {
                DEBUG(DBG_UART,"\nSTUN ->");
                continue;
            }
            else
                DEBUG(DBG_UART,"%c",reg);
        }
    }
    if(counter)
        STUN_UART_WRITE32((uint32_t)hif_lp->out, (counter-1));
    else
        STUN_UART_WRITE32((uint32_t)hif_lp->out,(STUN_DBG_SIZE_S2H -1));
   
    return status;
}
#endif

void keepalive_timer_callback(unsigned long data)
{
   
    struct net_device * dev = (struct net_device *)data;
    struct net_local * lp  = dev->priv;
    struct stun_ahb_hif_local_data * hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    
    static uint32_t gpcom2_new;
    static uint32_t gpcom2_prev;
    uint32_t reg = 0;

#if 1    
    gpcom2_new =  STUN_READ_REG(hif_lp->MemBase + STUN_HIF_GPR2);
    if( (gpcom2_new == gpcom2_prev) && (gpcom2_new !=0))
    {
        printk("Reseting Wireless \n [%x] :" ,gpcom2_new);
        spin_lock(&lp->sm_lock);
        hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
        /* Again, Reset the Card */
        stun_ResetCard(hif_lp);
        /* Making the queues Empty */
        sm_drv_stun_ahb_hif_down(dev);
        /* Let the card reboot */
        /* Upload Firmware */
        if (sm_drv_up(dev, boot_mode) < 0)
        {
           DEBUG(DBG_ERR,"ERROR: %s could not start SHoCMAC on SOLOS-WLC \n", driver_name);
        }
                     /* According to the doxbox: 416843-D doc. */
        {
                reg = STUN_READ_REG( hif_lp->MemBase + STUN_HIF_CSR) ;
                DEBUG(DBG_CALLS,"23 = %08x,24 = %08x, 25 = %08x reg %08x\n", (0x00800000&reg),(0x01000000&reg),(0x02000000&reg),reg);
                /* Setting the CLKRUN bit here -- */
                STUN_WRITE_REG( hif_lp->MemBase +  STUN_HIF_CSR, (0x00800000|reg));
                reg = STUN_READ_REG( hif_lp->MemBase + STUN_HIF_CSR);
                DEBUG(DBG_CALLS,"23 = %08x,24 = %08x, 25 = %08x\n",
                (0x00800000&reg),
                (0x01000000&reg),
                (0x02000000&reg));
         }
              /* Tell the card about Control Block Address */
         reg = STUN_READ_REG( hif_lp->MemBase + STUN_CONTROL_BASE );
         /* Set up the Control Block Register here -- */
         STUN_WRITE_REG( hif_lp->MemBase + STUN_CONTROL_BASE,hif_lp->stun_ahbcontrolblockaddress);
         mdelay(DELAY);
            /* Setting the whole control block to 0 */
         memset(hif_lp->control_block,0,sizeof(struct stun_controlblock));
            /* initializing semaphore to 1 */
         sema_init(&lp->sm_sem,1);
            /* Running Firmware */
         spin_unlock(&lp->sm_lock);
         if(stun_RunFirmware(hif_lp) < 0)
         {
                DEBUG(DBG_ERR,"Not able to run Firmware \n");
         }
         mdelay(1500);
        /* Again, we startup using the boot mode setting */
         gpcom2_prev = 0;
         gpcom2_new = 0;
         del_timer_sync(lp->keepalive_timer);
         kfree(lp->keepalive_timer);
         lp->keepalive_timer = NULL;
         /* Scheduling a Work Queue to send a message to an UserLand 
            Application which will kill the PAED and Restart it */
         schedule_work(&(lp->NL_WorkQ));
         return ;
    }
    gpcom2_prev = gpcom2_new;
#endif
    sm_drv_psm_fsm( (struct net_device *)dev, PSMEVENT_RUN_BH);
    /* Rescheduling Timer */
    mod_timer(lp->keepalive_timer , jiffies + 50 ) ;
}

void led_timer_callback(unsigned long data)
{
    struct net_local *lp ;
    static unsigned int last_tx_pkt_count,last_rx_pkt_count;

    lp=(struct net_local *)data;
    if( (lp->stats.tx_packets - last_tx_pkt_count) || 
        (lp->stats.rx_packets - last_rx_pkt_count) )
    {
        /* There was some activity.So toggle the LED */
        toggle_wlan_led(); 
    }
    /* Store the current stats */
    last_tx_pkt_count = lp->stats.tx_packets;
    last_rx_pkt_count = lp->stats.rx_packets;
    /* Rescheduling Timer */
    mod_timer(lp->timer , jiffies + 25 ) ;
}

int keepalive_timer_init(struct net_device * dev)
{

    struct net_local * lp = dev->priv;

    if(lp->keepalive_timer == NULL)
    {
        lp->keepalive_timer=kmalloc(sizeof(struct timer_list),GFP_KERNEL);
        if( lp->keepalive_timer == NULL)
        {
            DEBUG(DBG_ERR ,"%s , Memory Allocation Failed \n" , __FUNCTION__);
            return -1;
        }    
        /* This Work Queue is for sending a message to User Land through Netlink */ 
        INIT_WORK( &(lp->NL_WorkQ) , (void *)send_netlink_msg, lp->nl_sk);

        lp->keepalive_timer->expires=jiffies + 5; /* In jiffies. One jiffie equal to 10ms */
        lp->keepalive_timer->function= keepalive_timer_callback;
        lp->keepalive_timer->data=(unsigned long)dev;
        init_timer(lp->keepalive_timer);
        mod_timer(lp->keepalive_timer , jiffies + 5);
    }
    return 0;
}


static int led_timer_init(struct net_local * lp)
{
    
    lp->timer=kmalloc(sizeof(struct timer_list),GFP_KERNEL);
    if( lp->timer == NULL)
    {
        DEBUG(DBG_ERR ,"%s , Memory Allocation Falied \n" , __FUNCTION__);
        return -1;
    }    
    lp->timer->expires=jiffies + 5; /* In jiffies. One jiffie equal to 10ms */
    lp->timer->function= led_timer_callback;
    lp->timer->data=(unsigned long)lp;
    init_timer(lp->timer);
    mod_timer(lp->timer , jiffies + 5);
    return 0;
}

void send_netlink_msg(struct sock *nl_sk)
{
    struct sk_buff *skb = NULL;
    struct nlmsghdr * nlh = NULL;
    printk("-->Reset Paed .. \n");

    skb=alloc_skb(NLMSG_SPACE(MAX_PAYLOAD),GFP_ATOMIC);
    nlh = (struct nlmsghdr *)skb->data;
    nlh->nlmsg_len = NLMSG_SPACE(MAX_PAYLOAD);
    nlh->nlmsg_pid = 0;  /* from kernel */
    nlh->nlmsg_flags = 0;
    strcpy(NLMSG_DATA(nlh), "Greeting from kernel!");
    /* sender is in group 1<<0 */
    NETLINK_CB(skb).groups = 1;
    NETLINK_CB(skb).pid = 0;  /* from kernel */
    NETLINK_CB(skb).dst_pid = 0;  /* multicast */
    /* to mcast group 1<<0 */
    NETLINK_CB(skb).dst_groups = 1;
    /*multicast the message to all listening processes*/
    netlink_broadcast(nl_sk, skb, 0, 1, GFP_ATOMIC);


}


module_init(sm_drv_stun_init);
module_exit(sm_drv_stun_cleanup);
