#include <linux/init.h>
#ifndef  _SM_DRV_STUN_H
#define   _SM_DRV_STUN_H


#include "sm_drv.h"

/* WLAN GPIO-LED Definitions */
#define LINK_UP 1
#define LINK_DOWN 0
#define WLAN_SET 1
#define WLAN_CLEAR 0
#define LED_TIMEOUT 250/1000
#define MAX_PAYLOAD 1024
#define MOD_LOAD_ERROR -1
#define DELAY			10
#define SUCCESS			0
#define ALLOC_ERR		-2
#define FOPEN_ERR		-3
#define ERROR			-1

#define STUN_PHY_TO_VIRT(addr) (addr + 0xcf800000)

#define STUN_READ_REG(_x) readl(_x)
#define STUN_WRITE_REG(_addr, _val) writel(_val, _addr)
#define STUN_WRITE_REG_2(_addr, _val) memcpy_toio((void *)_addr,_val,4)

#define STUN_WRITE_MEM(_MemBase,_window,_offset,_val)  \
					  {\
						uint32_t temp;  \
						temp =STUN_READ_REG(_MemBase + STUN_HIF_MAC_MEM_BASE);\
					       STUN_WRITE_REG(_MemBase+STUN_HIF_MAC_MEM_BASE,_window);\
						STUN_WRITE_REG(_offset, _val);\
						STUN_WRITE_REG(_MemBase+STUN_HIF_MAC_MEM_BASE,temp);\
					  }
						  

#define STUN_DEBUG_LEVEL 1
#define STUN_DEBUG_OUTPUT       printk

#if (STUN_DEBUG_LEVEL >= 1)
#define STUN_DEBUG_1(fmt,args...)   STUN_DEBUG_OUTPUT(fmt,##args)
#else
#define STUN_DEBUG_1(fmt,args...)   while(0) { STUN_DEBUG_OUTPUT(fmt,##args); }
#endif
#if (STUN_DEBUG_LEVEL >= 2)
#define STUN_DEBUG_2(fmt,args...)   STUN_DEBUG_OUTPUT(fmt,##args)
#else
#define STUN_DEBUG_2(fmt,args...)   while(0) { STUN_DEBUG_OUTPUT(fmt,##args); }
#endif
#if (STUN_DEBUG_LEVEL >= 3)
#define STUN_DEBUG_3(fmt,args...)   STUN_DEBUG_OUTPUT(fmt,##args)
#else
#define STUN_DEBUG_3(fmt,args...)   while(0) { STUN_DEBUG_OUTPUT(fmt,##args); }
#endif

#define STUN_SIZE_IN_PTR sizeof(uint32_t)
#define STUN_SIZE_OUT_PTR sizeof(uint32_t)

#define STUN_DBG_POOL_SIZE 0xcd0  
#define STUN_DBG_SIZE_H2S 0x50
#define STUN_DBG_SIZE_S2H 0xc80 
#define STUN_MAX_STUNUART_TIMEOUT 100
/* Mem accesser funtion */
#define STUN_UART_WRITE32( _regadd, _val)  writel(_val,_regadd)
#define STUN_UART_READU32( _regadd)  readl(_regadd)

#define STUN_UART_WRITE16( _regadd, _val)  writew(_val,_regadd)
#define STUN_UART_READU16( _regadd)  readw(_regadd)

#define STUN_UART_WRITE8( _regadd,_val) writeb(_val, _regadd)
#define STUN_UART_READU8( _regadd)  readb(_regadd)
/* Register Addresses */
#define STUN_REG_BASE	(0x30002000)

#define MAS_HOST_MEM_BASE_REG (0x00000000)



/* Register Offsets */
#define STUN_HIF_CSR	(0x78)   /* Control/Status register */
#define STUN_HIF_MAS_HOST_MEM_ADDR	(0x60)  /* Master Host Memory Address */
#define STUN_HIF_MAC_MEM_BASE 	(0x30) /* Mac Mem Base Reg. for defining the 
										  device memory window */
#define STUN_HIF_HOST_INT	(0x010)
#define STUN_HIF_HOST_INT_ACK   (0x14)
#define STUN_HIF_GPR1	(0x20)
#define STUN_HIF_GPR2	(0x24)
#define STUN_HIF_ARM_INT	(0x00)
#define STUN_MEM_WINDOW	(0x01000)
#define STUN_HIF_HOST_INT_EN      (0x018) 
#define STUN_HIF_HOST_INT_DEV_INIT      (0x04)
#define STUN_HIF_HOST_INT_DEV_UPDATE      (0x02)
#define STUN_HIF_ARM_INT_UPDATE 	(0x02)
#define STUN_HIF_ARM_INT_RESET  (0x01)
#define STUN_CONTROL_BASE (0x20) /* Control Block Base Register */

/* Bit Defintions */
#define STUN_HIF_CSR_RESET	 	(1<<28) /* Assert Reset Signal */ 
#define STUN_HIF_CSR_CLKRUN     (1<<23) 
#define STUN_HIF_CSR_RAMBOOT    (1<<29)
#define STUN_HIF_CSR_OVERRIDE   (1<<31)
#define STUN_HIF_CSR_HALTED     (1<<30)
#define STUN_HIF_CSR_CPU_EN     (1<<27)
#define STUN_HIF_HOST_INT_READY	(0x2000)
#define STUN_HIF_INT_ABORT	(1<<5)
#define STUN_HIF_ARM_INT_UART_DR	(0x8000)
#define STUN_HIF_HOST_INT_PCIUART_CTS	(0x4000)

#define STUN_HIF_MEM_WINDOW_SIZE	(0x01000)   /* Size of memory window*/
#define STUN_IMAGE_FILE  "/isfs/firmware.bin"

#ifndef TRUE_PDA
#define STUN_PDA_FILE      "/isfs/pda.bin"
#else
//#define STUN_PDA_FILE      "/isfs/truepda.bin"
#define STUN_PDA_FILE      "/var/pda/truepda.bin"
#endif

#define  STUN_FW_SRAMDEST		(0x1ffe0000)
#define  STUN_FW_SDRAMDEST  	(0x20000000)
#define  STUN_MAX_WAIT_BIT_MASK (0x00ffffff)
#define  STUN_RAM_BOOT_NEEDED   (0x80000000)


#define  STUN_INIT_TIMEOUT       (100000)


#define STUN_AHB_IRQNO		(21)


#define  PDA_INFO_SIZE    (8*1024)


/* CONTROL BLOCK DEFS  */
#define CB_QCOUNT                       6
#define CB_RX_DATAQ                     0
#define CB_TX_DATAQ                     1

#define CB_RX_HIGHDATAQ                 2
#define CB_TX_HIGHDATAQ                 3
#define CB_RX_CTRLQ                     4
#define CB_TX_CTRLQ                     5


#define STUN_QUEUE_COUNT       	(6) /* 6 queues between card and host */
#define STUN_RX_DATA_QSIZE     	(8)
#define STUN_TX_DATA_QSIZE    	(32)
#define STUN_RX_CTRL_QSIZE     	(4)
#define STUN_TX_CTRL_QSIZE     	(4)

#define STUN_RX_DATA_Q         	(0)
#define STUN_TX_DATA_Q         	(1)

#define STUN_RX_HIGH_DATA_Q    (2)
#define STUN_TX_HIGH_DATA_Q    (3)
#define STUN_RX_CTRL_Q         	(4)
#define STUN_TX_CTRL_Q         	(5)

/* Must be multiples of 2 ! */
#define CB_RX_DATA_QSIZE                8
#define CB_TX_DATA_QSIZE                32
#define CB_RX_HIGH_DATA_QSIZE      8
#define CB_TX_HIGH_DATA_QSIZE      32
#define CB_RX_CTRL_QSIZE                 4
#define CB_TX_CTRL_QSIZE                 4


#define STUN_HIF_HOST_INT_EN_DEV_INIT  (0x04) /* Triggered by device to tell that now 
                                                   * firmware is running ok
                                                   */
#define STUN_HIF_HOST_INT_EN_UPDATE     (0x02)   /* Queue status updated  */ 

#define STUN_HIF_HOST_INT_EN_UART_DR   (0x8000) /* Data Ready */

#define STUN_HIF_INT_ENABLES    ( STUN_HIF_HOST_INT_EN_DEV_INIT | \
                                      STUN_HIF_HOST_INT_EN_UPDATE | \
		                              STUN_HIF_HOST_INT_EN_UART_DR ) 


#define FRG_FLAG_MF        0x01
#define FRG_FLAG_CONTROL   0x02
#define FRG_FLAG_VSS       SM_FRAME_VSS /* 0x04 */

struct stun_fragment
{
    uint32_t host_address;  /* Address of buffer in host which PCI master can read */
    uint16_t size;          /* Length of buffer */
    uint8_t ssnr;          /* SSNR Field is used in case of Virtual AP */
    uint8_t flags;         /* Buffer options */
};

struct stun_controlblock
{
    uint32_t driver_curr_frag[ STUN_QUEUE_COUNT ];
    uint32_t device_curr_frag[ STUN_QUEUE_COUNT ];
    struct stun_fragment rx_data[ STUN_RX_DATA_QSIZE ];
    struct stun_fragment tx_data[ STUN_TX_DATA_QSIZE ];
    struct stun_fragment rx_high_data[ STUN_RX_DATA_QSIZE ];
    struct stun_fragment tx_high_data[ STUN_TX_DATA_QSIZE ];
    struct stun_fragment rx_control[ STUN_RX_CTRL_QSIZE ];
    struct stun_fragment tx_control[ STUN_TX_CTRL_QSIZE ];
};


struct s_fragq_hdl 
{
    struct sk_buff *skb;
    uint32_t sdram_phy_address;
};

struct local_control_block
{
    struct s_fragq_hdl rx_data_hdl[CB_RX_DATA_QSIZE];
    struct s_fragq_hdl tx_data_hdl[CB_TX_DATA_QSIZE];
    struct s_fragq_hdl rx_high_data_hdl[CB_RX_DATA_QSIZE];
    struct s_fragq_hdl tx_high_data_hdl[CB_TX_DATA_QSIZE];
    struct s_fragq_hdl rx_control_hdl[CB_RX_CTRL_QSIZE];
    struct s_fragq_hdl tx_control_hdl[CB_TX_CTRL_QSIZE];
};

/* Local Info */
struct stun_ahb_hif_local_data
{
    uint32_t MemBase;
    uint8_t * stungSdRamAddr;
    struct stun_controlblock        *control_block;         /* device control block */
    struct local_control_block   local_cb_handle;       /* local handles for each queue in the control block */
    uint32_t stun_ahbcontrolblockaddress;
    spinlock_t cb_lock;
    uint32_t                     fragment_c[CB_QCOUNT]; /* fragment consume index (CPU Endianness!) */
    uint32_t                     generate_update_interrupt;
    volatile int32_t *in;
    volatile int32_t *out;
    int32_t  *shared_dbg_htos;
    int32_t   *shared_dbg_stoh;
};

static const uint32_t frag_queue_size[CB_QCOUNT] = {CB_RX_DATA_QSIZE,
                                         CB_TX_DATA_QSIZE,
                                         CB_RX_HIGH_DATA_QSIZE,
                                         CB_TX_HIGH_DATA_QSIZE,
                                         CB_RX_CTRL_QSIZE,
                                         CB_TX_CTRL_QSIZE };
static inline void fragq_init(struct net_device *dev, int queue)
{
    struct net_local *lp = dev->priv;
    struct stun_controlblock *cb = ((struct stun_ahb_hif_local_data*)lp->hif_local_data)->control_block;
    struct stun_ahb_hif_local_data *hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    unsigned long flags;

    spin_lock_irqsave(&hif_lp->cb_lock, flags);
    cb->driver_curr_frag[queue] = cpu_to_le32(0);
    cb->device_curr_frag[queue] = cpu_to_le32(0);
    hif_lp->fragment_c[queue] = cpu_to_le32(0);
    spin_unlock_irqrestore(&hif_lp->cb_lock, flags);
}

static inline int fragq_drv_pspace(struct net_local *lp, int queue)
{
    struct stun_controlblock *cb = ((struct stun_ahb_hif_local_data*)lp->hif_local_data)->control_block;
    struct stun_ahb_hif_local_data *hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    int ret;
    unsigned long flags;

    spin_lock_irqsave(&hif_lp->cb_lock, flags);
    ret = (int)(frag_queue_size[queue] -
                (le32_to_cpu(cb->driver_curr_frag[queue]) -
                 hif_lp->fragment_c[queue]));
    spin_unlock_irqrestore(&hif_lp->cb_lock, flags);

    return ret;
}

/* Tell the device the fragment queues are updated. */
static inline void sm_drv_stun_trigger_device(struct stun_ahb_hif_local_data *hif_lp, int event)
{
  STUN_WRITE_REG(hif_lp->MemBase + STUN_HIF_ARM_INT , event);
}


/* Returns the number of entries that can be consumed by the driver */
static inline int fragq_drv_cspace(struct net_local *lp, int queue)
{
    struct stun_controlblock *cb = ((struct stun_ahb_hif_local_data*)lp->hif_local_data)->control_block;
    struct stun_ahb_hif_local_data *hif_lp = (struct stun_ahb_hif_local_data*)lp->hif_local_data;
    int ret;
    unsigned long flags;

    spin_lock_irqsave(&hif_lp->cb_lock, flags);
    ret = (int)(le32_to_cpu(cb->device_curr_frag[queue]) -
                hif_lp->fragment_c[queue]);
    spin_unlock_irqrestore(&hif_lp->cb_lock, flags);

    return ret;
}

/* Returns the number of entries that can be produced by the driver */
struct wrm_block
{
    uint32_t pda_dest_addr;
    uint32_t pda_size;
    uint32_t pda_src_addr;
};

/* Function decs */	
int stun_ResetCard(struct stun_ahb_hif_local_data *);
int stun_UploadFirmware(struct stun_ahb_hif_local_data *);
int stun_UploadPDA(struct stun_ahb_hif_local_data * );
int stun_RunFirmware(struct stun_ahb_hif_local_data *);
inline int ahb_MapAddressToSlaveAddress(
    uint32_t           stungSdRamAddr,
    void*               buffer,
    uint32_t           length,
    uint32_t           *AHBAddr );

struct net_device *sm_drv_stun_ahb_probe(struct net_device *dev, long ioaddr, int irq);
int sm_drv_stun_ahb_hif_up(struct net_device *dev);
void sm_drv_stun_ahb_bh(struct net_device *dev);
int sm_drv_stun_ahb_hif_down(struct net_device *dev);
void sm_drv_stun_ahb_cli(struct net_device *dev);

/* Memory leak fix*/
int sm_drv_stun_add_rx_queue(struct net_device *dev, int queue);
int fragq_drv_pspace(struct net_local *lp, int queue);
void sm_drv_add_tx_ctrl_queue(struct net_device *dev, uint8_t *pMsg, int32_t length);
int sm_drv_stun_interrupt(int irq, void *config, struct pt_regs *regs);
void sm_drv_stun_check_txed(struct net_device *dev);
void sm_drv_stun_ahb_bh(struct net_device *dev);
void sm_drv_stun_frag_txed(struct net_device *dev, struct stun_fragment *frag,
                           struct s_fragq_hdl *fragq_hdl);
void sm_drv_stun_frag_rx(struct net_device *dev, struct stun_fragment *frag,
                           struct s_fragq_hdl *fragq_hdl , int queue);

uint32_t sm_drv_uart(struct stun_ahb_hif_local_data *lp );

uint32_t  sm_drv_tell_to_radio(struct stun_ahb_hif_local_data *lp );

uint32_t  wait_for_dr_bit(struct stun_ahb_hif_local_data *lp);
int sm_drv_print_radio2host_block(struct stun_ahb_hif_local_data *lp);
void sm_drv_add_tx_queue(struct net_device *dev, struct sk_buff *skb);
void sm_drv_ahb_free_rx_queue(struct net_device *dev, int queue);
void fragq_produce(struct net_local *lp, int queue,
                   struct stun_fragment *frag, struct s_fragq_hdl *fragq_hdl);

struct stun_fragment *fragq_consume(struct net_local *lp, int queue,
                                      struct s_fragq_hdl **handlep);
#ifdef VSSDEV_SUPPORT
inline int check_skb_for_vss(struct sk_buff *skb, uint8_t *vssid);
#endif

/* WLAN GPIO-LED Definitions */
static int led_timer_init(struct net_local * lp);
void control_wlan_led(unsigned long int GPIONum , int mode);
void wlan_link_status(int link);

void send_netlink_msg(struct sock * );

#endif

