#ifdef IWCONFIG_SUPPORT
#include <linux/init.h>
#include <linux/types.h>
#include <linux/sched.h>
#include <linux/unistd.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/pci.h>
#include <linux/string.h>
#include <linux/poll.h>
#include <asm/uaccess.h>
#include <linux/usb.h>
#include <linux/if_arp.h>
#include <net/iw_handler.h>
#include <linux/wireless.h>
#include "sm_drv.h"
#include "sm_drv_ioctl.h"
struct iw_statistics iw_stats;
#define VERSIONID      "stun driver version:"

/*#define PAED_CONF_FILE  "/etc/paed/paed.wlan0.conf"*/
#define ACL_CONF_FILE  "/etc/paed/acl.conf" 
#define FOPEN_ERR            -3

char *fgets_local(struct file *infile,char *line,loff_t *);
static int update_file(struct file *ifp, char *str,int value);
int update_acllist(void);
static int set_key_conffile(struct file *ifp, char *str,uint8_t *value,int len);

//host apd support
#ifdef HOSTAPDCONF
#define PAED_CONF_FILE "/etc/paed/hostapd.wlan0.conf"
#else	
#define PAED_CONF_FILE  "/etc/paed/paed.wlan0.conf"
#endif

#ifdef HOSTAPDCONF
	char os_string[20]= "os_enable ";	 
	char wep_string[20]= "wep_enable=";
	char auth_string[20] = "auth_algo=";	
#else
        char os_string[20]= "os_enable ";
        char wep_string[20]= "wep_enable ";
#endif


//end



int get_qual_stat(struct net_device *ndev,struct iw_quality *qual)
{  
	struct net_local  *lp = ndev->priv;
    struct obj_bss bss;
    uint8_t bssid[6];
    uint32_t noise = 0,snr=0;
    int retval=0;
	if(lp->device_state != DEVSTATE_ACTIVE)  
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
 
    if( retval = sm_drv_get( ndev, DOT11_OID_NOISEFLOOR, 
 			   &noise, sizeof(uint32_t)), retval < 0 )
    {
         DEBUG(DBG_IWCONFIG, " %s noise floor failed : Error %d \n",
  	    __FUNCTION__,retval);
         return -EIO;
     }
  
    /*get bssid of the associated AP*/
    if( retval = sm_drv_get( ndev, DOT11_OID_BSSID,
  			   bssid,	sizeof(bssid) ), retval < 0 )
    {
         DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_BSSID \
  	    			failed\n",__FUNCTION__ ,retval );
         return -EIO;
     }
  	
    memcpy(bss.address,bssid,6);

	/*get the associated AP's information*/
    if( retval = sm_drv_get( ndev, DOT11_OID_BSSFIND,
  			   &bss,	sizeof(obj_bss) ), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_BSSFIND \
  	    			failed\n",__FUNCTION__ ,retval );
        return -EIO;
     }
  	
    qual->level 	= (uint8_t) bss.rssi;
    qual->noise 	= (uint8_t) noise;
    snr         	= (uint8_t) (bss.rssi - noise);
    qual->updated 	= 0x7;
    if(snr <0)
        qual ->qual = 0;
    else
    {
        if( snr >=0 && snr < 40)
            qual->qual = (5*snr)/2;
        else
            qual->qual = 100;
    }
    
    return 0;
}

struct iw_statistics *sm_drv_wireless_stats(struct net_device *ndev)
{
    struct net_local  *lp = ndev->priv;

    char retval;
    uint32_t misc1,misc2;
    uint32_t fmisc1,fmisc2;
	 
	if(lp->device_state != DEVSTATE_ACTIVE )
        return NULL ;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
  
    /*get the link quality information*/
    get_qual_stat(ndev,&iw_stats.qual);
    iw_stats.status = lp->device_state;
  
    if( retval = sm_drv_get( ndev, DOT11_OID_PRIVRXFAILED,
  			   &iw_stats.discard.code, sizeof(uint32_t) ), retval < 0 )
    {
         DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_PRIVRXFAILED \
  	    			failed\n",__FUNCTION__ ,retval );
         return NULL;
     }
  	
    if( retval = sm_drv_get( ndev,  DOT11_OID_MPDUTXFAILED,
  			   &iw_stats.discard.retries, sizeof(uint32_t) ), retval < 0 )
    {
         DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_MPDUTXFAILED \
  	    			failed\n",__FUNCTION__ ,retval );
         return NULL;
     }
  	
  
    if( retval = sm_drv_get( ndev,DOT11_OID_NUMMISSEDBEACONS, 
  			   &iw_stats.miss.beacon, sizeof(uint32_t) ), retval < 0 )
    {
         DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_NUMMISSEDBEACONS \
                                  failed\n",__FUNCTION__ ,retval );
         return NULL;
    }
  
    
    if( retval = sm_drv_get( ndev,DOT11_OID_FRAMEABORTS,
  		   &fmisc1, sizeof(uint32_t) ), retval < 0 )
    {
         DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_FRAMEABORTS \
                                  failed\n",__FUNCTION__ ,retval );
         return NULL;
    }
    if( retval = sm_drv_get( ndev,DOT11_OID_FRAMEABORTSPHY,
  			   &fmisc2, sizeof(uint32_t) ), retval < 0 )
    {
         DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_FRAMEABORTSPHY \
                                  failed\n",__FUNCTION__ ,retval );
         return NULL;
     }
  
    if( retval = sm_drv_get( ndev,DOT11_OID_PRIVRXNOKEY,
  			   &misc1, sizeof(uint32_t) ), retval < 0 )
    {
         DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_PRIVRXNOKEY \
                                  failed\n",__FUNCTION__ ,retval );
         return NULL;
     }
    if( retval = sm_drv_get( ndev,DOT11_OID_PRIVRXPLAIN,
  			   &misc2, sizeof(uint32_t) ), retval < 0 )
    {
         DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_PRIVRXPLAIN \
                                  failed\n",__FUNCTION__ ,retval );
         return NULL;
    }
  
    if( retval = sm_drv_get( ndev,GEN_OID_MSDURXFAILED,
  			   &iw_stats.discard.nwid, sizeof(uint32_t) ), retval < 0 )
    {
         DEBUG(DBG_IWCONFIG, "%s: Error %d, GEN_OID_MSDURXFAILED \
                                  failed\n",__FUNCTION__ ,retval );
         return NULL;
    }
  
    iw_stats.discard.fragment= fmisc1 + fmisc2;
    iw_stats.discard.misc= misc1 + misc2;
  	
    return (struct iw_statistics*)&iw_stats;
}



static ChannelData_t ChannelDataTable[] = 
{
    { // 2.4 GHz 
      // N.B. Placing first in table forces channels 1-14 to 2.4 GHz, not 5GHz
        2412, // First Channel MHz
        2484, // Last  Channel MHz
        5,    // Channel Spacing
        1,    // First Channel Number
        14,   // Last  Channel Number
    },
    { // 5 GHz
        5000, // First Channel MHz
        6000, // Last  Channel MHz
        5,    // Channel Spacing
        0,    // First Channel Number
        199,  // Last  Channel Number
    },
    { // 4.9 GHz
        4910, // First Channel MHz
        4995, // Last  Channel MHz
        5,    // Channel Spacing
        238,  // First Channel Number
        256,  // Last  Channel Number
    },
    {
        0     // 0MHz is terminator
    }
};

// @rdesc 0 if conversion failed
static uint32_t channelTokHz( uint32_t channel)
{
    
    int      i;
    uint32_t result = 0; 
    
    DEBUG(DBG_IWCONFIG	, " %s \n", __FUNCTION__);
    
    for (i = 0; ChannelDataTable[i].firstChannelMHz > 0; ++i)
    {
        if ((channel >= ChannelDataTable[i].firstChannelNbr)
         && (channel <= ChannelDataTable[i].lastChannelNbr))
        {
            channel -= ChannelDataTable[i].firstChannelNbr;
            channel *= ChannelDataTable[i].spacing;
            channel += ChannelDataTable[i].firstChannelMHz;
            // 2.4GHz channel 14 exception
            if (channel == 2477)
            {
                channel = 2484;
            }
            result = channel * 1000; // To kHz
        }
    }
    DEBUG(DBG_IWCONFIG	, " %s channel %d to Freq %d\n", __FUNCTION__,channel,result);
    return result;
}

/* Use a #define for the ESSID Length instead of 33 */
static int sm_drv_set_essid(struct net_device *ndev, 
		struct iw_request_info *info, struct iw_point *dwrq, char *extra)
{
	struct net_local  *lp = ndev->priv;
	struct obj_ssid essid;
	int retval = 0;
	
     unsigned int data = 0xd;
    DEBUG(DBG_IWCONFIG	, " %s \n", __FUNCTION__);

    DEBUG(DBG_IWCONFIG	, " %s essid to set %s \n", __FUNCTION__,extra);
    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

	memset(essid.octets, 0, 33);
	if (dwrq->flags && dwrq->length) {
		if (dwrq->length > min(33, IW_ESSID_MAX_SIZE + 1))
			return -E2BIG;
		essid.length = dwrq->length - 1;
		memcpy(essid.octets, extra, dwrq->length);
	} 
	else
		essid.length = 0;	

    if( retval = sm_drv_set( ndev,DOT11_OID_SSID , &essid, sizeof(struct obj_ssid)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s set essid failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }

    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &data , sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    }

	return 0;
}

static int sm_drv_get_essid(struct net_device *ndev, 
		struct iw_request_info *info, struct iw_point *dwrq, char *extra)
{

	struct net_local  *lp = ndev->priv;
	struct obj_ssid essid ;
	int retval = 0;

	memset(essid.octets,0,IW_ESSID_MAX_SIZE);
	
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
  //  if(lp->device_state != DEVSTATE_ACTIVE )
  //      interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;
	

    if( retval = sm_drv_get( ndev,DOT11_OID_SSID, &essid, sizeof(struct obj_ssid) ), retval < 0 )
    {
       	DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_SSID failed\n",__FUNCTION__ ,retval );
		return -EIO;
    }
    else
    {

		if(essid.length) {
			dwrq->flags = 1;
			dwrq->length = min(IW_ESSID_MAX_SIZE, essid.length + 1);
		} else {
			dwrq->flags = 0;
			dwrq->length = 0;
		}
		essid.octets[essid.length] = '\0';
		memcpy(extra, essid.octets, dwrq->length);
		DEBUG(DBG_IWCONFIG	, "%s: DOT11_OID_SSID returned %s\n",__FUNCTION__, extra);
		return 0;
	}
}


static int sm_drv_set_freq(struct net_device *ndev, struct iw_request_info *info,
		  struct iw_freq *fwrq, char *extra)
{
	struct net_local  *lp = ndev->priv;
	int retval = 0;
	unsigned int freq = 0;
	unsigned int data = 0;
	unsigned int channel = 0;

    DEBUG(DBG_IWCONFIG	, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

    DEBUG(DBG_IWCONFIG	, " %s fwrq->m %d e %d \n", __FUNCTION__,fwrq->m,fwrq->e);
	/*prepare the structure for the set object */
	if(fwrq->m < 1000)
		/*structure value contains a channel indication*/
		channel = fwrq->m;
	else{
		/*structure conatins a frequency indication and fwrq->e = 1 */
		if (fwrq->e != 1)
			return -EINVAL; 

		freq = fwrq->m / 100000;
		if((freq >=2412) && (freq <= 2484)){
			while((channel  < 14) && (freq != frequency_list_bg[channel ]))
				channel ++;
			if( channel  >= 14)
				return -EINVAL; 
		}else if ((freq >= (int)5170) && (freq <= (int)5320)){
			while((channel  < 12) && (freq != frequency_list_a[channel ]))
				channel ++;
			if( channel  >= 12)
				return -EINVAL; 
		}else
			return -EINVAL; 

		channel ++;
	}

	freq = channelTokHz(channel);

    if( retval = sm_drv_set( ndev, DOT11_OID_CHANNELMODE, &data , sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s channel mode failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }

	data = 0xd;
    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &data , sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s commit failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }

    if( retval = sm_drv_set( ndev, DOT11_OID_FREQUENCY, &freq, sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s set frequency failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }

	data = 0xd;
    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &data , sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s commit failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
	return 0;

}

static int sm_drv_get_freq(struct net_device *ndev, struct iw_request_info *info,
		  struct iw_freq *fwrq, char *extra)
{
	struct net_local  *lp = ndev->priv;
	unsigned long freq = 0;
	int retval = 0;
	
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

    if( retval = sm_drv_get( ndev, DOT11_OID_FREQUENCY, &freq, sizeof(unsigned long) ), retval < 0 )
    {
       	DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_FREQUENCY failed\n",__FUNCTION__ ,retval );
		return -EIO;
    }
    else
    {
		DEBUG(DBG_IWCONFIG	, "%s: DOT11_OID_FREQUENCY returned %ld\n",__FUNCTION__, freq);
	}

	freq = freq/1000;
	fwrq->m = freq;
	fwrq->e = 6;

	return 0;
}

static int sm_drv_get_range(struct net_device *ndev, 
		struct iw_request_info *info, struct iw_point *dwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	struct iw_range *range = NULL;
	uint32_t length = 0;
	struct obj_frequencies *freq = NULL;
	uint8_t *buff = NULL;
	uint8_t *data = NULL;
	int retval = 0;
	int m, i, temp, start, num;

    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

	length = sizeof(struct obj_frequencies) + (sizeof(u16) * IWMAX_FREQ);
	buff = (uint8_t *)kmalloc(length, GFP_KERNEL);

    if( retval = sm_drv_get( ndev, DOT11_OID_SUPPORTEDFREQUENCIES, buff, length), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s supported frequencies failed : Error %d \n", __FUNCTION__,retval);
		kfree(buff);
		return -EIO;
    }

	range = (struct iw_range *) extra;
	memset(range, 0, sizeof(struct iw_range));
	dwrq->length = sizeof(struct iw_range);
	range->we_version_source = WIRELESS_EXT;
	range->we_version_compiled = WIRELESS_EXT;

	range->min_nwid=range->max_nwid=0;
		
	freq = (struct obj_frequencies *)buff;
	num = le16_to_cpu(freq->nr);
	m = min(IW_MAX_FREQUENCIES, (int)le16_to_cpu(freq->nr));
	range->num_channels =  range->num_frequency = m;
	start = 0;	//can be any number

	for(i = 0; (i<m) && ((start+i)<num); i++)
	{
		temp = le16_to_cpu(freq->mhz[start+i]);
		range->freq[i].m = temp;
		range->freq[i].e = 6;
		if(temp < 2484)
			range->freq[i].i = (temp - 2407)/5;
		else if(temp == 2484)
			range->freq[i].i = 14;
		else if(temp < 5000)
			range->freq[i].i = (temp - 3720)/5;
		else
			range->freq[i].i = (temp - 5000)/5;
	}

	kfree(freq);

	length = IWMAX_BITRATES + 1;
	buff = (uint8_t *)kmalloc(length, GFP_KERNEL);
	memset(buff,0,length);

    if( retval = sm_drv_get( ndev, DOT11_OID_SUPPORTEDRATES, buff, length), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s supported rates failed : Error %d \n", __FUNCTION__,retval);
		kfree(buff);
		return -EIO;
    }

	i = 0;
	data = buff;
	while ((i < IW_MAX_BITRATES) && (*data != 0)) 
	{
		range->bitrate[i] = (__s32) (*data >> 1);
		range->bitrate[i] *= 1000000;
		//range->bitrate[i] = *data;
		i++;
		data++;
	}
	range->num_bitrates = i;
	
	if(range->num_bitrates > 2)
		range->throughput = 5*1000*1000;
	else
		range->throughput = 1.5*1000*1000;
	kfree(buff);

    if( retval = sm_drv_get( ndev, DOT11_OID_OUTPUTPOWER, &temp, sizeof(int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s outputpower failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }

    DEBUG(DBG_IWCONFIG, " %s get outputpower %d \n", __FUNCTION__,data);
    range->num_txpower=1;
    range->txpower[0]=temp/4;

    if( retval = sm_drv_get( ndev, DOT11_OID_PSM, &temp, sizeof(int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s outputpower failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }

    DEBUG(DBG_IWCONFIG, " %s get outputpower %d \n", __FUNCTION__,data);
    range->pm_capa=temp;

    return 0;
}

static int sm_drv_set_rate(struct net_device *ndev, struct iw_request_info *info,
		 struct iw_param *vwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	uint32_t profile, length;
	uint8_t *buff = NULL ;
	int32_t idx = 0;
	uint8_t *data = NULL;
	uint32_t rate = 0;
	int retval = 0, i = 0;
	unsigned int commit_data = 0xd;

    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

	if (vwrq->value == -1) 
	{
		/*auto mode No limit */
		profile = 1;
    	if( retval = sm_drv_set( ndev, DOT11_OID_PROFILES, &profile, sizeof(unsigned int)), retval < 0 )
    	{
    		DEBUG(DBG_IWCONFIG	, " %s profiles failed : Error %d \n", __FUNCTION__,retval);
			return -EIO;
    	}
		return 0;
	}

	length = IWMAX_BITRATES + 1;
	buff = (uint8_t *)kmalloc(length, GFP_KERNEL);
	memset(buff, '\0', length);
    if( retval = sm_drv_get( ndev, DOT11_OID_SUPPORTEDRATES, buff, length), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s supported rates failed : Error %d \n", __FUNCTION__,retval);
		kfree(buff);
		return -EIO;
    }
	buff[length-1] = '\0';

	rate = (uint32_t)(vwrq->value / 500000);

	data = (uint8_t *)buff;
	i = 0;
	while(data[i]) {
		if(rate && ((uint32_t)data[i] == rate))
			break;
		i++;
	}
	if(!data[i])
		return -EINVAL;
	kfree(data);
	idx = i;
#if 1    
if( retval = sm_drv_set( ndev, DOT11_OID_ALOFT_FIXEDRATE, &idx, sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s aloft fixedrate failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
#endif

    if( retval = sm_drv_set( ndev, GEN_OID_LINKSTATE, &rate, sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s linkstate failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
    return 0;
}

static int sm_drv_get_rate(struct net_device *ndev, struct iw_request_info *info,
		 struct iw_param *vwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	uint32_t bit_rate=0;
	uint32_t length = 0;
	uint8_t *buff = NULL;
	uint8_t *data = NULL;
	int retval = 0;

    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

    if( retval = sm_drv_get( ndev, GEN_OID_LINKSTATE, &bit_rate, sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s linkstate failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }

	vwrq->value = bit_rate * 500000;
	length = IWMAX_BITRATES + 1;

	buff = (uint8_t *)kmalloc(length, GFP_KERNEL);
        memset(buff,0,length);

    if( retval = sm_drv_get( ndev, DOT11_OID_RATES, buff, length), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s rates failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
	data = buff;
	vwrq->fixed = (data[0] != 0) && (data[1] == 0);
	vwrq->flags=1;
        vwrq->disabled=0;
	kfree(data);

	return 0;
}

static int sm_drv_set_scan(struct net_device *dev, struct iw_request_info *info,
		 struct iw_param *vwrq, char *extra)
{
	return 0;
}

inline char *
sm_drv_translate_bss(char *current_ev, char *end_buf,
			struct obj_bss *bss, char noise)
{
	struct iw_event iwe;	/* Temporary buffer */
	short cap;

    DEBUG(DBG_IWCONFIG	, " %s \n", __FUNCTION__);
	/* The first entry must be the MAC address */
	memcpy(iwe.u.ap_addr.sa_data, bss->address, 6);
	iwe.u.ap_addr.sa_family = ARPHRD_ETHER;
	iwe.cmd = SIOCGIWAP;
	current_ev = iwe_stream_add_event(current_ev, end_buf, &iwe, IW_EV_ADDR_LEN);

	/* The ESSID. */
	iwe.u.data.length = (__u16)bss->ssid[0];	
	iwe.u.data.flags = 1;
	iwe.cmd = SIOCGIWESSID;
	current_ev = iwe_stream_add_point(current_ev, end_buf, &iwe, &bss->ssid[1]);

	/* Capabilities */
#define CAP_ESS 0x01
#define CAP_IBSS 0x02
#define CAP_CRYPT 0x10

	/* Mode */
	cap = le16_to_cpu(bss->capinfo);
	iwe.u.mode = 0;
	if (cap & CAP_ESS)
		iwe.u.mode = IW_MODE_MASTER;
	else if (cap & CAP_IBSS)
		iwe.u.mode = IW_MODE_ADHOC;
	iwe.cmd = SIOCGIWMODE;
	if (iwe.u.mode)
		current_ev = iwe_stream_add_event(current_ev, end_buf, &iwe, IW_EV_UINT_LEN);

	/* Encryption capability */
	if (cap & CAP_CRYPT)
		iwe.u.data.flags = IW_ENCODE_ENABLED | IW_ENCODE_NOKEY;
	else
		iwe.u.data.flags = IW_ENCODE_DISABLED;
	iwe.u.data.length = 0;
	iwe.cmd = SIOCGIWENCODE;
	current_ev = iwe_stream_add_point(current_ev, end_buf, &iwe, NULL);

	/* Add frequency. (short) bss->channel is the frequency in MHz */
	iwe.u.freq.m = bss->channel;
	iwe.u.freq.e = 6;
	iwe.cmd = SIOCGIWFREQ;
	current_ev = iwe_stream_add_event(current_ev, end_buf, &iwe, IW_EV_FREQ_LEN);

	/* Add quality statistics */
	iwe.u.qual.level = bss->rssi;
	iwe.u.qual.noise = noise;
	/* do a simple SNR for quality */
	iwe.u.qual.qual = bss->rssi - noise;
	iwe.cmd = IWEVQUAL;
	current_ev = iwe_stream_add_event(current_ev, end_buf, &iwe, IW_EV_QUAL_LEN);

	return current_ev;
}
static int sm_drv_get_scan(struct net_device *ndev, struct iw_request_info *info,
		 struct iw_point *dwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	char *current_ev = extra;
	uint32_t noise = 0;
	int retval = 0;
	int i = 0;
	struct obj_bsslist *bsslist = NULL;
	uint32_t length = 0;
	uint8_t *buff = NULL;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

    if( retval = sm_drv_get( ndev, DOT11_OID_NOISEFLOOR, &noise, sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s noise floor failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
	
	length = sizeof(struct obj_bsslist) + sizeof(struct obj_bss[IWMAX_BSS]);
	buff = (uint8_t *)kmalloc(length, GFP_KERNEL);

    if( retval = sm_drv_get( ndev, DOT11_OID_BSSLIST, buff, length), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s bsslist failed : Error %d \n", __FUNCTION__,retval);
		kfree(buff);
		return -EIO;
    }

	bsslist = (struct obj_bsslist *)buff;

	for (i = 0; i < min(IW_MAX_AP, (int) le32_to_cpu(bsslist->nr)); i++)
		current_ev = sm_drv_translate_bss(current_ev, extra + IW_SCAN_MAX_DATA,
				&(bsslist->bsslist[i]), noise);
	dwrq->length = (current_ev - extra);
	dwrq->flags = 0;	

	kfree(bsslist);
	
    return 0;
}

static int sm_drv_get_name (struct net_device *ndev, struct iw_request_info *info, char *cwrq, char *extra)
{
	struct net_local  *lp = ndev->priv;
	unsigned int data = 0;
	int ret = 0;

    DEBUG(DBG_IWCONFIG	, " %s \n", __FUNCTION__);

    strncpy(cwrq,"IEEE 802.11DS",IFNAMSIZ);

    DEBUG(DBG_IWCONFIG , " %s: Name : %s \n", __FUNCTION__,cwrq);

	return 0;
}

static int sm_drv_commit(struct net_device *ndev, struct iw_request_info *info,
		  char *cwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	unsigned int data = 0xd;
	int retval = 0;

    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &data, sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s commit failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
    return 0;
}

static int sm_drv_get_txpower(struct net_device *ndev, 
		struct iw_request_info *info, struct iw_param *vwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	int data = 0;
	int retval = 0;

    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

    if( retval = sm_drv_get( ndev, DOT11_OID_OUTPUTPOWER, &data, sizeof(int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s outputpower failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }

    DEBUG(DBG_IWCONFIG, " %s get outputpower %d \n", __FUNCTION__,data);

	vwrq->value = data / 4;
	vwrq->fixed = 1;
	/* radio is not turned of
	 * btw: how is possible to turn off only the radio 
	 */
	vwrq->disabled = 0;
        vwrq->flags = IW_TXPOW_DBM;
	return 0;
}
static int sm_drv_set_txpower(struct net_device *ndev,
                struct iw_request_info *info, struct iw_param *vwrq, char *extra)
{

	struct net_local  *lp = ndev->priv ;
        int retval = 0;
	unsigned int commit_data = 0xd;	
	int txp = vwrq->value;
	
	txp = txp*4;	

    	DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    	if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    	
	if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;
    	if( retval = sm_drv_set( ndev,DOT11_OID_OUTPUTPOWER, &txp, sizeof(int)), retval < 0 )
    	{
        	DEBUG(DBG_IWCONFIG      , " %s fragthresh failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    	}
	
  	if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data , sizeof(unsigned int)), retval < 0 )
    	{
        	DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    	}
        return 0;

}

static int sm_drv_set_frag(struct net_device *ndev, 
		struct iw_request_info *info, struct iw_param *vwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	int retval = 0;
	unsigned int commit_data = 0xd;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

    if( retval = sm_drv_set( ndev, DOT11_OID_FRAGTHRESH, &vwrq->value, sizeof(int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s fragthresh failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data , sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    }
	
	return 0;
}

static int sm_drv_get_frag(struct net_device *ndev, 
		struct iw_request_info *info, struct iw_param *vwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	int data = 0;
	int retval = 0;

    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

    if( retval = sm_drv_get( ndev, DOT11_OID_FRAGTHRESH, &data, sizeof(int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s fragthresh failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }

    DEBUG(DBG_IWCONFIG, " %s get frag %d \n", __FUNCTION__,data);
	vwrq->value = data;

	return 0;
}

static int sm_drv_set_sens(struct net_device *ndev, 
		struct iw_request_info *info, struct iw_param *vwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	unsigned int sens;
	int retval = 0;
	unsigned int commit_data = 0xd;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

	/* by default  the card sets this to 20. */
	sens = vwrq->disabled ? 20 : vwrq->value;

    if( retval = sm_drv_set( ndev, DOT11_OID_EDTHRESHOLD, &sens, sizeof(int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s ethreshold failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
    
    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    }	
    
	return 0;
}

static int sm_drv_get_sens(struct net_device *ndev, 
		struct iw_request_info *info, struct iw_param *vwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	unsigned int data = 0;
	int retval = 0;

    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

    if( retval = sm_drv_get( ndev, DOT11_OID_EDTHRESHOLD, &data, sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s edthreshold failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }

    DEBUG(DBG_IWCONFIG, " %s get rts %d \n", __FUNCTION__,data);
	vwrq->value = data;
	vwrq->disabled = (vwrq->value == 0);
	vwrq->fixed = 1;

	return 0;
}

static int sm_drv_set_rts (struct net_device *ndev, struct iw_request_info *info,
		struct iw_param *vwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	int retval = 0;
	unsigned int commit_data = 0xd;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

    if( retval = sm_drv_set( ndev, DOT11_OID_RTSTHRESH, &vwrq->value, sizeof(int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s rtsthresh failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
    
    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    }

	return 0;
}

static int sm_drv_get_rts (struct net_device *ndev, struct iw_request_info *info,
		struct iw_param *vwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	int data = 0;
	int retval = 0;
	
	DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

	/* Sleep until UMAC and LMAC are completely initialized */
	if(lp->device_state != DEVSTATE_ACTIVE )
    	    interruptible_sleep_on(&lp->conf_waitq);

	/* Check if we were woken by a signal */
	if(lp->device_state != DEVSTATE_ACTIVE )
    	    return -EINTR;

	if( retval = sm_drv_get( ndev, DOT11_OID_RTSTHRESH, &data, sizeof(int)), retval < 0 )
	{
    	    DEBUG(DBG_IWCONFIG	, " %s rtsthresh failed : Error %d \n", __FUNCTION__,retval);
	    return -EIO;
	}

	DEBUG(DBG_IWCONFIG, " %s get rts %d \n", __FUNCTION__,data);
	vwrq->value = data;
	return 0;
}
//here
static int set_key_conffile(struct file *ifp, char *str,uint8_t *value,int len)
{

         loff_t offset = 0;
         char data[100];
         int i;

         while(fgets_local(ifp,data,&offset) != NULL)
         {
                char chr,ctemp = ' ';
                loff_t seek_cur;
                char *position;
		loff_t temp;
		uint8_t bspace[26];
                memset(bspace,' ',26);
		
                if ( (position = strstr(data,str)) != NULL )
                {
                        if ( (position = strchr(data, '#')) == NULL )
                        {
                                seek_cur = ifp->f_pos;
                                for ( i = 0 ; i <= strlen(str); i++ ) {
                                        ifp->f_op->read(ifp,&chr,1,&offset);
                                }
					
				temp = offset;
				while(1)
				{	
					ifp->f_op->read(ifp, &ctemp, 1, &temp);
					if(ctemp == '\n')
					break;
					
					//printk("%c",ctemp);
					temp = temp - 1;
					ctemp = ' ';
					ifp->f_op->write(ifp, &ctemp, 1, &temp);
				}
				//for(i=0;i < len; i++)
                               // printk("%c",(unsigned char)value[i]);			        	
				offset = offset - 1;	
                                ifp->f_op->write(ifp,value,len, &offset);
                        }
                }
        }/* while */

return 0;
}



//end

static int sm_drv_set_encode (struct net_device *ndev, 
	struct iw_request_info *info, struct iw_point *dwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	unsigned int data = 0;
	int retval = 0,force=0;
	int authen = DOT11_AUTH_OS, invoke = 0, exunencrypt = 0;

	char string[20];
	char mykey[20];
	mm_segment_t fs;
        struct file *ifp = NULL;


    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

	/* with the new API, it's impossible to get a NULL pointer.
	 * New version of iwconfig set the IW_ENCODE_NOKEY flag
	 * when no key is given, but older versions don't. */

	if (dwrq->length > 0) {
		/* we have a key to set */
		int index = (dwrq->flags & IW_ENCODE_INDEX) - 1;
		int current_index;
		struct obj_key key = { DOT11_PRIV_WEP, 0, "" };
                int i;
		//subrat
			        int string_flag = 0;
			 	uint8_t mykey[16];
                                int length;
				int myindex = index + 1;
                                length = dwrq->length;
                                memcpy(mykey,extra,length);
                                switch(myindex)
                                {
                                        case 1:       
						 #ifdef HOSTAPDCONF
							strcpy(string,"wep_key0=");
						 #else
							strcpy(string,"wep_key_1");
						 #endif
                                                        break;
                                        case 2:       
						 #ifdef HOSTAPDCONF
                                                        strcpy(string,"wep_key1=");
                                                 #else

							strcpy(string,"wep_key_2");
						 #endif
                                                        break;


                                        case 3:       
						 #ifdef HOSTAPDCONF
                                                        strcpy(string,"wep_key2=");
                                                 #else

							strcpy(string,"wep_key_3");
						 #endif
                                                        break;
                                        case 4:       
						 #ifdef HOSTAPDCONF
                                                        strcpy(string,"wep_key3=");
                                                 #else
							
							strcpy(string,"wep_key_4");
						 #endif
                                                        break;
                                }

				for(i=0;i < length; i++)
				{
					//printk("%c",(unsigned char)mykey[i]);
					if(mykey[i] < 48 || mykey[i] > 57)
					string_flag = 1;	
				}
                                #ifndef HOSTAPDCONF
				strcat(string," hex:");
				#endif
				if(string_flag == 1)
				{		
					uint8_t tempkey[18];
					char quote = '"';
					memcpy(tempkey,mykey,length);
					memset(mykey,0,length);
				//	memcpy(mykey,'h',1);

					memcpy(mykey,&quote,1);
					memcpy(mykey+1,tempkey,length);
					memcpy(mykey+1+length,&quote,1);
					
					//memcpy(mykey+1+length,'h',1);
					length = length + 2;
					

				}
                                fs = get_fs();
                                set_fs(KERNEL_DS);
                                ifp = filp_open(PAED_CONF_FILE , O_RDWR, 0);
                                if(IS_ERR(ifp))
                                {
                                        DEBUG(DBG_ERR,"Cannot open paed.conf file \n");
                                        set_fs(fs);
                                        kfree(data);
                                        return FOPEN_ERR;
                                        printk("can not open file paed.wlan0.conf");
                                }
				//printk("string:%s length:%d",string,length);
			
                                set_key_conffile(ifp,string,mykey,length);
                                filp_close(ifp,NULL);
                                set_fs(fs);



		//end
		/* get the current key index */
    	if( retval = sm_drv_get( ndev, DOT11_OID_DEFKEYID, &data, sizeof(unsigned int)), retval < 0 )
    	{
    		DEBUG(DBG_IWCONFIG	, " %s defkeyid failed : Error %d \n", __FUNCTION__,retval);
			return -EIO;
    	}
		current_index = data;
		/* Verify that the key is not marked as invalid */
		if (!(dwrq->flags & IW_ENCODE_NOKEY)) {
			key.length = dwrq->length > sizeof (key.key) ?
			    sizeof (key.key) : dwrq->length;
			memcpy(key.key, extra, key.length);
			if (key.length == 32)
				/* we want WPA-PSK */
				key.type = DOT11_PRIV_TKIP;
			if ((index < 0) || (index > 3))
				/* no index provided use the current one */
				index = current_index;

			/* now send the key to the card  */
    		if( retval = sm_drv_set( ndev, DOT11_OID_DEFKEY1 + index, &key, sizeof(struct obj_key )), retval < 0 )
    		{
    			DEBUG(DBG_IWCONFIG	, " %s defkey1 failed : Error %d \n", __FUNCTION__,retval);
				return -EIO;
    		}
		}
		/*
		 * If a valid key is set, encryption should be enabled 
		 * (user may turn it off later).
		 * This is also how "iwconfig ethX key on" works
		 */
		if ((index == current_index) && (key.length > 0))
			force = 1;
	} else {
		int index = (dwrq->flags & IW_ENCODE_INDEX) - 1;
		if ((index >= 0) && (index <= 3)) {

			//subrat
			
			
			
			#if 0
			int index = (dwrq->flags & IW_ENCODE_INDEX) - 1;
 			int current_index;
                        struct obj_key key = { DOT11_PRIV_WEP, 0, "" };
			

		
			if (dwrq->length > 0) 
			{
                		/* we have a key to set */
                		int index = (dwrq->flags & IW_ENCODE_INDEX) - 1;
                		int current_index;
                		struct obj_key key = { DOT11_PRIV_WEP, 0, "" };

                		/* get the current key index */
        			if( retval = sm_drv_get( ndev, DOT11_OID_DEFKEYID, &data, sizeof(unsigned int)), retval < 0 )
        			{
                			DEBUG(DBG_IWCONFIG      , " %s defkeyid failed : Error %d \n", __FUNCTION__,retval);
                        		return -EIO;
        			}
                		current_index = data;
                		/* Verify that the key is not marked as invalid */
                		if (!(dwrq->flags & IW_ENCODE_NOKEY)) {
                        	key.length = dwrq->length > sizeof (key.key) ?
                            	sizeof (key.key) : dwrq->length;
                        	memcpy(key.key, extra, key.length);
                        	if (key.length == 32)
                                /* we want WPA-PSK */
                                key.type = DOT11_PRIV_TKIP;
                        	if ((index < 0) || (index > 3))
                                /* no index provided use the current one */
                                index = current_index;

                        	/* now send the key to the card  */
			}	
			#endif
				uint8_t key[16];
				int length;
				length = dwrq->length;
				memcpy(key,extra,length);
				//("entered");	
				switch(index)
				{
					case 1:       strcpy(string,"wep_key_1");
							break;
					case 2:       strcpy(string,"wep_key_2");
							break;
					case 3:       strcpy(string,"wep_key_3");
							break;
					case 4:       strcpy(string,"wep_key_4");
							break;
				}	
				fs = get_fs();
                		set_fs(KERNEL_DS);
                		ifp = filp_open(PAED_CONF_FILE , O_RDWR, 0);
                		if(IS_ERR(ifp))
                		{
                        		DEBUG(DBG_ERR,"Cannot open paed.conf file \n");
                        		set_fs(fs);
                        		kfree(data);
                        		return FOPEN_ERR;
                        		printk("can not open file paed.wlan0.conf");
                		}
                		set_key_conffile(ifp,string,key,length);
                		filp_close(ifp,NULL);
                		set_fs(fs);
			//end

			/* we want to set the key index */
    		if( retval = sm_drv_set( ndev, DOT11_OID_DEFKEYID, &index, sizeof(unsigned int)), retval < 0 )
    		{
    			DEBUG(DBG_IWCONFIG	, " %s defkeyid failed : Error %d \n", __FUNCTION__,retval);
				return -EIO;
    		}
		} else {
			if (!dwrq->flags & IW_ENCODE_MODE) {
				/* we cannot do anything. Complain. */
				return -EINVAL;
			}
		}
	}

	/* now read the flags */
	if (dwrq->flags & IW_ENCODE_DISABLED) {
		/* Encoding disabled, 
		 * authen = DOT11_AUTH_OS;
		 * invoke = 0;
		 * exunencrypt = 0; */
	}
	if (dwrq->flags & IW_ENCODE_OPEN)
		/* Encode but accept non-encoded packets. No auth */
		invoke = 1;
	if ((dwrq->flags & IW_ENCODE_RESTRICTED) || force) {
		/* Refuse non-encoded packets. Auth */
		authen = DOT11_AUTH_BOTH;
		invoke = 1;
		exunencrypt = 1;
	}
	/* do the change if requested  */
	if ((dwrq->flags & IW_ENCODE_MODE) || force) {
    	if( retval = sm_drv_set( ndev, DOT11_OID_AUTHENABLE, &authen, sizeof(unsigned int)), retval < 0 )
    	{
    		DEBUG(DBG_IWCONFIG	, " %s Authenable failed : Error %d \n", __FUNCTION__,retval);
			return -EIO;
    	}
    	if( retval = sm_drv_set( ndev, DOT11_OID_PRIVACYINVOKED, &invoke, sizeof(unsigned int)), retval < 0 )
    	{
    		DEBUG(DBG_IWCONFIG	, " %s privacyinvoked failed : Error %d \n", __FUNCTION__,retval);
			return -EIO;
    	}
    	if( retval = sm_drv_set( ndev, DOT11_OID_EXUNENCRYPTED, &exunencrypt, sizeof(unsigned int)), retval < 0 )
    	{
    		DEBUG(DBG_IWCONFIG	, " %s exunencrypted failed : Error %d \n", __FUNCTION__,retval);
			return -EIO;
    	}

	}
	return 0;
}

static int sm_drv_get_encode (struct net_device *ndev, 
	struct iw_request_info *info, struct iw_point *dwrq, char *extra)
{
	struct net_local  *lp = ndev->priv ;
	uint32_t devindex, index = (dwrq->flags & IW_ENCODE_INDEX) - 1;
	uint32_t authen = 0, invoke = 0, exunencrypt = 0;
	uint32_t data = 0;
	int retval = 0;
	struct obj_key *key;

    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

    if( retval = sm_drv_get( ndev, DOT11_OID_AUTHENABLE, &data, sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s Authenable failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
	authen = data;
    DEBUG(DBG_IWCONFIG, " %s DOT11_OID_AUTHENABL %x \n", __FUNCTION__,authen);
	data=0;
    if( retval = sm_drv_get( ndev, DOT11_OID_PRIVACYINVOKED, &data, sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s privacyinvoked failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
	invoke = data;
    DEBUG(DBG_IWCONFIG, " %s DOT11_OID_PRIVACYINVOKED %x \n", __FUNCTION__,invoke);
	data=0;
    if( retval = sm_drv_get( ndev, DOT11_OID_EXUNENCRYPTED, &data, sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s exunencrypted failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
	exunencrypt = data;
    DEBUG(DBG_IWCONFIG,"%s DOT11_OID_EXUNENCRYPTED %x \n", __FUNCTION__,exunencrypt);

	if (invoke && (authen == DOT11_AUTH_BOTH) && exunencrypt)
		dwrq->flags = IW_ENCODE_RESTRICTED;
	else if ((authen == DOT11_AUTH_OS) && !exunencrypt) {
		if (invoke)
			dwrq->flags = IW_ENCODE_OPEN;
		else
			dwrq->flags = IW_ENCODE_DISABLED;
	} else
		/* The card should not work in this state */
		dwrq->flags = 0;

	/* get the current device key index */
	data=0;
    if( retval = sm_drv_get( ndev, DOT11_OID_DEFKEYID , &data, sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s defkeyid failed : Error %d \n", __FUNCTION__,retval);
		return -EIO;
    }
	devindex = data;
    DEBUG(DBG_IWCONFIG,"%s DOT11_OID_DEFKEYID %x \n", __FUNCTION__,devindex );

    DEBUG(DBG_IWCONFIG,"%s index get%x \n", __FUNCTION__, index);
	/* Now get the key, return it */
	if ((index < 0) || (index > 3))
		/* no index provided, use the current one */
		index = devindex;
    DEBUG(DBG_IWCONFIG,"%s index set %x \n", __FUNCTION__, index);
	
	key = kmalloc(sizeof(struct obj_key),GFP_KERNEL);

    if( retval = sm_drv_get( ndev, DOT11_OID_DEFKEY1 + index , key, sizeof(struct obj_key)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG	, " %s defkey 1  failed : Error %d \n", __FUNCTION__,retval);
		kfree(key);
		return -EIO;
    }
    DEBUG(DBG_IWCONFIG,"%s DOT11_OID_DEFKEYX key len %d type %d \n", __FUNCTION__, key->length,key->type );

	dwrq->length = key->length;
	memcpy(extra, key->key, dwrq->length);
	kfree(key);
	/* return the used key index */
	dwrq->flags |= devindex + 1;

	return 0;
}

static int sm_drv_set_wap (struct net_device *ndev, 
	struct iw_request_info *info, struct sockaddr *ap_addr, char *extra)
{
	int err,i;	
	uint8_t bssid[6];
	printk(" %s ",ap_addr->sa_data);
	memcpy(bssid,ap_addr->sa_data,6);

	for(i =0 ; i < 6 ;i++)
	    printk("%X:",bssid[i]);
    	if( err = sm_drv_set( ndev, DOT11_OID_BSSID, bssid, sizeof(bssid)), err < 0 )
    	{
    		DEBUG(DBG_IWCONFIG	, " %s get bsstype failed : Error %d \n", __FUNCTION__,err);
		return -EIO;
    	}
	
	return 0;
}

static int sm_drv_get_wap (struct net_device *ndev, 
	struct iw_request_info *info, struct sockaddr *ap_addr, char *extra)
{
	int err,i;	
	uint8_t bssid[6];
    	if( err = sm_drv_get( ndev, DOT11_OID_BSSID, bssid, sizeof(bssid)), err < 0 )
    	{
    		DEBUG(DBG_IWCONFIG	, " %s get bsstype failed : Error %d \n", __FUNCTION__,err);
		return -EIO;
    	}
	ap_addr->sa_family = ARPHRD_ETHER;
	memcpy(ap_addr->sa_data,bssid,6);
	ap_addr->sa_data[7]='\0';

return 0;
}

char NICKNAME[30]="\0";

static int sm_drv_set_nickname (struct net_device *ndev, 
	struct iw_request_info *info, struct iw_point *data, char *extra)
{
	int length;
	length=data->length;
	memcpy(NICKNAME,data->pointer,length);
	NICKNAME[length]='\0';

	return 0;
}
static int sm_drv_get_nickname (struct net_device *ndev, 
	struct iw_request_info *info, struct iw_point *data, char *extra)
{
	int len,i;
	for (i=0;NICKNAME[i];i++);
	data->length=len=i;
	strncpy(extra,NICKNAME,len);
	return 0;	
}

static int sm_drv_set_mode (struct net_device *ndev, 
	struct iw_request_info *info, uint32_t *mode, char *extra)
{       
	struct net_local *lp = ndev->priv;
	int err;
	switch(*mode)
	{
		case IW_MODE_ADHOC:

			*mode = DOT11_BSSTYPE_IBSS;
			
			if( err = sm_drv_set( ndev, DOT11_OID_BSSTYPE, mode, sizeof(uint32_t)), err < 0 )
        		{
                		DEBUG(DBG_IWCONFIG      , " %s set bsstype failed : Error %d \n", __FUNCTION__,err);
                		return -EIO;
        		}
	                break;

                case IW_MODE_INFRA:

			*mode = DOT11_BSSTYPE_INFRA;
			if( err = sm_drv_set( ndev, DOT11_OID_BSSTYPE, mode, sizeof(uint32_t)), err < 0 )
        		{
                		DEBUG(DBG_IWCONFIG      , " %s set bsstype failed : Error %d \n", __FUNCTION__,err);
                		return -EIO;
        		}
                        break;
	        case 3:
			break;	
        
	        default:
                        return -EINVAL;
	}
	return err;
}

static int sm_drv_get_mode (struct net_device *ndev, 
	struct iw_request_info *info, uint32_t* mode, char *extra)
{
        
	struct net_local *lp = ndev->priv;
	int err=0;

	if( err = sm_drv_get( ndev, 0x00000006, mode, sizeof(uint32_t)), err < 0 )
        {
                DEBUG(DBG_IWCONFIG      , " %s get bsstype failed : Error %d \n", __FUNCTION__,err);
                return -EIO;
        }
	if(*mode == 1)
	{
		*mode = IW_MODE_MASTER;
		return err;
	}
	
    	if( err = sm_drv_get( ndev, DOT11_OID_BSSTYPE, mode, sizeof(uint32_t)), err < 0 )
    	{
    		DEBUG(DBG_IWCONFIG	, " %s get bsstype failed : Error %d \n", __FUNCTION__,err);
		return -EIO;
    	}
	switch(*mode)
	{
		case DOT11_BSSTYPE_IBSS:
		
			*mode = IW_MODE_ADHOC;		
	                break;

                case DOT11_BSSTYPE_INFRA:
			*mode =  IW_MODE_INFRA;
                        break;

                default:
			
                        return -EINVAL;
	}
	
	return err;
}

static int sm_drv_set_power (struct net_device *ndev, 
	struct iw_request_info *info, struct iw_param *power,char *extra)
{
	int err;
	uint32_t mode;
	unsigned int commit_data = 0xd;
	int retval = 0;
	mode=power->value;
	if( err = sm_drv_set( ndev, DOT11_OID_PSM, &mode, sizeof(uint32_t)), err < 0 )
    	{
    		DEBUG(DBG_IWCONFIG	, " %s set PSM failed : Error %d \n", __FUNCTION__,err);
		return -EIO;
    	}
	if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)), retval < 0 )
    	{
        	DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    	}
    return 0;
}

static int sm_drv_get_power(struct net_device *ndev, 
	struct iw_request_info *info, struct iw_param* power,char *extra)
{
        int err;
	uint32_t mode;
	if( err = sm_drv_get( ndev, DOT11_OID_PSM, &mode, sizeof(uint32_t)), err < 0 )
    	{
    		DEBUG(DBG_IWCONFIG	, " %s get PSM failed : Error %d \n", __FUNCTION__,err);
		return -EIO;
    	}
	power->value=mode;
	return 0;
}

int set_longretries(struct net_device *ndev,uint32_t value)
{
	int err;

	if( err = sm_drv_set( ndev, DOT11_OID_LONGRETRIES, &value, sizeof(uint32_t)), err < 0 )
	{
 	   DEBUG(DBG_IWCONFIG	, " %s set long retries failed : Error %d \n", __FUNCTION__,err);
	   return -EIO;
	}
return 0;
}

int set_shortretries(struct net_device *ndev,uint32_t value)
{
	int err;
	unsigned int commit_data = 0xd;
	int retval = 0;
	if( err = sm_drv_set( ndev, DOT11_OID_SHORTRETRIES, &value, sizeof(uint32_t)), err < 0 )
    	{
    	   DEBUG(DBG_IWCONFIG	, " %s set short retries failed : Error %d \n", __FUNCTION__,err);
	   return -EIO;
    	}
	if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)), retval < 0 )
    	{
        	DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    	}

	return 0;
}

static int sm_drv_set_retry (struct net_device *ndev, 
	struct iw_request_info *info, struct iw_param *retry,char *extra)
{

	int err = -EINVAL;
	unsigned int commit_data = 0xd;
	if(retry->disabled) {
		return -EINVAL;
	}

	if(retry->flags & IW_RETRY_LIMIT) {
		if(retry->flags & IW_RETRY_MAX)	{
	   		if((err=set_longretries(ndev,retry->value),err < 0))
				return err;
		}
		else if (retry->flags & IW_RETRY_MIN) {
	   		if((err=set_shortretries(ndev,retry->value),err < 0))
				return err;
		}
           	else {
			/* No modifier : set both */
		
	   		if((err=set_longretries(ndev,retry->value),err < 0))
				return err;
	   		if((err=set_shortretries(ndev,retry->value),err < 0))
				return err;
	   	}
	}//endif(retry->flags & IW_RETRY_LIMIT)


	if(retry->flags & IW_RETRY_LIFETIME) {
		retry->value/=1024;
		if( err = sm_drv_set( ndev, DOT11_OID_MAXTXLIFETIME, &retry->value, sizeof(uint32_t)), err < 0 )
    		{
    	   		DEBUG(DBG_IWCONFIG	, " %s set max TX lifetime failed : Error %d \n", __FUNCTION__,err);
	   	return -EIO;
    		}
	}
	if( err = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)), err < 0 )
    	{
        	DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,err);
                return -EIO;
    	}
	return 0;
}

static int sm_drv_get_retry (struct net_device *ndev, 
	struct iw_request_info *info, struct iw_param *retry,char *extra)
{
	int err=0;
	
        retry->disabled = 0;      /* Can't be disabled */

        /* Note : by default, display the min retry number */
        if((retry->flags & IW_RETRY_TYPE) == IW_RETRY_LIFETIME) {
                retry->flags = IW_RETRY_LIFETIME;
		if( err = sm_drv_get( ndev, DOT11_OID_MAXTXLIFETIME, &retry->value, sizeof(uint32_t)), err < 0 )
    		{
    	   		DEBUG(DBG_IWCONFIG	, " %s get max TX lifetime failed : Error %d \n", __FUNCTION__,err);
	   	return -EIO;
    		}
                retry->value *= 1024;
		
        } else if((retry->flags & IW_RETRY_MAX)) {
                retry->flags = IW_RETRY_LIMIT | IW_RETRY_MAX;
		if( err = sm_drv_get( ndev, DOT11_OID_LONGRETRIES, &retry->value, sizeof(uint32_t)), err < 0 )
		{
	 	   	DEBUG(DBG_IWCONFIG	, " %s get long retries failed : Error %d \n", __FUNCTION__,err);
	   	return -EIO;
		}
        } else {
                retry->flags = IW_RETRY_LIMIT;
		
		if( err = sm_drv_get( ndev, DOT11_OID_SHORTRETRIES, &retry->value, sizeof(uint32_t)), err < 0 )
    		{
    		   	DEBUG(DBG_IWCONFIG	, " %s get short retries failed : Error %d \n", __FUNCTION__,err);
	   	   	return -EIO;
    		}
        }


	return 0;
}

static int sm_drv_set_spy(struct net_device *ndev, 
			  struct iw_request_info *info, struct iw_point *vwrq, char *extra)
{
    struct net_local *lp =	ndev->priv;
    int i;
    struct sockaddr	 *address = (struct sockaddr*) extra;
    DEBUG(DBG_IWCONFIG, "%s  length = %d\n", __FUNCTION__, vwrq->length);
    lp->spy.nr = 0;
  	
        if(lp->device_state != DEVSTATE_ACTIVE )
	 return -EINTR;
  	
    if(vwrq->length > 0)
    {
         for(i=0; i<vwrq->length; i++)
  	  memcpy(&lp->spy.mac[i*ETH_ALEN],address[i].sa_data, ETH_ALEN);
         lp->spy.nr = vwrq->length;	
    }
    return 0;
}

int sm_drv_get_spy_stat(struct net_device *ndev, struct iw_quality *qual)
{
    struct net_local *lp =	ndev->priv;
    int i, number    = lp->spy.nr;
    int retval;
    uint32_t noise = 0 , snr = 0;
    struct obj_sta sta_info;
    DEBUG(DBG_IWCONFIG, "%s \n", __FUNCTION__);
	 
	if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;
    
    if( retval = sm_drv_get( ndev, DOT11_OID_NOISEFLOOR, &noise, sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG	, " %s noise floor failed : Error %d \n", __FUNCTION__,retval);
        return -EIO;
    }

    for(i=0; i<number; i++)
    {
        memset(&sta_info, 0x00, sizeof(struct obj_sta));
        memcpy(&sta_info.address, &lp->spy.mac[i*ETH_ALEN], ETH_ALEN);
        DEBUG(DBG_IWCONFIG, "%02x %02x %02x %02x %02x %02x %d\n", \
	     sta_info.address[0],  sta_info.address[1],  sta_info.address[2],  sta_info.address[3], \
	     sta_info.address[4],  sta_info.address[5], ETH_ALEN);
   		
        if( retval = sm_drv_get( ndev, DOT11_OID_CLIENTFIND , 	\
   			       &sta_info, sizeof(struct obj_sta)), retval < 0 )
   	 {	
   	      DEBUG(DBG_IWCONFIG," %s DOT11_OID_CLIENTFIND 	\
   				: Error %d \n", __FUNCTION__,retval);
   	 }
   			
    snr = sta_info.rssi - (uint8_t)noise;
    if(snr <0)
        qual[i].qual = 0;
    else
    {
        if( snr >=0 && snr < 40)
            qual[i].qual = (5*snr)/2;
        else
            qual[i].qual = 100;
     }
        qual[i].level 	= (uint8_t) sta_info.rssi;
        qual[i].noise  	= (uint8_t) noise;

        /*		qual[i].updated	= 0x7;		 */
   	
        DEBUG(DBG_IWCONFIG, "level = %d qual = %d noise = %d  \n", qual[i].level, qual[i].qual, qual[i].noise);
    }
    return 0;
}

static int sm_drv_get_spy(struct net_device *ndev, 
			  struct iw_request_info *info, struct iw_point *vwrq, char *extra)
{
    struct net_local *lp =	ndev->priv;
    int i, number    = lp->spy.nr, retval;
    struct sockaddr	 *address = (struct sockaddr *) extra;
    struct iw_quality *spy_info;
    spy_info=(struct iw_quality*)(extra+(number*sizeof(struct sockaddr)));
    memset(extra, 0x00, (number*(sizeof(struct sockaddr)+	\
  			       sizeof(struct iw_quality))));
	 
	if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;
  
    for(i=0; i<number; i++)
    {
         memcpy(address[i].sa_data, &lp->spy.mac[i*ETH_ALEN],ETH_ALEN);
         address[i].sa_family = ARPHRD_ETHER;
    }
    if(retval = sm_drv_get_spy_stat(ndev, spy_info), retval < 0)
        return retval;
    vwrq->length=number;
    vwrq->flags=0;
    DEBUG(DBG_IWCONFIG, "%s no.of mac set %d\n", __FUNCTION__,number);
  
    return 0;
}

static int sm_drv_set_mlme(struct net_device *ndev,
			   struct iw_request_info *info,
			   struct iw_point *data, char *extra)
{
 
    struct iw_mlme *mlme = (struct iw_mlme *) extra;
    u16 reason;
    int retval;
    struct obj_mlme obj_ml; 
    long alg_uni =0;
    char order[4];
    
#if WIRELESS_EXT >=18
    reason = cpu_to_le16(mlme->reason_code);
#endif    
    order[0]=DOT11_AUTHALG_NONE;
    order[1]=DOT11_AUTHALG_NONE; 
  
    if( retval = sm_drv_set( ndev, DOT11_OID_PRIVACYINVOKED,(void*) &alg_uni, sizeof(alg_uni)), retval < 0 )
    {
         DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_PRIVACYINVOKED failed\n",__FUNCTION__ ,retval );
         return 0;
    }
  
  
    if( retval = sm_drv_set( ndev, DOT11_OID_AUTHORDER, (void*)&order, sizeof(order)), retval < 0 )
    {
         DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_AUTHORDER failed\n",__FUNCTION__ ,retval );
         return 0;         
     }
	
  #if WIRELESS_EXT >= 18
    switch (mlme->cmd)
    {
         case IW_MLME_DEAUTH:
                /*  REASON_MICHAEL_MIC_FAILURE */
                if (reason == 14)
                    return 0;
      
                memset(&obj_ml, 0x00, sizeof(obj_ml));
                memcpy(obj_ml.address,mlme->addr.sa_data, 6);
                obj_ml.code = 1;
                obj_ml.id = -1;
      
                if( retval = sm_drv_set( ndev,  DOT11_OID_DEAUTHENTICATE,(void*)&obj_ml, sizeof(obj_ml)), retval < 0 )
               {
                     DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_DEAUTHENTICATE failed\n",__FUNCTION__ ,retval );
      	              break;
                }
      
          break;
        case IW_MLME_DISASSOC:
               memset(&obj_ml, 0x00, sizeof(obj_ml));
               memcpy(obj_ml.address,mlme->addr.sa_data, 6);
               obj_ml.code = 1;
               obj_ml.id = -1;
      
               if( retval = sm_drv_set( ndev,  DOT11_OID_DISASSOCIATE,(void*)&obj_ml, sizeof(obj_ml)), retval < 0 )
               {
                    DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_DISASSOCIATE failed\n",__FUNCTION__ ,retval );
      	             break;
                }
          break;
        default:
             return -EOPNOTSUPP;
        }
  #endif
    return 0;
}                                              

static int sm_drv_get_waplist(struct net_device *ndev, 
			      struct iw_request_info *info, struct iw_point *dwrq, char *extra)
{
    struct net_local  *lp = ndev->priv;	
    uint32_t maxsta;
    char *current_ev = extra;
    int retval = 0;
    int i = 0;
    uint32_t length = 0;
    struct sockaddr addr;
    struct obj_sta sta;
    int j,loopcount=0;
	
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    
	if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;
     if(sm_drv_get( ndev,DOT11_OID_CLIENTS/*DOT11_OID_CLIENTSASSOCIATED*/, (void *)&maxsta, sizeof(unsigned int)) < 0)
	return -EIO; 
    for(i=0;i<maxsta;i++)
    { 

	 memset(&sta,0,sizeof(struct obj_sta));
	
	sm_drv_get(ndev,(DOT11_OID_CLIENT1 + i),(void *)&sta, sizeof(sta)); 
	

	if(sta.state == 5 )
	{	
        	addr.sa_family = ARPHRD_ETHER;
		memcpy(addr.sa_data,sta.address,6);
        	memcpy((current_ev+(loopcount*sizeof(struct sockaddr))),&addr,sizeof(struct sockaddr));
		loopcount ++;
	}
    }
    dwrq->length = loopcount;
    dwrq->flags = 0;
    return 0;
}

static const iw_handler sm_drv_handler[] = {
  (iw_handler) sm_drv_commit,	/* SIOCSIWCOMMIT */ 
  (iw_handler) sm_drv_get_name ,	/* SIOCGIWNAME */
  (iw_handler) NULL,		/* SIOCSIWNWID */
  (iw_handler) NULL,		/* SIOCGIWNWID */
  (iw_handler) sm_drv_set_freq,	/* SIOCSIWFREQ */
  (iw_handler) sm_drv_get_freq,	/* SIOCGIWFREQ */
  (iw_handler) sm_drv_set_mode,	/* SIOCSIWMODE */
  (iw_handler) sm_drv_get_mode,	/* SIOCGIWMODE */
  (iw_handler) sm_drv_set_sens,	/* SIOCSIWSENS */
  (iw_handler) sm_drv_get_sens,	/* SIOCGIWSENS */
  (iw_handler) NULL,		/* SIOCSIWRANGE */
  (iw_handler) sm_drv_get_range,	/* SIOCGIWRANGE */
  (iw_handler) NULL,		/* SIOCSIWPRIV */
  (iw_handler) NULL,		/* SIOCGIWPRIV */
  (iw_handler) NULL,		/* SIOCSIWSTATS */
  (iw_handler) NULL,		/* SIOCGIWSTATS */
  (iw_handler) sm_drv_set_spy,	/* SIOCSIWSPY */
  (iw_handler) sm_drv_get_spy,	/* SIOCGIWSPY */
#if WIRELESS_EXT <= 15	
  (iw_handler) NULL, 		/* SIOCSIWHRSPY */
  (iw_handler) NULL, 		/* SIOCGIWTHRSPY */
#else
  (iw_handler) NULL, 		/* SIOCSIWTHRSPY */
  (iw_handler) NULL, 		/* SIOCGIWTHRSPY */
#endif
#if WIRELESS_EXT >= 15
  (iw_handler) sm_drv_set_wap,	/* SIOCSIWAP */
  (iw_handler) sm_drv_get_wap,	/* SIOCGIWAP */
  (iw_handler) sm_drv_set_mlme,	/* SIOCSIWMLME */
  (iw_handler) sm_drv_get_waplist,/* SIOCGIWAPLIST depreciated */
  (iw_handler) sm_drv_set_scan, 	/* SIOCSIWSCAN */
  (iw_handler) sm_drv_get_scan,	/* SIOCGIWSCAN */
  (iw_handler) sm_drv_set_essid,	/* SIOCSIWESSID */
  (iw_handler) sm_drv_get_essid,	/* SIOCGIWESSID */
  (iw_handler) sm_drv_set_nickname,	/* SIOCSIWNICKN */
  (iw_handler) sm_drv_get_nickname, 	/* SIOCGIWNICKN */
  (iw_handler) NULL,		/* -- hole -- */
  (iw_handler) NULL,		/* -- hole -- */
  (iw_handler) sm_drv_set_rate, 	/* SIOCSIWRATE 		*/
  (iw_handler) sm_drv_get_rate, 	/* SIOCGIWRATE 		*/
  (iw_handler) sm_drv_set_rts,	/* SIOCSIWRTS 	*/
  (iw_handler) sm_drv_get_rts,	/* SIOCGIWRTS 	*/
  (iw_handler) sm_drv_set_frag,	/* SIOCSIWFRAG 		*/
  (iw_handler) sm_drv_get_frag,	/* SIOCGIWFRAG 		*/
  (iw_handler) sm_drv_set_txpower,/* SIOCSIWTXPOW */
  (iw_handler) sm_drv_get_txpower,/* SIOCGIWTXPOW */
  (iw_handler) sm_drv_set_retry,  /* SIOCSIWRETRY		*/
  (iw_handler) sm_drv_get_retry,  /* SIOCGIWRETRY 	*/
  (iw_handler) sm_drv_set_encode, /* SIOCSIWENCODE 	*/
  (iw_handler) sm_drv_get_encode, /* SIOCGIWENCODE 	*/
  (iw_handler) sm_drv_set_power,	/* SIOCSIWPOWER 	*/
  (iw_handler) sm_drv_get_power,	/* SIOCGIWPOWER 	*/
#endif 
#if WIRELESS_EXT >= 18
  (iw_handler) NULL,              /* -- hole -- */
  (iw_handler) NULL,              /* -- hole -- */
  (iw_handler) sm_drv_set_genie,	/* SIOCSIWGENIE */
  (iw_handler) sm_drv_get_genie,  /* SIOCGIWGENIE */
  (iw_handler) sm_drv_set_auth,   /* SIOCSIWAUTH */
  (iw_handler) sm_drv_get_auth,   /* SIOCGIWAUTH */
  (iw_handler) sm_drv_set_encodeext,/* SIOCSIWENCODEEXT */
  (iw_handler) sm_drv_get_encodeext, 		/* SIOCGIWENCODEEXT */
  (iw_handler) sm_drv_set_pmksa, /* SIOCSIWPMKSA */
  (iw_handler) NULL,              /* -- hole -- */
#endif 

};


int sm_drv_get_data(struct net_device *ndev, uint32_t n, char *str)
{   struct net_local  *lp = ndev->priv;
    uint32_t retval=0, size;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    DEBUG(DBG_IWCONFIG, "ioctl#=%d oid=%08X type=%02X size=%d\n"	\
  	,n, oid_t[n].oid,oid_t[n].type, oid_t[n].size);
  
  //  printk( "ioctl#=%d oid=%X type=%02X size=%d\n"	\
  //	  ,n, oid_t[n].oid,oid_t[n].type, oid_t[n].size);
	
	if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;
	
    switch (oid_t[n].type & OID_FLAG_TYPE)
    {
        case OID_TYPE_INT:
        {
            int32_t data;
            size = sizeof(int32_t);
            retval = sm_drv_get(ndev,oid_t[n].oid, &data, size);
            return snprintf(str, PRIV_STR_SIZE, "%d\n", data);
            break;
        }

        case OID_TYPE_U32:
	 {
            uint32_t data;
            size = sizeof(uint32_t);
            retval = sm_drv_get(ndev,oid_t[n].oid, &data, size);
            return snprintf(str, PRIV_STR_SIZE, "%u\n", data);
            break;
        }

        case OID_TYPE_BSS:
	 {
            uint32_t data;
            size = sizeof(uint32_t);
            retval = sm_drv_get(ndev,oid_t[n].oid, &data, size);
            return snprintf(str, PRIV_STR_SIZE, "%u\n", data);
            break;
        }

        case OID_TYPE_BSSLIST:
	 {
            struct obj_bsslist *list;
            int i, k;
            size = 4 + sizeof(struct obj_bss[IWMAX_BSS]);
            size = (size > PRIV_STR_SIZE)?PRIV_STR_SIZE:size;
            list = (struct obj_bsslist*)kmalloc(size, GFP_KERNEL);
            retval = sm_drv_get(ndev,oid_t[n].oid, list, size);
            k = snprintf(str, PRIV_STR_SIZE, "nr=%u\n", list->nr);
            for (i = 0; i < list->nr; i++)
            {	
                list->bsslist[i].ssid[list->bsslist[i].ssid[0]+1]='\0';
                k += snprintf(str + k, PRIV_STR_SIZE - k,
                    "\nssid=%s\n"
                    "age=%u\nchannel=%u\n"
                   	"capinfo=0x%X\nrates=0x%X\n"
                   	"basic_rates=0x%X\n",
                   	&list->bsslist[i].ssid[1],
                   	list->bsslist[i].age,
                   	list->bsslist[i].channel,
                   	list->bsslist[i].capinfo,
                   	list->bsslist[i].rates,
                   	list->bsslist[i].basic_rates);
            }
            kfree(list);
            return k;
            break;
        }
        
        case OID_TYPE_FREQUENCIES:
	 {
            struct obj_frequencies *freq;
            int i, t;
            size = sizeof(struct obj_frequencies) +	\
                sizeof(uint16_t[IWMAX_FREQ]);
            size = (size > PRIV_STR_SIZE)?PRIV_STR_SIZE:size;
            freq = (struct obj_frequencies*)kmalloc(size, GFP_KERNEL);
            retval = sm_drv_get(ndev,oid_t[n].oid, freq, size);
            t = snprintf(str, PRIV_STR_SIZE, "nr=%u\n", freq->nr);
            for (i = 0; i < freq->nr; i++)
                t += snprintf(str + t, PRIV_STR_SIZE - t, "mhz[%u]=%u\n", i, freq->mhz[i]);
            kfree(freq);
            return t;
            break;
        }
        
        case OID_TYPE_MLME:
	 {
            struct obj_mlme mlme;
            size = sizeof(struct obj_mlme);
            retval = sm_drv_get(ndev,oid_t[n].oid, &mlme, size);
            return snprintf(str, PRIV_STR_SIZE,
                "\nid=0x%X\nstate=0x%X\ncode=0x%X\n",
                mlme.id, mlme.state, mlme.code);
            break;
        }

        case OID_TYPE_ATTACH:
	 {
            struct obj_attachment attach;
            size = sizeof(struct obj_attachment);
            retval = sm_drv_get(ndev,oid_t[n].oid, &attach, size);
            return snprintf(str, PRIV_STR_SIZE,
                "\nid=%d\nsize=%d\n",
                attach.id,
                attach.size);
            break;
        }

        case OID_TYPE_SSID:
     	 { 
            struct obj_ssid ssid;
            size = sizeof(struct obj_ssid);
            retval = sm_drv_get(ndev,oid_t[n].oid, &ssid, size);
            return snprintf(str, PRIV_STR_SIZE,
               	"\nlength=%u\noctets=%.*s\n",
               	ssid.length, ssid.length,
               	ssid.octets);
        break;
        }

        case OID_TYPE_KEY:
	 {
            struct obj_key key;
            int t, i;
            size = sizeof(obj_key);
            retval = sm_drv_get(ndev,oid_t[n].oid, &key, size);
            t = snprintf(str, PRIV_STR_SIZE,
                "\ntype=0x%X\nlength=0x%X\nkey=0x",
                key.type, key.length);
            for (i = 0; i < key.length; i++)
                t += snprintf(str + t, PRIV_STR_SIZE - t, "%02X:", key.key[i]);
            t += snprintf(str + t, PRIV_STR_SIZE - t, "\n");
            return t;
            break;	
        }

        case OID_TYPE_COUNTRY:
        {
            struct obj_country country;
            memset(&country, 0x00, sizeof(struct obj_country));
            size = sizeof(struct obj_country);
            retval = sm_drv_get(ndev,oid_t[n].oid, &country, size);
            return snprintf(str, PRIV_STR_SIZE,
                "\nRegion code=%c%c%c\n",
                country.string[0],country.string[1],country.string[2]);
            break;
        }

        case OID_TYPE_DRIVER:
        {
            return snprintf(str, PRIV_STR_SIZE,"%s\n", VERSIONID);
            break;
        }  

        case OID_TYPE_SCAN:
        {
            struct obj_scan sc;
            memset(&sc, 0x00, sizeof(struct obj_scan));
            retval = sm_drv_get(ndev,oid_t[n].oid, &sc, sizeof(obj_scan));
            return snprintf(str, PRIV_STR_SIZE,
                "\nBackground scan parameter:-\nmin=%d\nmax=%d\ninterval=%d\namin=%d\namax=%d\n",
                sc.min,sc.max,sc.interval,sc.amin,sc.amax);
            break;
        }     

        case OID_TYPE_RAW:
        case OID_TYPE_ADDR:
        {
            uint8_t *buff;
            int t, i;
            size=(oid_t[n].size>PRIV_STR_SIZE)?PRIV_STR_SIZE:oid_t[n].size;
            buff=(uint8_t*)kmalloc(size,GFP_KERNEL);
            memset(buff, 0x00, size);
            retval = sm_drv_get(ndev,oid_t[n].oid, buff, size);
            
            t = snprintf(str, PRIV_STR_SIZE, "\nhex data=");
                            
            for (i = 0; (i < oid_t[n].size)&&(buff[i] != '\0' ||oid_t[n].type==OID_TYPE_ADDR); i++)
                 t += snprintf(str + t, PRIV_STR_SIZE - t,
            		    "%02X ", buff[i]);
            t += snprintf(str + t, PRIV_STR_SIZE - t, "\n");
            kfree(buff);
            return t;
            break;
        }
           
        default:
             DEBUG(DBG_IWCONFIG,"Not Supported\n");
    }
    return retval;
}

static int sm_drv_get_oid(struct net_device *ndev, struct iw_request_info *info,
    struct iw_point *dwrq, char *extra)
{   
    struct net_local  *lp = ndev->priv;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
	 
	if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;
    dwrq->length = sm_drv_get_data(ndev, dwrq->flags, dwrq->pointer);
    return (dwrq->length);
}

static int sm_drv_set_oid(struct net_device *ndev, struct iw_request_info *info,
    uint32_t *uwrq, char *extra)
{
    struct net_local  *lp = ndev->priv;
    uint32_t n = uwrq[0], *buff = &uwrq[1];
    unsigned int commit_data = 0xd;

    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    DEBUG(DBG_IWCONFIG, "ioctl#=%d oid=%08X type=%02X size=%d\n"	\
	, n, oid_t[n].oid,oid_t[n].type, oid_t[n].size);
	 
	if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;
    if(sm_drv_set(ndev, oid_t[n].oid, buff, sizeof(uint32_t)) < 0)
    return -EIO;
    if(sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)) < 0)
    return -EIO;
    
}

static int sm_drv_set_raw(struct net_device *ndev, struct iw_request_info *info,
    struct iw_point *dwrq, char *extra)
{
    struct net_local  *lp = ndev->priv;
    uint32_t n = dwrq->flags;
     unsigned int commit_data = 0xd;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
	 
	if(lp->device_state != DEVSTATE_ACTIVE )
         return -EINTR;
	
    dwrq->length=(dwrq->length > oid_t[n].size)? oid_t[n].size:dwrq->length;
    if( sm_drv_set(ndev, oid_t[n].oid, dwrq->pointer, dwrq->length) < 0)
    return -EIO;
     
    if(sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)) < 0)
    return -EIO;
    
}

static int sm_drv_get_ifname(struct net_device *ndev,
    struct iw_request_info *info, struct iwreq *dwrq, char *extra)
{
    struct net_local  *lp = ndev->priv;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    strncpy(dwrq ->ifr_ifrn.ifrn_name,ndev->name,IFNAMSIZ);
    return 0;
}

static int sm_drv_set_listen_interval(struct net_device *ndev,
    struct iw_request_info *info, uint32_t *dwrq, char *extra)
{
    struct net_local  *lp = ndev->priv;
    uint32_t retval;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    uint32_t *buff = &dwrq[0];
    unsigned int commit_data = 0xd;
    if( retval = sm_drv_set( ndev,  DOT11_OID_LISTENINTERVAL  ,(void*) buff, sizeof(uint32_t)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d,  DOT11_OID_LISTENINTERVAL   failed\n", __FUNCTION__ , retval );
        return 0;
    }
     if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    }
   
    return 0;
}

static int sm_drv_get_listen_interval(struct net_device *ndev,
    struct iw_request_info *info, uint32_t *dwrq, char *extra)
{
    struct net_local  *lp = ndev->priv;
    uint32_t retval,buff;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    if( retval = sm_drv_get( ndev,  DOT11_OID_LISTENINTERVAL  ,(void*) &buff, sizeof(uint32_t)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d,  DOT11_OID_LISTENINTERVAL   failed\n", __FUNCTION__ , retval );
        return 0;
     }
    *dwrq=buff;
    return 0;
}

static int sm_drv_set_wme(struct net_device *ndev,
    struct iw_request_info *info, uint8_t *dwrq, char *extra)
{
    struct net_local  *lp = ndev->priv;
    uint32_t retval;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    uint32_t *buff =(uint32_t *) &dwrq[0];
    unsigned int commit_data = 0xd;
    if( retval = sm_drv_set( ndev,  DOT11_OID_QOS_DISABLE ,(void*)buff, sizeof(uint8_t)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_QOS_DISABLE    failed\n",__FUNCTION__ ,retval );
        return 0;
    }
    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    }
    return 0;
}

static int sm_drv_get_wme(struct net_device *ndev,
    struct iw_request_info *info, uint32_t *dwrq, char *extra)
{   
    struct net_local  *lp = ndev->priv;
    uint32_t retval,buff=0;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    if( retval = sm_drv_get( ndev,  DOT11_OID_QOS_DISABLE   ,(void*) &buff, sizeof(uint32_t)), retval < 0 )
    {
        struct obj_mlme mlme;
        memset(&mlme,0x00,sizeof(mlme));
        DEBUG(DBG_IWCONFIG, "%s: Error %d,  DOT11_OID_QOS_DISABLE   failed\n", __FUNCTION__ , retval );
        return 0;
    }
    *dwrq = buff;
    return 0;
}



static int sm_drv_set_bg_scan(struct net_device *ndev,
    struct iw_request_info *info, uint32_t *dwrq, char *extra)
{   
    struct net_local  *lp = ndev->priv;
    uint32_t retval;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    uint32_t *buff = &dwrq[0];
    unsigned int commit_data = 0xd;
    if( retval = sm_drv_set( ndev,  DOT11_OID_SCANDISABLE ,(void*)buff, sizeof(uint32_t)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_SCANDISABLE    failed\n", __FUNCTION__ , retval );
        return 0;
    }
    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    }
    return 0;
}



static int sm_drv_set_reassociate(struct net_device *ndev,
                struct iw_request_info *info, uint32_t *dwrq, char *extra)
{
    struct net_local  *lp = ndev->priv;
    uint32_t retval, size;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    uint8_t *buff;
    struct obj_mlme mlme;
    memset(&mlme,0x00,sizeof(mlme));
    size=ETH_ALEN;
    buff=(uint8_t*)kmalloc(size,GFP_KERNEL);
    memset(buff, 0x00, size);
    retval = sm_drv_get(ndev,GEN_OID_MACADDRESS, buff, size);
    
    memcpy(mlme.address,buff,6);
    mlme.code = 1;
    mlme.id=-1;
    
    if( retval = sm_drv_set( ndev,  DOT11_OID_DISASSOCIATE ,(void*)&mlme, sizeof(mlme)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_REASSOCIATE failed\n",__FUNCTION__ ,retval );
        return 0;
    }
    
    return 0;
}

static int sm_drv_set_disassociate(struct net_device *ndev,
                struct iw_request_info *info, uint32_t *dwrq, char *extra)
{   
    struct net_local  *lp = ndev->priv;
    uint32_t retval, size;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    struct obj_mlme mlme;
    memset(&mlme,0x00,sizeof(mlme));
    struct obj_ssid essid ;
    memset(&essid, 0, sizeof(struct obj_ssid));
    
    uint8_t *buff;
    unsigned int commit_data = 0xd;
    size=ETH_ALEN;
    buff=(uint8_t*)kmalloc(size,GFP_KERNEL);
    memset(buff, 0x00, size);
    retval = sm_drv_get(ndev,GEN_OID_MACADDRESS, buff, size);
    
    memcpy(mlme.address,buff,6);
    mlme.code = 1;
    mlme.id=-1;
    
    memset(essid.octets, 0, 33);
    essid.length=33;  
    
    if( retval = sm_drv_set( ndev,  DOT11_OID_DISASSOCIATE ,(void*)&mlme, sizeof(mlme)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_DISASSOCIATE failed\n", __FUNCTION__ , retval );
        return 0;
    }
    
    retval = sm_drv_set( ndev,DOT11_OID_SSID, &essid, sizeof(struct obj_ssid));    
    sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int));
    return 0;
}

static int sm_drv_get_bg_scan(struct net_device *ndev,
    struct iw_request_info *info, uint32_t *dwrq, char *extra)
{   
    struct net_local  *lp = ndev->priv;
    uint32_t retval,buff=0;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    if( retval = sm_drv_get( ndev,  DOT11_OID_SCANDISABLE   ,(void*) &buff, sizeof(uint32_t)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d,  DOT11_OID_SCANDISABLE   failed\n",__FUNCTION__ ,retval );
        return 0;
    }
    *dwrq = buff;
    return 0;
}

static int sm_drv_set_region_code(struct net_device *ndev,
    struct iw_request_info *info, char *dwrq, char *extra)
{   
    struct net_local  *lp = ndev->priv;
    uint32_t retval;
    unsigned int data = 0;
    struct obj_country country;
    struct s_sm_conf  smconf;

    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    memset(&country, 0x00, sizeof(struct obj_country));
    
    memset(&country,'\0', sizeof(struct obj_country));
    country.string[0] = dwrq[0];
    country.string[1] = dwrq[1];
    country.string[2] = dwrq[2];
    country.string[3] = '0';

    if( retval = sm_drv_set( ndev,0x1f000002,country.string,4), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_COUNTRY failed\n",__FUNCTION__ ,retval );
        return -EIO;
     }
    
    data = 0xd;
    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &data , sizeof(unsigned int)), retval < 0 )
    {
      	DEBUG(DBG_IWCONFIG	, " %s commit failed : Error %d \n", __FUNCTION__,retval);
    	return -EIO;
    }
    
    return 0;
}

static int sm_drv_get_region_channel(struct net_device *ndev,
    struct iw_request_info *info, uint32_t *dwrq, char *extra)
{
    struct net_local  *lp = ndev->priv;
    uint32_t retval;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    struct obj_country country ;
    
    memset(&country, 0x00, sizeof(struct obj_country));
    
    if( retval = sm_drv_get( ndev, DOT11_OID_COUNTRY     ,(void*) &country, sizeof(struct obj_country)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d,  DOT11_OID_COUNTRY    failed\n",__FUNCTION__ ,retval );
        return -EIO;
    }
    
    *dwrq = country.frequencies[0].mhz;
    return 0;
}

static int sm_drv_set_bg_scan_conf (struct net_device *ndev,
    struct iw_request_info *info, uint32_t *dwrq, char *extra)
{   
    struct net_local  *lp = ndev->priv;
    uint32_t retval;
    struct obj_scan ob;
    uint32_t *buff = &dwrq[0];
    unsigned int commit_data = 0xd;
    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    }

    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    memset(&ob, 0x00, sizeof(struct obj_scan));
    
    if( retval = sm_drv_get( ndev,  DOT11_OID_SCAN     ,(void*) &ob, sizeof(struct obj_scan)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_SCAN     failed\n",__FUNCTION__ ,retval );
        return 0;
    }
    ob.interval = *buff;
    
    if( retval = sm_drv_set( ndev,  DOT11_OID_SCAN     ,(void*) &ob, sizeof(struct obj_scan)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_SCAN     failed\n",__FUNCTION__ ,retval );
        return 0;
    }
    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    }
   
    return 0;
}

static int sm_drv_get_80211d_status (struct net_device *ndev,
    struct iw_request_info *info, uint32_t *dwrq, char *extra)
{   
    struct net_local  *lp = ndev->priv;
    uint32_t retval,status;
    
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    if( retval = sm_drv_get( ndev, DOT11_OID_WWRMODE,(void*) &status, sizeof(uint32_t)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG, "%s: Error %d,  DOT11_OID_WMRMODE  failed\n",__FUNCTION__ ,retval );
        return 0;
    }
    *dwrq = status;
    return 0;
}

static char ssid_latch[IW_ESSID_MAX_SIZE];

static int sm_drv_set_hidessid(struct net_device *ndev,
                struct iw_request_info *info, uint32_t *dwrq, char *extra)
{
        struct net_local  *lp = ndev->priv;
        struct obj_ssid essid;
        int retval = 0;
	uint32_t buff = dwrq[0];	
	unsigned int data = 0xd;
    DEBUG(DBG_IWCONFIG  , " %s \n", __FUNCTION__);

    DEBUG(DBG_IWCONFIG  , " %s essid to set %s \n", __FUNCTION__,extra);
    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;
	if(buff == 1)
	{
		
		 if( retval = sm_drv_get( ndev,DOT11_OID_SSID, &essid, sizeof(struct obj_ssid) ), retval < 0 )
    		{
        		DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_SSID failed\n",__FUNCTION__ ,retval );
                	return -EIO;
    		}
		int len;
		len = min(IW_ESSID_MAX_SIZE, essid.length + 1);
               	essid.octets[essid.length] = '\0';
               	memcpy(ssid_latch, essid.octets, len);
	        memset(essid.octets, 0, 33);
	        essid.length = 0;
   
       			if( retval = sm_drv_set( ndev,DOT11_OID_SSIDOVERRIDE, &essid, sizeof(struct obj_ssid)), retval < 0 )
    			{
        			DEBUG(DBG_IWCONFIG      , " %s set essid failed : Error %d \n", __FUNCTION__,retval);
                		return -EIO;
    			}

    			if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &data , sizeof(unsigned int)), retval < 0 )
    			{
       	 			DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                		return -EIO;
    			}
        }
	if(buff == 0)
	{
 		int len;
		len = strlen(ssid_latch);
		memcpy(essid.octets,ssid_latch,len);
		essid.length = len;
		 if( retval = sm_drv_set( ndev,DOT11_OID_SSID, &essid, sizeof(struct obj_ssid)), retval < 0 )
                 {
                                DEBUG(DBG_IWCONFIG      , " %s set essid failed : Error %d \n", __FUNCTION__,retval);
                                return -EIO;
                 }
		
		 if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &data , sizeof(unsigned int)), retval < 0 )
                 {
                                DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                                return -EIO;
                 }
	}

        return 0;
}

static int sm_drv_set_hibernate(struct net_device *ndev,
                struct iw_request_info *info, uint32_t *dwrq, char *extra)
{
        struct net_local  *lp = ndev->priv;
        int retval = 0;
        uint32_t buff = dwrq[0];
        unsigned int data = 0xd;
    DEBUG(DBG_IWCONFIG  , " %s \n", __FUNCTION__);
	buff = !( dwrq[0]);
    DEBUG(DBG_IWCONFIG  , " %s essid to set %s \n", __FUNCTION__,extra);
    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

    if( retval = sm_drv_set( ndev,GEN_OID_HIBERNATE, &buff, sizeof(unsigned int)), retval < 0 )
    {
    	DEBUG(DBG_IWCONFIG      , " %s set hibernation failed : Error %d \n", __FUNCTION__,retval);
        printk("hibernate failed");
        return -EIO;
    }

    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &data , sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
        return -EIO;
    }

  return 0;
		
}

static int sm_drv_get_hibernate(struct net_device *ndev,
                struct iw_request_info *info, uint32_t *dwrq, char *extra)
{
        struct net_local  *lp = ndev->priv;
        int retval = 0;
        uint32_t buff = 0;
        unsigned int data = 0xd;
    DEBUG(DBG_IWCONFIG  , " %s \n", __FUNCTION__);

    DEBUG(DBG_IWCONFIG  , " %s essid to set %s \n", __FUNCTION__,extra);
    /* Sleep until UMAC and LMAC are completely initialized */
    if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

    /* Check if we were woken by a signal */
    if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;
    
    if( retval = sm_drv_get( ndev,GEN_OID_HIBERNATE, &buff, sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG      , " %s set hibernation failed : Error %d \n", __FUNCTION__,retval);
        printk("hibernate failed");
        return -EIO;
    }
   
  *dwrq = !(buff);
  return 0;

}

static int sm_drv_set_wmm_vap_disable(struct net_device *ndev,
    struct iw_request_info *info, uint32_t *dwrq, char *extra)
{
    struct net_local  *lp = ndev->priv;
    uint32_t retval;
    unsigned int commit_data = 0xd;
    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);
    uint32_t *buff = &dwrq[0];
    if( retval = sm_drv_set( ndev, DOT11_OID_WME_ALL_VAP, (void*)buff, sizeof(uint32_t)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_SCANDISABLE    failed\n", __FUNCTION__ , retval );
        return 0;
    }
    if( retval = sm_drv_set( ndev, GEN_OID_COMMIT, &commit_data, sizeof(unsigned int)), retval < 0 )
    {
        DEBUG(DBG_IWCONFIG      , " %s commit failed : Error %d \n", __FUNCTION__,retval);
                return -EIO;
    }
    return 0;
}

static int sm_drv_get_region_list(struct net_device *ndev,
                struct iw_request_info *info, struct iw_point *dwrq, char *extra)
{

        struct net_local  *lp = ndev->priv;
        int retval = 0;
	uint32_t length = 0;
        uint8_t *buff = NULL;
        uint8_t *data = NULL;
        int i=0;
        uint8_t *ptr;
        uint8_t *ptr1;

        /* Sleep until UMAC and LMAC are completely initialized */
        if(lp->device_state != DEVSTATE_ACTIVE )
        interruptible_sleep_on(&lp->conf_waitq);

        /* Check if we were woken by a signal */
        if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

        length = MAX_COUNTRYLIST_SIZE + 1;

        buff = (uint8_t *)kmalloc(length, GFP_KERNEL);
        memset(buff,0,length);

    	if(lp->device_state != DEVSTATE_ACTIVE )
        return -EINTR;

	if( retval = sm_drv_get( ndev, DOT11_OID_COUNTRYLIST,buff, length), retval < 0 )
    	{
        	DEBUG(DBG_IWCONFIG, "%s: Error %d, DOT11_OID_SSID failed\n",__FUNCTION__ ,retval );
                return -EIO;
    	}

	data = buff;
       	ptr = (uint8_t *)kmalloc(length, GFP_KERNEL);
       	memset(ptr,0,length);
        ptr1 = ptr;
       	while(*data)
       	{
          	int len = strlen(data);
               	strncpy(ptr, data, len);
               	ptr[len] = ' ';
               	ptr +=len;
               	data += 4;
       }
       ptr = ptr1;
       i = strlen(ptr);
       memset(buff,0,i);
       memcpy(buff,ptr, i);
       buff[i] = '\0';
       i = strlen(buff);
       if(strlen(buff)) 
       {
       		dwrq->flags = 1;
                dwrq->length = min(200, i);
       } 
       else 
       {
       		dwrq->flags = 0;
                dwrq->length = 0;
       }
       memcpy(extra, buff, dwrq->length);
       DEBUG(DBG_IWCONFIG      , "%s: DOT11_OID_SSID returned %s\n",__FUNCTION__, extra);
       kfree(data);
       kfree(ptr);
       return 0;
}

#define ACLMAX 32
struct acllist
{
	uint8_t addr[6];
        int flag;
	int aclmode;
}acl_list[ACLMAX];

static char macmode = 1;

int initialize_acllist(void)
{
	int index;

	acl_list[0].flag = 1;
	acl_list[0].aclmode = 1;
	acl_list[0].addr[0] = 0xff;
	acl_list[0].addr[1] = 0xff;
	acl_list[0].addr[2] = 0xff;
 	acl_list[0].addr[3] = 0xff;
 	acl_list[0].addr[4] = 0xff;
	acl_list[0].addr[5] = 0xff;

	for(index=1;index < ACLMAX;index++)
	{
		 acl_list[index].flag = 0;
	}
	
	update_acllist();
 return 0;
}

int update_acllist(void)
{

        int i,j;
	uint8_t buff[26];

        mm_segment_t fs;
        struct file *ifp = NULL;
        long bcount = 0;
	
	fs = get_fs();
        set_fs(KERNEL_DS);
        ifp = filp_open(ACL_CONF_FILE , O_TRUNC | O_CREAT | O_WRONLY, 0);

	if(IS_ERR(ifp))
        {
                DEBUG(DBG_ERR,"Cannot open paed.conf file \n");
                set_fs(fs);
                return FOPEN_ERR;
                printk("can not open file acl.conf");
        }
	
	for(i=0,j=0;i < ACLMAX; i++)
	{
		if(acl_list[i].flag == 1)
		{       memset(buff,0,26);
			sprintf(buff,"Acl %d %2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x %d\n",j,(unsigned char)acl_list[i].addr[0],
				(unsigned char)acl_list[i].addr[1],(unsigned char)acl_list[i].addr[2],(unsigned char)acl_list[i].addr[3],
					(unsigned char)acl_list[i].addr[4],(unsigned char)acl_list[i].addr[5],acl_list[i].aclmode);
			
        		bcount = ifp->f_op->write(ifp, buff, 26, &ifp->f_pos);
			j++;
		}
	}
        filp_close(ifp,NULL);
        set_fs(fs);
	return 0;
}
int allowed_acl(char *mac)
{
	int index;	
	for(index=0;index < ACLMAX;index++)
	{
		if(acl_list[index].flag == 1)
			if(!strcmp(mac, acl_list[index].addr))
				return 1;
	}
	return 0;
}


static int ieee80211_ioctl_addmac(struct net_device *dev, struct iw_request_info *info,
	void *w, char *extra)
{
	struct sockaddr *sa = (struct sockaddr *)extra;
	int index,j;
        
	for(index=0;index < ACLMAX; index ++)
        {
                if( acl_list[index].addr[0]==sa->sa_data[0] &&
                	acl_list[index].addr[1]==sa->sa_data[1] &&
                		acl_list[index].addr[2]==sa->sa_data[2] &&
        				acl_list[index].addr[3]==sa->sa_data[3] &&
                				acl_list[index].addr[4]==sa->sa_data[4] &&
                					acl_list[index].addr[5]==sa->sa_data[5])
                	{
			if(acl_list[index].flag == 1)
                        return -EIO;
                	}
        }
	
	for(index=0;index < ACLMAX;index++)
        {
                if(acl_list[index].flag == 0)
                break;
        }
        if(index == ACLMAX)
        return -EIO;

        for(j=0;j<6;j++)
        {
          acl_list[index].addr[j] = sa->sa_data[j];
          acl_list[index].flag = 1;
	  acl_list[index].aclmode = macmode;
        }

	update_acllist();
	return 0;
}


static int ieee80211_ioctl_delmac(struct net_device *dev, struct iw_request_info *info,
	void *w, char *extra)
{
	struct sockaddr *sa = (struct sockaddr *)extra;
	int index,j;

        for(index=0;index < ACLMAX; index ++)
        {
                if( acl_list[index].addr[0]==sa->sa_data[0] &&
                        acl_list[index].addr[1]==sa->sa_data[1] &&
                          acl_list[index].addr[2]==sa->sa_data[2] &&
                                 acl_list[index].addr[3]==sa->sa_data[3] &&
                                         acl_list[index].addr[4]==sa->sa_data[4] &&
                                                 acl_list[index].addr[5]==sa->sa_data[5])
                {		
                	        acl_list[index].flag = 0;
				
				update_acllist();
				return 0;
                }
        }
	return -EIO;
}

static int sm_drv_set_authmode(struct net_device *ndev,
    struct iw_request_info *info, uint32_t *dwrq, char *extra)
{

    	uint32_t *buff = &dwrq[0];
    	char line[100];
  	char* position;
  	int i = 0;
	loff_t offset;

        uint8_t *data = NULL;
        mm_segment_t fs;
        struct file *ifp = NULL;
        long bcount = 0;
        data = kmalloc(100, GFP_KERNEL);

	//char os_string[20]= "os_enable ";	 
	//char wep_string[20]= "wep_enable ";	
	//auth_string
	//open authentication

	if(*buff == 1)
	{
	
		fs = get_fs();
        	set_fs(KERNEL_DS);
		ifp = filp_open(PAED_CONF_FILE , O_RDWR, 0);
	  	if(IS_ERR(ifp))
        	{
                	DEBUG(DBG_ERR,"Cannot open paed.conf file \n");
                	set_fs(fs);
                	kfree(data);
                	return FOPEN_ERR;
                	printk("can not open file paed.wlan0.conf");
		}
		#ifdef HOSTAPDCONF	
		update_file(ifp, auth_string, 1);
		#else
		update_file(ifp,os_string, 1);
		#endif
		filp_close(ifp,NULL);
        	set_fs(fs);


		fs = get_fs();
        	set_fs(KERNEL_DS);
		ifp = filp_open(PAED_CONF_FILE , O_RDWR, 0);
                if(IS_ERR(ifp))
                {
                        DEBUG(DBG_ERR,"Cannot open paed.conf file \n");
                        set_fs(fs);
                        kfree(data);
                        return FOPEN_ERR;
                        printk("can not open file paed.wlan0.conf");
                }
		
                update_file(ifp,wep_string, 0);
                filp_close(ifp,NULL);
                set_fs(fs);
		
        }
	
	//wep enabled open authentication

	if(*buff == 2)

	{	
		fs = get_fs();
                set_fs(KERNEL_DS);
		ifp = filp_open(PAED_CONF_FILE , O_RDWR, 0);
                if(IS_ERR(ifp))
                {
                        DEBUG(DBG_ERR,"Cannot open paed.conf file \n");
                        set_fs(fs);
                        kfree(data);
                        return FOPEN_ERR;
                        printk("can not open file paed.wlan0.conf");
                }
		 #ifdef HOSTAPDCONF
                update_file(ifp, auth_string, 2);
                #else
	
                update_file(ifp,os_string, 0);
		#endif
                filp_close(ifp,NULL);
                set_fs(fs);

		fs = get_fs();
                set_fs(KERNEL_DS);
                ifp = filp_open(PAED_CONF_FILE , O_RDWR, 0);
                if(IS_ERR(ifp))
                {
                        DEBUG(DBG_ERR,"Cannot open paed.conf file \n");
                        set_fs(fs);
                        kfree(data);
                        return FOPEN_ERR;
                        printk("can not open file paed.wlan0.conf");
                }
                update_file(ifp,wep_string, 1);
                filp_close(ifp,NULL);
                set_fs(fs);
		
	}
	//shared authentication
	if(*buff == 4)
	{       

	 	fs = get_fs();
                set_fs(KERNEL_DS);
		ifp = filp_open(PAED_CONF_FILE , O_RDWR, 0);
                if(IS_ERR(ifp))
                {
                        DEBUG(DBG_ERR,"Cannot open paed.conf file \n");
                        set_fs(fs);
                        kfree(data);
                        return FOPEN_ERR;
                        printk("can not open file paed.wlan0.conf");
                }
		 #ifdef HOSTAPDCONF
                update_file(ifp, auth_string, 3);
                #else
	
                update_file(ifp,os_string, 0);
		#endif
                filp_close(ifp,NULL);
	        set_fs(fs);

		fs = get_fs();
                set_fs(KERNEL_DS);
                ifp = filp_open(PAED_CONF_FILE , O_RDWR, 0);
                if(IS_ERR(ifp))
                {
                        DEBUG(DBG_ERR,"Cannot open paed.conf file \n");
                        set_fs(fs);
                        kfree(data);
                        return FOPEN_ERR;
                        printk("can not open file paed.wlan0.conf");
                }
                update_file(ifp,wep_string, 1);
                filp_close(ifp,NULL);
                set_fs(fs);
		
	}
	kfree(data);
	return 0;
}

static int update_file(struct file *ifp, char *str,int value)
{

	 loff_t offset = 0;
	 char data[100];
         int i;

	 while(fgets_local(ifp,data,&offset) != NULL)
         {
                char delimeter,chr;
                loff_t seek_cur;
		char *position;
                if ( (position = strstr(data,str)) != NULL )
                {
                        if ( (position = strchr(data, '#')) == NULL )
                        {
                                seek_cur = ifp->f_pos;
                                for ( i = 0 ; i <= strlen(str); i++ ) {
                                        ifp->f_op->read(ifp,&chr,1,&offset);
                                }
                                if(value == 1)
                                delimeter = '1';
                                if(value == 0)
                                delimeter = '0';
			
				if(value == 2)
                                delimeter = '2';
				if(value == 3)
                                delimeter = '3';
				#ifdef  HOSTAPDCONF				
				offset = offset - 1;
				#endif
                                ifp->f_op->write(ifp,&delimeter,1, &offset);
                        }
                }
        }/* while */

return 0;
}


char *fgets_local(struct file *infile,char *line,loff_t *offset)
{
	char chr = 0 , i = 0 ;
	int ret = 0;

	*offset = infile->f_pos;

        while ( infile->f_op->read(infile,&chr,1,&infile->f_pos) != 0 ) {
	        line[i++] = chr;
	        while( 1 ) 
		{ 
	        	ret = infile->f_op->read(infile,&chr,1,&infile->f_pos);

                        if( chr != '\n' && ret != 0 ) {
                                line[i++] = chr;

                        } else if( chr == '\n' ) {
				

                                line[i++] = chr;
                                line[i] = '\0';
                                return line;

                        } else if ( ret == 0 ) {

                                line[i] = '\0';
                                return line;

                        } else {
				/* do nothing */
			}
                }
        } /* while */
        return NULL;
}

static int sm_drv_set_maccmd(struct net_device *ndev, struct iw_request_info *info, uint32_t *dwrq, char *extra)
{
    	struct net_local  *lp = ndev->priv;
    	uint32_t *buff = &dwrq[0];

	if(*buff == 1 || *buff == 2)
	macmode = *buff;

	if(*buff == 3)
	{
		int index,j;

        	for(index=0;index < ACLMAX; index ++)
        	{
                        acl_list[index].flag = 0;
                }
                update_acllist();
        }
	return 0;
}

static int sm_drv_get_acllist(struct net_device *ndev,
                              struct iw_request_info *info, struct iw_point *dwrq, char *extra)
{
    char *current_ev = extra;
    int i = 0;
    uint32_t length = 0;
    struct sockaddr addr;
    int loopcount=0;

    DEBUG(DBG_IWCONFIG, " %s \n", __FUNCTION__);

	for(i=0;i<ACLMAX;i++)
    	{
        	if(acl_list[i].flag == 1)
        	{	
                	addr.sa_family = ARPHRD_ETHER;
			memcpy(addr.sa_data, acl_list[i].addr, 6);
			memcpy((current_ev+(loopcount*sizeof(struct sockaddr))),&addr,sizeof(struct sockaddr));
                	loopcount ++;
        	}
    	}
    dwrq->length = loopcount;
    dwrq->flags = 0;
    
    return 0;
}

#define SM_DRV_SET_OID_U32	SIOCIWFIRSTPRIV
#define SM_DRV_GET_OID		SIOCIWFIRSTPRIV+1
#define	SM_DRV_SET_OID_STR	SIOCIWFIRSTPRIV+2
#define SM_DRV_GET_OID_REGIONLIST		SIOCIWFIRSTPRIV+3
#define	SM_DRV_SET_OID_ADDR	SIOCIWFIRSTPRIV+4
/* SIOCIWFIRSTPRIV+5 left for get */
#define	SM_DRV_SET_OID_LISTENINTERVAL 	SIOCIWFIRSTPRIV+6
#define	SM_DRV_SET_OID_QOS_DISABLE	SIOCIWFIRSTPRIV+8
#define	SM_DRV_GET_OID_QOS_DISABLE 	SIOCIWFIRSTPRIV+9
#define	SM_DRV_SET_OID_REGION_CODE	SIOCIWFIRSTPRIV+10
     /* SIOCIWFIRSTPRIV+11 left for get */
#define IEEE80211_PARAM_MACCMD		SIOCIWFIRSTPRIV+12

/* SIOCIWFIRSTPRIV+11 left for get */
#define	SM_DRV_SET_AUTH_MODE		SIOCIWFIRSTPRIV+14
#define	SM_DRV_GET_OID_REGION_CHANNEL	SIOCIWFIRSTPRIV+17
#define	SM_DRV_SET_OID_DISASSOCIATE 	SIOCIWFIRSTPRIV+18
#define	SM_DRV_GET_OID_80211d_STATUS	SIOCIWFIRSTPRIV+19
#define SM_DRV_SET_OID_HIDESSID			SIOCIWFIRSTPRIV+20
#define SM_DRV_SET_OID_WMMVAP			SIOCIWFIRSTPRIV+21
#define	IEEE80211_IOCTL_ADDMAC		(SIOCIWFIRSTPRIV+22)
#define	IEEE80211_IOCTL_DELMAC		(SIOCIWFIRSTPRIV+24)
#define	SM_DRV_SET_OID_HIBERNATE	(SIOCIWFIRSTPRIV+16)

#define SM_DRV_GET_OID_HIBERNATE        (SIOCIWFIRSTPRIV+15)
#define SM_DRV_GET_ACL_LIST 		(SIOCIWFIRSTPRIV+7)		


#define IWPRIV_SET_U32(n,x)	{ n, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "set_"x }
#define IWPRIV_SET_SSID(n,x)	{ n, IW_PRIV_TYPE_CHAR | IW_PRIV_SIZE_FIXED| IW_ESSID_MAX_SIZE+1 , 0, "set_"x }
#define IWPRIV_SET_ADDR(n,x)	{ n, IW_PRIV_TYPE_ADDR | IW_PRIV_SIZE_FIXED | 1, 0, "set_"x }
#define IWPRIV_GET(n,m,x)	{ n, 0, IW_PRIV_TYPE_CHAR | IW_PRIV_SIZE_FIXED | PRIV_STR_SIZE, "get_"x }

#define IWPRIV_U32(n,m,x)	IWPRIV_SET_U32(n,x),  IWPRIV_GET(n,m,x)
#define IWPRIV_SSID(n,m,x)	IWPRIV_SET_SSID(n,x), IWPRIV_GET(n,m,x)
#define IWPRIV_ADDR(n,m,x)	IWPRIV_GET(n,m,x)

static const struct iw_priv_args sm_drv_private_args[] = {
  /*{ cmd, set_args, get_args, name } */
  /* --- sub-ioctls handlers --- */
  
  {SM_DRV_GET_OID,
  0, IW_PRIV_TYPE_CHAR | IW_PRIV_SIZE_FIXED | PRIV_STR_SIZE, ""},
  
  {SM_DRV_SET_OID_U32,
  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, ""},
  
  {SM_DRV_SET_OID_STR,
  IW_PRIV_TYPE_BYTE | IW_PRIV_SIZE_FIXED | 1, 0, ""},
  
  {SM_DRV_GET_OID_REGIONLIST,
  0,IW_PRIV_TYPE_CHAR | MAX_COUNTRYLIST_SIZE, "get_region_list"},
  
  {SM_DRV_SET_OID_ADDR,
  IW_PRIV_TYPE_ADDR | IW_PRIV_SIZE_FIXED | 1, 0, ""},
  
  {SM_DRV_SET_OID_LISTENINTERVAL,
   IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "set_lsten_intrvl"},

  {SM_DRV_GET_ACL_LIST,
   0, IW_PRIV_TYPE_ADDR | 20, "getacllist"},
  
  {SM_DRV_SET_OID_QOS_DISABLE ,
   IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "set_wme_disable"},
  {SM_DRV_GET_OID_QOS_DISABLE ,
   0, IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1 , "get_wme_disable"},
  {SM_DRV_SET_OID_REGION_CODE	  ,
   IW_PRIV_TYPE_CHAR | IW_PRIV_SIZE_FIXED|2 , 0, "set_region_code"},
  
  {IEEE80211_PARAM_MACCMD,
   IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "maccmd" },
  
 
   {SM_DRV_SET_AUTH_MODE ,
   IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "authmode"},

   {SM_DRV_GET_OID_HIBERNATE,
   0,IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, "get_rf"},

   {SM_DRV_SET_OID_HIBERNATE,
   IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "set_rf"},

   {SM_DRV_SET_OID_HIDESSID,
   IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "set_hidessid"},

   {SM_DRV_SET_OID_WMMVAP,
   IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1, 0, "set_wmm_vap"},
   

   {IEEE80211_IOCTL_ADDMAC,
   IW_PRIV_TYPE_ADDR | IW_PRIV_SIZE_FIXED | 1, 0,"addmac" },
   { IEEE80211_IOCTL_DELMAC,
   IW_PRIV_TYPE_ADDR | IW_PRIV_SIZE_FIXED | 1, 0,"delmac" },
   
  /* --- sub-ioctls definitions --- */
  IWPRIV_ADDR(0, GEN_OID_MACADDRESS, "addr"),
  IWPRIV_GET(1, GEN_OID_LINKSTATE, "linkstate"),
  IWPRIV_U32(3, DOT11_OID_BSSTYPE, "bsstype"),
  IWPRIV_ADDR(4, DOT11_OID_BSSID, "bssid"),
  IWPRIV_U32(6, DOT11_OID_STATE, "state"),
  IWPRIV_U32(9, DOT11_OID_BEACONPERIOD, "beacon"),
  IWPRIV_U32(10, DOT11_OID_DTIMPERIOD, "dtimperiod"),
  IWPRIV_U32(13, DOT11_OID_AUTHENABLE, "authenable"),
  IWPRIV_U32(14, DOT11_OID_PRIVACYINVOKED, "privinvok"),
  IWPRIV_U32(15, DOT11_OID_EXUNENCRYPTED, "exunencrypt"),

  IWPRIV_U32(31, DOT11_OID_MAXTXLIFETIME, "maxtxlife"),
  IWPRIV_U32(32, DOT11_OID_MAXRXLIFETIME, "maxrxlife"),
  IWPRIV_U32(39, DOT11_OID_ALOFT_FIXEDRATE, "fixedrate"),
  IWPRIV_U32(42, DOT11_OID_MAXFRAMEBURST, "frameburst"),
  IWPRIV_U32(43, DOT11_OID_PSM, "psm"),

  IWPRIV_U32(47, DOT11_OID_CLIENTS, "clients"),


  IWPRIV_U32(54, DOT11_OID_DOT1XENABLE, "dot1xenable"),
  IWPRIV_U32(73, DOT11_OID_ANTENNARX, "rxant"),
  IWPRIV_U32(74, DOT11_OID_ANTENNATX, "txant"),
  IWPRIV_U32(75, DOT11_OID_ANTENNADIVERSITY, "antdivers"),
  IWPRIV_U32(77, DOT11_OID_EDTHRESHOLD, "edthresh"),
  IWPRIV_U32(78, DOT11_OID_PREAMBLESETTINGS, "preamble"),
  IWPRIV_GET(79, DOT11_OID_RATES, "rates"),

  IWPRIV_U32(82, DOT11_OID_OUTPUTPOWER, "dot11outpower"),
  IWPRIV_GET(83, DOT11_OID_SUPPORTEDRATES, "supprates"),
  IWPRIV_GET(85, DOT11_OID_SUPPORTEDFREQUENCIES, "suppfreq"),

  IWPRIV_U32(86, DOT11_OID_NOISEFLOOR, "noisefloor"),
  IWPRIV_U32(87, DOT11_OID_NONERPPROTECTION, "nonerpprotec"),
  IWPRIV_U32(90, DOT11_OID_PROFILES, "profile"),
  IWPRIV_GET(91, DOT11_OID_EXTENDEDRATES, "extrates"),

  IWPRIV_GET(102, DOT11_OID_BSSS, "bsss"),
  IWPRIV_GET(104, DOT11_OID_BSSLIST, "bsslist"),
  IWPRIV_GET(105, DOT11_OID_COUNTRY, "region_code"),
  IWPRIV_GET(106, DOT11_OID_SCAN, "bg_scan_param"),
  IWPRIV_GET(107, 0 , "driver_ver"),
};


static const iw_handler sm_drv_private_handler[] = {
  (iw_handler) sm_drv_set_oid,
  (iw_handler) sm_drv_get_oid,
  (iw_handler) sm_drv_set_raw,
  (iw_handler) sm_drv_get_region_list, 
  (iw_handler) sm_drv_set_raw,
  (iw_handler) NULL,
  (iw_handler) sm_drv_set_listen_interval,
  (iw_handler) sm_drv_get_acllist, 
  (iw_handler) sm_drv_set_wme,
  (iw_handler) sm_drv_get_wme,
  (iw_handler) sm_drv_set_region_code,   /* umac is not setting it */
  (iw_handler) NULL, 
  (iw_handler) sm_drv_set_maccmd,  
  (iw_handler) NULL,
  (iw_handler) sm_drv_set_authmode,  
   (iw_handler) sm_drv_get_hibernate,
  (iw_handler) sm_drv_set_hibernate,  
  (iw_handler) sm_drv_get_region_channel,  
  (iw_handler) sm_drv_set_disassociate,  
  (iw_handler) sm_drv_get_80211d_status, 
  (iw_handler) sm_drv_set_hidessid,
  (iw_handler) sm_drv_set_wmm_vap_disable,
  (iw_handler) ieee80211_ioctl_addmac,		/* SIOCIWFIRSTPRIV+22 */
  (iw_handler) NULL,				/* SIOCIWFIRSTPRIV+23 */
  (iw_handler) ieee80211_ioctl_delmac,		/* SIOCIWFIRSTPRIV+24 */
};


const struct iw_handler_def sm_drv_handler_def = {
	.num_standard = sizeof (sm_drv_handler) / sizeof (iw_handler),
	.num_private = sizeof (sm_drv_private_handler) / sizeof (iw_handler),
	.num_private_args = sizeof (sm_drv_private_args) / sizeof (struct iw_priv_args), 
	.standard = (iw_handler *) sm_drv_handler,
	.private = (iw_handler *) sm_drv_private_handler,
	.private_args = (struct iw_priv_args *) sm_drv_private_args,
};
#endif
