/* ###########################################################################
# (c) Copyright Conexant Systems, Inc. 2004
#
# Conexant Systems, Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc. and its subsidiaries (collectively,
# "Conexant"). It may only be read, used, copied, adapted, modified or 
# otherwise dealt with by you if you have entered into a license agreement 
# with Conexant and then subject to the terms of that agreement and any other 
# applicable confidentiality agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a license agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/

/*****************************************************************************
 *
 * GPIO_BF.H  Bandit family specific GPIO support
 *
 ****************************************************************************/
#ifndef GPIO_LIB_BF_H
#define GPIO_LIB_BF_H

#include "bf_gpio.h"
#include <stddef.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define BF_GPIO  (0x40000100 | 0xFF900000)
#define BF_IOCONF (0x40000600 | 0xFF900000)

/* Convenient types */

typedef enum {
    INPUT      = BF_GPIO_CONTROL_INPUT,       
    INVINPUT   = BF_GPIO_CONTROL_INVINPUT,   
    FUNCTION0  = BF_GPIO_CONTROL_FUNCTION0, 
    FUNCTION1  = BF_GPIO_CONTROL_FUNCTION1,
    FUNCTION2  = BF_GPIO_CONTROL_FUNCTION2, 
#ifdef GALLUS_WLC
    FUNCTION3  = BF_GPIO_CONTROL_FUNCTION3,
    FUNCTION4  = BF_GPIO_CONTROL_FUNCTION4, 
    FUNCTION5  = BF_GPIO_CONTROL_FUNCTION5,
    FUNCTION6  = BF_GPIO_CONTROL_FUNCTION6, 
#else
    INT0_HIGH  = BF_GPIO_CONTROL_IRQ0_HIGH,
    INT0_LOW   = BF_GPIO_CONTROL_IRQ0_LOW,
    INT1_HIGH  = BF_GPIO_CONTROL_IRQ1_HIGH,
    INT1_LOW   = BF_GPIO_CONTROL_IRQ1_LOW,
    INT2_HIGH  = BF_GPIO_CONTROL_IRQ2_HIGH, 
    INT2_LOW   = BF_GPIO_CONTROL_IRQ2_LOW,
#endif
    OUTPUT     = BF_GPIO_CONTROL_OUTPUT,   
    INVOUTPUT  = BF_GPIO_CONTROL_INVOUTPUT, 
    OPENDRAIN  = BF_GPIO_CONTROL_OPENDRAIN,
    OPENSOURCE = BF_GPIO_CONTROL_OPENSOURCE,
    PP_INTHIGH, /* These are not aliases: they correspond to asking the */
    PP_INTLOW,   /* GPIO library to set the mode right for the IRQ this GPIO */
                /* has been given */
    ACTIVEHIGH = PP_INTHIGH,
    ACTIVELOW = PP_INTLOW
} gpio_mode_t;
#define FUNCTION FUNCTION0 /* Backwards-compatible with gpio_augustus.h */

typedef enum {
    IRQ_L,
    IRQ_M,
    IRQ_H,
    FIQ
} gpio_intrlevel_t;

typedef void * gpio_srlevel_t;

/*****************************************************************************
 *
 * Bandit Family GPIO hardware access helper macros:
 *
 *   xxx_BITS_PER_PIN       Width in bits of per-pin information in register
 *   xxx_PINS_PER_REG       Number of pins sharing 32bit register
 *
 *   GPIO_OFFSET()          Register offset for pin information
 *   GPIO_SHIFT()           Bit shift within register for pin information
 *   GPIO_MASK()            Bit mask  within register for pin information
 *
 ****************************************************************************/
#ifndef GPIO_NUM_PINS
#ifdef GALLUS_WLC
#define GPIO_NUM_PINS (23)              /* Number of GPIO pins in Gallus WLC */
#else
#define GPIO_NUM_PINS (48)              /* Number of GPIO pins in Gallus */
#endif
#endif

#ifdef GALLUS_WLC
#define GPIO_TRUE_GPIOS (17)
#define GPIO_STUN_GPIOS (5)
#define GPIO_STUN_GPIO_INPUT_BIT  (0)
#define GPIO_STUN_GPIO_MODE_BITS  (1)
#define GPIO_STUN_GPIO_OUTPUT_BIT (3)
#define GPIO_STUN_INPUT (0)
#define GPIO_STUN_BI (1)
#define GPIO_STUN_OUTPUT (2)
#define GPIO_STUN_TEST (3)
#endif

/* GPIO_CONTROLxxx registers */
#define GPIO_CONTROL_BITS_PER_PIN       (BF_GPIO_CONTROL1_GPIO1_BIT - BF_GPIO_CONTROL1_GPIO0_BIT)
#define GPIO_CONTROL_PINS_PER_REG       (32 / GPIO_CONTROL_BITS_PER_PIN)
#define GPIO_CONTROL_PIN_MASK           (BF_GPIO_CONTROL1_MASK)

/* GPIO_SETxxx registers */
#define GPIO_SET_BITS_PER_PIN           (1)
#define GPIO_SET_PINS_PER_REG           (32 / GPIO_SET_BITS_PER_PIN)
#define GPIO_SET_PIN_MASK               (1)
                                        
/* GPIO_CLEARxxx registers */
#define GPIO_CLEAR_BITS_PER_PIN         (1)
#define GPIO_CLEAR_PINS_PER_REG         (32 / GPIO_CLEAR_BITS_PER_PIN)
#define GPIO_CLEAR_PIN_MASK             (1)
                                        
/* GPIO_INPUTxxx registers */
#define GPIO_INPUT_BITS_PER_PIN         (1)
#define GPIO_INPUT_PINS_PER_REG         (32 / GPIO_INPUT_BITS_PER_PIN)
#define GPIO_INPUT_PIN_MASK             (1)

/* GPIO_OUTPUTxxx registers */
#define GPIO_OUTPUT_BITS_PER_PIN        (1)
#define GPIO_OUTPUT_PINS_PER_REG        (32 / GPIO_OUTPUT_BITS_PER_PIN)
#define GPIO_OUTPUT_PIN_MASK            (1)

/* Register offset, and mask/shift for SET,CLEAR,INPUT,OUTPUT,CONTROL and INFO registers */
#define GPIO_OFFSET(reg,gpioNum)        ( (gpioNum) / GPIO_##reg##_PINS_PER_REG)
#define GPIO_REG_NUM(reg,gpioNum)        (( (gpioNum) / GPIO_##reg##_PINS_PER_REG)+1)
#define GPIO_SHIFT(reg,gpioNum)         (((gpioNum) % GPIO_##reg##_PINS_PER_REG) * GPIO_##reg##_BITS_PER_PIN)
#define GPIO_MASK(reg,gpioNum)          (GPIO_##reg##_PIN_MASK << GPIO_SHIFT(reg,gpioNum))
#define CONCAT(_a, _b) _a##_b

/* Read write register for GPIO pin */
#define REGIO_GPIO(reg, gpioNum)        *(volatile U32*)(BF_GPIO + offsetof(BF_GPIO_REGS, reg##1) + (sizeof(U32)*GPIO_OFFSET(reg, gpioNum)))


/*----------------------------------------------------------------------------
 * Sanity checking: test helper macros
 *--------------------------------------------------------------------------*/
/* Check register bits per pin */
#if (GPIO_CONTROL_BITS_PER_PIN != 4)
# error "GPIO_BITS_PER_PIN defines have incorrect values"
#endif

/* Check register offset for pin */
#if (  (GPIO_OFFSET(INPUT,0)  != 0) || (GPIO_OFFSET(INPUT,31) != 0) \
    || (GPIO_OFFSET(INPUT,32) != 1) || (GPIO_OFFSET(INPUT,47) != 1) )
# error "GPIO_OFFSET(INPUT,pin) gives incorrect results"
#endif

/* Check shift within register for pin */
#if (  (GPIO_SHIFT(INPUT,0)  != 0) || (GPIO_SHIFT(INPUT,1)  !=  1) || (GPIO_SHIFT(INPUT,31) != 31) \
    || (GPIO_SHIFT(INPUT,32) != 0) || (GPIO_SHIFT(INPUT,47) != 15) )
# error "GPIO_SHIFT(reg,pin) gives incorrect results"
#endif

/* Check mask within register for pin */
#if (  (GPIO_MASK(INPUT,0)  != 0x00000001UL) || (GPIO_MASK(INPUT,1)  != 0x00000002UL) \
    || (GPIO_MASK(INPUT,31) != 0x80000000UL) || (GPIO_MASK(INPUT,32) != 0x00000001UL) \
    || (GPIO_MASK(INPUT,63) != 0x80000000UL) )
# error "GPIO_MASK(reg,pin) gives incorrect results"
#endif
#if (  (GPIO_MASK(CONTROL,0)  != 0x0000000FUL) || (GPIO_MASK(CONTROL,1) != 0x000000F0UL) \
    || (GPIO_MASK(CONTROL,7)  != 0xF0000000UL) || (GPIO_MASK(CONTROL,8) != 0x0000000FUL) \
    || (GPIO_MASK(CONTROL,63) != 0xF0000000UL) )
# error "GPIO_MASK(CONTROL,pin) gives incorrect results"
#endif



#ifdef __cplusplus
}
#endif

#endif /* GPIO_LIB_BF_H */
