#include <linux/delay.h>
#include <gpio_bf.h>
#include <linux/module.h>
#include <bf_config_io.h>
#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/netlink.h>
#include <linux/proc_fs.h>

static char MyBuf[32];
#define REGIO(_dev, _reg) ( ((BF_##_dev##_REGS *)(BF_##_dev))->_reg )
struct timer_list gpio_timer;
/* for netlnk message */

#define MAX_PAYLOAD 1024 
static struct work_struct WorkQ;
struct sock *nl_sk = NULL;


void nl_data_ready (struct sock *sk, int len)
{
	printk("CallBack called \n");
}

void send_message(void *arg)
{ 
 struct sk_buff *skb = NULL;
 struct nlmsghdr *nlh;
 
 skb=alloc_skb(NLMSG_SPACE(MAX_PAYLOAD),GFP_KERNEL);
 nlh = (struct nlmsghdr *)skb->data;
 nlh->nlmsg_len = NLMSG_SPACE(MAX_PAYLOAD);
 nlh->nlmsg_pid = 0;  /* from kernel */
 nlh->nlmsg_flags = 0;
 strcpy(NLMSG_DATA(nlh), "Greeting from kernel!");
 /* sender is in group 1<<0 */
 NETLINK_CB(skb).groups = 1;
 NETLINK_CB(skb).pid = 0;  /* from kernel */
 NETLINK_CB(skb).dst_pid = 0;  /* multicast */
 /* to mcast group 1<<0 */
 NETLINK_CB(skb).dst_groups = 1;
 /*multicast the message to all listening processes*/
 netlink_broadcast(nl_sk, skb, 0, 1, GFP_KERNEL);


 printk("Bottom Half Called \n");
 set_current_state(TASK_INTERRUPTIBLE);
 schedule_timeout(100);
 set_current_state(TASK_RUNNING);
 //enable_irq(GPIO_0_IRQ_NUM);

}

/*********************/
void gpio_timer_callback(unsigned long data)
{
    static unsigned long val,new_val,tmp1,tmp2;
    
    val =50;
    new_val = REGIO(IOCONF, CONFIG2);
    
     /*
      * when PB is pressed we get 12222222 and when released we get 2222222.
      */
    tmp1 = 0x22222202UL&0x000000f0;
    tmp2 = new_val&0x000000f0;
    if( tmp1 == tmp2)
    {   
        printk("Value Changed New_Val[%lx] \n",new_val);
        schedule_work(&WorkQ);
        mod_timer( (struct timer_list *)data , jiffies + 500 ) ;
        
    }
    else
    {
        /* Restart the timer */
        mod_timer( (struct timer_list *)data , jiffies + 50 ) ;
    }

}

static int buttonIsPushed(void)
{
    static unsigned long val,new_val,tmp1,tmp2;
    
    val =50;
    new_val = REGIO(IOCONF, CONFIG2);
//    printk("buttonIsPushed Value is [%lx] \n",new_val); 
    tmp1 = 0x22222211UL&0x0000000f;
    tmp2 = new_val&0x0000000f;
//    printk("0x22222211UL&0x0000000f=%x,new_val&0x0000000f=%x\n",tmp1,tmp2);
	
    if(tmp1 == tmp2)
		return 1;
    return 0;
}

static ssize_t proc_read_push_button_fops(struct file *filp,
                                 char *buf,size_t count , loff_t *offp)
{
    int len=0;
    if(*offp!=0)
        return 0;

    *MyBuf = '\0';
    if(buttonIsPushed()) {
//	printk("buttonIsPushed\n");
        strcat(MyBuf, "RST ");
    }
	len=strlen(MyBuf)+1;
	copy_to_user(buf,MyBuf,len);
//	printk("Mybuf=%s\n",MyBuf);
	*offp = len;
	return len;
}

static ssize_t proc_write_push_button_fops(struct file *filp,const char *buffer,
                                     size_t count , loff_t *offp)
{
	return count;
}

struct file_operations push_button_fops = {
        read: proc_read_push_button_fops,
        write: proc_write_push_button_fops,
};

static struct proc_dir_entry *push_button_file;

static int  push_button_file_init_module(void)
{
	printk("init...\n");
	push_button_file=create_proc_entry("push_button",0666,&proc_root);
	push_button_file->owner = THIS_MODULE;
	push_button_file->proc_fops = &push_button_fops;
	REGIO(IOCONF, CONFIG2) &= ~(0x6);/* Configuring GPIO No 5 in Input Mode */

	return 0;
}
static void  push_button_file_cleanup_module(void)
{
	remove_proc_entry("push_button",&proc_root);
	printk("cleanup...\n");
}

int my_init(void)
{
    unsigned long new_val;
    printk("MY INIT Called \n");
    new_val = REGIO(IOCONF, CONFIG2);
    printk("Value init to [%lx] \n",new_val);
    REGIO(IOCONF, CONFIG2) &=  ~(0x60); /* Configuring GPIO No 6 in Input Mode */
    new_val = REGIO(IOCONF, CONFIG2);
    printk("Value init to [%lx] \n",new_val);
    gpio_timer.expires=jiffies + 5;           /* In jiffies. One jiffie equal to 10ms */
    gpio_timer.function= gpio_timer_callback;
    gpio_timer.data=(unsigned long)&gpio_timer;
    init_timer(&gpio_timer);
    mod_timer(&gpio_timer , jiffies + 5);
    INIT_WORK(&WorkQ , (void *)send_message, NULL);
    nl_sk = netlink_kernel_create(NETLINK_WSC_PB,nl_data_ready); 
    push_button_file_init_module();
    return 0;
}

void my_exit(void)
{
  /* Cancel Timer Here */
  push_button_file_cleanup_module();
}

module_init(my_init);
module_exit(my_exit);

