/* ###########################################################################
# (c) Copyright Conexant Systems, Inc. 2004
# (c) Copyright GlobespanVirata, Inc., 2001-2004
#
# Conexant Systems, Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc. and its subsidiaries (collectively 
# "Conexant"). It may only be read, used, copied, adapted, modified or 
# otherwise dealt with by you if you have entered into a license agreement 
# with Conexant and then subject to the terms of that agreement and any other
# applicable confidentiality agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a license agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/
 
/** @file   cnxt_hsl.h 
 *  @brief  Wrapper function for 3rd party OS, Eg: Linux. 
 */

#ifndef CNXT_HSL_H
#define CNXT_HSL_H

# ifndef AS_HZ
#  define AS_HZ        100     /* Internal kernel timer frequency */
# endif

#include <stdarg.h>
#include <linux/version.h>

#include "asm/domain.h"

#define AS_DOMAIN_USER		DOMAIN_USER
#define AS_DOMAIN_MANAGER	DOMAIN_MANAGER 
#define AS_DOMAIN_CLIENT	DOMAIN_CLIENT 

#define AS_TASK_RUNNING        0
#define AS_TASK_INTERRUPTIBLE  1
#define AS_TASK_UNINTERRUPTIBLE    2
#define AS_TASK_STOPPED        4
#define AS_TASK_TRACED     8
#define AS_EXIT_ZOMBIE     16
#define AS_EXIT_DEAD       32

#define AS_INT_MAX     ((int)(~0U>>1))
#define AS_INT_MIN     (-AS_INT_MAX - 1)
#define AS_UINT_MAX    (~0U)
#define AS_LONG_MAX    ((long)(~0UL>>1))
#define AS_LONG_MIN    (-AS_LONG_MAX - 1)
#define AS_ULONG_MAX   (~0UL)

#define AS_KERN_EMERG  "<0>"   /* system is unusable           */
#define AS_KERN_ALERT  "<1>"   /* action must be taken immediately */
#define AS_KERN_CRIT   "<2>"   /* critical conditions          */
#define AS_KERN_ERR    "<3>"   /* error conditions         */
#define AS_KERN_WARNING    "<4>"   /* warning conditions           */
#define AS_KERN_NOTICE "<5>"   /* normal but significant condition */
#define AS_KERN_INFO   "<6>"   /* informational            */
#define AS_KERN_DEBUG  "<7>"   /* debug-level messages         */


#define AS_IRQ_GPIO0        (0)    /* GPIO */
#define AS_IRQ_GPIO1        (1)    /* GPIO */
#define AS_IRQ_GPIO2        (2)    /* GPIO */
#define AS_IRQ_SPI0         (3)    /* SPI channel 0 */
#define AS_IRQ_SPI1         (4)    /* SPI channel 1 */
#define AS_IRQ_CRYPT        (5)    /* crypto unit */
 
#define AS_IRQ_ETH0         (7)    /* On-board MAC 0 */
#define AS_IRQ_ETH1         (8)    /* On-board MAC 1 */
#define AS_IRQ_USB0     (9)    /* 1st USB HC */
#define AS_IRQ_USB1     (10)   /* 2nd ... */
 
#define AS_IRQ_SELF         (24)   /* Self IRQ                       */
#define AS_IRQ_WDOG         (25)   /* Watchdog expiry                */
#define AS_IRQ_TIMER2       (26)   /* Timer 2 expiry                 */
#define AS_IRQ_TIMER3       (27)   /* Timer 3 expiry                 */
#define AS_IRQ_TIMER4       (28)   /* Timer 4 expiry                 */
#define AS_IRQ_UART         (29)   /* UART TX empty / RX full        */
#define AS_IRQ_COMMRX       (30)   /* ARM ICE character received     */
#define AS_IRQ_COMMTX       (31)   /* ARM ICE character sent         */
    
#define AS_NR_IRQS           (32)

/* Driver transmit return codes */
#define ETH_NETDEV_TX_OK 0      /* driver took care of packet */
#define ETH_NETDEV_TX_BUSY 1    /* driver tx path was busy*/
#define ETH_NETDEV_TX_LOCKED -1 /* driver tx lock was already taken */

/* GFP Flags for memory allocations */

/*
 * GFP bitmasks..
 */
/* Zone modifiers in GFP_ZONEMASK (see linux/mmzone.h - low two bits) */
#define AS__GFP_DMA   0x01
#define AS__GFP_HIGHMEM   0x02
 
/*
 * Action modifiers - doesn't change the zoning
 *
 * AS__GFP_REPEAT: Try hard to allocate the memory, but the allocation attempt
 * _might_ fail.  This depends upon the particular VM implementation.
 *
 * AS__GFP_NOFAIL: The VM implementation _must_ retry infinitely: the caller
 * cannot handle allocation failures.
 *
 * AS__GFP_NORETRY: The VM implementation must not retry indefinitely.
 */
#define AS__GFP_WAIT  0x10    /* Can wait and reschedule? */
#define AS__GFP_HIGH  0x20    /* Should access emergency pools? */
#define AS__GFP_IO    0x40    /* Can start physical IO? */
#define AS__GFP_FS    0x80    /* Can call down to low-level FS? */
#define AS__GFP_COLD  0x100   /* Cache-cold page required */
#define AS__GFP_NOWARN    0x200   /* Suppress page allocation failure warning */
#define AS__GFP_REPEAT    0x400   /* Retry the allocation.  Might fail */
#define AS__GFP_NOFAIL    0x800   /* Retry for ever.  Cannot fail */
#define AS__GFP_NORETRY   0x1000  /* Do not retry.  Might fail */
#define AS__GFP_NO_GROW   0x2000  /* Slab internal usage */
#define AS__GFP_COMP  0x4000  /* Add compound page metadata */
#define AS__GFP_ZERO  0x8000  /* Return zeroed page on success */
 
#define AS__GFP_BITS_SHIFT 16 /* Room for 16 __GFP_FOO bits */
#define AS__GFP_BITS_MASK ((1 << AS__GFP_BITS_SHIFT) - 1)
 
/* if you forget to add the bitmask here kernel will crash, period */
#define AS_GFP_LEVEL_MASK (AS__GFP_WAIT|AS__GFP_HIGH|AS__GFP_IO|AS__GFP_FS| \
            AS__GFP_COLD|AS__GFP_NOWARN|AS__GFP_REPEAT| \
            AS__GFP_NOFAIL|AS__GFP_NORETRY|AS__GFP_NO_GROW|AS__GFP_COMP)
 
#define AS_GFP_ATOMIC  (AS__GFP_HIGH)
#define AS_GFP_NOIO    (AS__GFP_WAIT)
#define AS_GFP_NOFS    (AS__GFP_WAIT | AS__GFP_IO)
#define AS_GFP_KERNEL  (AS__GFP_WAIT | AS__GFP_IO | AS__GFP_FS)
#define AS_GFP_USER    (AS__GFP_WAIT | AS__GFP_IO | AS__GFP_FS)
#define AS_GFP_HIGHUSER    (AS__GFP_WAIT | AS__GFP_IO | AS__GFP_FS | AS__GFP_HIGHMEM)
 
/* Flag - indicates that the buffer will be suitable for DMA.  Ignored on some
   platforms, used as appropriate on others */
 
#define AS_GFP_DMA     AS__GFP_DMA

/**************************** ATM *************************/
/* Traffic description */
 
#define AS_ATM_NONE    0       /* no traffic */
#define AS_ATM_UBR     1
#define AS_ATM_CBR     2
#define AS_ATM_VBR     3
#define AS_ATM_ABR     4 
#define AS_ATM_ANYCLASS    5       /* compatible with everything */
 
#define ATM_MAX_PCR -1      /* maximum available PCR */
/* general ATM constants */
#define AS_ATM_CELL_SIZE           53  /* ATM cell size incl. header */
#define AS_ATM_CELL_PAYLOAD        48  /* ATM payload size */
#define AS_ATM_AAL0_SDU            52  /* AAL0 SDU size */
#define AS_ATM_MAX_AAL34_PDU    65535  /* maximum AAL3/4 PDU payload */
#define AS_ATM_AAL5_TRAILER         8  /* AAL5 trailer size */
#define AS_ATM_MAX_AAL5_PDU     65535  /* maximum AAL5 PDU payload */
#define AS_ATM_MAX_CDV       9999  /* maximum (default) CDV */
#define AS_ATM_NOT_RSV_VCI         32  /* first non-reserved VCI value */
 
#define AS_ATM_MAX_VPI        255  /* maximum VPI at the UNI */
#define AS_ATM_MAX_VPI_NNI       4096  /* maximum VPI at the NNI */
#define AS_ATM_MAX_VCI      65535  /* maximum VCI */
 
 
/* "protcol" values for the socket system call */
#define AS_ATM_NO_AAL  0       /* AAL not specified */
#define AS_ATM_AAL0    13      /* "raw" ATM cells */
#define AS_ATM_AAL1    1       /* AAL1 (CBR) */
#define AS_ATM_AAL2    2       /* AAL2 (VBR) */
#define AS_ATM_AAL34   3       /* AAL3/4 (data) */
#define AS_ATM_AAL5    5       /* AAL5 (data) */

enum {
    AS_ATM_VF_ADDR,        /* Address is in use. Set by anybody, cleared
                   by device driver. */
    AS_ATM_VF_READY,       /* VC is ready to transfer data. Set by device
                   driver, cleared by anybody. */
    AS_ATM_VF_PARTIAL,     /* resources are bound to PVC (partial PVC
                   setup), controlled by socket layer */
    AS_ATM_VF_REGIS,       /* registered with demon, controlled by SVC
                   socket layer */
    AS_ATM_VF_BOUND,       /* local SAP is set, controlled by SVC socket
                   layer */
    AS_ATM_VF_RELEASED,    /* demon has indicated/requested release,
                   controlled by SVC socket layer */
    AS_ATM_VF_HASQOS,      /* QOS parameters have been set */
    AS_ATM_VF_LISTEN,      /* socket is used for listening */
    AS_ATM_VF_META,        /* SVC socket isn't used for normal data
                   traffic and doesn't depend on signaling
                   to be available */
    AS_ATM_VF_SESSION,     /* VCC is p2mp session control descriptor */
    AS_ATM_VF_HASSAP,      /* SAP has been set */
    AS_ATM_VF_CLOSE,       /* asynchronous close - treat like VF_RELEASED*/
    AS_ATM_VF_WAITING,     /* waiting for reply from sigd */
    AS_ATM_VF_IS_CLIP,     /* in use by CLIP protocol */
};

/* One2One mapping to SIOCDEVPRIVATE of linux */

#define AS_SIOCDEVPRIVATE 0x89F0  /* to 89FF */ 

/* if_ether.h */
#define AS_ETH_P_IP    0x0800      /* Internet Protocol packet */

#define AS_CHECKSUM_NONE 0
#define AS_CHECKSUM_HW 1
#define AS_CHECKSUM_UNNECESSARY 2

#define AS_OPER_ADD 1
#define AS_OPER_SUB 2


typedef void * voidp;

/* wrapper func for printk */
extern int as_printk(const char *fmt, ...);
 
/* Wrapper function for tasklet_schedule */
void as_bh_schedule(void *t);
 
/* Wrapper function for tasklet_init */
void as_bh_init(
        void **t,
        void (*func)(unsigned long),
        unsigned long data);
 
/* wrapper func for kfree */
void as_kfree(const void * mem);
 
/* wrapper func for kmalloc */
void *as_kmalloc(int size, int flags);
    
/* wrapper functions for cpy-to-usr & cpy-frm-usr */
unsigned long as_copy_to_user(
        void *to,
        const void *from,
        unsigned long n);
 
unsigned long as_copy_from_user(
        void *to,
        const void *from,
        unsigned long n);
 
/* wrapper func for wake-up-interruptible */
void as_wake_up_interruptible(void *wqh);
 
/* wrapper func for poll_wait */
void as_poll_wait(void *filp,
        void *wait_addr,
        void *p);

/* wrapper for request_irq */
int as_request_irq(unsigned int irq, 
        int (*handler)(int, void *, 
        void *),
        unsigned long irqflags, 
        const char * devname, 
        void *dev_id);

/* wrapper for performing the file operations, refer linux/fs.h */
void as_fs_ops(void *file,unsigned int addr);

/* wrapper function for initialising the sk_buff queue */
void as_init_skb_qhead(void **qhead, int size, int num_buffs);

int as_skb_to_nw_stack(void *pBuf, void * dev, unsigned int len);

void as_skb_qtail(void *queue, void *pBuf);

unsigned int as_skb_len(void *pBuf);

void as_skb_set_len(void *pBuf, unsigned int len);

unsigned int as_skb_ptr_diff(void *pBuf);

unsigned int as_skb_q_len(void *queue);

unsigned int as_skb_cloned(void *pBuf);

void *as_alloc_nwbuf(unsigned int size);

void as_reserve_nwbuf(void * pBuf, unsigned int len);

void *as_nwbuf_dequeue(void * queue );

void as_nwbuf_print(const char *str, void *pBuf, unsigned int len, 
        unsigned int offset);

void as_dev_kfree_skb( void *pBuf );

void * as_channel_frm_netdev(void *dev);

char *as_skb_data(void *pBuf);

unsigned int as_sizeof_nwbuf(void );

unsigned int as_offsetof_nwbuf(unsigned int truesize);

unsigned int as_skb_truesize(void *pBuf);

int as_skb_to_nw_stack(void *pBuf, void * dev, unsigned int len);

int as_skb_netrx(void *pBuf);

void as_init_skb_bf_eth(void *pBuf, unsigned int trueSize, 
        unsigned int skb_size);

void as_chk_free_skb_tsklet(void *pBuf, unsigned int skb_size, 
        void* refillQueue, 
        unsigned int refill_queue_len);

void as_free_irq(unsigned int irq, void *dev_id);

unsigned char *as_push_nwbuf(void *pBuf, unsigned int bytes);

unsigned char *as_put_nwbuf(void *skb, unsigned int len);


/*** BUN_IF ***/

typedef void (*atmdev_close)(void *);
typedef int  (*atm_open)(void *vcc);
typedef void (*atm_close)(void *vcc);
typedef int  (*atm_send)(void *vcc,void *skb);

void as_init_atm_ds(void **devops,
        void **phyops1,
        atmdev_close dclose,
        atm_open open,
        atm_close close,
        atm_send send);

void as_init_atm_ds(void **devops,
        void **phyops1,
        atmdev_close dclose,
        atm_open open,
        atm_close close,
        atm_send send);
void *as_register_atmdev(const char *type, 
        void *devops, 
        int number, 
        unsigned long *flags);
void as_deregister_atmdev(void *dev);
void *as_get_atm_vcc_dev_data(void *vcc);
void as_init_atm_dev_data(void *dev, void *data);
void *as_get_atm_dev_esi(void *dev);
void as_set_atm_vpi_bits(void *dev, unsigned int bits);
void as_set_atm_vci_bits(void *dev, unsigned int bits);
void as_set_atm_link_rate(void *dev, unsigned int rate);
void *as_get_atm_qos(void *vcc);
unsigned char as_get_atm_qos_traffic_class(void *qos);
int as_get_atm_qos_pcr(void *qos);
int as_get_atm_qos_max_pcr(void *qos);
int as_get_atm_qos_max_cdv(void *qos);
int as_get_atm_vcc_linkrate(void *vcc);
void *as_get_atm_vcc_ddev_data(void *vcc);
void **as_get_atm_vcc_dev_data_addr(void *vcc);
int as_get_atm_vpi(void *vcc);
short as_get_atm_vci(void *vcc);
unsigned char as_get_atm_qos_aal(void *vcc);
void as_set_atmdevops_module(void *devops, void *owner);

void * as_get_atm_vcc_flags_addr(void *vcc);
void as_set_bit(int posn, void *addr);
void as_clear_bit(int posn, void *addr);
void as_atm_pop_skb(void *vcc, void *skb);
void as_atm_push_skb(void *vcc, void *skb);

/* PROC FS */
typedef int (*proc_write_t)(void *file, const char *buffer,
                                  unsigned long count, void *data);
typedef int (*proc_read_t)( char *page, char **start, long offset, int count,
                               int *eof, void *data );

/* Pointers to interface service routines.  */
typedef int (*open_t)(void *dev);
typedef int (*stop_t)(void *dev);
typedef int (*hard_start_xmit_t) (void *skb, void *dev);
typedef int (*poll_t) (void *dev, int *quota);
typedef int (*hard_header_t) (void *skb, void *dev,
        unsigned short type,void *daddr,void *saddr, 
        unsigned len);
typedef int (*rebuild_header_t)(void *skb);
typedef void (*set_multicast_list_t)(void *dev);
typedef int (*set_mac_address_t)(void *dev,void *addr);
typedef int (*do_ioctl_t)(void *dev, void *ifr, int cmd);
typedef int (*set_config_t)(void *dev, void *map);
typedef int (*hard_header_cache_t)(void *neigh, void *hh);
typedef void (*header_cache_update_t)(void *hh,void *dev,
        unsigned char *  haddr);
typedef int (*change_mtu_t)(void *dev, int new_mtu); 
typedef void (*tx_timeout_t) (void *dev); 
typedef void (*vlan_rx_register_t)(void *dev, void *grp);
typedef void (*vlan_rx_add_vid_t)(void *dev, unsigned short vid);
typedef void (*vlan_rx_kill_vid_t)(void *dev, unsigned short vid);    
typedef int (*hard_header_parse_t)(void *skb, unsigned char *haddr);
typedef int (*neigh_setup_t)(void *dev, void *);
typedef int (*accept_fastpath_t)(void *, void *);
typedef void(*poll_controller_t)(void *dev);
typedef void * (*get_stats_t)(void *dev);
/* The device initialization function. Called only once. */
typedef int (*init_t)(void *dev);

void *as_get_nwbuf_head(void * pBuf);
void *as_get_nwbuf_end(void * pBuf);
void *as_get_nwbuf_shinfo(void *pBuf);
unsigned int as_get_nwbuf_shinfo_size(void );
void as_set_nwbuf_dev(void *pBuf, void *dev);
void as_set_nwbuf_protocol_ethtypetrans(void *pBuf, void *dev);
void as_set_nwbuf_protocol(void *pBuf, short prot);
void as_set_nwbuf_mac_raw(void *pBuf);
void as_set_nwbuf_ip_summed(void *pBuf, int sum);
void as_set_skb_data(void *pBuf, char * data);
void as_arith_skb_data(void *pBuf, int offset, int oper);
void as_set_skb_tail(void *pBuf);
int as_skb_priority(void *pBuf);

void *as_create_procfs_entry(const char *name,void *parent, proc_write_t pfs_write, proc_read_t pfs_read);
void as_remove_procfs(const char *name, void *parent);
void *as_get_ifru_data(void *ifr);
void *as_get_ndev_priv(void *dev);
void *as_set_ndev_priv(void *dev, void *priv);
int as_ndev_iff_promisc(void *dev);
int as_isalpha_lx(char cmd);
char as_toupper_lx(char c);
void as_init_ether_ndev(
        void                     *dev,
        get_stats_t                get_stats,
        open_t                     open,
        stop_t                     stop,
        hard_start_xmit_t         hard_start_xmit,
        set_multicast_list_t     set_multicast_list,
        do_ioctl_t                 do_ioctl);

void as_interruptible_sleep_on(void *q);
signed long as_schedule_timeout( signed long timeout);
void as_set_current_state(int state_value);
void as_init_ether_ptp(void *dev, 
        unsigned int hdr_len, 
        unsigned int mtu, 
        unsigned int addr_len, 
        unsigned int txq_len, 
        get_stats_t get_stats, 
        open_t open, 
        stop_t stop, 
        hard_start_xmit_t xmit, 
        do_ioctl_t do_ioctl );
unsigned int as_sizeof_ndev(void );
void *as_get_ndev_name(void *dev);
void as_set_ndev_init_func(void * dev, init_t init_func);
void as_set_ndev_hard_header_len(void * dev, unsigned int len);
int as_get_ndev_hard_header_len(void * dev);
void *as_get_ndev_dev_addr(void *dev);
int as_register_ndev(void *dev);
void as_unregister_ndev(void *dev);
void *as_ndev_get_by_name(const char *name);
void as_ndev_put(void *dev);
void as_mutex_init(void **mutex);
int as_mutex_down_interruptible(void *mutex);
void as_mutex_up(void *mutex);
int as_ndev_type_arphrd_ether(void *dev);
void as_update_ndev_stats(void *stats, int rx_pack, int tx_pack, int rx_byte, int tx_byte);


#define AS_MAX_ETHS 20
void as_add_ndev_to_ndev_array( void *dev);
int as_del_ndev_frm_ndev_array( void *dev);
void *as_get_stats_array_for_ndev(void *dev);

void as_local_irq_enable();
void as_local_irq_disable();

int as_in_interrupt();
int as_cond_resched();
void as_delay_with_resched(unsigned int delay);
int as_kthread(int (*fn)(void *), void *arg, unsigned long flags);
void as_udelay(unsigned long delay);
void as_mdelay(unsigned long delay);
void as_modify_domain(int d1, int d2);
void as_flush_icache_range(int addr1, int addr2);

/* Defined in mach-solos/arch.c */
extern int as_snprintf(char * buf, int size, const char *fmt, ...);
extern int as_sprintf(char * buf, const char *fmt, ...);
extern int as_sscanf(const char *buf, const char *fmt, ...);

extern unsigned long as_simple_strtoul(const char *cp,char **endp,unsigned int base);
extern char * as_strcat(char *str1, const char *str2);
extern char* as_strncat(char *str1, const char *str2, unsigned long size);
extern char* hsl_strrchr(const char *str,int ch);
extern char* hsl_strchr(const char *str,int ch);
extern int hsl_strcmp(const char * str1,const char *str2);
extern char* hsl_strcpy(char * str1,const char *str2);
extern char* hsl_strncpy(char *str1,const char *str2, unsigned long size);
extern int hsl_strncmp(const char *str1,const char *str2,unsigned long size);
extern char * hsl_strstr(const char *str1,const char *str2);
unsigned long hsl_strspn(const char *str1,const char *str2);
extern unsigned long hsl_strlen(const char *str1);

extern unsigned int *as_psystem_serial_low;   
extern unsigned int *as_psystem_serial_high; 

extern int as_vsnprintf(char *s,int size, const char *format, va_list *arg);
extern int as_vprintk(const char *format, va_list *args);
char* as_strsep(char **str1,const char *str2);

#define strcpy   	hsl_strcpy
#define strcspn  	hsl_strcspn
#define strcmp   	hsl_strcmp 
#define strrchr 	hsl_strrchr
#define strchr 		hsl_strchr 
#define strncmp 	hsl_strncmp
#define strlen 		hsl_strlen
#define strstr 		hsl_strstr
#define strspn 		hsl_strspn
#define strncpy 	hsl_strncpy
#define strncat 	as_strncat
#define snprintf 	as_snprintf
#define strcat 		as_strcat
#define sscanf   	as_sscanf
#define sprintf  	as_sprintf

/* TODO: Dont uncomment following defines, as this leads to hang.
 */


#if 0
char * as_strstr(const char *str1,const char *str2);

/************************************* STRING FUNCTIONS **********************************/

#define snprintf as_snprintf
//#define stderr as_stderr defined in lx_microls
//#define stdout as_stdout defined in lx_microls  
#endif // end of comment

#endif /* CNXT_HSL_H */
