/* ###########################################################################
# (c) Copyright Conexant Systems, Inc. 2006
# (c) Copyright GlobespanVirata, Inc., 1997-2004
#
# Conexant Systems, Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc. and its subsidiaries (collectively 
# "Conexant"). It may only be read, used, copied, adapted, modified or 
# otherwise dealt with by you if you have entered into a license agreement 
# with Conexant and then subject to the terms of that agreement and any other
# applicable confidentiality agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a license agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/

/**@file       cnxt_hsl_port.c  
*  @brief      Linux interface code.
*  @ingroup    message
*  
* This file contains Shim layer for Linux kernel interface. 
*/

#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/tty_driver.h>
#include <linux/smp_lock.h>
#include <linux/console.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/interrupt.h>            /* For in_interrupt() */
#include <linux/config.h>
#include <linux/delay.h>
#include <linux/smp.h>
#include <linux/security.h>
#include <linux/bootmem.h>
#include <linux/syscalls.h>
#include <linux/version.h>
#include <linux/dma-mapping.h>  	 
#include <linux/phy.h> 	 
#include <linux/ethtool.h> 	 
#include <linux/mii.h> 	 
#include <linux/if_vlan.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
#include <linux/hardirq.h>
#include <asm/cacheflush.h>
#else
#include <asm/hardirq.h>
#endif

#if LINUX_VERSION_CODE == KERNEL_VERSION(2,6,8)
#include <asm/domain.h>
#endif

#include <linux/completion.h>
#include <linux/sched.h>
#include <linux/proc_fs.h>

#include <linux/atm.h>
#include <linux/atmdev.h>
#include <net/xfrm.h>

#include "cnxt_hsl.h"
#include "cnxt_hsl_port.h"
#include "cnxt_hsl_module.h"

#include <linux/linkage.h>

/* Undefine the following symbols as this leads to recursive calls.
 */

#ifdef CNXT_HSL_H
  #undef strcpy 
  #undef strcspn
  #undef strcmp
  #undef strrchr
  #undef strchr
  #undef strncmp 
  #undef strcat
  #undef strncat
  #undef strlen
  #undef strstr
  #undef strspn
  #undef strncpy
#endif

unsigned int *as_psystem_serial_low = (unsigned int *)&system_serial_low ;
EXPORT_SYMBOL(as_psystem_serial_low);

unsigned int *as_psystem_serial_high = (unsigned int *)&system_serial_high;
EXPORT_SYMBOL(as_psystem_serial_high);


void as_udelay(unsigned long delay)
{
	udelay(delay);
}

void as_mdelay(unsigned long delay)
{
	mdelay(delay);
}

EXPORT_SYMBOL(as_mdelay);
EXPORT_SYMBOL(as_udelay);

int as_in_interrupt()
{
	return (in_interrupt());
}

EXPORT_SYMBOL(as_in_interrupt);

/* Create a thread in the kernel mode.
 */

int as_kthread(int (*fn)(void *), void *arg, unsigned long flags)
{
	return (kernel_thread(fn, arg, flags));
}

EXPORT_SYMBOL(as_kthread);

/* Following ds will be used bun_if for storing net_device pointers & statistics */
static struct net_device_stats as_stats_array[AS_MAX_ETHS];
static struct net_device *as_dev_array[AS_MAX_ETHS] = { NULL };


 /**
 *  This  function is used for scheduling the bottom half. 
 *  @param  t               Void pointer.
 *  @return                 None.
 */
   
#include <linux/interrupt.h>
void as_bh_schedule(void *t)
{
    tasklet_schedule((struct tasklet_struct *)t);
}   

 /**
 *  This  function initialises the bottom half, 
 *  allocated mem for the bh structure & then 
 *  doest the init.
 *  @param  t               tasklet struct.
 *  @param  func            fuction for bh.
 *  @param  data            properatory data.
 *  @return                 None.
 */

void as_bh_init(void **t,
                  void (*func)(unsigned long), 
                  unsigned long data)
{
    *t = (struct tasklet_struct *)kmalloc( 
            sizeof(struct tasklet_struct), 
            GFP_KERNEL);

    tasklet_init((struct tasklet_struct *)*t, func , data);
}

 /**
   Sets the current state of the scheduler.
 *  @param  state_value     value to be set.
 *  @return                 None.
 */
void as_set_current_state(int state_value)
{
    set_current_state(state_value);
}
 
/**
 * as_schedule_timeout - sleep until timeout
 * @timeout: timeout value in jiffies
 * In all cases the return value is guaranteed to be non-negative.
 */

signed long as_schedule_timeout( signed long timeout)
{
    return (schedule_timeout( timeout));
}

/**
  * as_kfree - free previously allocated memory
  * @mem: pointer returned by kmalloc.
  *
  * Don't free memory not originally allocated by kmalloc()
  * or you will run into trouble.
  */

#include <linux/slab.h>
 
void as_kfree(const void * mem)
{
    kfree(mem);
}

void * as_memset(void *src,int c,int size)
{
    memset(src,c,size);
}
EXPORT_SYMBOL(as_memset);
 
unsigned long
as_copy_to_user(void *to, const void *from, 
        unsigned long n)
{
    return (copy_to_user( to, from, n));
}
 
unsigned long
as_copy_from_user(void *to, const void *from, 
        unsigned long n)
{
    return (copy_from_user( to, from, n));
}
 
void as_wake_up_interruptible(void *wqh)
{
    wake_up_interruptible ((wait_queue_head_t *)wqh);
}
 
void as_poll_wait(void *filp, void *wait_addr, void *p)
{
    poll_wait((struct file * )filp, 
            (wait_queue_head_t * )wait_addr, 
            (poll_table *)p);
}
 
void as_interruptible_sleep_on(void *q)
{
    interruptible_sleep_on((wait_queue_head_t *)q);
}
 
void *as_kmalloc(int size, int flags)
{
    return (kmalloc( size, flags));
}

/* wrapper for request_irq */

typedef int (*lxhandler_t)(int, void *, struct pt_regs *);

int as_request_irq(unsigned int irq, 
        int (*handler)(int, void *, void *),
        unsigned long irqflags, 
        const char * devname, 
        void *dev_id)
{
    return (request_irq( irq, 
                (lxhandler_t )handler, 
                irqflags, 
                devname, 
                dev_id));    
}

/**
 *  as_free_irq - free an interrupt
 *  @irq: Interrupt line to free
 *  @dev_id: Device identity to free
 *
 *  Remove an interrupt handler. The handler is removed and if the
 *  interrupt line is no longer in use by any driver it is disabled.
 *  On a shared IRQ the caller must ensure the interrupt is disabled
 *  on the card it drives before calling this function.
 *
 *  This function must not be called from interrupt context.
 */
void as_free_irq(unsigned int irq, void *dev_id)
{
    free_irq(irq,dev_id);
}

void as_fs_ops(void *file,unsigned int addr)
{
    ((struct file*)file)->f_pos=addr;
}

/* wrapper function for initialising the sk_buff queue */

void as_init_skb_qhead(void **qhead, int size, int num_buffs)
{
    int i = 0;
    struct sk_buff  *pBuf ;
    *qhead = (struct sk_buff_head *)kmalloc( 
            sizeof(struct sk_buff_head), 
            GFP_KERNEL);
    skb_queue_head_init((struct sk_buff_head *)*qhead);

    /* If we invoke this function by passing num_buffs as '0'
       then only the qhead will be initialised!
     */
    if ( num_buffs )
    {
        for (i=0; i < num_buffs ; i++ ) {
            if ( (pBuf = (struct sk_buff *)dev_alloc_skb(size)) )
                __skb_queue_tail((struct sk_buff_head *)*qhead, 
                        pBuf );
        }
    }
}

void as_skb_qtail(void *queue, void *pBuf)
{
    struct sk_buff_head *pQhead = (struct sk_buff_head *) queue;
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;

    __skb_queue_tail( pQhead, pSkb );
}

unsigned int as_skb_len(void *pBuf)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    return(pSkb->len);
}

void as_skb_set_len(void *pBuf, unsigned int len)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    (pSkb->len) = len;
}

unsigned int as_skb_ptr_diff(void *pBuf)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    return (pSkb->end - pSkb->head);
}

unsigned int as_skb_q_len(void *queue)
{
    struct sk_buff_head *pQhead = (struct sk_buff_head *) queue;
    return(skb_queue_len(pQhead));
}

/**
 *  Returns true if the buffer was generated with skb_clone() and is
 *  one of multiple shared copies of the buffer. Cloned buffers are
 *  shared data so must not be written to under normal circumstances.
 */
unsigned int as_skb_cloned(void *pBuf)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    return (skb_cloned(pSkb));
    
}

void *as_alloc_nwbuf(unsigned int size)
{
    return((void *) dev_alloc_skb(size));
}

void as_reserve_nwbuf(void * pBuf, unsigned int len)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    skb_reserve(pSkb, len);    
}

void *as_nwbuf_dequeue(void * queue )
{
    struct sk_buff_head *pQhead = (struct sk_buff_head *) queue;
    return ((void *)__skb_dequeue( pQhead ));
}

unsigned char *as_push_nwbuf(void *pBuf, unsigned int bytes)
{
    return (skb_push((struct sk_buff *)pBuf, bytes));
}
void as_nwbuf_print(const char *str, void *pBuf, unsigned int len, 
        unsigned int offset)
{
    unsigned int ii=0;
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    printk("%s\n", str);
    for(ii=0;ii<=len;ii++)
    {
        if(ii%16)
        {
            printk("%x ", pSkb->data[ii+offset]);
        }
        else
        {
            printk("\n%x ", pSkb->data[ii+offset]);
        }
    }
}

void as_dev_kfree_skb( void *pBuf )
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    dev_kfree_skb(pSkb); 
}

void * as_channel_frm_netdev(void *dev)
{
    struct net_device *ndev = (struct net_device *) dev; 
    return (ndev->priv);
}





char *as_skb_data(void *pBuf)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    return(pSkb->data);
}

unsigned int as_sizeof_nwbuf(void )
{
    return(sizeof(struct sk_buff));
}

unsigned int as_offsetof_nwbuf(unsigned int truesize)
{
    return (offsetof(struct sk_buff, truesize));
}

unsigned int as_skb_truesize(void *pBuf)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    return(pSkb->truesize);    
}

//FIXME: move to diff file, driver spec function
int as_skb_to_nw_stack(void *pBuf, void * dev, unsigned int len)
{
    struct net_device *ndev = (struct net_device *) dev; 
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;

    pSkb->len     = len;
    pSkb->tail     = pSkb->data + pSkb->len;
    pSkb->dev     = (struct net_device *)ndev;

    if ( pSkb->dev->type != ARPHRD_ETHER )
    {
      pSkb->protocol     = htons(ETH_P_IP);
      pSkb->mac.raw     = pSkb->data;
    }
    else
    {
       pSkb->protocol = eth_type_trans( pSkb, pSkb->dev );
    }

    pSkb->ip_summed = CHECKSUM_NONE;
    return(netif_rx(pSkb));
}
/* wrapper for netif_rx */
int as_skb_netrx(void *pBuf)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    return(netif_rx(pSkb));
}

void as_init_skb_bf_eth(void *pBuf, unsigned int trueSize, 
        unsigned int skb_size)
{
     struct sk_buff *pSkb = (struct sk_buff *) pBuf;
     pSkb->truesize = trueSize;
	/* Nitin Added since 16 bytes are reserved by kernel at the time of allocation of skb */
     pSkb->data=pSkb->head + 16;
     pSkb->tail=pSkb->head + 16;
     pSkb->end=pSkb->head + skb_size + 16;
     atomic_set(&pSkb->users,1);
     atomic_set(&(skb_shinfo(pSkb)->dataref),1);
     skb_shinfo(pSkb)->nr_frags=0;
     skb_shinfo(pSkb)->tso_size=0;
     skb_shinfo(pSkb)->tso_segs=0;
     skb_shinfo(pSkb)->frag_list=0;
}

void as_chk_free_skb_tsklet(void *pBuf, unsigned int skb_size, 
        void* refillQueue, 
        unsigned int refill_queue_len)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
//#ifdef BF_SKB_CLONE
    if( (pSkb->end - pSkb->head >= skb_size ) &&
        (skb_queue_len((struct sk_buff_head *)refillQueue) <
         refill_queue_len) && (!pSkb->cloned) &&
        (atomic_read(&pSkb->users )==1)  && 
        (atomic_read(&(skb_shinfo(pSkb)->dataref))==1) &&
        ((skb_shinfo(pSkb)->nr_frags==0)) && 
        (skb_shinfo(pSkb)->frag_list==NULL))  {
 
        dst_release(pSkb->dst);
#ifdef CONFIG_XFRM               
        secpath_put(pSkb->sp);
#endif
        if ( pSkb->destructor )
        {
            pSkb->destructor(pSkb);
        }
#ifdef CONFIG_NETFILTER
            nf_conntrack_put(pSkb->nfct);
#ifdef CONFIG_BRIDGE_NETFILTER
            nf_bridge_put(pSkb->nf_bridge);
#endif
#endif
            __skb_queue_tail((struct sk_buff_head *) refillQueue, pSkb );
        }
        else
//#endif
        {
            dev_kfree_skb(pSkb);
        }
}

/** 
 *  as_nwbuf_put - add data to a buffer
 *  @skb: buffer to use
 *  @len: amount of data to add
 *
 *  This function extends the used data area of the buffer. If this would
 *  exceed the total buffer size the kernel will panic. A pointer to the
 *  first byte of the extra data is returned.
 */ 
unsigned char *as_put_nwbuf(void *skb, unsigned int len)
{
    return(skb_put((struct sk_buff *)skb,len));
}

/********************************************************************************
  *
  * SHIM LAYER FOR bun_if ATM
  *
  ******************************************************************************/
//#include <linux/module.h>
//#include <linux/atm.h>
//#include <linux/atmdev.h>

void as_init_atm_ds(void **devops,
        void **phyops1,
        atmdev_close dclose,
        atm_open open,
        atm_close close,
        atm_send send)
{
    *devops = (struct atmdev_ops *)kmalloc(sizeof(struct atmdev_ops), GFP_KERNEL);
    *phyops1 = (struct atmphy_ops *)kmalloc(sizeof(struct atmphy_ops), GFP_KERNEL);
     
    ((struct atmdev_ops *)*devops)->dev_close    = dclose;
    ((struct atmdev_ops *)*devops)->open        = open;
    ((struct atmdev_ops *)*devops)->close        = close;
    ((struct atmdev_ops *)*devops)->ioctl        = NULL;
    ((struct atmdev_ops *)*devops)->getsockopt = NULL;
    ((struct atmdev_ops *)*devops)->setsockopt = NULL;
    ((struct atmdev_ops *)*devops)->send        = send;
    ((struct atmdev_ops *)*devops)->send_oam    = NULL;
    ((struct atmdev_ops *)*devops)->phy_put    = NULL;
    ((struct atmdev_ops *)*devops)->phy_get     = NULL;
    ((struct atmdev_ops *)*devops)->change_qos = NULL;
    ((struct atmdev_ops *)*devops)->proc_read  = NULL;
    ((struct atmdev_ops *)*devops)->owner         = NULL;

    
    ((struct atmphy_ops*)*phyops1)->start         = NULL;
    ((struct atmphy_ops*)*phyops1)->ioctl        = NULL;
    ((struct atmphy_ops*)*phyops1)->interrupt    = NULL;
    ((struct atmphy_ops*)*phyops1)->stop         = NULL;
}

void *as_register_atmdev(const char *type, 
        void *devops, 
        int number, 
        unsigned long *flags)
{
    return(atm_dev_register( type, (struct atmdev_ops *)devops, number, flags));    
}

void as_set_atmdevops_module(void *devops, void *owner)
{
    ((struct atmdev_ops *)devops)->owner = (struct module *)THIS_MODULE;
}

void as_deregister_atmdev(void *dev)
{
    atm_dev_deregister((struct atm_dev*)dev);    
}

void as_init_atm_dev_data(void *dev, void *data)
{
   ((struct atm_dev*)dev)->dev_data = data;
}

/* gets the mac */
void *as_get_atm_dev_esi(void *dev)
{
    return(((struct atm_dev*)dev)->esi);
}

/* set the vpi bits */
void as_set_atm_vpi_bits(void *dev, unsigned int bits)
{
   ((struct atm_dev*)dev)->ci_range.vpi_bits = bits;
}

/* set the vci bits */
void as_set_atm_vci_bits(void *dev, unsigned int bits)
{
   ((struct atm_dev*)dev)->ci_range.vci_bits = bits;
}

/* set the vci bits */
void as_set_atm_link_rate(void *dev, unsigned int rate)
{
   ((struct atm_dev*)dev)->link_rate = rate;
}

void *as_get_atm_qos(void *vcc)
{
    return ((&((struct atm_vcc *)vcc)->qos));
}

unsigned char as_get_atm_qos_traffic_class(void *qos)
{
    return (((struct atm_qos *)qos)->txtp.traffic_class);
}

/* desired PCR in cells per second */
int as_get_atm_qos_pcr(void *qos)
{
    return (((struct atm_qos *)qos)->txtp.pcr);
}

/* MAX PCR in cells per second */
int as_get_atm_qos_max_pcr(void *qos)
{
    return (((struct atm_qos *)qos)->txtp.max_pcr);
}

/* maximum CDV in microseconds */
int as_get_atm_qos_max_cdv(void *qos)
{
    return (((struct atm_qos *)qos)->txtp.max_cdv);
}

int as_get_atm_vcc_linkrate(void *vcc)
{
    return(((struct atm_vcc *)vcc)->dev->link_rate);
}
/* returns vcc->dev->dev_data */
void *as_get_atm_vcc_ddev_data(void *vcc)
{
    return(((struct atm_vcc *)vcc)->dev->dev_data);
}

/* returns vcc->dev_data */
void *as_get_atm_vcc_dev_data(void *vcc)
{
    return(((struct atm_vcc *)vcc)->dev_data);
}

/* returns &vcc->dev_data */
void **as_get_atm_vcc_dev_data_addr(void *vcc)
{
    return(&(((struct atm_vcc *)vcc)->dev_data));
}

/* returns the VPI for this vcc */
int as_get_atm_vpi(void *vcc)
{
    return(((struct atm_vcc *)vcc)->vpi);
}

/* returns the VCI for this vcc */
short as_get_atm_vci(void *vcc)
{
    return(((struct atm_vcc *)vcc)->vci);
}

unsigned char as_get_atm_qos_aal(void *vcc)
{
    return(((struct atm_vcc *)vcc)->qos.aal);
}


void * as_get_atm_vcc_flags_addr(void *vcc)
{
    return((void *)(&(((struct atm_vcc *)vcc)->flags)));
}
void as_set_bit(int posn, void *addr)
{
    set_bit(posn, addr);
}
void as_clear_bit(int posn, void *addr)
{
    clear_bit(posn, addr);
}

void as_atm_pop_skb(void *vcc, void *skb)
{
  if(((struct atm_vcc *)vcc)->pop)
  {
      ((struct atm_vcc *)vcc)->pop( vcc,(struct sk_buff *)skb );
  }
  else
  {
     dev_kfree_skb((struct sk_buff *)skb); 
  }
}

void as_atm_push_skb(void *vcc, void *skb)
{
  if(((struct atm_vcc *)vcc)->push)
  {
      ((struct atm_vcc *)vcc)->push( vcc,(struct sk_buff *)skb );
  }
}
/********************************************************************************
  *
  * SHIM LAYER FOR bun_if PROC FS 
  *
  ******************************************************************************/

void *as_create_procfs_entry(const char *name,void *parent, proc_write_t pfs_write, proc_read_t pfs_read)
{
    struct proc_dir_entry *bun_if;    
    bun_if = create_proc_entry(name, S_IFREG | S_IRUSR ,(struct proc_dir_entry *)parent);
    bun_if->write_proc = pfs_write;
    bun_if->read_proc = pfs_read;
    return((void *)bun_if);
}


/*
 * Remove a /proc entry and free it if it's not currently in use.
 * If it is in use, we set the 'deleted' flag.
 */
void as_remove_procfs(const char *name, void *parent)
{
    remove_proc_entry( name, (struct proc_dir_entry *)parent);    
}


void *as_get_ifru_data(void *ifr)
{
    return(((struct ifreq *)ifr)->ifr_ifru.ifru_data);
}


#define LEGACY_ETHER_PROC 1

#if	(defined(LEGACY_ETHER_PROC) && defined(CONFIG_ARM_SOLOS_ETHER) )

static struct ethtool_cmd vlan_cmd;

int as_set_phy_attr(char *attrname,struct phy_device *phydev,int val,int second_val)
{
	struct ethtool_cmd cmd;
	memset(&cmd,0,sizeof(cmd));

	if (! strcmp ( attrname,"SwtchLearnEnbl" ))
	{
		cmd.swtchInfo.configure_switch=1;

		if ( val >=1)
			cmd.swtchInfo.swtchLrnNeeded=1;
		if ( val == 0 )
			cmd.swtchInfo.swtchLrnNeeded=-1;

		configure_switch(phydev,&cmd);
		return 0;
	}
    else if(! strcmp ( attrname,"FlowControl" ))
	{
		cmd.swtchInfo.configure_switch=1;

		if ( val >=1)
			cmd.swtchInfo.flowctrlNeeded=1;
		if ( val == 0 )
			cmd.swtchInfo.flowctrlNeeded=-1;
		configure_switch(phydev,&cmd);
		return 0;
	}
	else if(!strcmp(attrname,"VLANEnabled"))
	{
		vlan_cmd.swtchInfo.configure_switch=1;

        if ( val == 0 )
		{
				/* Disable the VLAN from Switch */
	            vlan_cmd.swtchInfo.vlanNeeded=-1;
				vlan_cmd.swtchInfo.NVlans=0;	
				configure_switch(phydev,&vlan_cmd);
				memset(&vlan_cmd,0,sizeof(vlan_cmd));
				return 0;
		}			
        if ( val >=1)
            vlan_cmd.swtchInfo.vlanNeeded=1;
		

	}
	else if(!strcmp(attrname,"NumVlans"))
		 vlan_cmd.swtchInfo.NVlans=val;
	else if( !strcmp(attrname,"VlanMask"))
	{
		vlan_cmd.swtchInfo.VMask[val]=second_val;		
	}
/* since vlan is configured with multiple commands needed Configure the VLAN only if all the information needed is available */
	if( ( vlan_cmd.swtchInfo.vlanNeeded >=1 ) && ( vlan_cmd.swtchInfo.NVlans > 0 ))
	{
		int i;
		for(i=0;i<vlan_cmd.swtchInfo.NVlans;i++)
		{
			if(vlan_cmd.swtchInfo.VMask[i] == 0 )
				return 0;
		}	

		configure_switch(phydev,&vlan_cmd);
        vlan_cmd.swtchInfo.vlanNeeded=-1;
	}  

}
EXPORT_SYMBOL(as_set_phy_attr); 


int as_get_phy_attr(char *attrname,struct phy_device *phydev)
{
	if( !strcmp(attrname,"LinkSpeed"))
		return phydev->speed;
	else if( !strcmp(attrname,"PortLinkStatus"))
		return phydev->link;
	else if( !strcmp(attrname,"FullDuplex"))	
		return phydev->duplex;
	else if( !strcmp ( attrname,"ChipName" ))
		printk("\n ChipName		:: %s",phydev->drv->name);
	else if (! strcmp ( attrname,"SwtchLearnEnbl" )) 
		return phydev->bus->learning_status;
 	else if(! strcmp ( attrname,"FlowControl" ))
		return phydev->bus->flow_status;
 	else if(! strcmp ( attrname,"VLANEnabled" ))
		return phydev->bus->vlan_status;

	return 0;
}

EXPORT_SYMBOL(as_get_phy_attr); 
#endif


void *as_get_ndev_priv(void *dev)
{
    return (((struct net_device *)dev)->priv);
}

void *as_set_ndev_priv(void *dev, void *priv)
{
    ((struct net_device *)dev)->priv = (void *)priv;
}

int as_ndev_iff_promisc(void *dev)
{
    return((((struct net_device *)dev)->flags & IFF_PROMISC));
}

void *as_get_nwbuf_head(void * pBuf)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    return(pSkb->head);
}

void *as_get_nwbuf_end(void * pBuf)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    return(pSkb->end);
}

void *as_get_nwbuf_shinfo(void *pBuf)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    return(pSkb->end);
    
}

unsigned int as_get_nwbuf_shinfo_size(void )
{
    return(sizeof(struct skb_shared_info ));
}



/* Setup the ether_device & initialises the function pointers */

void as_init_ether_ndev(
        void                     *dev,
        get_stats_t                get_stats,
        open_t                     open,
        stop_t                     stop,
        hard_start_xmit_t         hard_start_xmit,
        set_multicast_list_t     set_multicast_list,
        do_ioctl_t                 do_ioctl)
{
    struct net_device * ndev = (struct net_device *)dev;
    int tmp = ndev->hard_header_len;

    ether_setup(ndev);
    ndev->hard_header_len += tmp;
    ndev->get_stats = get_stats;
    ndev->open = open;
    ndev->stop = stop;
    ndev->hard_start_xmit = hard_start_xmit;
    ndev->set_multicast_list = set_multicast_list;
    ndev->do_ioctl = do_ioctl;
}

void as_init_ether_ptp(void *dev, 
        unsigned int hdr_len, 
        unsigned int mtu, 
        unsigned int addr_len, 
        unsigned int txq_len, 
        get_stats_t get_stats, 
        open_t open, 
        stop_t stop, 
        hard_start_xmit_t xmit, 
        do_ioctl_t do_ioctl )
{
    struct net_device * ndev = (struct net_device *)dev;

    ndev->type = ARPHRD_VOID;
    ndev->hard_header_len = hdr_len;
    ndev->mtu = mtu;
    ndev->addr_len = addr_len;
    ndev->tx_queue_len = txq_len;
    
    ndev->open = open;
    ndev->get_stats = get_stats;
    ndev->stop = stop;
    ndev->hard_start_xmit = xmit;
    ndev->do_ioctl = do_ioctl;

    /* Not ethernet-like */
    ndev->flags = IFF_POINTOPOINT | IFF_NOARP | IFF_MULTICAST;
}

unsigned int as_sizeof_ndev(void )
{
    return(sizeof(struct net_device));
}

void *as_get_ndev_name(void *dev)
{
    struct net_device * ndev = (struct net_device *)dev;
    return(ndev->name);
}

/* for rem warning */
typedef int (*init_lx_t)(struct net_device *dev);

void as_set_ndev_init_func(void * dev, init_t init_func)
{
    struct net_device * ndev = (struct net_device *)dev;
    ndev->init = (init_lx_t )init_func;
}

void as_set_ndev_hard_header_len(void * dev, unsigned int len)
{
    struct net_device * ndev = (struct net_device *)dev;
    ndev->hard_header_len = len;
}

int as_get_ndev_hard_header_len(void * dev)
{
    struct net_device * ndev = (struct net_device *)dev;
    return(ndev->hard_header_len);
}

void *as_get_ndev_dev_addr(void *dev)
{
    struct net_device * ndev = (struct net_device *)dev;
    return(ndev->dev_addr);
}


int as_register_ndev(void *dev)
{
    return (register_netdev((struct net_device *)dev));
}

int as_isalpha_lx(char cmd)
{
    return(isalpha(cmd));
}

char as_toupper_lx(char c)
{
    return(toupper(c));
}

/**
 *  as_unregister_netdev - remove device from the kernel
 *  @dev: device 
 *  
 *  This function shuts down a device interface and removes it
 *  from the kernel tables. On success 0 is returned, on a failure
 *  a negative errno code is returned.
 *
 *  This is just a wrapper for unregister_netdevice that takes
 *  the rtnl semaphore.  In general you want to use this and not
 *  unregister_netdevice.
 */

void as_unregister_ndev(void *dev)
{
    unregister_netdev((struct net_device *)dev);
}

/**
 *  dev_get_by_name     - find a device by its name
 *  @name: name to find
 *
 *  Find an interface by name. This can be called from any
 *  context and does its own locking. The returned handle has
 *  the usage count incremented and the caller must use dev_put() to
 *  release it when it is no longer needed. %NULL is returned if no
 *  matching device is found.
 */
 
void *as_ndev_get_by_name(const char *name)
{
    return (dev_get_by_name(name));
}

void as_ndev_put(void *dev)
{
    dev_put((struct net_device *)dev);
}

/* MUTEX */

void as_mutex_init(void **mutex)
{
    /* Allocate memory for semaphore*/
    *mutex = (struct semaphore *)kmalloc( sizeof(struct semaphore), GFP_KERNEL);
    sema_init((struct semaphore *)*mutex, 1);
}

int as_mutex_down_interruptible(void *mutex)
{
    return(down_interruptible(mutex));
}

void as_mutex_up(void *mutex)
{
    return(up(mutex));
}

void as_set_nwbuf_dev(void *pBuf, void *dev)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    struct net_device *ndev = (struct net_device *) dev; 

    pSkb->dev = ndev;
}

int as_ndev_type_arphrd_ether(void *dev)
{
    struct net_device *ndev = (struct net_device *) dev; 
    if (ndev->type == ARPHRD_ETHER)
        return 1;
    else
        return 0;
}

void as_set_nwbuf_protocol_ethtypetrans(void *pBuf, void *dev)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    struct net_device *ndev = (struct net_device *) dev; 

    pSkb->protocol = eth_type_trans( pSkb, ndev );
    
}

void as_set_nwbuf_protocol(void *pBuf, short prot)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;

    pSkb->protocol = htons(prot);
}

void as_set_nwbuf_mac_raw(void *pBuf)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;

    pSkb->mac.raw = pSkb->data;
}

void as_set_nwbuf_ip_summed(void *pBuf, int sum)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;

    pSkb->ip_summed = sum;
}

void as_set_skb_data(void *pBuf, char * data)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    pSkb->data = data;
}

void as_arith_skb_data(void *pBuf, int offset, int oper)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    if(oper == AS_OPER_ADD)
        pSkb->data += offset;
    else if (oper == AS_OPER_SUB)
        pSkb->data -= offset;
}

void as_set_skb_tail(void *pBuf)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    pSkb->tail     = pSkb->data + pSkb->len;
}


void as_update_ndev_stats(void *stats, int rx_pack, int tx_pack, int rx_byte, int tx_byte)
{
    struct net_device_stats *ndev_stats = (struct net_device_stats *)stats; 

    ndev_stats->rx_packets = rx_pack;
    ndev_stats->tx_packets = tx_pack;
    ndev_stats->rx_bytes   = rx_byte;
    ndev_stats->tx_bytes   = tx_byte;
}

int as_skb_priority(void *pBuf)
{
    struct sk_buff *pSkb = (struct sk_buff *) pBuf;
    return(pSkb->priority);
}

/* maintaining device structure & statistics ,
 * NOTE: user needs to take care of limiting 
 * the creation of new ndevs after AS_MAX_ETHS 
 * limit.
 */

void as_add_ndev_to_ndev_array( void *dev)
{
    int ii = 0;
    struct net_device *ndev = (struct net_device *) dev; 

    for( ii=0; ii < AS_MAX_ETHS; ii++)
    {
        if ( as_dev_array[ii] == NULL )
        {
            as_dev_array[ii] = ndev;
            break;
        }
    }
}

int as_del_ndev_frm_ndev_array( void *dev)
{
    int ii = 0;
    struct net_device *ndev = (struct net_device *) dev; 

    for( ii=0; ii < AS_MAX_ETHS; ii++)
    {
        if ( ndev == as_dev_array[ii] )
        {
            as_dev_array[ii] = NULL;
            return 0; //SUCCESS
        }
    }
    return -1; //SUCCESS
}

void *as_get_stats_array_for_ndev(void *dev)
{
    int ii = 0;
    struct net_device *ndev = (struct net_device *) dev; 

    for( ii=0; ii < AS_MAX_ETHS; ii++)
    {
        if ( as_dev_array[ii] == ndev )
        {
            return (&as_stats_array[ii]);
        }
    }
    return NULL;
}

/* irq */
void as_local_irq_disable()
{
    local_irq_disable();
}

void as_local_irq_enable()
{
    local_irq_enable();
}


int as_cond_resched()
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
	return(cond_resched());
#else
	cond_resched();
	return 0 ;
#endif
}

void as_delay_with_resched(unsigned int delay)
{
	unsigned int real_delay = delay + jiffies;

    while(time_before(jiffies, real_delay))
	{
		cond_resched();
	}	
}


void as_modify_domain(int d1, int d2)
{
#if LINUX_VERSION_CODE == KERNEL_VERSION(2,6,8)
	modify_domain(d1, d2);
#endif
}

void as_flush_icache_range(int addr1, int addr2)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
	flush_icache_range(addr1, addr2);
#endif
}

/******************** STRING MANIPULATIONS ********************/

unsigned long as_simple_strtoul(const char *cp,char **endp,unsigned int base)
{
	return(simple_strtoul(cp,endp,base));
}

char * as_strcat(char *str1, const char *str2)
{
	return (strcat(str1, str2));	
}

char * hsl_strchr(const char *str,int ch)
{
	return (strchr(str, ch));
}

int hsl_strcmp(const char * str1,const char *str2)
{
	return(strcmp(str1, str2));
}

char * hsl_strcpy(char * str1,const char *str2)
{
	return(strcpy(str1, str2));
}

unsigned long hsl_strcspn(const char *str1,const char *str2)
{
	return (strcspn(str1, str2));
}

unsigned long hsl_strlen(const char *str1)
{
	return (strlen(str1));
}
unsigned long as_memcpy(const char *str1, const char *str2, int len)
{
	return (memcpy(str1, str2, len));
}

char *as_strncat(char *str1, const char *str2, unsigned long size)
{
	return(strncat(str1, str2, size));
}

int hsl_strncmp(const char *str1,const char *str2,unsigned long size)
{
	return (strncmp(str1, str2, size));
}

char * hsl_strncpy(char *str1,const char *str2, unsigned long size)
{
	return(strncpy(str1, str2, size));
}

char * hsl_strrchr(const char *str1,int ch)
{
	return(strrchr(str1, ch));
}

unsigned long hsl_strspn(const char *str1,const char *str2)
{
	return(strspn(str1, str2));
}

char * hsl_strstr(const char *str1,const char *str2)
{
	return(strstr(str1, str2));
}

char * as_strsep(char **str1,const char *str2)
{
	return(strsep(str1, str2));
}

EXPORT_SYMBOL(as_strcat);
EXPORT_SYMBOL(hsl_strchr);
EXPORT_SYMBOL(hsl_strcmp);
EXPORT_SYMBOL(hsl_strcpy);
EXPORT_SYMBOL(hsl_strcspn);
EXPORT_SYMBOL(hsl_strlen);
EXPORT_SYMBOL(as_memcpy);
EXPORT_SYMBOL(as_strncat);
EXPORT_SYMBOL(hsl_strncmp);
EXPORT_SYMBOL(hsl_strncpy);
EXPORT_SYMBOL(hsl_strrchr);
EXPORT_SYMBOL(hsl_strspn);
EXPORT_SYMBOL(hsl_strstr);
EXPORT_SYMBOL(as_simple_strtoul);
EXPORT_SYMBOL(as_strsep);


/******************** STRING MANIPULATIONS ********************/


EXPORT_SYMBOL(as_local_irq_enable);
EXPORT_SYMBOL(as_local_irq_disable);

/* net_device statistics */

EXPORT_SYMBOL(as_get_stats_array_for_ndev);
EXPORT_SYMBOL(as_del_ndev_frm_ndev_array);
EXPORT_SYMBOL(as_add_ndev_to_ndev_array);

/* PROC FS */
EXPORT_SYMBOL(as_update_ndev_stats);
EXPORT_SYMBOL(as_ndev_type_arphrd_ether);
EXPORT_SYMBOL(as_mutex_up);
EXPORT_SYMBOL(as_mutex_down_interruptible);
EXPORT_SYMBOL(as_mutex_init);
EXPORT_SYMBOL(as_ndev_put);
EXPORT_SYMBOL(as_ndev_get_by_name);
EXPORT_SYMBOL(as_unregister_ndev);
EXPORT_SYMBOL(as_register_ndev);
EXPORT_SYMBOL(as_get_ndev_dev_addr);
EXPORT_SYMBOL(as_get_ndev_hard_header_len);
EXPORT_SYMBOL(as_set_ndev_hard_header_len);
EXPORT_SYMBOL(as_set_ndev_init_func);
EXPORT_SYMBOL(as_get_ndev_name);
EXPORT_SYMBOL(as_sizeof_ndev);
EXPORT_SYMBOL(as_init_ether_ptp);
EXPORT_SYMBOL(as_init_ether_ndev);






EXPORT_SYMBOL(as_ndev_iff_promisc);
EXPORT_SYMBOL(as_set_ndev_priv);
EXPORT_SYMBOL(as_get_ndev_priv);
EXPORT_SYMBOL(as_get_ifru_data);
EXPORT_SYMBOL(as_remove_procfs);
EXPORT_SYMBOL(as_create_procfs_entry);
EXPORT_SYMBOL(as_skb_priority);
EXPORT_SYMBOL(as_set_skb_tail);
EXPORT_SYMBOL(as_arith_skb_data);
EXPORT_SYMBOL(as_set_skb_data);
EXPORT_SYMBOL(as_set_nwbuf_ip_summed);
EXPORT_SYMBOL(as_set_nwbuf_mac_raw);
EXPORT_SYMBOL(as_set_nwbuf_protocol);
EXPORT_SYMBOL(as_set_nwbuf_protocol_ethtypetrans);
EXPORT_SYMBOL(as_set_nwbuf_dev);
EXPORT_SYMBOL(as_get_nwbuf_shinfo_size);
EXPORT_SYMBOL(as_get_nwbuf_shinfo);
EXPORT_SYMBOL(as_get_nwbuf_end);
EXPORT_SYMBOL(as_get_nwbuf_head);


EXPORT_SYMBOL(as_isalpha_lx);
EXPORT_SYMBOL(as_toupper_lx);

/* ATM */
EXPORT_SYMBOL(as_get_atm_vcc_dev_data_addr);
EXPORT_SYMBOL(as_set_atmdevops_module);
EXPORT_SYMBOL(as_get_atm_qos_aal);
EXPORT_SYMBOL(as_get_atm_vcc_flags_addr);
EXPORT_SYMBOL(as_atm_push_skb);
EXPORT_SYMBOL(as_atm_pop_skb);
EXPORT_SYMBOL(as_put_nwbuf);
EXPORT_SYMBOL(as_push_nwbuf);
EXPORT_SYMBOL(as_clear_bit);
EXPORT_SYMBOL(as_set_bit);
EXPORT_SYMBOL(as_get_atm_vci);
EXPORT_SYMBOL(as_get_atm_vpi);
EXPORT_SYMBOL(as_get_atm_vcc_dev_data);
EXPORT_SYMBOL(as_get_atm_vcc_linkrate);
EXPORT_SYMBOL(as_get_atm_vcc_ddev_data);
EXPORT_SYMBOL(as_get_atm_qos_max_cdv);
EXPORT_SYMBOL(as_get_atm_qos_pcr);
EXPORT_SYMBOL(as_get_atm_qos_max_pcr);
EXPORT_SYMBOL(as_get_atm_qos_traffic_class);
EXPORT_SYMBOL(as_get_atm_qos);
EXPORT_SYMBOL(as_deregister_atmdev);
EXPORT_SYMBOL(as_set_atm_link_rate);
EXPORT_SYMBOL(as_set_atm_vci_bits);
EXPORT_SYMBOL(as_set_atm_vpi_bits);
EXPORT_SYMBOL(as_get_atm_dev_esi);
EXPORT_SYMBOL(as_init_atm_dev_data);
EXPORT_SYMBOL(as_register_atmdev);
EXPORT_SYMBOL(as_init_atm_ds);

EXPORT_SYMBOL(as_free_irq);
EXPORT_SYMBOL(as_chk_free_skb_tsklet);
EXPORT_SYMBOL(as_init_skb_bf_eth);
EXPORT_SYMBOL(as_skb_netrx);
EXPORT_SYMBOL(as_skb_to_nw_stack);
EXPORT_SYMBOL(as_skb_truesize);
EXPORT_SYMBOL(as_offsetof_nwbuf);
EXPORT_SYMBOL(as_sizeof_nwbuf);
EXPORT_SYMBOL(as_nwbuf_print);
EXPORT_SYMBOL(as_nwbuf_dequeue);
EXPORT_SYMBOL(as_reserve_nwbuf);
EXPORT_SYMBOL(as_alloc_nwbuf);
EXPORT_SYMBOL(as_skb_ptr_diff);
EXPORT_SYMBOL(as_skb_set_len);
EXPORT_SYMBOL(as_skb_qtail);
EXPORT_SYMBOL(as_skb_len);
EXPORT_SYMBOL(as_skb_q_len);
EXPORT_SYMBOL(as_skb_cloned);
EXPORT_SYMBOL(as_dev_kfree_skb);
EXPORT_SYMBOL(as_channel_frm_netdev);
EXPORT_SYMBOL(as_skb_data);

EXPORT_SYMBOL(as_init_skb_qhead);
EXPORT_SYMBOL(as_fs_ops);
EXPORT_SYMBOL(as_request_irq);
EXPORT_SYMBOL(as_kmalloc);
EXPORT_SYMBOL(as_interruptible_sleep_on);
EXPORT_SYMBOL(as_poll_wait);
EXPORT_SYMBOL(as_wake_up_interruptible);
EXPORT_SYMBOL(as_copy_from_user);
EXPORT_SYMBOL(as_copy_to_user);
EXPORT_SYMBOL(as_kfree);
EXPORT_SYMBOL(as_schedule_timeout);
EXPORT_SYMBOL(as_set_current_state);
EXPORT_SYMBOL(as_bh_init);
EXPORT_SYMBOL(as_bh_schedule);
EXPORT_SYMBOL(as_cond_resched);
EXPORT_SYMBOL(as_delay_with_resched);
EXPORT_SYMBOL(as_modify_domain);
EXPORT_SYMBOL(as_flush_icache_range);

