
#define MODULE
#include "deltdiag_mod.h"

#include <linux/cdev.h>
#include <linux/fs.h>
#include <linux/poll.h>
#include <linux/signal.h>

#include "deltdiag.h"
#include "getdeltstatus.h"
 
deltdiag_opened = 0 ;

char status[DELTBITSDN_SZ] ;

static int deltdiag_open(struct inode *inode, struct file *file) ;
static int deltdiag_release(struct inode *inode, struct file *file) ;
static ssize_t deltdiag_read(struct file *filp, char *buffer, size_t length, loff_t *offset) ;
static ssize_t deltdiag_write(struct file *filp, const char *buffer, size_t len, loff_t *offset) ;
static unsigned int deltdiag_ioctl(struct inode *inode, struct file *file,
										unsigned int cmd, unsigned long arg) ;

static struct file_operations deltdiag_fops = {
    .read = deltdiag_read, 
    .write = deltdiag_write,
    .open = deltdiag_open,
    .release = deltdiag_release,
    .ioctl = deltdiag_ioctl
};

static int deltdiag_open(struct inode *inode, struct file *file)
{
	(void*) inode ;
	(void*) file ;

    if (deltdiag_opened) return -EBUSY;
    deltdiag_opened = 1;
    
    return 0;
}

static int deltdiag_release(struct inode *inode, struct file *file)
{
	(void*) inode ;
	(void*) file ;

	deltdiag_opened = 0 ;
    return 0;
}

/* Called when a process, which already opened the dev file, attempts to
   read from it.
*/
static ssize_t deltdiag_read(struct file *filp,
    char *buffer,    /* The buffer to fill with data */
    size_t length,   /* The length of the buffer     */
    loff_t *offset)  /* Our offset in the file       */
{
	(void*) filp ;
	(void*) buffer ;
	(void*) length ;
	(void*) offset ;

	return 0 ;
}

/*  Called when a process writes to dev file */
static ssize_t deltdiag_write(struct file *filp,
                               const char *buffer,
                               size_t len,
                               loff_t *offset)
{
	(void*) filp ;
	(void*) buffer ;
	(void*) len ;
	(void*) offset ;

    return 0 ;
}

static  unsigned int deltdiag_ioctl(struct inode *inode,
									struct file *file,
									unsigned int cmd,
									unsigned long arg)
{
	common_val* v ;
	(void*) inode ;
	(void*) file ;

	if(!getDeltStatus) {
		printk(KERN_ERR "Cnxt modules not loaded yet!!!\n") ;
		return -EINVAL ;
	}
	
	switch(cmd) {
		case DELTSNRDN : {
			deltsnrdn stat ;
			
			copy_from_user(&stat, (deltsnrdn*) arg, sizeof(deltsnrdn)) ;

			if(stat.stat_size != DELTSNRDN_SZ) 
				return -EINVAL ;

			v = stat.val ;
			stat.val = (common_val*) status ;
			getDeltStatus("deltsnrdn", (char*) &stat) ;
			copy_to_user(v, stat.val, DELTSNRDN_SZ) ;
		} break ;

		case DELTQLNDN : {
			deltqlndn stat ;
			
			copy_from_user(&stat, (deltqlndn*) arg, sizeof(deltqlndn)) ;

			if(stat.stat_size != DELTQLNDN_SZ) 
				return -EINVAL ;

			v = stat.val ;
			stat.val = (common_val*) status ;
			getDeltStatus("deltqlndn", (char*) &stat) ;
			copy_to_user(v, stat.val, DELTQLNDN_SZ) ;
		} break ;

		case DELTHLINDNI : {
			delthlindni stat ;
			
			copy_from_user(&stat, (delthlindni*) arg, sizeof(delthlindni)) ;

			if(stat.stat_size != DELTHLINDNI_SZ) 
				return -EINVAL ;

			v = stat.val ;
			stat.val = (common_val*) status ;
			getDeltStatus("delthlindni", (char*) &stat) ;
			copy_to_user(v, stat.val, DELTHLINDNI_SZ) ;
		} break ;

		case DELTHLINDNR : {
			delthlindnr stat ;
			
			copy_from_user(&stat, (delthlindnr*) arg, sizeof(delthlindnr)) ;

			if(stat.stat_size != DELTHLINDNR_SZ) 
				return -EINVAL ;

			v = stat.val ;
			stat.val = (common_val*) status ;
			getDeltStatus("delthlindnr", (char*) &stat) ;
			copy_to_user(v, stat.val, DELTHLINDNR_SZ) ;
		} break ;

		case DELTBITSDN : {
			deltbitsdn stat ;
			
			copy_from_user(&stat, (deltbitsdn*) arg, sizeof(deltbitsdn)) ;

			if(stat.stat_size != DELTBITSDN_SZ) 
				return -EINVAL ;

			v = stat.val ;
			stat.val = (common_val*) status ;
			getDeltStatus("deltbitsdn", (char*) &stat) ;
			copy_to_user(v, stat.val, DELTBITSDN_SZ) ;
		} break ;

		case DELTACTATP : {
			deltactatp stat ;
			
			copy_from_user(&stat, (deltactatp*) arg, sizeof(deltactatp)) ;

			if(stat.stat_size != DELTACTATP_SZ) 
				return -EINVAL ;

			v = stat.val ;
			stat.val = (common_val*) status ;
			getDeltStatus("deltactatp", (char*) &stat) ;
			copy_to_user(v, stat.val, DELTACTATP_SZ) ;
		} break ;

		case DELTACTPSD : {
			deltactpsd stat ;
			
			copy_from_user(&stat, (deltactpsd*) arg, sizeof(deltactpsd)) ;

			if(stat.stat_size != DELTACTPSD_SZ) 
				return -EINVAL ;

			v = stat.val ;
			stat.val = (common_val*) status ;
			getDeltStatus("deltactpsd", (char*) &stat) ;
			copy_to_user(v, stat.val, DELTACTPSD_SZ) ;
		} break ;

		case DELTGAINDN : {
			deltgaindn stat ;
			
			copy_from_user(&stat, (deltgaindn*) arg, sizeof(deltgaindn)) ;

			if(stat.stat_size != DELTGAINDN_SZ) 
				return -EINVAL ;

			v = stat.val ;
			stat.val = (common_val*) status ;
			getDeltStatus("deltgaindn", (char*) &stat) ;
			copy_to_user(v, stat.val, DELTGAINDN_SZ) ;
		} break ;

		case DELTSNRM : {
			deltsnrm stat ;
			
			copy_from_user(&stat, (deltsnrm*) arg, sizeof(deltsnrm)) ;

			if(stat.stat_size != DELTSNRM_SZ) 
				return -EINVAL ;

			v = stat.val ;
			stat.val = (common_val*) status ;
			getDeltStatus("deltsnrm", (char*) &stat) ;
			copy_to_user(v, stat.val, DELTSNRM_SZ) ;
		} break ;

		case DELTATTNDR : {
			deltattndr stat ;
			
			copy_from_user(&stat, (deltattndr*) arg, sizeof(deltattndr)) ;

			if(stat.stat_size != DELTATTNDR_SZ)
				return -EINVAL ;

			v = stat.val ;
			stat.val = (common_val*) status ;
			getDeltStatus("deltattndr", (char*) &stat) ;
			copy_to_user(v, stat.val, DELTATTNDR_SZ) ;
		} break ;

		default :
			return -EINVAL ;
			break ;
	}
    return 0;
}


int __init init_deltdiag(void)
{
    int deltdiag_major = register_chrdev(169, "deltdiag", &deltdiag_fops) ;
    
    if (deltdiag_major < 0) {
         as_printk ("Registering the character device failed with %d\n", 
		 	deltdiag_major) ;
	     return deltdiag_major ;
    }

	return 0;
}

void __exit exit_deltdiag( void)
{
	as_printk("Not Supported yet!!\n");
}

module_init(init_deltdiag);
module_exit(exit_deltdiag);

