#define MODULE
#define EXPORT_SYMTAB

#include "quantum/as_types.h"
#ifdef CNXT_HSL_H 
  #undef strncmp
  #undef snprintf 
  #undef strcat
  #undef strlen
  #undef strncat
  #undef strspn
  #undef strncpy
#endif
#include "linux/module.h"
#include "linux/kernel.h"
#include "linux/slab.h"
#include "atypes.h"
#include "fiq_rel_module.h"

static void fixup_symbol(const char *name, int  offset );
static void do_fiq_relocation(void);
extern U32 fiq_code_start, fiq_code_end;


extern void *atm_fiqstate;
extern void *ports;
extern void *master_ring;
extern long master_ring_length;
extern void *switchActivationQueues;
extern void portSchedulerFiq(void);
extern void ICacheInvalidate(void);

void fixup_symbol( const char *name, int off )
{
  struct module *self = THIS_MODULE;
#ifdef LINUX_OVER_SOLOS
  struct kernel_symbol *sym;
#else
  struct module_symbol *sym;
#endif /* LINUX_OVER_SOLOS */
  int i, n;

#ifdef LINUX_OVER_SOLOS
  n = self->num_syms;
#else
  n = self->nsyms;
#endif /* LINUX_OVER_SOLOS */

  for (i = 0; i < n; i++ )
    {
      sym = self->syms + i;
      if ( !strcmp( sym->name, name ) )
      {
        sym->value = sym->value + off;
        return;
      }
    }
  printk("Symbol %s not found, bozo.\n", name );
}

void do_fiq_relocation()
{
  U32 start = (U32)(&fiq_code_start);
  U32 end = (U32) (&fiq_code_end);
  int offset;
  char *new_fiq;

  /* Move the FIQ handler and associated data into malloced space */
//  as_printk("FIQ code is at 0x%lx and of length %lx\nMagic is 0x%lx\n", start, end - start, *((U32*)start) );
  new_fiq = kmalloc( end - start + 8, GFP_KERNEL );
  memcpy( new_fiq, (char*)start, end - start );
 // offset = (int)((((U32)new_fiq) & 0x0fffffff) - (int)(start & 0x0fffffff) );
  offset = (int)((((U32)new_fiq) ) - (int)(start) );
//  as_printk("Relocated to 0x%p, offset = 0x%x\n", new_fiq, offset );

  /* Now fixup the module exports to point into the relocated code. */
  fixup_symbol( "master_ring", offset );
  fixup_symbol( "master_ring_length", offset );
  fixup_symbol( "atm_fiqstate", offset );
  fixup_symbol( "switchActivationQueues", offset );
  fixup_symbol( "portSchedulerFiq", offset );
  fixup_symbol( "ports", offset );
}


/*
 * Error : For 2.6 kernel module support
 * Warning : static declaration for `init_module' follows non-static
 */
static int __init fiq_init_module(void) 
{
  do_fiq_relocation();  
  return 0;
}
/*
 * Error : For 2.6 kernel module support
 */
static void __exit fiq_cleanup_module(void)
{

}

/*
 * Error : For 2.6 kernel module support
 */
EXPORT_SYMBOL( atm_fiqstate );
EXPORT_SYMBOL( ports );
EXPORT_SYMBOL( master_ring );
EXPORT_SYMBOL( master_ring_length );
EXPORT_SYMBOL( switchActivationQueues );
EXPORT_SYMBOL( portSchedulerFiq );
EXPORT_SYMBOL(ICacheInvalidate);

module_init(fiq_init_module);
module_exit(fiq_cleanup_module);
