/* ###########################################################################
# (c) Copyright Conexant Systems, Inc 2001, 2006
#
# Conexant Systems, Inc Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc ("Conexant"). It may only be read,
# used, copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Conexant and then subject to
# the terms of that confidentiality agreement and any other applicable
# agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a confidentiality agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/

#define MODULE

#include "linux/kernel.h"
#include "linux/module.h"
#include <linux/crypto.h>

/* Module magic */
#include "hsl_mod_module.h"

#include "cnxt_hsl.h"
#include "lx_shim_fp.h"
#include "lx_shim_defines.h"
#include "quantum_shim.h"

#include "quantum/xs_startup.h"
#include "quantum/hs_interface.h"
#include "quantum/hs_message.h"
#include "quantum/hs_messagehdr.h"
#include "quantum/hs_pool.h"
#include "quantum/hs_task.h"
#include "quantum/hs_semaphore.h"
#include "quantum/hs_timer.h"
#include "quantum/hs_mem.h"
#include "led_if.h"
#define MODULE_LOAD_DELAY 1



/* Export the functions used by led char dev, ledif.c will be compiled in
 * open-src/HSL/linux/.
 */
 
EXPORT_SYMBOL(led_GlobalEnable);
EXPORT_SYMBOL(led_GlobalDisable);
EXPORT_SYMBOL(led_Enable);
EXPORT_SYMBOL(led_Disable);
EXPORT_SYMBOL(led_On);
EXPORT_SYMBOL(led_Off);
EXPORT_SYMBOL(led_Trigger);
EXPORT_SYMBOL(led_Message);
EXPORT_SYMBOL(led_Polarity);
EXPORT_SYMBOL(led_initialise);

//#ifdef TURBO_INCLUDE_WHIP_TCP
#include "gswhip.h"

EXPORT_SYMBOL(whipTxWaitQ);
EXPORT_SYMBOL(whipRxWaitQ);
EXPORT_SYMBOL(char_dev_tx_buf);
EXPORT_SYMBOL(char_dev_rx_buf);
EXPORT_SYMBOL(char_dev_tx_buf_len);
EXPORT_SYMBOL(char_dev_tx_buf_free);
EXPORT_SYMBOL(char_dev_rx_buf_len);
EXPORT_SYMBOL(char_dev_rx_buf_free);
EXPORT_SYMBOL(turboTxBufQHead);
EXPORT_SYMBOL(turboTxBufQTail);
EXPORT_SYMBOL(whip_mod_not_initialized);
//#endif

/* For DELT */

#include "getdeltstatus.h"

EXPORT_SYMBOL(getDeltStatus) ;

typedef int (*lx_kthread_entry)(void *);
#ifdef CNXT_CTL_DRIVER
/*
*	Conexant CTL driver
*/
extern int (*DSLattrib)(char *string,char *ret);
int bif_portCmds(char *bun_string, char *ret);
#endif

static int __init hsl_init_module(void)
{
  tASErr err = 0;
  int pid;

  tHSITaskEntryDefinition *pEntry;
  int done = 0;
  unsigned int delay;
  char mac[5],i;

  mac[1]= *as_psystem_serial_high & 0xff;
  mac[0]=(*(as_psystem_serial_high) & (0xff<<8)) >> 8;
  for (i=5; i > 1; i-- )
    mac[i]=(*(as_psystem_serial_low) & (0xff << 8*(5-i))) >> (8*(5-i));
  as_printk("BASE MAC ADDRESS %02x:%02x:%02x:%02x:%02x:%02x\n",mac[0],mac[1],mac[2]
          ,mac[3],mac[4],mac[5]);
  /* Initialise the shim layer function pointers defined in lx_shim_fp.h */
  hsl_ObjectNew                         = shim_ObjectNew;
  hsl_ObjectNewAlloc            = shim_ObjectNewAlloc;
  hsl_ObjectListSet             = shim_ObjectListSet;
  hsl_ObjectDelete                      = shim_ObjectDelete;
  hsl_ObjectEnable                      = shim_ObjectEnable;
  hsl_ObjectSetTypeHandle       = shim_ObjectSetTypeHandle;
  hsl_ObjectHandle                      = shim_ObjectHandle;
  hsl_ObjectFromAlloc           = shim_ObjectFromAlloc;
  hsl_ObjectOpen                        = shim_ObjectOpen;
  hsl_ObjectClose                       = shim_ObjectClose;
  hsl_ObjectMkdir                       = shim_ObjectMkdir;
  hsl_ObjectTypeHandle          = shim_ObjectTypeHandle;
  hsl_ObjectIsEnabled           = shim_ObjectIsEnabled;
  hsl_ObjectTypeRegisterSimple = shim_ObjectTypeRegisterSimple;
  hsl_ObjectTypeRegisterDerived = shim_ObjectTypeRegisterDerived;
  hsl_ObjectTypeRegisterBase    = shim_ObjectTypeRegisterBase;
  hsl_UartWriteByte                     = shim_UartWriteByte;
  hsl_ObjectTypeRegisterCast    = shim_ObjectTypeRegisterCast;
  hsl_ObjectTransmute                   = shim_ObjectTransmute;
  hsl_InterfaceInit                             = shim_InterfaceInit;
  hsl_DebugRegisterCommands             = shim_DebugRegisterCommands;
  hsl_StrSkip                                   = shim_StrSkip;
  hsl_hw_TimerInterruptEnable   = shim_hw_TimerInterruptEnable;
  hsl_hw_startup                                = shim_hw_startup;
  hsl_hw_systemFigureOutChipType= shim_hw_systemFigureOutChipType;
  hsl_hw_systemBenchmarkCPU             = shim_hw_systemBenchmarkCPU;

  hsl_TaskStart                                 = shim_TaskStart;
  hsl_MemRegionNew                              = shim_MemRegionNew;
  hsl_MemRegionClose                    = shim_MemRegionClose;
  hsl_InterfaceOpen                             = shim_InterfaceOpen;
  hsl_MemSync                                   = shim_MemSync;
  hsl_MessageFree                               = shim_MessageFree;
  hsl_MessageReply                              = shim_MessageReply;
  hsl_Debug                                             = shim_Debug;

#ifdef CNXT_CTL_DRIVER
/*
*	Conexant CTL driver function pointer initialisation
*/
DSLattrib = bif_portCmds;
#endif
  /* xs_StartUp will call StartScheduler, which is not allowed
     to return and will thus call sys_exit. Hence it seems wise
     to run that in its own thread. */
  pid = as_kthread( (lx_kthread_entry)xs_Startup, NULL, 0 );
  //pid = kernel_thread( (lx_kthread_entry)xs_Startup, NULL, 0 );
  if ( pid < 0 )
  if ( pid < 0 )
    {
    as_printk("Failed to start kernel thread to run xs_Startup: %d\n", pid );
    err = pid;
    }
    as_printk("\nStarting Conexant drivers\n");
	while (!xs_SystemParallelism())
 	{
  		as_printk("\r.");
  		as_cond_resched();
 	}

#ifdef _LINUX_GALLUS_USB
 usb_host_init_module();
#ifndef BSP_FAST_BOOT
 as_delay_with_resched(AS_HZ *MODULE_LOAD_DELAY);
 as_printk(".");
#endif /* BSP_FAST_BOOT */
#endif

#if (defined (LINUX_GALLUS_DSL)) || (defined (LINUX_GALLUS_CRYPTO))
 crypto_init_module();
#ifndef BSP_FAST_BOOT
 as_delay_with_resched(AS_HZ *MODULE_LOAD_DELAY);
 as_printk(".");
#endif /* BSP_FAST_BOOT */
#endif
#ifdef LX_GALLUS_ARM32
 fiqc_init_module();
#ifndef BSP_FAST_BOOT
 as_delay_with_resched(AS_HZ *MODULE_LOAD_DELAY);
 as_printk(".");
#endif /* BSP_FAST_BOOT */
#endif
 task_glue_init_module();
#ifndef BSP_FAST_BOOT
 as_delay_with_resched(AS_HZ *MODULE_LOAD_DELAY);
 as_printk(".");
#endif /* BSP_FAST_BOOT */
#ifdef LINUX_GALLUS_CRYPTO
 init_solos_crypto_API();
#ifndef BSP_FAST_BOOT
 as_delay_with_resched(AS_HZ *MODULE_LOAD_DELAY);
 as_printk(".");
#endif /* BSP_FAST_BOOT */
#endif
 as_printk("\nConexant drivers started\n");
 return err;
}

int mac_GlobalClaim(const char *client, U8 * pMac, int *pIndex)
{

 return mac_GlobalClaimSpecific(*pIndex,client,pMac);


}
int mac_GlobalPrimaryRegister(const char *client, U8 * pMac,
                              int *pHandle)
{
  return (mac_GlobalClaimSpecific(0,"mac_GlobalPrimaryRegister",pMac));
}
int mac_GlobalPrimaryGetAddress(U8 *pMac)
{

  return (mac_GlobalClaimSpecific(0,"mac_GlobalPrimaryGetAddress",pMac));

}
int mac_GlobalClaimSpecific(int index, const char *client, U8 * pMac)
{
 static DECLARE_MUTEX(lx_mac);
 int   temp=0;
 char i=0;
 down(&lx_mac);
 if ( 0xffffffff - *(as_psystem_serial_low) < index )
 {
  as_printk("mac addresss overflow\n");
  (*as_psystem_serial_high)++;
 }
  temp = *(as_psystem_serial_low) + index;
  for (i=5; i > 1; i-- )
    pMac[i]=(temp & (0xff << (8*(5-i)))) >> (8*(5-i));

    pMac[1]= *(as_psystem_serial_high) & 0xff;
    pMac[0]=(*(as_psystem_serial_high) & (0xff<<8)) >> 8;
 up(&lx_mac);
 return 0;
}

static void __exit hsl_exit_module(void)
{
}

EXPORT_SYMBOL(mac_GlobalClaimSpecific);
EXPORT_SYMBOL(mac_GlobalPrimaryGetAddress);
EXPORT_SYMBOL(mac_GlobalPrimaryRegister);
EXPORT_SYMBOL(mac_GlobalClaim);

EXPORT_SYMBOL(arm9_CleanDataRegion);
EXPORT_SYMBOL(arm9_InvalidateDataRegion);


module_init(hsl_init_module);
module_exit(hsl_exit_module);
