/**############################################################################
 * # (c) Copyright Conexant Systems, Inc. 2006
 * #
 * # Confidential.  All rights reserved.  This publication and its contents are
 * # proprietary to Conexant Systems, Inc.  No part of this publication may be
 * # reproduced in any form or by any means without the written permission of
 * # Conexant Systems, Inc.
 * #
 * #
 * # If you are unsure, or to report violations, please contact
 * # support@Conexant.com
 * #
 * ############################################################################
 */
/** @file    ledif.h
 *  @brief   led library interface header
 *  @ingroup led
 *
 *  Linux header for led library interface 
 */
#ifndef BFIF_LED_H
#define BFIF_LED_H


/* Major number, Equate to zero for dynamic selection */
#define LEDIF_MAJOR   250  


#define LEDIF_CHAR_NAME "led"
/* This #def is also present in */
#define LED_PATTERN_MAX_LEN 32
/* #define LEDIF_DEVICE_NODE "/dev/led" */

/* Structure for passing arguments, passed with ioctl command */ 
struct sLedIfArg {
    int  led;                    /* Numeric value for LED on which command will operate */
    char pattern[LED_PATTERN_MAX_LEN];            /* Pattern string on led */
    int  activehigh;             /* Boolean to Polarity level */
};



/*  Led interface control commands */

#define BFIF_LED_IOC_TYPE           LEDIF_MAJOR 

#define BFIF_LED_GLOBAL_ENABLE      _IO(BFIF_LED_IOC_TYPE, 1)
        /* Enables all LEDs on board, returns Success/Failure int value */

#define BFIF_LED_GLOBAL_DISABLE     _IO(BFIF_LED_IOC_TYPE, 2)
        /* Disables all LEDs on board, returns Success/Failure int value */

#define BFIF_LED_ENABLE             _IOW(BFIF_LED_IOC_TYPE, 3, int)
        /* Enable LED specified as no in argument, returns Success/Failure int value */

#define BFIF_LED_DISABLE            _IOW(BFIF_LED_IOC_TYPE, 4, int)
        /* Disable LED specified as no in argument, returns Success/Failure int value */

#define BFIF_LED_ON                 _IOW(BFIF_LED_IOC_TYPE, 5, int)
        /* Turns on LED specified as no in argument, returns Success/Failure int value */

#define BFIF_LED_OFF                _IOW(BFIF_LED_IOC_TYPE, 6, int)
        /* Turns off LED specified as no in argument, returns Success/Failure int value */

#define BFIF_LED_TRIGGER            _IOW(BFIF_LED_IOC_TYPE, 7, int)
        /* Trigger LED specified as no in argument with pattern argument, 
        returns Success/Failure int value */

#define BFIF_LED_MESSAGE            _IOW(BFIF_LED_IOC_TYPE, 8, int)
        /* Message LED specified as no in argument with pattern argument, 
        returns Success/Failure int value */

#define BFIF_LED_POLARITY           _IOW(BFIF_LED_IOC_TYPE, 9, int)
        /* Polarity of LED specified as no in argument with level argument, 
        returns Success/Failure int value */

#define BFIF_LED_INITIALIZE         _IO(BFIF_LED_IOC_TYPE, 10)
        /* Initialize all LEDs */

#define BFIF_LED_IOC_MAXNR  10

#endif /* BFIF_LED_H  */

