/* ###########################################################################
# (c) Copyright Conexant Systems, Inc. 2006
#
# Conexant Systems, Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc. and its subsidiaries (collectively 
# "Conexant"). It may only be read, used, copied, adapted, modified or 
# otherwise dealt with by you if you have entered into a license agreement 
# with Conexant and then subject to the terms of that agreement and any other
# applicable confidentiality agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a license agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/

#ifndef _LX_ERR_H 
#define __LX_ERR_H

/** @file       lx_err.h
 *  @brief      Error code definitions.
 *  @ingroup    util
 *
 * The kHSLExxx error codes are possible return values from any of the
 * HSL module functions.
 *
 * The values (and names) are chosen to map to the original ATMOS error
 * codes.  Unfortunately, these need to use a different name prefix to
 * avoid potential namespace clashes when used under a third party OS
 * platform.
 *
 * When checking error status returns, only \em kHSLEOK (and its
 * synonym \em kHSLESUCCESS) is normally used to indicate successful
 * completion.  Any other value will normally indicate an error
 * condition, and hopefully the value returned may give a reasonable
 * clue as to what went wrong.
 *
 * Use caution when testing explicit error codes other than \em kHSLEOK.
 * The Quantum APIs are mapped to a large number of different host OS
 * platforms, each of which may generate subtly different error
 * conditions.  Application code may only reply on the explicit error
 * codes documented with the API being called, and should treat any
 * other possible return values as "other errors".
 *
 *  @note
 *      This file may be included by assembler code.  Use only valid C
 *      pre-processor syntax in this header.
 *
 *  @todo
 *      There needs to be an error translation API, to map host dependent
 *      error codes to suitable Quantum error codes.  Such an API will
 *      necessarily be platform dependent (and used only for platform
 *      specific interface code).
 */


#define kHSLEOK          0
/**< EOK          Successful                  */
#define kHSLESUCCESS     kHSLEOK
/**< ESUCCESS     Successful                  */
#define kHSLEPERM        1
/**< EPERM        Not Owner                   */
#define kHSLENOENT       2
/**< ENOENT       No such file or directory   */
#define kHSLESRCH        3
/**< ESRCH        No such process             */
#define kHSLEINTR        4
/**< EINTR        Interrupted System call     */
#define kHSLEIO          5
/**< EIO          I/O Error                   */
#define kHSLENXIO        6
/**< ENXIO        No such device or address   */
#define kHSLE2BIG        7
/**< E2BIG        Arg list too long           */
#define kHSLENOEXEC      8
/**< ENOEXEC      Exec format error           */
#define kHSLEBADF        9
/**< EBADF        Bad file number             */
#define kHSLECHILD       10
/**< ECHILD       No children                 */
#define kHSLEAGAIN       11
/**< EAGAIN       No more processes           */
#define kHSLENOMEM       12
/**< ENOMEM       Not enough core             */
#define kHSLEACCES       13
/**< EACCES       Permission denied           */
#define kHSLEFAULT       14
/**< EFAULT       Bad address                 */
#define kHSLENOTBLK      15
/**< ENOTBLK      Block device required       */
#define kHSLEBUSY        16
/**< EBUSY        Mount device busy           */
#define kHSLEEXIST       17
/**< EEXIST       File exists                 */
#define kHSLEXDEV        18
/**< EXDEV        Cross-device link           */
#define kHSLENODEV       19
/**< ENODEV       No such device              */
#define kHSLENOTDIR      20
/**< ENOTDIR      Not a directory             */
#define kHSLEISDIR       21
/**< EISDIR       Is a directory              */
#define kHSLEINVAL       22
/**< EINVAL       Invalid argument            */
#define kHSLENFILE       23
/**< ENFILE       File table overflow         */
#define kHSLEMFILE       24
/**< EMFILE       Too many open files         */
#define kHSLENOTTY       25
/**< ENOTTY       Not a typewriter            */
#define kHSLETXTBSY      26
/**< ETXTBSY      Text file busy              */
#define kHSLEFBIG        27
/**< EFBIG        File too large              */
#define kHSLENOSPC       28
/**< ENOSPC       No space left on device     */
#define kHSLESPIPE       29
/**< ESPIPE       Illegal seek                */
#define kHSLEROFS        30
/**< EROFS        Read-only file system       */
#define kHSLEMLINK       31
/**< EMLINK       Too many links              */
#define kHSLEPIPE        32
/**< EPIPE        Broken pipe                 */

/* math.h errors (&R) B4  */

#define kHSLEDOM         33
/**< EDOM         Argument too large          */
#define kHSLERANGE       34
/**< ERANGE       Result too large            */

/* non-blocking and interrupt i/o  */

#define kHSLEWOULDBLOCK  35
/**< EWOULDBLOCK  Operation would block       */
#define kHSLEINPROGRESS  36
/**< EINPROGRESS  Operation now in progress   */
#define kHSLEALREADY     37
/**< EALREADY     Operation in progress       */

/* Network errors: these use the 'socket' codes */

#define kHSLENOTSOCK     38
/**< ENOTSOCK     Socket op. on non-socket    */
#define kHSLEDESTADDRREQ 39
/**< EDESTADDRREQ Destination addr required   */
#define kHSLEMSGSIZE     40
/**< EMSGSIZE     Message too long            */
#define kHSLEPROTOTYPE   41
/**< EPROTOTYPE   Protocol wrong for socket   */
#define kHSLENOPROTOOPT  42
/**< ENOPROTOOPT  Protocol not available      */
#define kHSLEPROTONOSUPPORT 43
/**< EPROTONOSUPPORT Protocol not supported      */
#define kHSLESOCKTNOSUPPORT 44
/**< ESOCKTNOSUPPORT Socket type not supported   */
#define kHSLEOPNOTSUPP   45
/**< EOPNOTSUPP   Operation not supported     */
#define kHSLEPFNOSUPPORT 46
/**< EPFNOSUPPORT Prot. family not supported  */
#define kHSLEAFNOSUPPORT 47
/**< EAFNOSUPPORT Addr. family not supported  */
#define kHSLEADDRINUSE   48
/**< EADDRINUSE   Address already in use      */
#define kHSLEADDRNOTAVAIL 49
/**< EADDRNOTAVAIL Can not assign address      */

#define kHSLENETDOWN     50
/**< ENETDOWN     Network is down             */
#define kHSLENETUNREACH  51
/**< ENETUNREACH  Network is unreachable      */
#define kHSLENETRESET    52
/**< ENETRESET    Network reset               */
#define kHSLECONNABORTED 53
/**< ECONNABORTED Software aborted connection */
#define kHSLECONNRESET   54
/**< ECONNRESET   Connection reset by peer    */
#define kHSLENOBUFS      55
/**< ENOBUFS      No buffer space available   */
#define kHSLEISCONN      56
/**< EISCONN      Socket is already connected */
#define kHSLENOTCONN     57
/**< ENOTCONN     Socket is not connected     */
#define kHSLESHUTDOWN    58
/**< ESHUTDOWN    Can not send after shutdown */
#define kHSLETOOMANYREFS 59
/**< ETOOMANYREFS Too many refs: can not splice */
#define kHSLETIMEDOUT    60
/**< ETIMEDOUT    Connection timed out        */
#define kHSLECONNREFUSED 61
/**< ECONNREFUSED Connection refused          */
#define kHSLEHOSTDOWN    62
/**< EHOSTDOWN    Host is down                */
#define kHSLEHOSTUNREACH 63
/**< EHOSTUNREACH No route to host            */

#define kHSLELOOP        64
/**< ELOOP        Too many levels of symlinks */
#define kHSLENAMETOOLONG 65
/**< ENAMETOOLONG File name too long          */
#define kHSLENOTEMPTY    66
/**< ENOTEMPTY    Directory not empty         */

/* quotas  */

#define kHSLEPROCLIM     67
/**< EPROCLIM     Too many processes          */
#define kHSLEUSERS       68
/**< EUSERS       Too many users              */
#define kHSLEDQUOT       69
/**< EDQUOT       Disc quota exceeded         */

/* NFS error codes  */

#define kHSLESTALE       70
/**< ESTALE       Stale file handle           */
#define kHSLEREMOTE      71
/**< EREMOTE      File is remote              */
 
/* ATMOS local codes  */

#define kHSLEBADQ        80
/**< EBADQ        Bad Message Queue Id        */
#define kHSLEBADTHREAD   81
/**< EBADTHREAD   Bad Thread Context          */
#define kHSLEINAPPOP     82
/**< EINAPPOP     Inappropriate Operation     */
#define kHSLENOTFOUND    83
/**< ENOTFOUND    Not found                   */
#define kHSLESYNTAX      84
/**< ESYNTAX      Syntax Error                */

/* DNS client codes  */

#define kHSLEHOSTNOTFOUND  85
/**< EHOSTNOTFOUND  specified host is unknown   */
#define kHSLENOADDRESS     86
/**< ENOADDRESS     requested name is valid but does not have an IP address */
#define kHSLENODATA        kHSLENOADDRESS   
/**< ENODATA        @todo document it           */
#define kHSLENORECOVERY    87
/**< ENORECOVERY    non-recoverable error       */
#define kHSLETRYAGAIN      88
/**< ETRYAGAIN      Try again later             */
#define kHSLENONSADDRESS   89
/**< ENONSADDRESS   no name server addresses provided, config error      */
#define kHSLENOSEARCHLIST  90
/**< ENOSEARCHLIST  no search list when using relative host name         */

/* Crypto Library defines */

#define kHSLEVERIFY      91
/**< EVERIFY      Verification failure         */
#define kHSLESPSETUP     92
/**< ESPSETUP     Service Provider Setup fail  */
#define kHSLESPPROC      93
/**< ESPPROC      Service Provider Process fail*/

/* Doomed status */

#define kHSLEDOOMED      94
/**< EDOOMED      Task or system is, well, doomed*/

/* Error name specified by ISO/IEC 9945 and needed by fileGEM. */
#define kHSLENOSYS       95
/**< ENOSYS       Function not implemented     */
#define kHSLEMORE        96
/**< EMORE        DNS Client err code          */

#define kHSLEMAXNUMBER   96
/**< EMAXNUMBER   Highest error number         */


/** An pointer value that is invalid for any pointer type */
#define kHSLNULL         (0)

/** @file       derived from xs_types.h
 *
 * This section defines the portable types for the Helium
 * native platform.
 *
 */

/* Object handles.
 */
//typedef struct sXSObject*       tXSObject;           /**< Object handle (opaque type) */

#define kHSLObjectInvalid        (0)       /**< The invalid object handle */

/* Memory region handles.
 */

#define kHSLMemRegionInvalid     (0)    /**< The invalid memory region handle */

/* Memory allocator handles.
 */

#define kHSLMemAllocatorInvalid  (0) /**< The invalid memory allocator handle */

/* Log output handles.
 */

#define kHSLLogInvalid           (0)          /**< The invalid log output handle */

/* Command output handles.
 */

#define kHSLCommandInvalid       (0)      /**< The invalid command output handle */


/* Interface handles (transmit only).
 */

#define kHSLInterfaceInvalid (0)        /**< The invalid interrupt handle */

/* Interrupt handles
 */

#define kHSLInterruptInvalid (0)        /**< The invalid interrupt handle */

/* Interrupt controller handles
 */

#define kHSLIrqControllerInvalid (0) /**< The invalid interrupt controller handle */

/* Variables
 */

#define kHSLVariableInvalid (0)          /**< The invalid stream handle */

#endif  /* _LX_ERR_H */
