/* ###########################################################################
# (c) Copyright Conexant Systems, Inc. 2004
# (c) Copyright GlobespanVirata, Inc., 2001-2004
#
# Conexant Systems, Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc. and its subsidiaries (collectively 
# "Conexant"). It may only be read, used, copied, adapted, modified or 
# otherwise dealt with by you if you have entered into a license agreement 
# with Conexant and then subject to the terms of that agreement and any other
# applicable confidentiality agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a license agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/

/** @file   lx_interrupt.c
 *  @brief  Host specific interrupt code.
 */

#ifdef QUANTUM_OLD_INTERRUPTS
#ifndef QUANTUM_LEAN
#include "lx_shim_types.h"
#endif
#endif
#include "lx_shim_defines.h"
#include "lx_shim_fp.h"

#include "config.h"

#include "quantum/hs_uart.h"
#include "quantum/hs_mem.h"
#include "quantum/hs_interface.h"
#include "quantum/hs_interrupt.h"

#ifdef LINUX_OVER_SOLOS
#include "bf.h"
#else
#include "hf.h"
#endif /* LINUX_OVER_SOLOS */

#include "lx_err.h"
#include "lx_interrupt.h"
#include "hw_interrupt.h"

#include "linux/sched.h"
#include "asm/signal.h"
#include "asm/hardirq.h"

/* ---------------------------------------------------------------------------
 *
 *      ARM Interrupt control
 *
 * ---------------------------------------------------------------------------
 */


#include "asm/system.h" /* Linux header for processor flags */


#ifdef LINUX_OVER_SOLOS
#include "linux/interrupt.h"
#endif /* LINUX_OVER_SOLOS */

/* ---------------------------------------------------------------------------
 *
 *      Attribute Handling - Generic Interrupt Objects
 *
 * ---------------------------------------------------------------------------
 */


#ifdef QUANTUM_OLD_INTERRUPTS
#ifndef QUANTUM_LEAN

#define DEBUG_BADIRQ

static void lx_IRQHandler( int irq, void *context, struct pt_regs * regs)
{
  tHSInterface iface = (tHSInterface)context;
  tHSMessage msg = hs_MessageGet(iface);
#ifdef DEBUG_BADIRQ
  /* In non-debug builds, we'll crash in hs_MessageReply(), so there's
   * no risk of things going wrong very far downstream. Hence leave the 
   * test out of non-debug builds. 
   */
  if ( msg == 0 )
    panic("lx_IRQHandler: no message on interface!\n");
#endif
  hs_MessageReply( msg );
}

static void lx_IRQSetInterface( int source, tHSInterface iface, const char *name )
{
  int err;
  err = request_irq( source, lx_IRQHandler, SA_SHIRQ, name , iface );
  if ( err )
    as_printk("lx_IRQSetInterface: Failed to claim interrupt source %d\n", source ); 
  else
      irq_cookies[source] = iface;
}

/* The following function is invoked for an attribute-set operation
 * on an interrupt object.  We configure the interface for the given
 * source (assuming that it is valid).
 */
static int lx_interruptSetSource(void *object, const int* pInput)
{
    tHSInterrupt interrupt = (tHSInterrupt) hsl_ObjectHandle(object);
    int source = *pInput;
    enum eHFInterfaceProtocol protocol;
    U32 type;
    int err;
    const char *name = "quantum";

    /* Fill out some hardware-specific information. */
    hw_interruptSourceInfo( source, &type, &protocol, &name, interrupt );

    if (kHFInterfaceProtocolInvalid == protocol)
    {
        err = kHSLEINVAL;
    }
    else
    {
        err = lx_InterfaceConfigure(&interrupt->interface, protocol);
        if (kHSLEOK == err)
        {
            err = hsl_ObjectTransmute(object, type);
        }
    }

    if (kHSLEOK == err)
    {
        interrupt->source = source;
        
        /* Plug in the interface to the vector table */
        lx_IRQSetInterface( source, &(interrupt->interface), name );
    }
    
    return err;
}

/* Get handler for the source.
 */
static int lx_interruptGetSource(void * object, int* pOutput)
{
    tHSInterrupt interrupt = (tHSInterrupt) hsl_ObjectHandle(object);
    *pOutput = interrupt->source;
    return kHSLEOK;
}


/** Generic Interrupt Object Attributes.
 */
const tHSlObjectKey lxInterruptKeys[] =
{
    HSL_OBJECT_KEY_ENUM("source", lx_interruptSetSource, lx_interruptGetSource, hwInterruptSourceEnum /* From hw_interrupt.h */),
    HSL_OBJECT_KEY_END
};

 
#endif
#endif /* QUANTUM_OLD_INTERRUPTS */





/* ---------------------------------------------------------------------------
 *
 *      Exported API
 *
 * ---------------------------------------------------------------------------
 */



#ifdef QUANTUM_OLD_INTERRUPTS

/**
 * Set the object representing an sem if unset.
 *
 *  @param  txif            The interface handle.
 *  @param  obj             The object this interface represents
 *  @return                 An error code
 *
 * This function can be used to set a the object that an interface
 * repressents if it is previously uninitialized.
 *
 * Possible error codes include
 *   - kHSLEINVAL:  interface handle or object handle are invalid
 *   - kHSLEEXIST:  object handle has already been set
 *   - kHSLEOK      update was successful
 */
static int hs_InterruptSetObject(tHSInterrupt interrupt, void *obj)
{   return hs_InterfaceSetObject(&interrupt->interface, obj);
}




/**
 * Create a new interrupt object.
 *
 *  @param      pInterrupt      Used to return the interrupt handle.
 *  @param      pszName         Object name.
 *  @param      pszDefaults     Property list.
 *  @param      owner           Task creating the object.
 *  @return                     An error status code.
 */
int hs_InterruptNew(tASInterrupt* pInterrupt, const char* pszName, const char* pszDefaults, tHSTask owner)
{
    tHSInterrupt interrupt;
    void * object;
    int err;
        
    /* Create a new interrupt object.
     */
    interrupt = (tHSInterrupt) hs_MemAlloc(sizeof *interrupt, owner);
    if (0 == interrupt)
    {
        as_printk("%s: Insuffcient memory for interrupt object %s\n", __FUNCTION__, pszName);
        interrupt = kHSInterruptInvalid;
        err = kHSLENOMEM;
    }
    else
    {
        /* Clear the structure down and put it in to a safe state.
         */
        hsl_BlockZero(interrupt, sizeof *interrupt);

        lx_InterfaceInit(&interrupt->interface, kHSLObjectInvalid,
                         kHFInterfaceProtocolInvalid);

        interrupt->source = kHFInterruptSourceInvalid;

        /* Create the object for this interrupt, implicitly
         * applying any configuration.  Interrupt start life
         * as the generic form, but are then transmogrified
         * in to a source specific flavour later on.
         */
        err = hsl_ObjectNew(
            &object,
            pszName,
            pszDefaults,
            interrupt,
            kHSLTypeInterrupt,
            owner);

        if (kHSLEOK != err)
        {
#ifndef QUANTUM_TESTS
            as_printk("%s: error %d creating interrupt %s : %s\n", __FUNCTION__, err, pszName, pszDefaults);
#endif
            hs_MemFree(interrupt, owner);
            interrupt = kHSInterruptInvalid;
        }
        else if (kHFInterruptSourceInvalid == interrupt->source)
        {
#ifndef QUANTUM_TESTS
            as_printk("%s: asked to created interrupt without source %s : %s\n", __FUNCTION__, pszName, pszDefaults);
#endif
            err = kHSLEINVAL;
            xs_ObjectDelete(object, owner);
            hs_MemFree(interrupt, owner);
            interrupt = kHSInterruptInvalid;
        }
        else
        {
            /* Save back reference to the object type for delete operations */
            (void)hs_InterruptSetObject(interrupt, object);
            xs_ObjectEnable(object);
        }
    }
    
    *pInterrupt = interrupt;
    return err;
}



/**
 * Delete an interrupt object.
 *
 *  @param  interrupt   The interrupt object.
 *  @param  caller      The task requestion deletion.
 *  @return             An error status code.
 */
int hs_InterruptDelete(tHSInterrupt interrupt, tHSTask caller)
{
    if (0 == interrupt->interface.control)
    {
        int err = xs_ObjectDelete(interrupt->object, caller);
        if (kHSLEOK == err)  hs_MemFree(interrupt, caller);
        return err;
    }
    else
    {
        /* The interface is in use - it is not safe to delete yet.
         * Note that this will in practise probably mean that the interrupt
         * object is never deleted.  This doesn't matter much since it will
         * only happen if there is an actual coding/configuration error.
         * The memory leaked is very small.
         */
        return kHSLEBUSY;
    }
}


#endif /* QUANTUM_OLD_INTERRUPTS */


/**
 * Initialise the interrupt library.
 *
 *  @return     An error status code.
 */
int hs_InterruptLibraryInitialise(void)
{
 return  kHSLEOK;
}


