/* ###########################################################################
# (c) Copyright Virata Limited 1998
#
# Virata Limited Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Virata Limited ("Virata").  It may only be read, used,
# copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Virata and then subject to the
# terms of that confidentiality agreement and any other applicable agreement
# between you and Virata.  If you are in any doubt as to whether you are
# entitled to access, read, use, copy, adapt, modify or otherwise deal with
# the Software or whether you are entitled to disclose the Software to any
# other person you should contact Virata.  If you have not entered into a
# confidentiality agreement with Virata granting access to this Software you
# should forthwith return all media, copies and printed listings containing
# the Software to Virata.
#
# Virata reserves the right to take legal action against you should you breach
# the above provisions.
#
# If you are unsure, or to report violations, please contact
# support@virata.com
# ##########################################################################*/


#ifndef _QUANTUM_HF_MEM_H
#define _QUANTUM_HF_MEM_H

/** @file       hf_mem.h
 *  @brief      Helium (ARM) specific internal memory APIs definitions.
 *
 * This file exports the internal memory API functions for use
 * by the HS and LS layers.  These implement memory allocation in a
 * fashion that is compatible with existing ATMOS-1 code.
 *
 *  @note
 *      Ultimately, it will be necessary to retire all ATMOS-1
 *      code, allowing some half decent memory allocation
 *      scheme to be used.
 */
#include "config.h"
//#include "quantum/as_types.h"
#include "quantum/hs_task.h"

#ifdef LINUX_OVER_SOLOS
#include "bf.h"
#else
#include "cf.h"
#endif /* LINUX_OVER_SOLOS */

extern void* hf_MemPhysicalStart(void);
extern void* hf_MemPhysicalEnd(void);
extern void* hf_MemImageStart(void);
extern void* hf_MemImageEnd(void);
extern void* hf_MemAllocBase(U32 size, int fromtop, tHSTask owner);
extern void hf_MemFreeBase(PTR address, tHSTask caller);
extern void* hf_MemReallocBase(void* p, U32 size, tHSTask caller);
extern U32 hf_MemSpaceBase(void);
extern void hf_MemSetTypeBase(void* vaddr, U32 type);
extern U32 hf_MemGetTypeBase(void* vaddr);


#ifdef LINUX_OVER_SOLOS
/* If you call any of these with something other than a DRAM address you will regret it. Also
 * they don't preserve NULL */
#define FAST_UNCACHED( _X )  ((void*) (((U32)_X & ~BF_PP_VADDR_SDRAMC) | BF_PP_VADDR_SDRAMU )) 
#define FAST_CACHED( _x )    ((void*)((((U32)_x) & ~BF_PP_VADDR_SDRAMU) | BF_PP_VADDR_SDRAMC))
#define FAST_ISCACHED( _x ) ((((U32)_x) & BF_PP_VADDR_SDRAMC) == BF_PP_VADDR_SDRAMC )

#else

/* If you call any of these with something other than a DRAM address you will regret it. Also
 * they don't preserve NULL */
#define FAST_UNCACHED( _X )  ((void*) (((U32)_X & ~CF_PP_VADDR_SDRAMC) | CF_PP_VADDR_SDRAMU )) 
#define FAST_CACHED( _x )    ((void*)((((U32)_x) & ~CF_PP_VADDR_SDRAMU) | CF_PP_VADDR_SDRAMC))
#define FAST_ISCACHED( _x ) ((((U32)_x) & CF_PP_VADDR_SDRAMC) == CF_PP_VADDR_SDRAMC )

#endif /* LINUX_OVER_SOLOS */

#endif  /* _QUANTUM_HF_MEM_H */
