/* ###########################################################################
# (c) Copyright Conexant Systems, Inc 2001, 2004
#
# Conexant Systems, Inc Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc ("Conexant"). It may only be read,
# used, copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Conexant and then subject to
# the terms of that confidentiality agreement and any other applicable
# agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a confidentiality agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/


#ifndef _QUANTUM_HF_MESSAGE_H
#define _QUANTUM_HF_MESSAGE_H

/*! \file   hf_message.h
    \brief  Internal message handling (Helium family)

    This header defines internal functions used by the message handling
    code.
    
    At present, its main purpose it to provide compile-time checks for
    message structure validity.
*/


/*! Diagnostic enable
 
    Set this to zero to disable message diagnostic functions, or one
    to enable them.
*/
#ifndef HF_MESSAGE_DIAGNOSTIC
#define HF_MESSAGE_DIAGNOSTIC   (1)     /**< Message diagnostics are enabled by default */
#endif


#if defined (HF_ASSEMBLER_INCLUDE)

/* --------------------------------------------------------------------------
 *
 *              Assembler specific declarations
 *
 * --------------------------------------------------------------------------
 */

#if (HF_MESSAGE_DIAGNOSTIC)

        macro
        message_check   $msg, $file, $func
        import  _hf_MessageCheck
        stmfd   sp!, {r0-r14}
        mov     r0, $msg
        adr     r1, $1f
        adr     r2, $2f
        mov     r3, pc
        bl      _hf_MessageCheckHandler
        b       $9f
$1:     dcb     $file, 0
$2:     dcb     $func, 0
        align
$9:     ldmfd   sp!, {r0-r14}
        mend

#else   /* HF_MESSAGE_DIAGNOSTIC */

        macro
        message_check   $msg, $file, $func
        mend

#endif  /* HF_MESSAGE_DIAGNOSTIC */



#else /* defined HF_ASSEMBLER_INCLUDE */

/* --------------------------------------------------------------------------
 *
 *              C specific declarations
 *
 * --------------------------------------------------------------------------
 */


//#include "quantum/as_types.h"


/* Private function prototypes.
 */
extern void hf_MessageInit(
    tHSMessage msg,
    tHSInterface home,
    tHSInterface reply,
    tHSInterface target,
    tHSTask task,
    void* pAlloc,
    U32 prefix,
    U32 postfix);

extern void hf_MessageCheckHandler(tHSMessage msg, const char* pszFile, const char* pszFunc, U32 locn);


/*! \brief  Internal message structure check for local code
    \param  _msg    The message handle to be checked
    
    This macro checks a message handle for validity.  Typically, this
    option will only be enabled in debug builds - it incurs significant
    performance penalties.
*/
#if (HF_MESSAGE_DIAGNOSTIC)
#define hf_MessageCheck(_msg)    hf_MessageCheckHandler((_msg), __FILE__, __FUNCTION__, __LINE__);
#else
#define hf_MessageCheck(_msg)   ((void) 0)
#endif

/* Internal lx port functionality */

extern void hf_messagePrint(tHSMessage msg, int verbosity);

#endif  /* HF_ASSEMBLER_INCLUDE */
#endif  /* _QUANTUM_HS_MESSAGE_H */
