/* ###########################################################################
# (c) Copyright GlobespanVirata Limited 2004
#
# GlobespanVirata Limited Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to GlobespanVirata (GSV). It may only be read, used,
# copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with GSV and then subject to the
# terms of that confidentiality agreement and any other applicable agreement
# between you and GSV.  If you are in any doubt as to whether you are
# entitled to access, read, use, copy, adapt, modify or otherwise deal with
# the Software or whether you are entitled to disclose the Software to any
# other person you should contact GSV.  If you have not entered into a
# confidentiality agreement with GSV granting access to this Software you
# should forthwith return all media, copies and printed listings containing
# the Software to GSV.
#
# GSV reserves the right to take legal action against you should you breach
# the above provisions.
#
# If you are unsure, or to report violations, please contact
# support@globespanvirata.com
# ##########################################################################*/

/* This file contains all the out-of-line functions provided for legacy support */
/* Note that it is #included by lx_module.c rather than linked with it to make
 * the EXPORT_SYMBOL work. A better long-term solution might be to have an 
 * internal header file and move all the EXPORT_SYMBOLS into lx_module.c */

FILE *stdout = NULL;
FILE *stderr = NULL;
EXPORT_SYMBOL(stdout); 
EXPORT_SYMBOL(stderr); 

#include "linux/sched.h"
#include "linux/kernel.h"
#include "asm/fiq.h"
#include "linux/mm.h"
#include "asm/page.h"
#include "stdio.h"
#include "stdarg.h"
#include "lx_err.h"
#include "lx_task.h"
#include "lx_interrupt.h"
#include "lx_shim_fp.h"

#ifdef LINUX_OVER_SOLOS
#include "asm/memory.h"
#endif /* LINUX_OVER_SOLOS */

int atoi(const char *s);
//static char bf[1024];
/* Note the complete non-reentacy of these two functions... */

void kprintf(const char *fmt, ...)
{
  char bf[200];
  va_list args;
  va_start(args, fmt);
  vsnprintf(bf, 200, fmt, args);
  as_printk(bf);
}

EXPORT_SYMBOL( kprintf );

/* Review: use the CAKE version at some point... */
int printf(const char *format, ...)
{
  int n;
  char bf[200];
  va_list args;
  va_start(args, format);
  n = vsnprintf(bf, 200, format, args);
  printk(bf);
  return n;
}

EXPORT_SYMBOL( printf );

int fprintf( FILE * f, const char *format, ... )
{
  int n;
  char bf[200];
  va_list args;
  va_start(args, format);
  (void)f;
  n = vsnprintf(bf, 200, format, args);
  printk( bf );
  return n;
}
EXPORT_SYMBOL( fprintf );

int vfprintf( void *ignored, const char *format, va_list args )
{
  int n;
  char bf[200];
  (void)ignored;
  n = vsnprintf(bf, 200, format, args);
  printk( bf );
  return n;
}
EXPORT_SYMBOL( vfprintf );

extern int sendreply(ATMOS_MESSAGE *);
EXPORT_SYMBOL(sendreply);


void atmos_handlersendmessage(ATMOS_MQID queue, ATMOS_MESSAGE *msg) 
{
  tASInterface iface =  hs_TaskCurrentPrimary();
  hs_MessageSend( (tHSInterface)queue, (tHSMessage)msg, hs_TaskCurrentPrimary() );
}
EXPORT_SYMBOL(atmos_handlersendmessage);


ATMOS_MESSAGE *atmos_awaitmessage(void) 
{
    tHSInterface iface = hs_TaskCurrentPrimary();
    tHSMessage msg = hs_MessageWait(iface);
    return (ATMOS_MESSAGE *)msg;
}
EXPORT_SYMBOL(atmos_awaitmessage);

#ifndef LINUX_OVER_SOLOS
#include "hf_types.h"
static PPNP_SHARED *ppNp_shared = (PPNP_SHARED *)HF_SHARED;
EXPORT_SYMBOL( ppNp_shared );
#endif /* LINUX_OVER_SOLOS */

BITS atmos_startcritical( void )
{
  BITS flags;

#ifdef LINUX_OVER_SOLOS
  local_save_flags( flags );
  local_irq_disable();
#else
  save_flags( flags );
  cli();
#endif /* LINUX_OVER_SOLOS */

  return flags;
}

void atmos_endcritical( BITS flags )
{
#ifdef LINUX_OVER_SOLOS
  local_irq_restore( flags );
#else
  restore_flags( flags );
#endif /* LINUX_OVER_SOLOS */
}

EXPORT_SYMBOL( atmos_startcritical );
EXPORT_SYMBOL( atmos_endcritical );

int atmos_schedule_enable; /* This was a really bad idea even under ATMOS. */
EXPORT_SYMBOL( atmos_schedule_enable );


const char *atmos_pcb_current_get_name()
{
  tHSTask self = hs_TaskCurrent();
  if ( self != 0 )
    return self->pszName;
  return "(null)";
}
EXPORT_SYMBOL(atmos_pcb_current_get_name);

void _initio(void) 
{

};
EXPORT_SYMBOL(_initio);

#include "linux/interrupt.h"
int atmos_ininterrupt(void) { return in_interrupt(); }
EXPORT_SYMBOL(atmos_ininterrupt);

extern tASInterface atmos_task_get_current(void);
EXPORT_SYMBOL(atmos_task_get_current);

BOOL console_register_process(const char *name, BOOL help)
{
#ifdef CONSOLEAGENT_ON_LINUX
    printk("Registering process %s with console\n", name ); 
    return xs_ConsoleLegacyRegister(name, help, 0, hs_TaskCurrent());
#else
    return TRUE;
#endif
}
EXPORT_SYMBOL(console_register_process);

int sendmessage(register ATMOS_MESSAGE *m, BITS code, ATMOS_MQID dest)
{
  hs_MessageSetCode((tHSMessage)m, code );
  return (int)hs_MessageSend((tHSInterface) dest, (tHSMessage) m, hs_TaskCurrentPrimary());
}
EXPORT_SYMBOL(sendmessage);

ATMOS_MQID atmos_findqueue( const char *name )
{
  tASInterface iface;
  int err;
  char fullname[50];
  sprintf( fullname, "/task/%s", name );
  err = hsl_InterfaceOpen( (void*)&iface, fullname );
  if ( err != kHSLEOK )
    {
      printk("atmos_findqueue:  hsl_InterfaceOpen returned \n");
      iface = 0;
    }
  return iface;
}
EXPORT_SYMBOL( atmos_findqueue );

ATMOS_MQID findqueue( const char *name ) __attribute__ (( alias("atmos_findqueue")));
EXPORT_SYMBOL(findqueue);

static int fl_errno = 0;

FILE *fopen(const char *name, const char *mode)
{
        struct file *fd;
        if ( *mode == 'r' )
          fd = filp_open(name, O_RDONLY, 0);
        else
          fd = filp_open(name, O_WRONLY | O_CREAT, 0);
        if (IS_ERR(fd)) {
                fl_errno = PTR_ERR(fd);
                return NULL;
        }
        return (FILE*)fd;
}


EXPORT_SYMBOL(fopen);


int fclose( FILE *fp )
{
    struct file *fd = (struct file *)fp;
        int error;
        error = filp_close(fd, current->files);
        fl_errno = error;
        return error;
}
EXPORT_SYMBOL(fclose);

#include "asm/uaccess.h" 

char *fgets(char *buffer, int size, FILE *fp )
{
        mm_segment_t oldfs;
        unsigned long total;
        int error = 1;
        char *tmp = buffer;
        char *end = buffer + size - 1;
        struct file *fd = (struct file *)fp;
        loff_t last_pos = fd->f_pos;
        /* The file operations expect to be given a user-space
         * buffer unless we turn off checking around this */
        oldfs = get_fs();
        set_fs(KERNEL_DS);

        while( (error > 0) && (end > tmp ) )
        {
            error = fd->f_op->read(fd, tmp, 1, &fd->f_pos);
            if ( *tmp == '\n' )
            {
                tmp++;
                break;
            }
            tmp++;
            if (error < 0) {
                printk("read returned: %d\n", error);
                fl_errno = error;
                break;
            }
            if ( last_pos == fd->f_pos )
            {
                error = -1;
                break;
            }
            last_pos = fd->f_pos;
        }
        set_fs(oldfs);
        *tmp = '\0';
        return (error < 0 ) ? NULL : buffer;
}

EXPORT_SYMBOL(fgets);

int fread(char *buffer, int num, int size, FILE *fp )
{
        mm_segment_t oldfs;
        unsigned long total;
        int error;
        struct file *fd = (struct file*)fp;
        total = num * size;
        /* The file operations expect to be given a user-space
         * buffer unless we turn off checking around this */
        oldfs = get_fs();
        set_fs(KERNEL_DS);
        error = fd->f_op->read(fd, buffer, total, &fd->f_pos);
        set_fs(oldfs);

        if (error < 0) {
                printk("read returned: %d\n", error);
                fl_errno = error;
                return error;
        }
        return error;
}
EXPORT_SYMBOL(fread);

int fwrite(char *buffer, int num, int size, FILE *fp )
{
        mm_segment_t oldfs;
        unsigned long total;
        int error;
        struct file *fd = (struct file*)fp;
        total = num * size;
        /* The file operations expect to be given a user-space
         * buffer unless we turn off checking around this */
        oldfs = get_fs();
        set_fs(KERNEL_DS);
        error = fd->f_op->write(fd, buffer, total, &fd->f_pos);
        set_fs(oldfs);

        if (error < 0) {
                printk("write returned: %d\n", error);
                fl_errno = error;
                return error;
        }
        return error;
}
EXPORT_SYMBOL(fwrite);

#ifdef LINUX_OVER_SOLOS

U32 clock(void)
{
    struct timespec ts;
    U32 stime, utime;
    ts = xtime;
    stime  =  ts.tv_sec;
    /* 
     * Convert nanoseconds to microseconds.
     */
    utime  =  ts.tv_nsec / 1000;

    return stime*CLOCKS_PER_SEC + (CLOCKS_PER_SEC * utime)/1000000;

}
#else

U32 clock(void)
{
    struct timeval tv;
    U32 stime, utime;
    get_fast_time(&tv);
    stime  =  tv.tv_sec;
    utime  =  tv.tv_usec;

    return stime*CLOCKS_PER_SEC + (CLOCKS_PER_SEC * utime)/1000000;

}

#endif /* LINUX_OVER_SOLOS */
EXPORT_SYMBOL(clock);
#if OS_HOSTED == LINUX 
int atoi(const char *s)
{
  int result;
  hsl_StrToS32(&result, NULL, s, 10);
  return result;

}
#endif
#ifdef SOLOS_WITH_SIMI
int atol(const char *s)
{
  int result;
  hsl_StrToU32(&result, NULL, s, 16);
  return result;

}
EXPORT_SYMBOL(atol);
#endif

EXPORT_SYMBOL(atoi);
