/* ###########################################################################
# (c) Copyright Virata Limited 2002
#
# Virata Limited Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Virata Limited ("Virata").  It may only be read, used,
# copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Virata and then subject to the
# terms of that confidentiality agreement and any other applicable agreement
# between you and Virata.  If you are in any doubt as to whether you are
# entitled to access, read, use, copy, adapt, modify or otherwise deal with
# the Software or whether you are entitled to disclose the Software to any
# other person you should contact Virata.  If you have not entered into a
# confidentiality agreement with Virata granting access to this Software you
# should forthwith return all media, copies and printed listings containing
# the Software to Virata.
#
# Virata reserves the right to take legal action against you should you breach
# the above provisions.
#
# If you are unsure, or to report violations, please contact
# support@virata.com
# ##########################################################################*/


/** @file       lx_procfiles.c
 *  @brief      Linux /proc code for pseudo-console
 */

#include "lx_shim_fp.h"
#include "lx_shim_defines.h"
#include "config.h"
#include "config/quantum.h"
#include "linux/proc_fs.h"
#include "linux/string.h"

#include "quantum/hs_task.h"
#include "lx_task.h"

#ifdef LINUX_OVER_SOLOS
#include <asm/uaccess.h>
#endif /* LINUX_OVER_SOLOS */

extern void lx_SetupProc( void  );

struct proc_dir_entry *quantum_dir;
static struct proc_dir_entry *quantum_tasks;
static struct proc_dir_entry *quantum_cons;
static struct proc_dir_entry *quantum_version;

#ifdef PROC_DSL_APP
struct proc_dir_entry *adsl_dir;
#endif
#ifndef QUANTUM_LEAN
static int quant_tsk_read(char *buffer, char **start, off_t offset, int count, int *eof, void *data)
{
        int len = hsi_taskCmdEntryHelper( kHSITaskEntryMain, buffer, count );
        if ( len == -1 ) /* Didn't all fit */
            len = count;
        *eof = 1;
        return len;
}


static  int quant_cons_write(struct file *file, const char *buffer,
                          unsigned long count, void *data)
{
#ifdef LINUX_OVER_SOLOS
  /*
   * The string from userspace strangely contains a newline character.
   * before the NULL terminating character. This causes incorrect
   * parsing of command line args for the debug commands. Hence we
   * allocate our kernel memory and copy till "count - 1" and add our
   * own NULL terminator
   */
  
  char *cmd_string = NULL;
  int retval = count;
  
  cmd_string = kmalloc(count, GFP_KERNEL);
  if(!cmd_string){
    printk("ERR : %s : Unable to allocate memory \n", __FUNCTION__);
    retval = -ENOMEM;
    goto cleanup;
  }
  
  hsl_BlockZero(cmd_string, count);
  
  if(copy_from_user(cmd_string, buffer, count - 1)) {
    printk("ERR : %s : Unable to 'copy_from_user' \n",  __FUNCTION__);
    retval = -EFAULT;
    goto cleanup;
  }
  
  cmd_string[count] = '\0';
  
  xs_DebugParseCommand( (char *) cmd_string);

 cleanup:
  if(cmd_string)
    kfree(cmd_string);
  return retval;
  
#else
  
  xs_DebugParseCommand( (char *)buffer );
  return count;
  
#endif /* LINUX_OVER_SOLOS */
  
}
#endif

static int quant_ver_read( char *buffer, char **start, off_t offset, int count, int *eof, void *data) 
{
        int n;
        n = snprintf( buffer, count, "LINUX RELEASE "  ATML_VERSION_STRING " \n");
        n += snprintf( buffer + n, count - n,  CSP_VERSION "\n" );
        n += snprintf( buffer + n, count - n,  BSP_VERSION "\n" );
        *eof =1 ;
        return n;
}

void lx_SetupProc( void  )
{
        quantum_dir = create_proc_entry("quantum", S_IFDIR | S_IRUGO | S_IXUGO, &proc_root );
        #ifdef PROC_DSL_APP
        adsl_dir =  create_proc_entry( "adsl",S_IFDIR | S_IRUGO | S_IXUGO,&proc_root );
        #endif
#ifndef QUANTUM_LEAN
        quantum_tasks = create_proc_entry( "tasks", S_IFREG | S_IRUSR, quantum_dir );
        quantum_cons = create_proc_entry( "console", S_IFREG | S_IRUSR, quantum_dir );
        quantum_tasks->read_proc = quant_tsk_read;
        quantum_cons->write_proc = quant_cons_write;
#endif
        quantum_version = create_proc_entry( "version", S_IFREG | S_IRUGO, quantum_dir );

        quantum_version->read_proc = quant_ver_read;
}
