/* ###########################################################################
# (c) Copyright Conexant Systems, Inc 2001, 2006
#
# Conexant Systems, Inc Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc ("Conexant"). It may only be read,
# used, copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Conexant and then subject to
# the terms of that confidentiality agreement and any other applicable
# agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a confidentiality agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/

#ifndef LX_SHIM_TYPES_H
#define LX_SHIM_TYPES_H

/** @def    HSL_TYPE(c1, c2, c3, c4)
 *  @brief  Create a value of type \c tHSLType using the supplied four
 *  character code.
 *
 * Quantum makes extension use of a dynamic data typing mechanism that may 
 * be used for run-time assert() checking or to provide run-time dynamic
 * object typing.
 *
 * Type codes are of type \c tHSLType and may be assigned constant values 
 * generated from four single HSLCII characters using the macro \c HSL_TYPE().
 *
 * A naming convention is used to reduce the risk of name collision of types.
 * All type codes begining with a 'q' are reserved for use within Quantum.
 * A code of all zeros is reserved to denote an invalid type code.
 * All other possible codes may be used by application code.
 *
 * The following HSLCII characters are legal within a type code:
 *
 *  <ul>
 *      <li> Upper or lower case letters
 *
 *      <li> Numeric digits
 *  </ul>
 *
 * Non printable characters, punctuation and quotation marks are reserved
 * and must not be used.  Case is significant in type codes.
 *  
 * Within Quantum, the following conventions are used:
 *
 *  <ul>
 *      <li> \c qa**    Quantum HSL public types (eg: U32, tHSLTask, etc)
 *
 *      <li> \c qx**    Quantum XS internal types
 *
 *      <li> \c qh**    Quantum HS host implementation specific types
 *  </ul>
 *  
 * If a type is used with Quantum's internal object management system, it must 
 * be globally unique.  It is therefore important to manage object type codes
 * effectively in order to avoid problematic clashes.
 *
 * The scheme used to avoid application type code clashes is outside of this
 * specification.  However, it is recommended that each source module that
 * wishes to register a type code should be assigned a unique two character
 * prefix (eg: 'bn' for BUN).  The remaining two characters may be used within
 * the module for different data types.
 */

typedef unsigned int     tHSLType;
typedef void*             tHSLObject;
typedef void*            tHSLTask ;

#if 0
//#if HSL_ENDIAN_LITTLE // lavin
#define HSL_TYPE(_a, _b, _c, _d)     ((tHSLType) ((_a) + ((_b) << 8) + ((_c) << 16) + ((_d) << 24)))
#else
#define HSL_TYPE(_a, _b, _c, _d)     ((tHSLType) ((_d) + ((_c) << 8) + ((_b) << 16) + ((_a) << 24)))
#endif

/** A type code that is guaranteed to be invalid */
#define kHSLTypeInvalid         ((tHSLType)(0))
/** Indicates that no type information is known (kind of like \c void* in C */
#define kHSLTypeNone            HSL_TYPE('q', 'a', '0', '0')
/** Type code for \c unsigned \c int types */
#define kHSLTypeUnsigned        HSL_TYPE('q', 'a', 'u', '0')
/** Type code for U8 */
#define kHSLTypeU8              HSL_TYPE('q', 'a', 'u', '1')
/** Type code for U16 */
#define kHSLTypeU16             HSL_TYPE('q', 'a', 'u', '2')
/** Type code for U32 */
#define kHSLTypeU32             HSL_TYPE('q', 'a', 'u', '3')
/** Type code for U64 */
#define kHSLTypeU64             HSL_TYPE('q', 'a', 'u', '4')
/** Type code for \c int types */
#define kHSLTypeInteger         HSL_TYPE('q', 'a', 's', '0')
/** Type code for S8 */
#define kHSLTypeS8              HSL_TYPE('q', 'a', 's', '1')
/** Type code for S16 */
#define kHSLTypeS16             HSL_TYPE('q', 'a', 's', '2')
/** Type code for S32 */
#define kHSLTypeS32             HSL_TYPE('q', 'a', 's', '3')
/** Type code for S64 */
#define kHSLTypeS64             HSL_TYPE('q', 'a', 's', '4')
/** Type code for ADR */
#define kHSLTypeADR             HSL_TYPE('q', 'a', 'p', '0')
/** Type code for \c void* pointers */
#define kHSLTypePTR             HSL_TYPE('q', 'a', 'p', '1')
/** Type code for null terminated C strings */
#define kHSLTypeString          HSL_TYPE('q', 'a', 'p', '2')
/** Type code for Quantum enumerations (using \c int type) */
#define kHSLTypeEnum            HSL_TYPE('q', 'a', 'e', '0')
/** Type code for Boolean enumerations (using \c int type) */
#define kHSLTypeBool            HSL_TYPE('q', 'a', 'e', '1')
/** Type code for tHSLObject */
#define kHSLTypeObject          HSL_TYPE('q', 'a', 'h', '0')
/** Type code for tHSLInterface */
#define kHSLTypeInterface       HSL_TYPE('q', 'a', 'h', '1')
/** Type code for tHSLMessage */
#define kHSLTypeMessage         HSL_TYPE('q', 'a', 'h', '2')
/** Type code for tHSLSemaphore */
#define kHSLTypeSemaphore       HSL_TYPE('q', 'a', 'h', '3')
/** Type code for tHSLTask */
#define kHSLTypeTask            HSL_TYPE('q', 'a', 'h', '4')
/** Type code for tHSLPool */
#define kHSLTypePool            HSL_TYPE('q', 'a', 'h', '5')
/** Type code for tHSLMsgInterface */
#define kHSLTypeMsgInterface    HSL_TYPE('q', 'a', 'h', '7')
/** Type code for tHSLMonitoredPool */
#define kHSLTypeMonitoredPool   HSL_TYPE('q', 'a', 'h', '8')
/** Type code for tHSLTaskService */
#define kHSLTypeTaskService     HSL_TYPE('q', 'a', 'h', '9')
/** Type code for tHSLQueue */
#define kHSLTypeQueue           HSL_TYPE('q', 'a', 'l', '0')
/** Type code for tHSLMemRegion */
#define kHSLTypeMemRegion       HSL_TYPE('q', 'a', 'm', '0')
/** Type code for tHSLShMem (shared memory section) */
#define kHSLTypeShMem           HSL_TYPE('q', 'a', 'm', '1')
/** Type code for tHSLStream */
#define kHSLTypeStream          HSL_TYPE('q', 'a', 'i', '0')
/** Type code for tHSLBoundInterface */
#define kHSLTypeBoundInterface  HSL_TYPE('q', 'a', 'i', '1')
/** Type code for tHSLVariable */
#define kHSLTypeVariable        HSL_TYPE('q', 'a', 'v', '0')
/** Type code for tHSLIrqController */
#define kHSLTypeIrqController   HSL_TYPE('q', 'a', 'q', '0')
/** Type code for tHSLInterrupt */
#define kHSLTypeInterrupt       HSL_TYPE('q', 'a', 'q', '1') 

/** Object limit configuration:
 *  The following maxima for string lengths include the '\0' character
 *  that is used to terminate a string.
 */

#define kHSLObjectNameMaxlen      256 /**< max size of object name e.g. A/B/C */
#define kHSLObjectKeyMaxlen       128 /**< max size of object attribute name */
#define kHSLObjectKeyValMaxlen    128 /**< max size of object attribute value */


/** @following defenitions derived from xs_types.h
 *  @brief      Host independent type definitions.
 *  @ingroup    platform.
 *
 * This file defines the portable types for the Helium
 * native platform.
 *
 */

/* Object handles.
 */
#define kHSLObjectInvalid        (0)       /**< The invalid object handle */

/* Memory region handles.
 */
#define kHSLMemRegionInvalid     (0)    /**< The invalid memory region handle */

/* Memory allocator handles.
 */

#define kHSLMemAllocatorInvalid  (0) /**< The invalid memory allocator handle */


/* Log output handles.
 */

#define kHSLLogInvalid           (0)          /**< The invalid log output handle */

/* Command output handles.
 */

#define kHSLCommandInvalid       (0)      /**< The invalid command output handle */


/* Interface handles (transmit only).
 */

#define kHSLInterfaceInvalid (0)        /**< The invalid interrupt handle */

/* Interrupt handles
 */

#define kHSLInterruptInvalid (0)        /**< The invalid interrupt handle */

/* Interrupt controller handles
 */

#define kHSLIrqControllerInvalid (0) /**< The invalid interrupt controller handle */


/**
 * Enumerated key type definition.  This specifies the mapping between a
 * series of text labels and integer values.
 *
 * An array of these terminated by \c HSL_OBJECT_ENUM_END is used to define
 * the permissible values of a key of type kHSLObjectKeyTypeEnum and the framework
 * will use this data to perform the translation from string to integer directly.
 */

typedef struct sHSLObjectEnum
{
    const char* pszName;        /**< The enumeration name */
    int value;                  /**< The integer value to associate with the name */
} tHSLObjectEnum;

#define HSL_OBJECT_ENUM(_name, _value)    { _name, _value }   /**< Use this macro to initialise a single \c tHSLObjectEnum instance */
#define HSL_OBJECT_ENUM_END               { 0, 0 }            /**< Use this macro to mark the end of an array of \c tHSLObjectEnum */


/**
 * Generic prototype for attribute-set access functions.
 *
 * This is the generic prototype for a 'set' access function.  This
 * is specified in the array of key definitions (see tHSLObjectKey)
 * or as a constant of zero to denote that 'set' operations are never
 * permitted.
 *
 * Actual implementations of set methods should use actual data types
 * rather than \c void* pointers.  The size of the object supplied
 * with the \e pInput argument is implicit from the key type.
 */
typedef int (*tHSLObjectKeySetGeneric)(tHSLObject object, 
                                         const void* pInput);


/**
 * Generic prototype for attribute-get access functions.
 *
 * This is the generic prototype for a 'get' access function.
 *
 * The \e size argument may be ignored for all types other than
 * string, where it specifies the size of the output buffer in
 * characters.  For other types, the size will simply be the size
 * of the type described by the key - eg 'sizof (U32)' etc.
 */
typedef int (*tHSLObjectKeyGetGeneric)(tHSLObject object, 
                                         void* pOutput, unsigned int size);


/**
 * Generic prototype for string formatted set access.
 *
 * This is the generic prototype for a 'string-set' access function.
 * Actual implementations of this method should use actual data types
 * rather than the \c void* pointer.
 */
typedef int (*tHSLObjectKeySetString)(tHSLObject object, 
                                        const char* pszInput);


/**
 * Generic prototype for string formatted get access.
 *
 * This is the generic prototype for a 'string-get' access function.
 * Actual implementations of this method should use actual data types
 * rather than the \c void* pointer.  The \e size parameter gives the
 * size of the output buffer (as number of characters).  The supplied
 * size is guaranteed to always be greater than zero.
 */
typedef int (*tHSLObjectKeyGetString)(tHSLObject object, 
                                        char* pszOutput, const unsigned int size);


/* Specific access method types.  Defining these allows the use of C99
 * union initialisers to provide some measure of compile-time type
 * safety.
 */
typedef int (*tHSLObjectKeySetU32)(tHSLObject obj, const unsigned int * pInput);
typedef int (*tHSLObjectKeySetS32)(tHSLObject obj, const int* pInput);
typedef int (*tHSLObjectKeySetU64)(tHSLObject obj, const unsigned long* pInput);
typedef int (*tHSLObjectKeySetS64)(tHSLObject obj, const long* pInput);
typedef int (*tHSLObjectKeySetPTR)(tHSLObject obj, const void** pInput);
typedef int (*tHSLObjectKeySetBool)(tHSLObject obj, const int* pInput);
typedef int (*tHSLObjectKeySetEnum)(tHSLObject obj, const int* pInput);
typedef int (*tHSLObjectKeySetObj)(tHSLObject obj, const tHSLObject* pInput);
    
typedef int (*tHSLObjectKeyGetU32)(tHSLObject obj, unsigned int* pOutput);
typedef int (*tHSLObjectKeyGetS32)(tHSLObject obj, int* pOutput);
typedef int (*tHSLObjectKeyGetU64)(tHSLObject obj, unsigned long* pOutput);
typedef int (*tHSLObjectKeyGetS64)(tHSLObject obj, long* pOutput);
typedef int (*tHSLObjectKeyGetPTR)(tHSLObject obj, void** pInput);
typedef int (*tHSLObjectKeyGetBool)(tHSLObject obj, int* pOutput);
typedef int (*tHSLObjectKeyGetEnum)(tHSLObject obj, int* pOutput);
typedef int (*tHSLObjectKeyGetObj)(tHSLObject obj, tHSLObject *pOutput);



/**
 * Table entry used to construct lists of attribute handlers.
 *
 * Code should not use this structure directly, but should instead
 * use the macros \c HSL_OBJECT_KEY_xx() to create an array of
 * these entries that define the attributes implemented by a given
 * object.
 *
 *  @todo
 *      Iterators and mechanisms to query the range of permitted values
 *      for a given attribute key.
 */
struct sHSLObjectKey
{
    const char* pszName;            /**< Attribute key name */
    unsigned int type;                   /**< Type code information */
    unsigned int size;                       /**< Size or size limit */
    
    union {
        const char *psEnumDir;      /**< Object directory for ennumeration by object */
        const tHSLObjectEnum* pEnum; /**< Enumeration for auto-enumerated objects */
    } arg;
    
    union {
        tHSLObjectKeySetGeneric pfGeneric;   /**< Generic set handler */
        tHSLObjectKeySetU32 pfU32;           /**< Type specific set handler */
        tHSLObjectKeySetS32 pfS32;           /**< Type specific set handler */
        tHSLObjectKeySetU64 pfU64;           /**< Type specific set handler */
        tHSLObjectKeySetS64 pfS64;           /**< Type specific set handler */
        tHSLObjectKeySetPTR pfPTR;           /**< Type specific set handler */
        tHSLObjectKeySetBool pfBool;         /**< Type specific set handler */
        tHSLObjectKeySetEnum pfEnum;         /**< Type specific set handler */
        tHSLObjectKeySetObj pfObj;           /**< Type specific set handler */
        tHSLObjectKeySetString pfString;     /**< Type specific set handler */
    } set;                          /**< Handler for set operations */
    union {
        tHSLObjectKeyGetGeneric pfGeneric;   /**< Generic get handler */
        tHSLObjectKeyGetU32 pfU32;           /**< Type specific set handler */
        tHSLObjectKeyGetS32 pfS32;           /**< Type specific set handler */
        tHSLObjectKeyGetU64 pfU64;           /**< Type specific set handler */
        tHSLObjectKeyGetS64 pfS64;           /**< Type specific set handler */
        tHSLObjectKeyGetPTR pfPTR;           /**< Type specific set handler */
        tHSLObjectKeyGetBool pfBool;         /**< Type specific set handler */
        tHSLObjectKeyGetEnum pfEnum;         /**< Type specific set handler */
        tHSLObjectKeyGetObj pfObj;           /**< Type specific set handler */
        tHSLObjectKeyGetString pfString;     /**< Type specific set handler */
    } get;                          /**< Handler for get operations */
    tHSLObjectKeySetString pfStrSet; /**< Handler for string-set operations, or zero if none */
    tHSLObjectKeyGetString pfStrGet; /**< Handler for string-get operations, or zero if none */

};


typedef struct sHSLObjectKey tHSLObjectKey;



/* Macros used to build an array of sHSLAttributeKeyDescription for a
 * given object class.
 */
#if !defined(__GNUC__) && (__STDC_VERSION__ < 199901)
/* Pre-C99 compiler - can't use explicit initialisation of union fields
 *
 * Initialisers for union fields must be cast to the first type in their union.
 *
 * These casts are provided in these macros to avoid the need to make explicit
 * casts in user code and to avoid compiler warnings for implicit type
 * warnings. (All of which unfortunately reduces compile-time type safety).
 */
#define HSL_OBJECT_KEY_U32(_key, _set, _get) \
{ _key, kHSLTypeU32,   sizeof(U32), 0, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, 0, 0 }
/**< Define a U32 attribute, using default string formatting */

#define HSL_OBJECT_KEY_S32(_key, _set, _get) \
{ _key, kHSLTypeS32,   sizeof(S32), 0, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, 0, 0 }
/**< Define an S32 attribute */

#define HSL_OBJECT_KEY_U64(_key, _set, _get) \
{ _key, kHSLTypeU64,   sizeof(U64), 0, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, 0, 0 }
/**< Define a U64 attribute */

#define HSL_OBJECT_KEY_S64(_key, _set, _get) \
{ _key, kHSLTypeS64,   sizeof(S64), 0, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, 0, 0 }
/**< Define an S64 attribute */

#define HSL_OBJECT_KEY_EXPRU32(_key, _set, _get, _root) \
{ _key, kHSLTypeU32,   sizeof(U32), _root, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, 0, 0 }
/**< Define a U32 attribute, using default string formatting */

#define HSL_OBJECT_KEY_EXPRS32(_key, _set, _get, _root) \
{ _key, kHSLTypeS32,   sizeof(S32), _root, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, 0, 0 }
/**< Define an S32 attribute expression */

#define HSL_OBJECT_KEY_EXPRU64(_key, _set, _get, _root) \
{ _key, kHSLTypeU64,   sizeof(U64), _root, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, 0, 0 }
/**< Define a U64 attribute expression */

#define HSL_OBJECT_KEY_EXPRS64(_key, _set, _get, _root) \
 { _key, kHSLTypeS64,   sizeof(S64), _root, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, 0, 0 }
/**< Define an S64 attribute expression */

#define HSL_OBJECT_KEY_PTR(_key, _set, _get) \
{ _key, kHSLTypePTR,   sizeof(PTR), 0, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, 0, 0 }
/**< Define a PTR attribute */

#define HSL_OBJECT_KEY_BOOL(_key, _set, _get) \
{ _key, kHSLTypeBool,  sizeof(int), 0, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, 0, 0 }
/**< Define a boolean attribute */

#define HSL_OBJECT_KEY_ENUM(_key, _set, _get, _enm) \
{ _key, kHSLTypeEnum,  sizeof(int), (const char*)_enm, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, 0, 0 }
/**< Define an enumerated attribute */

#define HSL_OBJECT_KEY_OBJ(_key, _set, _get, _root) \
{ _key, kHSLTypeObject,   sizeof(U64), _root, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, 0, 0 }
/**< Define a U64 attribute expression */

#define HSL_OBJECT_KEY_STRING(_key, _set, _get) \
{ _key, kHSLTypeString, 0, 0, (tHSLObjectKeySetGeneric)_set, (tHSLObjectKeyGetGeneric)_get, (tHSLObjectKeySetString)_set, (tHSLObjectKeyGetString)_get }
/**< Define a string attribute */

#define HSL_OBJECT_KEY_END  { 0, 0, 0, 0, 0, 0, 0, 0 }   /**< Mark the end of a list of attributes */

#else/* C99 Standard */
/* C99 compatible compiler - can use explicit initialisation of union fields 
 *
 * casts are used in the assignment of the function pointers, increasing type safety,
 * using a union for the function types and the C99 specific extension for initialising
 * a specific field in a union.
 */
#define HSL_OBJECT_KEY_U32(_key, _set, _get) \
{ _key, kHSLTypeU32,   sizeof(U32), { 0 }, { .pfU32 = _set }, { .pfU32 = _get }, 0, 0 }
/**< Define a U32 attribute, using default string formatting */

#define HSL_OBJECT_KEY_S32(_key, _set, _get) \
{ _key, kHSLTypeS32,   sizeof(S32), { 0 }, { .pfS32 = _set }, { .pfS32 = _get }, 0, 0 }
/**< Define an S32 attribute */

#define HSL_OBJECT_KEY_U64(_key, _set, _get) \
{ _key, kHSLTypeU64,   sizeof(U64), { 0 }, { .pfU64 = _set }, { .pfU64 = _get }, 0, 0 }
/**< Define a U64 attribute */

#define HSL_OBJECT_KEY_S64(_key, _set, _get) \
{ _key, kHSLTypeS64,   sizeof(S64), { 0 }, { .pfS64 = _set }, { .pfS64 = _get }, 0, 0 }
/**< Define an S64 attribute */

#define HSL_OBJECT_KEY_EXPRU32(_key, _set, _get, _root) \
{ _key, kHSLTypeU32,   sizeof(U32), { .psEnumDir = _root }, { .pfU32 = _set }, { .pfU32 = _get }, 0, 0 }
/**< Define a U32 attribute, using default string formatting */

#define HSL_OBJECT_KEY_EXPRS32(_key, _set, _get, _root) \
{ _key, kHSLTypeS32,   sizeof(S32), { .psEnumDir = _root }, { .pfS32 = _set }, { .pfS32 = _get }, 0, 0 }
/**< Define an S32 attribute */

#define HSL_OBJECT_KEY_EXPRU64(_key, _set, _get, _root) \
{ _key, kHSLTypeU64,   sizeof(U64), { .psEnumDir = _root }, { .pfU64 = _set }, { .pfU64 = _get }, 0, 0 }
/**< Define a U64 attribute */

#define HSL_OBJECT_KEY_EXPRS64(_key, _set, _get, _root) \
{ _key, kHSLTypeS64,   sizeof(S64), { .psEnumDir = _root }, { .pfS64 = _set }, { .pfS64 = _get }, 0, 0 }
/**< Define an S64 attribute */

#define HSL_OBJECT_KEY_PTR(_key, _set, _get) \
{ _key, kHSLTypePTR,   sizeof(PTR), { 0 }, { .pfPTR = _set }, { .pfPTR = _get }, 0, 0 }
/**< Define a PTR attribute */

#define HSL_OBJECT_KEY_BOOL(_key, _set, _get) \
{ _key, kHSLTypeBool,  sizeof(int), { 0 }, { .pfBool = _set }, { .pfBool = _get }, 0, 0 }
/**< Define a boolean attribute */

#define HSL_OBJECT_KEY_ENUM(_key, _set, _get, _enm) \
 { _key, kHSLTypeEnum,  sizeof(int), { .pEnum = _enm }, { .pfEnum = _set }, { .pfEnum = _get }, 0, 0 }
/**< Define an enumerated attribute */

#define HSL_OBJECT_KEY_OBJ(_key, _set, _get, _root) \
{ _key, kHSLTypeObject,   sizeof(tHSLObject), { .psEnumDir = _root }, { .pfObj = _set }, { .pfObj = _get }, 0, 0 }
/**< Define an tHSLObject attribute */

#define HSL_OBJECT_KEY_STRING(_key, _set, _get) \
{ _key, kHSLTypeString, 0, { 0 }, { .pfString = _set }, { .pfString = _get }, _set, _get }
/**< Define a string attribute */

#define HSL_OBJECT_KEY_END  { 0, 0, 0, { 0 }, { 0 }, { 0 }, 0, 0 }
/**< Mark the end of a list of attributes */
#endif  /* C99 Standard */

#define kHSLTypeLevelCastBase      (0)
#define kHSLTypeLevelAggregate     (1)
#define kHSLTypeLevelAggregates(n) (n)

#endif /* LX_SHIM_TYPES_H */
