/* ###########################################################################
# (c) Copyright Conexant Systems, Inc 2002, 2004
#
# Conexant Systems, Inc Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc ("Conexant"). It may only be read,
# used, copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Conexant and then subject to
# the terms of that confidentiality agreement and any other applicable
# agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a confidentiality agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/

#include "quantum/hs_task.h"
#include "quantum/hs_messagehdr.h"

#if defined (LX_ASSEMBLER_INCLUDE)

/* Task data structure.
 */
                        macro
$name                   sHSTask
$(name)
$(name).primary         sHSInterface
$(name).arrival         ds  1
$(name).pNextTask       ds  1
$(name).level           ds  1
$(name).pStack          ds  1
$(name).stackSize       ds  1
$(name).atmos1parent    ds  1
$(name).atmos1thread    ds  1
$(name).pszName         ds  1
$(name).pszArgument     ds  1
$(name).prologue        ds  1
$(name).epilogue        ds  1
$(name).entry           ds  1
$(name).qhandle         ds  1
$(name).secondary       sHSInterface
$(name).controlmsg      sHSMessage
$(name).constructed     ds  1
$(name).object          ds  1
$(name).linux_pid       ds  1 
$(name).linux_thread    ds  1
$(name).comp            ds  1
$(name).tasklib_nextfree ds  1
$(name).tasklib_remaining ds  1
$(name).magic           ds  1
$(name)_size            equ *-$(name)
                        mend
                
                dorg    0
tHSTask         sHSTask


#else


#include "linux/types.h"
#include "linux/completion.h"


/*! \brief      The task control block.

    \warning
        This is a subclass of tASInterface - the task's public interface
        must come at the start of this structure.
*/
struct sHSTask
{
    struct sHSInterface primary;        /**< Primary interface */
    struct sHSMessage  *arrival;        /**< Newly arrived message */
    tASTask pNextTask;                  /**< Next task in run queue or on waiting queue */
    U32 level;                          /**< Run level */
    void** pStack;                      /**< Initial stack pointer (higest address) */
    U32 stacksize;                      /**< Stack size (bytes) */
    struct sHSTask* atmos1parent;       /**< ATMOS1 Parent task, or zero if not threaded */
    tHSInterface atmos1thread;          /**< Used to implement ATMOS1 threads */
    const char* pszName;                /**< Used to hold the task name */
    const char* pszArgument;            /**< Argument string, or zero if none specified */
    int (*prologue)(int, const char**); /**< The ATMOS1 prologue */
    void (*epilogue)(int, const char**);/**< The ATMOS1 epilogue */
    int (*entry)(int, const char**);    /**< The main entry point */
    int (*qhandle)(struct sHSMessage*); /**< The ATMOS1 active queue handler */
    struct sHSInterface secondary;      /**< Secondary interface */
    struct sHSMessage controlmsg;       /**< Built in control message */
    int constructed;                    /**< Non-zero once task is fully constructed */
    tXSObject object;                   /**< Object Handle */
    pid_t linux_pid;
    struct task_struct *linux_thread;
    struct completion  comp;
    void *tasklib_nextfree;             /**< Area of memory for task libraries */
    _as_size_t tasklib_remaining;       /**< Remaining size of task library area */
    tASType magic;                      /**< kASTypeTask */
    
    /* (Area for pointers to task libraries) */
    /* (Area pointed to by tasklib_nextfree) */
};

extern int hsi_taskCmdEntryHelper(const U32 type, char *buffer, U32 len );

#endif  /* defined HF_ASSEMBLER_INCLUDE */
