#/* ###########################################################################
# (c) Copyright Virata Limited 2001
#
# Virata Limited Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Virata Limited ("Virata").  It may only be read, used,
# copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Virata and then subject to the
# terms of that confidentiality agreement and any other applicable agreement
# between you and Virata.  If you are in any doubt as to whether you are
# entitled to access, read, use, copy, adapt, modify or otherwise deal with
# the Software or whether you are entitled to disclose the Software to any
# other person you should contact Virata.  If you have not entered into a
# confidentiality agreement with Virata granting access to this Software you
# should forthwith return all media, copies and printed listings containing
# the Software to Virata.
#
# Virata reserves the right to take legal action against you should you breach
# the above provisions.
#
# If you are unsure, or to report violations, please contact
# support@virata.com
# ##########################################################################*/


/** @file       lx_timer.c
 *  @brief      Timer services (generic linux).
 *  @ingroup    util
 */

#include "lx_shim_fp.h"
#include "config.h"
#include "linux/sched.h"
#include "quantum/hs_timer.h"


/**
 * Delay execution for a specified number of milliseconds (or longer,
 * depending on system activity).
 *
 *  @param  delay   The delay in milliseconds.
 *
 * See as_TimerWait() for a detailed description of this function.
 */
void hs_TimerWait(U32 delay)
{
  set_current_state( TASK_INTERRUPTIBLE );

  /* Converting msdelay to jiffies. Changing the multi factor form 10 to 9, 
   * to have the correct precision of timeout value; avoiding extra timeout 
   * when delay >= 10 
   */

  schedule_timeout( (int)( (delay + 9) * HZ / 1000 ));
}


U32 hs_TimerSeconds( void )
{
    return jiffies/HZ;
}

#ifdef LINUX_OVER_SOLOS
int hs_TimerClock(struct as_timeval *tv)
{
  struct timespec ts;
#ifdef as_Cassert
  /* The as_timeval and the linux timeval are almost certainly the same, 
   * but check if we can... */
  as_Cassert( (U32)((struct timeval*)tv)->tv_sec == (U32)tv->tv_sec );
  as_Cassert( (U32)((struct timeval*)tv)->tv_usec == (U32)tv->tv_usec );
#endif
 
  ts = xtime;
  tv->tv_sec = ts.tv_sec;
  tv->tv_usec = ts.tv_nsec / 1000;
  return ESUCCESS;
}

#else

int hs_TimerClock(struct as_timeval *tv)
{
#ifdef as_Cassert
    /* The as_timeval and the linux timeval are almost certainly the same, 
     * but check if we can... */
    as_Cassert( (U32)((struct timeval*)tv)->tv_sec == (U32)tv->tv_sec );
    as_Cassert( (U32)((struct timeval*)tv)->tv_usec == (U32)tv->tv_usec );
#endif
    get_fast_time((struct timeval*)tv);
    return ESUCCESS;
}

#endif /* LINUX_OVER_SOLOS */
