/* ###########################################################################
# (c) Copyright Conexant Systems, Inc. 2004
# (c) Copyright GlobespanVirata, Inc., 2001-2004
#
# Conexant Systems, Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc. and its subsidiaries (collectively 
# "Conexant"). It may only be read, used, copied, adapted, modified or 
# otherwise dealt with by you if you have entered into a license agreement 
# with Conexant and then subject to the terms of that agreement and any other
# applicable confidentiality agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a license agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/

#ifndef _QUANTUM_HS_INTERFACE_H
#define _QUANTUM_HS_INTERFACE_H

/** @file   hs_interface.h
 *  @brief  Interface handling (Linux)
 */

/* #define MSGINTERFACE_NEW_VERSION */

#ifdef  MSGINTERFACE_NEW_VERSION
#define sXSInterface_size 3
#endif

/* Version code for the interface implementation.
 */
#ifdef  MSGINTERFACE_NEW_VERSION
#define kHSInterfaceVersion     (101)
#else
#define kHSInterfaceVersion     (100)
#endif


#if defined (HF_ASSEMBLER_INCLUDE)

/* --------------------------------------------------------------------------
 *
 *              Assembler specific declarations
 *
 * --------------------------------------------------------------------------
 */

/* Interface data structure.  This is the host specific
 * portion of the interface, which must be exactly 64 bytes
 * in size to map to the publically visible structures.
 */             
                        macro
$name                   sHSInterface
$(name)
$(name).qhandle         ds  1
$(name).tail            ds  1
$(name).control         ds  1
$(name).head            ds  1
$(name).put             ds  1
$(name).wait            ds  1
$(name).get             ds  1
$(name).magic           ds  1
#ifdef MSGINTERFACE_NEW_VERSION
$(name).pad1            ds  1 ; - Was object - now implied by interface
#else
$(name).object          ds  1
#endif
$(name).protocol        ds  1
$(name).sys             ds  1
$(name).lang            ds  1
$(name).socket_handler  ds  1
$(name).user            ds  1
$(name).errnop          ds  1
$(name).openlist        ds  1
$(name).envptr          ds  1
#ifdef MSGINTERFACE_NEW_VERSION
$(name).txif            ds  sXSInterface_size
#endif
$(name)_size            equ *-$(name)
                        mend

                dorg    0
tHSInterface    sHSInterface


#else /* defined HF_ASSEMBLER_INCLUDE */


/* --------------------------------------------------------------------------
 *
 *              C specific declarations
 *
 * --------------------------------------------------------------------------
 */

//#include "quantum/as_types.h"
#include "quantum/xs_object.h"

#ifdef MSGINTERFACE_NEW_VERSION
#include "quantum/xs_interface.h"

/* We can't conveniently include xs_interface.h in the assembler code   */
/* above - so we have to check that our guess is correct!               */
#if sXSInterface_size < kXSInterface_words
#error Not enougth space reserved in sHSInterface for a sXSInterface
#else
#if sXSInterface_size > kXSInterface_words
#error Too much space reserved in sHSInterface for a sXSInterface
#endif
#endif
#endif /* MSGINTERFACE_NEW_VERSION */

/* Stops the compiler complaining about a declaration of struct atmos_message
 * from within the interface object.
 */
extern struct sHSMessage* __dummy_atmos_message_declaration;

/** Interface protocol codes.
 */
enum eHFInterfaceProtocol
{
    kHFInterfaceProtocolInvalid =        1, /**< Invalid protocol */
    kHFInterfaceProtocolGeneric =        2, /**< Generic inter-task messaging */
    kHFInterfaceProtocolATMOS1 =         3, /**< Generic inter-task messaging with Queue Handle */
    kHFInterfaceProtocolPool =           4, /**< Message free-pools */
    kHFInterfaceProtocolUartRx =         5, /**< UART rx service */
    kHFInterfaceProtocolUartTx =         6, /**< UART tx service */
    kHFInterfaceProtocolGpio =           7, /**< GPIO #1 service */
    kHFInterfaceProtocolTimer4 =         8, /**< TIMER4 service */
#ifdef LINUX_OVER_SOLOS
    /*
     * Using Timer 3 instead of Timer 2.
     */
    kHFInterfaceProtocolTimer2 =         9, /**< TIMER2 service */
#else
    kHFInterfaceProtocolTimer2 =         9, /**< TIMER2 service */
#endif /* LINUX_OVER_SOLOS */
    kHFInterfaceProtocolDoorbell =      10, /**< TIMER2 service */
    kHFInterfaceProtocolPriority =      11, /**< Priority queue (32 bit priority code) */
    kHFInterfaceProtocolEthernet0 =     12, /**< Ethernet 0 (unused) */
    kHFInterfaceProtocolEthernet1 =     13, /**< Ethernet 0 (unused) */
    kHFInterfaceZCopyTx         =       14, /**< Linux interworking for Tx */
    kHFInterfaceZCopyRx         =       15, /**< Linux interworking for Rx */

    kHFInterfaceProtocolMin = 1,        /**< Smallest valid protocol code */
    kHFInterfaceProtocolMax = 15        /**< Largest valid protocol code */
};

typedef tASErr(*tHFSWIHandlerPut)(struct  sHSInterface *, struct sHSMessage *);
typedef tHSMessage(*tHFSWIHandlerWait)(struct  sHSInterface *);
typedef tHSMessage(*tHFSWIHandlerGet)(struct  sHSInterface *);


/**
 * The interface object structure.
 *
 * Interfaces interact directly with the ARM SWI trap mechanisms.  The
 * head structure here may be mapped to a tHFSWIDispatch structure, using
 * a sliding base pointer to select one of the three virtual functions using
 * the constanct offsets \c kHSInterfaceOffsetPut, \c kHSInterfaceOffsetWait
 * and kHSInterfaceOffsetGet.
 *
 * Different function indirections are used to implement function-call
 * polymorphism for the interface objects.
 *
 *  @warning
 *      The first four fields are mimic'd by struct sHSQueue so that
 *      queue handling functions can be applied directly to a
 *      struct sHSInterface
 *
 *  @warning
 *      This structure is used (e.g. as part of a pool) by the Network
 *      Processor, and therefore must be duplicated (at least in terms of
 *      key fields and its overall size) in all other users of the same
 *      Network Processor  (e.g. other host operating systems)
 */
struct sHSInterface
{
    int (*qhandle)(struct sHSMessage*); /**< Active queue handler for ATMOS1 compatibility*/
    void* pTail;                /**< List tail (message or task) */
    S32 control;                /**< Control word (message or task) */
    void* pHead;                /**< List head */
    tHFSWIHandlerPut put;       /**< Handler for put */
    tHFSWIHandlerWait wait;     /**< Handler for wait */
    tHFSWIHandlerGet get;       /**< Handler for get */
    U32 magic;                  /**< kHSTypeInterface */
#ifdef MSGINTERFACE_NEW_VERSION
    void *pad;                  /**< Was object - now implied by interface */
#else
    void *object;           /**< Host independent info */
    //tXSObject object;           /**< Host independent info */
#endif
    enum eHFInterfaceProtocol protocol; /**< The interface protocol */

    /* The following fields are included for compatibility with the
     * ATMOS-1 Process Control Block structure.  They are not used by
     * Quantum, and should be retired when ATMOS-1 is no more than
     * a wistful dream (ATMOS-1 is bound to seem better in retrospect
     * than in reality...)
     */
    void* sys;                          /**< System pointer */
    void* lang;                         /**< Language specific pointer */
    void* socket_handler;               /**< Sockets kludge pointer */
    void* user;                         /**< User pointer */
    int* errnop;                        /**< errno pointer */
    struct _iobuf *openlist;            /**< List of open files */
    struct atmos_env *envptr;           /**< List of environment variables */
#ifdef MSGINTERFACE_NEW_VERSION
    struct sXSInterface txif;           /**< Transmit interface representaiton */
#endif
};

#define kHSInterfaceInvalid     ((tHSInterface)0)   /**< The invalid interface handle */

#ifdef __cplusplus
extern "C" {
#endif

/* Exported APIs for local use within the HF module.
 * REVIEW: these should really be in a local header.
 */
extern tASErr lx_InterfaceInit(tHSInterface interface, void *obj, enum eHFInterfaceProtocol protocol);
extern tASErr lx_InterfaceConfigure(tHSInterface interface, enum eHFInterfaceProtocol protocol);
extern tASErr hf_InterfaceSetWaitingTask(tHSInterface interface, tASTask task);


/* Prototypes.
 */
#define _LINKAGE_hs_InterfaceLibraryInitialise  EXTERN
#define _LINKAGE_hs_InterfaceOpen               EXTERN
#define _LINKAGE_hs_InterfaceClose              EXTERN
#define _LINKAGE_hs_InterfaceNew                EXTERN
#define _LINKAGE_hs_InterfaceDelete             EXTERN
    
extern tASErr hs_InterfaceLibraryInitialise(void);
extern tASErr hs_InterfaceOpen(tHSInterface* pInterface, const char* pszName, tASTask owner);
extern tASErr hs_InterfaceClose(tHSInterface interface, tASTask owner);
extern tASErr hs_InterfaceNew(tHSInterface* pInterface, const char* pszName, const char* pszDefaults, tASTask owner);
extern tASErr hs_InterfaceDelete(tHSInterface interface, tASTask caller);

extern U32 hs_InterfaceMessageCount(tHSInterface interface);
extern U32 hs_InterfaceTaskCount(tHSInterface interface);

/* Note: these functions belong in this header not the hs_message header */
/*       they should be named after Interfaces, not Messages             */
    
#define _LINKAGE_hs_InterfacePut        _LINKAGE_hs_MessagePut
#define _LINKAGE_hs_InterfaceGet        _LINKAGE_hs_MessageGet
#define _LINKAGE_hs_InterfaceWait       _LINKAGE_hs_MessageWait
    
#define hs_InterfacePut      hs_MessagePut      
#define hs_InterfaceGet      hs_MessageGet      
#define hs_InterfaceWait     hs_MessageWait

#ifndef MSGINTERFACE_NEW_VERSION

#define _LINKAGE_hs_MessageSend                 EXTERN
#define _LINKAGE_hs_MessageReply                EXTERN
#define _LINKAGE_hs_MessageSendWait             EXTERN
#define _LINKAGE_hs_MessageFree                 EXTERN
#define _LINKAGE_hs_MessagePass                 EXTERN

extern tASErr hs_MessageSend(tHSInterface target, tHSMessage message,
                             tHSInterface reply);
extern tASErr hs_MessageReply(tHSMessage message);
extern tHSMessage hs_MessageSendWait(tHSInterface target, tHSMessage message);
extern void hs_MessageFree(tHSMessage message);
extern tASErr hs_MessagePass(tHSInterface target, tHSMessage message);

#endif /* MSGINTERFACE_NEW_VERSION */

#define _LINKAGE_hs_MessagePut                  EXTERN
#define _LINKAGE_hs_MessageGet                  EXTERN
#define _LINKAGE_hs_MessageWait                 EXTERN

extern tHSMessage hs_MessageWait(tHSInterface interface);


static inline tHSMessage hs_MessageGet(tHSInterface interface)
{
  return (tHSMessage)(*interface->get)( interface );
}

static inline tASErr hs_MessagePut(tHSInterface interface, tHSMessage _message)
{
  return (*interface->put)( interface, _message);
}

    
#define _LINKAGE_hs_InterfaceTx                 INLINE
#define _LINKAGE_hs_InterfacePrepend            EXTERN
#define _LINKAGE_hs_InterfaceObject             INLINE
#define _LINKAGE_hs_InterfaceSetObject          INLINE
#define _LINKAGE_hs_InterfaceQueue              INLINE

/**
 * Return interface's associated transmit interface
 *
 *  @param      iface       The interface handle
 *  @return                 The interface's object
 */
#ifndef DOXYGEN_PARSE
QUANTUM_INLINE
#endif
tASInterface hs_InterfaceTx(tHSInterface interface)
{
#ifdef MSGINTERFACE_NEW_VERSION
    return &interface->txif;
#else
    return interface;
#endif
}

/**
 * Return the interface's object.
 *
 *  @param      iface       The interface handle
 *  @return                 The interface's object
 */
#ifndef DOXYGEN_PARSE
QUANTUM_INLINE
#endif
tXSObject hs_InterfaceObject(tHSInterface interface)
{
#ifdef MSGINTERFACE_NEW_VERSION
     return xs_InterfaceObject(hs_InterfaceTx(interface));
#else
     return interface->object;
#endif
}

/**
 * Set the object representing an interface if unset.
 *
 *  @param  interface       The interface handle.
 *  @param  obj             The object this interface represents
 *  @return                 An error code
 *
 * This function can be used to set a the object that an interface
 * represents.  It also initializes any of the object's underlying handles
 * if interfaces are represented by an aggregate object type.
 *
 * It can not be used if the interface's object has previously been
 * uninitialized.
 *
 * Possible error codes include
 *   - kASEINVAL:  interface handle or object handle are invalid
 *   - kASEEXIST:  object handle has already been set
 *   - kASEOK      update was successful
 */
#ifndef DOXYGEN_PARSE
QUANTUM_INLINE
#endif
tASErr hs_InterfaceSetObject(tHSInterface interface, tXSObject object)
{   if (interface == kHSInterfaceInvalid || object == kXSObjectInvalid)
        return kASEINVAL;
#ifdef MSGINTERFACE_NEW_VERSION
    {   (void)xs_ObjectSetTypeHandle(object, kASTypeInterface,
                                    hs_InterfaceTx(interface));
        return xs_InterfaceSetObject(hs_InterfaceTx(interface), object);
    }
#else
    else if (interface->object != kXSObjectInvalid)
        return kASEEXIST;
    else
    {   interface->object = object;
        return kASEOK;
    }
#endif
}

/**
 * Cast an interface to a message queue
 *
 *  @param  interface       The interface handle.
 *  @return                 A pointer to a queue or kHSQueueInvalid
 *
 * This function can be used to provide direct access to a queue of messages
 * awaiting processing at an interface.  Not all types of interface, however,
 * have an underlying queue and kHSQueueInvalid is returned in these cases.
 *
 * The queue of messages is also invalid if its length (returned using
 * hs_QueueLength() is negative).  Also, validity checks on the queue returned
 * may not be successful.
 *
 * Note that interface queues are updated asynchronously.  Almost any
 * manipulation of this queue will require the use of critical sections
 * provided, e.g. by hs_InterruptGlobalCritical().
 *
 * Not all Quantum host ports can provide this function.
 */
#ifndef DOXYGEN_PARSE
QUANTUM_INLINE
#endif
tHSQueue *hs_InterfaceQueue(tHSInterface interface)
{   return (tHSQueue *)interface;
}

#ifdef __cplusplus
}
#endif

#endif  /* defined HF_ASSEMBLER_INCLUDE */


#endif  /* defined _QUANTUM_HS_INTERFACE_H */
