/* ###########################################################################
# (c) Copyright Virata Limited 2001
#
# Virata Limited Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Virata Limited ("Virata").  It may only be read, used,
# copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Virata and then subject to the
# terms of that confidentiality agreement and any other applicable agreement
# between you and Virata.  If you are in any doubt as to whether you are
# entitled to access, read, use, copy, adapt, modify or otherwise deal with
# the Software or whether you are entitled to disclose the Software to any
# other person you should contact Virata.  If you have not entered into a
# confidentiality agreement with Virata granting access to this Software you
# should forthwith return all media, copies and printed listings containing
# the Software to Virata.
#
# Virata reserves the right to take legal action against you should you breach
# the above provisions.
#
# If you are unsure, or to report violations, please contact
# support@virata.com
# ##########################################################################*/

#ifndef _QUANTUM_HS_INTERRUPT_H
#define _QUANTUM_HS_INTERRUPT_H

/*  \file   hs_interrupt.h
    \brief  Interrupt support (Helium)
*/

#include "quantum/as_types.h"
#include "quantum/as_stdbool.h"
#include "quantum/as_err.h"

#include "quantum/xs_interrupt.h"

#ifdef __cplusplus
extern "C" {
#endif


#define kHSInterruptVersion     100
/* Prototypes.
 */

#define _LINKAGE_hs_InterruptLibraryInitialise  EXTERN
#define _LINKAGE_hs_InterruptNew                INLINE
#define _LINKAGE_hs_InterruptDelete             INLINE
#define _LINKAGE_hs_InterruptOpen               INLINE
#define _LINKAGE_hs_InterruptClose              INLINE
#define _LINKAGE_hs_InterruptObject             INLINE
    
extern tASErr hs_InterruptLibraryInitialise(void);

#ifndef DOXYGEN_PARSE
QUANTUM_INLINE
#endif
tASErr hs_InterruptNew(tHSInterrupt* pInterrupt, const char* pszName, const char* pszProperties, tASTask owner)
{
    return xs_InterruptNew(pInterrupt, pszName, pszProperties, owner);
}
        
#ifndef DOXYGEN_PARSE
QUANTUM_INLINE
#endif
tASErr hs_InterruptDelete(tHSInterrupt interrupt, tASTask caller)
{   
    return xs_InterruptDelete(interrupt, caller);
}
        
#ifndef DOXYGEN_PARSE
QUANTUM_INLINE
#endif
tASErr hs_InterruptOpen(tHSInterrupt* lv_irq, const char* pszName,
                        tASTask owner)
{
    return xs_InterruptOpen(lv_irq, pszName, owner);
}

#ifndef DOXYGEN_PARSE
QUANTUM_INLINE
#endif
tASErr hs_InterruptClose(tHSInterrupt irq, tASTask owner)
{
    return xs_InterruptClose(irq, owner);
}

#ifndef DOXYGEN_PARSE
QUANTUM_INLINE
#endif
tASObject hs_InterruptObject(tXSInterrupt irq)
{
    return xs_InterruptObject(irq);
}
    
extern tASErr hs_InterruptLibraryInitialise(void);
extern tASErr hs_InterruptNew(tASInterrupt* pInterrupt, const char* pszName, const char* pszProperties, tASTask owner);
extern tASErr hs_InterruptDelete(tASInterrupt interrupt, tASTask caller);


/* Saved thread state needed to leave critical section
 */
typedef U32    tHSCritical;

#define _LINKAGE_hs_InterruptGlobalCritical     EXTERN
#define _LINKAGE_hs_InterruptGlobalEndCritical  EXTERN
#define _LINKAGE_hs_InterruptGlobalInCritical   EXTERN

extern tHSCritical hs_InterruptGlobalCritical(void);
extern void hs_InterruptGlobalEndCritical(tHSCritical state);
extern as_bool hs_InterruptGlobalInCritical(void);
    

#ifdef __cplusplus
}
#endif

#endif  /* defined _QUANTUM_HS_INTERRUPT_H */
