/* ###########################################################################
# (c) Copyright Virata Limited 2001
#
# Virata Limited Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Virata Limited ("Virata").  It may only be read, used,
# copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Virata and then subject to the
# terms of that confidentiality agreement and any other applicable agreement
# between you and Virata.  If you are in any doubt as to whether you are
# entitled to access, read, use, copy, adapt, modify or otherwise deal with
# the Software or whether you are entitled to disclose the Software to any
# other person you should contact Virata.  If you have not entered into a
# confidentiality agreement with Virata granting access to this Software you
# should forthwith return all media, copies and printed listings containing
# the Software to Virata.
#
# Virata reserves the right to take legal action against you should you breach
# the above provisions.
#
# If you are unsure, or to report violations, please contact
# support@virata.com
# ##########################################################################*/


#ifndef _QUANTUM_HS_MEM_H
#define _QUANTUM_HS_MEM_H
#include "quantum/as_types.h"
#include "quantum/as_stdbool.h"
#include "quantum/hs_task.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Functions handled by the memory allocation task.
 * Some functions are no-ops on systems without an MMU, or
 * are simply implemented in-line for efficiency.
 */
    
#define _LINKAGE_hs_MemLibraryInitialise        EXTERN
#define _LINKAGE_hs_MemSharingStart             EXTERN
#define _LINKAGE_hs_MemFree                     EXTERN
#define _LINKAGE_hs_MemAlloc                    EXTERN
#define _LINKAGE_hs_MemRealloc                  EXTERN
#define _LINKAGE_hs_MemAllocConstrained         EXTERN
#define _LINKAGE_hs_MemAllocShared              EXTERN
#define _LINKAGE_hs_MemFreeShared               EXTERN
#define _LINKAGE_hs_MemOpenShared               EXTERN
#define _LINKAGE_hs_MemCloseShared              EXTERN
#define _LINKAGE_hs_MemAllocCommon              EXTERN
#define _LINKAGE_hs_MemFreeCommon               EXTERN
    
extern tASErr hs_MemLibraryInitialise(void);
extern tASErr hs_MemInitialiseShared(void);
extern void  hs_MemFree(const void* vaddr, tASTask task);
extern void* hs_MemAlloc(U32 size, tASTask task);
extern void* hs_MemRealloc(const void* vaddr, U32 size, tASTask task);
extern void* hs_MemAllocConstrained(U32 size, tHSMemPhysical paddrmin,
                                    tHSMemPhysical paddrmax, U32 alignment,
                                    U32 attr, tASTask task);
extern void* hs_MemAllocShared(const char* pszName, U32 size, tASTask task);
extern tASErr hs_MemFreeShared(const char* pszName, tASTask caller);
extern void* hs_MemOpenShared(const char* pszName, tASTask task);
extern void hs_MemCloseShared(const void* vaddr, tASTask task);
extern void* hs_MemAllocCommon(U32 size);
extern void hs_MemFreeCommon(void *addr);


/* Diagnostic functions used to validate memory addresses.
 */
#define _LINKAGE_hs_MemValidate                 EXTERN
#define _LINKAGE_hs_MemValidateAlloc            EXTERN
#define _LINKAGE_hs_MemValidateAligned          EXTERN
#define _LINKAGE_hs_MemInstructionptrValid      EXTERN
#define _LINKAGE_hs_MemInstructionptrLast       EXTERN

extern as_bool hs_MemValidate(const void* vaddr, tASTask task);
extern as_bool hs_MemValidateAlloc(const void* vaddr, tASTask task);
extern as_bool hs_MemValidateAligned(const void* vaddr, tASTask task);

extern as_bool hs_MemInstructionptrValid(void *pc);
extern void* hs_MemInstructionptrLast(void);

#ifdef __cplusplus
}
#endif

#endif  /* _QUANTUM_HS_MEM_H */
