/* ###########################################################################
# (c) Copyright Conexant Systems, Inc 2004
#
# Conexant Systems, Inc Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc ("Conexant"). It may only be read,
# used, copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Conexant and then subject to
# the terms of that confidentiality agreement and any other applicable
# agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a confidentiality agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/


#ifndef _QUANTUM_HS_MEMMAP_H
#define _QUANTUM_HS_MEMMAP_H
#include "config.h"

#ifdef CLAUDIUS_FAMILY
#include <asm/proc-fns.h>
#endif

#include "cache_ops.h"
#include "quantum/as_stdbool.h"
#include "quantum/xs_assert.h"
#include "quantum/hs_task.h"


/** Where uncached mapping starts.
 */
#include "hw_lib.h"

#ifdef LINUX_OVER_SOLOS
/*
 * Add the Linux specific mapping here as well.
 */
#ifdef BF_PP_VADDR_SDRAMU
#define QUANTUM_HW_SDRAMU BF_PP_VADDR_SDRAMU
#endif
#else /* LINUX_OVER_SOLOS */
#ifdef CF_PP_VADDR_SDRAMU
#define QUANTUM_HW_SDRAMU CF_PP_VADDR_SDRAMU
#elif defined(HF_PP_SDRAMU)
#define QUANTUM_HW_SDRAMU HF_PP_SDRAMU
#elif defined(CF_PP_SDRAMU)
#define QUANTUM_HW_SDRAMU CF_PP_SDRAMU
#endif
#endif /* LINUX_OVER_SOLOS */

#ifdef __cplusplus
extern "C" {
#endif


#define _LINKAGE_hs_MemMapPhysical              INLINE
#define _LINKAGE_hs_MemUnmapPhysical            INLINE
#define _LINKAGE_hs_MemFlush                    INLINE
#define _LINKAGE_hs_MemSync                     INLINE
#define _LINKAGE_hs_MemPhysicalToUncached       INLINE
#define _LINKAGE_hs_MemPhysicalToCached         INLINE

#ifdef LINUX_OVER_SOLOS
  /* 
   * Add Linux over Gallus mappings here as well.
   */
#define kHSIMemVbaseCached    BF_PP_VADDR_SDRAMC
#else
#define kHSIMemVbaseCached    0x00000000u
#endif /* LINUX_OVER_SOLOS */

#define kHSIMemVbaseUncached  QUANTUM_HW_SDRAMU

QUANTUM_INLINE tHSMemPhysical hs_MemMapPhysical(const void* vaddr,
                                                tASTask task)
{
    (void) task;
    return (tHSMemPhysical)vaddr;
    /* REVIEW: should this be the uncached address?
       cgg - no it should be a physical address (which are coincident with
             the uncached address range in the Native host port
    */
}

QUANTUM_INLINE void hs_MemUnmapPhysical(const void* vaddr, tASTask task)
{
    /* Nothing */
    (void) task;
    (void) vaddr;
}

QUANTUM_INLINE void hs_MemFlush(const void* vaddr, U32 size, tASTask task)
{
    cache_clean_data_region(vaddr, size );
//    cpu_dcache_clean_range( (unsigned long)vaddr, (unsigned long)vaddr + size);
}

QUANTUM_INLINE void hs_MemSync(void* vaddr, U32 size, tASTask task)
{
    (void) task;
    cache_invalidate_data_region(vaddr, size );
    //   cpu_dcache_invalidate_range( (unsigned long)vaddr, (unsigned long)vaddr + size);
}

/* Diagnostic functions used to validate memory addresses.
 */
#define _LINKAGE_hs_MemPhysicalValidate         INLINE

QUANTUM_INLINE as_bool hs_MemPhysicalValidate(tHSMemPhysical paddr)
{
    (void)paddr;
    return as_true; /*! \todo provide a better implementation */
}


QUANTUM_INLINE void *hs_MemPhysicalToUncached(tHSMemPhysical paddr)
{
    xs_Assert((tHSMemPhysical)0 != paddr);
#ifdef QUANTUM_DEBUG_MEMPHYSICAL
    if (!hs_MemPhysicalValidate(paddr))
        xs_Printf("%C: %08X is not a physical address to uncache\n", paddr);
#endif /* QUANTUM_DEBUG_MEMPHYSICAL */
    xs_Assert(hs_MemPhysicalValidate(paddr));
#ifdef LINUX_OVER_SOLOS
    /*
     * Add Linux over Gallus changes here as well
     * Use the UNCACHED & CACHED macros defined in
     * arm32.hw to perform the mappings in Linux.
     */
    return (void *) UNCACHED(paddr);
#else
    return (void *)((ADR)paddr + kHSIMemVbaseUncached);
#endif /* LINUX_OVER_SOLOS */
}

QUANTUM_INLINE void *hs_MemPhysicalToCached(tHSMemPhysical paddr)
{
    xs_Assert((tHSMemPhysical)0 != paddr);
#ifdef QUANTUM_DEBUG_MEMPHYSICAL
    if (!hs_MemPhysicalValidate(paddr))
        xs_Printf("%C: %08X is not a physical address to cache\n", paddr);
#endif /* QUANTUM_DEBUG_MEMPHYSICAL */
    xs_Assert(hs_MemPhysicalValidate(paddr));
#ifdef LINUX_OVER_SOLOS
    /*
     * Add Linux over Gallus changes here as well
     * Use the UNCACHED & CACHED macros defined in
     * arm32.hw to perform the mappings in Linux.
     */
    return (void *) CACHED(paddr);
#else
    return (void *)((ADR)paddr + kHSIMemVbaseCached);
#endif /* LINUX_OVER_SOLOS */
}



#ifdef __cplusplus
}
#endif

#endif  /* _QUANTUM_HS_MEMMAP_H */
