/* ###########################################################################
# (c) Copyright Conexant Systems, Inc 2001, 2003 - 2004
#
# Conexant Systems, Inc Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc ("Conexant"). It may only be read,
# used, copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Conexant and then subject to
# the terms of that confidentiality agreement and any other applicable
# agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a confidentiality agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/


#ifndef _QUANTUM_HS_MESSAGE_H
#define _QUANTUM_HS_MESSAGE_H

/** @file   hs_message.h
 *  @brief  Message handling (Linux)
 *
 *  @todo
 *      Use inline code for many of the functions here, to eliminate
 *      the function call overhead for most message field accesses.
 */


#ifndef HF_ASSEMBLER_INCLUDE



#ifdef __cplusplus
extern "C" {
#endif
#include "quantum/hs_messagehdr.h"

/* Prototypes.
 */
    
#define _LINKAGE_hs_MessageLength               EXTERN
#define _LINKAGE_hs_MessageSetLength            EXTERN
#define _LINKAGE_hs_MessageMapBytes             EXTERN
#define _LINKAGE_hs_MessageUnmapBytes           EXTERN
#define _LINKAGE_hs_MessageMapAligned           EXTERN
#define _LINKAGE_hs_MessageUnmapAligned         EXTERN
#define _LINKAGE_hs_MessageMapStructure         EXTERN
#define _LINKAGE_hs_MessageUnmapStructure       EXTERN
#define _LINKAGE_hs_MessagePeekU8               EXTERN
#define _LINKAGE_hs_MessagePokeU8               EXTERN
#define _LINKAGE_hs_MessageRead                 EXTERN
#define _LINKAGE_hs_MessageWrite                EXTERN
#define _LINKAGE_hs_MessagePrepend              EXTERN
#define _LINKAGE_hs_MessagePostfix              EXTERN
#define _LINKAGE_hs_MessageInsert               EXTERN
#define _LINKAGE_hs_MessageCut                  EXTERN
#define _LINKAGE_hs_MessageFragmentCount        EXTERN
#define _LINKAGE_hs_MessageFragmentInfo         EXTERN
#define _LINKAGE_hs_MessageFragmentPhysicalInfo EXTERN

extern U32 hs_MessageLength(tHSMessage message);
extern void hs_MessageSetLength(tHSMessage message, U32 length);
extern void* hs_MessageMapBytes(tHSMessage message, U32 offset, U32 length);
extern void hs_MessageUnmapBytes(tHSMessage message, const void* pMappedAddress);
extern void* hs_MessageMapAligned(tHSMessage message, U32 offset, U32 length);
extern void hs_MessageUnmapAligned(tHSMessage message, const void* pMappedAddress);
extern void* hs_MessageMapStructure(tHSMessage message, U32 size);
extern void hs_MessageUnmapStructure(tHSMessage message, const void* pMappedAddress);
extern U8 hs_MessagePeekU8(tHSMessage message, U32 offset);
extern void hs_MessagePokeU8(tHSMessage message, U32 offset, U8 value);
//extern tASErr hs_MessagePrepend(tHSMessage message, void* pSource, U32 length);
extern tASErr hs_MessagePostfix(tHSMessage message, void* pSource, U32 length);
extern tASErr hs_MessageInsert(tHSMessage message, U32 offset, const void* pSource, U32 length);
extern tASErr hs_MessageCut(tHSMessage message, U32 offset, U32 length);
//extern U32 hs_MessageFragmentCount(tASMessage message);
extern int hs_MessageFragmentInfo(tASMessage message, U32 fragment, void** pvaddr, U32* pSize);
extern int hs_MessageFragmentPhysicalInfo(tASMessage message, U32 fragment, void** ppaddr, U32* pSize);

/* REVIEW - implement better versions of these:
 */

#define _LINKAGE_hs_MessagePeekU16      INLINE
#define _LINKAGE_hs_MessagePeekU32      INLINE
#define _LINKAGE_hs_MessagePokeU16      INLINE
#define _LINKAGE_hs_MessagePokeU32      INLINE
#define _LINKAGE_hs_MessageRead6        INLINE
#define _LINKAGE_hs_MessageRead14       INLINE
#define _LINKAGE_hs_MessageRead16       INLINE
#define _LINKAGE_hs_MessageRead20       INLINE
#define _LINKAGE_hs_MessageRead40       INLINE
#define _LINKAGE_hs_MessageWrite6       INLINE
#define _LINKAGE_hs_MessageWrite14      INLINE
#define _LINKAGE_hs_MessageWrite16      INLINE
#define _LINKAGE_hs_MessageWrite20      INLINE
#define _LINKAGE_hs_MessageWrite40      INLINE

QUANTUM_INLINE U16 hs_MessagePeekU16(tHSMessage msg, U32 offset)
{
    U16 data;
    hs_MessageRead( msg, (U8 *)&data, offset, sizeof( data ) );
    return data;
}
QUANTUM_INLINE void hs_MessageRead6(tHSMessage msg, U8* pTarget, U32 offset)
{
    hs_MessageRead( msg, pTarget, offset, 6 );
}
QUANTUM_INLINE void hs_MessageRead14(tHSMessage msg, U8* pTarget, U32 offset)
{
    hs_MessageRead( msg, pTarget, offset, 14 );
}
QUANTUM_INLINE void hs_MessageRead16(tHSMessage msg, U8* pTarget, U32 offset)
{
    hs_MessageRead( msg, pTarget, offset, 16 );
}
QUANTUM_INLINE void hs_MessageRead20(tHSMessage msg, U8* pTarget, U32 offset)
{
    hs_MessageRead( msg, pTarget, offset, 20 );
}
QUANTUM_INLINE void hs_MessageRead40(tHSMessage msg, U8* pTarget, U32 offset)
{
    hs_MessageRead( msg, pTarget, offset, 40 );
}
QUANTUM_INLINE void hs_MessageWrite6(tHSMessage msg, U32 offset, const U8* pSource)
{
    hs_MessageWrite( msg, offset, pSource, 6 );

}
QUANTUM_INLINE void hs_MessageWrite14(tHSMessage msg, U32 offset, const U8* pSource)
{
    hs_MessageWrite( msg, offset, pSource, 14 );
}
QUANTUM_INLINE void hs_MessageWrite16(tHSMessage msg, U32 offset, const U8* pSource)
{
    hs_MessageWrite( msg, offset, pSource, 16 );
}
QUANTUM_INLINE void hs_MessageWrite20(tHSMessage msg, U32 offset, const U8* pSource)
{
    hs_MessageWrite( msg, offset, pSource, 20 );
}
QUANTUM_INLINE void hs_MessageWrite40(tHSMessage msg, U32 offset, const U8* pSource)
{
    hs_MessageWrite( msg, offset, pSource, 40 );
}


#ifdef __cplusplus
}
#endif

#endif  /* HF_ASSEMBLER_INCLUDE */
#endif  /* _QUANTUM_HS_MESSAGE_H */
