/* ###########################################################################
# (c) Copyright Conexant Systems, Inc 2001, 2004
#
# Conexant Systems, Inc Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc ("Conexant"). It may only be read,
# used, copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Conexant and then subject to
# the terms of that confidentiality agreement and any other applicable
# agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a confidentiality agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/

#ifndef _QUANTUM_HS_POOL_H
#define _QUANTUM_HS_POOL_H


/** @file   hs_pool.h
 *  @brief  Message Pool Support (Linux)
 */

/* Version information for the pool module.
 */
#define kHSPoolVersion  (100)                   /**< Implementation version */


#if defined (HF_ASSEMBLER_INCLUDE)

/* --------------------------------------------------------------------------
 *
 *              Assembler specific declarations
 *
 * --------------------------------------------------------------------------
 */

#include "quantum/hs_interface.h"

/* Pool structure for assembly code.
 */
                        macro
$name                   sHSPool
$(name)
$(name).interface       sHSInterface
$(name).object          ds  1
$(name).count           ds  1
$(name).size            ds  1
$(name).prefix          ds  1
$(name).messages        ds  1
$(name).messagesize     ds  1
$(name).name            ds  1
$(name).constructed     dsb 1
$(name).has_messages    dsb 1
$(name).pad1            dsb 1
$(name).pad2            dsb 1
$(name).extramessage    ds  1
$(name).region          ds  1
$(name).guard           ds  1
$(name)_size            equ *-$(name)
                        mend

                dorg    0
tHSPool         sHSPool


#else /* defined HF_ASSEMBLER_INCLUDE */


/* --------------------------------------------------------------------------
 *
 *              C specific declarations
 *
 * --------------------------------------------------------------------------
 */

#include "quantum/as_types.h"
#include "quantum/as_interface.h"
#include "quantum/hs_interface.h"

#ifdef __cplusplus
extern "C" {
#endif


/** @brief      Pool data type
 *
 * This is a subclass of the interface type and may be used
 * with normal interface APIs.
 *
 *  @warning
 *      The structure field names are chosen to match those present
 *      in the original CYANPOOL definition, so that legacy ATMOS-1
 *      code can continue to function unchanged (the main exception
 *      being code which tried to directly access the queue structure
 *      in the pool, which will now be stopped by a compile time error).
 */

struct sHSPool
{
    struct sHSInterface interface;  /**< The public interface for buffer access */
    tXSObject object;               /**< Back reference to the object handle */
    U32 count;                      /**< Number of buffers created */
    U32 size;                       /**< Data size for each buffer */
    U32 prefix;                     /**< Prefix size for each buffer */
    tHSMessage messages;            /**< Ptr to start of array of allocated messages */
    U32 messagesize;                /**< Size of each message (after rounding etc) */
    const char* name;               /**< Pool name */
    U8 constructed;                 /**< Flags if the buffers have been constructed yet */
    U8 has_messages;                /**< CYAN Compatibility: Flags if buffers have messages (always true) */
    U8 pad1;
    U8 pad2;                        /**< Reserved */
    
    U32 extramessage;               /**< Size of area following Quantum message (e.g. for ATMOS_MESSAGE data[]) */
    tXSMemRegion region;            /**< Unused at the moment. */
    U32 guard;                      /**< CYAN Guard word */
};

/* These are used to provide cyan-usable field names for the pool       */
/* data structure when used as a cyan pool                              */
#define hs_cyan_pool_size       size
#define hs_cyan_pool_prefix     prefix
#define hs_cyan_pool_count      count

/* Prototypes.
 */

#define _LINKAGE_hs_PoolLibraryInitialise       EXTERN
#define _LINKAGE_hs_PoolNew                     EXTERN
#define _LINKAGE_hs_PoolDelete                  EXTERN
#define _LINKAGE_hs_PoolOpen                    EXTERN
#define _LINKAGE_hs_PoolClose                   EXTERN
#define _LINKAGE_hs_PoolInterface               INLINE
#define _LINKAGE_hs_PoolObject                  INLINE
    
extern tASErr hs_PoolLibraryInitialise(void);
extern tASErr hs_PoolNew(tHSPool* pPool, const char* pszName, const char* pszProperties, tASTask owner);
extern tASErr hs_PoolDelete(tHSPool pool, tASTask caller);
extern tASErr hs_PoolOpen(tHSPool* pPool, const char* pszName, tASTask caller);
extern tASErr hs_PoolClose(tHSPool pool, tASTask caller);

#ifndef DOXYGEN_PARSE
QUANTUM_INLINE
#endif
tHSInterface hs_PoolInterface(tHSPool pool)
{ return &pool->interface; }

#ifndef DOXYGEN_PARSE
QUANTUM_INLINE
#endif
tXSObject hs_PoolObject(tHSPool pool)
{ return hs_InterfaceObject(hs_PoolInterface(pool)); }


/* Functions referred to by hs_provides.h */

extern tHSPool hs_TaskPoolGeneric(void);
extern tHSPool hs_TaskPoolData(void);
extern tHSPool hs_TaskPoolControl(void);
    
#define _LINKAGE_hs_MessageAllocGet    EXTERN
#define _LINKAGE_hs_MessageAllocGetControl EXTERN
#define _LINKAGE_hs_MessageAllocGetData    EXTERN

extern tASMessage hs_MessageAllocGet(void);
extern tASMessage hs_MessageAllocGetControl(void);
extern tASMessage hs_MessageAllocGetData(void);

#ifdef __cplusplus
}
#endif

#endif  /* HF_ASSEMBLER_INCLUDE */
#endif  /* defined _QUANTUM_HS_POOL_H */
