/* ###########################################################################
# (c) Copyright GlobespanVirata Limited 2001, 2003
#
# GlobespanVirata Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to GlobespanVirata Inc. ("GlobespanVirata").  It may only be 
# read, used, copied, adapted, modified or otherwise dealt with by you if you 
# have entered into a confidentiality agreement with GlobespanVirata and then 
# subject to the terms of that confidentiality agreement and any other 
# applicable agreement between you and GlobespanVirata.  If you are in any 
# doubt as to whether you are entitled to access, read, use, copy, adapt, 
# modify or otherwise deal with the Software or whether you are entitled to 
# disclose the Software to any other person you should contact
# GlobespanVirata. If you have not entered into a confidentiality agreement
# with GlobespanVirata granting access to this Software you should forthwith
# return all media, copies and printed listings containing the Software to
# GlobespanVirata.
#
# GlobespanVirata reserves the right to take legal action against you should
# you breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# support@globespanvirata.com
# ##########################################################################*/

#ifndef _QUANTUM_HS_SEMAPHORE_H
#define _QUANTUM_HS_SEMAPHORE_H


/**
 * Semaphore handling (Linux).
 *
 *  @file       hs_semaphore.h
 *  @ingroup    synchronisation
 */

#ifdef __cplusplus
extern "C" {
#endif

/** Version code for the semaphore implementation.
 */
#define kHSSemaphoreVersion     (100)


/* --------------------------------------------------------------------------
 *
 *              C specific declarations
 *
 * --------------------------------------------------------------------------
 */

#include "quantum/as_types.h"
#include "quantum/xs_object.h"


/**
 * The semaphore object structure is hidden in lx_semaphore.h in hosts/linux.
 *
 */


/* Exported APIs.
 */
#define kHSSemaphoreInvalid     ((tHSSemaphore)0)   /**< The invalid semaphore handle */

/* Prototypes.
 */
    
#define _LINKAGE_hs_SemaphoreLibraryInitialise	EXTERN
#define _LINKAGE_hs_SemaphoreOpen		EXTERN
#define _LINKAGE_hs_SemaphoreClose		EXTERN
#define _LINKAGE_hs_SemaphoreNew		EXTERN
#define _LINKAGE_hs_SemaphoreDelete		EXTERN
#define _LINKAGE_hs_SemaphorePost		EXTERN
#define _LINKAGE_hs_SemaphoreWait		EXTERN
#define _LINKAGE_hs_SemaphoreTry		EXTERN
#define _LINKAGE_hs_SemaphoreObject		EXTERN

extern tASErr hs_SemaphoreLibraryInitialise(void);
extern tASErr hs_SemaphoreOpen(tHSSemaphore* pSemaphore, const char* pszName, tASTask owner);
extern tASErr hs_SemaphoreClose(tHSSemaphore semaphore, tASTask owner);
extern tASErr hs_SemaphoreNew(tHSSemaphore* pSemaphore, const char* pszName, const char* pszDefaults, tASTask owner);
extern tASErr hs_SemaphoreDelete(tHSSemaphore semaphore, tASTask caller);
extern void hs_SemaphorePost(tHSSemaphore semaphore);
extern void hs_SemaphoreWait(tHSSemaphore semaphore);
extern tASErr hs_SemaphoreTry(tHSSemaphore semaphore);

/**
 * Find the object representing this semaphore
 *  @internal
 *  @param      semaphore       The sempahore
 *  @return                     The semaphore's object
 *
 * The sempahore records a back pointer to the object used to represent it
 */
extern void *hs_SemaphoreObject(tHSSemaphore semaphore);


#ifdef __cplusplus
}
#endif

#endif  /* defined _QUANTUM_HS_SEMAPHORE_H */
