/* ###########################################################################
# (c) Copyright GlobespanVirata Limited 2001, 2003
#
# GlobespanVirata Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to GlobespanVirata Inc. ("GlobespanVirata").  It may only be 
# read, used, copied, adapted, modified or otherwise dealt with by you if you 
# have entered into a confidentiality agreement with GlobespanVirata and then 
# subject to the terms of that confidentiality agreement and any other 
# applicable agreement between you and GlobespanVirata.  If you are in any 
# doubt as to whether you are entitled to access, read, use, copy, adapt, 
# modify or otherwise deal with the Software or whether you are entitled to 
# disclose the Software to any other person you should contact
# GlobespanVirata. If you have not entered into a confidentiality agreement
# with GlobespanVirata granting access to this Software you should forthwith
# return all media, copies and printed listings containing the Software to
# GlobespanVirata.
#
# GlobespanVirata reserves the right to take legal action against you should
# you breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# support@globespanvirata.com
# ##########################################################################*/


#ifndef _QUANTUM_HS_TIMER_H
#define _QUANTUM_HS_TIMER_H

/** @file       hs_timer.h
 *  @brief      Timer support functions.
 *  @ingroup    util
 *
 * The timer APIs allow application code to determine the current
 * system time, pause execution for a short period, or issue messages
 * that are delivered after a specified delay.
 */
#include "quantum/hs_interface.h"
#include "quantum/hs_message.h"

#include "config.h"

#ifdef __cplusplus
extern "C" {
#endif

#define _LINKAGE_hs_TimerStart			EXTERN

/* Type of function returned by this Host O/S port to handle the passage of
   a single tick */
typedef void (fTickCallback)(void);

/* Host O/S Timer driver initialization function */    
extern fTickCallback *hs_TimerStart(void);

/* Timer state block for delay function.
 */
typedef struct sHSTimerDelay
{
    tHSMessage message;                 /**< Message associated with the request */
    struct sHFTimerRingEntry* entry;    /**< Reference to the ring the message was left at */
} tHSTimerDelay;

/* Direct mapping of hardware timer functions.
 */
#include "hw_timer.h"

#define kHSTimerUsPerTick (1000) /* REVIEW: Is this relevant? */
    
/* Exported Prototypes.
 */

#define _LINKAGE_hs_TimerLibraryInitialise	EXTERN
#define _LINKAGE_hs_TimerTicks			EXTERN
#define _LINKAGE_hs_TimerSeconds		EXTERN
#define _LINKAGE_hs_TimerClock			EXTERN
#define _LINKAGE_hs_TimerHardware		INLINE
#define _LINKAGE_hs_TimerHardwareResolution	INLINE
#define _LINKAGE_hs_TimerWait			EXTERN
#define _LINKAGE_hs_TimerWaitUs			EXTERN
#define _LINKAGE_hs_TimerDelay			EXTERN
#define _LINKAGE_hs_TimerDelayCancel	        EXTERN
#define _LINKAGE_hs_TimerExpire			EXTERN
#define _LINKAGE_hs_TimerDelayUs                INLINE


extern tASErr hs_TimerLibraryInitialise(void);
extern U32 hs_TimerSeconds(void);
/* returns the current value of the system clock and a return code of 0,
   or -1 if it is unavailable */
extern tASErr hs_TimerClock(struct hs_timeval *tv);
extern U32 hs_TimerHardware(void);
extern U32 hs_TimerHardwareResolution(void);
extern void hs_TimerWait(U32 delay_ms);
extern tASErr hs_TimerDelay(tHSTimerDelay* state, tHSMessage message, tHSInterface dest, U32 delay_ms);
extern tASErr hs_TimerDelayCancel(tHSTimerDelay* state);
extern void hs_TimerExpire(tHSTimerDelay* state);

//#include "linux/delay.h"
#include "cnxt_hsl.h"
static inline void hs_TimerDelayUs( U32 us ) { as_udelay( us ); }

#ifdef __cplusplus
}
#endif

#endif  /* _QUANTUM_HS_TIMER_H */

