/* ###########################################################################
# (c) Copyright GlobespanVirata Inc. 2001, 2003
#
# GlobespanVirata Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to GlobespanVirata Inc. ("GlobespanVirata").  It may only be
# read, used, copied, adapted, modified or otherwise dealt with by you if you
# have entered into a confidentiality agreement with GlobespanVirata and then
# subject to the terms of that confidentiality agreement and any other
# applicable agreement between you and GlobespanVirata. If you are in any
# doubt as to whether you are entitled to access, read, use, copy, adapt,
# modify or otherwise deal with the Software or whether you are entitled to
# disclose the Software to any other person you should contact GlobespanVirata.
# If you have not entered into a confidentiality agreement with GlobespanVirata
# granting access to this Software you should forthwith return all media,
# copies and printed listings containing the Software to GlobespanVirata.
#
# GlobespanVirata reserves the right to take legal action against you should
# you breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# support@globespanvirata.com
# ##########################################################################*/


#ifndef _QUANTUM_HS_TYPES_H
#define _QUANTUM_HS_TYPES_H

#define QUANTUM_HOST_IS_LINUX

//#include <asm/byteorder.h>

#ifndef _SIZE_T
#define _SIZE_T
typedef unsigned long size_t;
#endif /*  _SIZE_T */
//#include <linux/types.h>


/** @file       hs_types.h
 *  @brief      Helium specific type definitions
 *  @ingroup    platform
 *
 * This file defines the portable types for the Helium
 * linux platform.
 *  
 *  @note
 *      This file is implicitly included by application
 *      code through the as_types.h header.
 *  
 *  @note
 *      This particular implementation may also be included
 *      from assembly code to get the structure type magic
 *      numbers (handy for assert() checking in the assembler).
 */

#if !defined (HF_ASSEMBLER_INCLUDE)

/* C/C++ Definitions for GCC/ARM.
 */
//#define HS_TYPES_U8  __u8     /**< Unsigned 8 bit integer */
//#define HS_TYPES_S8  __s8     /**< Signed 8 bit integer */
//#define HS_TYPES_U16 __u16    /**< Unsigned 16 bit integer */
//#define HS_TYPES_S16 __s16    /**< Signed 16 bit integer */
//#define HS_TYPES_U32 __u32    /**< Unsigned 32 bit integer */
//#define HS_TYPES_S32 __s32    /**< Signed 32 bit integer */
//#define HS_TYPES_U64 __u64    /**< Unsigned 64 bit integer */
//#define HS_TYPES_S64 __s64    /**< Signed 64 bit integer */

#define HS_TYPES_U8  unsigned char     	/**< Unsigned 8 bit integer */
#define HS_TYPES_S8  char     			/**< Signed 8 bit integer */
#define HS_TYPES_U16 unsigned short 	/**< Unsigned 16 bit integer */
#define HS_TYPES_S16 short    			/**< Signed 16 bit integer */
#define HS_TYPES_U32 unsigned int		/**< Unsigned 32 bit integer */
#define HS_TYPES_S32 int			    /**< Signed 32 bit integer */
#define HS_TYPES_U64 unsigned long long /**< Unsigned 64 bit integer */
#define HS_TYPES_S64 long long		    /**< Signed 64 bit integer */
#define HS_TYPES_ADR unsigned int       /**< Smallest integer that can contain a \c void* with no information loss */

#ifndef NULL
#define NULL ((void *)0)
#endif

typedef HS_TYPES_ADR tHSMemPhysical; /**< A Physical address */

/* Map the host specific object handles to structure pointers (all
 * Helium family devices use a flat address space at present).
 */
typedef struct sHSTask*      tHSTask;               /**< Task handle (opaque type) */
typedef struct sHSMutex*     tHSMutex;              /**< Mutex handle (opaque type) */
typedef struct sHSSemaphore* tHSSemaphore;          /**< Semaphore handle (opaque type) */
typedef struct sHSPool*      tHSPool;               /**< Pool handle (opaque type) */
typedef struct sHSMessage*   tHSMessage;            /**< Message handle (opaque type) */
typedef struct sHSInterface* tHSInterface;          /**< Interface handle (opaque type) */
typedef struct sXSInterrupt* tHSInterrupt;          /**< Hardware interfaces (opaque type) */
typedef struct sHSQueue      tHSQueue;              /**< Local message queues (opaque type) */
typedef        void*         tHSMonitoredPool;      /**< Monitored pool handle (opaque type) - unimplemented */

#define kHSTaskInvalid      ((tHSTask)0)            /**< The invalid task handle */
#define kHSPoolInvalid      ((tHSPool)0)            /**< The invalid pool handle */
#define kHSMutexInvalid     ((tHSMutex)0)           /**< The invalid mutex handle */
#define kHSInterfaceInvalid ((tHSInterface)0)       /**< The invalid interface handle */
#define kHSSemaphoreInvalid ((tHSSemaphore)0)       /**< The invalid semaphore handle */
#define kHSMessageInvalid   ((tHSMessage)0)         /**< The invalid message handle */
#define kHSInterruptInvalid ((tHSInterrupt)0)       /**< The invalid hardware handle */
#define kHSMonitoredPoolInvalid ((tHSMonitoredPool)0)            /**< The invalid monitored pool handle */

#endif  /* HF_ASSEMBLER_INCLUDE */

/* To support some POSIX-style time functions:
 * normally found in sys/time.h
 */
struct hs_timeval {
    HS_TYPES_U32    tv_sec;         /* seconds */
    HS_TYPES_U32    tv_usec;        /* microseconds */
};

/* C/C++/Assembler definitions.
 * This section is available to assembler code and may only use
 * pre-processor directives.
 */
#define kHSTypeInterface    AS_TYPE('H', 'S', 'I', 'N') /**< Identifies tHSInterface */
#define kHSTypeMessage      AS_TYPE('H', 'S', 'M', 'E') /**< Identifies tHSMessage */
#define kHSTypeSemaphore    AS_TYPE('H', 'S', 'S', 'E') /**< Identifies tHSSemaphore */
#define kHSTypeTask         AS_TYPE('H', 'S', 'T', 'A') /**< Identifies tASTask */
#define kHSTypePool         AS_TYPE('H', 'S', 'P', 'O') /**< Identifies tHSPool */
#define kHSTypeInterrupt    AS_TYPE('H', 'S', 'I', 'R') /**< Identifies tHSInterrupt */



/* Endian invariance.
 */
#ifdef __LITTLE_ENDIAN
#define HS_ENDIAN_LITTLE        (1)     /**< This is a little-endian processor */
#define HS_ENDIAN_BIG           (0)     /**< This is not a big-endian processor */
#else
#define HS_ENDIAN_LITTLE        (0)     /**< This is not a little-endian processor */
#define HS_ENDIAN_BIG           (1)     /**< This is a big-endian processor */
#endif

#define HS_HTOB_U16(_n)         __cpu_to_be16(_n)
#define HS_HTOB_U32(_n)         __cpu_to_be32(_n)
#define HS_HTOL_U16(_n)         __cpu_to_le16(_n)
#define HS_HTOL_U32(_n)         __cpu_to_le32(_n)
#define HS_BTOH_U16(_n)         __be16_to_cpu(_n)
#define HS_NTOH_U32(_n)         __be32_to_cpu(_n)
#define HS_LTOH_U16(_n)         __le16_to_cpu(_n)
#define HS_LTOH_U32(_n)         __le32_to_cpu(_n)
#define HS_LTOH_U64(_n)         __le64_to_cpu(_n)
#define HS_HTOB_U64(_n)         __cpu_to_be64(_n)
#define HS_BTOH_U32(_n)         __be32_to_cpu(_n)
#define HS_HTOL_U64(_n)         __cpu_to_le64(_n)
#define HS_BTOH_U64(_n)         __be64_to_cpu(_n)



#endif  /* defined _QUANTUM_HS_TYPES_H */
