/*
 * linux/arch/arm/mach-cx2417x/arch.c
 *
 * Author: Guido Barzini
 * Copyright (C) 2005 Conexant inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */
#include <linux/config.h>
#include <linux/init.h>
#include <linux/timex.h>
#include <linux/interrupt.h>
#include <linux/module.h>
#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/mach/map.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/time.h>
#include <asm/mach/irq.h>
#include "board.h"

static struct map_desc cx2417x_io_desc[] __initdata = {
	/* The core peripherals */
	{ASX0_BASE_VIRT, ASX0_BASE_PHYS, ASX0_SIZE, MT_DEVICE}
};

/* Create the static I/O mappings */
static void __init cx2417x_map_io(void)
{
	iotable_init(cx2417x_io_desc, ARRAY_SIZE(cx2417x_io_desc));
        /* Right, the I/O world is mapped in, now do any board-specific start-of-day */
        cx2417x_board_init();
}

static irqreturn_t cx2417x_timer_interrupt(int irq, void *dev_id,
					 struct pt_regs *regs)
{
	/* Was this our timer ? This is a nasty bodge. */
	if ( readl(TIMER_BASE + TIMER_IRQS) & 1 ) {
		/* Is there some fixup needed here? */
		timer_tick(regs);
	}
	return IRQ_HANDLED;
}

static struct irqaction cx2417x_timer2_irq = {
	.name = "System Timer",
	.flags = SA_INTERRUPT,
	.handler = cx2417x_timer_interrupt
};

/* Set up timer 0 to generate system timer interrupt */
static void __init cx2417x_init_time(void)
{
	u32 ctl;
	setup_irq(IRQ_TIMERS, &cx2417x_timer2_irq);
	/* LATCH is the value calculated by the system from CLOCK_TICK_RATE */
	writel( LATCH, TIMER_LIMIT_0 );
	/* Clear the timer */
	writel(0, TIMER_VALUE_0 );
	/* Set timer 0 up for a 2MHz timer */
	writel( TIMER_BASE_FREQ / CLOCK_TICK_RATE, TIMER_TIMEBASE_0 );
	/* Enable interrupts, set reload mode and start it */
	ctl = TIMER_INT_EN  | TIMER_RESET_CTR | TIMER_ENABLE;
	writel(ctl, TIMER_CTRL_0 );
}

/* Review: this will clear the IRQ pending bit. Need to be sure all the
 * drivers get the information from their own blocks rather than from
 * here */
static void ack_cx2417x_irq_grp1(unsigned int irq)
{
	writel( (1 << irq), IRQ_STATCLR_GRP1);
}

static void mask_cx2417x_irq_grp1(unsigned int irq)
{
	u32 mask = readl( IRQ_EN_GRP1 );
	mask &= ~(1 << irq);
	writel( mask, IRQ_EN_GRP1);
}

static void unmask_cx2417x_irq_grp1(unsigned int irq)
{
	u32 mask = readl( IRQ_EN_GRP1 );
	mask |= (1 << irq);
	writel( mask, IRQ_EN_GRP1);
}

static struct irqchip cx2417x_irq1 = {
	.ack = ack_cx2417x_irq_grp1,
	.mask = mask_cx2417x_irq_grp1,
	.unmask = unmask_cx2417x_irq_grp1,
	.retrigger = NULL,
	.type = NULL
};

/* And the same again for the 2nd interrupt controller */
static void ack_cx2417x_irq_grp2(unsigned int irq)
{
	writel( (1<<irq), IRQ_STATCLR_GRP2);
}

static void mask_cx2417x_irq_grp2(unsigned int irq)
{
	u32 mask = readl(IRQ_EN_GRP2);
	mask &= ~(1 << irq);
	writel( mask,IRQ_EN_GRP2);
}

static void unmask_cx2417x_irq_grp2(unsigned int irq)
{
	u32 mask = readl(IRQ_EN_GRP2 );
	mask |= (1 << irq);
	writel( mask, IRQ_EN_GRP2);
}

static struct irqchip cx2417x_irq2 = {
	.ack = ack_cx2417x_irq_grp2,
	.mask = mask_cx2417x_irq_grp2,
	.unmask = unmask_cx2417x_irq_grp2,
	.retrigger = NULL,
	.type = NULL
};




/* Set up all the other IRQs */
static void __init cx2417x_init_irq(void)
{
	int i;
	/* Set up all of group 1 */
	for (i = 0; i < NR_IRQS_GRP1; i++) {
		set_irq_chip(i, &cx2417x_irq1);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID | IRQF_PROBE);
	}
	/* Set up all of group 2 */
	for (i = NR_IRQS_GRP1; i < NR_IRQS; i++) {
		set_irq_chip(i, &cx2417x_irq2);
		set_irq_handler(i, do_level_IRQ);
		set_irq_flags(i, IRQF_VALID | IRQF_PROBE);
	}
	/* By default they're all IRQ rather than FIQ */
	writel( (u32)-1, IRQ_DST_GRP1 );
	writel( (u32)-1, IRQ_DST_GRP2 );
}



MACHINE_START(CX2417X, "Conexant CX2417X (Trinity)")
    MAINTAINER("Conexant inc.")
    MAPIO(cx2417x_map_io)
    INITIRQ(cx2417x_init_irq)
    INITTIME(cx2417x_init_time)
    BOOT_MEM(0, ASX0_BASE_PHYS, ASX0_BASE_VIRT)
    BOOT_PARAMS(0x04)
MACHINE_END
