/*
 * drivers/net/phy/ip175c.c
 *
 * Driver for IP175 PHYs
 * (c) Copyright Conexant System 
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 */
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/unistd.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/spinlock.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/mii.h>
#include <linux/ethtool.h>
#include <linux/phy.h>

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <linux/ethtool.h>
#include<linux/if_vlan.h>
#include "ip175c.h"

MODULE_DESCRIPTION("IP175 PHY driver");
MODULE_AUTHOR("Nitin Sehgal");
MODULE_LICENSE("GPL");

static int init=0;
int ip175_setup_forced(struct phy_device *phydev)
{
    short int ctl=phy_read(phydev,MII_BMCR,0); 
    phydev->pause = phydev->asym_pause = 0;


	/* Nitin to force disable autoneg */
	ctl &=~BMCR_ANENABLE;  	

	 
	 if (100 == phydev->speed)
        ctl |= BMCR_SPEED100;
    else if (10 == phydev->speed)
        ctl &= ~BMCR_SPEED100;

	
    if (DUPLEX_FULL == phydev->duplex)
        ctl |= BMCR_FULLDPLX;
	else
        ctl &= ~BMCR_FULLDPLX;

    ctl = phy_write(phydev, MII_BMCR, ctl,0);

    ctl=phy_read(phydev,MII_BMCR,0); 

    if (ctl < 0)
        return ctl;

    /* We just reset the device, so we'd better configure any
     * settings the PHY requires to operate */
    if (phydev->drv->config_init && !init)
        ctl = phydev->drv->config_init(phydev);

    init=1;

 return ctl;
}
/* Solos WV has five ports while solosW has 4 */
#ifdef CONFIG_ARCH_SOLOS_208PIN
	#define MAX_PORTS 4
#else
	#define MAX_PORTS 5
#endif

static int ip175_init_vlan(struct phy_device *phydev,struct ethtool_cmd *cmd)
{
	int i=0;
	int bit_val=0;
	int loop=0;
	unsigned int PVID_ARRAY[MAX_PORTS];
    u32 reg = 0;
    u32 vlanmask = 0, vlanId = 0;
		for(i=0;i<cmd->swtchInfo.NVlans;i++)
		{
			unsigned short mask = cmd->swtchInfo.VMask[i];
			unsigned short pvid= ( cmd->swtchInfo.VMask[i] >> 16 ); 
			bit_val=1;
			loop=0;
			for(loop=0;loop<MAX_PORTS;loop++)
			{
				if(mask & bit_val)
					PVID_ARRAY[loop]=pvid;
				
				 bit_val=bit_val *2;
			}
		}		

		//Nitin
	    if( cmd->swtchInfo.vlanNeeded == 1)
   		{
		      /***********************************************
        	   **
        	   ** Configure PVIDs
        	   **
        	   ***********************************************/
		       /* Add tags to packets going out from port5(to Proc) and Remove tags from packets going out from ports 0-4 */
/* No need to enable VLAN tag insertion or removal 
we will use special tag to identify the source and destination port */

#if NITIN
	        reg = IP175_TAG_CTL_ADD_P5_OUT_PKT_TAGS | IP175_TAG_CTL_REM_P04_OUT_PKT_TAGS;
			/* phydev,register,val,phy address */
			phy_write(phydev,IP175_TAG_CTL,reg,IP175_TAG_CTL_PHY);

    	    /* Now register 29.24-27, to setup PVID */
			for(loop=0;loop<MAX_PORTS;loop++)
        	{
            	// Check if PVID has to be configured for this port
            	if(PVID_ARRAY[loop] !=0)
            	{
					/* phydev,register,val,phy address */
					phy_write(phydev,IP175_TAG_P0 + loop,PVID_ARRAY[loop],IP175_TAG_P0_PHY);
            	}
        	}

		//#ifdef GALLUS_WLC
				/* phydev,register,val,phy address */
				phy_write(phydev,IP175_TAG_P5,PVID_ARRAY[ MAX_PORTS - 1 ],IP175_TAG_P5_PHY);
		//#endif

#endif
 			/***********************************************
        	**
        	** Configure VLAN Members
        	**
        	***********************************************/
         	/* VLAN Membership masks are for 4 + 1 ports, here we have 5 +1 ports, make changes */
/* VLAn membership is required to provide port splitting */
        	for (i = 0; i< cmd->swtchInfo.NVlans; i++)
        	{
            	// Get VLAN ID from VLAN Mask
            	vlanId = (cmd->swtchInfo.VMask[i] >> 16);
            	vlanmask = ((cmd->swtchInfo.VMask[i] & 0x000000FF) | 0x20);
            	// Configure VLAN
            	if( vlanId & 0x01)  // odd
            	{
                	/* odd register address = (vlanId +1)/2 */
					reg = phy_read(phydev,(vlanId +1) >> 1,IP175_TAG_VLAN_01_PHY);
                	reg &= IP175_VLAN_MASK_RIGHT;
                	reg |= (vlanmask << 8);
					/* phydev,register,val,phy address */
					phy_write(phydev,(vlanId +1) >> 1,reg,IP175_TAG_VLAN_01_PHY);
            	}
            	else
            	{
                	/* even register address = vlanId/2 + 1 */
                	/*  Port 0,2 .... , 5 in VLAN 2 (PVID of Port 0) */
					reg = phy_read(phydev,(vlanId >> 1) + 1,IP175_TAG_VLAN_01_PHY);
                	reg &= IP175_VLAN_MASK_LEFT;
                	reg |= vlanmask;
					/* phydev,register,val,phy address */
					phy_write(phydev,(vlanId >> 1) + 1,reg,IP175_TAG_VLAN_01_PHY);

            	}

           }

  		/***********************************************
    	**
    	** Configure VLAN Enable Registers
    	**
    	***********************************************/


    	/* Program Smart MAC control */
		reg = phy_read(phydev,IP175_ROUTER_CTL,IP175_ROUTER_CTL_PHY);
        reg &= ~( IP175_ROUTER_CTL_WAN_MASK );
        reg |= ( IP175_RTR_CTL_WAN_PORTS << IP175_ROUTER_CTL_WAN_SHIFT );
    	reg |= IP175_ROUTER_CTL_TAG_VLAN;
    	reg &= ~( IP175_ROUTER_CTL_VID_MASK );
    	reg |= IP175_ROUTER_CTL_ENABLE;
    	reg &= ~( IP175_ROUTER_CTL_LAN_MASK );
        reg |= (  IP175_RTR_CTL_NUM_VLANS );    // 1 VLANs (2 for WLC)
		/* phydev,register,val,phy address */
		phy_write(phydev,IP175_ROUTER_CTL,reg,IP175_ROUTER_CTL_PHY);

			phydev->bus->vlan_status=1;
			
    	}
     	else if( cmd->swtchInfo.vlanNeeded == -1)
    	{
        	// Command to disable the VLANs
			/* phydev,register,val,phy address */
			phy_write(phydev,IP175_TAG_CTL,0,IP175_TAG_CTL_PHY);
			reg = phy_read(phydev,IP175_ROUTER_CTL,IP175_ROUTER_CTL_PHY);
        	reg &= (IP175_ROUTER_CTL_TAG_VLAN_DSBL);
        	reg &= (IP175_ROUTER_CTL_DISABLE);
			/* phydev,register,val,phy address */
			phy_write(phydev,IP175_ROUTER_CTL,reg,IP175_ROUTER_CTL_PHY);

			phydev->bus->vlan_status=-1;
    	}


}
static int ip175_config_learning(struct phy_device *phydev,struct ethtool_cmd *cmd)
{
	u32 reg;
	reg = phy_read(phydev,IP175_LEARNING_DSBL,IP175_LEARNING_DSBL_PHY);
	if( cmd->swtchInfo.swtchLrnNeeded == 1)
	{
		reg = phy_read(phydev,IP175_LEARNING_DSBL,IP175_LEARNING_DSBL_PHY);
        // Enable Learning
        reg |= IP175_LEARNING_ENBL_ALL;
		phy_write(phydev,IP175_LEARNING_DSBL,reg,IP175_LEARNING_DSBL_PHY);

			phydev->bus->learning_status=1;
	}
	else if( cmd->swtchInfo.swtchLrnNeeded == -1)
	{
		reg = phy_read(phydev,IP175_LEARNING_DSBL,IP175_LEARNING_DSBL_PHY);
        // disable Learning
        reg &= IP175_LEARNING_DSBL_ALL;
		phy_write(phydev,IP175_LEARNING_DSBL,reg,IP175_LEARNING_DSBL_PHY);

		phydev->bus->learning_status=-1;
	}

		reg = phy_read(phydev,IP175_LEARNING_DSBL,IP175_LEARNING_DSBL_PHY);
		return 0;
}

static int ip175_config_flowControl(struct phy_device *phydev,struct ethtool_cmd *cmd)
{
		 u32 reg = 0;

	    if(cmd->swtchInfo.flowctrlNeeded == 1)
    	{
        	// Enable Learning
			reg = phy_read(phydev,IP175_LEARNING_DSBL,IP175_LEARNING_DSBL_PHY);
    	    // Enable Learning
	        reg |= IP175_LEARNING_ENBL_ALL;
			/* phydev,register,val,phy address */
			phy_write(phydev,IP175_LEARNING_DSBL,reg,IP175_LEARNING_DSBL_PHY);
		
			phydev->bus->flow_status=1;

    	}
    	else if(cmd->swtchInfo.flowctrlNeeded == -1)
    	{
        	// Disable Learning
			reg = phy_read(phydev,IP175_LEARNING_DSBL,IP175_LEARNING_DSBL_PHY);
        	// disable Learning
        	reg &= IP175_LEARNING_DSBL_ALL;
			/* phydev,register,val,phy address */
			phy_write(phydev,IP175_LEARNING_DSBL,reg,IP175_LEARNING_DSBL_PHY);
		
			phydev->bus->flow_status=-1;
    	}
		return 0;
}

static void ConfigureSpecialTag(struct phy_device *phydev , struct ethtool_cmd *cmd)
{
        int reg;
        #define IP175_TAG_CTL_ADD_P5_OUT_PKT_TAGS   (0x0002)
        #define IP175_TAG_CTL_REM_P04_OUT_PKT_TAGS  (0x07C0)
        #define IP175_TAG_CTL_PHY                   (29)
        #define IP175_TAG_CTL                       (23)

	if(cmd->swtchInfo.vlanNeeded == 1)
	{

		reg = phy_read(phydev,IP175_TAG_CTL,IP175_TAG_CTL_PHY);

        reg = IP175_TAG_CTL_ADD_P5_OUT_PKT_TAGS | IP175_TAG_CTL_REM_P04_OUT_PKT_TAGS;
        phy_write(phydev,IP175_TAG_CTL,reg,IP175_TAG_CTL_PHY);

        /* Enable Special tagging Phy :30 */
		reg = phy_read(phydev,16,30);
        reg=reg | 0x0080;
		phy_write(phydev , 16 , reg, 30 );

		/* Configure Membership Register with each port only in membership with cpu port */

		reg = phy_read(phydev,19,29);
		reg &=	0xF0F0;
		phy_write( phydev , 19, reg , 29 );

		reg = phy_read(phydev,19,29);
		reg |=	0x1112;
		phy_write( phydev , 19, reg , 29 );


/**********************/

		reg = phy_read(phydev,20,29);
		reg &=	0xF0F0;
		phy_write( phydev , 20, reg , 29 );

		reg = phy_read(phydev,20,29);
		reg |=	0x1418;
		phy_write( phydev , 20, reg , 29 );



		phydev->bus->vlan_status=1;

        // disable Learning
		reg = phy_read(phydev,IP175_LEARNING_DSBL,IP175_LEARNING_DSBL_PHY);
        reg &= IP175_LEARNING_DSBL_ALL;
        phy_write(phydev,IP175_LEARNING_DSBL,reg,IP175_LEARNING_DSBL_PHY);

        phydev->bus->learning_status=-1;

	}
	else if ( cmd->swtchInfo.vlanNeeded == -1 )
	{

		/* Disable tag insertion */
		reg =0;
        phy_write(phydev,IP175_TAG_CTL,reg,IP175_TAG_CTL_PHY);

        /* Disable Special tagging Phy :30 */
		reg = phy_read(phydev,16,30);
        reg=reg & 0xFF7F;
		phy_write(phydev , 16 , reg, 30 );

		reg = phy_read(phydev,19,29);
		reg &=	0xF0F0;
		phy_write( phydev , 19, reg , 29 );

		reg = phy_read(phydev,19,29);
		reg |=	0x1F1F;
		phy_write( phydev , 19, reg , 29 );
/*************************/
		reg = phy_read(phydev,20,29);
		reg &=	0xF0F0;
		phy_write( phydev , 20, reg , 29 );

		reg = phy_read(phydev,20,29);
		reg |=	0x1F1F;
		phy_write( phydev , 20, reg , 29 );
		/*Disable Membership configuration and put all the ports in same group */	
		phydev->bus->vlan_status= -1;


		 reg = phy_read(phydev,IP175_LEARNING_DSBL,IP175_LEARNING_DSBL_PHY);
        // Enable Learning
        reg |= IP175_LEARNING_ENBL_ALL;
        phy_write(phydev,IP175_LEARNING_DSBL,reg,IP175_LEARNING_DSBL_PHY);
        phydev->bus->learning_status=1;

	}


}


static int ip175_config_switch(struct phy_device *phydev,struct ethtool_cmd *cmd)
{
	if(cmd->swtchInfo.vlanNeeded)
	{
		ConfigureSpecialTag( phydev , cmd );	

//		ip175_init_vlan(phydev,cmd);		
	}
	if(cmd->swtchInfo.flowctrlNeeded)
	{
		ip175_config_flowControl(phydev,cmd);
	}
	if(cmd->swtchInfo.swtchLrnNeeded)
	{
		ip175_config_learning(phydev,cmd);	
	}
	return 0;
}

static int ip175_config_aneg(struct phy_device *phydev)
{
	int err;
    int val;
 if (AUTONEG_ENABLE == phydev->autoneg) {


        err = genphy_config_advert(phydev);

        if (err < 0)
            return err;

        err = genphy_restart_aneg(phydev);
    }
	 else
        {
			err=ip175_setup_forced(phydev);
        }
    return err;
	
}
static int ip175_hw_init(struct phy_device *phydev)
{
	u32 reg;
// Force IP175 registers to work in IP175C mode and not IP175A
    reg = IP175_REG_MODE_IP175C;
			/* phydev,register,val,phy address */
	phy_write(phydev,IP175_REG_MODE_REG,reg,IP175_REG_MODE_PHY);


    // Force P4 connected to ARM to work in 100Mbps Full Duplex, just in case
    reg = IP175_FORCE_P4_FULL_100;
	phy_write(phydev,IP175_CTL_SPEED_DUP_REG,reg,IP175_CTL_SPEED_DUP_PHY);

    /* Force MII1 to PHY Mode*/
    /* This is being done as IP175 on WLC reference design boards was not
    ** coming up with 10 and 10/100 BT pure Hubs. As suggested by ICPlus
    ** people, on reference design boards, Pin54 is floating. Pin 54 is for
    ** MII1 port PHY Mode selection. Be default MII1 is in MAC mode and
    ** creates problems when MII0 is connected with Half Duplex devices.
    ** MII1 was enabling back pressure control and there were a lot of
    ** collisions on the link because of this.
    ** Hardware MCR will be there, but still, just in case. -- Gagan
    */

    reg = IP175_FORCE_MII1_PHY_MODE;
	phy_write(phydev,IP175_MII_CTL_REG,reg,IP175_MII_CTL_PHY);

	/* Learning and Flow Control Enabled by default */
#if 1
		 reg = phy_read(phydev,IP175_LEARNING_DSBL,IP175_LEARNING_DSBL_PHY);
        // disable Learning
        reg &= IP175_LEARNING_DSBL_ALL;
        phy_write(phydev,IP175_LEARNING_DSBL,reg,IP175_LEARNING_DSBL_PHY);

        phydev->bus->learning_status=-1;
#endif

	phydev->bus->flow_status=1;
	phydev->bus->vlan_status=-1;

	return 0;

}

static int GetPortFromTag(int stag)
{


        int port=0;
        int loc=1;
        int k;
        for(k=0;k<5;k++)
        {
            if(stag & loc)
            {
                port=k;
                break;
            }
            else
                loc=loc<<1;

        }

        return port;
}
void * InsertSpecialTag(struct sk_buff *skb,int tag)
{

    struct vlan_ethhdr *veth;

    if (skb_headroom(skb) < VLAN_HLEN) {
        struct sk_buff *sk_tmp = skb;
        skb = skb_realloc_headroom(sk_tmp, VLAN_HLEN);
        kfree_skb(sk_tmp);
        if (!skb) {
            printk(KERN_ERR "vlan: failed to realloc headroom\n");
            return NULL;
        }
    } else {
        skb = skb_unshare(skb, GFP_ATOMIC);
        if (!skb) {
            printk(KERN_ERR "vlan: failed to unshare skbuff\n");
            return NULL;
        }
    }

    veth = (struct vlan_ethhdr *)skb_push(skb, VLAN_HLEN);

    /* Move the mac addresses to the beginning of the new header. */
    memmove(skb->data, skb->data + VLAN_HLEN, 2 * VLAN_ETH_ALEN);

    /* first, the ethernet type */
    //veth->h_vlan_proto = __constant_htons(ETH_P_8021Q);
    veth->h_vlan_proto = __constant_htons(0x8100 | tag);

    /* now, the tag */
    veth->h_vlan_TCI = htons(tag);

//    skb->protocol = __constant_htons(ETH_P_8021Q);
    skb->mac.raw -= VLAN_HLEN;
    skb->nh.raw -= VLAN_HLEN;

    return skb;

}

static struct sk_buff * ip175_handle_portsplit_tx( struct sk_buff *skb , int portno)
{
    int loc=1;
    int k=0;
         for(k=0;k<portno ;k++)
             loc=loc<<1;
         return InsertSpecialTag( skb , loc );
}

static int ip175_handle_portsplit_rx( struct sk_buff *skb )
{


          if( ( *( skb->data + 12 ) == 0x81 )  && ( *( skb->data + 13 ) != 0x00 ) )
            {
				/* Packet has special Tag */
                unsigned int tagval =  *( skb->data + 13 ) & 0x0FFF;
                unsigned int port = GetPortFromTag( tagval );
				/* if the tag val is 1 then this means untagged packet was received */
				
				if ( *( skb->data + 15 ) == 0x1 )
				{
	                /* Strip The Tag Off */
	                 skb_pull(skb, VLAN_HLEN);
	                /* Move the Mac Address by four bytes */
	                 memcpy(skb->data , skb->data - VLAN_HLEN , 12 );
				}
				else
				{
					*( skb->data + 13 ) = 0x00;
				}

                 return port;
            }

        return -1;
}

static struct phy_driver ip175c_driver = {
	.phy_id		= 0x02430D80,
	.phy_id_mask	= 0xffffffff,
/* Four ports on solosW and 5 on solosWV */
#ifdef CONFIG_ARCH_SOLOS_208PIN
	.ignore_phy_mask = 0xfffffff0,
#else
	.ignore_phy_mask = 0xffffffe0,
#endif
	.name		= "IP175",
	.features	= PHY_BASIC_FEATURES,
	.config_aneg	= &ip175_config_aneg,
	.read_status	= &genphy_read_status,
	.config_switch    = &ip175_config_switch,
	.config_init = &ip175_hw_init,
	.handle_portsplit_rx = &ip175_handle_portsplit_rx,
    .handle_portsplit_tx = &ip175_handle_portsplit_tx,
	.driver 	= { .owner = THIS_MODULE,},
};

static int __init ip175_init(void)
{
	
	return phy_driver_register(&ip175c_driver);
}

static void __exit ip175_exit(void)
{
	phy_driver_unregister(&ip175c_driver);
}

module_init(ip175_init);
module_exit(ip175_exit);
