/*
 * drivers/net/phy/ip175c.h
 *
 * Driver for IP175 PHYs
 * (c) Copyright Conexant System
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 */

#ifndef _UNI_ETH_IP175C_
#define _UNI_ETH_IP175C_

#define IP175_NUM_PHY_PORTS (5)

/* Port registers */
#define IP175_SMI_ID1_VALUE             (0x0243)
#define IP175_SMI_ID2_VALUE             (0x0D80)

#define IP175_FLOW_CTL_PHY                  (29)
#define IP175_FLOW_CTL_REG					(18)
#define IP175_FLOW_CTL_DSBL                 (0xFBFF)
#define IP175_FLOW_CTL_ENBL                 (0x0400)

#define IP175_CTL_SPEED_DUP_PHY				(29)
#define IP175_CTL_SPEED_DUP_REG				(22)
#define IP175_FORCE_P4_FULL_100				(0x8420)

#define IP175_TAG_CTL_PHY                   (29)
#define IP175_TAG_CTL                       (23)

#define IP175_TAG_P0_PHY                    (29)
#define IP175_TAG_P0                        (24)

#define IP175_TAG_P1_PHY                    (29)
#define IP175_TAG_P1                        (25)

#define IP175_TAG_P2_PHY                    (29)
#define IP175_TAG_P2                        (26)

#define IP175_TAG_P3_PHY                    (29)
#define IP175_TAG_P3                        (27)

#define IP175_TAG_P4_PHY                    (29)
#define IP175_TAG_P4                        (28)

#define IP175_TAG_P5_PHY                    (29)
#define IP175_TAG_P5                        (30)

#define IP175_REG_MODE_PHY					(29)
#define IP175_REG_MODE_REG					(31)
#define IP175_REG_MODE_IP175C				(0x175C)

#define IP175_TAG_VLAN_01_PHY               (30)
#define IP175_TAG_VLAN_01                    (1)

#define IP175_TAG_VLAN_23_PHY               (30)
#define IP175_TAG_VLAN_23                    (2)
#define IP175_TAG_VLAN_45                    (3)

#define IP175_ROUTER_CTL_PHY                (30)
#define IP175_ROUTER_CTL                     (9)

#define IP175_LEARNING_DSBL_PHY				(30)
#define IP175_LEARNING_DSBL					(16)
#define IP175_LEARNING_DSBL_ALL				(0xE0FF)
#define IP175_LEARNING_ENBL_ALL				(0x1F00)

#define IP175_MII_CTL_PHY					(31)
#define IP175_MII_CTL_REG					(5)
#define IP175_FORCE_MII1_PHY_MODE			(0x90f0)		// Value suggested by ICPLUS people, default was 0xa0f0


#define IP175_TAG_CTL_P04_ADD_TAG_MASK  (0xf800)
#define IP175_TAG_CTL_P04_ADD_TAG_SHIFT     (11)
#define IP175_TAG_CTL_P04_REM_TAG_MASK  (0x07c0)
#define IP175_TAG_CTL_P04_REM_TAG_SHIFT      (6)
#define IP175_TAG_CTL_P5_ADD_TAG        (0x0002)
#define IP175_TAG_CTL_P5_REMOVE_TAG     (0x0001)

#define IP175_TAG_VLAN_L_MASK           (0x003f)
#define IP175_TAG_VLAN_L_SHIFT               (0)
#define IP175_TAG_VLAN_H_MASK           (0x3f00)
#define IP175_TAG_VLAN_H_SHIFT               (8)

#define IP175_ROUTER_CTL_WAN_MASK       (0x1f00)
#define IP175_ROUTER_CTL_WAN_SHIFT           (8)
#define IP175_ROUTER_CTL_TAG_VLAN       (0x0080)
#define IP175_ROUTER_CTL_TAG_VLAN_DSBL  (0xFF7F)
#define IP175_ROUTER_CTL_VID_MASK       (0x0070)
#define IP175_ROUTER_CTL_VID_SHIFT           (4)
#define IP175_ROUTER_CTL_ENABLE         (0x0008)
#define IP175_ROUTER_CTL_DISABLE        (0xFFF7)
#define IP175_ROUTER_CTL_LAN_MASK       (0x0007)
#define IP175_ROUTER_CTL_LAN_SHIFT           (0)


#define IP175_TAG_CTL_ADD_P5_OUT_PKT_TAGS	(0x0002)
#define IP175_TAG_CTL_REM_P04_OUT_PKT_TAGS	(0x07C0)

#ifdef GALLUS_WLC
#define IP175_RTR_CTL_NUM_VLANS				(0x0002)
#define IP175_RTR_CTL_WAN_PORTS				(0x0018)
#else
#define IP175_RTR_CTL_NUM_VLANS				(0x0001)
#define IP175_RTR_CTL_WAN_PORTS				(0x001f)
#endif

#define IP175_VLAN_MASK_LEFT				(0xFF00)
#define IP175_VLAN_MASK_RIGHT				(0x00FF)

#define IP175_VLAN_MASK_P0_P5				(0x0021)
#define IP175_VLAN_MASK_P1_P5				(0x2200)
#define IP175_VLAN_MASK_P2_P5				(0x0024)
#define IP175_VLAN_MASK_P3_P5				(0x2800)

#define IP175_MII1_PORT						4	

/* Function Prototypes */
void ip175_InitVLAN(void *pContext);
void ip175_HandleLearning(void *pContext);
void ip175_HndlFlowCtrl(void *pContext);
void ip175_HWInit(void *pContext);

#endif

