/*
 * drivers/net/phy/rtl8305sc.c
 *
 * Driver for RTL8305SC PHY
 * (c) Copyright Conexant System 
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 */
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/unistd.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/spinlock.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/mii.h>
#include <linux/ethtool.h>
#include <linux/phy.h>

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <linux/ethtool.h>
#include<linux/if_vlan.h>
#include "rtl8305sc.h"

MODULE_DESCRIPTION("RTL8305SC PHY driver");
MODULE_AUTHOR("Nitin Sehgal");
MODULE_LICENSE("GPL");

static int init=0;
#define PHY_PORTS 4

static int SDSWITCH; 

static struct phy_device * phy_get_phydev_by_addr( struct phy_device * phydev, int address )
{
    int num;



/* since phy devices are located in phy map at index based on physical phyAddress so scan the whole list */

         for(num=0;num<PHY_MAX_ADDR;num++)
         {
            struct phy_device *tmpphydev=phydev->bus->phy_map[num];
            if(tmpphydev == NULL)
                continue;

            if ( tmpphydev->addr == address )
                return tmpphydev;

        }
    return NULL;
}

void rtl8305sc_soft_reset(struct phy_device *phydev)
{
	/* Only PHY zero has this register need to write in PHY zero*/
	struct phy_device *tmpphydev = phy_get_phydev_by_addr(phydev , 0);
	int i;

	if(tmpphydev)
	{
	    unsigned short int regval=phy_read(tmpphydev,PHY_REG_16,0); 
		regval = ( regval | PHY_REG_16_SOFT_RESET );
	    phy_write(tmpphydev, PHY_REG_16 , regval,0);
	}	
}

int rtl8305sc_setup_forced(struct phy_device *phydev)
{
    short int ctl=phy_read(phydev,MII_BMCR,0); 
    phydev->pause = phydev->asym_pause = 0;

	/* Nitin to force disable autoneg */
	ctl &=~BMCR_ANENABLE;  	

	 
	 if (100 == phydev->speed)
        ctl |= BMCR_SPEED100;
    else if (10 == phydev->speed)
        ctl &= ~BMCR_SPEED100;

	
    if (DUPLEX_FULL == phydev->duplex)
        ctl |= BMCR_FULLDPLX;
	else
        ctl &= ~BMCR_FULLDPLX;

    ctl = phy_write(phydev, MII_BMCR, ctl,0);

    ctl=phy_read(phydev,MII_BMCR,0); 

    if (ctl < 0)
        return ctl;

	if (!SDSWITCH) // NS:: ENABLE FOR 8305SC SWITCH
		rtl8305sc_soft_reset(phydev);

    /* We just reset the device, so we'd better configure any
     * settings the PHY requires to operate */
    if (phydev->drv->config_init && !init)
        ctl = phydev->drv->config_init(phydev);

    init=1;

 return ctl;
}

static int rtl8305sc_init_vlan(struct phy_device *phydev,struct ethtool_cmd *cmd)
{
	int i=0;
	int bit_val=0;
	int loop=0;
	unsigned int PVID_ARRAY[PHY_PORTS];
    u32 reg = 0;
    u32 vlanmask = 0, vlanId = 0;
	int index=0;

//########## GET PVID and mask ############
		for(i=0;i<cmd->swtchInfo.NVlans;i++)
		{
			unsigned short mask = cmd->swtchInfo.VMask[i];
			unsigned short pvid= ( cmd->swtchInfo.VMask[i] >> 16 ); 
			bit_val=1;
			loop=0;
			for(loop=0;loop<PHY_PORTS;loop++)
			{
				if(mask & bit_val)
				{

					PVID_ARRAY[loop]=pvid;

				/* Program the index of VLAN membership also */
					reg = phy_read(phydev , PHY_REG_24 , loop);
					reg = (reg & ~PHY_VLAN_INDEX_MASK);	
					reg |= (( i << 12 ) & ( PHY_VLAN_INDEX_MASK ));
					phy_write(phydev, PHY_REG_24 , reg , loop);

					reg = phy_read(phydev , PHY_REG_24 , loop);
				}				
				 bit_val=bit_val *2;
			}
		}		
//#########################################

 if( cmd->swtchInfo.vlanNeeded == 1)
{
	 for(index=0;index < PHY_PORTS ; index++)
 	 {
			/*
				1. Register 25 for VLANID
			    2. Bits [ 0 - 11 ]
			*/

			//NS
		    	  /***********************************************
    	    	   **
	        	   ** Configure PVIDs
	        	   **
	        	   ***********************************************/
			/* clear the Vlanid register */
					
				reg = phy_read(phydev , PHY_REG_25 , index);
				reg = ( reg & ( ~PHY_VLAN_ID_MASK ) );
				phy_write(phydev, PHY_REG_25 , reg , index);

			/* Now Write the new vlan ID value */
	
				reg = phy_read(phydev , PHY_REG_25 , index );
				reg |= ( PVID_ARRAY[ index ] & PHY_VLAN_ID_MASK  );
				phy_write(phydev, PHY_REG_25 , reg , index );

				udelay(1000);

			/* For Physical Ports, remove tag while sending the packets out */
			
				reg = phy_read(phydev , PHY_REG_22 , index);
				reg = (( reg & ~PHY_VLAN_TAG_CTRL_MASK) | PHY_VLAN_TAG_CTRL_01);
				phy_write(phydev, PHY_REG_22 , reg , index);
	
			udelay(1000);


	  }
			/*For MII Port, Insert VLAN Tag before sending the packets towards processor */

#if 1
				reg = phy_read(phydev , PHY_REG_22 , 4);
				reg = ( reg & ~PHY_VLAN_TAG_CTRL_MASK );	
				phy_write(phydev, PHY_REG_22 , reg , 4);

#endif
#if 0
				reg = phy_read(phydev , PHY_REG_22 , 5);
				reg = ( reg & ~PHY_VLAN_TAG_CTRL_MASK );	
				phy_write(phydev, PHY_REG_22 , reg , 5);
#endif

				udelay(1000);


	        /***********************************************
  		     **
             ** Configure VLAN Members
             **
             ***********************************************/ 

	        	for (i = 0; i< cmd->swtchInfo.NVlans; i++)
	        	{

/* NS :: I feel mii port whis is 4 should be in all the vlans 
still not sure why this is not done in isos */

					reg = phy_read(phydev , PHY_REG_24 , i);
					reg |= ( ( cmd->swtchInfo.VMask[i] & PHY_VLAN_ID_MB_MASK ) | 0x30 ); // FOR HU
					phy_write(phydev, PHY_REG_24 , reg , i);
		         }

  		/***********************************************
    	**
    	** Configure VLAN Enable Registers
    	**
    	***********************************************/

/* NS : Need to write in PHY 0 and if we pass 0 as override phy value it means 
use phydev->addr value so need to pass the phydevice for phy 0 */

						struct phy_device * tmpphydev = phy_get_phydev_by_addr ( phydev , 0 );

						reg = phy_read(tmpphydev , PHY_REG_18 , 0);
						reg = ( reg & ( ~PHY_REG18_DIS_VLAN ) );
						phy_write(tmpphydev, PHY_REG_18 , reg , 0);
						
						udelay(1000);

				if (!SDSWITCH) // NS:: ENABLE FOR 8305SC SWITCH
					rtl8305sc_soft_reset(phydev);

					phydev->bus->vlan_status=1;
			
}
else if( cmd->swtchInfo.vlanNeeded == -1 )
{
						struct phy_device * tmpphydev = phy_get_phydev_by_addr ( phydev , 0 );
						reg = phy_read(tmpphydev , PHY_REG_18 , 0);
				reg = ( reg |  PHY_REG18_DIS_VLAN  );
						phy_write(tmpphydev, PHY_REG_18 , reg , 0);
#if 1
				reg = phy_read(phydev , PHY_REG_22 , 4);
				reg = ( reg | PHY_VLAN_TAG_CTRL_MASK );	
				phy_write(phydev, PHY_REG_22 , reg , 4);

#endif
						
						udelay(1000);
			
						phydev->bus->vlan_status=-1;
}
}

static int rtl8305sc_config_learning(struct phy_device *phydev,struct ethtool_cmd *cmd)
{
	u32 reg;
	int i;


	  phydev=phydev->bus->phy_map[0];

	/* Check the List of PHY enumerated */
	for(i=0;i<4 ;i++)
	{
	    struct phy_device *tmpphydev=phydev;
		
		if(tmpphydev == NULL)
			continue;

		if( cmd->swtchInfo.swtchLrnNeeded == 1)
		{
			reg = phy_read(phydev,PHY_REG_24,i);
	        // Enable Learning
			/* Change the learning Bit to Enable */
            reg |= 0x200; // Bit 9 has to be set

			phy_write(phydev,PHY_REG_24,reg,i);
			reg = phy_read(phydev,PHY_REG_24,i);

			phydev->bus->learning_status=1;
		}
		else if( cmd->swtchInfo.swtchLrnNeeded == -1)
		{
			reg = phy_read(phydev,PHY_REG_24,i);
    	    // disable Learning
			/* Change the learning Bit to Disable */
		        reg &= 0xFDFF;    // Bit 9 has to be reset
			phy_write(phydev,PHY_REG_24,reg,i);

			reg = phy_read(phydev,PHY_REG_24,i);

			phydev->bus->learning_status=-1;
		}
	}
	if (!SDSWITCH) // NS:: ENABLE FOR 8305SC SWITCH
		rtl8305sc_soft_reset(phydev);

		return 0;
}

static int rtl8305sc_config_flowControl(struct phy_device *phydev,struct ethtool_cmd *cmd)
{
		phydev->bus->flow_status=-2; /* Flow Control Not supported */
		return 0;
#if 0
		 u32 reg = 0;

	    if(cmd->swtchInfo.flowctrlNeeded == 1)
    	{
        	// Enable Learning
			reg = phy_read(phydev,IP175_LEARNING_DSBL,IP175_LEARNING_DSBL_PHY);
    	    // Enable Learning
	        reg |= IP175_LEARNING_ENBL_ALL;
			/* phydev,register,val,phy address */
			phy_write(phydev,IP175_LEARNING_DSBL,reg,IP175_LEARNING_DSBL_PHY);
		
			phydev->bus->flow_status=1;

    	}
    	else if(cmd->swtchInfo.flowctrlNeeded == -1)
    	{
        	// Disable Learning
			reg = phy_read(phydev,IP175_LEARNING_DSBL,IP175_LEARNING_DSBL_PHY);
        	// disable Learning
        	reg &= IP175_LEARNING_DSBL_ALL;
			/* phydev,register,val,phy address */
			phy_write(phydev,IP175_LEARNING_DSBL,reg,IP175_LEARNING_DSBL_PHY);
		
			phydev->bus->flow_status=-1;
    	}
		return 0;
#endif
}

#define RTL8306_REGPAGE0		0x0
#define RTL8306_REGPAGE1		0x1
#define RTL8306_REGPAGE2		0x2
#define RTL8306_REGPAGE3		0x3
unsigned int  rtl8306_setPage(struct phy_device *phydev, unsigned int phyad, unsigned int npage ) 
{
	unsigned int rdata; 


	/* Select PHY Register Page through configuring PHY 0 Register 16 [bit1 bit15] */
	rdata = phy_read(phydev,16,phyad);
	switch (npage) {
	case RTL8306_REGPAGE0:
		phy_write(phydev,16, (rdata & 0x7FFF) | 0x0002 ,phyad);
		break;
	case RTL8306_REGPAGE1:
		phy_write(phydev,16, rdata | 0x8002 ,phyad);
		break;
	case RTL8306_REGPAGE2:
		phy_write(phydev,16, rdata & 0x7FFD ,phyad);
		break;
	case RTL8306_REGPAGE3:
		phy_write(phydev,16, (rdata & 0xFFFD) | 0x8000 ,phyad);
		break;
	default:
		return -1;
	}
	
	return 0;
}

/*Enable for ZTE requirement*/
//#define FOR_ZTE 1

static int rtl8305sc_init_cputag(struct phy_device *phydev,struct ethtool_cmd *cmd)
{
	unsigned int rdata;

	phydev=phydev->bus->phy_map[0];

 if( cmd->swtchInfo.vlanNeeded == 1)
 {

#ifndef FOR_ZTE
		/* Enable VLAN */

		int reg = phy_read(phydev , PHY_REG_18 , 0);
                reg = ( reg & ( ~PHY_REG18_DIS_VLAN ) );
                phy_write(phydev, PHY_REG_18 , reg , 0);

#endif

	phydev->bus->vlan_status=1;

	/* 1 Enable CPU TAG AWARE */
	#define ENABLE_CPU_TAG_AWARE 0x80;

	rdata = phy_read(phydev,21,4);
	rdata = rdata | ENABLE_CPU_TAG_AWARE;
	phy_write(phydev,21, rdata,4);

	/* SPECIFY THE CPU PORT ( PORT 4 ) */
	
	#define CPU_PORT_MASK 0x7
	#define CPU_PORT 0x4

	rdata = phy_read(phydev,21,4);
	rdata = ( rdata & ( ~CPU_PORT_MASK ) ) | CPU_PORT;
	phy_write(phydev,21, rdata,4);

	/* Change the page to page 3 */
	 rtl8306_setPage(phydev, 0, 3 ); 

	/* Enable CPU PORT E\FUNCTIONALITY */

	#define CPUPORT_ENABLE 0x7FFF

	rdata = phy_read(phydev,21,2);
	rdata = rdata & CPUPORT_ENABLE;
	phy_write(phydev,21, rdata,2);

	
	/* Enable Insert CPU TAG */	
	
	#define ENABLE_CPUTAG_INSERTION 0x1000

	rdata = phy_read(phydev,21,2);
	rdata = rdata | ENABLE_CPUTAG_INSERTION ;
	phy_write(phydev,21, rdata,2);

	/* Enable CPU tag Removal */

	#define ENABLE_CPUTAG_REMOVAL 0x0800

	rdata = phy_read(phydev,21,2);
	rdata = rdata | ENABLE_CPUTAG_REMOVAL ;
	phy_write(phydev,21, rdata,2);


        /* Change the page to page 0 */
         rtl8306_setPage(phydev, 0, 0 );


  }
  else if( cmd->swtchInfo.vlanNeeded == -1 )
  {

#ifndef FOR_ZTE
	/* DISABLE VLAN */
	int reg = phy_read(phydev , PHY_REG_18 , 0);
        reg = ( reg | ( PHY_REG18_DIS_VLAN ) );
        phy_write(phydev, PHY_REG_18 , reg , 0);
#endif

	phydev->bus->vlan_status=-1;

	/* 1 Enable CPU TAG AWARE */
        #define DISABLE_CPU_TAG_AWARE 0xFF7F;

        rdata = phy_read(phydev,21,4);
        rdata = rdata & DISABLE_CPU_TAG_AWARE;
        phy_write(phydev,21, rdata,4);

        /* Change the page to page 3 */
         rtl8306_setPage(phydev, 0, 3 );

        /* Enable CPU PORT DISABLE FUNCTIONALITY */

        #define CPUPORT_DISABLE 0x8000

        rdata = phy_read(phydev,21,2);
        rdata = rdata | CPUPORT_DISABLE;
        phy_write(phydev,21, rdata,2);


        /* Enable Insert CPU TAG */

        #define DISABLE_CPUTAG_INSERTION 0xEFFF

        rdata = phy_read(phydev,21,2);
        rdata = rdata & DISABLE_CPUTAG_INSERTION ;
        phy_write(phydev,21, rdata,2);

	/* Enable CPU tag Removal */

        #define ENABLE_CPUTAG_REMOVAL 0xF7FF

        rdata = phy_read(phydev,21,2);
        rdata = rdata & ENABLE_CPUTAG_REMOVAL ;
        phy_write(phydev,21, rdata,2);

        /* Change the page to page 0 */
         rtl8306_setPage(phydev, 0, 0 );

	phydev->bus->vlan_status=-1;
	
  
 }
		
	
}

static int GetPortFromTag( int tag )
{
    int loc=1;
        int k;
    int port=0;

        for(k=0;k<5;k++)
        {
            if(tag & loc)
            {
                port=k;
                break;
            }
            else
                loc=loc<<1;

        }
return port;
}

static int rtl_handle_portsplit_rx( struct sk_buff *skb )
{

			/* In Case of SD Switch we use Cpu tagging for Port Splitting */
			/* Dont check for tag value */

//		 	if( ( *( skb->data + 12 ) == 0x88 )  && ( *( skb->data + 13 ) == 0x99 ) )
	        {
	            unsigned int tagval =  *( skb->data + 15 ) & 0x0FFF;
	            unsigned int port = GetPortFromTag( tagval );
				/* Strip The Tag Off */
				 skb_pull(skb, VLAN_HLEN);	
				/* Move the Mac Address by four bytes */
				 memcpy(skb->data , skb->data - VLAN_HLEN , 12 );
	
				 return port;
			}

		return -1;
}

struct sk_buff * InsertCpuTag(struct sk_buff *skb,int tag)
{

    struct vlan_ethhdr *veth;

    if (skb_headroom(skb) < VLAN_HLEN) {
        struct sk_buff *sk_tmp = skb;
        skb = skb_realloc_headroom(sk_tmp, VLAN_HLEN);
        kfree_skb(sk_tmp);
        if (!skb) {
            printk(KERN_ERR "vlan: failed to realloc headroom\n");
            return NULL;
        }
    } else {
        skb = skb_unshare(skb, GFP_ATOMIC);
        if (!skb) {
            printk(KERN_ERR "vlan: failed to unshare skbuff\n");
            return NULL;
        }
    }

    veth = (struct vlan_ethhdr *)skb_push(skb, VLAN_HLEN);

    /* Move the mac addresses to the beginning of the new header. */
    memmove(skb->data, skb->data + VLAN_HLEN, 2 * VLAN_ETH_ALEN);

    /* first, the ethernet type */
    //veth->h_vlan_proto = __constant_htons(ETH_P_8021Q);

	if(SDSWITCH)
	{
		/* Use Cpu tagging for Port splitting in case of SD Switch */
	    veth->h_vlan_proto = __constant_htons(0x8899);
	    /* now, the tag */
	    veth->h_vlan_TCI = htons((0x93<<8) |( (0xC<<4)|tag ));
	}
	else
	{
		/* Use Normal Vlan based method for port splitting */
	    veth->h_vlan_proto = __constant_htons(0x8100);
	    /* now, the tag */
	    veth->h_vlan_TCI = htons( tag );
	}
//    skb->protocol = __constant_htons(ETH_P_8021Q);
    skb->mac.raw -= VLAN_HLEN;
    skb->nh.raw -= VLAN_HLEN;

    return skb;

}

static struct sk_buff * rtl_handle_portsplit_tx( struct sk_buff *skb , int portno)
{
	int loc=1;
	int k=0;

	if(SDSWITCH)
	{
		 for(k=0;k<portno ;k++)
	    	 loc=loc<<1;

		 return InsertCpuTag( skb , loc );	
	}
	else
	{
		 return InsertCpuTag( skb , portno + 2 );	
				
	}
}


static int rtl8305sc_config_switch(struct phy_device *phydev,struct ethtool_cmd *cmd)
{

	if(cmd->swtchInfo.vlanNeeded)
	{
		if ( SDSWITCH )
			rtl8305sc_init_cputag(phydev,cmd);
		else
			rtl8305sc_init_vlan(phydev,cmd);		
	}
	if(cmd->swtchInfo.flowctrlNeeded)
	{
		rtl8305sc_config_flowControl(phydev,cmd);
	}
	if(cmd->swtchInfo.swtchLrnNeeded)
	{
		rtl8305sc_config_learning(phydev,cmd);	
	}
	return 0;
}

static int rtl8305sc_config_aneg(struct phy_device *phydev)
{
	int err;
    int val;
 
 if (AUTONEG_ENABLE == phydev->autoneg) {


        err = genphy_config_advert(phydev);

        if (err < 0)
            return err;

        err = genphy_restart_aneg(phydev);
    }
	 else
        {
			err=rtl8305sc_setup_forced(phydev);
        }
    return err;
	
}
static int rtl8305sc_hw_init(struct phy_device *phydev)
{

int i=0;
unsigned int reg;

if(phydev)
{


/* Check if it is SD Switch or Not */		
	 reg = phy_read(phydev,31,4);
	 if ( reg == 0x0100 )
		SDSWITCH = 1;
	 else 
		SDSWITCH = 0;


/* Disable The CPU Port Learning */
	for(i=0;i<=4;i++)
	{
						reg = phy_read(phydev,PHY_REG_24,i);
                        /* Change the learning Bit to Disable */
                        reg &= 0xFDFF;    // Bit 9 has to be reset
                        phy_write(phydev,PHY_REG_24,reg,i);
                        reg = phy_read(phydev,PHY_REG_24,i);


			/*START ADD :jaffen for cross detection*/
			reg = phy_read(phydev,PHY_REG_24,i);
			reg |= 0x0080;
			phy_write(phydev,PHY_REG_24,reg,i);
			/*END ADD :jaffen for cross detection*/		

	}

	/* Port Splitting Disabled in begning */
	phydev->bus->vlan_status=-1;

}

	
	return 0;
}

static struct phy_driver rtl8305sc_driver = {
	.phy_id		= 0x001CC852,
	.phy_id_mask	= 0xffffffff,
	.ignore_phy_mask = 0xfffffff0,
	.name		= "RTL18305SC",
	.features	= PHY_BASIC_FEATURES,
	.config_aneg	= &rtl8305sc_config_aneg,
	.read_status	= &genphy_read_status,
	.config_switch    = &rtl8305sc_config_switch,
    .handle_portsplit_rx = &rtl_handle_portsplit_rx,
    .handle_portsplit_tx = &rtl_handle_portsplit_tx,
	.config_init = &rtl8305sc_hw_init,
	.driver 	= { .owner = THIS_MODULE,},
};

static int __init rtl8305sc_init(void)
{
	
	return phy_driver_register(&rtl8305sc_driver);
}

static void __exit rtl8305sc_exit(void)
{
	phy_driver_unregister(&rtl8305sc_driver);
}

module_init(rtl8305sc_init);
module_exit(rtl8305sc_exit);
