/* ###########################################################################
# (c) Copyright Conexant Limited 2006
#
# Conexant Limited Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Limited ("Conexant").  It may only be read, used,
# copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Conexant and then subject to the
# terms of that confidentiality agreement and any other applicable agreement
# between you and Conexant.  If you are in any doubt as to whether you are
# entitled to access, read, use, copy, adapt, modify or otherwise deal with
# the Software or whether you are entitled to disclose the Software to any
# other person you should contact Conexant.  If you have not entered into a
# confidentiality agreement with Conexant granting access to this Software you
# should forthwith return all media, copies and printed listings containing
# the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you breach
# the above provisions.
#
# If you are unsure, or to report violations, please contact
# support@conexant.com
# ##########################################################################*/

/**HEADER*********************************************************
-----------------------------------------------------------------------------
 CONFIDENTIAL AND PROPRIETARY INFORMATION OF TRANSDIMENSION, INC. AND/OR ITS
 WHOLLY OWNED SUBSIDIARY SOFTCONNEX TECHNOLOGIES, INC. THIS NOTICE IS NOT TO
 BE DELETED, MODIFIED, MOVED OR CHANGED IN ANY WAY.

 (TransDimension, Inc. and SoftConnex Technologies, Inc. are referred to
 collectively below as TransDimension, Inc.) 
 Copyright (c) 1999 - 2004 by TransDimension, Inc. 

 This software is protected by copyright laws and international copyright 
 treaties, as well as other intellectual property laws and treaties.  This
 software is a CONFIDENTIAL, unpublished work of authorship, and with portions 
 constituting TRADE SECRETS of TransDimension, Inc..  Any unauthorized use, 
 disclosure, and/or reproduction of this software, or any part of this software;
 or distribution of this software in any form or by any means; or storage of 
 this software in any database or retrieval system, without the express written
 consent of, and license from, TransDimension, Inc. is strictly prohibited.  
 This software is protected under the copyright and/or trade secret laws in 
 other countries in addition to USA.  All Rights Reserved.  Failure to abide by
 the use, disclosure and/or reproduction restrictions may result in civil and
 or criminal penalties, and will be prosecuted to the maximum extent of the law.
 -------------------------------------------------------------------------------

*** Description:      
***  This file contains the VUSB_HS Device Controller interface 
***  function to cancel a transfer.
***                                                               
**************************************************************************
**END*********************************************************/
#include "devapi.h"
#include "usbprv_dev.h"
#ifdef __USB_OS_MQX__
   #include "mqx_arc.h"
#endif

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_cancel_transfer
*  Returned Value : USB_OK or error code
*  Comments       :
*        Cancels a transfer
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_cancel_transfer
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
     
      /* [IN] the Endpoint number */
      uint_8                     ep_num,
            
      /* [IN] direction */
      uint_8                     direction
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   VUSB20_EP_TR_STRUCT_PTR                      dTD_ptr, check_dTD_ptr;
   VUSB20_EP_QUEUE_HEAD_STRUCT_PTR              ep_queue_head_ptr;
   XD_STRUCT_PTR                                xd_ptr;
   uint_32                                      temp, bit_pos;
   int                                          timeout;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_cancel_transfer");
   #endif
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   bit_pos = (1 << (16 * direction + ep_num));
   temp = (2*ep_num + direction);

#ifdef _VIRT_MEM
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      phys2virt(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR) + 
      temp;
#else
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR + 
      temp;
#endif

   /* Unlink the dTD */
   dTD_ptr = usb_dev_ptr->EP_DTD_HEADS[temp];
   
   if (dTD_ptr) {
#ifdef _VIRT_MEM
      check_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)
	  ((uint_32)phys2virt(dTD_ptr->NEXT_TR_ELEM_PTR & VUSBHS_TD_ADDR_MASK)) ;
#else
      check_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)((uint_32)dTD_ptr->NEXT_TR_ELEM_PTR & 
         VUSBHS_TD_ADDR_MASK);
#endif
      if (dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE) {
         /* Flushing will halt the pipe */
         /* Write 1 to the Flush register */
         dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH = bit_pos;

         /* Wait until flushing completed */
         timeout = 1000;
         while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH & bit_pos) {
            /* ENDPTFLUSH bit should be cleared to indicate this 
            ** operation is complete 
            */
             if (timeout-- == 0)
             {
                 USBERR("Timeout while cancelling td\n");
                 return USBERR_UNKNOWN_ERROR;
             }
             
         } /* EndWhile */
         
         while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS & 
            bit_pos) 
         {
            /* Write 1 to the Flush register */
            dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH = bit_pos;
         
            /* Wait until flushing completed */
            while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH & bit_pos) {
               /* ENDPTFLUSH bit should be cleared to indicate this 
               ** operation is complete 
               */
            } /* EndWhile */
         } /* EndWhile */
      } /* Endif */
      
      /* Retire the current dTD */
      dTD_ptr->SIZE_IOC_STS = 0;
      dTD_ptr->NEXT_TR_ELEM_PTR = VUSBHS_TD_NEXT_TERMINATE; //vOK
      
      /* The transfer descriptor for this dTD */
      xd_ptr = (XD_STRUCT_PTR)dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD;
      /* Free the dTD */
      _usb_dci_vusb20_free_dTD((pointer)dTD_ptr);
      
      /* Update the dTD head and tail for specific endpoint/direction */
      if (!check_dTD_ptr) {
         usb_dev_ptr->EP_DTD_HEADS[temp] = NULL;
         usb_dev_ptr->EP_DTD_TAILS[temp] = NULL;
         if (xd_ptr) {
            /* Free the transfer descriptor */
            _usb_device_free_XD((pointer)xd_ptr);
         } /* Endif */
         /* No other transfers on the queue */
         ep_queue_head_ptr->NEXT_DTD_PTR = VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE; //vOK
         ep_queue_head_ptr->SIZE_IOC_INT_STS = 0;
      } else {
         usb_dev_ptr->EP_DTD_HEADS[temp] = check_dTD_ptr; //v2v
            
         if (xd_ptr) {
            if ((uint_32)check_dTD_ptr->SCRATCH_PTR->\
               XD_FOR_THIS_DTD != (uint_32)xd_ptr) 
            {
               /* Free the transfer descriptor */
               _usb_device_free_XD((pointer)xd_ptr);
            } /* Endif */
         } /* Endif */

         
         if (check_dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE) {

            /* Start CR 1015 */
            /* Prime the Endpoint */
            dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME = bit_pos;
            if (!(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS & 
               bit_pos)) 
            {
               while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME & bit_pos) {
                  /* Wait for the ENDPTPRIME to go to zero */
               } /* EndWhile */

               if (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS & 
                  bit_pos) 
               {
                  /* The endpoint was not not primed so no other transfers on 
                  ** the queue 
                  */
                  goto done;
               } /* Endif */
            } else {
               goto done;
            } /* Endif */

            /* No other transfers on the queue */
#ifdef _VIRT_MEM
            ep_queue_head_ptr->NEXT_DTD_PTR = (uint_32)virt2phys((void *)check_dTD_ptr) ;
#else
            ep_queue_head_ptr->NEXT_DTD_PTR = (uint_32)check_dTD_ptr;
#endif
            ep_queue_head_ptr->SIZE_IOC_INT_STS = 0;
   
            /* Prime the Endpoint */
            dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME = bit_pos;
         } /* Endif */
      } /* Endif */
   } /* Endif */
done:
   
   return USB_OK;

   /* End CR 1015 */

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_cancel_transfer, SUCCESSFUL");
   #endif
   
} /* EndBody */

/* EOF */
