/* ###########################################################################
# (c) Copyright Conexant Limited 2006
#
# Conexant Limited Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Limited ("Conexant").  It may only be read, used,
# copied, adapted, modified or otherwise dealt with by you if you have
# entered into a confidentiality agreement with Conexant and then subject to the
# terms of that confidentiality agreement and any other applicable agreement
# between you and Conexant.  If you are in any doubt as to whether you are
# entitled to access, read, use, copy, adapt, modify or otherwise deal with
# the Software or whether you are entitled to disclose the Software to any
# other person you should contact Conexant.  If you have not entered into a
# confidentiality agreement with Conexant granting access to this Software you
# should forthwith return all media, copies and printed listings containing
# the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you breach
# the above provisions.
#
# If you are unsure, or to report violations, please contact
# support@conexant.com
# ##########################################################################*/

/**HEADER*********************************************************
-----------------------------------------------------------------------------
 CONFIDENTIAL AND PROPRIETARY INFORMATION OF TRANSDIMENSION, INC. AND/OR ITS
 WHOLLY OWNED SUBSIDIARY SOFTCONNEX TECHNOLOGIES, INC. THIS NOTICE IS NOT TO
 BE DELETED, MODIFIED, MOVED OR CHANGED IN ANY WAY.

 (TransDimension, Inc. and SoftConnex Technologies, Inc. are referred to
 collectively below as TransDimension, Inc.) 
 Copyright (c) 1999 - 2004 by TransDimension, Inc. 

 This software is protected by copyright laws and international copyright 
 treaties, as well as other intellectual property laws and treaties.  This
 software is a CONFIDENTIAL, unpublished work of authorship, and with portions 
 constituting TRADE SECRETS of TransDimension, Inc..  Any unauthorized use, 
 disclosure, and/or reproduction of this software, or any part of this software;
 or distribution of this software in any form or by any means; or storage of 
 this software in any database or retrieval system, without the express written
 consent of, and license from, TransDimension, Inc. is strictly prohibited.  
 This software is protected under the copyright and/or trade secret laws in 
 other countries in addition to USA.  All Rights Reserved.  Failure to abide by
 the use, disclosure and/or reproduction restrictions may result in civil and
 or criminal penalties, and will be prosecuted to the maximum extent of the law.
 -------------------------------------------------------------------------------

*** Description:      
***  This file contains the main VUSB_HS Device Controller interface 
***  functions.
***                                                               
**************************************************************************
**END*********************************************************/

#include "devapi.h"
#include "usbprv_dev.h"
#include <linux/interrupt.h>
#ifdef __USB_OS_MQX__
   #include "mqx_arc.h"
#endif

#define UNUSED(_param) ((void)(_param))

USB_DEV_STATE_STRUCT_PTR global_usb_device_state_struct_ptr = NULL;

#ifdef __MISSING_RESUME_INTERRUPT_WORKAROUND__
uint_32 port_change_counter=0;
uint_32 port_change_log[100];
uint_32 sof_after_suspend =0;
#endif /* __MISSING_RESUME_INTERRUPT_WORKAROUND__ */

unsigned int Usb2PortValue = 0;
extern unsigned char setup_state;
static inline void _usb_process_tr_complete(uint_32 ep_num, uint_32 direction, USB_DEV_STATE_STRUCT_PTR usb_dev_ptr);

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_init
*  Returned Value : USB_OK or error code
*  Comments       :
*        Initializes the USB device controller.
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_init
   (
      /* [IN] the USB device controller to initialize */
      uint_8                     devnum,

      /* [OUT] the USB_dev_initialize state structure */
      _usb_device_handle         handle
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   VUSB20_REG_STRUCT_PTR                        cap_dev_ptr;
   uint_32                                      temp;
#ifndef CONFIG_ARCH_SOLOSW_GALLUS
   uint_32                                      total_memory=0;
   uchar_ptr                                    driver_memory;
#endif

   DEBUG_LOG_TRACE_SPL("_usb_dci_vusb20_init");
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   DEBUG_LOG_TRACE_SPL("usb_dev_ptr %x\n",usb_dev_ptr);
   cap_dev_ptr = (VUSB20_REG_STRUCT_PTR)
      _bsp_get_usb_capability_register_base(devnum);
   DEBUG_LOG_TRACE_SPL("cap_dev_ptr %x\n",cap_dev_ptr);
      
   /* Get the base address of the VUSB_HS registers */
   usb_dev_ptr->DEV_PTR = _bsp_get_usb_capability_register_base(devnum);
   DEBUG_LOG_TRACE("usb_dev_ptr->DEV_PTR=%x\n", usb_dev_ptr->DEV_PTR);
   global_usb_device_state_struct_ptr = usb_dev_ptr;

   /* Get the maximum number of endpoints supported by this USB controller */
   usb_dev_ptr->MAX_ENDPOINTS = 
       (cap_dev_ptr->REGISTERS.CAPABILITY_REGISTERS.DCC_PARAMS & 
        VUSB20_MAX_ENDPTS_SUPPORTED);
   
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;
   
   temp = (usb_dev_ptr->MAX_ENDPOINTS * 2);

#ifndef CONFIG_ARCH_SOLOSW_GALLUS
   /****************************************************************   
         Consolidated memory allocation    
   ****************************************************************/   
   total_memory = ((temp * sizeof(VUSB20_EP_QUEUE_HEAD_STRUCT)) + 2048) +
       ((MAX_EP_TR_DESCRS * sizeof(VUSB20_EP_TR_STRUCT)) + 32) +
       (sizeof(SCRATCH_STRUCT)*MAX_EP_TR_DESCRS);

   driver_memory = (uchar_ptr) USB_Uncached_memalloc(total_memory);
   
   if (driver_memory == NULL) {
       #ifdef _DEVICE_DEBUG_
       DEBUG_LOG_TRACE("_usb_dci_vusb20_init, malloc failed");
       #endif
       return USBERR_ALLOC;
   }
   
   #ifdef _DATA_CACHE_   
   /****************************************************************   
      Flush the zeroed memory if cache is enabled
   ****************************************************************/   
   USB_dcache_flush_mlines((pointer)driver_memory,total_memory);
   
   #endif

   /****************************************************************   
     Keep a pointer to driver memory alloctaion
   ****************************************************************/   
   usb_dev_ptr->DRIVER_MEMORY = driver_memory;
#endif   
                    
   /****************************************************************   
      Assign QH base
   ****************************************************************/   
   usb_dev_ptr->EP_QUEUE_HEAD_BASE = usb2_allocateFromHeap(
       temp * sizeof(VUSB20_EP_QUEUE_HEAD_STRUCT) + 2048,
       2048);
   if (usb_dev_ptr->EP_QUEUE_HEAD_BASE == NULL)
   {
       USBERR("Can't allocate queueHeadBuf\n");
       return USBERR_ALLOC;
   }

   USB_memzero((void *)usb_dev_ptr->EP_QUEUE_HEAD_BASE,
                temp * sizeof(VUSB20_EP_QUEUE_HEAD_STRUCT) + 2048);



   usb_dev_ptr->EP_QUEUE_HEAD_PTR = usb_dev_ptr->EP_QUEUE_HEAD_BASE;

   DEBUG_LOG_TRACE_SPL("usb_dev_ptr->EP_QUEUE_HEAD_PTR %x\n",usb_dev_ptr->EP_QUEUE_HEAD_PTR);


   /****************************************************************   
      Assign DTD base
   ****************************************************************/   
   usb_dev_ptr->DTD_BASE_PTR = usb2_allocateFromHeap(
       MAX_EP_TR_DESCRS * sizeof(VUSB20_EP_TR_STRUCT) + 2048,
       64);

   if (usb_dev_ptr->DTD_BASE_PTR == NULL)
   {
       USBERR("Can't allocate tdBuf\n");
       return USBERR_ALLOC;
   }
           

   USB_memzero((void *)usb_dev_ptr->DTD_BASE_PTR,
                MAX_EP_TR_DESCRS * sizeof(VUSB20_EP_TR_STRUCT) + 2048);
   usb_dev_ptr->DTD_ALIGNED_BASE_PTR = usb_dev_ptr->DTD_BASE_PTR;
       
   /****************************************************************   
      Assign SCRATCH Structure base
   ****************************************************************/   
   usb_dev_ptr->SCRATCH_STRUCT_BASE = usb2_allocateFromHeap(
       sizeof(SCRATCH_STRUCT)*MAX_EP_TR_DESCRS + 2048,
       4);

   if (usb_dev_ptr->SCRATCH_STRUCT_BASE == NULL)
   {
       USBERR("Can't allocate scratch buf\n");
       return USBERR_ALLOC;
   }

   USB_memzero(usb_dev_ptr->SCRATCH_STRUCT_BASE,
                sizeof(SCRATCH_STRUCT)*MAX_EP_TR_DESCRS + 2048);


   usb_dev_ptr->USB_STATE = USB_STATE_UNKNOWN;

   /* Initialize the VUSB_HS controller */   
   _usb_dci_vusb20_chip_initialize((pointer)usb_dev_ptr);
   
   DEBUG_LOG_TRACE_SPL("_usb_dci_vusb20_init, SUCCESSFUL");

   return USB_OK;   
} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_chip_initialize
*  Returned Value : USB_OK or error code
*  Comments       :
*        Initializes the USB device controller.
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_chip_initialize
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   VUSB20_REG_STRUCT_PTR                        cap_dev_ptr;
   VUSB20_EP_QUEUE_HEAD_STRUCT_PTR              ep_queue_head_ptr;
   VUSB20_EP_TR_STRUCT_PTR                      dTD_ptr;
   uint_32                                      i, port_control;
   SCRATCH_STRUCT_PTR                           temp_scratch_ptr;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE_SPL("_usb_dci_vusb20_chip_initialize");
   #endif
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   DEBUG_LOG_TRACE_SPL("usb_dev_ptr=%x\n", usb_dev_ptr);
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;
   DEBUG_LOG_TRACE_SPL("dev_ptr=%x\n", dev_ptr);
   cap_dev_ptr = (VUSB20_REG_STRUCT_PTR)
      _bsp_get_usb_capability_register_base(usb_dev_ptr->DEV_NUM);
   DEBUG_LOG_TRACE_SPL("cap_dev_ptr=%x\n", cap_dev_ptr);
   /* Stop the controller */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD &= 
      ~EHCI_CMD_RUN_STOP;
      
#ifdef CONFIG_ARCH_SOLOSW_GALLUS

   /* Select the phy interface to use, ULPI for 2.0, Serial for 1.1 */
   port_control = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[0];   
   port_control &= ~(EHCI_PORTSCX_XCVR_SELECT_MASK);

   if (readl(SOLOS_LAHB_VIRT + SOLOS_DEVCONF_CONTROL) & SOLOS_DEVCONF_CONFIG_EXT_USB_PHY_EN)
	{
	 port_control |= EHCI_PORTSCX_XCVR_SELECT_ULPI;
	DEBUG_LOG_TRACE_D("BF_DEVCONF_CONFIG_EXT_USB_PHY_EN\n");
	}
   else
        {
	 port_control |= EHCI_PORTSCX_XCVR_SELECT_SERIAL;
	DEBUG_LOG_TRACE_D("BF_DEVCONF_CONFIG_EXT_USB_PHY_EN - NO\n");
	}
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[0] = port_control;

   /* Initialize Stony Point to fix power sequencing problems */
   if (readl(SOLOS_LAHB_VIRT + SOLOS_DEVCONF_CONTROL) & SOLOS_DEVCONF_CONFIG_EXT_USB_PHY_EN)
   {
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ULPI_VIEWPORT = 0x20040055;
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ULPI_VIEWPORT = 0x60040055;
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ULPI_VIEWPORT = 0x40040055;
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ULPI_VIEWPORT = 0x20040015;
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ULPI_VIEWPORT = 0x60040015;
   }
#endif
      
   /* Reset the controller to get default values */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD = EHCI_CMD_CTRL_RESET;

   while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD & 
      EHCI_CMD_CTRL_RESET) 
   {
      /* Wait for the controller reset to complete */
   } /* EndWhile */

   #ifdef TRIP_WIRE
   /* Program the controller to be the USB device controller */ 
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_MODE = 
      (VUSBHS_MODE_CTRL_MODE_DEV | VUSBHS_MODE_SETUP_LOCK_DISABLE);
   #else
   /* Program the controller to be the USB device controller */ 
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_MODE = 
      VUSBHS_MODE_CTRL_MODE_DEV;
   #endif

   /* Initialize the internal dTD head and tail to NULL */   
   usb_dev_ptr->DTD_HEAD = NULL;
   usb_dev_ptr->DTD_TAIL = NULL;

   /* Make sure the 16 MSBs of this register are 0s */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT = 0;
   
   ep_queue_head_ptr = usb_dev_ptr->EP_QUEUE_HEAD_PTR; //virt to virt
   DEBUG_LOG_TRACE_SPL("ep_queue_head_ptr=%x\n",ep_queue_head_ptr);

   /* Initialize all device queue heads */
   for (i=0;i<(usb_dev_ptr->MAX_ENDPOINTS * 2U);i++) {
      /* Interrupt on Setup packet */
      (ep_queue_head_ptr + i)->MAX_PKT_LENGTH = 
         (((uint_32)USB_MAX_CTRL_PAYLOAD << 
         VUSB_EP_QUEUE_HEAD_MAX_PKT_LEN_POS) | VUSB_EP_QUEUE_HEAD_IOS);
      (ep_queue_head_ptr + i)->NEXT_DTD_PTR = 
         VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE;
   } /* Endfor */

   /* Configure the Endpoint List Address */
#ifdef _VIRT_MEM
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR =  (uint_32)virt2phys((void *)ep_queue_head_ptr);
#else
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR = 
      (uint_32)ep_queue_head_ptr;
#endif

   if (cap_dev_ptr->REGISTERS.CAPABILITY_REGISTERS.HCS_PARAMS & 
       VUSB20_HCS_PARAMS_PORT_POWER_CONTROL_FLAG) 
   {
      port_control = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[0];
      port_control &= (~EHCI_PORTSCX_W1C_BITS | ~EHCI_PORTSCX_PORT_POWER);
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[0] = port_control;
   } /* Endif */
   
   dTD_ptr = usb_dev_ptr->DTD_ALIGNED_BASE_PTR; //v2v

   temp_scratch_ptr = usb_dev_ptr->SCRATCH_STRUCT_BASE; //v2v

   /* Enqueue all the dTDs */   
   for (i=0;i<MAX_EP_TR_DESCRS;i++) {
      dTD_ptr->SCRATCH_PTR = temp_scratch_ptr;
      /* Set the dTD to be invalid */
      dTD_ptr->NEXT_TR_ELEM_PTR = VUSBHS_TD_NEXT_TERMINATE;
      /* Set the Reserved fields to 0 */
      dTD_ptr->SIZE_IOC_STS &= ~VUSBHS_TD_RESERVED_FIELDS;
      _usb_dci_vusb20_free_dTD((pointer)dTD_ptr);
      dTD_ptr++;
      temp_scratch_ptr++;
   } /* Endfor */

   /* Initialize the endpoint 0 properties */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[0] = 
      (EHCI_EPCTRL_TX_DATA_TOGGLE_RST | EHCI_EPCTRL_RX_DATA_TOGGLE_RST);
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[0] &= 
      ~(EHCI_EPCTRL_TX_EP_STALL | EHCI_EPCTRL_RX_EP_STALL);
   
   /* Enable interrupts don't enable if one is disabled by a compiler switch*/
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_INTR = 
         (
            EHCI_INTR_INT_EN | 
          #ifndef ERROR_INTERRUPT_DISABLE
            EHCI_INTR_ERR_INT_EN | 
          #endif
          #ifndef PORT_CHANGE_INTERRUPT_DISABLE
            EHCI_INTR_PORT_CHANGE_DETECT_EN | 
          #endif  
          #ifndef SOF_INTERRUPT_DISABLE  
            EHCI_INTR_SOF_UFRAME_EN | 
          #endif
          #ifndef SUSPEND_INTERRUPT_DISABLE  
            EHCI_INTR_DEVICE_SUSPEND |
          #endif  
            EHCI_INTR_RESET_EN
          );

    #ifndef SMALL_CODE_SIZE
      usb_dev_ptr->USB_STATE = USB_STATE_UNKNOWN;
    #endif
    
   /* Set the Run bit in the command register */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD = EHCI_CMD_RUN_STOP;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE_SPL("_usb_dci_vusb20_chip_initialize, SUCCESSFUL");
   #endif
   
} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_free_dTD
*  Returned Value : void
*  Comments       :
*        Enqueues a dTD onto the free DTD ring.
*
*END*-----------------------------------------------------------------*/

void _usb_dci_vusb20_free_dTD
   (
      /* [IN] the dTD to enqueue */
      pointer  dTD_ptr
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)
      global_usb_device_state_struct_ptr;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_free_dTD");
   #endif

   /*
   ** This function can be called from any context, and it needs mutual
   ** exclusion with itself.
   */
   USB_lock();

   /*
   ** Add the dTD to the free dTD queue (linked via PRIVATE) and
   ** increment the tail to the next descriptor
   */
   EHCI_DTD_QADD(usb_dev_ptr->DTD_HEAD, usb_dev_ptr->DTD_TAIL, 
      (VUSB20_EP_TR_STRUCT_PTR)dTD_ptr);
   usb_dev_ptr->DTD_ENTRIES++;

   USB_unlock();

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_free_dTD, SUCCESSFUL");
   #endif
   

} /* Endbody */
extern void dispData(unsigned char *, int);
/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_add_dTD
*  Returned Value : USB_OK or error code
*  Comments       :
*        Adds a device transfer desriptor(s) to the queue.
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_add_dTD
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
     
      /* [IN] The transfer descriptor address */
      XD_STRUCT_PTR              xd_ptr
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   VUSB20_EP_TR_STRUCT_PTR                      dTD_ptr, temp_dTD_ptr, first_dTD_ptr = NULL;
   VUSB20_EP_QUEUE_HEAD_STRUCT_PTR              ep_queue_head_ptr;
   uint_32                                      curr_pkt_len, remaining_len; 
   uint_32                                      curr_offset, temp, bit_pos, temp_ep_stat;
   boolean                                      read_safe;
   /*********************************************************************
   For a optimal implementation, we need to detect the fact that
   we are adding DTD to an empty list. If list is empty, we can
   actually skip several programming steps esp. those for ensuring
   that there is no race condition.The following boolean will be useful
   in skipping some code here.
   *********************************************************************/
   boolean                                      list_empty = FALSE;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE_SPL("_usb_dci_vusb20_add_dTD");
   #endif

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

#ifdef _VIRT_MEM
   xd_ptr->WSTARTADDRESS += xd_ptr->OFFSET ;
#endif

   remaining_len = xd_ptr->WTOTALLENGTH;

   curr_offset = 0;
   temp = (2 * xd_ptr->EP_NUM + xd_ptr->BDIRECTION);
   bit_pos = (1 << (16 * xd_ptr->BDIRECTION + xd_ptr->EP_NUM));

#ifdef _VIRT_MEM
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      phys2virt(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR) + 
      temp;
#else
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR + 
      temp;
#endif

   /*********************************************************************
   This loops iterates through the length of the transfer and divides
   the data in to DTDs each handling the a max of 0x4000 bytes of data.
   The first DTD in the list is stored in a pointer called first_dTD_ptr.
   This pointer is later linked in to QH for processing by the hardware.   
   *********************************************************************/

   do {
      /* Check if we need to split the transfer into multiple dTDs */
      if (remaining_len > VUSB_EP_MAX_LENGTH_TRANSFER) {
         curr_pkt_len = VUSB_EP_MAX_LENGTH_TRANSFER;
      } else {
         curr_pkt_len = remaining_len;
      } /* Endif */
   
      remaining_len -= curr_pkt_len;

      /* Get a dTD from the queue */   
      EHCI_DTD_QGET(usb_dev_ptr->DTD_HEAD, usb_dev_ptr->DTD_TAIL, dTD_ptr);

      if (!dTD_ptr) {
            printk("_usb_dci_vusb20_add_dTD, FAILED");
         #ifdef _DEVICE_DEBUG_
            DEBUG_LOG_TRACE("_usb_dci_vusb20_add_dTD, FAILED");
         #endif
         return USBERR_TR_FAILED;
      } /* Endif */

      usb_dev_ptr->DTD_ENTRIES--;

      if (curr_offset == 0) {
         first_dTD_ptr = dTD_ptr;
	
	DEBUG_LOG_TRACE_SPL("first_dTD_ptr:%x\n",first_dTD_ptr);
      } /* Endif */

      #ifdef _DATA_CACHE_
      /**************************************************************
      USB Memzero does not bypass the cache and hence we must use
      DTD pointer to update the memory and bypass the cache. If
      your DTD are allocated from an uncached regigio, you can
      eliminitate this approach and switch back to USB_memzero().
      **************************************************************/
      
         dTD_ptr->NEXT_TR_ELEM_PTR   = 0;
         dTD_ptr->SIZE_IOC_STS       = 0;
         dTD_ptr->BUFF_PTR0          = 0;
         dTD_ptr->BUFF_PTR1          = 0;
         dTD_ptr->BUFF_PTR2          = 0;
         dTD_ptr->BUFF_PTR3          = 0;
         dTD_ptr->BUFF_PTR4          = 0;

      #else
         /* Zero the dTD. Leave the last 4 bytes as that is the scratch pointer */
         USB_memzero((void *) dTD_ptr,(sizeof(VUSB20_EP_TR_STRUCT) - 4));
      #endif   

      /* Set the Terminate bit */
      dTD_ptr->NEXT_TR_ELEM_PTR = VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE; //vOK


      /*************************************************************
      FIX ME: For hig-speed and high-bandwidth ISO IN endpoints,
      we must initialize the multiplied field so that Host can issues
      multiple IN transactions on the endpoint. See the DTD data
      structure for MultiIO field.
      
      S Garg 11/06/2003
      *************************************************************/
      
      /* Fill in the transfer size */
      if (!remaining_len) {
         dTD_ptr->SIZE_IOC_STS = ((curr_pkt_len << 
            VUSBHS_TD_LENGTH_BIT_POS) | (VUSBHS_TD_IOC) | 
            (VUSBHS_TD_STATUS_ACTIVE));
      } else {
         dTD_ptr->SIZE_IOC_STS = ((curr_pkt_len << VUSBHS_TD_LENGTH_BIT_POS) 
            | VUSBHS_TD_STATUS_ACTIVE);
      } /* Endif */
 
      dTD_ptr->SIZE_IOC_STS &= ~VUSBHS_TD_RESERVED_FIELDS;

      /* 4K apart buffer page pointers */
#ifdef _VIRT_MEM
      dTD_ptr->BUFF_PTR0 = (uint_32)((uint_8*) virt2phys((void*) xd_ptr->WSTARTADDRESS) +
		                                                           curr_offset) ;
#else
      dTD_ptr->BUFF_PTR0 = (uint_32)(xd_ptr->WSTARTADDRESS + curr_offset);
#endif
      dTD_ptr->BUFF_PTR1 = (dTD_ptr->BUFF_PTR0 + 4096);
      dTD_ptr->BUFF_PTR2 = (dTD_ptr->BUFF_PTR1 + 4096);
      dTD_ptr->BUFF_PTR3 = (dTD_ptr->BUFF_PTR2 + 4096);
      dTD_ptr->BUFF_PTR4 = (dTD_ptr->BUFF_PTR3 + 4096);

      curr_offset += curr_pkt_len;

      /* Maintain the first and last device transfer descriptor per 
      ** endpoint and direction 
      */
      if (!usb_dev_ptr->EP_DTD_HEADS[temp]) 
      {
         usb_dev_ptr->EP_DTD_HEADS[temp] = dTD_ptr; 
         /***********************************************
         If list does not have a head, it means that list
         is empty. An empty condition is detected.
         ***********************************************/
         list_empty = TRUE;
      } /* Endif */ 
   
      /* Check if the transfer is to be queued at the end or beginning */
      temp_dTD_ptr = usb_dev_ptr->EP_DTD_TAILS[temp];
      
      /* Remember which XD to use for this dTD */
      dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD = (pointer)xd_ptr;
      
      /* New tail */
      usb_dev_ptr->EP_DTD_TAILS[temp] = dTD_ptr; //vOK
      if (temp_dTD_ptr) {
         /* Should not do |=. The Terminate bit should be zero */
#ifdef _VIRT_MEM
         temp_dTD_ptr->NEXT_TR_ELEM_PTR = (uint_32)virt2phys((void*) dTD_ptr) ;
#else
         temp_dTD_ptr->NEXT_TR_ELEM_PTR = (uint_32)dTD_ptr;
#endif
      } /* Endif */

   } while (remaining_len); /* EndWhile */


   /**************************************************************
   In the loop above DTD has already been added to the list
   However endpoint has not been primed yet. If list is not empty we need safter ways to add DTD to the
   existing list. Else we just skip to adding DTD to QH safely.
   **************************************************************/
   if(!list_empty)
   {
       /*********************************************************
               Hardware v3.2+ require the use of semaphore to ensure that
               QH is safely updated.
       *********************************************************/
               
       /*********************************************************
               Check the prime bit. If set goto done
       *********************************************************/
       if (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME & bit_pos) 
       {
	goto done;
       }

       read_safe = FALSE;                      
       while(!read_safe)
       {

           /*********************************************************
           start with setting the semaphores
           *********************************************************/
           dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD |= 
               EHCI_CMD_ATDTW_TRIPWIRE_SET;
                      
           /*********************************************************
           Read the endpoint status
           *********************************************************/
                                                 
           temp_ep_stat = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS & bit_pos; 

           /*********************************************************
           Reread the ATDTW semaphore bit to check if it is cleared.
           When hardware see a hazard, it will clear the bit or
           else we remain set to 1 and we can proceed with priming
           of endpoint if not already primed.
           *********************************************************/
           if(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD & 
              EHCI_CMD_ATDTW_TRIPWIRE_SET)
           {
               read_safe = TRUE;
           }

       }/*end while loop */

       /*********************************************************
       Clear the semaphore
       *********************************************************/
       dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD &= 
           EHCI_CMD_ATDTW_TRIPWIRE_CLEAR;

       /*********************************************************
       If endpoint is not active, we activate it now.               
       *********************************************************/
       if(!temp_ep_stat)
       {
           /* No other transfers on the queue */
#if 1 
#ifdef _VIRT_MEM
           ep_queue_head_ptr->NEXT_DTD_PTR = (uint_32) virt2phys((void*) first_dTD_ptr) ;
#else
           ep_queue_head_ptr->NEXT_DTD_PTR = (uint_32)first_dTD_ptr;
#endif
           ep_queue_head_ptr->SIZE_IOC_INT_STS = 0;

           /* Prime the Endpoint */
           dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME = bit_pos;
#else
 	   /* Add the next active transfer on the queue */
           temp_dTD_ptr = usb_dev_ptr->EP_DTD_HEADS[temp];
           if((unsigned long)(phys2virt((void*)ep_queue_head_ptr->NEXT_DTD_PTR)) == VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE)
           {
               while(  temp_dTD_ptr &&
                      ((unsigned long)(phys2virt((void*)(temp_dTD_ptr->NEXT_TR_ELEM_PTR)))!=VUSB_EP_QUEUE_HEAD_NEXT_TERMINATE) &&
                     !(temp_dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE))
               {
                   temp_dTD_ptr=(VUSB20_EP_TR_STRUCT_PTR)phys2virt((void*)(temp_dTD_ptr->NEXT_TR_ELEM_PTR));
               }
               if(temp_dTD_ptr && (temp_dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE))
                   ep_queue_head_ptr->NEXT_DTD_PTR=(uint_32)virt2phys((void*)temp_dTD_ptr);
           }

           /* Prime the Endpoint */
           dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME = bit_pos;
#endif
       }

   }
   else
   {
         /* No other transfers on the queue */
#ifdef _VIRT_MEM
       ep_queue_head_ptr->NEXT_DTD_PTR = (uint_32) virt2phys((void*) first_dTD_ptr) ;
#else
       ep_queue_head_ptr->NEXT_DTD_PTR = (uint_32)first_dTD_ptr;
#endif
       ep_queue_head_ptr->SIZE_IOC_INT_STS = 0;
   
	/* Prime the Endpoint */
       dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME = bit_pos;
   }

done:
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE_SPL("_usb_dci_vusb20_add_dTD, SUCCESSFUL");
   #endif
   return USB_OK;
} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_send_data
*  Returned Value : USB_OK or error code
*  Comments       :
*        Sends data by adding and executing the dTD. Non-blocking.
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_send_data
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
     
      /* [IN] The transfer descriptor address */
      XD_STRUCT_PTR              xd_ptr
   )
{ /* Body */

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE_SPL("_usb_dci_vusb20_send_data, SUCCESSFUL");
   #endif

   /* Add and execute the device transfer descriptor */
   return(_usb_dci_vusb20_add_dTD(handle, xd_ptr));
} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_recv_data
*  Returned Value : USB_OK or error code
*  Comments       :
*        Receives data by adding and executing the dTD. Non-blocking.
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_recv_data
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
     
      /* [IN] The transfer descriptor address */
      XD_STRUCT_PTR              xd_ptr
   )
{ /* Body */

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_recv_data, SUCCESSFUL");
   #endif
   /* Add and execute the device transfer descriptor */
   return(_usb_dci_vusb20_add_dTD(handle, xd_ptr));
} /* EndBody */


/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_process_tr_complete
*  Returned Value : None
*  Comments       :
*        Services transaction complete interrupt
*
*END*-----------------------------------------------------------------*/
#ifdef USB_TASKLET
void usb_TrCompleteTaskletHandler(unsigned long data)
#else
void _usb_dci_vusb20_process_tr_complete
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
   )
#endif
{ /* Body */
    USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
    VUSB20_REG_STRUCT_PTR                        dev_ptr;
    uint_32                                      ep_num, direction, bit_pos,i;
  
#ifdef USB_TASKLET
    _usb_device_handle handle;
    usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)global_usb_device_state_struct_ptr;
    handle = (_usb_device_handle)usb_dev_ptr;
#else
    usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
#endif
    dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

#ifdef _DEVICE_DEBUG_
    DEBUG_LOG_TRACE_SPL("_usb_dci_vusb20_process_tr_complete");
#endif
	
#if 1
   /* Process any zero len packets received/transmitted before the setup packets
    * to maintain the setup state machine
    */	 
   bit_pos = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCOMPLETE;
   if (unlikely ((setup_state == TX_ZERO_LEN) && (bit_pos & 0x00010000))){
        setup_state = 0;
        /* Process transmit complete for ep0 */
        dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCOMPLETE = (0x00010000);
   	_usb_process_tr_complete(0, 1, usb_dev_ptr);
   }	  
   else if (unlikely ((setup_state == RX_ZERO_LEN) && (bit_pos & 0x00000001))){
        /* Process receive complete for ep0 */
        setup_state = 0;
        dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCOMPLETE = (0x00000001);
    	_usb_process_tr_complete(0, 0, usb_dev_ptr);
   }	 	
#endif   
    /* We use separate loops for ENDPTSETUPSTAT and ENDPTCOMPLETE because the 
    ** setup packets are to be read ASAP 
    */
 
    /* Process all Setup packet received interrupts */
    bit_pos = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT;

    if (likely (bit_pos)) {
        for (i=0;i<USB_MAX_CONTROL_ENDPOINTS;i++) {
            if (bit_pos & (1 << i)) {
                _usb_device_call_service(handle, i, TRUE, 0, 0, 8, 0);
            } /* Endif */
        } /* Endfor */
    } /* Endif */

    /* Don't clear the endpoint setup status register here. It is cleared as a 
    ** setup packet is read out of the buffer 
    */

    /* Process non-setup transaction complete interrupts */   
    bit_pos = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCOMPLETE;

    /* Clear the bits in the register */
    dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCOMPLETE = bit_pos;

    if (likely (bit_pos))
    {
        /* Get the endpoint number and the direction of transfer */
      
        for (i=0;i<USB_MAX_ENDPOINTS*2;i++)
        {
            if ((i < USB_MAX_ENDPOINTS) && (bit_pos & (1 << i)))
            {
                ep_num = i;
                direction = 0;
            }
            else if((i >= USB_MAX_ENDPOINTS) && (bit_pos & (1 << (i+16-USB_MAX_ENDPOINTS))))
            {
                ep_num = (i - USB_MAX_ENDPOINTS);
                direction = 1;
            }
            else
            {
                continue;
            }
            DEBUG_LOG_TRACE("ep=%ld dir=%ld\n", ep_num, direction);
	    _usb_process_tr_complete(ep_num, direction, usb_dev_ptr);
#if 0            
            temp = (2*ep_num + direction);

            /* Get the first dTD */      
            dTD_ptr = usb_dev_ptr->EP_DTD_HEADS[temp];
            DEBUG_LOG_TRACE("dTD_ptr=%x\n", dTD_ptr);
            
#ifdef _VIRT_MEM
            ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
                phys2virt(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR) + 
                temp;
#else
            ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
                dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR + 
                temp;
#endif
            /* Process all the dTDs for respective transfers */
	    loop_cnt=0;	    
            while (dTD_ptr)
            {
                if (dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE) {
                    // USBTRACE("no dTDs, ep=%d dir=%d td=%x", ep_num, direction, dTD_ptr);
                    break;
                } /* Endif */
#if 1
		if((direction == USB_RECV) && ((++loop_cnt)==33))
			break; 
#endif
               
                /* Get the correct internal transfer descriptor */
                xd_ptr = (XD_STRUCT_PTR)dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD;
                if (xd_ptr) {
                    buff_start_address = xd_ptr->WSTARTADDRESS;
                    actual_transfer_length = xd_ptr->WTOTALLENGTH;
                    temp_xd_ptr = xd_ptr;
                } /* Endif */
               
                /* Get the address of the next dTD */
#ifdef _VIRT_MEM
                temp_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)
                    (phys2virt(dTD_ptr->NEXT_TR_ELEM_PTR & VUSBHS_TD_ADDR_MASK));

#else
                temp_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)
                    (dTD_ptr->NEXT_TR_ELEM_PTR & VUSBHS_TD_ADDR_MASK);
#endif
                  
                /* Read the errors */
                errors = (dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_ERROR_MASK);
                  
                if (!errors) {
                    /* No errors */
                    /* Get the length of transfer from the current dTD */   
                    remaining_length += ((dTD_ptr->SIZE_IOC_STS & VUSB_EP_TR_PACKET_SIZE) >> 16);
                    actual_transfer_length -= remaining_length;
                }
                else {
                    if (errors & VUSBHS_TD_STATUS_HALTED) {
                         printk("%s (ERR): %x", errors);
                        /* Clear the errors and Halt condition */
                        ep_queue_head_ptr->SIZE_IOC_INT_STS &= ~errors;                        
                    } /* Endif */
                } /* Endif */
               
                /* Retire the processed dTD */
                _usb_dci_vusb20_cancel_transfer(handle, ep_num, direction);
                if (temp_dTD_ptr) {
                    if ((uint_32)temp_dTD_ptr->SCRATCH_PTR->\
                        XD_FOR_THIS_DTD != (uint_32)temp_xd_ptr) 
                    {
                        /* Transfer complete. Call the register service function for the 
                        ** endpoint 
                        */
                        _usb_device_call_service(handle, ep_num, FALSE, direction, 
                                                 buff_start_address, actual_transfer_length, errors);
                        remaining_length = 0;
                    } /* Endif */
                } else {
                    /* Transfer complete. Call the register service function for the 
                    ** endpoint 
                    */
                    _usb_device_call_service(handle, ep_num, FALSE, direction, 
                                             buff_start_address, actual_transfer_length, errors);
                } /* Endif */
                dTD_ptr = temp_dTD_ptr;
                errors = 0;
            } /* Endwhile */
#endif
        } /* Endif */
    } /* Endif */

#ifdef USB_TASKLET
    /* Enable Interrupt for Transaction Complete */
    dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_INTR |= EHCI_INTR_INT_EN;
#endif
 
#ifdef _DEVICE_DEBUG_
    DEBUG_LOG_TRACE_SPL("_usb_dci_vusb20_process_tr_complete, SUCCESSFUL");
#endif
   
} /* EndBody */

static inline void _usb_process_tr_complete(uint_32 ep_num, uint_32 direction, USB_DEV_STATE_STRUCT_PTR usb_dev_ptr)
{
    VUSB20_REG_STRUCT_PTR                        dev_ptr;
    VUSB20_EP_TR_STRUCT_PTR                      dTD_ptr;
    VUSB20_EP_TR_STRUCT_PTR                      temp_dTD_ptr;
    VUSB20_EP_QUEUE_HEAD_STRUCT_PTR              ep_queue_head_ptr;
    uint_32                                      temp, loop_cnt=0;
    uint_32                                      remaining_length = 0;
    uint_32                                      actual_transfer_length = 0;
    uint_32                                      errors = 0;
    XD_STRUCT_PTR                                xd_ptr;
    XD_STRUCT_PTR                                temp_xd_ptr = NULL;
    uchar_ptr                                    buff_start_address = NULL;
    _usb_device_handle handle;


    dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;
    handle = (_usb_device_handle)usb_dev_ptr;

	
    temp = (2*ep_num + direction);

    /* Get the first dTD */
    dTD_ptr = usb_dev_ptr->EP_DTD_HEADS[temp];
    DEBUG_LOG_TRACE("dTD_ptr=%x\n", dTD_ptr);

    ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
                phys2virt(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR) +
                temp;

    /* Process all the dTDs for respective transfers */
    loop_cnt = 0;	
    while (dTD_ptr)
    {
                if (dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_STATUS_ACTIVE) {
                   break;
                } /* Endif */
#if 1
		/* The loop_cnt check is with the qlen defined in ether.c */
		if((direction == USB_RECV) && ((++loop_cnt)==33))
			break; 
#endif

                /* Get the correct internal transfer descriptor */
                xd_ptr = (XD_STRUCT_PTR)dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD;
                if (likely (xd_ptr)) {
                    buff_start_address = xd_ptr->WSTARTADDRESS;
                    actual_transfer_length = xd_ptr->WTOTALLENGTH;
                    temp_xd_ptr = xd_ptr;
                } /* Endif */

                /* Get the address of the next dTD */
                temp_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)
                    (phys2virt(dTD_ptr->NEXT_TR_ELEM_PTR & VUSBHS_TD_ADDR_MASK));

                /* Read the errors */
                errors = (dTD_ptr->SIZE_IOC_STS & VUSBHS_TD_ERROR_MASK);

                if (likely (!errors)) {
                    /* No errors */
                    /* Get the length of transfer from the current dTD */
                    remaining_length += ((dTD_ptr->SIZE_IOC_STS & VUSB_EP_TR_PACKET_SIZE) >> 16);
                    actual_transfer_length -= remaining_length;
                }
                else {
                    if (errors & VUSBHS_TD_STATUS_HALTED) {
                     /* Clear the errors and Halt condition */
                        ep_queue_head_ptr->SIZE_IOC_INT_STS &= ~errors;
                    } /* Endif */
                } /* Endif */

                /* Retire the processed dTD */
                _usb_dci_vusb20_cancel_transfer(handle, ep_num, direction);
                if (likely (temp_dTD_ptr)) {
                    if (likely ((uint_32)temp_dTD_ptr->SCRATCH_PTR->\
                        XD_FOR_THIS_DTD != (uint_32)temp_xd_ptr))
                    {
                        /* Transfer complete. Call the register service function for the
                        ** endpoint
                        */
                        _usb_device_call_service(handle, ep_num, FALSE, direction,
                                                 buff_start_address, actual_transfer_length, errors);
                        remaining_length = 0;
                    } /* Endif */
                } else {
                    /* Transfer complete. Call the register service function for the
                    ** endpoint
                    */
                    _usb_device_call_service(handle, ep_num, FALSE, direction,
                                             buff_start_address, actual_transfer_length, errors);
                } /* Endif */
                dTD_ptr = temp_dTD_ptr;
                errors = 0;
    } /* Endwhile */
}



/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_isr
*  Returned Value : None
*  Comments       :
*        Services all the VUSB_HS interrupt sources
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_isr
   (
       void* arg
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   uint_32                                      status;

   UNUSED(arg);
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)global_usb_device_state_struct_ptr;

   if (usb_dev_ptr->id != 0x12345678) 
   { 
       USBERR("bad id\n"); 
       return;
   }

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("***** _usb_dci_vusb20_isr *****");
   #endif


   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

//   for (;;) {
      status = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_STS;

      if (!(status & dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_INTR)) {
       //  break;
	return;
      } /* Endif */


	/* Clear all the interrupts occured */
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_STS = status;
      
      if (unlikely (status & EHCI_STS_RESET)) {
         _usb_dci_vusb20_process_reset((pointer)usb_dev_ptr);
      } /* Endif */

      #ifndef PORT_CHANGE_INTERRUPT_DISABLE   
      if (unlikely (status & EHCI_STS_PORT_CHANGE)) {
         _usb_dci_vusb20_process_port_change((pointer)usb_dev_ptr);
      } /* Endif */
      #endif

      #ifndef ERROR_INTERRUPT_DISABLE   
      if (unlikely (status & EHCI_STS_ERR)) {
         _usb_dci_vusb20_process_error((pointer)usb_dev_ptr);
      } /* Endif */
      #endif

      #ifndef SOF_INTERRUPT_DISABLE   
      if (likely(status & EHCI_STS_SOF)) {

         _usb_dci_vusb20_process_SOF((pointer)usb_dev_ptr);
      } /* Endif */
      #endif
         
      if (likely (status & EHCI_STS_INT)) {
#ifdef USB_TASKLET
         /* Disable Interrupt for Transaction Complete */
         dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_INTR &= ~EHCI_INTR_INT_EN;
	 tasklet_schedule((struct tasklet_struct *)usb_TrCompleteTasklet);
#else
         _usb_dci_vusb20_process_tr_complete((pointer)usb_dev_ptr);
#endif
      } /* Endif */

      #ifndef SUSPEND_INTERRUPT_DISABLE   
      if (unlikely (status & EHCI_STS_SUSPEND)) {
         _usb_dci_vusb20_process_suspend((pointer)usb_dev_ptr);
         return;
      } /* Endif */
      #endif
//   } /* Endfor */

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("***** _usb_dci_vusb20_isr, SUCCESSFUL *****\n");
   #endif

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_process_reset
*  Returned Value : None
*  Comments       :
*        Services reset interrupt
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_process_reset
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR             dev_ptr;
   uint_32                                      temp;
   uint_8                                       i;
   int                                          timeout;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE_D("_usb_dci_vusb20_process_reset");
   #endif

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;
   
   /* Inform the application so that it can cancel all previously queued transfers */
   _usb_device_call_service(usb_dev_ptr, USB_SERVICE_BUS_RESET, 0, 0, 0, 0, 0);

   /* The address bits are past bit 25-31. Set the address */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.DEVICE_ADDR &= ~0xFE000000;
   
   /* Clear all the setup token semaphores */
   temp = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT;
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT = temp;

   /* Clear all the endpoint complete status bits */   
   temp = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCOMPLETE;
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCOMPLETE = temp;

   timeout = 1000;
   while (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME & 0xFFFFFFFF) {
      /* Wait until all ENDPTPRIME bits cleared */
       if (timeout-- == 0)
       {
           USBERR("_usb_dci_vusb20_process_reset: ENDPTPRIME Timeout\n");
           return;
       }
   } /* Endif */
   
   /* Write 1s to the Flush register */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTFLUSH = 0xFFFFFFFF;

   /* Unstall all endpoints */
   for (i=0;i<usb_dev_ptr->MAX_ENDPOINTS;i++) {
      _usb_dci_vusb20_unstall_endpoint(handle, i, 0);
      _usb_dci_vusb20_unstall_endpoint(handle, i, 1);
   } /* Endfor */

   Usb2PortValue = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[0];

   if (Usb2PortValue & EHCI_PORTSCX_PORT_RESET) 
   {
      usb_dev_ptr->BUS_RESETTING = TRUE;
      usb_dev_ptr->USB_STATE = USB_STATE_POWERED;
   } else { 
      /* re-initialize */
      _usb_dci_vusb20_chip_initialize((pointer)usb_dev_ptr);
      #ifdef _DEVICE_DEBUG_
         DEBUG_LOG_TRACE("_usb_dci_vusb20_process_reset, SUCCESSFUL reinit hw");
      #endif
      return;
   } /* Endif */
   
   /* Inform the application so that it can cancel all previously queued transfers */
   _usb_device_call_service(usb_dev_ptr, USB_SERVICE_BUS_RESET, 0, 0, 0, 0, 0);

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE_D("_usb_dci_vusb20_process_reset, SUCCESSFUL");
   #endif
   
} /* EndBody */

#ifndef SUSPEND_INTERRUPT_DISABLE
/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_process_suspend
*  Returned Value : None
*  Comments       :
*        Services suspend interrupt
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_process_suspend
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_process_suspend");
   #endif
	
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   
   usb_dev_ptr->USB_DEV_STATE_B4_SUSPEND = usb_dev_ptr->USB_STATE;
   
   usb_dev_ptr->USB_STATE = USB_STATE_SUSPEND;

   /* Inform the upper layers */
   _usb_device_call_service(usb_dev_ptr, USB_SERVICE_SLEEP, 0, 0, 0, 0, 0);

   #ifdef __MISSING_RESUME_INTERRUPT_WORKAROUND__
   sof_after_suspend =0;
   #endif /* __MISSING_RESUME_INTERRUPT_WORKAROUND__ */

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_process_suspend, SUCCESSFUL");
   #endif
   
} /* EndBody */
#endif

#ifndef SOF_INTERRUPT_DISABLE
/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_process_SOF
*  Returned Value : None
*  Comments       :
*        Services SOF interrupt
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_process_SOF
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_process_SOF");
   #endif
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   /* Inform the upper layer */   
   _usb_device_call_service(usb_dev_ptr, USB_SERVICE_SOF, 0, 0, 0, 
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_FRINDEX, 0);

  #ifdef __MISSING_RESUME_INTERRUPT_WORKAROUND__
  #ifdef __USB_OTG__
   /* 10 SOFs after the suspend means that we should provide
   a resume callback */
   if ((usb_dev_ptr->USB_STATE == USB_STATE_SUSPEND) &&
      (sof_after_suspend > TB_A_SUSPEND))
   {
       usb_dev_ptr->USB_STATE = usb_dev_ptr->USB_DEV_STATE_B4_SUSPEND;
       /* Inform the upper layers */
       _usb_device_call_service(usb_dev_ptr, USB_SERVICE_RESUME, 0, 0, 0, 0, 0);
   }
   #endif
   #endif /* __MISSING_RESUME_INTERRUPT_WORKAROUND__ */
   

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_process_SOF, SUCCESSFUL");
   #endif

} /* EndBody */
#endif

#ifndef PORT_CHANGE_INTERRUPT_DISABLE
/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_process_port_change
*  Returned Value : None
*  Comments       :
*        Services port change detect interrupt
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_process_port_change
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_process_port_change");
   #endif
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   if (usb_dev_ptr->BUS_RESETTING) {
      /* Bus reset operation complete */
      usb_dev_ptr->BUS_RESETTING = FALSE;
   } /* Endif */

#ifdef __MISSING_RESUME_INTERRUPT_WORKAROUND__
   port_change_log[port_change_counter] = \
       dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[0];
   port_change_counter++;
#endif /* __MISSING_RESUME_INTERRUPT_WORKAROUND__ */

   if (!(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[0] & 
         EHCI_PORTSCX_PORT_RESET)) 
   {
      /* Get the speed */
      if (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[0] & 
         EHCI_PORTSCX_PORT_HIGH_SPEED) 
      {
         usb_dev_ptr->SPEED = USB_SPEED_HIGH;
      } else {
         usb_dev_ptr->SPEED = USB_SPEED_FULL;
      } /* Endif */

      /* Inform the upper layers of the speed of operation */      
      _usb_device_call_service(usb_dev_ptr, USB_SERVICE_SPEED_DETECTION, 0, 0, 
         0, usb_dev_ptr->SPEED, 0);
   
   } /* Endif */
      
#ifndef SMALL_CODE_SIZE      
   if (dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[0] & 
      EHCI_PORTSCX_PORT_SUSPEND) 
   {
   
      usb_dev_ptr->USB_DEV_STATE_B4_SUSPEND = usb_dev_ptr->USB_STATE;
      usb_dev_ptr->USB_STATE = USB_STATE_SUSPEND;

      /* Inform the upper layers */
      _usb_device_call_service(usb_dev_ptr, USB_SERVICE_SUSPEND, 0, 0, 0, 0, 0);
   } /* Endif */
#endif
   
   if (!(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[0] & 
      EHCI_PORTSCX_PORT_SUSPEND) && 
      (usb_dev_ptr->USB_STATE == USB_STATE_SUSPEND)) 
   {
      usb_dev_ptr->USB_STATE = usb_dev_ptr->USB_DEV_STATE_B4_SUSPEND;
      /* Inform the upper layers */
      _usb_device_call_service(usb_dev_ptr, USB_SERVICE_RESUME, 0, 0, 0, 0, 0);

      #ifdef _DEVICE_DEBUG_
         DEBUG_LOG_TRACE("_usb_dci_vusb20_process_port_change, SUCCESSFUL, resumed");
      #endif
      return;
   } /* Endif */

   usb_dev_ptr->USB_STATE = USB_STATE_DEFAULT;
   
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_process_port_change, SUCCESSFUL");
   #endif
   
} /* EndBody */
#endif

#ifndef ERROR_INTERRUPT_DISABLE
/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_process_error
*  Returned Value : None
*  Comments       :
*        Services error interrupt
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_process_error
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_process_error");
   #endif
      printk("_usb_dci_vusb20_process_error");
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   
   /* Increment the error count */
   usb_dev_ptr->ERRORS++;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_process_error, SUCCESSFUL");
   #endif
   
} /* EndBody */
#endif
/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_set_speed_full
*  Returned Value : None
*  Comments       :
*        Force the controller port in full speed mode.
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_set_speed_full
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
     
      /* The port number on the device */
      uint_8                     port_number
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   uint_32                                      port_control;
   
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_set_speed_full");
   #endif
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   port_control = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[port_number];   
   port_control |= EHCI_PORTSCX_FORCE_FULL_SPEED_CONNECT;
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[port_number] = port_control;

   
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_set_speed_full, SUCCESSFUL");
   #endif
   
} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_suspend_phy
*  Returned Value : None
*  Comments       :
*        Suspends the PHY in low power mode
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_suspend_phy
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
     
      /* The port number on the device */
      uint_8                     port_number
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   uint_32                                      port_control;
   
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_set_speed_full");
   #endif
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   port_control = dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[port_number];   
   port_control |= EHCI_PORTSCX_PHY_CLOCK_DISABLE;
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.PORTSCX[port_number] = port_control;

   
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_set_speed_full, SUCCESSFUL");
   #endif
   
} /* EndBody */



/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_set_address
*  Returned Value : None
*  Comments       :
*        Sets the newly assigned device address
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_set_address
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
     
      /* Address of the device assigned by the host */
      uint_8                     address
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR             dev_ptr;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE_SPL("_usb_dci_vusb20_set_address");
   #endif
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;
   
   #ifdef SET_ADDRESS_HARDWARE_ASSISTANCE   
   /***********************************************************
   Hardware Rev 4.0 onwards have special assistance built in
   for handling the set_address command. As per the USB specs
   a device should be able to receive the response on a new
   address, within 2 msecs after status phase of set_address is
   completed. Since 2 mili second may be a very small time window
   (on high interrupt latency systems) before software could 
   come to the code below and write the device register,
   this routine will be called in advance when status phase of
   set_address is still not finished. The following line in the
   code will set the bit 24 to '1' and hardware will take
   the address and queue it in an internal buffer. From which
   it will use it to decode the next USB token. Please look
   at hardware rev details for the implementation of this
   assistance.
   
   Also note that writing bit 24 to 0x01 will not break
   any old hardware revs because it was an unused bit.
   ***********************************************************/
   /* The address bits are past bit 25-31. Set the address 
   also set the bit 24 to 0x01 to start hardware assitance*/
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.DEVICE_ADDR = 
      ((uint_32)address << VUSBHS_ADDRESS_BIT_SHIFT) | 
      (0x01 << (VUSBHS_ADDRESS_BIT_SHIFT -1)); 
   #else
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.DEVICE_ADDR = 
      (uint_32)address << VUSBHS_ADDRESS_BIT_SHIFT; 
   #endif
   
   usb_dev_ptr->USB_STATE = USB_STATE_ADDRESS;
   
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE_SPL("_usb_dci_vusb20_set_address, SUCCESSFUL");
   #endif
   
} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_get_setup_data
*  Returned Value : None
*  Comments       :
*        Reads the Setup data from the 8-byte setup buffer
*
*END*-----------------------------------------------------------------*/
void _usb_dci_vusb20_get_setup_data
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
            
      /* [IN] the Endpoint number */
      uint_8                     ep_num,
            
      /* [OUT] address of the buffer to read the setup data into */
      uchar_ptr                  buffer_ptr
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   VUSB20_EP_QUEUE_HEAD_STRUCT_PTR              ep_queue_head_ptr;
   int i;	   
   #ifdef TRIP_WIRE
   boolean                                      read_safe;
   #endif

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_get_setup_data");
   #endif
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   /* Get the endpoint queue head */
#ifdef _VIRT_MEM
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      phys2virt(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR) +
	  2 * ep_num + USB_RECV;

#else
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR + 2*ep_num + 
      USB_RECV;
#endif
   /********************************************************************
   CR 1219. Hardware versions 2.3+ have a implementation of tripwire 
   semaphore mechanism that requires that we read the contents of 
   QH safely by using the semaphore. Read the USBHS document to under
   stand how the code uses the semaphore mechanism. The following are
   the steps in brief
   
   1. USBCMD Write 1 to Setup Tripwire in register.
   2. Duplicate contents of dQH.StatusBuffer into local software byte
      array.
   3  Read Setup TripWire in register. (if set - continue; if
      cleared goto 1.)
   4. Write '0' to clear Setup Tripwire in register.
   5. Process setup packet using local software byte array copy and
      execute status/handshake phases.
   
           
   ********************************************************************/
  #ifdef TRIP_WIRE  /*if semaphore mechanism is used the following code
                     is compiled in*/
  read_safe = FALSE;                      
  while(!read_safe)
  {
      /*********************************************************
      start with setting the semaphores
      *********************************************************/

      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD |= 
                                                EHCI_CMD_SETUP_TRIPWIRE_SET;

      /*********************************************************
      Duplicate the contents of SETUP buffer to our buffer
      *********************************************************/
        #ifdef _DATA_CACHE_
        for(i=0; i <8; i++)
        {
              *(buffer_ptr + i) = ep_queue_head_ptr->SETUP_BUFFER[i];
        }
        #else    
         /* Copy the setup packet to private buffer */
         USB_memcopy((uchar_ptr)ep_queue_head_ptr->SETUP_BUFFER, buffer_ptr, 8);
        #endif 
      /*********************************************************
      If setup tripwire semaphore is cleared by hardware it means
      that we have a danger and we need to restart.
      else we can exit out of loop safely.
      *********************************************************/
      if(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD &
                                                   EHCI_CMD_SETUP_TRIPWIRE_SET)
      {
          read_safe = TRUE; /* we can proceed exiting out of loop*/
      }
  }
  
   /*********************************************************
   Clear the semaphore bit now
   *********************************************************/
  dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.USB_CMD &=
    EHCI_CMD_SETUP_TRIPWIRE_CLEAR;                         
  
  
  #else   /*when semaphore is not used */    
     #ifdef _DATA_CACHE_
        for(i=0; i <8; i++)
        {
           *(buffer_ptr + i) = ep_queue_head_ptr->SETUP_BUFFER[i];
        }
     #else    
      /* Copy the setup packet to private buffer */
      USB_memcopy((uchar_ptr)ep_queue_head_ptr->SETUP_BUFFER, buffer_ptr, 8);
     #endif 
  #endif   
  
   /* Clear the bit in the ENDPTSETUPSTAT */
   dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPT_SETUP_STAT = 
      (1 << ep_num);

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_get_setup_data, SUCCESSFUL");
   #endif
   
} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_init_endpoint
*  Returned Value : None
*  Comments       :
*        Initializes the specified endpoint and the endpoint queue head
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_init_endpoint
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
            
      /* [IN] the transaction descriptor address */
      XD_STRUCT_PTR              xd_ptr
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   VUSB20_EP_QUEUE_HEAD_STRUCT _PTR_            ep_queue_head_ptr;
   uint_32                                      bit_pos;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_init_endpoint");
   #endif
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;

   /* Get the endpoint queue head address */
#ifdef _VIRT_MEM
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      phys2virt(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR) + 
      2 * xd_ptr->EP_NUM + xd_ptr->BDIRECTION ;
#else
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR + 
      2*xd_ptr->EP_NUM + xd_ptr->BDIRECTION;
#endif      

   bit_pos = (1 << (16 * xd_ptr->BDIRECTION + xd_ptr->EP_NUM));
   
   /* Check if the Endpoint is Primed */
   if ((!(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTPRIME & bit_pos)) && 
      (!(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTSTATUS & bit_pos))) 
   { 
      /* Set the max packet length, interrupt on Setup and Mult fields */
      if (xd_ptr->EP_TYPE == USB_ISOCHRONOUS_ENDPOINT) {
         /* Mult bit should be set for isochronous endpoints */
         ep_queue_head_ptr->MAX_PKT_LENGTH = ((xd_ptr->WMAXPACKETSIZE << 16) | 
            ((xd_ptr->MAX_PKTS_PER_UFRAME ?  xd_ptr->MAX_PKTS_PER_UFRAME : 1) << 
            VUSB_EP_QUEUE_HEAD_MULT_POS));
      } else {
         if (xd_ptr->EP_TYPE != USB_CONTROL_ENDPOINT) {
            ep_queue_head_ptr->MAX_PKT_LENGTH =
                ((xd_ptr->WMAXPACKETSIZE << 16) | 
               VUSB_EP_QUEUE_HEAD_IOS |
               (xd_ptr->DONT_ZERO_TERMINATE ? VUSB_EP_QUEUE_HEAD_ZERO_LEN_TER_SEL : 0));
         } else {
            ep_queue_head_ptr->MAX_PKT_LENGTH =
                ((xd_ptr->WMAXPACKETSIZE << 16) | 
                 VUSB_EP_QUEUE_HEAD_IOS |
                 (xd_ptr->DONT_ZERO_TERMINATE ? VUSB_EP_QUEUE_HEAD_ZERO_LEN_TER_SEL : 0));
         } /* Endif */
      } /* Endif */
      
      /* Enable the endpoint for Rx and Tx and set the endpoint type */
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.ENDPTCTRLX[xd_ptr->EP_NUM] |= 
         ((xd_ptr->BDIRECTION ? (EHCI_EPCTRL_TX_ENABLE | 
         EHCI_EPCTRL_TX_DATA_TOGGLE_RST) : 
         (EHCI_EPCTRL_RX_ENABLE | EHCI_EPCTRL_RX_DATA_TOGGLE_RST)) | 
         (xd_ptr->EP_TYPE << (xd_ptr->BDIRECTION ? 
         EHCI_EPCTRL_TX_EP_TYPE_SHIFT : EHCI_EPCTRL_RX_EP_TYPE_SHIFT)));
   } else 
   { 
      #ifdef _DEVICE_DEBUG_
         DEBUG_LOG_TRACE("_usb_dci_vusb20_init_endpoint, error ep init");
      #endif
      return USBERR_EP_INIT_FAILED;
   } /* Endif */
      
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_init_endpoint, SUCCESSFUL");
   #endif

   return USB_OK;
   
} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_get_transfer_status
*  Returned Value : USB_OK or error code
*  Comments       :
*        Gets the status of a transfer
*
*END*-----------------------------------------------------------------*/
uint_8 _usb_dci_vusb20_get_transfer_status
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
     
      /* [IN] the Endpoint number */
      uint_8                     ep_num,
            
      /* [IN] direction */
      uint_8                     direction
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_EP_TR_STRUCT_PTR                      dTD_ptr;
   XD_STRUCT_PTR                                xd_ptr;
   uint_8                                       status;
   
   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;

   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_get_transfer_status");
   #endif
   
   /* Unlink the dTD */
   dTD_ptr = usb_dev_ptr->EP_DTD_HEADS[2*ep_num + direction];

   if (dTD_ptr) {
      /* Get the transfer descriptor for the dTD */
      xd_ptr = (XD_STRUCT_PTR)dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD;
      status = xd_ptr->BSTATUS;
   } else {
      status = USB_STATUS_IDLE;
   } /* Endif */
   
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_get_transfer_status, SUCCESSFUL");
   #endif

   return (status);

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _usb_dci_vusb20_get_transfer_details
*  Returned Value : pointer to structure that has details for transfer
*        Gets the status of a transfer
*
*END*-----------------------------------------------------------------*/
XD_STRUCT_PTR  _usb_dci_vusb20_get_transfer_details
   (
      /* [IN] the USB_dev_initialize state structure */
      _usb_device_handle         handle,
     
      /* [IN] the Endpoint number */
      uint_8                     ep_num,
            
      /* [IN] direction */
      uint_8                     direction
   )
{ /* Body */
   USB_DEV_STATE_STRUCT_PTR                     usb_dev_ptr;
   VUSB20_REG_STRUCT_PTR                        dev_ptr;
   VUSB20_EP_TR_STRUCT_PTR                      dTD_ptr, temp_dTD_ptr;
   XD_STRUCT_PTR                                xd_ptr;
   uint_32                                      temp, remaining_bytes;
   VUSB20_EP_QUEUE_HEAD_STRUCT_PTR              ep_queue_head_ptr;

   usb_dev_ptr = (USB_DEV_STATE_STRUCT_PTR)handle;
   dev_ptr = (VUSB20_REG_STRUCT_PTR)usb_dev_ptr->DEV_PTR;
   temp = (2*ep_num + direction);

   /* get a pointer to QH for this endpoint */
#ifdef _VIRT_MEM
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      phys2virt(dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR) + 
      temp;
#else
   ep_queue_head_ptr = (VUSB20_EP_QUEUE_HEAD_STRUCT_PTR)
      dev_ptr->REGISTERS.OPERATIONAL_DEVICE_REGISTERS.EP_LIST_ADDR + 
      temp;
#endif
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_get_transfer_status");
   #endif
   
   /* Unlink the dTD */
   dTD_ptr = usb_dev_ptr->EP_DTD_HEADS[2*ep_num + direction];

   if (dTD_ptr) 
   {
      
      /* Get the transfer descriptor for the dTD */
      xd_ptr = (XD_STRUCT_PTR)dTD_ptr->SCRATCH_PTR->XD_FOR_THIS_DTD;
      if(!xd_ptr) return NULL;
      
      /* Initialize the transfer length field */
      xd_ptr->WSOFAR =0;
      remaining_bytes =0;
      
      /*if length of this transfer is greater than 20K
      we have multiple DTDs to count */
      if(xd_ptr->WTOTALLENGTH > VUSB_EP_MAX_LENGTH_TRANSFER)
      {
         /* it is a valid DTD. We should parse all DTDs for this XD
         and find the total bytes used so far */
         temp_dTD_ptr = dTD_ptr;
      
         /*loop through the list of DTDS until an active DTD is found
         or list has finished */
         while(!(dTD_ptr->NEXT_TR_ELEM_PTR & VUSBHS_TD_NEXT_TERMINATE))         
         {
            
            /**********************************************************
            If this DTD has been overlayed, we take the actual length 
            from QH.
            **********************************************************/
#ifdef _VIRT_MEM
            if ((uint_32)(phys2virt(ep_queue_head_ptr->CURR_DTD_PTR & VUSBHS_TD_ADDR_MASK)) ==
                 (uint_32)temp_dTD_ptr)
            {
                remaining_bytes += 
                  ((ep_queue_head_ptr->SIZE_IOC_INT_STS & VUSB_EP_TR_PACKET_SIZE) >> 16);
            }
#else
            if ((uint_32)(ep_queue_head_ptr->CURR_DTD_PTR & VUSBHS_TD_ADDR_MASK) ==
                  (uint_32)temp_dTD_ptr)
            {
                remaining_bytes += 
                  ((ep_queue_head_ptr->SIZE_IOC_INT_STS & VUSB_EP_TR_PACKET_SIZE) >> 16);
            }
#endif
            else
            {
               /* take the length from DTD itself */
                remaining_bytes += 
                  ((temp_dTD_ptr->SIZE_IOC_STS & VUSB_EP_TR_PACKET_SIZE) >> 16);
            }
   
            dTD_ptr = temp_dTD_ptr;
             
            /* Get the address of the next dTD */
#ifdef _VIRT_MEM
            temp_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)
                  (phys2virt(temp_dTD_ptr->NEXT_TR_ELEM_PTR & VUSBHS_TD_ADDR_MASK));
#else
            temp_dTD_ptr = (VUSB20_EP_TR_STRUCT_PTR)
                  (temp_dTD_ptr->NEXT_TR_ELEM_PTR & VUSBHS_TD_ADDR_MASK);
#endif
         }
         xd_ptr->WSOFAR = xd_ptr->WTOTALLENGTH - remaining_bytes;

      }
      else
      {
         /*look at actual length from QH*/
         xd_ptr->WSOFAR = xd_ptr->WTOTALLENGTH - 
            ((ep_queue_head_ptr->SIZE_IOC_INT_STS & VUSB_EP_TR_PACKET_SIZE) >> 16);
         
      }
      
   } else {
      xd_ptr = NULL;
   } /* Endif */
   
   #ifdef _DEVICE_DEBUG_
      DEBUG_LOG_TRACE("_usb_dci_vusb20_get_transfer_status, SUCCESSFUL");
   #endif

   return (xd_ptr);

} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _bsp_get_usb_base
*  Returned Value : Address of the VUSB register base
*  Comments       :
*        Get the USB register base address
*END*-----------------------------------------------------------------*/

pointer _bsp_get_usb_base
   (
      uint_8 device_number
   )
{ /* Body */

    UNUSED(device_number);
#ifdef _VIRT_MEM
	return (pointer)ARC_USB_DEVICE_AMBA ;
#else
    return (pointer) 0x30000000;
#endif
} /* EndBody */

/*FUNCTION*-------------------------------------------------------------
*
*  Function Name  : _bsp_get_usb_capability_register_base
*  Returned Value : Address of the VUSB1.1 capability register base
*  Comments       :
*        Get the USB capability register base address
*END*-----------------------------------------------------------------*/

pointer _bsp_get_usb_capability_register_base
   (
      /* [IN] the device number */
      uint_8 device_number
   )
{ /* Body */

    UNUSED(device_number);
#ifdef _VIRT_MEM
	return (pointer) ARC_USB_DEVICE_AMBA ;
#else
    return (pointer) 0x30000000;
#endif
} /* EndBody */

void _DCACHE_INVALIDATE_MLINES(void * p, int n)
{
    UNUSED(p);
    UNUSED(n);
}
    
void _DCACHE_FLUSH_MLINES(void * p, int n)
{
    UNUSED(p);
    UNUSED(n);
}

/*void* _uncached(void *ptr)
{
    return (void *)((U8 *)ptr + 0x20000000);
}
*/
/* EOF */

