/*
 *	Generic parts
 *	Linux ethernet bridge
 *
 *	Authors:
 *	Lennert Buytenhek		<buytenh@gnu.org>
 *
 *	$Id: br.c,v 1.3 2008-04-28 09:54:18 red_liu Exp $
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 */

#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/miscdevice.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/init.h>

#include "br_private.h"

#if defined(CONFIG_ATM_LANE) || defined(CONFIG_ATM_LANE_MODULE)
#include "../atm/lec.h"
#endif

#include <net/ip.h>
#include <net/tcp.h>
#include <linux/fs.h>
#include <linux/proc_fs.h>
#include <linux/ip.h>
#include <linux/in.h>

#define DEV_NAME_LENGTH 10
#define IPADDR_LENGTH 15
#define MAX_VPN_POLICY	8
#define BRTRIGGER_WRITE_LENGTH	120

#if defined(CONFIG_ARCH_SOLOS) && defined(CONFIG_ATM_BR2684)
extern struct net_bridge_fdb_entry *(*br_fdb_get_hook)(struct net_bridge *br,
                                                unsigned char *addr);
extern void (*br_fdb_put_hook)(struct net_bridge_fdb_entry *ent);
#endif

int (*br_should_route_hook) (struct sk_buff **pskb) = NULL;

#ifdef raul_proc_bridge


typedef struct vpn_trigger_s {
	unsigned char on;
	__u32   ss;
	__u32   se;
	__u32   ds;
	__u32   de;
	char name[13];
} vpn_trigger_t;

static vpn_trigger_t  vpn_trigger[MAX_VPN_POLICY];
static int vpn_total_triggers = 0;
//static unsigned long vpn_server_itself = 0;
static struct proc_dir_entry *br_trigger = NULL;
//DECLARE_MUTEX(vpn_write_sema);
DECLARE_MUTEX_LOCKED(vpn_read_sema);

static int br_trigger_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data){
	char *p = page;
	int len;
	int i;

	down_interruptible(&vpn_read_sema);
	for(i=0;i<vpn_total_triggers;i++)
		if(vpn_trigger[i].on==2)
			break;
	p += sprintf(p, "%s", vpn_trigger[i].name);
	//vpn_trigger[i].on=0;
	//up(&vpn_write_sema);

	len = (p - page) - off; 
	if (len < 0) 
		len = 0; 
	*eof = (len <= count) ? 1 : 0; 
	*start = page + off; 
	return len;
}

static int br_trigger_write_proc(struct file *file, const char *buffer, unsigned long count, void *data){
	int len;
	char value[BRTRIGGER_WRITE_LENGTH+1];
	struct br_filter *new_filter, *last_filter;
	vpn_trigger_t new;
	int i;
	int offset=0;
	
	len = count > BRTRIGGER_WRITE_LENGTH ? BRTRIGGER_WRITE_LENGTH : count;
	if ( copy_from_user(value, buffer, len) ) 
		return -EFAULT;
	value[len] = '\0';

	if(len<2) {
		vpn_total_triggers=0;
		memset(vpn_trigger, 0, sizeof(vpn_trigger_t)*MAX_VPN_POLICY);
	} else if(len<16){

		if(value[0]=='\1')  /* IPsec SA establish */
			offset=1;
#if 0
		else if(value[0]=='\2') { /* setup vpn_server_itself */
			sscanf(value+1,"%u", &vpn_server_itself);
			return len;
		} else
#endif
		else
			offset=0;

		for(i=0;i<vpn_total_triggers;i++) {
			if(memcmp(vpn_trigger[i].name, value+offset, len-1-offset)==0){
				if(offset) /* IPsec SA establish */
					vpn_trigger[i].on=0;
				else
					vpn_trigger[i].on=1;
			}
		}
				
	} else {
		sscanf(value,"%u %u %u %u %s", &new.ss, &new.se, &new.ds, &new.de, new.name);
		new.on=1;
		memcpy(&vpn_trigger[vpn_total_triggers++], &new, sizeof(vpn_trigger_t));
	}
	return len;
}

/* 2005.03.31 add for Any subnet */
static unsigned long my_br_ifa_address=0;

void set_br_ifa_address(unsigned long addr) {
	my_br_ifa_address=addr;
}

int br_trigger_hook(struct sk_buff *skb) {
	int i;
	unsigned long saddr, daddr;
	struct in_device *in_dev;
#if 1   //defined (CONFIG_MIPS_AVALANCHE_FAST_BRIDGE)
	/* todo: Endien Issue ? */
        daddr = (skb->data[16]<<24) | (skb->data[17]<<16) | (skb->data[18]<<8) | (skb->data[19]);
	if(daddr==my_br_ifa_address)
		return 0;
        saddr = (skb->data[12]<<24) | (skb->data[13]<<16) | (skb->data[14]<<8) | (skb->data[15]);
#else 	/* CONFIG_MIPS_AVALANCHE_FAST_BRIDGE */
        daddr=ntohl(skb->nh.iph->daddr);
	if(daddr==my_br_ifa_address)
		return 0;
        saddr=ntohl(skb->nh.iph->saddr);
#endif 	/* CONFIG_MIPS_AVALANCHE_FAST_BRIDGE */

	for(i=0;i<vpn_total_triggers;i++){
		if( (vpn_trigger[i].on!=0) &&
		    (saddr <= vpn_trigger[i].se) &&
		    (saddr >= vpn_trigger[i].ss) &&
		    (daddr <= vpn_trigger[i].de) &&
		    (daddr >= vpn_trigger[i].ds)){
			if(vpn_trigger[i].on==1) {
				vpn_trigger[i].on=2;
				up(&vpn_read_sema);
				return 1;
			} else if(vpn_trigger[i].on==2){
				return 1;
			} 
		}
	}
	return 0;
}

/********************************************************/

/*
*  added qos_mark under /proc/ 4.xx.2008 by red_liu
*  command(add/flush):type(1/2):up/down(0/1):rules
*  add;1;0;proto:0/1/2(tcp/udp/all);port:10-20;ip:192.168.0.2-192.168.0.4;mark:10
*	 add;2;1;port:0/1/2/3/4(eth0-3/wlan0);mark:20	 
*	 flush
*/
struct qos_mark_entry{
	struct qos_mark_entry *next;

	__u8 type;		// 1/2
	__u8 dir;
	__u8 proto; 	// 0/1/2 tcp/udp/all
	
	/* port range */
	__u16 sport;	// start port
	__u16 eport;	// end port
	
	/* ip range same as port*/
	unsigned saddr;
	unsigned eaddr;
	
	char port[10];
	/* mark */
	unsigned mark;
};
static struct qos_mark_entry *qos_mark_root = NULL;
static struct proc_dir_entry *qos_mark = NULL;

#define QOS_MARK_LENGTH	120
#define QOS_MARK_CMD_ADD	"add"
#define QOS_MARK_CMD_FLUSH "flush"

#if 0
#define duprintf(format, args...) printk(format , ## args)
#else
#define duprintf(format, args...)
#endif

int br_qos_mark(struct net_bridge_port *dst, struct sk_buff *skb, int layer3)
{
	struct iphdr *iph;
	struct tcphdr *th;
	struct udphdr *uh;
	
	struct qos_mark_entry *entry = qos_mark_root;	
	iph = skb->nh.iph;
	
	while(entry)
	{
		/* check the stream rely on src dev and dst dev */
		if(entry->dir)
		{
			if(memcmp(skb->dev->name, "nas", 3) != 0
				&& memcmp(skb->dev->name, "br", 2) != 0)break;
		}
		else if(memcmp(dst->dev->name, "nas", 3) != 0)break;				
		
		duprintf("entry->dir %d net dev %s, skb dev %s\n",entry->dir, dst->dev->name, skb->dev->name);
		
		switch(entry->type)
		{
			case 2:
				if(iph->protocol == IPPROTO_TCP 
						|| iph->protocol == IPPROTO_UDP
						|| iph->protocol == IPPROTO_ICMP)
				{
					if(entry->dir)
					{
						if(strcmp(dst->dev->name, entry->port) == 0)
						{
							duprintf("1set mark as %u\n", entry->mark);
							skb->nfmark = entry->mark;							
						}
					}
					else if(strcmp(skb->dev->name, entry->port) == 0)
					{
						duprintf("2set mark as %u\n", entry->mark);
						skb->nfmark = entry->mark;
					}
				}
				break;			
			case 1:	
				
				// for type 1 only check packets only from ip layer  
				if(layer3 == 0)break;

				duprintf("br_qos_mark: iph->daddr:%x, entry->saddr:%x, entry->eaddr:%x\n", iph->daddr,entry->saddr, entry->eaddr);
				if(iph->daddr >= entry->saddr && iph->daddr <= entry->eaddr)// dst ip range
				{

					duprintf("br_qos_mark: entry->proto:%u, iph->protocol:%u\n", entry->proto, iph->protocol);
					// proto match
					if(entry->proto == 2 
						|| (entry->proto == 0 && iph->protocol == IPPROTO_TCP))
					{
						th = (struct tcphdr *)(skb->data + (iph->ihl * 4));

						//th = skb->h.th;
						duprintf("th->dest,0x%x, 0x%x\n", th->dest, ntohs(th->dest));
						if(th->dest >= entry->sport && th->dest <= entry->eport)
						{
							duprintf("set mark as %u\n", entry->mark);
							skb->nfmark = entry->mark;
						}
					}
					
					if(entry->proto == 2 
						|| (entry->proto == 1 && iph->protocol == IPPROTO_UDP))
					{
						uh = (struct udphdr *)(skb->data + (iph->ihl * 4));

						//uh = skb->h.uh;
						if(uh->dest >= entry->sport && uh->dest <= entry->eport)
						{
							duprintf("set mark as %u\n", entry->mark);
							skb->nfmark = entry->mark;
						}
					}
				}
				break;
			default:
				duprintf("br_qos_mark: error rule\n");
				break;
		}
		
		entry = entry->next;
	}
	
	return 0;
}

static int qos_mark_read_proc(char *page, 
															char **start, 
															off_t off, 
															int count, 
															int *eof, 
															void *data)
{
	char *p = page;
	int len, i = 0;
	struct qos_mark_entry *entry;
	
	entry = qos_mark_root;

	p += sprintf(p, "QosMarkEntrys:\n");
	while(entry)
	{
		if(entry->type == 1)
		{
			p += sprintf(p, "%u:%s proto:%u;port:%u-%u;ip:%x-%x;mark:%u\n",
						++i,
						entry->dir ? "down" : "up",
						entry->proto,
						ntohs(entry->sport),
						ntohs(entry->eport),
						entry->saddr,
						entry->eaddr,
						entry->mark);
		}
		else
			p += sprintf(p, "%u:%s lanport:%s;mark:%u\n", 
											++i, entry->dir ? "down" : "up", entry->port, entry->mark);
			
		entry = entry->next;
	}

	len = (p - page) - off; 
	if (len < 0)len = 0; 
	*eof = (len <= count) ? 1 : 0; 
	*start = page + off;
	
	return len;
}

static int qos_mark_write_proc(struct file *file,
																const char *buffer, 
																unsigned long count, 
																void *data)
{
	int len, port = 0;
	unsigned int tm;
	char cmd[QOS_MARK_LENGTH + 1], rule[QOS_MARK_LENGTH + 1];
	struct qos_mark_entry *tmp, *last;	
	
	len = count > QOS_MARK_LENGTH ? QOS_MARK_LENGTH : count;
	if(copy_from_user(cmd, buffer, len)) 
		return -EFAULT;		
	
	/* find last entry */
	last = qos_mark_root;
	cmd[len] = '\0';
	
	if(strncmp(cmd, QOS_MARK_CMD_ADD, strlen(QOS_MARK_CMD_ADD)) == 0)
	{
		tmp = kmalloc(sizeof(struct qos_mark_entry), GFP_KERNEL);
		if(!tmp)
		{
			printk("qos_mark_write_proc: kmalloc error\n");
			return -1;
		}
		
		memset(tmp, 0, sizeof(struct qos_mark_entry));
		
		sscanf(cmd + strlen(QOS_MARK_CMD_ADD), ";%s", rule);
		if(*rule == '1') // type 1
		{
			sscanf(rule, "%u;%u;proto:%u;port:%u-%u;ip:%u-%u;mark:%u",
							&tmp->type,
							&tmp->dir,
							&tmp->proto,
							&tmp->sport,
							&tmp->eport,
							&tmp->saddr,
							&tmp->eaddr,
							&tmp->mark
						);
			printk("%u;%s;proto:%u;port:%u-%u;ip:%u-%u;mark:%u\n",
							tmp->type,
							tmp->dir ? "down" : "up",
							tmp->proto,
							tmp->sport,
							tmp->eport,
							tmp->saddr,
							tmp->eaddr,
							tmp->mark
						);
			tmp->sport = htons(tmp->sport);
			tmp->eport = htons(tmp->eport);
			
			if(tmp->saddr > tmp->eaddr)
			{
				tm = tmp->saddr;
				tmp->saddr = tmp->eaddr;
				tmp->eaddr = tm;
			}
			
			if(tmp->sport > tmp->eport)
			{
				tm = tmp->sport;
				tmp->sport = tmp->eport;
				tmp->eport = tm;
			}
		}
		else if(*rule == '2')
		{
			sscanf(rule, "%u;%u;port:%d;mark:%u", &tmp->type, &tmp->dir, &port, &tmp->mark);
			if(port == 4)sprintf(tmp->port, "wlan0");
			else sprintf(tmp->port, "eth%d", port);
			
			printk("%u;%s,lanport:%s;mark:%u\n",tmp->type, tmp->dir ? "down" : "up", tmp->port, tmp->mark);
		}
		else
			printk("error type just can be 1 or 2!\n");
		
		if(last == NULL)qos_mark_root = tmp;
		else
		{
			while(last->next)last = last->next;
			last->next = tmp;	
		}	 
	}
	else if(strncmp(cmd, QOS_MARK_CMD_FLUSH, strlen(QOS_MARK_CMD_FLUSH)) == 0)
	{
		qos_mark_root = NULL;
		
		tmp = last;
		while(last)
		{
			last = last->next;
			kfree(tmp);
			tmp = last;	
		}
		
		printk("flush ok!\n");
	}
	else printk("Bad command\n");
	
	return len;		
}


/********************************************************/
struct br_filter {
	char dev_name[DEV_NAME_LENGTH+1];
	//char valid_ip[IPADDR_LENGTH+1];
	__u32   saddr;
	struct net_device  *dev; 
	struct br_filter *next;
	struct br_filter *prev;
};

static struct br_filter *br_filter_root=NULL;
static struct proc_dir_entry *br_filter = NULL;

int br_filter_pass(struct net_bridge_port *p, struct sk_buff *skb){
	struct br_filter *fp;
	unsigned char ip[4];
	struct iphdr *iph;

	if(p==NULL || skb==NULL || br_filter_root==NULL || (skb->protocol!=htons(ETH_P_IP)))
		return 1;
	fp=br_filter_root;
	do {
		if((fp->dev==p->dev)||(strcmp(p->dev->name, fp->dev_name)==0)){
			fp->dev=p->dev;
#ifdef BRFILTER_DEBUG
		        ip[0]=fp->saddr>>24;
		        ip[1]=(fp->saddr&0x00ffffff)>>16; 
			ip[2]=(fp->saddr&0x0000ffff)>>8; 
			ip[3]=(fp->saddr&0x000000ff); 
			printk("fp->saddr=%hd.%hd.%hd.%hd ", ip[0],ip[1],ip[2],ip[3]);
		        ip[0]=skb->nh.iph->saddr>>24;
		        ip[1]=(skb->nh.iph->saddr&0x00ffffff)>>16; 
			ip[2]=(skb->nh.iph->saddr&0x0000ffff)>>8; 
			ip[3]=(skb->nh.iph->saddr&0x000000ff); 
			printk("skb->nh.iph->saddr=%hd.%hd.%hd.%hd\n", ip[0],ip[1],ip[2],ip[3]);
#endif
			iph = &skb->data[ETH_HLEN];
//			if(fp->saddr == iph->saddr)
			if(skb->nh.iph->saddr==fp->saddr)	
				{
				printk("br.c debug pass ok\n");
				return 1;
			}
			else
			{
				//printk("br.c debug can not pass \n");				
				return 0;
			}
		}
		fp=fp->next;
	} while(fp!=NULL);
	return 1;
}

static int br_filter_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data){
	char *p = page;
	int len;
	struct br_filter *filter;
	int i;
	
	filter=br_filter_root;
	if(filter!=NULL) {
		do {
			p += sprintf(p, "br_filter:%s %x\n", filter->dev_name, filter->saddr); 
			filter=filter->next;
		} while(filter!=NULL);
	}

	p += sprintf(p, "\n-- BR_TRIGGER_ENTRIES --\n");
	for(i=0;i<MAX_VPN_POLICY;i++){
		p += sprintf(p, "vpn_trigger[%d]: %d %x %x %x %x %s\n", \
			     i, vpn_trigger[i].on, vpn_trigger[i].ss, \
			     vpn_trigger[i].se, vpn_trigger[i].ds, \
			     vpn_trigger[i].de, vpn_trigger[i].name);
	}

	len = (p - page) - off; 
	if (len < 0) 
		len = 0; 
	*eof = (len <= count) ? 1 : 0; 
	*start = page + off; 
	return len;
}

#define BRFILTER_WRITE_LENGTH	120
#define BRFILTER_CMD_ADD	'1'
#define BRFILTER_CMD_RESET	'2'
static int br_filter_write_proc(struct file *file,
																const char *buffer, 
																unsigned long count, 
																void *data)
{
	int len;
	char value[BRFILTER_WRITE_LENGTH+1];
	int cmd;
	struct br_filter *new_filter, *last_filter;
	
	len = count > BRFILTER_WRITE_LENGTH ? BRFILTER_WRITE_LENGTH : count;
	if ( copy_from_user(value, buffer, len) ) 
		return -EFAULT;
	value[len] = '\0';
	/* find last_filter */
	last_filter=br_filter_root;
	if(last_filter!=NULL)
		while(last_filter->next!=NULL)
			last_filter=last_filter->next;
	switch (value[0]) {
		case BRFILTER_CMD_ADD:
			new_filter=kmalloc(sizeof(struct br_filter), GFP_KERNEL);
			memset(new_filter, 0, sizeof(struct br_filter));
			sscanf(value, "1:%u:%s", &new_filter->saddr, new_filter->dev_name);
#ifdef BRFILTER_DEBUG
			printk("[%s][%x]\n", new_filter->dev_name, new_filter->saddr);
#endif
			if(last_filter==NULL)
				br_filter_root=new_filter;
			else {
				new_filter->prev=last_filter;
				last_filter->next=new_filter;
			}
			break;
		case BRFILTER_CMD_RESET:
			if(last_filter!=NULL)
				do{
					new_filter=last_filter->prev;
					kfree(last_filter);
					last_filter=new_filter;
				} while(last_filter!=NULL);
				br_filter_root=NULL;
			break;
		default:
			printk("Unknown Command!\n");
	}
#ifdef BRFILTER_DEBUG
	printk("br_filter_write_proc:%s\n", value);
#endif
	return len;
}
/*
void br_dec_use_count()
{
	MOD_DEC_USE_COUNT;
}

void br_inc_use_count()
{
	MOD_INC_USE_COUNT;
}
*/
#endif
static int __init br_init(void)
{
	br_fdb_init();

#ifdef CONFIG_BRIDGE_NETFILTER
	if (br_netfilter_init())
		return 1;
#endif
	brioctl_set(br_ioctl_deviceless_stub);
	br_handle_frame_hook = br_handle_frame;

#if defined(CONFIG_ATM_LANE) || defined(CONFIG_ATM_LANE_MODULE) \
	|| (defined(CONFIG_ARCH_SOLOS) && defined(CONFIG_ATM_BR2684))
	br_fdb_get_hook = br_fdb_get;
	br_fdb_put_hook = br_fdb_put;
#endif
	register_netdevice_notifier(&br_device_notifier);

#ifdef raul_proc_bridge

	br_filter = create_proc_entry("br_filter", 0600, &proc_root);
	br_filter->read_proc = br_filter_read_proc;
	br_filter->write_proc = br_filter_write_proc;

	qos_mark = create_proc_entry("qos_mark", 0600, &proc_root);
	qos_mark->read_proc = qos_mark_read_proc;
	qos_mark->write_proc = qos_mark_write_proc;
#endif

	return 0;
}

static void __exit br_deinit(void)
{
#ifdef CONFIG_BRIDGE_NETFILTER
	br_netfilter_fini();
#endif
	unregister_netdevice_notifier(&br_device_notifier);
	brioctl_set(NULL);

	br_cleanup_bridges();

	synchronize_net();

#if defined(CONFIG_ATM_LANE) || defined(CONFIG_ATM_LANE_MODULE)
	br_fdb_get_hook = NULL;
	br_fdb_put_hook = NULL;
#endif

#ifdef raul_proc_bridge

	remove_proc_entry("br_filter", &proc_root);
	remove_proc_entry("qos_mark", &proc_root);
#endif
	br_handle_frame_hook = NULL;
	br_fdb_fini();
}

EXPORT_SYMBOL(br_should_route_hook);

module_init(br_init)
module_exit(br_deinit)
MODULE_LICENSE("GPL");
