#include <linux/module.h>
#include <linux/types.h>
#include <linux/udp.h>
#include <linux/skbuff.h>

#include <linux/netfilter_ipv4/ipt_inputdev.h>
#include <linux/netfilter_ipv4/ip_tables.h>

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Secret");
MODULE_DESCRIPTION("iptables inputdev match module");

#if 1
#define duprintf(format, args...) printk(format , ## args)
#else
#define duprintf(format, args...)
#endif

/* Called when user tries to insert an entry of this type. */
static int
checkentry(const char *tablename,
	   const struct ipt_ip *ip,
	   void *matchinfo,
	   unsigned int matchsize,
	   unsigned int hook_mask)
{
	return (matchsize == IPT_ALIGN(sizeof(struct ipt_inputdev)));
}

static int
match(const struct sk_buff *skb,
      const struct net_device *in,
      const struct net_device *out,
      const void *matchinfo,
      int offset,
      int *hotdrop)
{
	const struct ipt_inputdev *multiinfo = matchinfo;
	u_int8_t devnum = multiinfo->count;
	u_int8_t i;
	int len, len1;
	
	if(!skb->input_dev)
	{	
	//	duprintf("input_dev is null, dev: name is %s\n", skb->dev->name);
		return 0; 
	}

	if(skb->input_dev->name == NULL)
	{
                duprintf("input_dev is null, dev: name is %s\n", skb->dev->name);
		return 0; 
	}
//duprintf("input_dev is null, dev: name is %s, inputdev name is: %s\n", skb->dev->name, skb->input_dev->name);		
	for(i = 0; i < devnum; i++)
	{	
		len = strlen(multiinfo->devs[i]);
		len1 = strlen(skb->input_dev->name);
		if(len == 0 || len1 == 0)
		{
			duprintf("inputdev: rules len %d, skbuf len %d", len, len1);
			duprintf("input_dev is null, dev: name is %s, inputdev name is: %s\n", skb->dev->name, skb->input_dev->name);
		}
		len = len > len1 ? len : len1;		
		if(strncmp(multiinfo->devs[i], skb->input_dev->name, len) == 0)
		{
	//		duprintf("ipt_inputdev %s matchs dev name: %s\n",multiinfo->devs[i], skb->input_dev->name);
			if(multiinfo->invert)return 0;
			else return 1;			
		}
	}
  
	return multiinfo->invert;
}

static struct ipt_match multidevs_match = {
	.name		= "inputdev",
	.revision	= 0,
	.match		= &match,
	.checkentry	= &checkentry,
	.me		= THIS_MODULE,
};

static int __init init(void)
{
	return ipt_register_match(&multidevs_match);
}

static void __exit fini(void)
{
	ipt_unregister_match(&multidevs_match);
}

module_init(init);
module_exit(fini);
