

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <time.h>

#include "deltdiag.h"

static float f_print(unsigned int value)
{
	int v = (short) value, sign = 1 ;

	if(v < 0) {
		v = -v ;
		sign = -1 ;
	}

	return ((v * 10000) >> 15) / 10000. * sign ;  /* divided by 2**15 */
}

static float e_print(unsigned int value)
{
	float temp = value ;       /* take the argument as unsigned */

	temp = (value * 10000) >> 15;  /* divided by 2**15 */

	while(temp && (temp / 10000. == 0)) {
		temp *= 10;
	}
	return temp / 10000. ;
}


static float signed_print(unsigned int value)
{
    int temp;

    temp = value << 6;
    if (temp < 0) {
        temp = -temp;
    }
    value = ((unsigned int) temp) >> 6;
	return ((float) value) / 10 ;
}


static float actpsd_print(int value)
{
	return (float)(value / 10. +
					.1 * ((value % 10) / 2) +
					.01 * (((value % 10) % 2) *5)) ;
}


static float gain_print(int value)
{
	return (float) value / 100. ;
}


int main(int argc, char** argv)
{
	int fd, len, cmd ;
	char* delt_data ;
	common_val *val ;

	if(argc == 1) {
		printf("  usage: deltshow COMMAND\n"
				"  where COMMAND is one of\n"
				"\tdeltsnrdn\n"
				"\tdeltqlndn\n"
				"\tdelthlindni\n"
				"\tdelthlindnr\n"
				"\tdeltactatp\n"
				"\tdeltsnrm\n"
				"\tdeltattndr\n") ;
		exit(1) ;
	}

	if((fd = open("/dev/deltdiag", O_RDONLY, 0)) < 0) {
		printf("failed open\n") ;
		exit(1) ;
	}

	if(((cmd = strcmp(argv[1], "deltsnrdn")) == 0) ||
		(strcmp(argv[1], "deltqlndn") == 0)) {

		deltsnrdn stat ;

		if(!(stat.val = malloc(DELTSNRDN_SZ))) {
			printf("No mem\n") ;
			exit(1) ;
		}
		stat.stat_size = DELTSNRDN_SZ ;
		
		cmd = cmd ? DELTQLNDN : DELTSNRDN ;
		
		if(ioctl(fd, cmd, &stat) < 0) {
			printf("ioctl failed\n") ;
			exit(1) ;
		}
		val = stat.val ;
		printf("\tTone Number\t%s\n", argv[1]) ;
		for(len = 0 ; len < 512 ; len++, val++) {
			printf("\t%6d     \t%3d\n", val->tone_num, val->value) ;
		}
		printf("\n") ;
		free(stat.val) ;
	} else if(strcmp(argv[1], "deltbitsdn") == 0) {

		deltbitsdn stat ;

		if(!(stat.val = malloc(DELTBITSDN_SZ))) {
			printf("No mem\n") ;
			exit(1) ;
		}
		stat.stat_size = DELTBITSDN_SZ ;
		if(ioctl(fd, DELTBITSDN, &stat) < 0) {
			printf("ioctl failed\n") ;
			exit(1) ;
		}
		val = stat.val ;
		printf("\tTone Number\t%s\n", argv[1]) ;
		for(len = 0 ; len < 1024 ; len++, val++) {
			printf("\t%6d     \t%3d\n", val->tone_num, val->value) ;
		}
		printf("\n") ;
		free(stat.val) ;
	} else if(strcmp(argv[1], "deltactatp") == 0) {
		
		deltactatp stat ;

		if(!(stat.val = malloc(DELTACTATP_SZ))) {
			printf("No mem\n") ;
			exit(1) ;
		}
		stat.stat_size = DELTACTATP_SZ ;
		if(ioctl(fd, DELTACTATP, &stat) < 0) {
			printf("ioctl failed\n") ;
			exit(1) ;
		}
		val = stat.val ;
		for(len = 0 ; len < 2 ; len++, val++) {
			printf("%6.1f dB\n", signed_print(val->value)) ;
		}
		printf("\n") ;
		free(stat.val) ;
	} else if(((cmd = strcmp(argv[1], "delthlindni")) == 0) ||
				(strcmp(argv[1], "delthlindnr") == 0)) {
		
		delthlindnr stat ;

		if(!(stat.val = malloc(DELTHLINDNI_SZ))) {
			printf("No mem\n") ;
			exit(1) ;
		}
		stat.stat_size = DELTHLINDNI_SZ ;

		cmd = cmd ? DELTHLINDNR : DELTHLINDNI ;
		
		if(ioctl(fd, cmd, &stat) < 0) {
			printf("ioctl failed\n") ;
			exit(1) ;
		}
		val = stat.val ;
		printf("scale: %e\n", val->value) ;
		val++ ;
		printf("\tTone Number\t%s\n", argv[1]) ;
		for(len = 0 ; len < 512 ; len++, val++) {
			printf("\t%6d     \t%d\n", val->tone_num, val->value) ;
		}
		printf("\n") ;
		free(stat.val) ;
	} else if(strcmp(argv[1], "deltactpsd") == 0) {
		
		deltactpsd stat ;

		if(!(stat.val = malloc(DELTACTPSD_SZ))) {
			printf("No mem\n") ;
			exit(1) ;
		}
		stat.stat_size = DELTACTPSD_SZ ;
		if(ioctl(fd, DELTACTPSD, &stat) < 0) {
			printf("ioctl failed\n") ;
			exit(1) ;
		}
		val = stat.val ;
		for(len = 0 ; len < 2 ; len++, val++) {
			printf("%6.2f dB\n", actpsd_print(val->value)) ;
		}
		printf("\n") ;
		free(stat.val) ;
	} else if(strcmp(argv[1], "deltgaindn") == 0) {
		
		deltgaindn stat ;

		if(!(stat.val = malloc(DELTGAINDN_SZ))) {
			printf("No mem\n") ;
			exit(1) ;
		}
		stat.stat_size = DELTGAINDN_SZ ;
		if(ioctl(fd, DELTGAINDN, &stat) < 0) {
			printf("ioctl failed\n") ;
			exit(1) ;
		}
		val = stat.val ;
		printf("\tTone Number\t%s\n", argv[1]) ;
		for(len = 0 ; len < 512 ; len++, val++) {
			printf("\t%6d     \t%4.2f\n", val->tone_num, gain_print(val->value)) ;
		}
		printf("\n") ;
		free(stat.val) ;
	} else if(strcmp(argv[1], "deltsnrm") == 0) {
		
		deltsnrm stat ;

		if(!(stat.val = malloc(DELTSNRM_SZ))) {
			printf("No mem\n") ;
			exit(1) ;
		}
		stat.stat_size = DELTSNRM_SZ ;
		if(ioctl(fd, DELTSNRM, &stat) < 0) {
			printf("ioctl failed\n") ;
			exit(1) ;
		}
		val = stat.val ;
		for(len = 0 ; len < 2 ; len++, val++) {
			printf("%6.2f dB\n", signed_print(val->value)) ;
		}
		printf("\n") ;
		free(stat.val) ;
	} else if(strcmp(argv[1], "deltattndr") == 0) {
		
		deltattndr stat ;

		if(!(stat.val = malloc(DELTATTNDR_SZ))) {
			printf("No mem\n") ;
			exit(1) ;
		}
		stat.stat_size = DELTATTNDR_SZ ;
		if(ioctl(fd, DELTATTNDR, &stat) < 0) {
			printf("ioctl failed\n") ;
			exit(1) ;
		}
		val = stat.val ;
		for(len = 0 ; len < 2 ; len++, val++) {
			printf("%ld dB\n", val->value) ;
		}
		printf("\n") ;
		free(stat.val) ;
	} else {
		printf("Not Supported Yet!!\n") ;
	}
	close(fd) ;
}


