#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <bun_if_char.h>

#include "drvctl.h"

#ifndef LEAN
void printhelp(void);
void printhelp(void)
{
	printf("Conexant DRV-CTL Help\n");
	printf("Options:\n");
	printf("-h\t:\tTo Display the Help options\n");
	printf("-g<attr>\t:\tTo get a DSL attribute <attr>\n");
	printf("-s<attr>:<value>\t:\tTo set a DSL attribute <attr> with <value>\n");
	printf("-v <attrib>:<number of elements>\t:\tDisplay the vector <attrib> of the <port> along with their values\n");
	printf("-m <attrib>:<index>:<value>\t:\tSet the vector <attrib> with <index> of the <port> with <value>\n");
	printf("-p<port name>\t:\tName of the Port <port> on which operations are necessary\n");
	printf("-l:\tList the attributes of the port <port>\n");
}
#endif
int main(int argc, char *argv[])
{
	int c=0;
	int fd=-1,dsl_set=-1,port=-1,port_list=-1,vect_disp=-1,vect_set=-1;

	char value[BUF_SIZE * 2] = ""; //Values returned can be very long, so using 2 * BUF_SIZE
	char attribute[BUF_SIZE] = "";
	char portname[BUF_SIZE] = "";

	printf("Conexant : DRV-CTL\n");
	fflush(stdout);

	fd = open(DSLCTL_FILE,O_RDWR);
	if (fd <0) {
		printf("Open failed\n");
		goto FAIL;
	}

	while(1) {
		c = getopt(argc,argv,"lhg:s:v:p:m:");
		if(c != -1) {
			switch(c) {
				case 'g':
					if(optarg) {
						if(getCNXTattrib(fd,"a1",optarg,value) < 0)
							printf("Error in getting argument %s\n",optarg);
						printf("%s  : %s\n",optarg, value);
					}
					break;
				case 'p':
					if(optarg) {
						strncpy(portname,optarg,BUF_SIZE);
						port = 0;
					}
					break;
				case 's':
					if(optarg) {
						strncpy(attribute,optarg,BUF_SIZE);
						dsl_set = 0;
					}
					break;		
				case 'v':
					if(optarg) {
						strncpy(attribute,optarg,BUF_SIZE);
						vect_disp = 0;
					}
					break;
				case 'm':
					if(optarg) {
						strncpy(attribute,optarg,BUF_SIZE);
						vect_set = 0;
					}
					break;
				case 'l':
						port_list = 0;
					break;
				default:
				case 'h':
#ifndef LEAN
					printhelp();
#endif
					break;
			}
		}
		else
			break;
	}
	if(!port && !dsl_set) {
		setCNXTattrib(fd,portname,attribute);
	}
	if(!port && !port_list) {
		if(listCNXTattrib(fd,portname) < 0)
		printf("Error in Listing %s port\n",portname);
	}
	if(!port && !vect_disp) {
		/*
		* Vector Display return data can be bigger than BUF_SIZE
		*/
		if(dispCNXTvect(fd,portname,attribute,value) < 0) {
			printf("Display Vector failed on port %s\n",portname);
		}
		else 
			printf("%s : %s : %s\n",portname,attribute,value);
	}
	if(!port && !vect_set) {
		if(setCNXTvect(fd,portname,attribute) <0) {
			printf("Setting Vector failed on port %s\n",portname);
		}
	}
CLOSE:
	close(fd);	
	return 0;
FAIL:
	return -1;
}
