/*************************************************************/
/*************DSL APPLICATION*********************************/
/*************************************************************/ 

/******************************HEADERS************************/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <getopt.h>

#include <dslconf.h>
#include <deltdiag.h>
/******************************HEADERS END********************/

/****************************** MACRO DEFINITIONS*************/
#define MAX_SIZE 32
#define MAX_OPTIONS 10
#define STATE_IDLE 0
#define STATE_SHOWTIME 1
#define DEBUG
#define INVALID_CMD "Invalid command,"
#define HELP_USAGE "%s -h Desc:Displays help\n"
#define OAM_USAGE  "%s -oam <f4/f5> -t <etoe/segment> -v <vpi.vci> -n <repetitions> -T <timeout>\n"
#define LINK_USAGE "%s -link <up/down>\n"
#define MODE_USAGE "%s -mode <auto/g.dmt/t1.413/adsl2/adsl2+>\n"
#define VERSION_USAGE "%s -V Desc:Version info\n"
#define CONNECTION_USAGE "%s -C Desc:Displays the connection info\n"
#define STATUS_USAGE "%s -S Desc:Displays the status info\n"
#define PERFORMANCE_USAGE "%s -P Desc:Displays performance stats\n"
#define DELT_USAGE "%s -delt <start/status/statistics>\n"
#define DELT_STATS_USAGE "%s -delt statistics <snrdn/qlndn/hlindni/hlindnr/actatp/snrm/attndr>\n"

/*********************** MACRO DEFNS ENDS**********************/

/*************************VARIABLE DEFNS**********************/
static char appName[MAX_SIZE];
extern char *optarg;
extern int optind, opterr, optopt;
typedef enum cmd {
  INVALID_COMMAND=0,
  CONNECTION_ACTIVATE,
  CONNECTION_DEACTIVATE,
  MODE_AUTO,
  MODE_GDMT,
  MODE_T1413,
  MODE_ADSL2,
  MODE_ADSL2P,
  MODE_BITSWAP_ENABLE,
  MODE_BITSWAP_DISABLE,
  MODE_AUTOSRA_ENABLE,
  MODE_AUTOSRA_DISABLE,
  INFO_CONNECTION,
  INFO_STATUS,
  INFO_VERSION,
  INFO_PERFORMANCE,
  OAM_F4_ETOE,
  OAM_F5_ETOE,
  OAM_F4_SEGMENT,
  OAM_FS_SEGMENT,
  DELT_SNRDN,
  DELT_QLNDN,
  DELT_HLINDNI,
  DELT_HLINDNR,
  DELT_ACTATP,
  DELT_SNRM,
  DELT_ATTNDR
}CLI_OPERATION;

/****************************************VARIABLE DEFNS ENDS*****************/
void helpHandler()
{
   printf("\nUsage\n");
   printf(HELP_USAGE,appName);
   printf(VERSION_USAGE,appName);
   printf(CONNECTION_USAGE,appName);
   printf(STATUS_USAGE,appName);
   printf(PERFORMANCE_USAGE,appName);
   printf(LINK_USAGE, appName);
   printf(MODE_USAGE, appName);
   printf(OAM_USAGE, appName);
   printf(DELT_USAGE,appName);
   printf(DELT_STATS_USAGE,appName);
}

CLI_OPERATION getNewCliDslMode(char *cmd);

CLI_OPERATION getNewCliDslMode(char *cmd)
{
  CLI_OPERATION mode = MODE_AUTO;

  if((!strncmp(cmd, "auto", strlen(cmd))))
  {
    mode = MODE_AUTO;
  } 
  else if((!strncmp(cmd, "g.dmt", strlen(cmd))))
  {
    mode = MODE_GDMT;
  }
  else if((!strncmp(cmd, "t1.413", strlen(cmd))))
  {
    mode = MODE_T1413;
  }
  else if((!strncmp(cmd, "adsl2", strlen(cmd))))
  {
    mode = MODE_ADSL2;
  }
  else if((!strncmp(cmd, "adsl2+", strlen(cmd))))
  {
    mode = MODE_ADSL2P;
  }
  else
  {
    mode = MODE_AUTO;
  }

  return mode;
}
/********** Function prototypes**************/
void print_generic_attributes(cnxtStats_t *p_cnxtStats);
void print_gdmt_attributes(cnxtStats_t *p_cnxtStats);
void print_adsl2_attributes(cnxtStats_t *p_cnxtStats);
void print_dsl_counters(cnxtStats_t *p_cnxtStats);
void  print_performance_stats( turboMibInfo_t *turboMibInfo );
/************ Ends***************************/



void print_generic_attributes(cnxtStats_t *p_cnxtStats)
{    char *newline;
     /* Prints the generic statuses */
     //printf("\tCodeTypeStatus: %s",p_cnxtStats->GenericAttribute.dslMode1);
     //printf("\t%s\n",p_cnxtStats->GenericAttribute.dslMode2);
     //printf("\tChannels %s\n",p_cnxtStats->GenericAttribute.dslChannel);
     printf("\t\t\tLine Status: %s\n\n",p_cnxtStats->GenericAttribute.dslLineStatus);
     //printf("\tStatus: %s",p_cnxtStats->GenericAttribute.dslTrainingStatus);
     printf("%-32s:%-8s","RemoteSNRMargin",p_cnxtStats->GenericAttribute.dslSnrUp);
     printf("\t%-32s:%-8s\n","LocalSNRMargin",p_cnxtStats->GenericAttribute.dslSnrDown);

     newline = strchr(p_cnxtStats->GenericAttribute.dslAttnUp,'\n');
     if ( newline != NULL )
     {
        *newline = '\0'; /* overwrite trailing newline */
     }
     printf("%-32s:%-8s","RemoteLineAttn",p_cnxtStats->GenericAttribute.dslAttnUp);
     printf("\t%-32s:%-8s","LocalLineAttn",p_cnxtStats->GenericAttribute.dslAttnDown);
     newline = strchr(p_cnxtStats->GenericAttribute.dslPwrUp,'\n');
     if ( newline != NULL )
     {
        *newline = '\0'; /* overwrite trailing newline */
     }

     printf("%-32s:%-8s","RemoteTxPower",p_cnxtStats->GenericAttribute.dslPwrUp);
     printf("\t%-32s:%-8s","LocalTxPower",p_cnxtStats->GenericAttribute.dslPwrDown);
     printf("%-32s:%-8d","TxCellRate",p_cnxtStats->GenericAttribute.dslMaxUp);
     printf("\t%-32s:%-8d\n","RxCellRate",p_cnxtStats->GenericAttribute.dslMaxDown);
     printf("%-32s:%-8d","TxBitRate",p_cnxtStats->GenericAttribute.dslRateUp);
     printf("\t%-32s:%-8d\n","RxBitRate",p_cnxtStats->GenericAttribute.dslRateDown);
}

void print_gdmt_attributes(cnxtStats_t *p_cnxtStats)
{ 
   /* Not supported in the library as yet*/
}
void print_adsl2_attributes(cnxtStats_t *p_cnxtStats)
{ 
    /* Not supported in the library as yet*/
}
void print_dsl_counters(cnxtStats_t *p_cnxtStats)
{
   printf("%-32s:%-8d","RemoteFastChannelHEC",p_cnxtStats->Counters.dslHECUp);
   printf("\t%-32s:%-8d\n","LocalFastChannelHEC",p_cnxtStats->Counters.dslHECDown);
   printf("\t\t\t\t\t\t%-32s:%-8d\n","LocalFastChannelOCD",p_cnxtStats->Counters.dslOCDDown);
   printf("%-32s:%-8d","PhyTXCellCount",p_cnxtStats->Counters.dslTotalCellsUp);
   printf("\t%-32s:%-8d\n","PhyRXCellCount",p_cnxtStats->Counters.dslTotalCellsDown);
   printf("%-32s:%-8d\n","PhyCellDropCount",p_cnxtStats->Counters.dslDropCells);
   printf("%-32s:%-8d","RemoteTPSTCBitError",p_cnxtStats->Counters.dslBitErrorsUp);
   printf("\t%-32s:%-8d\n","LocalTPSTCBitError",p_cnxtStats->Counters.dslBitErrorsDown);
   printf("%-32s:%-8d","RemoteMgmtErroredSec",p_cnxtStats->Counters.dslESUp);
   printf("\t%-32s:%-8d\n","LocalMgmtErroredSec",p_cnxtStats->Counters.dslESDown);
   printf("%-32s:%-8d","RemoteMgmtSeverelyErroredSec",p_cnxtStats->Counters.dslSESUp);
   printf("\t%-32s:%-8d\n","LocalMgmtSeverelyErroredSec",p_cnxtStats->Counters.dslSESDown);
   printf("%-32s:%-8d","RemoteMgmtUnavailErroredSec",p_cnxtStats->Counters.dslUASUp);
   printf("\t%-32s:%-8d\n\n\n","LocalMgmtUnavailErroredSec",p_cnxtStats->Counters.dslUASDown);
}

void  print_performance_stats( turboMibInfo_t *turboMibInfo )
{
    printf("\n\tAtucChanPerfCurr1DayTimeElapsed: %d\n",turboMibInfo->performaceData.perfCurr1DayTimeElapsed);
    printf("\tAtucChanPerfPrev1DayMoniSecs: %d\n",turboMibInfo->performaceData.aturPerfPrev1DayMoniSecs);
    printf("\tAtucPerfLoss: %d\n",turboMibInfo->performaceData.perfTotal.Loss);
    printf("\tAtucPerfLofs: %d\n",turboMibInfo->performaceData.perfTotal.Lofs);
    printf("\tAtucPerfESs: %d\n",turboMibInfo->performaceData.perfTotal.ESs);
    printf("\tAturChanReceivedBlks: %d\n",turboMibInfo->FastChPerfData.perfTotal.channelRxBlks);
    printf("\tAturChanUncorrectBlks: %d\n",turboMibInfo->FastChPerfData.perfTotal.channelUncorrectBlks);
    printf("\tAtucPerfCurr1DayTimeElapsed: %d\n",turboMibInfo->performaceData.perfCurr1DayTimeElapsed);
    printf("\tAtucPerfCurr1DayLoss: %d\n",turboMibInfo->performaceData.perfCurr1Day.Loss);
    printf("\tAtucPerfCurr1DayLofs: %d\n",turboMibInfo->performaceData.perfCurr1Day.Lofs);
    printf("\tAtucPerfCurr1DayESs: %d\n",turboMibInfo->performaceData.perfCurr1Day.ESs);
    printf("\tAturChanPerfCurr1DayReceivedBlks: %d\n",turboMibInfo->FastChPerfData.perfCurr1Day.channelRxBlks);
    printf("\tAturChanPerfCurr1DayUncorrectBlks: %d\n",turboMibInfo->FastChPerfData.perfCurr1Day.channelUncorrectBlks);
    printf("\tAtucPerfCurr15MinTimeElapsed: %d\n",turboMibInfo->performaceData.perfCurr15MinTimeElapsed);
    printf("\tAtucPerfCurr15MinLoss: %d\n",turboMibInfo->performaceData.perfCurr15Min.Loss);
    printf("\tAtucPerfCurr15MinLofs: %d\n",turboMibInfo->performaceData.perfCurr15Min.Lofs);
    printf("\tAtucPerfCurr15MinESs: %d\n",turboMibInfo->performaceData.perfCurr15Min.ESs);
    printf("\tAturChanPerfCurr15MinReceivedBlks: %d\n",turboMibInfo->FastChPerfData.perfCurr15Min.channelRxBlks);
    printf("\tAturChanPerfCurr15MinUncorrectBlks: %d\n",turboMibInfo->FastChPerfData.perfCurr15Min.channelUncorrectBlks);
    printf("\tAtucPerfPrev1DayMoniSecs: %d\n",turboMibInfo->performaceData.aturPerfPrev1DayMoniSecs);
    printf("\tAtucPerfPrev1DayLoss: %d\n",turboMibInfo->performaceData.perfPrev1Day.Loss);
    printf("\tAtucPerfPrev1DayLofs: %d\n",turboMibInfo->performaceData.perfPrev1Day.Lofs);
    printf("\tAtucPerfPrev1DayESs: %d\n",turboMibInfo->performaceData.perfPrev1Day.ESs);
    printf("\tAturChanPerfPrev1DayReceivedBlks: %d\n",turboMibInfo->FastChPerfData.perfPrev1Day.channelRxBlks);
    printf("\tAturChanPerfPrev1DayUncorrectBlks: %d\n\n",turboMibInfo->FastChPerfData.perfPrev1Day.channelUncorrectBlks);
}

void print_delt_stats(int stats)
{
   int len,fd, cmd;
   char str_cmd[40];
   common_val *val ;
   switch (stats )
   {
      case DELT_SNRDN:
      case DELT_QLNDN:
      {
            deltsnrdn stats_delt;
            stats_delt.stat_size = DELTSNRDN_SZ;
            stats_delt.val = malloc(DELTSNRDN_SZ);
            if ( stats == DELT_SNRDN )
            {   cmd =CNXT_DELT_SNRDN ;
                strcpy(str_cmd,"SNR");
            }
            if ( stats == DELT_QLNDN )
            {   cmd = CNXT_DELT_QLNDN ;
                strcpy(str_cmd,"QLN");
            }
            if ( cnxtGetDeltParameters(cmd,(void*)(&stats_delt)) )
                printf("Error while fetching stats for %s\n",cmd);
            else
            {   val = stats_delt.val ;
                printf("\tTone Number\t%s\n",str_cmd);
                for ( len = 0 ; len < 512 ; len++,val++)
                {
                       printf("\t%6d     \t%3d\n", val->tone_num, val->value) ;
                }
                printf("\n") ;
            }
            break;
       }
       case DELT_HLINDNI:
       case DELT_HLINDNR:
       {
            delthlindnr stats_delt ;
            if(!(stats_delt.val = malloc(DELTHLINDNI_SZ))) {
	       printf("No mem\n") ;
	       exit(1) ;
	    }
            stats_delt.stat_size = DELTHLINDNI_SZ ;
            if ( stats == DELT_HLINDNI )
            {   cmd = CNXT_DELT_HLINDNI;
                strcpy(str_cmd,"HLINDNI");
            }
            if ( stats == DELT_HLINDNR )
            {   cmd = CNXT_DELT_HLINDNR;
                strcpy(str_cmd,"HLINDNR");
            }
            if ( cnxtGetDeltParameters(cmd,(void*)(&stats_delt)))
                printf("Error while fetching stats for %s\n",cmd);
            else
            {
		val = stats_delt.val ;
                printf("scale: %e\n", val->value) ;
		val++ ;
		printf("\tTone Number\t%s\n", str_cmd) ;
		for(len = 0 ; len < 512 ; len++, val++) {
			printf("\t%6d     \t%d\n", val->tone_num, val->value) ;
		}
		printf("\n") ;
            }
		free(stats_delt.val) ;
                break;
        }
       case DELT_ACTATP:
       {
            deltactatp stats_delt ;
            if(!(stats_delt.val = malloc(DELTACTATP_SZ))) {
			printf("No mem\n") ;
			exit(1) ;
		}
		stats_delt.stat_size = DELTACTATP_SZ ;
                if ( cnxtGetDeltParameters(CNXT_DELT_ACTATP,(void*)(&stats_delt)))
                    printf("Error while fetching stats for DELTACTATP\n");
                else
		{
                    val = stats_delt.val ;
		    for( len = 0 ; len < 2 ; len++, val++) {
                        if (len == 0)printf("\n\tACTATPup : ");
                        else printf("\tACTATPdown : ");
			printf("%6.1f dB\n", signed_print(val->value)) ;
		}
		printf("\n") ;
                }
		free(stats_delt.val) ;
                break;
                }
  case DELT_SNRM:
  {
       deltsnrm delt_stats ;
       if(!(delt_stats.val = malloc(DELTSNRM_SZ))) {
			printf("No mem\n") ;
			exit(1) ;
        }
	delt_stats.stat_size = DELTSNRM_SZ ;
        if ( cnxtGetDeltParameters(CNXT_DELT_SNRM,(void*)(&delt_stats)))
                printf("Error while fetching stats for DELTSNRM\n");
        else
        {
	     val = delt_stats.val ;
	     for(len = 0 ; len < 2 ; len++, val++) {
             if (len == 0)printf("\n\tSNRMup : ");
               else printf("\tSNRMdown : ");

	  	printf("%6.2f dB\n", signed_print(val->value)) ;
	     }
		printf("\n") ;
        }
         break;
	 free(delt_stats.val) ;
  }
  case DELT_ATTNDR:
  {      
        deltattndr delt_stats ;
        if(!(delt_stats.val = malloc(DELTATTNDR_SZ))) {
			printf("No mem\n") ;
			exit(1) ;
	}
	delt_stats.stat_size = DELTATTNDR_SZ ;
        if ( cnxtGetDeltParameters(CNXT_DELT_ATTNDR,(void*)(&delt_stats)))
                    printf("Error while fetching stats for DELTATTNDR\n");
        else
        {
		val = delt_stats.val ;
		for(len = 0 ; len < 2 ; len++, val++) {
                if (len == 0)printf("\n\tATTNDRup : ");
                else printf("\tATTNDRdown : ");
			printf("%ld dB\n", val->value) ;
		}
		printf("\n") ;
        }
        break;
	free(delt_stats.val) ;
 }
}/* End of switch */

}

int main(int argc, char **argv)
{
  char *optstring = "l::m::o::d::h::t::v::n::T::V::C::S::P";
  int c;
  int ret = 0;
  int i=0;
  CNXTDSL_STATUS libstatus = STATUS_ERROR;
  CLI_OPERATION command = INVALID_COMMAND;
 
  memset(appName,0,sizeof(appName));
  strcpy(appName, *argv);
      
  if(argc > 1) {
    c = getopt(argc, argv, optstring);
    switch(c) {
      case 'h': {
                  helpHandler();
                  break;
                }
      case 'V': {
                    /* Get the version info now */
                    turboVersionInfo_t turboVersionInfo;
                    memset(&turboVersionInfo, 0, sizeof(turboVersionInfo_t));
                    if ( cnxtDslGetVersionInfo(&turboVersionInfo) )
                        printf("ERROR\n");
                    else
                    {   printf("\n\t%s",turboVersionInfo.phyVerStr);
                        printf("\t%s",turboVersionInfo.drvVerStr);
                        printf("\t%s\n\n",turboVersionInfo.fwVersion);
                         
                    }
                    break;
                }
      case 'C': {
                    connInfo_t connInfo;
                    memset(&connInfo, 0, sizeof(connInfo_t));
                    if ( cnxtDslGetConnectionInfo(&connInfo) )
                        printf("ERROR\n");
                    else
                    {   
                         printf("\t\t\t State: %s \n",connInfo.LinkState);
                         if ( !strcmp(connInfo.LinkState,"Showtime") )
                         { 
                             printf("\n%-32s:%8d","TX bit rate", connInfo.TxBitRate);
                             printf("\t%-32s:%8d\n","RX bit rate",connInfo.RxBitRate);
                             printf("%-32s:%8d", "LocalFastChannelTxRate",connInfo.FastUSRate);
                             printf("\t%-32s:%8d\n","LocalFastChannelRxRate",connInfo.FastDSRate);
                             printf("%-32s:%8d","LocalInterleavedChannelTxRate",connInfo.InterUSRate);
                             printf("\t%-32s:%8d\n\n","LocalInterleavedChannelRxRate",connInfo.InterDSRate);
                         }
                    }
                    break;
                }
      case 'S': {   
                    cnxtStats_t cnxtStats;
                    connInfo_t connInfo;
                    memset(&connInfo, 0, sizeof(connInfo_t));
                    memset(&cnxtStats,0,sizeof(cnxtStats_t));
                    if ( cnxtDslGetStatusInfo(&cnxtStats) )
                       printf("ERROR");
                    else
                    { 
                        if ( cnxtDslGetConnectionInfo(&connInfo) )
                            printf("ERROR\n");
                        else
                            {   printf("\t\t\t State: %s \n",connInfo.LinkState);
                                if ( !strcmp(connInfo.LinkState,"Showtime") )
                                {   print_generic_attributes(&cnxtStats);
                                    print_gdmt_attributes(&cnxtStats);
                                    print_adsl2_attributes(&cnxtStats);
                                    print_dsl_counters(&cnxtStats);
                                }
                            }
                    }

                    break;
                }
      case 'P': { 
                   turboMibInfo_t turboMibInfo;
                   memset(&turboMibInfo,0,sizeof(turboMibInfo_t));
                   if ( cnxtGetPerformanceStats(&turboMibInfo) )
                       printf("ERROR");
                   else
                      print_performance_stats(&turboMibInfo);
                   break;
                } 
      case 'd': {
                 int stats = 0, result;
                 if (!strcmp(argv[optind],"start"))
                 { 
                        if (cnxtStartDeltTest())
                            printf("Error while starting DELT test\n");
                        else
                            printf("\tDELT test started\n");
                 } 
                 else if (!strcmp(argv[optind],"status"))
                 {    
                        result = cnxtGetDeltStatus();
                        if ( result == DELT_COMPLETED )
                           printf("\tState = DELT\n" );
                        else if ( result == DELT_RUNNING )
                           printf("\tState = DELT in progress\n");
                        else if ( result == DATA_MODE )
                           printf("\tState =  Data\n");
                        else printf("\tInvalid State\n");
                 }
                 else if (!strcmp(argv[optind],"statistics"))
                 { 
                     /*Statistics are only shown in case of DELT state*/
                     if (!strcmp("snrdn",argv[optind+1]))
                        stats = DELT_SNRDN;
                     else if (!strcmp("qlndn",argv[optind+1]))
                        stats = DELT_QLNDN;
                     else if (!strcmp("hlindni",argv[optind+1]))
                        stats = DELT_HLINDNI;
                     else if (!strcmp("hlindnr",argv[optind+1]))
                        stats = DELT_HLINDNR;
                     else if (!strcmp("actatp",argv[optind+1]))
                        stats = DELT_ACTATP;
                     else if (!strcmp("snrm",argv[optind+1]))
                         stats = DELT_SNRM;
                     else if (!strcmp("attndr",argv[optind+1]))
                         stats = DELT_ATTNDR;
                     else
                     {   
                            printf("Incorrect Option\n");
                            printf(DELT_STATS_USAGE,appName);
                            break;
                     }
                      print_delt_stats(stats);
                 }
                 else 
                 {
                        printf("Wrong option\n");
                        printf(DELT_USAGE,appName);
                 }
                 break;
                }
     case 'l': {
                  if(!strncmp(argv[optind-1], "-link", strlen(argv[optind-1])))
                  {
                    if(argv[optind])
                    {
                      if((!strncmp(argv[optind], "up", strlen(argv[optind]))) || 
                          (!strncmp(argv[optind], "down", strlen(argv[optind]))))
                      {
#ifdef USE_DSLCONF_LIB
                        libstatus = (!strncmp(argv[optind], "up", strlen(argv[optind])))?
                          cnxtDslActivateLine():cnxtDslDeActivateLine();
                        if(libstatus != STATUS_SUCCESS)
                        {
                          printf("Error: %s %s\n", argv[optind-1], argv[optind]);
                          break;
                        }
#endif /* USE_DSLCONF_LIB */ 
                      }
                      else
                      {
                        printf("wrong option for -link: use -link <up/down>\n");
                        printf(LINK_USAGE, appName);
                        break;
                      }
                    }
                    else
                    {
                      printf("wrong option for -link: use -link <up/down>\n");
                      printf(LINK_USAGE, appName);
                      break;
                    }
                  }
                  else
                  {
                    printf("wrong option : use -link <up/down>\n");
                    printf(LINK_USAGE, appName);
                    break;
                  }

                } break;

      case 'm': {
                  dslConfigInfo_t dslConfInfo;

                  memset(&dslConfInfo, 0, sizeof(dslConfigInfo_t));
                  DEBUG("option m used\n");
                  if(!strncmp(argv[optind-1], "-mode", strlen(argv[optind-1])))
                  {
                    if(argv[optind])
                    {
                      CLI_OPERATION mode;                    
                      mode = getNewCliDslMode(argv[optind]);

                      switch(mode)
                      {
                        case MODE_AUTO: {
                                          printf("AUTO mode selected\n");
                                          dslConfInfo.dslAnnexAParam = kAutoModeOnly;
                                          break;
                                        } 
                        case MODE_GDMT: {
                                          printf("G.DMT mode selected\n");
                                          dslConfInfo.dslAnnexAParam = kGdmtOnly;
                                          break;
                                        } 
                        case MODE_T1413: {
                                           printf("T1.413 mode selected\n");
                                           dslConfInfo.dslAnnexAParam = kT1413Only;
                                           break;
                                         } 
                        case MODE_ADSL2: {
                                           printf("ADSL2 mode selected\n");
                                           dslConfInfo.dslAnnexAParam = kAdsl2Only;
                                           break;
                                         } 
                        case MODE_ADSL2P: {
                                            printf("ADSL2P mode selected\n");
                                            dslConfInfo.dslAnnexAParam = kAdsl2plusOnly;
                                            break;
                                          }
                        default: {
                                   printf("Unknown mode: Selecting AUTO mode selected\n");
                                   dslConfInfo.dslAnnexAParam = kAutoModeOnly;
                                 }                        
                      }

#ifdef USE_DSLCONF_LIB
                      libstatus = cnxtDslModifyConfiguration(&dslConfInfo);
                      if(libstatus != STATUS_SUCCESS)
                      {
                        printf("Error: Modifying DSL configuration.\n");
                        break;
                      }
#endif /* USE_DSLCONF_LIB */ 
                    }
                    else
                    {
                      printf("no option specified for -mode\n");
                      printf(MODE_USAGE, appName);
                      break;
                    }
                  }
                  else
                  {
                    printf("wrong option %s\n", argv[optind-1]);
                    printf(MODE_USAGE, appName);
                    break;
                  }
                } break;

      case 'o': {
                  /* local variable which will be passed to the library */
                  cnxtOamLoop_t oamLoopInfo;
                  if(!strncmp(argv[optind-1], "-oam", strlen(argv[optind-1])))
                  {

                    memset(&oamLoopInfo, 0, sizeof(cnxtOamLoop_t));

                    /* check F4 or F5 type */
                    if(argv[optind])
                    {
                      if((!strncmp(argv[optind], "f4", strlen(argv[optind]))) || 
                          (!strncmp(argv[optind], "f5", strlen(argv[optind]))))
                      {
                        DEBUG("%s\n", argv[optind]);
                        strcpy(oamLoopInfo.flags,
                            (!strncmp(argv[optind], "f4", strlen(argv[optind])))?
                            CNXT_OAMCLI_FLAGSTR_F4:CNXT_OAMCLI_FLAGSTR_F5);
                      }
                      else
                      {
                        printf("wrong option for -oam: use -o <f4/f5>\n");
                        printf(OAM_USAGE , appName);
                        break;
                      }
                    }
                    else
                    {
                      printf("wrong option for -oam: use -o <f4/f5>\n");
                      printf(OAM_USAGE , appName);
                      break;
                    }

                    /* we'll reach here iff -oam <f4/f5>, opt is correct  */
                    /* now go for the etoe/segment string */

                    if ((c = getopt(argc, argv, optstring)) != -1)
                    {
                      switch(c) {
                        case 't': {
                                    DEBUG("option %c used\n", c);
                                    if(argv[optind])
                                    {
                                      /* extract the sub-type (etoe/segment) here */
                                      if((!strncmp(argv[optind], "etoe", strlen(argv[optind]))) || 
                                          (!strncmp(argv[optind], "segment", strlen(argv[optind]))))
                                      {
                                        DEBUG("%s\n", argv[optind]);
                                        /* This implies type is segment */
                                        strcpy(oamLoopInfo.type,
                                            (!strncmp(argv[optind], "etoe", strlen(argv[optind])))?
                                            CNXT_OAMCLI_FLAGSTR_ETOE:CNXT_OAMCLI_FLAGSTR_SEGMENT);
                                        /* after extracting 'sub-type' go n check for 'vpi.vci' */
                                      }
                                      else
                                      {
                                        printf("wrong option for -t: use -t <etoe/segment>\n");
                                        printf(OAM_USAGE , appName);
                                        break;
                                      }
                                    }
                                    else
                                    {
                                      printf("wrong option for -t: use -t <etoe/segment>\n");
                                      printf(OAM_USAGE , appName);
                                      break;
                                    }

                                    /* we'll reach here iff -t <etoe/segment>, opt is correct  */
                                    /* now go for the vpi.vci string */

                                    if ((c = getopt(argc, argv, optstring)) != -1)
                                    {
                                      switch(c) {
                                        case 'v':
                                          {
                                            DEBUG("option %c used\n", c);
                                            if(argv[optind])
                                            {
                                              {
                                                if(strchr(argv[optind], '.'))
                                                {                                              
                                                  /* extract the vpi.vci here */
                                                  DEBUG("%s\n", argv[optind]);
                                                  sscanf(argv[optind], "%d.%d", &oamLoopInfo.vpi,&oamLoopInfo.vci);
                                                }
                                                else
                                                {
                                                  printf("Specify the vpi & vci, -v <vpi.vci>\n");
                                                  printf(OAM_USAGE , appName);
                                                  break;
                                                }
                                              }

                                            }      
                                            else
                                            {
                                              printf("wrong option for -v: use -v <vpi.vci> option\n");
                                              printf(OAM_USAGE , appName);
                                              break;
                                            }

                                            /* we'll reach here iff -v <vpi.vci>, opt is correct  */
                                            /* go for number of repetions & timeout */

                                            if ((c = getopt(argc, argv, optstring)) != -1)
                                            {
                                              switch(c) {
                                                case 'n':
                                                  {
                                                    DEBUG("option %c used\n", c);
                                                    if(argv[optind])
                                                    {
                                                      {
                                                        /* extract the number of rep here */
                                                        DEBUG("%s\n", argv[optind]);
                                                        oamLoopInfo.numRepetitions = strtoul(argv[optind], NULL, 10);
                                                        if (!oamLoopInfo.numRepetitions)
                                                        {
                                                          oamLoopInfo.numRepetitions = 1;
                                                        }
                                                      }
                                                    }
                                                    else
                                                    {
                                                      printf("no value for -n <repetiton>\n");
                                                      printf(OAM_USAGE , appName);
                                                      break;
                                                    }
                                                  } break;                                                
                                                default:
                                                  {
                                                    printf("no -n <repetiton>\n");
                                                    printf(OAM_USAGE , appName);
                                                    break;
                                                  }                                                
                                              }
                                            }
                                            else
                                            {
                                              printf("no -n <repetiton> option\n");
                                              printf(OAM_USAGE , appName);
                                              break;
                                            }

                                            /* we'll reach here iff -n <repetition>, opt is correct  */
                                            /* go for number of timeout */

                                            if ((c = getopt(argc, argv, optstring)) != -1)
                                            {

                                              switch(c) {
                                                case 'T':
                                                  {
                                                    DEBUG("option %c used\n", c);
                                                    if(argv[optind])
                                                    {
                                                      /* extract the Timeout value here */
                                                      DEBUG("%s\n", argv[optind]);
                                                      oamLoopInfo.timeout = strtoul(argv[optind], NULL, 10);
                                                      printf("Got Complete command for OAM Loopback test\n");
                                                      printf("vpi:%d vci: %d\n", oamLoopInfo.vpi, oamLoopInfo.vci);
                                                      printf("flags: %s, type: %s\n", oamLoopInfo.flags, oamLoopInfo.type);
                                                      printf("timeout: %d, repetitions : %d\n", oamLoopInfo.timeout, oamLoopInfo.numRepetitions);

#ifdef USE_DSLCONF_LIB
                                                      libstatus = cnxtStartOamTest(&oamLoopInfo);
                                                      if(libstatus != STATUS_SUCCESS)
                                                      {
                                                        printf("Error: Performing OAM test.\n");
                                                        break;
                                                      }
#endif /* USE_DSLCONF_LIB */ 
                                                    }
                                                    else
                                                    {
                                                      printf("no timeout value for, -T <timeout> option\n");
                                                      printf(OAM_USAGE , appName);
                                                      break;
                                                    }
                                                    break;
                                                  }
                                                default:
                                                  {
                                                    printf("no -T <timeout> option\n");
                                                    printf(OAM_USAGE , appName);
                                                    break;
                                                  }
                                              }
                                            }
                                            else
                                            {
                                              printf("no -T <timeout> option\n");
                                              printf(OAM_USAGE , appName);
                                              break;
                                            }


                                          } break;

                                        default:
                                          {
                                            printf("no -v <vpi.vci> option\n");
                                            printf(OAM_USAGE , appName);
                                          }
                                      }
                                    }
                                    else
                                    {
                                      printf("no -v <vpi.vci> option\n");
                                      printf(OAM_USAGE , appName);
                                    }
                                    break;
                                  }
                        default: {
                                   printf(OAM_USAGE , appName);
                                 }

                      }
                    }
                    else
                    {
                      printf(OAM_USAGE , appName);
                    }
                  }
                  else
                  {
                    printf(OAM_USAGE , appName);
                  }
                } break;

      default: {
                 helpHandler();
               }
    }
  }
  else
  {
    helpHandler();
  }
}
