

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <time.h>

#include <dslconf.h>
#include <deltdiag.h>

float f_print(unsigned int value)
{
  int v = (short) value, sign = 1 ;

  if(v < 0) {
	v = -v ;
	sign = -1 ;
  }

  return ((v * 10000) >> 15) / 10000. * sign ;  /* divided by 2**15 */
}

float e_print(unsigned int value)
{
  float temp = value ;       /* take the argument as unsigned */

  temp = (value * 10000) >> 15;  /* divided by 2**15 */

  while(temp && (temp / 10000. == 0)) {
	temp *= 10;
  }
  return temp / 10000. ;
}

float signed_print(unsigned int value)
{
  int temp;

  temp = value << 6;
  if (temp < 0) {
	temp = -temp;
  }
  value = ((unsigned int) temp) >> 6;
  return ((float) value) / 10 ;
}


float actpsd_print(int value)
{
  return (float)(value / 10. +
	  .1 * ((value % 10) / 2) +
	  .01 * (((value % 10) % 2) *5)) ;
}


float gain_print(int value)
{
  return (float) value / 100. ;
}

int cnxtSetDslAttribute(char *attribute , char *value);
int cnxtGetDslAttribute(char *attribute, char* out);

CNXTDSL_STATUS cnxtStartDeltTest(void)
{
  CNXTDSL_STATUS ret = STATUS_SUCCESS;

  ret = cnxtSetDslAttribute("ActivateLine", "Abort");
  if(ret == STATUS_ERROR)
  {
	return STATUS_ERROR;
  }

  ret = cnxtSetDslAttribute("Action", "DELT");
  if(ret == STATUS_ERROR)
  {
	return STATUS_ERROR;
  }

  ret = cnxtSetDslAttribute("ActivateLine", "Start");
  if(ret == STATUS_ERROR)
  {
	return STATUS_ERROR;
  }
  return STATUS_SUCCESS;
}

DSL_DELT_STATUS cnxtGetDeltStatus(void)
{
  DSL_DELT_STATUS status;
  connInfo_t connInfo;
  cnxtDslGetConnectionInfo( &connInfo );
  if(!strcmp(connInfo.LinkState,"Showtime"))
  {
	status = DATA_MODE;
  }
  else if (!strcmp(connInfo.LinkState,"DELT"))
  {
	status = DELT_COMPLETED;
  }
  else
  {
	status = DELT_RUNNING;
  }

  return status;
}

CNXTDSL_STATUS cnxtChangeDeltToDataMode(void)
{
  CNXTDSL_STATUS ret = STATUS_SUCCESS;

  ret = cnxtSetDslAttribute("ActivateLine", "Abort");
  if(ret == STATUS_ERROR)
  {
	return STATUS_ERROR;
  }

  ret = cnxtSetDslAttribute("Action", "Startup");
  if(ret == STATUS_ERROR)
  {
	return STATUS_ERROR;
  }

  ret = cnxtSetDslAttribute("ActivateLine", "Start");
  if(ret == STATUS_ERROR)
  {
	return STATUS_ERROR;
  }
  return STATUS_SUCCESS;
}

CNXTDSL_STATUS cnxtGetDeltParameters(CNXT_DELT_PARAM deltParam, void *deltResults)
{
  int fd, len, cmd ;
  char* delt_data ;
  common_val *val ;

  if (cnxtGetDeltStatus() == DELT_COMPLETED)
  {

	if((fd = open("/dev/deltdiag", O_RDONLY, 0)) < 0) {
	  printf("failed open\n") ;
	  return STATUS_ERROR;
	}

	if((deltParam == CNXT_DELT_QLNDN) || (deltParam == CNXT_DELT_SNRDN)) {

	  if(!deltResults) {
		printf("No mem\n") ;
		return STATUS_ERROR;
	  }

	  cmd = deltParam;
	  cmd = cmd ? DELTQLNDN : DELTSNRDN ;

	  if(ioctl(fd, cmd, deltResults) < 0) {
		printf("ioctl failed\n") ;
		return STATUS_ERROR;
	  }

#ifdef PRINT_STATS
	  printf("\tTone Number\t%s\n", argv[1]) ;
	  for(len = 0 ; len < 512 ; len++, val++) {
		printf("\t%6d     \t%3d\n", val->tone_num, val->value) ;
	  }
	  printf("\n") ;
#endif
	} else if(deltParam == CNXT_DELT_ACTATP) {

	  if(!deltResults) {
		printf("No mem\n") ;
		return STATUS_ERROR;
	  }

	  if(ioctl(fd, DELTACTATP, deltResults) < 0) {
		printf("ioctl failed\n") ;
		return STATUS_ERROR;
	  }
#ifdef PRINT_STATS
	  for(len = 0 ; len < 2 ; len++, val++) {
		printf("%6.1f dB\n", signed_print(val->value)) ;
	  }
	  printf("\n") ;
#endif
	} else if((deltParam == CNXT_DELT_HLINDNI) || (deltParam == CNXT_DELT_HLINDNR)) {

	  if(!deltResults) {
		printf("No mem\n") ;
		return STATUS_ERROR;
	  }

	  if(deltParam == CNXT_DELT_HLINDNI)
	  {
		cmd = DELTHLINDNI;
	  }
	  else
	  {
		cmd = DELTHLINDNR;
	  }

	  if(ioctl(fd, cmd, deltResults) < 0) {
		printf("ioctl failed\n") ;
		return STATUS_ERROR;
	  }
#ifdef PRINT_STATS
	  printf("scale: %e\n", val->value) ;
	  val++ ;
	  printf("\tTone Number\t%s\n", argv[1]) ;
	  for(len = 0 ; len < 512 ; len++, val++) {
		printf("\t%6d     \t%d\n", val->tone_num, val->value) ;
	  }
	  printf("\n") ;
#endif
	} else if(deltParam == CNXT_DELT_SNRM) {

	  if(!deltResults) {
		printf("No mem\n") ;
		return STATUS_ERROR;
	  }

	  if(ioctl(fd, DELTSNRM, deltResults) < 0) {
		printf("ioctl failed\n") ;
		return STATUS_ERROR;
	  }
#ifdef PRINT_STATS
	  for(len = 0 ; len < 2 ; len++, val++) {
		printf("%6.2f dB\n", signed_print(val->value)) ;
	  }
	  printf("\n") ;
#endif
	} else if(deltParam == CNXT_DELT_ATTNDR) {

	  if(!deltResults) {
		printf("No mem\n") ;
		return STATUS_ERROR;
	  }

	  if(ioctl(fd, DELTATTNDR, deltResults) < 0) {
		printf("ioctl failed\n") ;
		return STATUS_ERROR;
	  }
#ifdef PRINT_STATS
	  for(len = 0 ; len < 2 ; len++, val++) {
		printf("%ld dB\n", val->value) ;
	  }
	  printf("\n") ;
#endif
	} else {
	  printf("Not Supported Yet!!\n") ;
	}
	close(fd) ;

	return STATUS_SUCCESS;
  }
  else
  {
	return STATUS_ERROR;
  }
}

