#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include <dslconf.h>

/* Function prototypes */

static int cnxtSystemCmd(char *name);
int cnxtSetDslAttribute(char *attribute , char *value);
int cnxtGetDslAttribute(char *attribute, char* out);
static int cnxt_get_dsl_config_attr(char *attribute, char* out, cnxtStats_t *status);

/***************************************************************************
 * Function Name: cnxtDslActivateLine 
 * Description  : Start ADSL connection.
 * Returns      : SUCCESS  successful or error status.
 ***************************************************************************/

CNXTDSL_STATUS cnxtDslActivateLine( void )
{
  status_t stat;

  stat.status = STATUS_ERROR;

  /* ADSL connection is enabled by default.
   * Following command can be used to activate
   * the link manually:
   */
  stat.status  = cnxtSetDslAttribute("Action", "Startup");
  stat.status  = cnxtSetDslAttribute("ActivateLine", "Start");

  return( stat.status );
} /* cnxtDslActivateLine */

/***************************************************************************
 * Function Name: cnxtDslDeActivateLine 
 * Description  : Aborts DSL connection.
 * Returns      : SUCCESS  successful or error status.
 ***************************************************************************/

CNXTDSL_STATUS cnxtDslDeActivateLine( void )
{
  return( cnxtSetDslAttribute("ActivateLine", "Abort"));
}

/***************************************************************************
 * Function Name: cnxtDslGetConnectionInfo
 * Description  : This function updates the user provided structure 
 *                [pConnInfo].
 * Returns      : CNXTDSL_STATUS 
 ***************************************************************************/

CNXTDSL_STATUS cnxtDslGetConnectionInfo( pConnInfo_t pConnInfo )
{

  char out [512];
  char *value_str;
  int err = -1;

  memset(out,0,512);
  cnxtGetDslAttribute("State",out);
  value_str = strchr(out, ':'); 

  if ((strncmp(value_str+2,"Idle", strlen("Idle")) == 0) )
  {
    strcpy(pConnInfo->LinkState,"Idle"); 
  }
  else if ((strncmp(value_str+2,"Showtime", strlen("Showtime")) == 0)) 
  {
    strcpy(pConnInfo->LinkState,"Showtime"); 
  }
  else if ((strncmp(value_str+2,"BitSwapping", strlen("BitSwapping")) == 0)) 
  {
    strcpy(pConnInfo->LinkState,"BitSwapping"); 
  }
  else if ((strncmp(value_str+2,"HandShake", strlen("HandShake")) == 0)) 
  {
    strcpy(pConnInfo->LinkState,"HandShake"); 
  }
  else if ((strncmp(value_str+2,"Training", strlen("Training")) == 0)) 
  {
    strcpy(pConnInfo->LinkState,"Training"); 
  }
  else if ((strncmp(value_str+2,"Discovery", strlen("Discovery")) == 0)) 
  {
    strcpy(pConnInfo->LinkState,"Discovery"); 
  }
  else if ((strncmp(value_str+2,"Training(DELT)", strlen("Training(DELT)")) == 0)) 
  {
    strcpy(pConnInfo->LinkState,"Training(DELT)");; 
  }
  else if ((strncmp(value_str+2,"DELT", strlen("DELT")) == 0)) 
  {
    strcpy(pConnInfo->LinkState,"DELT"); 
  }
  else if ((strncmp(value_str+2,"SpectrumTest", strlen("SpectrumTest")) == 0)) 
  {
    strcpy(pConnInfo->LinkState,"SpectrumTest"); 
  }
  else if ((strncmp(value_str+2,"RSMemoryTest", strlen("RSMemoryTest")) == 0)) 
  {
    strcpy(pConnInfo->LinkState,"RSMemoryTest");
  }

  /* get TxBitRate */
  memset(out,0,512);
  err = cnxtGetDslAttribute("TxBitRate", out);
  if(err == -1)
  {
    printf(" error happened \n");
    return 1;
  }
  value_str = strchr(out, ':');
  pConnInfo->TxBitRate = strtoul(value_str+1, NULL, 0);

  /* get RxBitRate */
  memset(out,0,512);
  err = cnxtGetDslAttribute("RxBitRate", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  pConnInfo->RxBitRate = strtoul(value_str+1, NULL, 0);

  /* get LocalFastChannelTxRate */
  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalFastChannelTxRate", out);
  if(err == -1)
  {
    printf(" error happened \n");
    return 1;
  }
  value_str = strchr(out, ':');
  pConnInfo->FastUSRate = strtoul(value_str+1, NULL, 0);
  /* get LocalFastChannelRxRate */
  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalFastChannelRxRate", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  pConnInfo->FastDSRate = strtoul(value_str+1, NULL, 0);


  /* get LocalInterleavedChannelRxRate */
  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalInterleavedChannelRxRate", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  pConnInfo->InterUSRate = strtoul(value_str+1, NULL, 0);
  /* get LocalInterleavedChannelTxRate */
  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalInterleavedChannelTxRate", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  pConnInfo->InterDSRate = strtoul(value_str+1, NULL, 0);

  return 0;
} /* cnxtDslGetConnectionInfo */

/***************************************************************************
 * Function Name: cnxtDslGetStatusInfo 
 * Description  : This function updates the structure cnxtStats_t with the 
 *                latest connection information.
 * Returns      : CNXTDSL_STATUS 
 ***************************************************************************/

CNXTDSL_STATUS cnxtDslGetStatusInfo( cnxtStats_t *status)
{

  char out [512];
  char *value_str;
  char intValInStr[32];
  char *endPtr;
  int err = -1, len = 0;

  /* Generic DSL Attribute */

  memset(out,0,512);
  err = cnxtGetDslAttribute("CodeTypeStatus", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  memset(status->GenericAttribute.dslMode1,0,32);
  strcpy(status->GenericAttribute.dslMode1, value_str+1);

  memset(status->GenericAttribute.dslMode2,0,32);

  //FIXME
  /* get Channels */
  memset(out,0,512);
  err = cnxtGetDslAttribute("Channels", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  strcpy(status->GenericAttribute.dslChannel, "TBD: FIXME"); //FIXME
  //strcpy(status->GenericAttribute.dslChannel, value_str+1);

  memset(out,0,512);
  err = cnxtGetDslAttribute("LineStatus", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  memset(status->GenericAttribute.dslLineStatus,0,32);
  strcpy(status->GenericAttribute.dslLineStatus, value_str+1);

  memset(out,0,512);
  err = cnxtGetDslAttribute("State", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  memset(status->GenericAttribute.dslTrainingStatus,0,32);
  strcpy(status->GenericAttribute.dslTrainingStatus, value_str+1);

  memset(out,0,512);
  err = cnxtGetDslAttribute("RemoteSNRMargin", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');

  memset(status->GenericAttribute.dslSnrUp ,0,32);
  strncpy(status->GenericAttribute.dslSnrUp , value_str+1, (strlen(value_str+1)-1));

  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalSNRMargin", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');

  memset(status->GenericAttribute.dslSnrDown ,0,32);
  strncpy(status->GenericAttribute.dslSnrDown, value_str+1, (strlen(value_str+1)-1));

  memset(out,0,512);
  err = cnxtGetDslAttribute("RemoteLineAttn", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  strcpy(status->GenericAttribute.dslAttnUp, value_str+1);

  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalLineAttn", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  strcpy(status->GenericAttribute.dslAttnDown, value_str+1);

  memset(out,0,512);
  err = cnxtGetDslAttribute("RemoteTxPower", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  strcpy(status->GenericAttribute.dslPwrUp , value_str+1);

  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalTxPower", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  strcpy(status->GenericAttribute.dslPwrDown , value_str+1);

  memset(out,0,512);
  err = cnxtGetDslAttribute("TxCellRate", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->GenericAttribute.dslMaxUp = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("RxCellRate", out); 
  if(err == -1)
  {
    return 1;
  }
  //printf("out: %s \n");
  value_str = strchr(out, ':');
  status->GenericAttribute.dslMaxDown= strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("TxBitRate", out); 
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->GenericAttribute.dslRateUp = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("RxBitRate", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->GenericAttribute.dslRateDown = strtoul(value_str+1, NULL, 0);

  /* ADSL2 Framing */
  #if 0
  memset(out,0,512);
  err = cnxt_get_dsl_config_attr("IntlAttr", out, status );
  if(err == -1)
  {
    return 1;
  }
  #endif
  /* Counters */

  memset(out,0,512);
  err = cnxtGetDslAttribute("RemoteFastChannelHEC", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslHECUp = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalFastChannelHEC", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslHECDown = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalFastChannelOCD", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslOCDDown = strtoul(value_str+1, NULL, 0);

  /* get OCD up */
  /* N/A */

  memset(out,0,512);
  err = cnxtGetDslAttribute("PhyTXCellCount", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslTotalCellsUp = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("PhyRXCellCount", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslTotalCellsDown = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("PhyCellDropCount", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslDropCells = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("RemoteTPSTCBitError", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslBitErrorsUp = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalTPSTCBitError", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslBitErrorsDown = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("RemoteMgmtErroredSec", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslESUp = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalMgmtErroredSec", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslESDown = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("RemoteMgmtSeverelyErroredSec", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslSESUp = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalMgmtSeverelyErroredSec", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslSESDown = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("RemoteMgmtUnavailErroredSec", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslUASUp = strtoul(value_str+1, NULL, 0);

  memset(out,0,512);
  err = cnxtGetDslAttribute("LocalMgmtUnavailErroredSec", out);
  if(err == -1)
  {
    return 1;
  }
  value_str = strchr(out, ':');
  status->Counters.dslUASDown = strtoul(value_str+1, NULL, 0);

  return 0;

} /* cnxtDslGetStatusInfo */

/***************************************************************************
 * Function Name: cnxtDslModifyConfiguration
 * Description  : This function configures the dsl line as per the values 
 *                filled in the structure pAdslCfg to understand more about 
 *                valid values see the structure definitions & the valid 
 *                values  for each member. 
 * Returns      : CNXTDSL_STATUS 
 ***************************************************************************/

CNXTDSL_STATUS cnxtDslModifyConfiguration(pDslConfigInfo_t pAdslCfg)
{

  long    dslConfigInfo;
  int err=0;
  char out[512];
  char *value_str;
  int chsk = 0;

  dslConfigInfo = pAdslCfg->dslAnnexAParam;

  /* We get the configuration request in the structure pAdslCfg
   * now we should parse these structures & do the required 
   * action.
   * refer:  pAdslCfg->adslAnnexAParam = dslConfigInfo; in the
   * parser function.
   */

  cnxtGetDslAttribute("CommonHandshake", out);

  value_str = strchr(out, ':');
  if(!strncmp(value_str +2, "Disable", 7))
  {
    //printf("CommonHandshake is Disable\n");
    chsk = 1;
  }
  else
  {
    //printf("CommonHandshake is Enable\n");
  }

  /* check for operating mode options */

  /* for option 'a' */
  if( dslConfigInfo == 0) 
  {
    if( chsk == 1)
    {
      printf("Common handshake is disabled! dsl config support only in chsk enabled mode\n");
      err = -1;
      if (err == -1)
      {
        return STATUS_ERROR;
      }
    }
    else
    {
        err = cnxtSetDslAttribute("AnnexAStandard", "Multimode");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("AnnexACapability", "AnnexA/T1413");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("BisAStandard", "BisPlusAuto");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("BisACapability", "A2/A2+");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("BisMStandard", "BisPlusAuto");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("BisMCapability", "M2+/M2");
        if (err == -1)
          return STATUS_ERROR;

        err = cnxtSetDslAttribute("ActivateLine", "Abort");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("ActivateLine", "Start");
        if (err == -1)
          return STATUS_ERROR;
    }
  }
  else
  {

    /* G.DMT or ADSL */

    if (dslConfigInfo & kGdmtOnly )
    {
      if( chsk == 1)
      {
        printf("Common handshake is disabled! dsl config support only in chsk enabled mode\n");
        err = -1;
        if (err == -1)
        {
          return STATUS_ERROR;
        }
      }
      else
      {
        err = cnxtSetDslAttribute("AnnexAStandard", "G.Dmt");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("AnnexACapability", "AnnexA");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("BisACapability", "disable");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("BisMCapability", "disable");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("ActivateLine", "Abort");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("ActivateLine", "Start");
        if (err == -1)
          return STATUS_ERROR;
      }
    } 
    else if (dslConfigInfo & kGliteOnly)
    {
      /* NOT SUPPORTED */
      printf("g.lite mode is not supported\n");
      return STATUS_ERROR;
    } 
    else if (dslConfigInfo & kT1413Only)
    {
      if( chsk == 1)
      {
        printf("Common handshake is disabled! dsl config support only in chsk enabled mode\n");
        err = -1;
        if (err == -1)
        {
          return STATUS_ERROR;
        }
      }
      else
      {
        err = cnxtSetDslAttribute("AnnexAStandard", "T1.413");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("AnnexACapability", "T1413");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("BisACapability", "disable");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("BisMCapability", "disable");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("ActivateLine", "Abort");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("ActivateLine", "Start");
        if (err == -1)
          return STATUS_ERROR;
      }
    }
    else if (dslConfigInfo & kAdsl2Only)
    {

      if(chsk == 1)
      {
        printf("Common handshake is disabled! dsl config support only in chsk enabled mode\n");
        err = -1;
        if (err == -1)
        {
          return STATUS_ERROR;
        }
      }
      else
      {
        err = cnxtSetDslAttribute("BisAStandard", "G.Dmt.Bis");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("BisACapability", "A2");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("BisMCapability", "disable");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("ActivateLine", "Abort");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("ActivateLine", "Start");
        if (err == -1)
          return STATUS_ERROR;
      }
    }
    else if ((dslConfigInfo & kAdsl2plusOnly) && 
        (dslConfigInfo & kAdsl2Only))
    {

      if(chsk == 1)
      {
        printf("Common handshake is disabled! dsl config support only in chsk enabled mode\n");
        err = -1;
        if (err == -1)
        {
          return STATUS_ERROR;
        }
      }
      else
      {
        err = cnxtSetDslAttribute("BisAStandard", "G.Dmt.BisPlusOnly");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("BisACapability", "A2+");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("BisMCapability", "disable");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("ActivateLine", "Abort");
        if (err == -1)
          return STATUS_ERROR;
        err = cnxtSetDslAttribute("ActivateLine", "Start");
        if (err == -1)
          return STATUS_ERROR;
      }

    }

    /* for option 'm' */

    if (pAdslCfg->dsl2Param & kAdsl2AnnexMEnabled)
    {
      /* there are other modes for Annex M like M2plus auto M2plus only */
      cnxtSetDslAttribute("Standard", "AnnexM2");
      if (err == -1)
      {
        return STATUS_ERROR;
      }
    }
  }


#if 0
  /* Check for Line pair Options */
  /* PAIR SWITCHING has to be enabled : by default inner pair is selected */

  /* for option 'i' */
  if ( (dslConfigInfo & kAdslCfgLineInnerPair) == kAdslCfgLineInnerPair)
  {
    /* set the InnerPair */ 
    cnxtSetDslAttribute("PSMode", "Inner");
    if (err == -1)
    {
      return STATUS_ERROR;
    }
  }

  /* for option 'o' */

  if ((dslConfigInfo & kAdslCfgLineOuterPair) == kAdslCfgLineOuterPair)
  {
    /* set the OuterPair */ 
    cnxtSetDslAttribute("PSMode", "Outer");
    if (err == -1)
    {
      return STATUS_ERROR;
    }

  }
#endif

  if( (pAdslCfg->dslDemodCapMask & kAdsl2BitSwap) == kAdsl2BitSwap)
  {
    /* we need to set bitswap */
    if( (pAdslCfg->dslDemodCapValue & kAdsl2BitSwapEnable) == kAdsl2BitSwapEnable)
    {
      /* we need to switch on bitswap */
      if( chsk == 1)
      {
        cnxtSetDslAttribute("BitSwap", "Enable");
        if (err == -1)
        {
          return STATUS_ERROR;
        }
      }
      else
      {
        cnxtSetDslAttribute("AnnexABitSwap", "Enable");
        if (err == -1)
        {
          return STATUS_ERROR;
        }
      }
    }
    else
    {
      /* switch off bitswap */
      if( chsk == 1)
      {
        cnxtSetDslAttribute("BitSwap", "Disable");
        if (err == -1)
        {
          return STATUS_ERROR;
        }
      }
      else
      {
        cnxtSetDslAttribute("AnnexABitSwap", "Disable");
        if (err == -1)
        {
          return STATUS_ERROR;
        }
      }
    }
  }

  if( (pAdslCfg->dslDemodCapMask & kAdsl2AutoSRA) == kAdsl2AutoSRA)
  {
    /* we need to set SRA */
    /* following need to be defined GS_INCLUDE_AUTO_SRA */
    if( (pAdslCfg->dslDemodCapValue & kAdsl2AutoSRAEnable) == kAdsl2AutoSRAEnable)
    {
      /* we need to switch on SRA */
      cnxtSetDslAttribute("AutoSRA", "Enable");
      if (err == -1)
      {
        return STATUS_ERROR;
      }
    }
    else
    {
      /* switch off SRA */
      cnxtSetDslAttribute("AutoSRA", "Disable");
      if (err == -1)
      {
        return STATUS_ERROR;
      }
    }
  }

  return STATUS_SUCCESS;
}

/***************************************************************************
 * Function Name: cnxtDslGetVersionInfo
 * Description  : This function retrieves the dsp version, driver version, 
 *                firmware version & the code type status values from turbo 
 *                driver and updates the pTurboVersionInfo_t structure. 
 * Returns      : CNXTDSL_STATUS 
 ***************************************************************************/

CNXTDSL_STATUS cnxtDslGetVersionInfo(pTurboVersionInfo_t dslVersionInfo)
{
  int err = 0;
  char out[256];
  char *value_str;

  err = cnxtGetDslAttribute("DspVersion", dslVersionInfo->phyVerStr);
  if (err == -1)
  {
    return STATUS_ERROR;
  }

  err = cnxtGetDslAttribute("DriverVersion", dslVersionInfo->drvVerStr);
  if (err == -1)
  {
    return STATUS_ERROR;
  }

  err = cnxtGetDslAttribute("FirmwareVersion", dslVersionInfo->fwVersion);
  if (err == -1)
  {
    return STATUS_ERROR;
  }


  err = cnxtGetDslAttribute("CodeTypeStatus", out);
  if (err == -1)
  {
    return STATUS_ERROR;
  }

  value_str = strchr(out, ':');

  if( strncmp(value_str+2, "ADSL2", 5) == 0)
  {
    dslVersionInfo->phyType = kSADSL;
  }
  if( strncmp(value_str+2, "AnnexA", 6) == 0)
  {
    dslVersionInfo->phyType = kAnnexA;
  }
  if( strncmp(value_str+2, "AnnexB", 6) == 0)
  {
    dslVersionInfo->phyType = kAnnexB;
  }

  return(err);
}

/***************************************************************************
 * Function Name: cnxtGetPerformanceStats
 * Description  : This function updates the turboMib  structure with all the 
 *                available performance statistics. 
 * Returns      : CNXTDSL_STATUS 
 ***************************************************************************/

CNXTDSL_STATUS cnxtGetPerformanceStats(pTurboMibInfo_t    turboMib)
{
  char out[512];
  char *value_str;
  int err = -1;

  channelPerformaceData_t    *pChannelPerformanceData = &turboMib->FastChPerfData;

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucChanPerfCurr1DayTimeElapsed", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfCurr1DayTimeElapsed = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucChanPerfPrev1DayMoniSecs", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.aturPerfPrev1DayMoniSecs = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfLoss", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfTotal.Loss = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfLofs", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfTotal.Lofs = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfESs", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfTotal.ESs = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AturChanReceivedBlks", out);

  value_str = strchr(out, ':');
  pChannelPerformanceData->perfTotal.channelRxBlks = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AturChanUncorrectBlks", out);

  value_str = strchr(out, ':');
  pChannelPerformanceData->perfTotal.channelUncorrectBlks = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  /* Latest one day */

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfCurr1DayTimeElapsed", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfCurr1DayTimeElapsed = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfCurr1DayLoss", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfCurr1Day.Loss = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }


  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfCurr1DayLofs", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfCurr1Day.Lofs = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfCurr1DayESs", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfCurr1Day.ESs = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AturChanPerfCurr1DayReceivedBlks", out);

  value_str = strchr(out, ':');
  pChannelPerformanceData->perfCurr1Day.channelRxBlks = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AturChanPerfCurr1DayUncorrectBlks", out);

  value_str = strchr(out, ':');
  pChannelPerformanceData->perfCurr1Day.channelUncorrectBlks = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  /* Latest 15 minutes */

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfCurr15MinTimeElapsed", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfCurr15MinTimeElapsed = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfCurr15MinLoss", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfCurr15Min.Loss = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfCurr15MinLofs", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfCurr15Min.Lofs = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfCurr15MinESs", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfCurr15Min.ESs = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AturChanPerfCurr15MinReceivedBlks", out);

  value_str = strchr(out, ':');
  pChannelPerformanceData->perfCurr15Min.channelRxBlks = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AturChanPerfCurr15MinUncorrectBlks", out);

  value_str = strchr(out, ':');
  pChannelPerformanceData->perfCurr15Min.channelUncorrectBlks = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  /* Previous 1 day */

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfPrev1DayMoniSecs", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.aturPerfPrev1DayMoniSecs = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfPrev1DayLoss", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfPrev1Day.Loss = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfPrev1DayLofs", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfPrev1Day.Lofs = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AtucPerfPrev1DayESs", out);

  value_str = strchr(out, ':');
  turboMib->performaceData.perfPrev1Day.ESs = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AturChanPerfPrev1DayReceivedBlks", out);

  value_str = strchr(out, ':');
  pChannelPerformanceData->perfPrev1Day.channelRxBlks = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  memset(out,0,512);
  err = cnxtGetDslAttribute("AturChanPerfPrev1DayUncorrectBlks", out);

  value_str = strchr(out, ':');
  pChannelPerformanceData->perfPrev1Day.channelUncorrectBlks = strtoul(value_str+1, NULL, 0);
  if(err == -1)
  {
    return STATUS_ERROR;
  }

  return STATUS_SUCCESS;
}

/***************************************************************************
 * Function Name: 
 * Description  :
 * Returns      : 
 ***************************************************************************/

static int parse_attr_index(char *out, char *attribute)
{
  if(!strncmp(attribute,"IntlAttr",8))
  {
    if(strstr(out,"GdmtS_Up"))
    {
      /* indx is 1 */
      return(1);  
    }
    else if(strstr(out,"GdmtD_Up"))
    {
      /* indx is 2 */
      return(2);  
    } 
    else if(strstr(out,"GdmtR_Up"))
    {
      /* indx is 3 */
      return(3);  
    } 
    else if(strstr(out,"GdmtS_Down"))
    {
      /* indx is 4 */
      return(4);  
    } 
    else if(strstr(out,"GdmtD_Down"))
    {
      /* indx is 5 */
      return(5);  
    } 
    else if(strstr(out,"GdmtR_Down"))
    {
      /* indx is 6 */
      return(6);  
    } 
    else
    {
      /* indx is unkown */
      return(-1);  
    }

  }
}

/***************************************************************************
 * Function Name: 
 * Description  :
 * Returns      : 
 ***************************************************************************/
static int cnxt_get_dsl_config_attr(char *attribute, char* out, cnxtStats_t *status)
{
  FILE *fp;
  int ret = -1;
  int cmd = 0;
  int indx = 0;

  char proc_file[] = "/proc/quantum/drv_ctl";
  char port[] = "a1";

  char sys_cmd[1024];
  char *value_str;

  if(!strncmp(attribute,"IntlAttr",8))
  {
    cmd = 1;    
  }

  sprintf(sys_cmd,"echo \"Pda1:%s\" > %s",attribute,proc_file);
  if ( (ret=cnxtSystemCmd(sys_cmd))== -1) return -1;

  fp = fopen("/proc/quantum/drv_ctl","r");
  while(feof(fp) == 0)
  {
    memset(out,0,512);
    fgets(out, 512, fp);

    if(cmd == 1)
    {
      indx = parse_attr_index(out, attribute);
      //printf("%s, index = %d\n", __FUNCTION__, indx);
      switch(indx)
      {
        case 1:
          {
            value_str = strchr(out, ':');
            status->GdmtFraming.SUp = strtoul(value_str+1, NULL, 0);
            break;
          }

        case 2:
          {
            value_str = strchr(out, ':');
            status->GdmtFraming.DUp = strtoul(value_str+1, NULL, 0);
            break;
          }


        case 3:
          {
            value_str = strchr(out, ':');
            status->GdmtFraming.RUp = strtoul(value_str+1, NULL, 0);
            break;
          }

        case 4:
          {
            value_str = strchr(out, ':');
            status->GdmtFraming.SDown= strtoul(value_str+1, NULL, 0);
            break;
          }

        case 5:
          {
            value_str = strchr(out, ':');
            status->GdmtFraming.DDown = strtoul(value_str+1, NULL, 0);
            break;
          }

        case 6:
          {
            value_str = strchr(out, ':');
            status->GdmtFraming.RDown = strtoul(value_str+1, NULL, 0);
            break;
          }
        default:
          {
            //printf("%s: unknown option\n", out);
          }
      }
    }
  }

  fclose(fp);
  return ret;
}

/***************************************************************************
 * Function Name: 
 * Description  :
 * Returns      : 
 ***************************************************************************/
/* Keep this as static as of now, not exporting to outside world! */

static int config_pvc( int vpi, int vci, int qos, int mod )
{
  FILE *fp;
  int ret = 0;
  char mode[32];

  char proc_file[] = "/proc/quantum/drv_ctl";

  char sys_cmd[1024];

  switch(mod)
  {
    case CNXT_RFC_BRIDGED:
      sprintf(mode,"%s", "Llcbridged");
      break;

    case CNXT_RFC_ROUTED:
      sprintf(mode,"%s", "Llcrouted");
      break;
  }

  sprintf(sys_cmd,"echo \"e:wan0:/vpi=%d/vci=%d/rfc1483.mode=%s/port=a1\" > %s",vpi,vci,mode, proc_file);
  if ( (ret=cnxtSystemCmd(sys_cmd))== -1) return -1;

}


/***************************************************************************
 * Function Name: 
 * Description  :
 * Returns      : 
 ***************************************************************************/
static int cnxtSystemCmd(char *name)
{
  int err=0;

  if((err=system(name)) != 0 )
  {
    printf("command %s failed \n",name);
    return -1;
  }
  return 0;
}

/***************************************************************************
 * Function Name: 
 * Description  :
 * Returns      : 
 ***************************************************************************/
int cnxtGetDslAttribute(char *attribute, char* out)
{
  FILE *fp;
  int ret = 0;

  char proc_file[] = "/proc/quantum/drv_ctl";
  char port[] = "a1";

  char sys_cmd[1024];

  sprintf(sys_cmd,"echo \"Pda1:%s\" > %s",attribute,proc_file);
  if ( (ret=cnxtSystemCmd(sys_cmd))== -1) return -1;

  fp = fopen("/proc/quantum/drv_ctl","r");
  while(feof(fp) == 0)
  {
    fgets(out, 512, fp);
  }

  fclose(fp);
  return ret;
}

/***************************************************************************
 * Function Name: 
 * Description  :
 * Returns      : 
 ***************************************************************************/
int cnxtSetDslAttribute(char *attribute , char *value)
{
  FILE *fp;
  int ret = 0;
  char *complete_attr[512];

  char proc_file[] = "/proc/quantum/drv_ctl";

  char sys_cmd[512];

  sprintf(sys_cmd,"echo \"PSa1:%s:%s\" > %s",attribute,value,proc_file);
  if ( (ret=cnxtSystemCmd(sys_cmd))== -1) return -1;
  else
    return ret;
}


