
#ifndef __CNXT_COMMON_H
#define __CNXT_COMMON_H

#define CNXT_ATM_CONFIG 0x1

/* define ATM QOS parameters */
#define CNXT_ATMQOS_NONE 0x0
#define CNXT_ATMQOS_UBR 0x1
#define CNXT_ATMQOS_UBRPLUS 0x2
#define CNXT_ATMQOS_CBR 0x3
#define CNXT_ATMQOS_VBRRT 0x4
#define CNXT_ATMQOS_VBR 0x5

/* Flags for OAM mode */

#define CNXT_OAMCLI_CMD_ANY "any"
#define CNXT_OAMCLI_FLAGSTR_F4 "f4"
#define CNXT_OAMCLI_FLAGSTR_F5 "f5"
#define CNXT_OAMCLI_FLAGSTR_ETOE "etoe"
#define CNXT_OAMCLI_FLAGSTR_SEGMENT "segment"
#define MAX_ATTRIBUTE_LEN 32

typedef enum dslStatus
{
  STATUS_SUCCESS = 0,
  STATUS_ERROR
} CNXTDSL_STATUS;

typedef struct
{
  CNXTDSL_STATUS status;
} status_t, *pStatus_t;

typedef enum dslState
{

  IDLE=0,
  SHOWTIME,
  BITSWAPPING,
  HANDSHAKE,
  TRAINING,
  DISCOVERY,
  TRAINING_DELT,
  DELT,
  SPECTRUMTEST,
  RSMEMORYTEST
} DSL_LINK_STATE;

typedef struct dslConnectionInfo
{
  char LinkState [MAX_ATTRIBUTE_LEN ]; 
  unsigned int TxBitRate;
  unsigned int RxBitRate;
  unsigned int FastUSRate;
  unsigned int FastDSRate;
  unsigned int InterUSRate;
  unsigned int InterDSRate;
} connInfo_t, *pConnInfo_t;

/* structure used for OAM data & statistics */

typedef struct oma_report_s{
  int vpi; /* VPI no. */
  int vci; /* VCI no. */
  unsigned long numRepetitions;
  unsigned long timeout;
  unsigned long successCount;
  unsigned long failureCount;
  unsigned long avgRespTime;
  unsigned long minRespTime;
  unsigned long maxRespTime;
  char flags[50];
  char type[50];
}cnxtOamLoop_t;


typedef struct dslGenericAttributes_s {
  char  dslMode1 [MAX_ATTRIBUTE_LEN ];
  char  dslMode2 [MAX_ATTRIBUTE_LEN ];
  char  dslChannel [MAX_ATTRIBUTE_LEN ];
  char  dslLineStatus [MAX_ATTRIBUTE_LEN ];
  char  dslTrainingStatus [MAX_ATTRIBUTE_LEN ];
  char  dslSnrUp [MAX_ATTRIBUTE_LEN ]; 
  char  dslSnrDown [MAX_ATTRIBUTE_LEN ];
  char  dslAttnUp [MAX_ATTRIBUTE_LEN ]; 
  char  dslAttnDown [MAX_ATTRIBUTE_LEN ]; 
  char  dslPwrUp [MAX_ATTRIBUTE_LEN ];
  char  dslPwrDown [MAX_ATTRIBUTE_LEN ];
  unsigned long  dslMaxUp;
  unsigned long  dslMaxDown;
  unsigned long  dslRateUp; 
  unsigned long  dslRateDown; 
} GenericAttributes_t;


typedef struct dslGdmtFraming_s {
  unsigned long  KUp; 
  unsigned long  KDown; 
  unsigned long  RUp; 
  unsigned long  RDown; 
  unsigned long  SUp; 
  unsigned long  SDown; 
  unsigned long  DUp; 
  unsigned long  DDown; 
}GdmtFraming_t;

typedef struct dslAdsl2Framing_s {
  unsigned long  MSGcUp; 
  unsigned long  MSGcDown; 
  unsigned long  BUp; 
  unsigned long  BDown; 
  unsigned long  MUp; 
  unsigned long  MDown; 
  unsigned long  TUp; 
  unsigned long  TDown; 
  unsigned long  RUp; 
  unsigned long  RDown; 
  unsigned long  SUp; 
  unsigned long  SDown; 
  unsigned long  LUp; 
  unsigned long  LDown; 
  unsigned long  DUp; 
  unsigned long  DDown; 
}Adsl2Framing_t ;

typedef struct dslCounters_s {
  unsigned long dslSFUp;
  unsigned long dslSFDown;
  unsigned long dslSFErrUp;
  unsigned long dslSFErrDown;
  unsigned long dslRSUp;
  unsigned long dslRSDown;
  unsigned long dslRSCorrUp;
  unsigned long dslRSCorrDown;
  unsigned long dslRSUnCorrUp;
  unsigned long dslRSUnCorrDown;
  unsigned long dslHECUp;
  unsigned long dslHECDown;
  unsigned long dslOCDUp;
  unsigned long dslOCDDown;
  unsigned long dslLCDUp;
  unsigned long dslLCDDown;
  unsigned long dslTotalCellsUp;
  unsigned long dslTotalCellsDown;
  unsigned long dslDataCellsUp;
  unsigned long dslDataCellsDown;
  unsigned long dslDropCells;
  unsigned long dslBitErrorsUp;
  unsigned long dslBitErrorsDown;
  unsigned long dslESUp;
  unsigned long dslESDown;
  unsigned long dslSESUp;
  unsigned long dslSESDown;
  unsigned long dslUASUp;
  unsigned long dslUASDown;
  unsigned long dslMaxInterleaveDepth;
  unsigned long dslInterleaveDpDn;
  unsigned long dslInterleaveDpUp;
}DslCounters_t; 


typedef struct {
  GenericAttributes_t	GenericAttribute;
  GdmtFraming_t			GdmtFraming;
  Adsl2Framing_t		Adsl2Framing;
  DslCounters_t			Counters;
}cnxtStats_t, *pCnxtStats_t;


#define CNXT_RFC_BRIDGED 0
#define CNXT_RFC_ROUTED 1

#define BCMADSL_STATUS_SUCCESS 0
#define BCMADSL_STATUS_ERROR -1


typedef struct dslConfig {
    long        dslAnnexAParam;
    long        dslDemodCapMask;
    long        dslDemodCapValue;
    long        dsl2Param;
} dslConfigInfo_t, *pDslConfigInfo_t;


/* DSL configuration constanst which can be used for filling the above structure */

#define kAutoModeOnly             0x00000000
#define kGdmtOnly                 0x00000001
#define kGliteOnly                0x00000002
#define kT1413Only                0x00000004
#define kAdsl2Only                0x00001000
#define kAdsl2plusOnly            0x00002000
#define kAdsl2AnnexMEnabled       0x00000002
/* use following defines with dslDemodCapMask & dslDemodCapValue combination */
#define kAdsl2BitSwap			  0x00004000
#define kAdsl2BitSwapEnable		  0x00004000
#define kAdsl2AutoSRA             0x00800000
#define kAdsl2AutoSRAEnable 	  0x00800000



#define kStringMaxSize              32

typedef struct _turboVersionInfo {
    unsigned short  phyType;
    char            phyVerStr[kStringMaxSize];
    char            drvVerStr[kStringMaxSize];
    char            fwVersion[kStringMaxSize];
} turboVersionInfo_t, * pTurboVersionInfo_t ;

#define kUnknown                    0
#define kAnnexA                     1
#define kAnnexB                     2
#define kAnnexC                     3
#define kSADSL                      4

/* Performance statistics */

typedef struct channelCounters_s {
    unsigned long       channelRxBlks;
    unsigned long       channelTxBlks;
    unsigned long       channelCorrectedBlks;
    unsigned long       channelUncorrectBlks;
} channelCounters_t;

typedef struct performanceCounters_s{
    unsigned long       Lofs;
    unsigned long       Loss;
    unsigned long       Lols;   /* Loss of Link failures (ATUC only) */
    unsigned long       Lprs;
    unsigned long       ESs;    /* Count of Errored Seconds */
    unsigned long       Inits;  /* Count of Line initialization attempts (ATUC only) */
    unsigned long       UAS;    /* Count of Unavailable Seconds */
    unsigned long       SES;    /* Count of Severely Errored Seconds */
    unsigned long       LOSS;   /* Count of LOS seconds */
    unsigned long       FECs;   /* Count of FEC seconds  */
    unsigned long       LCDS;   /* Count of LCD Errored Seconds */
} performanceCounters_t;

typedef struct failureCounters_s {
    unsigned long       Retr;       /* Count of total retrains */
    unsigned long       RetrLof;    /* Count of retrains due to LOF */
    unsigned long       RetrLos;    /* Count of retrains due to LOS */
    unsigned long       RetrLpr;    /* Count of retrains due to LPR */
    unsigned long       RetrLom;    /* Count of retrains due to LOM */
    unsigned long       InitErr;    /* Count of training failures */
    unsigned long       InitTo;     /* Count of training timeouts */
    unsigned long       LineSearch; /* Count of line search inits */
} failureCounters_t;


typedef struct performanceData_s{
    performanceCounters_t	perfTotal;
    performanceCounters_t	perfSinceShowTime;
    unsigned long       	perfValidIntervals;
    unsigned long       	perfInvalidIntervals;
    performanceCounters_t	perfCurr15Min;
    unsigned long       	perfCurr15MinTimeElapsed;
    performanceCounters_t	perfCurr1Day;
    unsigned long       	perfCurr1DayTimeElapsed;
    performanceCounters_t	perfPrev1Day;
    unsigned long       	aturPerfPrev1DayMoniSecs;
    failureCounters_t		failTotal;
    failureCounters_t		failSinceShowTime;
} performanceData_t;

typedef struct channelPerformaceData_s {
    channelCounters_t	perfTotal;
    unsigned long       chanPerfValidIntervals;
    unsigned long       chanPerfInvalidIntervals;
    channelCounters_t	perfCurr15Min;
    unsigned long       perfCurr15MinTimeElapsed;
    channelCounters_t	perfCurr1Day;
    unsigned long       perfCurr1DayTimeElapsed;
    channelCounters_t	perfPrev1Day;
    unsigned long       aturPerfPrev1DayMoniSecs;
} channelPerformaceData_t ;


typedef struct turboMibInfo {
    performanceData_t		performaceData;
    channelPerformaceData_t InterleavedChPerfData;
    channelPerformaceData_t FastChPerfData;
} turboMibInfo_t, *pTurboMibInfo_t;

/*****************************************
 *  Function prototypes for Applications *
 *****************************************/

/* This function activates the DSL line. [At bootup dsl state is active 
 * by default] 
 */
CNXTDSL_STATUS cnxtDslActivateLine( void );

/* This function de-activates the DSL line. 
 */
CNXTDSL_STATUS cnxtDslDeActivateLine( void );

/* This function updates the user provided structure [pConnInfo] 
 */
CNXTDSL_STATUS cnxtDslGetConnectionInfo( pConnInfo_t pConnInfo );

/* This function updates the structure cnxtStats_t with the latest connection 
 * information.
 */
CNXTDSL_STATUS cnxtDslGetStatusInfo( cnxtStats_t *status);

/* This function configures the dsl line as per the values filled in the 
 * structure pAdslCfg to understand more about valid values see the structure
 * definitions & the valid values for each member.
 */
CNXTDSL_STATUS cnxtDslModifyConfiguration(pDslConfigInfo_t pAdslCfg);

/* This function retrieves the dsp version, driver version, firmware version 
 * & the code type status values from turbo driver and updates the 
 * pTurboVersionInfo_t structure. 
 */
CNXTDSL_STATUS cnxtDslGetVersionInfo(pTurboVersionInfo_t dslVersionInfo);

/* This function updates the turboMib  structure with all the available 
 * performance statistics. 
 */
CNXTDSL_STATUS cnxtGetPerformanceStats(pTurboMibInfo_t    turboMib);

#endif
