
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <mtd/mtd-user.h>

#define DBG(x...) 
//#define DBG(x...) printf(x)


int main()
{
	int fd0, fd1, fd2, fd3, fd4, len ;
	int argc = 2 ;
	char *argv[2] = {0, 0} ;
	unsigned int buf[4], offset = 0 ;
	unsigned char data[4096] ;

	if((fd1 = open("/dev/mtd1", O_RDWR)) < 0) {
		DBG("Exit : Failed to open /dev/mtd1\n");
		exit(1) ;
	}
	lseek(fd1, 1024, SEEK_SET) ;
	read(fd1, (char *) buf, 2 * sizeof(buf[0])) ;
	lseek(fd1, 2044, SEEK_SET) ;
	read(fd1, (char *) &buf[2], sizeof(buf[0])) ;
	if((buf[0] == 0xdeadbeef) && (buf[2] == 0xbeefdead)){
		lseek(fd1, 0, SEEK_SET) ;
		if(buf[1] == 0xAA55AA55) {

			DBG("Found :  0xAA55AA55 \n");
			printf("Starting Secondary Image Upgrade\n");
			buf[1] = 0xA050A050 ;
			/* copy primary to secondary */

			if(((fd2 = open("/dev/mtd2", O_RDWR)) < 0)
				|| ((fd3 = open("/dev/mtd3", O_RDWR)) < 0)
				|| ((fd4 = open("/dev/mtd4", O_RDWR)) < 0)) {
				printf("Exit : Failed to open mtd2/mtd3/mtd4\n");
				close(fd1) ;
				exit(1) ;
			}

			/* erase mtd3 */
			argv[1] = "/dev/mtd3" ;
			if(flash_erase_main(argc, argv)) {
				printf("Erasing %s failed, bailing out\n", argv[1]) ;
				exit(1) ;
			}

			/* Copy mtd1 to mtd3, Kernel */
			while(len = read(fd1, data, sizeof(data))) {
				write(fd3, data, len) ;
			}
			printf("Copy: MTD1 --> MTD3 Complete\n");

			/* erase mtd4 */
			argv[1] = "/dev/mtd4" ;
			if(flash_erase_main(argc, argv)) {
				printf("Erasing %s failed, bailing out\n", argv[1]) ;
				exit(1) ;
			}

			/* Copy mtd2 to mtd4, Rootfs */
			while(len = read(fd2, data, sizeof(data))) {
				write(fd4, data, len) ;
			}
			printf("Copy : MTD2 --> MTD4 Completed\n");
			lseek(fd1, 1028, SEEK_SET) ;
			write(fd1, (char *) &buf[1], sizeof(buf[0])) ;

			if((fd0 = open("/dev/mtd0", O_RDWR)) < 0) {
				printf("Exit : Open Fail mtd0\n");
				exit(1) ;
			}

			offset = lseek(fd0, -1, SEEK_END) ;
			offset += lseek(fd1, -1, SEEK_END) + 1 ;
			offset += lseek(fd2, -1, SEEK_END) + 1 ;
			offset += lseek(fd3, -1, SEEK_END) + 1 ;

			/* 
			 * offset is now the absolute sect end in
			 * bytes for mtd3 from top of flash
			 */

			/* close the handles that are not required anymore */
			close(fd1) ;
			close(fd2) ;
			close(fd3) ;
			close(fd4) ;

			lseek(fd0, 0, SEEK_SET) ;
			do {
				len = read(fd0, (char *) &buf, sizeof(buf[0])) ;
				if((buf[0] & 0xFFFF) == 0xdeaf) {
					buf[1] = buf[2] = 0 ;
					read(fd0, (char *) &buf, 3 * sizeof(buf[0])) ;
					if((buf[2] & 0xFFFF) != 0xfade) {
						close(fd0) ;
						exit(0) ;
					}

					/*
					 * Go back to where the sect info
					 * of the standby image is kept
					 */
					lseek(fd0, -2 * sizeof(int), SEEK_CUR) ;
					write(fd0, (char *) &offset, sizeof(int)) ;
					DBG("Offst : Secondary Image : 0x%x\n",offset);
					close(fd0) ;
					break ;
				}
			} while(len) ;
			close(fd0) ;
		}
	}
	else  {
		DBG("Exit : Failed to find 0xdeadbeef and  0xbeefdead\n");
	}
	exit(0) ;
}
