/* File to read and update configuration informtion from boot partition */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "flash_config.h"
#define OFFSET_TO_ADDR(x) ( x * sizeof(unsigned int))

#ifdef solosW

const unsigned short magics[][2] = {
//		{hdr, ftr}
		{0xdead, 0xbeef},	// PDA magic info from pda_support.h
		{0xcafe, 0xface},	// Lxcmdline magic
		{0xdeaf, 0xfade},
		{0, 0}
} ;

int configInfoFind(int fd, unsigned int *offset) ;

static int verify_hdr(magic)
{
int i = 0 ;	

	for(; magics[i][0] ; i++)
		if(magics[i][0] == magic) return i ;
	return -1 ;
}

extern int config_FlashOffset (unsigned int *, int) ;
extern int config_read(char*, char**, unsigned int*) ;
extern int getNVS(int fd, unsigned int *nvsStart, unsigned int *nvsSize) ;
extern hex_dump(char*, unsigned int) ;
#else
static unsigned int validateCmdlineData(int fd,unsigned int configInfoBase, unsigned int nvsSize) ;
static unsigned int cmdlineInfoFlashGet(int fd,unsigned int nvsSize, unsigned char **string,
						unsigned int *length, unsigned int boot_src,
						unsigned int *Config_offset_Value) ;
#endif


/*
 *	dev_name can be character device name /dev/mtd0 
 *
 */

int flash_config_main(char *device_name, char *outputFileName)
{
char *buf = NULL ;
int fd, fboot;
unsigned int len = 0 ;
unsigned int buf_size = 0 ;
unsigned int nvsSize = 0 ;
int err = 0;
int fileWritePermission = 0;

#ifdef solosW
unsigned int configOffsetStart = 0, configInfoSize = 0 ;
#else
unsigned char *string = NULL ;
unsigned int CmdLineOffset = 0 ;
unsigned int CmdLen = 0 ;
unsigned int boot_src = 1 ;
unsigned int Config_offset_Value = 0 ;
NVSHEADER nvs ;
#endif

        if (outputFileName == NULL) {
            outputFileName = "boot.bin";
        }
        else fileWritePermission = O_CREAT;

	if((fd = open(device_name, O_RDONLY)) < 0) {
		perror("open()") ;
		return(1) ;
	}

#ifdef solosW
			if(getNVS(fd, &configOffsetStart, &nvsSize) == ESUCCESS) {
				DbgPrint(("configOffsetStart %d nvsSize %d\n", configOffsetStart, nvsSize)) ;
				if(configInfoFind(fd, &configInfoSize) != ESUCCESS) {
					//close(fd) ;
					DbgPrint(("No valid configuration found\n")) ;
					//return(1) ;
				}
				buf_size = sizeof(unsigned int) + nvsSize + configInfoSize ;
				DbgPrint(("Allocation size is %d\n", buf_size)) ;
				buf = (char *) calloc(buf_size, sizeof(unsigned char)) ;
				if(!buf) {
					close(fd) ;
					DbgPrint(("No memory\n")) ;
					return(1) ;
				}
				configInfoSize = lseek(fd, configOffsetStart, SEEK_SET) ;
				if((read(fd, buf + sizeof(unsigned int), buf_size)) != buf_size) {
					close(fd) ;
					DbgPrint(("read faliure\n")) ;
					return(1) ;
				}
			} else {
				close(fd) ;
				DbgPrint(("No NVS found in %s\n", device_name)) ;
				device_name = "/dev/mtd1" ;
				DbgPrint(("Trying in %s\n", device_name)) ;
				if(config_read(device_name, &buf, &buf_size)
					!= ESUCCESS) {
					DbgPrint(("No NVS found in %s as well\n", device_name)) ;	
					return(1) ;
				}
			}
            if( (fboot=open(outputFileName,O_WRONLY|O_SYNC|fileWritePermission)) ) 
            {
            	int i=0;
            	
                            if (!fileWritePermission) {
                                configOffsetStart = lseek(fboot,0,SEEK_END) + sizeof(unsigned int);
                            }
		  		*(unsigned int *) buf = configOffsetStart / sizeof(unsigned int) ;
				DbgPrint(("configoffset %d buf %d\n", configOffsetStart, *(int*)buf)) ;
				buf_size = write(fboot,buf,buf_size);
      			close(fboot);
				close(fd) ;
				free(buf) ;
				return(0) ;
            }
            else {
				perror("boot.bin does not exist ");
			}
#else
			if (config_FlashRead_NVSHEADER(fd, &nvs, sizeof(NVSHEADER)) == ESUCCESS) {
				DbgPrint(("\n NVSHEADER READ SUCCESS")) ;
				nvsSize = NVS_SIZE(&nvs) ;
				if(cmdlineInfoFlashGet(fd, nvsSize, &string,
					&CmdLen, 0, &Config_offset_Value) == ESUCCESS) {
					if(cmdlineInfoFlashGet(fd, nvsSize, &string,
						&CmdLen, 1, &Config_offset_Value) == ESUCCESS) {
						buf_size = 4 + nvsSize +
									sizeof(lx_cmdline_header) +
									sizeof (lx_cmdline_data) +
									sizeof(lx_cmdline_footer) ;
						DbgPrint(("\n Allocation size is %d", buf_size)) ;
						buf = (unsigned char *) calloc(buf_size, sizeof(unsigned char)) ;
						lseek(fd, (Config_offset_Value - 1) * sizeof(unsigned int), SEEK_SET) ;
						len = read(fd, buf, buf_size) ;
                                                if( (fboot=fopen(outputFileName,"r")) )
                                                {
                                                       lseek(fboot,0,SEEK_END);
                                                       fwrite(buf,(size_t)1, buf_size,fboot);
                                                       fclose(fboot);
i                                                      return 0;
                                                      
                                                 }
                                                 else
                                                      perror("boot.bin does not exist ");

					} 
				} 
			} 
#endif
 return 1;
}


#ifdef solosW

#define BUF_SZ 2048

int configInfoFind(int fd, unsigned int *offset)
{
char buf[BUF_SZ] ;
int rc = ENOENT, idx ;
unsigned int len, offsetInBuf, numReads ;

	*offset = numReads = offsetInBuf = 0 ;
	do {
		if(!offsetInBuf) {
			if(!(len = read(fd, buf, sizeof(buf))))
				goto end ;
			if(len < sizeof(buf))
				memset(buf + len, 0xff, sizeof(buf) - len) ;
			numReads++ ;
    		DbgPrint(("numReads %d\n", numReads)) ;
		}
		idx = verify_hdr(*(unsigned short *) (buf + offsetInBuf)) ;
		if(idx >= 0) {
			/*
			 *	Valid header found go for the footer
			 */
			DbgPrint(("Header 0x%x @ offset %d after %d reads\n",
						*(unsigned short *) (buf + offsetInBuf),
						offsetInBuf, numReads)) ;
			offsetInBuf = (offsetInBuf += sizeof(unsigned short)) % BUF_SZ ;
			do {
				if(!offsetInBuf) {
					if(!(len = read(fd, buf, sizeof(buf))))
						goto end ;
					if(len < sizeof(buf))
						memset(buf + len, 0xff, sizeof(buf) - len) ;
					numReads++ ;
    				DbgPrint(("numReads %d\n", numReads)) ;
				}
				if(good_ftr(*(unsigned short *) (buf + offsetInBuf), idx)) {
					DbgPrint(("Footer 0x%x @ offset %d after %d reads\n",
								*(unsigned short *) (buf + offsetInBuf),
								offsetInBuf, numReads)) ;
					*offset = (numReads - 1) * BUF_SZ + offsetInBuf + sizeof(unsigned int) ;
					offsetInBuf = (offsetInBuf += sizeof(unsigned short)) % BUF_SZ ;
					DbgPrint(("config sz at this point %d and offset is %d\n",
								*offset, offsetInBuf)) ;
					rc = ESUCCESS ;
					break ;
				}
				offsetInBuf = (offsetInBuf += sizeof(unsigned short)) % BUF_SZ ;
			} while(1) ;
		} else {
			offsetInBuf = (offsetInBuf += sizeof(unsigned short)) % BUF_SZ ;
		}
	} while(1) ;
end:
    return rc ;
}

#else

int nvs_Validate(NVSHEADER *pHeader)
{
    int size ;
    int crc ;


    /* Read and validate the device size.
     */
    size = NVS_SIZE(pHeader) ;
    if (! NVS_VALID_SIZE(size))  return -1 ;

    /* Calculate and verify the CRC.
     */
    crc = crcnvs(pHeader, size) ;

    if (((crc >> 0) & 0xff) != pHeader->crcLo  ||
        ((crc >> 8) & 0xff) != pHeader->crcHi)
    {
        return -1 ;              /* CRC is invalid */
    }
    else
 {
        return 0 ;               /* CRC and header look good */
    }
}


int config_FlashRead_NVSHEADER(int fd, NVSHEADER *nvsHeader, unsigned int size)
{
int rc = EIO ;
unsigned int offset = 0 ;
int len = 0 ;
NVSHEADER *nvs ;

	if (ESUCCESS == config_FlashOffset(&offset, fd)) {
		unsigned int	nvsSize ;

		DbgPrint(("\n ### 2 Phase config_FlashOffset Success ###")) ;  

		/*
		 *	Read basic header to determine the amount of configuration information.
		 */

#ifdef __LINUX__
	
		lseek(fd, (offset * sizeof (unsigned int)), SEEK_SET) ;
		len = read(fd, nvsHeader, sizeof(NVSHEADER)) ;	
#else
		config_FlashReadWords (fd,offset * sizeof (unsigned int), (unsigned int *)nvsHeader, (sizeof (NVSHEADER) + sizeof (unsigned int) - 1) / sizeof (unsigned int)) ;
#endif
		nvsSize = NVS_SIZE(nvsHeader) ;
		DbgPrint(("\n OFFSET IN WORD IS %d nvsSize  %d NVSHEADER SIZE IS %d",
			offset,nvsSize,sizeof(NVSHEADER))) ;
		return ESUCCESS ;

		/*
		 *	Check that the size is reasonable and that the data will
		 *	fit into the space supplied.
		 */

		if(NVS_VALID_SIZE(nvsSize)) {
			if (nvsSize <= size) {
				/*
				 *	Read and validate information.
				 */

				DbgPrint(("\n NVS size valid %d", nvsSize)) ;
#ifdef __LINUX__
				lseek(fd, (offset * sizeof (unsigned int)), SEEK_SET) ;
				len = read(fd, nvs, nvsSize) ;	
				DbgPrint(("\n nvs is %x and len is %d",nvs,len)) ; 
#else 
        config_FlashReadWords (fd,offset * sizeof (unsigned int), (unsigned int *) nvs, (nvsSize + sizeof (unsigned int) - 1) / sizeof (unsigned int)) ;
#endif
				if (0 == nvs_Validate(nvs)) {
					rc = ESUCCESS ;
				}
            } else {
				/*
				 *	Too much configuration information for
				 *	the available space.
				 */
				rc = EFBIG ;
			}
		}
	}
	return  rc ;
}


//################################

//Validates Command Line Data and return offset of Commandline data
static unsigned int validateCmdlineData(int fd,unsigned int configInfoBase, unsigned int nvsSize)
{
  unsigned int cmdlineInfoOffset_Addr = 0 ;
  unsigned short header = 0 ;
  unsigned short footer = 0 ;
  int len = 0 ;
int ret = 0 ;
//configInfoBase is in Words Convert to bytes
DbgPrint(("\n configInfoBase offset is %d nvsSize %d",configInfoBase,nvsSize)) ;  
cmdlineInfoOffset_Addr =(sizeof(unsigned int)* configInfoBase) + nvsSize ;
#ifdef __LINUX__
    	lseek(fd,cmdlineInfoOffset_Addr,SEEK_SET) ;
	len = read(fd,&header,sizeof(unsigned short)) ;
	DbgPrint(("\n cmdlineOffset_Addr is %x header is %x",cmdlineInfoOffset_Addr,header)) ;	
#else
  flash_read_data(cmdlineInfoOffset, &header, sizeof(unsigned short)) ;
#endif

#ifdef __LINUX__
        DbgPrint(("\n cmdlineOffset_Addr is %x footer is %x",CMDLINE_FOOTER_OFFSET(cmdlineInfoOffset_Addr),footer)) ;
	lseek(fd,CMDLINE_FOOTER_OFFSET(cmdlineInfoOffset_Addr),SEEK_SET) ;
        len = read(fd,&footer,sizeof(unsigned short)) ;
        DbgPrint(("\n cmdlineOffset_Addr is %x footer is %x",CMDLINE_FOOTER_OFFSET(cmdlineInfoOffset_Addr),footer)) ;
	
#else
flash_read_data(CMDLINE_FOOTER_OFFSET(cmdlineInfoOffset),
          &footer,
          sizeof(unsigned short)) ;
#endif
  if((header == HEADER_MAGIC) && (footer == FOOTER_MAGIC))
#ifdef __LINUX__

	return (configInfoBase) ;
#else
    return cmdlineInfoOffset ;
#endif
  else
    return 0 ;

}


static unsigned int cmdlineInfoFlashGet(int fd,unsigned int nvsSize,
                    unsigned char **string,
                    unsigned int *length,
                    unsigned int boot_src,unsigned int * Config_offset_Value)
{
  unsigned int configInfoBase = 0 ;
  unsigned int rc = EINVAL ;

  rc = config_FlashOffset (&configInfoBase,fd) ;
  if (ESUCCESS == rc)
    {
      unsigned int cmdlineInfoOffset = 0 ;
	unsigned int len = 0 ;
#ifndef __LINUX__
      configInfoBase *= sizeof(unsigned int) ;
#endif
      cmdlineInfoOffset = validateCmdlineData(fd,configInfoBase, nvsSize) ;
      DbgPrint(("\n INside  cmdlineInfoFlashGet and cmdlineInfoOffset is %d",cmdlineInfoOffset)) ;
      if(cmdlineInfoOffset)
        {
		*Config_offset_Value = cmdlineInfoOffset ;	
      		*string = (unsigned char *) calloc(COMMAND_LINE_SIZE, sizeof(unsigned char)) ;
#ifdef __LINUX__
	lseek(fd,CMDLINE_BOOTSRC_OFFSET(cmdlineInfoOffset * sizeof(unsigned int), boot_src)+ nvsSize,SEEK_SET) ;
        len = read(fd,*string,COMMAND_LINE_SIZE) ;
		
#else
      flash_read_data(CMDLINE_BOOTSRC_OFFSET(cmdlineInfoOffset, boot_src),
              *string, COMMAND_LINE_SIZE) ;
#endif
      *length = strlen(*string) + 1 ;
	DbgPrint(("\nCMDLINE_BOOTSRC_OFFSET is %x Length of String is %d && String is %s",CMDLINE_BOOTSRC_OFFSET(cmdlineInfoOffset * sizeof(unsigned int), boot_src)+ nvsSize,*length,*string)) ;
	if ( *length == 1)
        	return -1 ;	 
    } else {
          rc = ENOENT ;
        }
    } 

  return rc ;
}

#endif


