
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <asm/ioctl.h>
#include <mtd/mtd-user.h>

#include "flash_config.h"
#include "writemac.h"
#define DEBUG
#ifdef DEBUG
  #define VERBOSE(x) printf x
#else
  #define VERBOSE(x)
#endif

static int parse_mac(char* mac_in, char* mac_out) ;

#define LINEFEED 16

void hex_dump(char *buf, unsigned int size)
{
	int i = 0 ;

	VERBOSE(("\n")) ;
	for(i = 0 ; i < size ; i++) {
		if(!(i % LINEFEED)) {
			VERBOSE(("\n0x%08x", i)) ;
		}
		VERBOSE((" %02x", buf[i])) ;
	}
	VERBOSE(("\n")) ;
}


/*
 *	Config offset value is offset value stored but if boot.bin exceeds 64k then problem
 */


/*
 *	This will return the offset in WORDS from where Config Section Starts
 */

static unsigned int config_FlashOffsetLocation(int fd)
{
int len = 0 ;
int tmp = 0 ;
unsigned int off ;
unsigned int offset = 0 ;
unsigned int scratch = 0 ;
unsigned int fs_imageSize = 0 ;
unsigned int pp_imageSize = 0 ;

	/*
	 *	Read size of NP image, skip past image, checksum.
	 *	Trap wild values for data read (possibly uninitialised flash).
	 */

	off = 0 ;
	lseek(fd, (off * sizeof(unsigned int)), SEEK_SET) ;
	len = read(fd, &fs_imageSize, sizeof(unsigned int)) ;
	DbgPrint(("\n fs imageSize is %d and no of bytes read is %d\
				fs ImageSize in hex %x OFFSET IN WORDS %d",
				fs_imageSize, len, fs_imageSize, off)) ;

	if(fs_imageSize < MAX_FLASH_WORDS) {
		off += (fs_imageSize + 2) ;
		tmp = off * 4 ;	
		len = 0 ;
		lseek(fd, tmp, SEEK_SET) ;
		len = read(fd, &scratch, sizeof(unsigned int)) ;
	    DbgPrint(("\n scratch is %d and no of bytes read is %d\
					tmp is %x scratch is %x OFFSET IN WORDS %d",
					scratch, len, tmp, scratch, off)) ;
		if(scratch < MAX_FLASH_WORDS) {
			off += (scratch + 1) ;

			/*
			 *	Read size of PP image, skip past image, checksum.
			 */
			tmp = off * 4 ;
			len = 0 ;
			lseek(fd, tmp, SEEK_SET) ;
			len = read(fd, &pp_imageSize, sizeof(unsigned int)) ;
			DbgPrint(("\n PP ImageSize is %d and no of bytes read is %d\
						tmpo is %x imageSize is %x OFFSET IN WORDS %d",
						pp_imageSize, len, tmp, pp_imageSize, off)) ;
			if(pp_imageSize < (MAX_FLASH_WORDS - fs_imageSize)) {
				off += (pp_imageSize + 2) ;
				tmp = off * 4 ;
				scratch = 0 ;
				len = 0 ;
				lseek(fd, tmp, SEEK_SET) ;
				len = read(fd, &scratch, sizeof(unsigned int)) ;
				DbgPrint(("\n PP Scratch is %d and no of bytes read is %d\
							addr is %x scratch is %x OFFSET IN WORDS is %d",
							scratch, len, tmp, scratch, off)) ;
				if (scratch < MAX_FLASH_WORDS) {
					off += (scratch + 1) ;
					if (off < MAX_FLASH_WORDS) {
						offset = off ;
					}
				}
			}
		}
	}
	return  offset ;
}


int config_FlashOffset (unsigned int *offset, int fd)
{
int rc = EIO ;
unsigned int scratch = 0 ;
unsigned int off = config_FlashOffsetLocation(fd) ;
int len = 0 ;

	DbgPrint(("\n Finally Offset Returned is %d Address is %x", off, off * sizeof(unsigned int))) ;	
    if(0 != off) {
		/*
		 *	off now points to the config offset so read that word which
		 *	*MAY* give the location of the configuration information.
		 */
		lseek(fd, (off * sizeof (unsigned int)), SEEK_SET) ;
		len = read(fd, &scratch, sizeof(unsigned int)) ;
		DbgPrint(("\n scratch ** is %d and no of bytes read is %d", scratch, len)) ; 
		/*
		 *	Check that the configuration information is within reasonable bounds
		 *	located within the first flash chip.
		 */
		if((scratch > off) && (scratch < MAX_FLASH_WORDS)) {
			*offset = scratch ;
			rc = ESUCCESS ;
		}
	}
	return  rc ;
}


int getNVS(int fd, unsigned int *nvsStart, unsigned int *nvsSize)
{
int rc = EIO ;
unsigned int offset = 0 ;
int len = 0 ;
NVSHEADER nvs ;

	if (ESUCCESS == config_FlashOffset(&offset, fd)) {
		DbgPrint(("NVS found @ word offset %d\n", offset)) ;
		*nvsStart = offset * sizeof (unsigned int) ;
		lseek(fd, *nvsStart, SEEK_SET) ;
		len = read(fd, &nvs, sizeof(NVSHEADER)) ;
		*nvsSize = NVS_SIZE(&nvs) ;
		DbgPrint(("nvsSize %d NVSHEADER sz %d\n", *nvsSize, sizeof(NVSHEADER))) ;
		if(NVS_VALID_SIZE(*nvsSize)) {
			lseek(fd, *nvsStart + *nvsSize, SEEK_SET) ;
			rc = ESUCCESS ;
		}
	}
	return rc ;
}
int readmac(char* mac_out)
{
    int rc = 1;
    NVSHEADER nvs;

    if(!mac_out) {
        printf("Null input\n") ;
        return 1 ;
    }
    if (readNVSHeader(&nvs) == 0) {
        mac_out[0] = nvs.mac[0] ; mac_out[1] = nvs.mac[1] ;
        mac_out[2] = nvs.mac[2] ; mac_out[3] = nvs.mac[3] ;
        mac_out[4] = nvs.mac[4] ; mac_out[5] = nvs.mac[5] ;
        rc = 0 ;
    }

    return rc ;
}

U16 nvs_GetFlags(NVSHEADER *pHeader)
{
    U16 configFlags = pHeader->flags0 | (pHeader->flags1 << 8);

    return configFlags;
}

void nvs_SetFlags(NVSHEADER *pHeader, U16 configFlags)
{
    pHeader->flags0 = configFlags;
    pHeader->flags1 = configFlags >> 8;
}


int readNVSHeader(NVSHEADER *nvsHeader)
{
    int rc = 1 ;
    unsigned int offset = 0 ;
    int len = 0, fd;

    if((fd = open("/dev/mtd0", O_RDONLY)) < 0) {
        printf("open failed /dev/mtd0\n") ;
        return 1 ;
    }
    if (ESUCCESS == config_FlashOffset(&offset, fd)) {
        VERBOSE(("NVS found @ offset %d\n", OFFSET_TO_ADDR(offset))) ;
        lseek(fd, OFFSET_TO_ADDR(offset), SEEK_SET) ;
        len = read(fd, nvsHeader, sizeof(NVSHEADER)) ;
        if(NVS_VALID_SIZE(NVS_SIZE(nvsHeader))) {
            rc = 0 ;
        }
    }
    close(fd) ;

    return rc ;
}
int writeNVSHeader(NVSHEADER *nvsHeader)
{
NVSHEADER *nvs ;
mtd_info_t meminfo;
erase_info_t erase;
int i = 0, fd, rc = 1, verbose = 0 ;
unsigned int offset = 0, flsstart = 0, crc = 0 ;
char *buf = (char*) NULL, *device = "/dev/mtd0" ;


	if((fd = open(device, O_RDWR)) < 0) {
		printf("open failed %s\n", device) ;
		goto exit_1 ;
	}
	if(ioctl(fd, MEMGETINFO, &meminfo) != 0) {
		printf("ioctl failed to get device info\n") ;
		goto exit_2 ;
	}

	VERBOSE(("flash partition size %d\nflash section size %d\n",
		meminfo.size, meminfo.erasesize)) ;

	if(!(buf = (char*) malloc(meminfo.erasesize))) {
		VERBOSE(("No memory to read from %s\n", device)) ;
		goto exit_2 ;
	}
    if (ESUCCESS == config_FlashOffset(&offset, fd)) {

		flsstart = ((offset *= 4) / meminfo.erasesize) * meminfo.erasesize ;

		VERBOSE(("NVS found @ offset %d\n", offset)) ;

        if(lseek(fd, flsstart, SEEK_SET) != flsstart) {
			VERBOSE(("lseek failed to set @ %d for reading\n", flsstart)) ;
			goto exit_3 ;
		}

		if(read(fd, buf, meminfo.erasesize) != meminfo.erasesize) {
			VERBOSE(("Read failed from %s\n", device)) ;
			goto exit_3 ;
		}
		offset -= flsstart ;
		nvs = (NVSHEADER*)(buf + offset) ;
        if(NVS_VALID_SIZE(NVS_SIZE(nvs))) {
			VERBOSE(("Found Valid NVS Header\n")) ;
		} else {
			VERBOSE(("NVS size is not OK\n")) ;
			goto exit_3 ;
		}
                // Update NVS HEADER
                memcpy(nvs, nvsHeader, sizeof(NVSHEADER));
		crc = crcnvs(nvs, NVS_SIZE(nvs)) ;
		nvs->crcLo = crc & 0xFF ;
		nvs->crcHi = (crc & 0xff00) >> 8 ;
		
		erase.start = flsstart ;
		erase.length = meminfo.erasesize ;

		if(ioctl(fd, MEMERASE, &erase) != 0) {
			VERBOSE(("Failed erasing sector %d\n", flsstart / meminfo.erasesize)) ;
			goto exit_3 ;
		} else {
			VERBOSE(("Erased sector %d\n", flsstart / meminfo.erasesize)) ;
		}

		if(lseek(fd, flsstart, SEEK_SET) != flsstart) {
			VERBOSE(("lseek failed to set @ %d for writing\n", flsstart)) ;
			goto exit_3 ;
		}

		if(write(fd, buf, meminfo.erasesize) != meminfo.erasesize) {
			printf("** PANIC!! Not able to write back to flash\n"
				"Sorry!! Boot Partition is gone\n") ;
			goto exit_3 ;
		} else {
			VERBOSE(("Written NVS Header\n")) ;
		}
	}

	rc = 0 ;

exit_3:
	free(buf) ;
exit_2:
	close(fd) ;
exit_1:
    return rc ;
}


int writemac(char* mac_in)
{
    NVSHEADER nvs ;
    int rc = 1;
    char mac[MAC_LEN] ;

    if(!mac_in) {
        printf("No mac\n") ;
        goto exit_1 ;
    }
    if(parse_mac(mac_in, mac) != 0) {
        goto exit_1 ;
    }

    if(readNVSHeader(&nvs)) {
        printf("Cannot read NVS Header\n") ;
        goto exit_1 ;
    }
    VERBOSE(("Old Mac: %02x:%02x:%02x:%02x:%02x:%02x\n", nvs.mac[0],
        nvs.mac[1], nvs.mac[2], nvs.mac[3], nvs.mac[4], nvs.mac[5])) ;
    nvs.mac[0] = mac[0], nvs.mac[1] = mac[1],
    nvs.mac[2] = mac[2], nvs.mac[3] = mac[3],
    nvs.mac[4] = mac[4], nvs.mac[5] = mac[5] ;
    if (writeNVSHeader(&nvs)) {
        printf("Cannot write NVS Header\n") ;
        goto exit_1 ;
    }
    VERBOSE(("Written Mac %02x:%02x:%02x:%02x:%02x:%02x\n", nvs.mac[0],
        nvs.mac[1], nvs.mac[2], nvs.mac[3], nvs.mac[4], nvs.mac[5])) ;
    rc = 0 ;

exit_1:
    return rc ;
}


static int parse_mac(char* mac_in, char* mac_out)
{
int i = 0 ;

	for(i = 0 ; (i < MAC_LEN) && *mac_in ; i++) {
		if(ISXDIGIT(*mac_in) && ISXDIGIT(*(mac_in + 1))) {
			mac_in += 2 ;
			if(ISDELIMITER(*mac_in)) {
				*mac_in = '\0' ;
				mac_out[i] = ATOX(mac_in - 2) ;
				mac_in++ ;
			} else
			if (!*mac_in) {
				mac_out[i] = ATOX(mac_in - 2) ;
			} else {
				printf("Cannot parse Mac\n") ;
				return 1 ;
			}
		} else {
			printf("Invalid Mac\n") ;
			return 1 ;
		}
	}
	if(i < MAC_LEN) {
		printf("Short Mac < %d bytes\n", MAC_LEN) ;
		return 1 ;
	} else
	if(*mac_in) {
		printf("Mac > %d bytes, using first %d bytes\n", MAC_LEN, MAC_LEN) ;
	}
	if(*mac_out &1) {
		printf("Multicast Mac ???\n") ;
		return 1 ;
	}
	return 0 ;
}


static int crcbit(int i, int crc)
{
    int j = (crc >> 15) & 1 ;
    if (i) j = 1 - j ;
    crc = crc << 1 ;
    if (j) crc = crc ^ 0x8005 ;
    crc = crc & 0xffff ;

    return crc ;
}


/* Bytewise CRC calculation.
 */
static int crcbyte(int x, int crc)
{
  int j ;

  for (j = 0  ; j < 8  ; j++)
  {
      crc = crcbit((x & 128) ? 1:0, crc) ;
      x = x << 1 ;
  }

  return crc ;
}


static int crcblock(unsigned char *pData, unsigned int size)
{
    int crc ;


    /* Compute CRC over header and data area.
     */
    crc = 0 ;
    while (size > 0)
    {
        crc = crcbyte(*pData ++, crc) ;
        size -- ;
    }

    return crc ;
}


int crcnvs(NVSHEADER *pHeader, int size)
{
    unsigned char *pByte ;

    /* Start address and length for CRC calculation.
     * The CRC is computed for the entire block except
     * for the first two bytes (which will receive the CRC).
     */
    pByte = (unsigned char *) pHeader ;
    return crcblock(pByte + 2,  size - 2) ;
}

