/* Utility for updating flash information */
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/mount.h>
#include <sys/wait.h>
#include <sys/sysmacros.h>
#include <fcntl.h>

//1KB HEADER support
#ifdef CUSTOMER_HEADER_SUPPORT
	#define KERNELPARTNO    1
	#define HEADER_LENGTH 1024 //We use 1KB of space
#endif

#define ROOTFSPARTNO	2
#define ERASE			1
#define LOADDIR			"/var/fs"

struct {
	char *char_dev_name; /* For erase and update operation */
	char *block_dev_name; /* For jffs2 update onl */
	char *file_name;  /* file name on tftp server */
	char *description;
} partitionInfo[] = {
	/* Partition 0 */
	{ "/dev/mtd0","","boot.bin","Boot Partition" },
	/* Partition 1 */
	{ "/dev/mtd1","","flashfs_update.bin","Kernel  Partition" },

	/* Partition 2 */
#if	defined(JFFS2)
	{ "/dev/mtd2","/dev/mtdblock2","jffs2.bin","JFFS2 Partition" },
#elif defined(SQUASHFS)
	{ "/dev/mtd2","/dev/mtdblock2","squashfs.bin","SQUASHFS Partition" },
#endif
	{"","","",""}
};


int localError=0;

int flash_updatePart(int netboot, char *serverAddress, unsigned int partNo, char erase);
int run_system_command(char *name);


void printHelp()
{
     printf("Flash Update supports following options \n");
     printf("\t <server address> Option for JFFS2 partition (2) update  \n");
     printf("\t -e=<part_no> <server_address>: Erase and update partition <part_no>.\n\t   No partition number for all \n");
     printf("\t -c=<output file name>: Save config information to output file\n");
     printf("\t -h: This help information \n");
     printf("\t -p: Print partitionon information \n");
     printf("\t Note:- By default flash_update updates the ROOTFS (/dev/mtd2) partition if no options are specified\n");
}


int main (int argc, char *argv[])
{
	char *optstring = "huc::pe::";
	char c,buffer[100];
	unsigned int maxPartition = 0;
	int netboot = 0;
	FILE *fp;
	struct stat *stats ;
	unsigned int i = 0,ret = 1,option = 1;
	
	while (strlen(partitionInfo[maxPartition].char_dev_name)>1) {
		printf("\rPartition 0 to %d are configured ", maxPartition++);
	}
	printf("\n");

	stats = (struct stat*) buffer ;
	if(stat(LOADDIR, stats) < 0) {
		mode_t mode = S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH ;
		printf("seems %s does not exists, trying to create it\n", LOADDIR) ;
		if(mkdir(LOADDIR, mode) < 0) {
			printf("Failed to create %s\n", LOADDIR) ;
			goto error1 ;
		}
	} else if(!S_ISDIR(stats->st_mode)) {
		printf("seems %s is not a directory 0x%x\n", LOADDIR, stats->st_mode) ;
		goto error1 ;
	}

	if((mount("none", LOADDIR, "tmpfs", 0, NULL)) || chdir(LOADDIR)) {
		printf("mount failed for %s\n", LOADDIR) ;
		goto error1 ;
	}
	memset(buffer, 0, sizeof(buffer)) ;

	c = getopt(argc, argv, optstring);
        if (c!='c' && c!= 'h')

	if ((fp = fopen("/proc/cmdline", "r")) && (fgets(buffer, sizeof(buffer), fp))) {
		(netboot = (int) strstr(buffer, "ram0")) ? printf("Network boot\n")
			: printf("Flash boot \n");
	} else
		goto error;

	if(argc > 1) {
		switch(c) {
			case 'h': {
				printHelp();
			} break;

			case 'p': {
				unsigned int i = 0;

				printf("option -p is selected\n");				
				printf(" dev_name\t file_nam                description \n");
				while(i < maxPartition) {
					printf(" %s\t %-20.20s  %s \n", partitionInfo[i].char_dev_name,
						partitionInfo[i].file_name,
						partitionInfo[i].description);
					i++;
				}
			} break;

			case 'u': {
			/*
			 *   Option -u has been removed.
			 */
				printf("Option -u has been discontinued\n");
			} break;

                    case 'c': {
                        if (flash_config_main(partitionInfo[0].char_dev_name, optarg)) {
                            goto error;
                        }
                    } break;

			case 'e': {
				if (option)
					printf("option -e is selected with server address %s\n", argv[optind]);

				if(optarg) {
					if((optarg[1] - '0' >= maxPartition) ||
						(flash_updatePart(netboot, argv[optind], optarg[1] - '0', option)))
						goto error;
				} else {
					i = 0;
					while(strlen(partitionInfo[i].char_dev_name) > 1) {
						if(flash_updatePart(netboot, argv[optind], i, option))
							goto error; 
						i++;
					}
				}
			} break;

			default: {
				printf("default option with server address %s\n", argv[optind]);

				/* Erase the partitions during update -- default behaviour */
				if(flash_updatePart(netboot, argv[optind], ROOTFSPARTNO, ERASE))
					goto error;
			} break;
		} /* end of switch */
	} else
		printHelp();

	ret = 0;

error:
	fclose(fp);
	if(ret)
		perror("update failed ");

	chdir("/"); 

//#if defined(JFFS2)
//	umount("/ramfs");
//	rmdir("/ramfs");
//#elif defined(SQUASHFS)
	umount(LOADDIR) ;
//#endif
error1:
	exit(ret);
}

/* update particular partition */

int flash_updatePart(int netboot, char *serverAddress, unsigned int partNo, char erase)
{
	char buffer[1024];
	signed int len;
	int fSrc, fTarget;
	int ret;
	int i=0;
#ifdef CUSTOMER_HEADER_SUPPORT
	char cust_buf[HEADER_LENGTH];
#endif
	printf("\nUpdate starts for %s \n", partitionInfo[partNo].description); 
	
	/* do not download files for network boot and jffs2 update */

#ifdef JFFS2
	if(!(netboot && (partNo == ROOTFSPARTNO)))
#endif
	{
		sprintf(buffer, "busybox tftp -g -r %s %s", partitionInfo[partNo].file_name, serverAddress);
		if((ret = run_system_command(buffer)) == -1)
			return -1;
	}

	/* see if boot partition update and then save config info first */

	if(!partNo) {
		printf("Restoring old configuration \n");
		if(flash_config_main(partitionInfo[partNo].char_dev_name,NULL)) {
			printf("Configuration update failed."
				"Ignore this error for uninitalized  boot partition \n");
		}
	}

#if defined(JFFS2) || defined(SQUASHFS)
	if(!netboot && (partNo == ROOTFSPARTNO)) {
		erase = 0;
	}
#endif

	if(erase) {
		char *tArgv[2] = {0,0};
		int tArgc = 2;
		tArgv[1] = (char*) malloc(100);

		#ifdef CUSTOMER_HEADER_SUPPORT
		if( partNo  == KERNELPARTNO) {
			memset(cust_buf,0,HEADER_LENGTH);
			fSrc = open(partitionInfo[partNo].char_dev_name,O_RDONLY);
			if(fSrc  < 0) {
				printf("Could not Open %s\n",partitionInfo[partNo].char_dev_name);
				return -1;
			}
			ret = read(fSrc,cust_buf,HEADER_LENGTH);
			if(ret  < 0) {
				printf("Failed to read  Customer area\n");
				return -1;
			}
			else if(ret < HEADER_LENGTH) {
				printf("WARNING: Customer Area Read :  Read only  %d bytes out of %d bytes\n",ret,HEADER_LENGTH);
			}
			close(fSrc);
		}
		#endif
		strcpy(tArgv[1], partitionInfo[partNo].char_dev_name);
		printf("Erase Start for %s \n", partitionInfo[partNo].char_dev_name);
		if(flash_erase_main(tArgc,tArgv))
			return -1;
		free(tArgv[1]);
		printf("Erase Complete \n");
	}

#ifdef JFFS2
	/* JFFS2 file is tar file so update it differently */
	if(netboot && (partNo == ROOTFSPARTNO)) {
		if(chdir("/"))
			return -1;
		if(mkdir("/ram", (mode_t)0))
			return -1;

		if(mount("/dev/mtdblock2", "/ram", "jffs2", 0, NULL)) {
			perror("mount failed");
			return -1;
		}

		sprintf(buffer,"cp -af bin lib dev isfs linuxrc sbin part2 tmp root home usr etc var /ram");

		if(run_system_command(buffer) == -1)
			return -1;
		mkdir("/ram/proc", (mode_t)0);
		umount("/ram");
		rmdir ("/ram");
		goto update_done ;
	} 
#endif
#if defined(JFFS2) || defined(SQUASHFS)
	if(!netboot && (partNo == ROOTFSPARTNO)) {
		printf("\n*******  THIS WILL REBOOT THE SYSTEM!! HANG ON  *********\n\n") ;
		//clear out the buffer
		memset(buffer,'\0', 1024);

		sprintf(buffer, "mv %s file2flash.bin; echo \"1\" > update; reboot",
			partitionInfo[partNo].file_name) ;
		run_system_command(buffer) ;
		exit(0) ;
	}
#endif	

	if((fSrc = open(partitionInfo[partNo].file_name, O_RDONLY)) &&
		(fTarget = open(partitionInfo[partNo].char_dev_name, O_WRONLY|O_SYNC))) {
		unsigned int count = 0;

		#ifdef  CUSTOMER_HEADER_SUPPORT 
		if(partNo == KERNELPARTNO) {
			/*We make sure that Header is written first  and then we move the fd of the file_name ahead also.
			*/
			len = write(fTarget,cust_buf,HEADER_LENGTH);  //Write the Customer Header  at  the start  
			lseek(fSrc,HEADER_LENGTH,SEEK_SET);//We skip the 1KB section from the start of Kernel Partition
			count += len;
		}
		#endif

		printf("Writing %s to flash\n", partitionInfo[partNo].file_name);
		while((len = read(fSrc, buffer, sizeof buffer)) > 0) {
			count += len;
			printf("\rWritten %d bytes", count);
			if(write(fTarget, buffer, len) <= 0) {
				close(fSrc);
				close(fTarget);
				perror("write error");
				return -1;
			}
			memset(buffer, 0, sizeof buffer) ;
		}
		close(fSrc);
		close(fTarget);
	} else
		return -1;

update_done:

	unlink(partitionInfo[partNo].file_name);
	printf("\n%s update successfully\n", partitionInfo[partNo].description);
	return 0;
}


int run_system_command(char *name)
{
	unsigned long count = 0;
	pid_t pid;
	char *hourGlass = "|/-\\" ;

	printf("Executing %s \n",name);
	pid = fork();
	localError;
	if(pid) {
		while(!waitpid(pid, &localError, WNOHANG)) {
			printf("\r%c", hourGlass[(count++ / 50) % sizeof(hourGlass)]) ;
		}
		return (localError ? -1 : 0) ;
	} else {
		if((localError = system(name)) != 0) {
			printf("command %s failed \n", name);
			exit(-1) ;
		}
		exit(0);
	}
}

