
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include "flash_config.h"
#include <mtd/mtd-user.h>


#define USAGE() printf("Usage : pda-rw -r <File-name> : To read the PDA into <File-name> from flash\n") ; \
		printf("        pda-rw -w <File-name> : To write a PDA file <File-name into flash>\n"); \
		printf("")

#define MTD_DEVICE "/dev/mtd0" //We expect that we will find PDA in mtd0 only
#define PDASIZE (8 * 1024) 

unsigned int validate_PDA(int fd,int *start);
int get_mtd_size(int fd);

int main(int argc, char** argv)
{
	int fd=-1,pdafd=-1,configOffsetStart=-1,nvsSize=-1,configInfoSize=-1;
	char *buf=NULL;
	int count=0;

	if(argc != 3) { //No filename passed or r/w operation
		USAGE();
		return 1;
	}
	DbgPrint(("argc : %d\n",argc));
	DbgPrint(("argv[0] : %s [1] : %s [2] : %s\n",argv[0],argv[1],argv[2]));

	if(!strcmp(argv[1],"-r")){

		/*
		*	Assumptions:
		*	Valid NVS Header in boot partition.
		*	0xdead and 0xbeef are the header and footer of PDA.
		*/

		//open PDA file		
		pdafd = open(argv[2],O_RDWR|O_CREAT|O_SYNC,0777);
		if(pdafd < 0) {
			printf("Cannot Open %s file\n",argv[2]);
			exit(1);
		}
		//open MTD device file		
		fd = open(MTD_DEVICE,O_RDONLY|O_SYNC);
		if(fd < 0) {
			printf("Cannot Open %s file\n",MTD_DEVICE);
			close(pdafd);
			exit(1);
		}

		//Start Search for the PDA in the flash. We assume the flash layout is of ISOS.
		if( getNVS(fd,&configOffsetStart, &nvsSize) == ESUCCESS) {
			DbgPrint(("configOffsetStart %d :  nvsSize %d\n", configOffsetStart, nvsSize)) ;

#if 0
			if(configInfoFind(fd, &configInfoSize) != ESUCCESS) {
				DbgPrint(("No Valid PDA config section\n"));
			}
			DbgPrint(("PDA found with size of %x\n",configInfoSize));

			memset(buf,0,configInfoSize + 4);

			configOffsetStart += nvsSize;


#endif
			//Jump to the start of NVS
			lseek(fd,configOffsetStart,SEEK_SET);

			//Lets start reading for the PDA Header
			while (1) {
				unsigned short header=0, footer=0;

				if(read(fd,&header,sizeof(unsigned short))< 0) {
					printf("Read less than PDASIZE\n");
				}
				if(header == 0xdead) { //Aloha !!!! We found PDA header
					//printf("HEADER COUNT %d\n",count * (sizeof(unsigned short)));
		                        count = 0 ;
		                        do {
                		                read(fd, &footer, sizeof(unsigned short)) ;
						if(!count) printf("COUNT 0 : %x\n",footer);
                                		count++ ;
		                        } while(footer != 0xbeef && (count < 5000)) ; // PDA is expected in the next 8k range
		                
				        if(footer != 0xbeef) { //Aloha !!!! We found PDA footer
						printf("Could Not find PDA\n");
						goto close;
					}
					else {
						int tmp=0;
						//Lets get back to the start of PDA
						tmp = lseek(fd,-(count * (sizeof(unsigned short))), SEEK_CUR) ;
						//printf("PDA Start %d PDA End %d\n",tmp,tmp+(count * (sizeof(unsigned short))));
						//printf("COUNT %d\n",count * (sizeof(unsigned short)));
						configInfoSize = (--count) * (sizeof(unsigned short));
						//printf("COUNT %d\n",count * (sizeof(unsigned short)));
						break;
					}
				}
				if(++count > 5000) { // Arghhh !!!! We are tired of searching.. Exiting
					printf("Could not Find PDA Header... Exiting \n");
					goto close;
				}
			}
			printf("Allocating Memory: %d\n",configInfoSize );
			buf = (char *) malloc(configInfoSize );
			if(!buf) {
				printf("Memory Alloc failed\n");
				goto close;
			}
			memset(buf,0xff,configInfoSize);

			if(read(fd,buf,configInfoSize)< 0 ) {
				printf("Failed to Read\n");
			}
 			if (write(pdafd,buf,configInfoSize) < 0) {
				printf("Could not write PDA into File\n");
			}
			printf("PDA Written in %s %d\n",argv[2],configInfoSize); //Success at Last!!!
			free(buf);
		}
		else {
			printf("Cannot Find NVS... Exiting\n");
			goto close;
		}

	}
	else if(!strcmp(argv[1],"-w")) {
		unsigned short header=0xdead, footer=0xbeef;
		int pda_start=0,pda_end=0,pda_size=0;
		short total;
		char *mtd_buf=NULL;

                //open PDA file
                pdafd = open(argv[2],O_RDONLY|O_SYNC);
                if(pdafd < 0) {
                        printf("Cannot Open %s file\n",argv[2]);
                        exit(1);
                }
                //open MTD device file
                fd = open(MTD_DEVICE,O_RDWR|O_SYNC);
                if(fd < 0) {
                        printf("Cannot Open %s file\n",MTD_DEVICE);
                        close(pdafd);
                        exit(1);
                }

                //Start Search for the PDA in the flash. We assume the flash layout is of ISOS.
                if( getNVS(fd,&configOffsetStart, &nvsSize) == ESUCCESS) {
                        DbgPrint(("ConfigOffsetStart %d :  nvsSize %d\n", configOffsetStart, nvsSize)) ;

			printf("MTDSIZE %d\n",get_mtd_size(fd));
			//Allocate memory for the whole boot partition:
			mtd_buf = (char *) malloc(get_mtd_size(fd));
			if(!mtd_buf) {
				printf("Unable to Allocate memory for MTD buffer\n");
				goto close;
			}
			//Read out the whole of the flash
			lseek(fd,0,SEEK_SET);
			read(fd,mtd_buf,get_mtd_size(fd));
	

                        //Jump over the  NVS header so that we can write PDA
                        lseek(fd,configOffsetStart + nvsSize,SEEK_SET);

			pda_size = validate_PDA(fd,&pda_start);
			if(!pda_size) {
				unsigned short header=0,footer=0;

				printf("PDA Not Found in Flash \n");

				//We did not find any PDA information in Config section.
				//Check if we have LXCMDLINE after NVS.
				//Jump over the  NVS header so that we can write PDA
				lseek(fd,configOffsetStart + nvsSize,SEEK_SET);

				read(fd,&header,(sizeof(unsigned short)));
				
				if(header == 0xcafe) {
					int count=0;

					while(1)  {
						//We found LXCMDLINE
						read(fd,&footer,sizeof(unsigned short));
						count++;
						if(footer ==  0xface)
							break; //We found the footer

						if(count > 2048) //We read 2048 * 2 bytes 
							break;
					}
					if(footer == 0xface) {
						char *tmp = NULL;
						struct stat fbuf;

						//We need to write the PDA  at this location

						//We read the PDA data  in the file into buf
						stat(argv[2],&fbuf);
						buf = (char *) malloc(fbuf.st_size);
						if(!buf)  {
							printf("Could not Allocate memory\n");
							goto close;
						}
	                                        memset(buf,0,fbuf.st_size);

	                                        //Read and Write the PDA Data from file
        	                                read(pdafd,buf,fbuf.st_size); //We dont have dead & beef in PDA file.

						tmp = mtd_buf;
						tmp += configOffsetStart + nvsSize;
						tmp += sizeof(unsigned short); //for LXCMDLINE header
						tmp += (count * (sizeof(unsigned short)));//for CMDLINE  and footer
						header = 0xdead;
						footer = 0xbeef;
						memcpy(tmp,&header,sizeof(unsigned short));
						tmp+=  sizeof(unsigned short);
						memcpy(tmp,buf,fbuf.st_size);
						tmp += fbuf.st_size;
						memcpy(tmp,&footer,sizeof(unsigned short));

	                                        printf("Wrote PDA Data of size start %d \n",fbuf.st_size);

        	                                //Erase the BOOOT Sector
                	                        erase_mtd(fd);

                        	                //Seek to the beginning of MTD
                                	        lseek(fd,0,SEEK_SET);

                                        	if(write(fd,mtd_buf,get_mtd_size(fd)) < get_mtd_size(fd)) {
                                               	 	printf("Wrote Less than %d bytes\n",sizeof(mtd_buf));
                                        	}
					}
				}
				else {
					//We did not find LXCMDLINE
					char *tmp = NULL;
                                        struct stat fbuf;
                                        //We read the PDA data  in the file into buf
                                        stat(argv[2],&fbuf);
                                        buf = (char *) malloc(fbuf.st_size);
                                        if(!buf)  {
                                        	printf("Could not Allocate memory\n");
                                                goto close;
                                        }
                                        memset(buf,0,fbuf.st_size);

                                        //Read and Write the PDA Data from file
                                        read(pdafd,buf,fbuf.st_size); //We dont have dead & beef in PDA file.

                                        tmp = mtd_buf;
                                        tmp += configOffsetStart + nvsSize;
                                        header = 0xdead;
                                        footer = 0xbeef;
                                        memcpy(tmp,&header,sizeof(unsigned short));
                                        tmp+=  sizeof(unsigned short);
                                        memcpy(tmp,buf,fbuf.st_size);
                                        tmp += fbuf.st_size;
                                        memcpy(tmp,&footer,sizeof(unsigned short));

                                        printf("Wrote PDA Data of size %d \n",fbuf.st_size);

                                        //Erase the BOOOT Sector
                                        erase_mtd(fd);

                                        //Seek to the beginning of MTD
                                        lseek(fd,0,SEEK_SET);

                                        if(write(fd,mtd_buf,get_mtd_size(fd)) < get_mtd_size(fd)) {
                                                printf("Wrote Less than %d bytes\n",sizeof(mtd_buf));
                                        }
				}
				
			}
			else {
				struct stat fbuf;
				char *tmp;
				printf("PDA Found in Flash \n");
				printf("start %d end %d size %d\n",pda_start,pda_start+pda_size,pda_size);

				//check if the PDA Calibration data inside the file fits in the space or not
				stat(argv[2],&fbuf);
				printf("FILE SIZE : %d\n",fbuf.st_size);


				if((pda_size - (2 * (sizeof(unsigned short)))) < fbuf.st_size ){ //size of PDA data
					printf("PDA Size Incompatible\n");
					//FIXME:: TODO
					free(mtd_buf);
					goto close;
				}
				else {
					
		                        buf = (char *) malloc(pda_size);
                		        if(!buf) {
                               			 printf("Allocation Failed\n");
						free(mtd_buf);
		                                goto close;
                		        }
		                        memset(buf,0,pda_size);

                		        //Read and Write the PDA Data from file
		                        read(pdafd,buf,fbuf.st_size); //We dont have dead & beef in PDA file.

					tmp = mtd_buf;
					tmp +=pda_start;
					tmp +=(sizeof(unsigned short));
					memcpy(tmp,buf,fbuf.st_size);

		                        printf("Wrote PDA Data of size start %d %d\n",tmp,tmp+pda_size);

					//Erase the BOOOT Sector
					erase_mtd(fd);
				
					//Seek to the beginning of MTD
					lseek(fd,0,SEEK_SET);
	
					if(write(fd,mtd_buf,get_mtd_size(fd)) < get_mtd_size(fd)) {
						printf("Wrote Less than %d bytes\n",sizeof(mtd_buf));
					}

				}
			}

#if 0
			//Write the PDA Header
			write(fd,&header,sizeof(unsigned short));
			
			buf = (char *) malloc(1024);
			if(!buf) {
				printf("Allocation Failed\n");
				goto close;
			}
			memset(buf,0,1024);

			//Read and Write the PDA Data
			while ((count = read(pdafd,buf,1024)) > 0 ) {
				
				if(count != 1024)
					write(fd,buf,count);
				else
					write(fd,buf,1024);
				memset(buf,0,1024);
				total+=count;
			}
			printf("Wrote PDA Data of size %d\n",total);

			//write the PDA footer 
			write(fd,&footer,sizeof(unsigned short));
				
			free(buf);		
#endif
DONE:
			printf("PDA Writing Completed...\n");
			free(mtd_buf);
			free(buf);		
			printf("PDA Writing Completed...\n");

		}


	}
	else {
		printf("Unknown option : %s\n",argv[1]);
		exit(1);
	}

close:
	close(fd);
	close(pdafd);
	return 0;
}

/*
*	This function scans for PDA area for upto 8192 bytes ahead from current location. 
*	If PDA is found, It will return the starting address,ending address and the size.
*/
unsigned int validate_PDA(int fd,int *start)
{
int count = 0, i ;
unsigned short header, footer ;

        do {
                read(fd, &header, sizeof(short)) ;
                if(header == 0xdead) {
                        DbgPrint(("Found header \n")) ;

                        count = 0 ;
                        do {
                                read(fd, &footer, sizeof(short)) ;
                                count++ ;
                        }
                        while(footer != 0xbeef && (count < 5000)) ; // PDA is expected in the next 8k range
                        if(footer == 0xbeef) {
                                count++ ;
                                i = lseek(fd, (-2 * count), SEEK_CUR) ;
                                DbgPrint(("found ftr %d %d\n", count, i)) ;
				*start = i;
                                return (count * (sizeof(short))) ; //4 is for the dead and beef 
                        } else {
                                return 0 ;
                        }
                }
        }
        while(count++ < 8192) ; // PDA is expected in 16k range
        return 0 ;
}

int get_mtd_size(int fd)
{

	mtd_info_t meminfo;
        if(ioctl(fd, MEMGETINFO, &meminfo) != 0) {
                printf("Unable to get MTD device info\n");
                return 0;
        }

	return ( meminfo.size);

}
int erase_mtd(int fd)
{
	mtd_info_t meminfo;
	erase_info_t erase;
        if(ioctl(fd, MEMGETINFO, &meminfo) != 0) {
                printf("Unable to get MTD device info\n");
                return -1;
        }

         //Erase the BOOOT Sector
         erase.start =  0; //FIXME: Hardcoded 
         erase.length = meminfo.erasesize ;

	for(;erase.start < meminfo.size; erase.start += meminfo.erasesize) {
		if(ioctl(fd, MEMERASE, &erase) != 0) {
                	printf("Failed erasing sector \n" ) ;
			return -1;
         	} else {
                	printf("Erased sector Complete\n") ;
         	}
	}
	return 0;	
}
