
/* ###########################################################################
# (c) Copyright Conexant Systems, Inc. 2005
# (c) Copyright GlobespanVirata, Inc., 1997-2005
#
# Conexant Systems, Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc. and its subsidiaries (collectively
# "Conexant"). It may only be read, used, copied, adapted, modified or
# otherwise dealt with by you if you have entered into a license agreement
# with Conexant and then subject to the terms of that agreement and any other
# applicable confidentiality agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a license agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/
/** @file    lvoice_potsim_isos.c
 *  @brief   pots simulator and runs on isos board  
 *  @ingroup linuxvoice/app_if
 */


#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <cyanlib.h>
#include <arpa/inet.h>

#include "ppots.h"
#include "vcore.h"


#define BASE_PORT 5500


#define PEER_LINUX_IP_ADDRESS "172.26.20.157"
extern unsigned int  lvoice_pPhone[10];
int  *lvoice_sock_addr;
extern char initialized ;
extern tASPool  pool;

extern void lvoice_sim_store_phone ( unsigned int index, unsigned int value);
extern void lvoice_sim_sig_txcall(CYANBUFFER *pBuffer, struct _iobuf *iobuf);
extern void lvoice_sim_audio_txcall(CYANBUFFER *pBuffer, struct _iobuf *iobuf);
extern void lvoice_sim_audio_txcall1(CYANBUFFER *pBuffer, struct _iobuf *iobuf);
extern int lvoice_sim_process (int argc, char *argv[]);

tASInterface lvoice_iface;



/**
 * Entry point of potsimulator on isos
 * This task waits for messages from socket and sends them to pots
 * @param  pCyan        pointer to message
 * @retun  success or error
 */
int lvoice_main (int argc, char *argv[])
{
   char buffer[100];

   struct sockaddr_in addr;
   fd_set   fds;
   int i,rc;
   unsigned short port=BASE_PORT,len; 
   char  peerAddress[20];
   tASMessage msg;
      
  
   strcpy(peerAddress, PEER_LINUX_IP_ADDRESS);
   //kprintf("%s process created .....\n",__FUNCTION__);
   //kprintf("%s : lvoice_sock_addr = 0x%x \n",__FUNCTION__, lvoice_sock_addr);

   switch (argc)
   {
     case 3: /* IP address and port */
            port = (unsigned short ) atoi(argv[2]);
     case 2:  /* IP address */
            strcpy(peerAddress, argv[1]);
            break; 
     default: break;
   }
   for (i=0; i < 2; i++)
   {

      if ( (lvoice_sock_addr[i]=socket(AF_INET, SOCK_DGRAM, 0)) < 0)
      {
        kprintf("%s Failed to create socket:%s", __FUNCTION__,as_StrErr(_errno));
        break;
      }
      //kprintf("%s socket created %d \n",__FUNCTION__,lvoice_sock_addr[i]);

      memset(&addr,0,sizeof(addr));
      addr.sin_family = AF_INET;
      addr.sin_addr.s_addr = htonl (INADDR_ANY);
      addr.sin_port = htons(port+i );

      if ( bind(lvoice_sock_addr[i], (struct sockaddr*)&addr,sizeof(addr)) < 0 )
      {
         kprintf("%sFailed to bind socket:%s\n",__FUNCTION__, as_StrErr(_errno));
         break;
      }
      //kprintf("%s bind success  on port %d \n",__FUNCTION__,port+i);
      /* now connect this to peer machine */
#if 0 /* NO connect on UDP in ISOS */
      memset(&addr,0,sizeof(addr));
      addr.sin_family = AF_INET;
      addr.sin_addr.s_addr = inet_addr(peerAddress);
      addr.sin_port = htons(port + i  );       
      while(1){
	if ( connect(lvoice_sock_addr[i], (struct sockaddr*)&addr,sizeof(addr) ) < 0)
	  {
	    kprintf("%sFailed to connect socket:%s\n",__FUNCTION__, as_StrErr(_errno));
	    hs_TimerWait(2000);
	  } else   break;
      }
#endif
      //kprintf(" %s connect success \n",__FUNCTION__);
   }
   /* check if socket and bind were success */
   if ( i  < 2 )
   {
      exit(1);
   }
   kprintf(" BIND AND CONNECT SUCCESS \n");

   //kprintf("%s : lvoice_sock_addr[0] 0x%x \n", __FUNCTION__, lvoice_sock_addr[0]);
   /* now create pool */
   if ( (i=as_PoolNew( &pool, "lvoice_sim_pool", "size=40/count=100" )) !=kASEOK )
   {
       kprintf("%s cound create error %d",__FUNCTION__,as_StrErr(_errno));
       exit(2);

   }
   //kprintf("%s going in while loop \n",__FUNCTION__); 
   initialized = 1;

  /* wait for message from other call */
   //kprintf("%s message received from task \n",__FUNCTION__);   

   while (1)
   {
     FD_ZERO(&fds);
     FD_SET(lvoice_sock_addr[0], &fds);
     FD_SET(lvoice_sock_addr[1], &fds);

     if ( select ( lvoice_sock_addr[1] +1, &fds, NULL,NULL,NULL) > 0 )
     {
        int bufSize=0;
        //kprintf("Select success \n");

        for ( i=0; i < 2; i++ )
        {
          if (FD_ISSET(lvoice_sock_addr[i],&fds))
          {
            //kprintf ("Reading from socket decsriptor %x i = %x\n", lvoice_sock_addr[i], i);
            if ( (bufSize = recv(lvoice_sock_addr[i], buffer, 100,0)) > 0 ) /* read the len*/
            {
               //bufSize= *(int *)buffer;
               //kprintf("%s : recv packet type %s",__FUNCTION__, (i ? "date":"signal"));
               if ( i  ) 
                lvoice_sim_process_audio(buffer,bufSize);
               else
                lvoice_sim_process_event(buffer,bufSize);
                
            }
            else perror("socket recieved");

          }
          else perror("FDSET ");
 
        }
 
     }
     else
       kprintf("%s spurious select \n",__FUNCTION__);

   }
}


