
/* ###########################################################################
# (c) Copyright Conexant Systems, Inc. 2005
# (c) Copyright GlobespanVirata, Inc., 1997-2005
#
# Conexant Systems, Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc. and its subsidiaries (collectively
# "Conexant"). It may only be read, used, copied, adapted, modified or
# otherwise dealt with by you if you have entered into a license agreement
# with Conexant and then subject to the terms of that agreement and any other
# applicable confidentiality agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a license agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/
/** @file    lvoice_potsim_linux.c
 *  @brief   pots simulator and runs on isos board  
 *  @ingroup linuxvoice/app_if
 */


#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <pthread.h>
#include <sys/poll.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <sched.h>

#include "lvoice_common.h"
#include "lvoice_ioctl.h"
#define MAX_SOCKET  3
#define LVOICE_MAX_DEVICE_FILES   2 /* one for control/event and
                                      one audio */
#define BASE_PORT  5500

#define LVOICE_DEVICE_PREFIX  "/dev/cnxtvoicesignal" /* prefix name for device files */
int lvoice_sock_addr[MAX_SOCKET];
int lvoice_fd[LVOICE_MAX_DEVICE_FILES];

typedef void   (*cmdFunction)(int, char *, char );

#define MAX_CMD    35
int logicalToUserId[3];
/* Function for voice driver hanlding */

static void * lvoice_sim_voice_event(void *arg)
{
   struct pollfd fds[2];  /* two upstreams */
   unsigned char index;
   char  name[20];
   int buffer[100];
   char errorStr[50];
   int ioctlCmd;
   //tLVOICE_IoctlArg ioctlArg;
   int ioctlArg;
   int ringindex=0;
   strcpy(name,LVOICE_DEVICE_PREFIX);
   int result;
   struct sockaddr_in addr;
   memset((void *)&ioctlArg, 0, sizeof ioctlArg); 
   //printf("***************%s thread started \n",__FUNCTION__);

   memset(&addr,0,sizeof(addr));
   addr.sin_family = AF_INET;
   addr.sin_addr.s_addr = inet_addr("172.26.20.158");
   addr.sin_port = htons(5500);
 
  while (1)
  {
     char buffer[200];
     fds[0].fd=lvoice_fd[0];
     fds[0].events=POLLPRI;
     if ( poll( fds, 1, -1 ) == - 1)
     {
         sprintf(errorStr,"%s poll return failure ",__FUNCTION__);
         perror(errorStr);

         return NULL;

     }
     if ( !(fds[0].revents & POLLPRI) )
     {
       printf("%s poll return without event\n",__FUNCTION__);
       continue;

     }
     //printf("%s: retrieving events \n",__FUNCTION__);
     while (1)
       {  /* read all available event */
          result= read( lvoice_fd[0], buffer, sizeof buffer);
          if ( result < 0 )
          {
            sprintf(errorStr,"%s error in read", __FUNCTION__);
            perror(errorStr);
            return NULL;
          }
          else if (result==0) break;
         

         if (sendto(lvoice_sock_addr[0],buffer, result, 0,
             (struct sockaddr *)&addr,sizeof(addr)) <  0)
         {

           sprintf(errorStr,"%s error while sending to peer",__FUNCTION__);
           perror(errorStr);

           return NULL;
         }
         //print_buffer("before",buffer);
         //printf("%s: Send to peer success \n",__FUNCTION__);
       }
     }
}

void *lvoice_sim_voice_audio(void *arg)
{
   struct pollfd fds[2];  /* only for events */
   unsigned char index,i,j;
   char  name[20];
   //tLVOICE_IoctlArg ioctlArg;
   int ioctlArg;
   //tLVOICE_Audio  audio;
   struct sockaddr_in addr;
   int result;
   int buffer[100];
   char errorStr[100];
   char *devName[LVOICE_MAX_DEVICE_FILES]={ "/dev/cnxtvoicesignal", "/dev/cnxtvoiceaudio" };
   
   memset((void *)&ioctlArg, 0, sizeof ioctlArg); 
   //printf("%s thread started \n",__FUNCTION__);

   memset(&addr,0,sizeof(addr));
   addr.sin_family = AF_INET;
   addr.sin_addr.s_addr = inet_addr("172.26.20.158");
   addr.sin_port = htons(5501); 

   for (index=1; index < LVOICE_MAX_DEVICE_FILES; index++)
   {

      if ( (lvoice_fd[index]=open(devName[index],O_RDWR))== -1 )
      {
          sprintf(errorStr,"%s could not open device %s", __FUNCTION__, devName[index]);
          perror(errorStr);

          return NULL;
      }
      else
      {
      // printf(" %s open file %s\n",__FUNCTION__, devName[index]);
      }
   }
   
   /* Now wait for audio events from driver */

  while (1)
  {
     /* wait for audio on upstream 1 and 3 for read as 2 and 4 r for downstream */
     fds[0].fd=lvoice_fd[1];
     fds[0].events=POLLIN;
     //fds[1].fd=lvoice_fd[2];
    // fds[1].events=POLLIN;
     //if ( poll( fds, 2, -1 ) == - 1)
     if ( poll( fds, 1, -1 ) == - 1)
     {
         sprintf(errorStr,"%s poll return failure ",__FUNCTION__);
         perror(errorStr);

         return NULL;

     }
     for (index=0; index < 1; index++ )
     {
       if ( !(fds[index].revents & POLLIN) )
       {
	 printf("%s poll return without audio\n");
	 continue;

       }
     /* audio is available and do read on this channel */

       while (1)
       {  /* read all available audio */
	  result= read( lvoice_fd[index], buffer, 100);
	  if ( result < 0 )
	  {
            sprintf(errorStr,"%s error in read", __FUNCTION__);
            perror(errorStr);
	    return NULL;
	  }
          else if (result==0) break;
            
	  //printf("%s: rcvs  port %d len %d \n", index+1, result);
	  /* now send this to peer */ 
#if 0
	  i=0;
	 
	  buffer[i++]=index;
	  buffer[i++]=audio.timeStamp;
	  buffer[i++]=audio.payloadType;
	  buffer[i++]=audio.payloadLen;
	  /* copy payload byte */
	  for (j=0; j < audio.payloadLen; j++)
	  {
	     buffer[i++]=audio.payload[j];
	  } 
	  
	  if (send(lvoice_sock_addr[1],buffer, i * sizeof( int), 0) <  0)
	  {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
	     return NULL;
	   }
#endif
           if (sendto(lvoice_sock_addr[1],buffer, result, 0,
             (struct sockaddr *)&addr,sizeof(addr)) <  0)
           {

               sprintf(errorStr,"%s error while sending to peer",__FUNCTION__);
               perror(errorStr);

              return NULL;
           }

        }
      }
   }
}


void lvoice_sim_process_cmd( char *buffer, char buffsize )
{

 int cmd = *((int *)buffer);
 int result=0;
 char errorStr[50];

 //printf("%s cmd %x \n",__FUNCTION__,cmd);

 struct sockaddr_in addr;

 memset(&addr,0,sizeof(addr));
 addr.sin_family = AF_INET;
 addr.sin_addr.s_addr = inet_addr("172.26.20.158");
 addr.sin_port = htons(5502);
 //printf(" lvoice_id %x \n", lvoice_fd[0]);
 
 print_buffer("before",buffer);
 switch (cmd)
 {
   case POTS_InitPots_Cmd:
          //printf("inside POTS_InitPots_Cmd \n");
          //printf(" buffer %x  sizeof cmd %x sizeof tPOTS_InitPots %x \n",
          //         buffer,    sizeof(cmd),   sizeof (tPOTS_InitPots));
          
          result= ioctl(lvoice_fd[0],cmd, (char *)(buffer+sizeof (cmd)));
          perror("ioctl errror");
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
     
          print_buffer("after POTS_InitPots_Cmd",buffer); 
          if (sendto(lvoice_sock_addr[0],buffer, sizeof (tPOTS_InitPots)+sizeof(cmd), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
  case POTS_StartSignalAnPhone_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (tPOTS_StartSignalAnPhone * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer,sizeof(cmd)+ sizeof (tPOTS_StartSignalAnPhone), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
  case POTS_StartRingAnPhone_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (tPOTS_StartRingAnPhone * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer, sizeof(cmd)+sizeof (tPOTS_StartRingAnPhone), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;


  case POTS_StopSignalAnPhone_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (tPOTS_StopSignalAnPhone * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer,sizeof(cmd)+ sizeof (tPOTS_StopSignalAnPhone), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
          
  case POTS_CallerIdAnPhone_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (tPOTS_CallerIdAnPhone * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer, sizeof(cmd)+sizeof (tPOTS_CallerIdAnPhone), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
          

  case POTS_ConnectPhone_Cmd:
          //printf("POTS_ConnectPhone_Cmd hPhone %x \n",(U32 * )(buffer+sizeof (cmd)));
          result= ioctl(lvoice_fd[0],cmd, (U32 * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer, sizeof (U32), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
          
  case POTS_AddUpstreamAudio_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (tPOTS_AddUpstreamAudio * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer, sizeof(cmd)+sizeof (tPOTS_AddUpstreamAudio), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
          

  case POTS_AddAudioPhone_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (tPOTS_AddAudioPhone * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer, sizeof(cmd)+sizeof (tPOTS_AddAudioPhone), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
  case POTS_DisconnectPhone_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (U32 * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer, sizeof (U32), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
          
  case POTS_ModifyPhone_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (tPOTS_ModifyPhone * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer, sizeof(cmd)+sizeof (tPOTS_ModifyPhone), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
          
  case POTS_SetFaxCoderName_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (tPOTS_SetFaxCoderName * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer,sizeof(cmd)+ sizeof (tPOTS_SetFaxCoderName), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
          
  case POTS_EnableAudioMixing_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (U32 * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer, sizeof (U32), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;

  case POTS_InitAnPhone_Cmd:
          //tPOTS_InitAnPhone*  pBuffer= (tPOTS_InitAnPhone * )(buffer+sizeof (cmd));
          //logicalToUserId[pBuffer->logicalId]=UserId;
          result= ioctl(lvoice_fd[0],cmd, (tPOTS_InitAnPhone * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer, sizeof(cmd)+sizeof (tPOTS_InitAnPhone), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
          
  case POTS_RemoveUpstreamAudio_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (U32 * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer, sizeof (U32), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
  case POTS_FailSafePots_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (U32 * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer, sizeof (U32), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
  
  case POTS_AddDelDCM_Cmd:
          result= ioctl(lvoice_fd[0],cmd, (tPOTS_AddDelDCM * )(buffer+sizeof (cmd)));
          /* replace command name with result in buffer */
          *((int *)buffer)= result;
          if (sendto(lvoice_sock_addr[0],buffer, sizeof(cmd)+sizeof (tPOTS_AddDelDCM), 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return ;
          }
          break;
  default:
         //printf("************%s:Unknown command %x",__FUNCTION__, cmd);        
         buffer[0]=1;
         /* So that application on gallus can proceed */
         if (sendto(lvoice_sock_addr[0],buffer, 1, 0,(struct sockaddr *)&addr,sizeof(addr)) <  0)
          {
             sprintf(errorStr,"%s error while sending to peer ",__FUNCTION__);
             perror(errorStr);
             return;
          }

        break;
 }

}
void lvoice_sim_process_audio( char *buffer, char buffsize )
{
  unsigned int index=0;
  int portId= *((int *)buffer + index++);
  char i, errorStr[100];
  //printf("%s called \n",__FUNCTION__);
#if 0
  tLVOICE_Audio audio;
  char i, errorStr[100];

  audio.timeStamp =*((int *)buffer + index++);
  audio.seqNo = *((int *)buffer + index++) >>16; 
  audio.payloadType=*((int *)buffer + index++) & 0xff; 
  audio.payloadLen= *((int *)buffer + index++);
  if ( audio.payloadLen >=LVOICE_MAX_AUDIO_LEN )
  {
    printf("%s rcvd len %d is not correct \n",__FUNCTION__, audio.payloadLen);
    return;
  }
  /* copy data */
  buffer += (index-1)*sizeof( int);
  for ( i =0; i < audio.payloadLen ; i++)
  {
   audio.payload[i]=buffer[i];

  }

  index= portId < 1? 1:3;
#endif 
  
  if ( write(lvoice_fd[1],buffer,buffsize) < 0 )
  {
    sprintf(errorStr,"%s for index %d",__FUNCTION__, index);
    perror(errorStr);

  }

}
#if 0
#define DATA_PAYLOAD_OFFSET 3 /* parm1, parm2, status and then data */

void lvoice_sendIoctlArg( int cmd, char *buffer, char buffsize)
{

  tLVOICE_IoctlArg arg;

  memset((void *)&arg, 0, sizeof arg); 
  arg.portId=(*(int *)buffer) & 0xff;
  int ioctlcmd;
  char errorStr[100];
  /* only two arguments */
  arg.arg0=*((int *)buffer+ DATA_PAYLOAD_OFFSET) & 0xff;
  arg.arg1=*((int *)buffer+ DATA_PAYLOAD_OFFSET+1) & 0xff;
  //printf(" %s portId %x  arg0 %x  arg1 %x \n", __FUNCTION__, arg.portId, arg.arg0, arg.arg1); 

  switch(cmd)
  {
    case kIPGENCMD_CODERTYPE:
         ioctlcmd=LVOICE_CMD_CODERTYPE;
         break;
    case kIPGENCMD_DTMFRELAY:
         ioctlcmd = LVOICE_CMD_DTMFRELAY; 
         break;
    case kIPGENCMD_ECHOCTRL:
         ioctlcmd = LVOICE_CMD_ECHOCTRL;
         break;
    case kIPGENCMD_GETLINESTATE:
         ioctlcmd = LVOICE_CMD_GETLINESTATE;
         break;
    case kIPGENCMD_DIALTONEDETECT:
         ioctlcmd = LVOICE_CMD_DIALTONE_DETECT;
         break;
    case kIPGENCMD_RING:
         ioctlcmd = LVOICE_CMD_RING;
         break;
    case kIPGENCMD_SOFTRELAY:
         ioctlcmd = LVOICE_CMD_SOFTRELAY;
         break;
    case kIPGENCMD_FXOCONTROL:
         ioctlcmd = LVOICE_CMD_FXOCONTROL;
         break;
    case kIPGENCMD_DIAL:
         ioctlcmd = LVOICE_CMD_DIAL;
         break;
        
    case kIPGENCMD_GETDSPCLK:
         ioctlcmd = LVOICE_CMD_GETDSPCLK;
         break;
    case kIPGENCMD_SETDSPCLK:
         ioctlcmd = LVOICE_CMD_SETDSPCLK;
         break;
    case kIPGENCMD_GAINCTRLUP:
         ioctlcmd = LVOICE_CMD_GAINCTRLUP;
         break;
    case kIPGENCMD_GAINCTRLDN:
         ioctlcmd = LVOICE_CMD_GAINCTRLDN;
         break;
    case kIPGENCMD_NLPCTRL:
         ioctlcmd = LVOICE_CMD_NLPCTRL;
         break;
    case kIPGENCMD_CONFADD:
         ioctlcmd = LVOICE_CMD_CONFADD;
         break;
    case kIPGENCMD_CONFREMOVE:
         ioctlcmd = LVOICE_CMD_CONFREMOVE;
         break;
    case kIPGENCMD_CID:
         ioctlcmd = LVOICE_CMD_CID;
         break;
    case kIPGENCMD_MWI:
         ioctlcmd = LVOICE_CMD_MWI;
         break;
    case kIPGENCMD_HARDRELAY:
         ioctlcmd = LVOICE_CMD_HARDRELAY;
         break;
    case kIPGENCMD_DTMFREMOVE:
         ioctlcmd = LVOICE_CMD_DTMFREMOVE;
         break;
   case kIPGENCMD_SETSID:
        ioctlcmd=LVOICE_CMD_SETSID;
        break;
    default:
           //printf("%s unknown cmd %d \n",__FUNCTION__,cmd);
          ioctlcmd=0;
          break;
  }

  if ( !ioctlcmd )
  return;
  if ( ioctl(lvoice_fd[0],ioctlcmd, &arg )!=0 )
  {
    sprintf(errorStr,"%s ioctl cmd error",__FUNCTION__);
    perror(errorStr);

  } 

}

void lvoice_sendDummy( int cmd, char *buffer, char buffsize)
{

  //printf("\n********************command %d is not yet supported \n",cmd);

}


void lvoice_sendIntArg( int cmd, char *buffer, char buffsize)
{
  int ioctlcmd=0;

  switch (cmd)
  {
   case kIPGENCMD_GETLINESTATE:
        ioctlcmd=LVOICE_CMD_GETLINESTATE;
        break;
   case kIPGENCMD_TALK:
        ioctlcmd=LVOICE_CMD_TALK;
        break;
   case kIPGENCMD_GETTONE:
        ioctlcmd=LVOICE_CMD_GETTONE;
        break;
   case kIPGENCMD_GETVERSION:
        ioctlcmd = LVOICE_CMD_VERSION;
        break;
   case kIPGENCMD_RESTART:
        ioctlcmd = LVOICE_CMD_RESTART;
        break;
   case kIPGENCMD_OOS:
        ioctlcmd = LVOICE_CMD_OOS;
        break;
   case kIPGENCMD_IDLE:
        ioctlcmd = LVOICE_CMD_IDLE;
        break;
/*
   case kIPGENCMD_SHUTDOWN:
        ioctlcmd = LVOICE_CMD_SHUTDOWN;
        break;
*/
    default:
         //printf("%s unknown cmd %d \n",__FUNCTION__,cmd);
         ioctlcmd=0;
         break;
  }

  if ( !ioctlcmd )
  return;
  if ( ioctl(lvoice_fd[0],ioctlcmd, *(int *)buffer & 0xff)!=0 )
  {
    perror("%s ioctl cmd error",__FUNCTION__);
  }

}
void lvoice_sendTone ( int cmd, char *buffer, char buffsize) 
{
  tLVOICE_IoctlArg arg;
  int ioctlcmd=0;

  memset((void *)&arg, 0, sizeof arg); 
  switch(cmd)
  {
    case kIPGENCMD_TONE:
         ioctlcmd=LVOICE_CMD_TONE;
         break;
    case kIPGENCMD_TONESTOP:
         ioctlcmd=LVOICE_CMD_TONESTOP;
         break;
    case kIPGENCMD_OUTTONE:
         ioctlcmd=LVOICE_CMD_OUTTONE;
         break;
    case kIPGENCMD_TONECID:
         ioctlcmd=LVOICE_CMD_TONECID;
         break;
    default:
         ioctlcmd=0;
         //printf("%s unknown cmd %d \n",__FUNCTION__,cmd);
         break;
  }
  if ( !ioctlcmd) return;
  arg.portId=*(int *)buffer & 0xff;
  
  arg.arg0 = *((int*)buffer + 1) >> 16; /* Direction */ 
  arg.arg1=*((int *)buffer+ DATA_PAYLOAD_OFFSET) & 0xff; /* Tone */

  //printf("%s cmd %x port %x  direction %x tone %x\n",__FUNCTION__, cmd, arg.portId, arg.arg0, arg.arg1); 
  if ( ioctl(lvoice_fd[0],ioctlcmd, &arg )!=0 )
  {
    perror("%s ioctl cmd error",__FUNCTION__);

  } 

}
/**
 * Entry point of potsimulator on isos
 * This task waits for messages from socket and sends them to pots
 */


#endif
int main ( char argc, char *argv[])
{
   char buffer[100];
   char errorStr[100];
   unsigned int *pInt;
   struct sockaddr_in addr;
   socklen_t         fromlen;
   fd_set   fds;
   int i;
   unsigned short port=BASE_PORT;
   char  peerAddress[20];
   pthread_t  thread;
   pthread_attr_t  attr;
   struct sched_param     param;
   unsigned short len;

   strcpy(peerAddress, "172.26.20.158");

   //printf("Entering voice application \n");

   switch (argc)
   {
     case 3: /* IP address and port */
            port = (unsigned short ) atoi(argv[2]);
     case 2:  /* IP address */
            strcpy(peerAddress, argv[1]);
            break;
     default:
            break;
   }
   sleep(1);
   for (i=0; i < MAX_SOCKET; i++)
   {
      //printf(" Opening socket \n");

      if ((lvoice_sock_addr[i]=socket(AF_INET, SOCK_DGRAM, 0)) < 0 )
      {
        perror(" Failed to create socket\n");
        break;
      }

      memset(&addr,0,sizeof(addr));
      addr.sin_family = AF_INET;
      addr.sin_addr.s_addr = htonl (INADDR_ANY);
      addr.sin_port = htons(port + i );

      if ( bind(lvoice_sock_addr[i], (struct sockaddr*)&addr,sizeof(addr)) <0 )
      {
         sprintf(errorStr,"port %d",port+i); 
         perror(errorStr);
         break;
      }
     // printf("bind ok %d\n",port+i);
      /* now connect this to peer machine */
      memset(&addr,0,sizeof(addr));
      addr.sin_family = AF_INET;
      addr.sin_addr.s_addr = inet_addr(peerAddress);
      addr.sin_port = htons(port + i  );
      while( 1 )
      {
      if ( connect(lvoice_sock_addr[i], (struct sockaddr*)&addr,sizeof(addr) ) < 0)
      {
  
         sprintf(errorStr,"%sFailed to connect socket\n",__FUNCTION__);
         perror(errorStr);
         sleep(5); 
      }
      else
        break;
     }
     //printf("connected \n");

   }
   /* check if socket and bind were success */
   if ( i  < MAX_SOCKET )
   {
      printf("Failure in socket call \n");
      //exit(1);
   }
   //printf("Opening file \n");
   if ( (lvoice_fd[0]=open(LVOICE_DEVICE_PREFIX,O_RDWR))== -1 )
   {
       sprintf(errorStr,"%s could not open %s device", __FUNCTION__, LVOICE_DEVICE_PREFIX);
       perror(errorStr);

       return -1;
   }
   //printf("File descriptor %x\n",lvoice_fd[0]);
   
   pthread_attr_init(&attr);
   //printf ("%s socket done \n",__FUNCTION__);
   /* make this thread real time with default policy */
   param.sched_priority=sched_get_priority_min(SCHED_RR);
   pthread_attr_setschedpolicy(&attr,SCHED_RR);
   pthread_attr_setschedparam(&attr, &param);
   /* now create thread for driver  */
   if ( pthread_create(&thread, &attr,lvoice_sim_voice_event,NULL ) !=0) 
   {
       perror("could not create voice driver event thread");
       exit (2);


   }
   if ( pthread_create(&thread, &attr,lvoice_sim_voice_audio,NULL ) !=0) 
   {
       perror("could not create voice driver audio thread ");
       exit (2);

   }
   /* make current process as real time as well */
   if ( quantum_sched_setscheduler(0, SCHED_RR, &param) != 0)
   {
      sprintf(errorStr,"%sCan not set scheduling policy for current task",__FUNCTION__);
      perror(errorStr);
      exit(3);

   }
   //printf("socket 0 %x   socket 1 %x \n",lvoice_sock_addr[0],lvoice_sock_addr[1]);
   while (1)
   {
     
     FD_ZERO(&fds);
     FD_SET(lvoice_sock_addr[0], &fds);
     FD_SET(lvoice_sock_addr[1], &fds);
     FD_SET(lvoice_sock_addr[2], &fds);
     //printf("%s : waiting for messages on socket \n",__FUNCTION__);
     if ( select ( lvoice_sock_addr[2] +1, &fds, NULL,NULL,NULL) > 0 )
     {
        int bufSize=0;
        int bufSize1=0;
        //printf("%s :  messages on socket \n",__FUNCTION__);

        for ( i=0; i < MAX_SOCKET; i++ )
        {
         if (FD_ISSET(lvoice_sock_addr[i],&fds))
          {
            bufSize1=50;
           // printf(" FD_ISSET is set for index %x\n",i);
            
            //if (( bufSize=recvfrom(lvoice_sock_addr[i], buffer, bufSize1,0, (struct sock_addr*)&addr,&fromlen)) > 0 ) /* read the len*/
            if (( bufSize=recv(lvoice_sock_addr[i], buffer, bufSize1,0)) > 0 ) /* read the len*/
            {
               //printf("%s recv packet type %s",__FUNCTION__,(i == 1 ? "audio":"signal"));

               //printf(" Buffer Total len %x address %x \n",bufSize, buffer);
               pInt = (unsigned int *)buffer;
              // printf("\n *** Buffer len %x  buffer[1] %x buffer[2] %x buffer[3]%x  buffer[4]%x **\n",
               //          bufSize, *(pInt++), *(pInt++), *(pInt++), *(pInt++));
               //printf(" Buffer address %x \n",buffer);
               if ( i == 1  )
                lvoice_sim_process_audio(buffer,bufSize);
               else
                lvoice_sim_process_cmd(buffer,bufSize);

            }
            else
              perror("No buffer\n");

          }

        }

     }
     else
       printf("%s spurious select \n",__FUNCTION__);

   }
}


print_buffer(char *str,char *buffer)
{
 unsigned int i;
 //printf("buffer %s %x \n", str, buffer);
 for(i=0; i < 10; i++ )
 //printf("%x  ",((int*)buffer)[i]);
 //printf("\n");
 ;
}
