/*
 * CONFIDENTIAL AND PROPRIETARY INFORMATION OF VIRATA CORPORATION.
 *
 * Notice to Users of this Software Product:
 *
 * This software product of Virata Corporation ("Virata"), 2933 Bunker Hill
 * Lane, Suite 201, Santa Clara, CA 95054 contains source code that is the
 * valuable, proprietary property of Virata, and is protected as a trade secret.
 * Under no circumstances may you copy this software or distribute it to anyone
 * else by any means whatsoever except in strict accordance with a license
 * agreement between Virata and your company. This software is also protected
 * under copyright laws of the United States and foreign countries, as well as
 * under the license agreement, against any unauthorized copying, modification,
 * or distribution. Please see your supervisor or manager regarding any
 * questions that you may have about your right to use this software product.
 * Thank you for your cooperation.
 *
 * Copyright (C) 2001 Virata Corporation. All Rights Reserved
 *
 *
 */


#ifndef __POTS_SIG_RING_H__
#define __POTS_SIG_RING_H__


/* Length of FSK messages */
#define POTS_SIGNAL_MSG_LEN (6)

typedef unsigned int U32;
typedef unsigned char U8;
typedef unsigned int BOOL;
/*
 * All possible ring ID numbers used when ringing a phone via a POTS Api
 */
typedef enum
{
    kSIGRING_NLC, /* Normal Loop current */
    kSIGRING_RLC, /* Reverse loop current */
    kSIGRING_OLC, /* Open loop Current (MGCP - "oi") */
    kSIGRING_FLIP, /* Flip polarity */
    kSIGRING_END_SLIC_CTRL = kSIGRING_FLIP,

    kSIGRING_START_PREDEFINED,
    kSIGRING_RING_A = kSIGRING_START_PREDEFINED,
    /* defined as Bellcore 1 for now */
    kSIGRING_RING_B, /* defined as Bellcore 2 for now */
    kSIGRING_RING_C, /* defined as Bellcore 3 for now */
    kSIGRING_RING_D, /* defined as Bellcore 4 for now */
    kSIGRING_RING_E, /* defined as Bellcore 5 for now */
    kSIGRING_RING_OFF, /* Ring Off */
    kSIGRING_END_PREDEFINED,

    kSIGRING_START,
    kSIGRING_RINGSPLASH = kSIGRING_START,
    kSIGRING_RR, /* Reminder Ring -1 second */
    kSIGRING_JAPAN, /* default ring for JAPAN */
    kSIGRING_BT,    /* Default ring for BT (UK) */
    kSIGRING_CHINA, /* Default ring for China */
    kSIGRING_R0, /* Distinctive pattern (MGCP -"r0") */
    kSIGRING_R1, /* Distinctive pattern (MGCP -"r1") */
    kSIGRING_R2, /* Distinctive pattern (MGCP -"r2") */
    kSIGRING_R3, /* Distinctive pattern (MGCP -"r3") */
    kSIGRING_R4, /* Distinctive pattern (MGCP -"r4") */
    kSIGRING_R5, /* Distinctive pattern (MGCP -"r5") */
    kSIGRING_R6, /* Distinctive pattern (MGCP -"r6") */
    kSIGRING_R7, /* Distinctive pattern (MGCP -"r7") */
    kSIGRING_END,

    kSIGRING_LASTRING = kSIGRING_END,
    kSIGRING_DEFAULT = kSIGRING_END
}ePOTS_Rings;

/*
 * All Possible Signal ID's (tones) used in the POTS Api
 * NOTE: Some are Virata signals, some are MGCP signals and
 * in the future some will be MEGACO defined
 */
typedef enum
{
    kSIGTONE_DTMF_0, /* */
    kSIGTONE_DTMF_1, /* */
    kSIGTONE_DTMF_2, /* */
    kSIGTONE_DTMF_3, /* */
    kSIGTONE_DTMF_4, /* */
    kSIGTONE_DTMF_5, /* These are obvious */
    kSIGTONE_DTMF_6, /* */
    kSIGTONE_DTMF_7, /* */
    kSIGTONE_DTMF_8, /* */
    kSIGTONE_DTMF_9, /* */
    kSIGTONE_DTMF_A, /* */
    kSIGTONE_DTMF_B, /* */
    kSIGTONE_DTMF_C, /* */
    kSIGTONE_DTMF_D, /* */
    kSIGTONE_DTMF_POUND, /* */
    kSIGTONE_DTMF_STAR, /* */
    kSIGTONE_DTMF_END,  /* end of DTMF tones */

    kSIGTONE_RT = kSIGTONE_DTMF_END, /* RingBack tone */
    kSIGTONE_RTC, /* Ringback on Connection */
    kSIGTONE_CT, /* confirm tone*/
    kSIGTONE_NCT, /* network congestion */
    kSIGTONE_IT, /* intercept tone */
    kSIGTONE_PT, /* preemption tone -unknown at this time */
    kSIGTONE_AT, /* Answer tone */
    kSIGTONE_BT, /* busy tone */
    kSIGTONE_SAS, /* Call waiting tone "Subscriber alerting signal" */
    kSIGTONE_DT, /* dial tone */
    kSIGTONE_NTB, /* network busy (same as reorder tone -per mgcp spec) */
    kSIGTONE_ROT, /* Reorder tone*/
    kSIGTONE_PROMT, /* prompt tone */
    kSIGTONE_ERROR, /* error tone */
    kSIGTONE_STUTTER, /* stutter dial tone */
    kSIGTONE_ALERT, /* alerting tone */
    kSIGTONE_RWT, /* reorder warning tone */
    kSIGTONE_CC, /* Calling card "Bong" tone */
    kSIGTONE_OHW, /* Off hook warning tone */
    kSIGTONE_MWI, /* message waiting tone "long stutter tone" */
    kSIGTONE_SIT, /* Only SIT number #6 -per mgcp spec */
    kSIGTONE_SILENCE, /*play silence */
    kSIGTONE_END, /* total 36 Tones*/

    kSIGMSG_START,
    kSIGMSG_ADSI = kSIGMSG_START,
    kSIGMSG_VMWI_ON,
    kSIGMSG_VMWI_OFF,
    kSIGTONE_CUSTOM,
    kSIGMSG_END = kSIGTONE_CUSTOM,
    kSIGTONE_ENDALL
} ePOTS_Signals;

typedef enum
{
    kPOTS_REGION_BELLCORE,
#ifndef VOICE_DRIVER_INTERNATIONALISATION
    kPOTS_REGION_JAPAN,
    kPOTS_REGION_BT,
    kPOTS_REGION_CHINA,
    kPOTS_REGION_END
#else
    kPOTS_REGION_JAPAN,
    kPOTS_REGION_END = kPOTS_REGION_JAPAN
#endif
} ePOTS_Regions;

/* event types */
/* must be keep in sync with pots_eventPrintMap in pots.c */

/* The FXO commands must remain together and in order */
enum{
    kPOTS_NO_COMMAND = 0,
    kPOTS_MODEM_TONE_DETECTED = 1,
    kPOTS_FAX_TONE_DETECTED, 
    kPOTS_DTMF_DIGIT_DETECTED,
    kPOTS_DTMF_DIGIT_DETECTED_LEADING_EDGE,
    kPOTS_OFF_HOOK_TRANSITION,
    kPOTS_ON_HOOK_TRANSITION,
    kPOTS_FLASH_HOOK,
    kPOTS_RING_LEADING_EDGE,
    kPOTS_1ST_RING_COMPLETE,
    kPOTS_RING_CADENCE_COMPLETE,
    kPOTS_CID_COMPLETE,
    kPOTS_FXO_FIRST,         
    kPOTS_FXO_BATT_FLIP = kPOTS_FXO_FIRST, 
    kPOTS_FXO_RING_LEADING_EDGE, 
    kPOTS_FXO_RING_TRAILING_EDGE,
    kPOTS_FXO_CID_RDATA,
    kPOTS_FXO_FAR_END_RELEASE,
    kPOTS_FXO_FIRST_BOTH,         
    kPOTS_FXO_RING_CADENCE_COMPLETE = kPOTS_FXO_FIRST_BOTH,
    kPOTS_FXO_INUSE,
    kPOTS_FXO_AVAILABLE,
    kPOTS_FXO_DIALTONE_ACTIVE,
    kPOTS_FXO_DIAL_COMPLETE,
    kPOTS_FXO_OTHER_SEIZE,
    kPOTS_FXO_LAST = kPOTS_FXO_OTHER_SEIZE, 
    kPOTS_DCM_COMPLETE,
    kPOTS_TONE_UP_COMPLETE,               
    kPOTS_TONE_DN_COMPLETE,
    kPOTS_JITTER_IDLE,
    kPOTS_INITIAL_HOOK_STATUS,
    kPOTS_RESTART_COMPLETE,
    kPOTS_BATTFLIP_TIMEOUT,
    kPOTS_CMD_CONN_ANPHONE,
    kPOTS_CMD_DISCONN_ANPHONE,
    kPOTS_CMD_RING_START,
    kPOTS_CMD_RING_STOP,
#ifdef VOIP_ISDN
    kPOTS_START_OF_ISDN_EVENTS,
    kPOTS_ISDN_LAYER1_ACTIVATE_CONF = kPOTS_START_OF_ISDN_EVENTS,
    kPOTS_ISDN_LAYER1_DEACTIVATE_CONF,
    kPOTS_ISDN_LAYER1_ACTIVATE_IND,
    kPOTS_ISDN_LAYER1_LOST_FRAMING_IND,
    kPOTS_ISDN_LAYER1_ACTIVATE_PEND_IND,
    kPOTS_ISDN_READ_FRAMER_REG,
#endif
    kPOTS_LAST_EVENT
};
#endif

