/*
 * THIS IS AN UNPUBLISHED WORK CONTAINING VIRATA CORPORATION
 * CONFIDENTIAL AND PROPRIETARY INFORMATION.  IF PUBLICATION
 * OCCURS, THE FOLLOWING NOTICE APPLIES:
 * "COPYRIGHT 2002 VIRATA CORPORATION. ALL RIGHTS RESERVED"
 *
 * This file defines the platform being used and all platform
 * related constants.
 */


#ifndef _VCORE_CMD_H
#define _VCORE_CMD_H


/*
 * -------- This defines the codec data format for serial
 * communication to the DSP's McBSP.
 * 
 * MULAW: PCM type mu-Law between McBSP and codec.
 * ALAW: PCM type a-Law between McBSP and codec.
 * LOOPBACK: digital loopback between McBSP and codec.
 * LINEAR: linear data between McBSP and codec.
 */
#define VCORE_PCMTYPE_MULAW       (0x10)   /* mu-law data format */
#define VCORE_PCMTYPE_ALAW        (0x18)   /* A-law data format */
#define VCORE_PCMTYPE_LOOPBACK    (0x08)   /* Digital Loopback on codec */
#define VCORE_PCMTYPE_LINEAR      (0x11)   /* linear format */
#define VCORE_PCMTYPE_LINEAR_MSB  (0x00)   /*  ms bit first */
#define VCORE_PCMTYPE_LINEAR_LSB  (0x08)   /*  ls bit first */


/*
 * The following constants define the compander used for the voice timeslots
 * on the PCM bus. This companding is implemented on a per port basis using
 * the COMM_mu2linear(), COMM_linear2mu(), COMM_a2linear(), and COMM_linear2a()
 * functions.
 */
#define VCORE_COMPANDER_ALAW        (0x01)   /* A-law data format (default) */
#define VCORE_COMPANDER_MULAW       (0x02)   /* mu-law data format */
#define VCORE_COMPANDER_NONE        (0x03)   /* 8-bit data, no companding */

#if (VCORE_BUILD == VCORE_BUILD_VOIPLD1)
/* The following constants are used as the argument to VCORE_cATTACH: */
#define VCORE_ATTACH_DISABLE             (VCORE_DISABLE) /* Detach FXS from FXO */
#define VCORE_ATTACH_ENABLE              (VCORE_ENABLE)  /* Attach FXS to FXO */
#define VCORE_ATTACH_HOLD                (0x02)          /* Detach FXS but leave FXO offhook */
#define VCORE_ATTACH_FLASH_DETECT_ENABLE (0x03)          /* Attach FXS but do not pass hookflash events to FXO */
#endif

/*
 * -------- The following enumerate the commands available for the platform
 *          Host to DSP Commands
 *
 * Unless marked by VoDSL or VoIP, commands are shared between platforms.
 * Low density commands
 */


#if (VCORE_BUILD == VCORE_BUILD_VOIPLD1)

#define VCORE_cNOOP            (0x00)     /* No operation */
#define VCORE_cSHUTDOWN        (0x01)     /* Put FXS port out of service */  
#define VCORE_cIDLE            (0x02)     /* put FXS port into IDLE state */
#define VCORE_cRING            (0x03)     /* ring FXS port */
#define VCORE_cRINGCID         (0x04)     /* ring FXS port with CID type 1 */
#define VCORE_cGENTONE         (0x05)     /* Gen tone to FXS port */
#define VCORE_cGETTONE         (0x06)     /* Get tone from FXS port */  
#define VCORE_cTALK            (0x07)     /* Put FXS port into TALK */
#define VCORE_cTALKATTRIB      (0x08)     /* talk state attributes */
#define VCORE_cCONFADD         (0x09)     /* FXS. conference add */
#define VCORE_cCONFREMOVE      (0x0A)     /* FXS. conference remove */
#define VCORE_cSETTONEDN       (0x0B)     /* Enable/Disable tone to FXS */
#define VCORE_cSETTONEUP       (0x0C)     /* Enable/Disable tone to net */
#define VCORE_cGETLINESTATUS   (0x0D)     /* get DSP FXS line status */
#define VCORE_cSETPHONE        (0x0E)     /* FXS driver cmd */
#define VCORE_cGETDIALTONE     (0x0F)     /* Get dial tone from FXO port */
#define VCORE_cATTACH          (0x10)     /* Attach an FXS and and FXO port */
#define VCORE_cPULSEDIAL       (0x11)     /* pulse dial N-digits to FXO  */
#define VCORE_cDTMFDIAL        (0x12)     /* DTMF dial to FXO */
#define VCORE_cFXOCOMMAND      (0x13)     /* FXO driver cmd */
#define VCORE_cSETPARAM        (0x14)     /* Cmd, Get Version */
#define VCORE_cSETTIME         (0x15)     /* Cmd, Set Time */
#define VCORE_cGETTIME         (0x16)     /* Cmd, Set Time */
#define VCORE_cVERSION         (0x17)     /* Cmd, Get Version */
/*
 * Constants below are not supporetd by DSP. They are here to make host
 * compile 
 */
#define VCORE_cALGCTRL         (0x18)     /* algorithm control */
#define VCORE_cGAINCTRL        (0x19)     /* gain control */
#define VCORE_cSETQUADTONE     (0x1A)     /* VoIP */
#define VCORE_cOUTTONE         (0x1B)
#define VCORE_cPLAY            (0x1C)     /* Reserved for future use */
#define VCORE_cRECORD          (0x1D)     /* Reserved for future use */
/*
 * Commands supporting CID state machine on host
 */
#define VCORE_cFSKS            (0x1E)     /* FSK Send to FXS */
#define VCORE_cFSKR            (0x1F)     /* Enable/Disable FSKR on FXO port */

#else

/* Normal commands */

#define VCORE_cNOOP                (0x00) /* No operation */
#define VCORE_cSHUTDOWN            (0x01) /* Put FXS port out of service */  
#define VCORE_cIDLE                (0x02) /* put FXS port into IDLE state */
#define VCORE_cRESERVED_3          (0x03) /* Custom use */
#define VCORE_cRING                (0x04) /* ring FXS port */
#define VCORE_cRINGCID             (0x05) /* ring FXS port with CID type 1 */
#define VCORE_cGENTONE             (0x06) /* Gen tone to FXS port */
#define VCORE_cGETTONE             (0x07) /* Get tone from FXS port */  
#define VCORE_cTALK                (0x08) /* Put FXS port into TALK */
#define VCORE_cTALKATTRIB          (0x09) /* talk state attributes */
#define VCORE_cCONFADD             (0x0A) /* FXS. conference add */
#define VCORE_cCONFREMOVE          (0x0B) /* FXS. conference remove */
#define VCORE_cRESERVED_0C         (0x0C) /* VoIP */
#define VCORE_cSETTONEDN           (0x0D) /* Enable/Disable tone to FXS */
#define VCORE_cRESERVED_0E         (0x0E) /* Reserved for future use */
#define VCORE_cRESERVED_0F         (0x0F) /* Reserved for future use */
#define VCORE_cSETTONEUP           (0x10) /* Enable/Disable tone to net */
#define VCORE_cGETLINESTATUS       (0x11) /* get DSP FXS line status */
#define VCORE_cSETPHONE            (0x12) /* FXS driver cmd */
#define VCORE_cSETPARAM            (0x1A) /* Cmd, Get Version */
#define VCORE_cSETTIME             (0x1B) /* Cmd, Set Time */
#define VCORE_cGETTIME             (0x1C) /* Cmd, Set Time */
#define VCORE_cRESERVED_1D         (0x1D) /* VoIP */
#define VCORE_cVERSION             (0x1E) /* Cmd, Get Version */
#define VCORE_cRESERVED_1F         (0x1F) /* for future use */
#define VCORE_cRESERVED_20         (0x20) /* for future use */
#define VCORE_cRESERVED_21         (0x21) /* for future use */
#define VCORE_cRESERVED_22         (0x22) /* for future use */
#define VCORE_cALGCTRL             (0x23) /* algorithm control */
#define VCORE_cGAINCTRL            (0x24) /* gain control */
#define VCORE_cSETQUADTONE         (0x25) /* VoIP */
#define VCORE_cOUTTONE             (0x26)
#define VCORE_cPLAY                (0x27) /* Reserved for future use */
#define VCORE_cRECORD              (0x28) /* Reserved for future use */
#define VCORE_cISDN_LAYER1_STATE   (0x29) /* set the ISDN Layer 1 State */ 
#define VCORE_cSET_COMPANDER       (0x2A) /* voice timeslot compander */
#define VCORE_cPROG_FRAMER_REG     (0x2B) /* program framer register */
#define VCORE_cSET_FAXMODEMCODER   (0x2C) /* voice timeslot compander */
#define VCORE_cREAD_FRAMER_REG     (0x2D) /* read framer register */
/*
 * Currently unsupported on non-LD builds
 */
#define VCORE_cFSKS        VCORE_cNOOP    /* FSK Send to FXS */
#define VCORE_cFSKR        VCORE_cNOOP    /* Enable/Disable FSKR on FXO port */


#endif
/* Production Test commands */
#define VCORE_cTSTMEMSTUCKBIT      (0xA1)  /* Stuck bit test */
#define VCORE_cTST8536REG          (0xA2)  /* t8536 register test */
#define VCORE_cTST8536LOOP         (0xA3)  /* t8536 loopback test */
#define VCORE_cTSTMEMADDRBUS       (0xA4)  /* addr bus test */
#define VCORE_cTSTMEM              (0xA5)  /* memory write verify test */
#define VCORE_cTSTCLKR0            (0xA6)  /* serial clk recv test */
#define VCORE_cTSTCLKX0            (0xA7)  /* serial clk xmit test */
#define VCORE_cTSTFSR0             (0xA8)  /* serial fsync recv test */
#define VCORE_cTSTFSX0             (0xA9)  /* serial fsync xmit test */
#define VCORE_cTSTCLKR1            (0xAA)  /* serial clk recv test */
#define VCORE_cTSTCLKX1            (0xAB)  /* serial clk xmit test */
#define VCORE_cTSTFSR1             (0xAC)  /* serial fsync recv test */
#define VCORE_cTSTFSX1             (0xAD)  /* serial fsync xmit test */
#define VCORE_cTSTCLKR2            (0xAE)  /* serial clk recv test */
#define VCORE_cTSTCLKX2            (0xAF)  /* serial clk xmit test */
#define VCORE_cTSTFSR2             (0xB0)  /* serial fsync recv test */
#define VCORE_cTSTFSX2             (0xB1)  /* serial fsync xmit test */
#define VCORE_cTSTDSPCLK           (0xB2)  /* test DSP clock */
#define VCORE_cTSTLOOPBACKMCBSP0   (0xB3)  /* Loopback on McBSP0 */ 
#define VCORE_cTSTLOOPBACKMCBSP1   (0xB4)  /* Loopback on McBSP1 */ 
#define VCORE_cTSTLOOPBACKMCBSP2   (0xB5)  /* Loopback on McBSP2 */
#define VCORE_cTSTDSPGO            (0xB6)  /* Test DSP_GO, BIO signal */
#define VCORE_cTSTDSPINT           (0xB7)  /* Test DSP INT */
#define VCORE_cTSTDSPINT1          (0xB8)  /* Test DSP INT1 */
#define VCORE_cTSTHINT             (0xB9)  /* Test DSP->Host INT */  
#define VCORE_cTSTXF               (0xBA)  /* Test XF pin */
#define VCORE_cTSTGENTONE          (0xBB)  /* Tone generation test */
#define VCORE_cTSTGETTONE          (0xBC)  /* Tone detection test */
#define VCORE_cTSTRINGLINE         (0xBD)  /* SLIC ringing test */
#define VCORE_cTSTGETRMS           (0xBE)  /* Measure Audio level test */
#define VCORE_cTSTTIBVER           (0xBF)  /* Check TIB board version */
#define VCORE_cTSTGENDTMF          (0xC0)  /* Generate cont. DTMF tone */
#define VCORE_cTSTXFHIGH           (0xC1)  /* not used for mds */
#define VCORE_cTSTXFLOW            (0xC2)  /* not used for mds */
#define VCORE_cTSTRECHINT          (0xC3)  /* Int every 5 ms to host*/
#define VCORE_cTSTHPIHRDY          (0xC4)  /* HRDY verification */
/*
 * Commands enumerated through 0xFF are reserved
 */



/*
 * -------- VoIP Attributes
 * valid attribute values for the VPI_talkAttrib()
 * used to modify DSP state attributes.
 * 
 *
 */
#define VCORE_aNOP                (0x00)
#define VCORE_aAUDIOUP            (0x01) /* turn on/off audio upstrm */
#define VCORE_aAUDIODN            (0x02)
#define VCORE_aCODERUP            (0x04) /* change upstrm coder type */
#define VCORE_aDRLYUP             (0x07) /* turn on/off upstrm dtmf relay */
#define VCORE_aDRLYDN             (0x08)
#define VCORE_aSID                (0x09) /* turn on/off upstrm SID */
#define VCORE_aECHOCTRL           (0x0A) /* turn on/off echo control */
#if  (VCORE_BUILD == VCORE_BUILD_ISDN)
#define VCORE_aFAXMODEMAUTOSWITCH (0x0B) /* turn on/off auto switch fax coder */
#else
#define VCORE_aDTMFRM             (0x0B) /* turn on/off DTMF remove alg */
#endif
/*
 * -------- VoIP Algorithm Control
 * The following are passed as arguments to VPI_algCtrl
 *
 */
#define VCORE_ALGCTRL_DCRM     (0)
#define VCORE_ALGCTRL_DTMF     (1)
#define VCORE_ALGCTRL_EC       (2)
#define VCORE_ALGCTRL_FMTD     (3)
#define VCORE_ALGCTRL_GENF     (4)
#define VCORE_ALGCTRL_NFE      (5)
#define VCORE_ALGCTRL_NLP      (6)
#define VCORE_ALGCTRL_NSE      (7)

#define VCORE_WARMINIT         (2)
#define VCORE_COLDINIT         (3)
#define VCORE_ECFREEZE         (4)
#define VCORE_ECBYPASS         (5)
#define VCORE_FMTDTONESEL      (6)
#define VCORE_NLPCTRLSIL       (7)
#define VCORE_NLPCTRLWNS       (8)
#define VCORE_NLPCTRLNFE       (9)
#define VCORE_NLPCTRLANG       (10)

#define VCORE_ALL_CHANNELS     (0xFF)

/*
 * -------- FMTD Tone detection Options
 * The following defines the bits which enable/disable detection of the
 * indicated tones. These are OR'ed together to form the arg1 field
 * corrresponding to the VCORE_FMTDTONESEL mode of the VCORE_ALGCTRL command.
 */
#define VCORE_FMTDSIL            (0x00) /* Tone Detection Disabled */
#define VCORE_FMTDFAXCALLING     (0x01) /* FAX Calling detection enabled */
#define VCORE_FMTDMODEMCALLING   (0x02) /* Modem Calling detection enabled */
#define VCORE_FMTDMODEMANSWERING (0x03) /* Modem Answering detection enabled */
#define VCORE_FMTDPHASEREVERSAL  (0x04) /* Phase Reversal detection enabled */
#define VCORE_FMTDV21PREAMBLE    (0x05) /* V21 Preamble detection enabled */
#define VCORE_FMTD2225HZ         (0x06) /* 2225 Hz detection enabled */
#define VCORE_FMTD2025HZ         (0x07) /* 2025 Hz detection enabled */

/*
 * -------- VoIP Gain controls (Loss Plan Implementation)
 * The following are parameters passed to VPI_volumeLine and VPI_volumeNet
 */
#define VCORE_GAIN_RESET       (0)
#define VCORE_GAIN_LEVEL       (1)
#define VCORE_GAIN_MUTE        (2)
#define VCORE_GAIN_DELTA       (3)

/*
 * -------- VoIP EC Control
 * The following are possible VCORE_aEC_CTRL values
 *
 */
#define VCORE_EC_RUN      (1)
#define VCORE_EC_INIT     (2)
#define VCORE_EC_BYPASS   (3)
#define VCORE_EC_FREEZE   (4)

/*
 * -------- VoIP generic parameters
 */
#define VCORE_UP       (0x00)  /* UP direction. DSP -> Host */
#define VCORE_DN       (0x01)  /* Down direction. Host -> DSP */ 
#define VCORE_PERCONN  (0x00)  /* Generic parameter. Cmd is per connection */ 
#define VCORE_PERPORT  (0x01)  /* Generic parameter. Cmd is per port */ 
#define VCORE_ENABLE   (0x01)  /* generic. Turn something on */
#define VCORE_DISABLE  (0x00)  /* generic use. */

/*
 * -------- VoIP Caller Id types
 * Caller ID types used to distinguise type 1 onhook from type 2 offhook
 */
#define VCORE_CALLER_ID_ONHOOK           (0)
#define VCORE_CALLER_ID_OFFHOOK          (1)
#define VCORE_MWI_ONHOOK                 (2)
#define VCORE_CALLER_ID_OFFHOOK_PREAMBLE (3)
#define VCORE_CALLER_ID_OFFHOOK_TRAILER  (4)

#define VCORE_CIDS1_DURING_RING         (1)
#define VCORE_CIDS1_PRIOR_RING_BATT_REV (2)
#define VCORE_CIDS1_PRIOR_RING_OTHER    (3)
#define VCORE_CIDS2                     (4)
#define VCORE_CIDSMWI                   (5)
#define VCORE_CIDS_NONE_CONFIGURED      (6)

/*
 * -------- The following enumerate the events available for the platform
 * DSP to Host Events (sGeneral Status Codes)
 * These are assigned to sGeneral in the status message
 * structure.
 *
 * Set VCORE_SEND_CMDACK to 1 if CMDCOMPLETE/ERROR status messages are
 * desired, e.g. for debug purposes.  
 * WARNING:  Setting VCORE_SEND_CMDACK to 1  may lead to status message queue
 * overflow if too many Commands/modifiers are sent in a single 5ms loop. Msg
 * queue overflow may cause the loss of status messages, some of which may be 
 * tone and line events.
 * It is recommended that VCORE_SEND_CMDACK be set to 0 by default.
 */

#define VCORE_DSP_SENDS_CMDACK     (0) /* DSP sends CMDCOMPLETE/ERROR status*/

#if (VCORE_BUILD == VCORE_BUILD_VOIPLD1)

#define VCORE_sCLOSE               (0)  /* Reserved for future use */
#define VCORE_sCMDCOMPLETE         (1)  /* DSP reports command completion */
#define VCORE_sERROR               (2)  /* DSP reports error condition */
#define VCORE_sRESERVED1           (3)
#define VCORE_sLINE                (4)
#define VCORE_sLINEFXS             (VCORE_sLINE)
#define VCORE_sLINEFXO             (5)  /* DSP reports FXS line status */
#define VCORE_sTONE                (6)  /* DSP reports detected tone */
#define VCORE_sTONEFXS             (VCORE_sTONE) /* DSP reports detected tone */
#define VCORE_sVERSION             (7)  /* DSP version information */
#define VCORE_sLINESTATUS          (8)  /* DSP answers FXS line status query */
#define VCORE_sCIDSDONE            (9)  /* DSP reports CIDS processing over */
#define VCORE_sTONEDNDONE          (10) /* DSP reports tone2line over  */
#define VCORE_sTONEUPDONE          (11) /* DSP reports tone2net over  */
#define VCORE_sFR38                (12) /* DSP reports FR38 event   */
#define VCORE_sDIALDONE            (13) /* A digit sequence dial is done */
#define VCORE_sGEN_RESTART_COMPLETE (14)
#define VCORE_sJITTER               (20) /* Fake event used on host only */
#define VCORE_sFSKSDONE             (21) /* DSP reports FSK send complete */
#define VCORE_sFSKRDATA             (22) /* DSP reports FSK data received */

#elif  (VCORE_BUILD == VCORE_BUILD_ISDN)

#define VCORE_sCLOSE               (0)  /* Reserved for future use */
#define VCORE_sCMDCOMPLETE         (1)  /* DSP reports command completion */
#define VCORE_sERROR               (2)  /* DSP reports error condition */
#define VCORE_sRESERVED1           (3)
#define VCORE_sRESERVED2           (4)
#define VCORE_sLINE                (5)  /* DSP reports a line status event */
#define VCORE_sTONE                (6)  /* DSP reports detected tone */
#define VCORE_sVERSION             (7)  /* DSP version information */
#define VCORE_sLINESTATUS          (8)  /* DSP answers a line status query */
#define VCORE_sCIDSDONE            (9)  /* DSP reports CIDS processing over */
#define VCORE_sTONEDNDONE          (10) /* DSP reports tone2line over  */
#define VCORE_sTONEUPDONE          (11) /* DSP reports tone2net over  */
#define VCORE_sFR38                (12) /* DSP reports FR38 event   */
#define VCORE_sISDN_LAYER1         (13) /* DSP reports ISDM Layer 1 event */ 
#define VCORE_sJITTER              (14) /* BUN reports jitter event to POTS */ 
#define VCORE_sREAD_FRAMER_REG     (15) /* DSP report Framer register value */ 
 
#else

#define VCORE_sCLOSE         (0)  /* Reserved for future use */
#define VCORE_sCMDCOMPLETE   (1)  /* DSP reports command completion */
#define VCORE_sERROR         (2)  /* DSP reports error condition */
#define VCORE_sRESERVED1     (3)  /* Reserved for custom use */
#define VCORE_sRESERVED2     (4)  /* Reserved for custom use */
#define VCORE_sLINE          (5)  /* LEGACY CONSTANT. FXS line status */
#define VCORE_sLINEFXS       (VCORE_sLINE)    /* FXS line status */
#define VCORE_sLINEFXO       (6)  /* DSP reports FXO line status */
#define VCORE_sTONE          (7)  /* LEGACY CONSTANT. Detected tone */
#define VCORE_sTONEFXS       (VCORE_sTONE) /* DSP reports detected tone */
#define VCORE_sVERSION       (8)  /* DSP version information */
#define VCORE_sLINESTATUS    (9)  /* DSP answers FXS line status query */
#define VCORE_sCIDSDONE      (10)  /* DSP reports CIDS processing over */
#define VCORE_sTONEDNDONE    (11) /* DSP reports tone2line over  */
#define VCORE_sTONEUPDONE    (12) /* DSP reports tone2net over  */
#define VCORE_sFR38          (13) /* DSP reports FR38 event   */
#define VCORE_sDIALDONE      (14) /* A digit sequence dial is done */
#define VCORE_sISDN_LAYER1   (15) /* DSP reports ISDM Layer 1 event */ 
#define VCORE_sJITTER        (16) /* Fake event used on host only */
#define VCORE_sREAD_FRAMER_REG     (17) /* DSP report Framer register value */
#define VCORE_sFSKSDONE             (21) /* DSP reports FSK send complete */
#define VCORE_sFSKRDATA             (22) /* DSP reports FSK data received */
#define VCORE_sGEN_RESTART_COMPLETE (30) /* dummy value that will never be sent */
#endif

/*
 * -------- VCORE_sJITTER parameters
 * Used on host only.
 */
#define VCORE_JITTERIDLE           (1)  /* Fake event used on host only */


/* The following will be used in conjunction with the new GenTone */
#define VCORE_MONO_TONE   (0x0000) /* single tone */
#define VCORE_DUAL_TONE   (0x0000) /* dual tone */
#define VCORE_TRI_TONE    (0x0000) /* triple tones */
#define VCORE_QUAD_TONE   (0x0000) /* quad tone */
#define VCORE_MOD_TONE    (0x4400) /* amplitude modulated tones */
#define VCORE_SIT_TONE    (0x1230) /* Special Information tones */
#define VCORE_BONG_TONE   (0x12BB) /* Bong tones */
#define VCORE_DECAY_TONE  (0x8888) /* decaying tones */
/*
 * Example decay time constant values 
 */
#define VCORE_DECAY_50MS   (26828)
#define VCORE_DECAY_100MS  (29650)
#define VCORE_DECAY_200MS  (31170)
#define VCORE_DECAY_300MS  (31694)
#define VCORE_DECAY_500MS  (32129)
#define VCORE_DECAY_1000MS (32442)

/* -------- VoIP sArg0's Line Status
 * Legal values for sArg0 for DSP_sLINESTATUS
 */
#define VCORE_OFFHOOK          (1)
#define VCORE_ONHOOK           (0)
 
/*
 * -------- Event Tone Code Masks for DSP_sTONE Event
 */
#define VCORE_sTONE_LE_MASK         (0x8000)
#define VCORE_sTONE_TE_MASK         (0x4000)
#define VCORE_sTONE_CP_MASK         (0x3000)
#define VCORE_sTONE_TYPE_MASK       (0x0F00)
#define VCORE_sTONE_RETCODE_MASK    (0x00FF)
/* Values to mask in CP field of sArg0 in DSP_sTONE Event */
#define VCORE_sTONE_CP_DTMF         (0x0000)
#define VCORE_sTONE_CP_FMTD_TOFXS   (0x2000)
#define VCORE_sTONE_CP_FMTD_FROMFXS (0x3000)
/* changes for v21 flag */
#ifdef VOICE_T38_FAX_SUPPORT
#define PTMS_V21_FLAG_TOFXS         (0x0002)
#define PTMS_V21_FLAG_FROMFXS       (0x0003)
#endif


//#define VCORE_sTONE_TYPE_MASK     (0x03FF) /* tone type mask */

/*
 * -------- Event sArg0's for sFR38 event
 * Various FR38 related events
 */
/*
 * Arrival of far end (downstream) FR38 pkts before FR38 is 
 * turned on locally (upstream).
 */
#define VCORE_sFR38_FAREND_PKT         (0) 
/* 
 * Local FR38 module returns DISCONNECT. Normal end of FAX
 */
#define VCORE_sFR38_DISCONNECT         (1)
/*
 * Miscellaneous error situations.
 */
#define VCORE_sFR38_NOT_FAX            (2)
#define VCORE_sFR38_NSS_UNSUPPORTED    (3)
#define VCORE_sFR38_FRAMING_LOSS       (4)

/*
 * -------- Event sArg0's for sERROR event
 */
#define VCORE_sERROR_BAD_CMD (1)
#define VCORE_sERROR_PARAM   (2)

/*
 * -------- Set Parameter Types
 *  These define markers for dynamic updating of algorithm parameters
 */
#define VCORE_PARAM_GLOBAL         (0x0A00)    /* VPO global parameter type */
#define VCORE_PARAM_UTD            (0x0B00)    /* UTD parameter type */
#define VCORE_PARAM_DTMF           (0x0C00)    /* DTMF parameter type */
#define VCORE_PARAM_NFE            (0x0D00)    /* NFE parameter type */
#define VCORE_PARAM_VAGC           (0x0E00)    /* VAGC parameter type */
#define VCORE_PARAM_FSKS           (0x1000)    /* FSKS parameter type */
#define VCORE_PARAM_EC             (0x1400)    /* VoDSL Echo canc. Control */
#define VCORE_PARAM_PLC            (0x1500)    /* VoDSL PLC control */
#define VCORE_PARAM_CIDS           (0x2000)    /* CAS parameter type  */
#define VCORE_PARAM_SID            (0x1100)    /* SID parameter type */

/*
 * -------- Test Mode parameters
 *   TSTMODE_REPEAT: Test repeats (if possible)
 *   TSTMODE_VOICE: Test command will run the audio driver
 */
#define VCORE_TSTMODE_REPEAT        (0x0010)
#define VCORE_TSTMODE_VOICE         (0x0020)

/*
 * -------- Transparent mode
 * The following constants define parameters for the transparent mode.  
 */
#define VCORE_FXSSTAT_TO_FXOCMD (0x0002)
#define VCORE_FXOSTAT_TO_FXSCMD (0x0300)

/*
 * -------- Event sArg0's for sLINE event
 * Legal values for sArg0 for DSP_sLINE status
 */
#define VCORE_SEIZE       (0x0001) /* FXS: term equip picks up line */
#define VCORE_RELEASE     (0x0002) /* FXS: term equip releases the line */

#define VCORE_RINGMAKE    (0x0003) /* FXS: beginning to generate ring cadence */
#define VCORE_RINGBREAK   (0x0004) /* FXS: end of one ring's cadence */
#define VCORE_RINGDONE    (0x0005) /* FXS: end of all rings requested */
#define VCORE_FLASHDETECT (0x0006) /* FXS: flash detect from termination */
#define VCORE_PULSEDIGIT  (0x0007) /* FXS: pulse digit detected */
#define VCORE_NO_EVENT    (0x0008) /* No line event to report */
#define VCORE_FXS_ERROR   (0x0009) /* FXS: missed a status message within 5ms */
/* 
 * The following are used on host only
 */
#define VCORE_CID_SEND    (0x000A) /* FXS: started sending CID Data */
#define VCORE_CAS_RECV    (0x000B) /* FXO: started receiving cas signal */
#define VCORE_MWI_SEND    (0x000C) /* FXS: started receiving MWI data */

/*
 * -------- Command subtypes for FXO
 * VCORE_cFXOCOMMAND (FXO Driver Control)
 */
#define VCORE_FXO_cOFFHOOK     (0x0001) /* go line side OFFHOOK */
#define VCORE_FXO_cONHOOK      (0x0002) /* go line side ONHOOK */

#define VCORE_FXO_cFLASH       (0x0003) /* send a flash signal */
#define VCORE_FXO_cPULSEDIAL   (0x0004) /* dial one digit + interdigit time */
#define VCORE_FXO_cREAD_LV     (0x0005) /* Read Line Volt, on or off hook */

/*
 * -------- Command subtypes for FXS
 * VCORE_cSETPHONE (FXS Driver Control)
 */
#define VCORE_RINGSTARTCUSTOM  (0x0100)  /* FXS Begin custom cadence */
#define VCORE_RINGSTOP         (0x0200)  /* FXS Stop the ring generator */
#define VCORE_BAT_FLIP         (0x0300)  /* FXS Set Battery Polarity */
#define VCORE_BAT_FOR          (0x0400)  /* FXS Set Battery Polarity */
#define VCORE_BAT_REV          (0x0500)  /* FXS Set Battery Polarity */
#define VCORE_BAT_OFF          (0x0600)  /* FXS Set Battery Polarity */
#define VCORE_RINGSTART1       (0x0700)  /* FXS Begin Bellcore Ring Pattern 1 */
#define VCORE_RINGSTART2       (0x0800)  /* FXS Begin Bellcore Ring Pattern 2 */
#define VCORE_RINGSTART3       (0x0900)  /* FXS Begin Bellcore Ring Pattern 3 */
#define VCORE_RINGSTART4       (0x0a00)  /* FXS Begin Bellcore Ring Pattern 4 */
#define VCORE_RINGSTART5       (0x0b00)  /* FXS Begin Bellcore Ring Pattern 5 */
#define VCORE_PROGCADENCE      (0x0c00)  /* FXS Program a custom cadence */
#define VCORE_SETRELEASETIME   (0x0d00)  /* Set tRELEASE parameter */
#define VCORE_SETMINFLASHTIME  (0x0e00)  /* Set tMINFLASH parameter */
#define VCORE_SETSEIZETIME     (0x0f00)  /* Set tSEIZE parameter */

/*
 * -------- Event sArg0's for sLINE event
 * Legal values for sArg0 for DSP_sLINEFXO status
 */
#define VCORE_FXO_sRINGMAKE    (0x0100)  /* ring min make event */
#define VCORE_FXO_sRINGBREAK   (0x0200)  /* ring min break event */
#define VCORE_FXO_sBATREV      (0x0300) /* Reverse Battery detected */

#define VCORE_FXO_sLCMAKE      (0x0400)  /* LC make event */
#define VCORE_FXO_sLCBREAK     (0x0500)  /* LC break event */
#define VCORE_FXO_sLVHIGH      (0x0600)  /* Line voltage is high */
#define VCORE_FXO_sLVLOW       (0x0700)  /* Line voltage is low */
#define VCORE_FXO_sDIALDONE    (0x0800)  /* pulse dialer finished a digit */
#define VCORE_FXO_sFLASHDONE   (0x0900)  /* flash signal completed */
#define VCORE_FXO_sRINGDONE    (0x0a00)  /* ring max break event */
#define VCORE_FXO_sDIALTONE    (0x0b00)  /* dial tone detected */

#if (VCORE_BUILD == VCORE_BUILD_VOIPLD1)
#define VCORE_FXO_sNO_EVENT    (0x0c00)  /* no status event to report */
#else
#define VCORE_FXO_sNO_EVENT    (VCORE_NO_EVENT)  /* no status event to report */
#endif

#define VCORE_FXO_sERROR       (0x0d00)  /* getEvent missed status message */
#define VCORE_FXO_sLVNONE      (0x0e00) /* Line voltage is low */

/*
 * -------- Timing Parameters for control of the FXS ports
 * The following timing parameters are in TIC driver timing units.
 * Multiply by 5ms to convert to timing in units of seconds.
 */
#define VCORE_FXS_tSEIZE         (20)  /* 0.100 sec */
#define VCORE_FXS_tRELEASE       (200) /* 1.0   sec */
/* Flash detection can be disabled if tMINFLASH > tRELEASE */
#define VCORE_FXS_tMINFLASH      (60)  /* 0.3 sec */
#define VCORE_FXS_tMINPULSEBREAK (4)   /* 0.020 sec */
#define VCORE_FXS_tMINPULSEMAKE  (2)   /* 0.010 sec */
#define VCORE_FXS_tMINPULSEPAUSE (80)  /* 0.400 sec */
#define VCORE_FXS_tSR_TIMEOUT    (1)   /* 0.005 sec, smallest unit */


/*
 * -------- ISDN TIC Driver 
 * The constants below define commands for the control of the ISDN layer 1
 * ST interface LT-S mode State Transition Diagram.
 */
#define VCORE_cST_RESET_REQ               (0x01)  /* Place ST Bus into RESET */
#define VCORE_cST_LOOPBACK_REQ            (0x02)  /* LOOPBACK for 2B + D */
#define VCORE_cST_ACTIVATE_REQ            (0x03)  /* ACTIVATE the ST Bus */
#define VCORE_cST_DEACTIVATE_REQ          (0x04)  /* DEACTIVATE the ST Bus */
#define VCORE_cST_TESTMODE_SSP_REQ        (0x05)  /* TESTMODE single pulses */
#define VCORE_cST_TESTMODE_SCP_REQ        (0x06)  /* TESTMODE continous pulses */

/*
 * The constants below define status from the ISDN layer 1 ST interface 
 * LT-S mode State Transition Diagram.
 */
#define VCORE_sST_ACTIVATE_CONF           (0x0a)
#define VCORE_sST_DEACTIVATE_CONF         (0x0b)
#define VCORE_sST_ACTIVATE_IND            (0x0c)
#define VCORE_sST_LOST_FRAMING_IND        (0x0d)
#define VCORE_sST_ACTIVATE_PEND_IND       (0x0e)

#endif /* _VCORE_CMD_H */
