/* ###########################################################################
# (c) Copyright Conexant Systems, Inc. 2005
# (c) Copyright GlobespanVirata, Inc., 1997-2005
#
# Conexant Systems, Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc. and its subsidiaries (collectively
# "Conexant"). It may only be read, used, copied, adapted, modified or
# otherwise dealt with by you if you have entered into a license agreement
# with Conexant and then subject to the terms of that agreement and any other
# applicable confidentiality agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a license agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/
/** @file    lvoice_bif.c
 *  @brief   voice driver bun interface
 *  @ingroup linuxvoice
 */
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/module.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <linux/kmod.h>
#include <asm/uaccess.h>

#include "pots.h"
#include "pots_sig_ring.h"
#include "vcore.h"
#include "bun_channel.h"
#include "lvoice_common.h"
#include "lvoice_ioctl.h"
#include "lvoice_driver.h"

extern tLVOICE_Object *pLVOICE_Object; 
int voice_memory_audio_release_count=0;
/**
 * Function for  receiving event 
 * @param  pCyan        pointer to message
 * @param  pUserEventId pointer to user event id supplied in POTS_InitAnPots_Cmd
 * @retun  none
  */
void LVOICE_statCallBack(CYANBUFFER *pCyan, void *pUserEventId)
{
   char *pData = pCyan->data;
   unsigned int len = pCyan->length;
   CYANBUFFER *pBuffer;
   unsigned char i;

   DEBUG_MACRO("Entering %s eventId %x  \n", __FUNCTION__, pUserEventId);
   if ( !pLVOICE_Object || !pLVOICE_Object->initialized )
   {
      DEBUG_MACRO(" %s driver is not yet ready \n", __FUNCTION__, pUserEventId);
      cyan_bufRelease(pCyan);
      return;
   }
   if ( (*pData == kPOTS_DTMF_DIGIT_DETECTED_LEADING_EDGE)|| (*pData ==kPOTS_JITTER_IDLE ))
   {/* ignore this event as it will be reported as kPOTS_DTMF_DIGIT_DETECTED and JITTER_IDLE is not required for application */
     cyan_bufRelease(pCyan);
     return;
   }
   i = pLVOICE_Object->pEventHandler[0]==pUserEventId ? 0: 1;

   pCyan->status = (int)pUserEventId;

  /* Since RD 376 is single processor board so and data is being accessed by
     BUN process and a application, no need to protect it as internal access is RR
     priority */

  /* save the message in message queue and wakeup the process if anyone is
   waiting */
    
   cyan_qPut(&pLVOICE_Object->eventQueue[i], pCyan);
   wake_up_interruptible(&pLVOICE_Object->waitQueue[LVOICE_EVENT_WAIT_QUEUE_ID] ); 
} 

/**
 * Function for  receiving audio for port
 * @param  pCyan        pointer to message
 * @param  audioId      pointer to userId given in POTS_AddUpStreamAudio
 * @retun  none 
  */

void LVOICE_audioCallBack(CYANBUFFER *pCyan, void *pAudio )
{
   unsigned char i=0;
   CYANBUFFER     *pBuffer;
   DEBUG_MACRO("Entering %s \n", __FUNCTION__);
   if ( !pLVOICE_Object || !pLVOICE_Object->initialized )
   {
      DEBUG_MACRO(" %s driver is not yet ready \n", __FUNCTION__);
      cyan_bufRelease(pCyan);
      return;
   }
   for (i=0; i < LVOICE_MAX_PORTS; i++)
        if ( pLVOICE_Object->pAudioHandler[i] == pAudio ) break;

   if ( i >= LVOICE_MAX_PORTS )
   {
     cyan_bufRelease(pCyan);
     return;
   }
   /* check if  any user has done a read on this in last 100 ms ,
      if not then flush the audio queue as audio sample will be off no use */
   if ( jiffies - lvoice_read_audio_jiffies >= HZ   )
   {
     cyan_bufRelease(pCyan);
     while(pLVOICE_Object->audioQueue[i].head )
     {
       if ((pCyan=cyan_qGet(&pLVOICE_Object->audioQueue[i]) ) !=NULL )
           cyan_bufRelease(pCyan);
     }
     lvoice_read_audio_jiffies=jiffies; /* Assusme that reading is done */
     return;
   }
   /* Stored  audioId in  status field */
   pCyan->status = (int)pAudio;
   cyan_qPut(&pLVOICE_Object->audioQueue[i], pCyan);
   wake_up(&pLVOICE_Object->waitQueue[LVOICE_MINOR_FXS0_DEV + i ]); 
   wake_up(&pLVOICE_Object->waitQueue[LVOICE_AUDIO_WAIT_QUEUE_ID ]); 
}

/**
 * Function for copying internal cyan buffer to user space. Used for event and audio
 * @param  ioctlCmd     Event or Audio cmd
 * @param  pUsrBuffer   pointer to user buffer
 * @param  pCyanBuffer  pointer to internal cyan buffer 
 * @retun  length of data copied 
  */
int  LVOICE_copyCyanToUser(unsigned int ioctlCmd, char *pUsrBuffer, CYANBUFFER *pCyanBuffer)
{
  unsigned int userIndex=0, length;
  char *pData = pCyanBuffer->data;
  DEBUG_MACRO("Entering %s ioctl %x\n",__FUNCTION__,ioctlCmd);
  /* IOCTL cmd */
  __copy_to_user(pUsrBuffer, (void *)&ioctlCmd, sizeof (ioctlCmd));
  userIndex+=sizeof (ioctlCmd);
  __copy_to_user(pUsrBuffer+userIndex, (void *)&(pCyanBuffer->parameter1),
                sizeof (pCyanBuffer->parameter1));
  userIndex+=sizeof (pCyanBuffer->parameter1);
  __copy_to_user(pUsrBuffer+userIndex, (void *)&(pCyanBuffer->parameter2), 
               sizeof (pCyanBuffer->parameter2));
  userIndex+=sizeof (pCyanBuffer->parameter2);
  __copy_to_user(pUsrBuffer+userIndex, (void *)&(pCyanBuffer->status),
               sizeof (pCyanBuffer->status));
  userIndex+=sizeof (pCyanBuffer->status);
  length =  pCyanBuffer->length;
  __copy_to_user(pUsrBuffer+userIndex, (void *)&(length), sizeof (length));
  userIndex+=sizeof (length);
  __copy_to_user (pUsrBuffer+userIndex, pData, length);
  return (20 + length);
}

/**
 * Function for copying user audio to internal 
 * @param  pUsrBuffer   pointer to user buffer
 * @param  pCyanBuffer  pointer to internal cyan buffer 
 * @retun  length of data copied 
 */
int  LVOICE_copyAudioUserToCyan(char *pUsrBuffer, CYANBUFFER *pCyanBuffer)
{
  unsigned int userIndex=0, length;
  char *pData = pCyanBuffer->data;

  DEBUG_MACRO("Entering %s \n", __FUNCTION__);
  __copy_from_user((void *)&(pCyanBuffer->parameter1), pUsrBuffer+userIndex,
                  sizeof (pCyanBuffer->parameter1));
  userIndex+=sizeof (pCyanBuffer->parameter1);
  __copy_from_user( (void *)&(pCyanBuffer->parameter2),pUsrBuffer+userIndex,
                  sizeof (pCyanBuffer->parameter2));
  userIndex+=sizeof (pCyanBuffer->parameter2);
  __copy_from_user((void *)&(pCyanBuffer->status),pUsrBuffer+userIndex, 
                 sizeof (pCyanBuffer->status));
  userIndex+=sizeof (pCyanBuffer->status);

  __copy_from_user((void *)&(pCyanBuffer->length),pUsrBuffer+userIndex,
                  sizeof (pCyanBuffer->length));
  userIndex+=sizeof (pCyanBuffer->length);
  length = pCyanBuffer->length;
  __copy_from_user (pData, pUsrBuffer+userIndex, length);
  return (20 + length);
}

void lvoice_DumpCyan(CYANBUFFER *pCyan)
{
  unsigned int i;

  printk("length %x \n", pCyan->length);
  printk("param1 %x \n",pCyan->parameter1);
  printk("param2 %x \n", pCyan->parameter2);
  printk("status %x \n",pCyan->status);
   for (i=0; i < pCyan->length; i++)
   {
     printk("%x ",*(pCyan->data + i ));

   }
   printk("\n");
}
