/* ###########################################################################
# (c) Copyright Conexant Systems, Inc. 2005
# (c) Copyright GlobespanVirata, Inc., 1997-2005
#
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc. and its subsidiaries (collectively
# "Conexant"). It may only be read, used, copied, adapted, modified or
# otherwise dealt with by you if you have entered into a license agreement
# with Conexant and then subject to the terms of that agreement and any other
# applicable confidentiality agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a license agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/

/* Linux Voice Driver  definition */

#ifndef __LVOICE_DRIVER_H__
#define __LVOICE_DRIVER_H__

/* Major and Minor number */
#define LVOICE_MAJOR   200  /* This major number will come from common file */
#define LVOICE_CHAR_NAME "cnxtvoice"
#define DEV_MINOR_ID(file) MINOR(file->f_dentry->d_inode->i_rdev)

typedef enum
{
  LVOICE_MINOR_SIGNALLING_DEV,      /* For control purpose of slic ports, relays */
  LVOICE_MINOR_AUDIO_DEV,       /* For Audio */
  LVOICE_MINOR_FXS0_DEV,       /* For FXS0 port */
  LVOICE_MINOR_FXS1_DEV,       /* For FXS1 port */
  LVOICE_MINOR_MAX_DEVS
}eLVOICE_MINOR_DEVS;

#define LVOICE_MAX_PORTS   2 
#define LVOICE_EVENT_WAIT_QUEUE_ID LVOICE_MINOR_SIGNALLING_DEV
#define LVOICE_AUDIO_WAIT_QUEUE_ID LVOICE_MINOR_AUDIO_DEV
#define LVOICE_AUDIO_WAIT_FXS0_QUEUE_ID LVOICE_MINOR_FXS0_DEV  /* For asterisk only */
#define LVOICE_AUDIO_WAIT_FXS1_QUEUE_ID LVOICE_MINOR_FXS1_DEV  /* for asterisk only */
#define LVOICE_MAX_DEVS  2 /* two minor devices, one for signalling and another for audio */


typedef void (*pfLvoiceTxCB)(CYANBUFFER* pCyanBuf, void* pUserId);
/* data maintained by driver for all ports */

#define LVOICE_POOL_NAME  "linuxvoice"
typedef struct sLVOICE_Object
{
  /* Event Queue for both FXS ports and FXO */
  unsigned int            *pEventHandler[LVOICE_MAX_PORTS]; /* upstream audio handler in upstream call */
  CYANQUEUE               eventQueue[LVOICE_MAX_PORTS];
  /* Audio queue for both fxs ports only */
  unsigned int            *pAudioHandler[LVOICE_MAX_PORTS]; /* upstream audio handler in upstream call */
  CYANQUEUE               audioQueue[LVOICE_MAX_PORTS];
 /* Two wait queues, one for event and another for audio*/     
  wait_queue_head_t       waitQueue[LVOICE_AUDIO_WAIT_FXS1_QUEUE_ID + 1];      
/* Pool for sending audio packets */
  CYANPOOL                *pAudioPool;
  /* whether driver is initialized or not */   
  BOOL                    initialized; 
  /* Upstream subchannel info, used in POTS_RemoveUpstreamAudio_Cmd */
  unsigned int            upStreamSubChannel[LVOICE_MAX_PORTS];
  /* hPhone info for disconnect call */
  unsigned int           hPhone[LVOICE_MAX_PORTS];
                                                    
}tLVOICE_Object;

extern char lvoice_debug;
extern int lvoice_read_audio_jiffies;
#define DEBUG_MACRO   if (lvoice_debug) printk
typedef struct sLvoiceCmdIdtoName
{
  unsigned int  cmd;
  char          name[100];

} tLvoiceCmdIdtoName;
void LVOICE_statCallBack(CYANBUFFER *pCyan, void *eventId);
void LVOICE_audioCallBack(CYANBUFFER *pCyan, void *audioId);
int  LVOICE_copyCyanToUser(unsigned int ioctlCmd, char *pUsrBuffer, CYANBUFFER *pCyanBuffer);
int  LVOICE_copyAudioUserToCyan(char *pUsrBuffer, CYANBUFFER *pCyanBuffer);

int  LVOICE_DriverInit(void);
void lvoice_DumpCyan(CYANBUFFER *pCyan);
#endif /* __LVOICE_DRIVER_H__ */
