/* ###########################################################################
# (c) Copyright Conexant Systems, Inc. 2005
# (c) Copyright GlobespanVirata, Inc., 1997-2005
#
# Conexant Systems, Inc. Confidential and Proprietary
#
# The following software source code ("Software") is strictly confidential and
# is proprietary to Conexant Systems, Inc. and its subsidiaries (collectively
# "Conexant"). It may only be read, used, copied, adapted, modified or
# otherwise dealt with by you if you have entered into a license agreement
# with Conexant and then subject to the terms of that agreement and any other
# applicable confidentiality agreement between you and Conexant.
#
# If you are in any doubt as to whether you are entitled to access, read,
# use, copy, adapt, modify or otherwise deal with the Software or whether you
# are entitled to disclose the Software to any other person you should
# contact Conexant.  If you have not entered into a license agreement
# with Conexant granting access to this Software you should forthwith return
# all media, copies and printed listings containing the Software to Conexant.
#
# Conexant reserves the right to take legal action against you should you
# breach the above provisions.
#
# If you are unsure, or to report violations, please contact
# licensee.support@conexant.com
# ##########################################################################*/
/** @file    lvoice_main.c
 *  @brief   voice driver main module
 *  @ingroup linuxvoice
 */

#include <asm/atomic.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/kmod.h>
#include <linux/time.h>
#include <linux/spinlock.h>
#include <linux/signal.h>
#include <linux/poll.h>
#include <linux/fs.h>
#include <asm/uaccess.h>

#include "pots.h"
#include "pots_sig_ring.h"
#include "ip_gencmd.h"
#include "bun_channel.h"
#include "lvoice_common.h"
#include "lvoice_ioctl.h"
#include "lvoice_driver.h"
#include "voice_common.h"


static int LVOICE_open (struct inode *pInode, struct file *pFile);
static int LVOICE_release (struct inode *pInode, struct file *pFile);
static int LVOICE_ioctl ( struct inode *inode, struct file *pFile, unsigned int
                          cmd, unsigned long arg);
static ssize_t LVOICE_read(struct file *pFile, char *uBuffer, size_t len, 
                                                              loff_t *pLoff);
static ssize_t LVOICE_write(struct file *pFile, const char *uBuffer, size_t len,
                                                                     loff_t *pLoff);
static  unsigned int LVOICE_poll ( struct file * pFile, poll_table *wait );
static void LVOICE_DriverExit (void );

char lvoice_debug= 0;
int lvoice_read_audio_jiffies=0;
/* local since USE_COUNT is not supported in 2.6*/
static unsigned char module_use[LVOICE_MAX_DEVS];

static struct file_operations lvoice_fops = 
{
   owner:    THIS_MODULE,
   llseek:   NULL, 
   open:     LVOICE_open,
   release:  LVOICE_release,
   ioctl:    LVOICE_ioctl,
   read:     LVOICE_read,
   write:    LVOICE_write,
   poll:     LVOICE_poll,
   flush:    NULL,
   mmap:     NULL,
   fsync:    NULL,
   fasync:   NULL,
   aio_read: NULL,
   aio_write:NULL,
   readdir:  NULL,
   aio_fsync: NULL,
   lock:      NULL,
   readv:     NULL,
   writev:    NULL,
   sendfile:  NULL,
   sendpage:  NULL,
   get_unmapped_area: NULL,
   check_flags: NULL,
   dir_notify:  NULL,
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11))
   flock:       NULL, 
   unlocked_ioctl: NULL,
   compat_ioctl:    NULL, 
#endif
};
CYANCBF    *pfTxCall=NULL;

tLVOICE_Object *pLVOICE_Object; 
#define LVOICE_MAX_CMD    22
tLvoiceCmdIdtoName lvoiceCmdtoName[LVOICE_MAX_CMD]=
{ 
 { POTS_InitPots_Cmd, "POTS_InitPots_Cmd"},  /* 0 */
 { POTS_StartSignalAnPhone_Cmd, "POTS_StartSignalAnPhone_Cmd"},
 { POTS_StartRingAnPhone_Cmd, "POTS_StartRingAnPhone_Cmd"},
 { POTS_StopSignalAnPhone_Cmd, "POTS_StopSignalAnPhone_Cmd" },
 { POTS_CallerIdAnPhone_Cmd, "POTS_CallerIdAnPhone_Cmd"},
 { POTS_ConnectPhone_Cmd, "POTS_ConnectPhone_Cmd" },
 { POTS_AddUpstreamAudio_Cmd, "POTS_AddUpstreamAudio_Cmd" },
 { POTS_AddAudioPhone_Cmd, "POTS_AddAudioPhone_Cmd" },
 { POTS_DisconnectPhone_Cmd, "POTS_DisconnectPhone_Cmd" }, 
 { POTS_ModifyPhone_Cmd,"POTS_ModifyPhone" },
 { POTS_SetFaxCoderName_Cmd, "POTS_SetFaxCoderName_Cmd"}, /* 10 */
 { POTS_EnableAudioMixing_Cmd, "POTS_EnableAudioMixing_Cmd" },
 { POTS_InitAnPhone_Cmd, "POTS_InitAnPhone_Cmd"},
 { POTS_RemoveUpstreamAudio_Cmd, "POTS_RemoveUpstreamAudio_Cmd" },
 { POTS_FailSafePots_Cmd, "POTS_FailSafePots_Cmd"},
 { POTS_AddDelDCM_Cmd, "POTS_AddDelDCM_Cmd" },
 { POTS_GetPstnStatus_Cmd, "POTS_GetPstnStatus_Cmd"},
 { POTS_FunctionReturn_Cmd, "POTS_FunctionReturn_Cmd"},
 { POTS_Event_Cmd, "POTS_Event_Cmd"},
 { POTS_Audio_Cmd, "POTS_Audio_Cmd"},
 { POTS_RemoveAudioPhone_Cmd, "POTS_RemoveAudioPhone_Cmd"}, /* 20 */
 { POTS_StopRingAnPhone_Cmd, "POTS_StopRingAnPhone_Cmd"}, /* 21 */
};

/**
 * Function for  Driver initialization  
 *
 * @retun  success or error
 */

int  LVOICE_DriverInit(void)
{
   int           result=0;
   unsigned char i=0;

   DEBUG_MACRO("Entering %s\n",__FUNCTION__);
   if ( (pLVOICE_Object = (tLVOICE_Object *) kmalloc( sizeof (tLVOICE_Object),
                                                       GFP_KERNEL)) == NULL )
   {
      printk(KERN_ERR "%s can not allocate voice object \n",__FUNCTION__);
      return -ENOMEM; 

   }
   memset(pLVOICE_Object, 0, sizeof (tLVOICE_Object));
 
   /* Register device */ 
   if( (result=register_chrdev(LVOICE_MAJOR,LVOICE_CHAR_NAME, &lvoice_fops)) < 0 ) 
   {
      printk(KERN_CRIT "%s can not register %s \n", __FUNCTION__, "slic"); 
      kfree(pLVOICE_Object);
      return result;
   }
   printk(KERN_INFO "Conexant Voice Driver is registered \n");

   /* waitQueue*/
   init_waitqueue_head (&pLVOICE_Object->waitQueue[LVOICE_EVENT_WAIT_QUEUE_ID ]);
   init_waitqueue_head (&pLVOICE_Object->waitQueue[LVOICE_AUDIO_WAIT_QUEUE_ID ]);
   init_waitqueue_head (&pLVOICE_Object->waitQueue[LVOICE_AUDIO_WAIT_FXS0_QUEUE_ID ]);
   init_waitqueue_head (&pLVOICE_Object->waitQueue[LVOICE_AUDIO_WAIT_FXS1_QUEUE_ID ]);

   for (i=0; i < LVOICE_MAX_PORTS; i++ )
   {
   /* Audio Queue */  
    cyan_qInit(&pLVOICE_Object->audioQueue[i]); 
    /* EventQueue */
    cyan_qInit(&pLVOICE_Object->eventQueue[i]); 
   }

   /* Create pool  100 buffers of 200 size for audio write*/
   if ( (pLVOICE_Object->pAudioPool = cyan_poolCreate( LVOICE_POOL_NAME, 100,
                                                             200,0,0) ) == NULL )  
   {
     printk(KERN_CRIT "%s could not allocate pool\n", __FUNCTION__);
     pLVOICE_Object->pAudioPool=NULL;
     /* Since it requires undoing whole things, i am calling cleanup directly */
     LVOICE_DriverExit();
     return -ENOMEM;
   }
   return 0;
}

/**
 * Driver Function for opening channel. It opens corresponsing bun channel
 * @param  pInode  Inode of the file (not used)
 * @param  pFile   Pointer to file struct for minor no.
 * @retun  success or error
  */

static int LVOICE_open (struct inode *pInode, struct file *pFile)
{
   int minor=MINOR(pInode->i_rdev);
   CYANBUFFER   *pCyanBuf;

   DEBUG_MACRO("Entering %s minor %x\n",__FUNCTION__,minor);
   
   if ( minor >=LVOICE_MINOR_MAX_DEVS )
   {
     printk(KERN_WARNING "%s minor %x out of range \n",__FUNCTION__, minor);
     return -ENOENT;

   }
   if ( module_use[minor] )
   { /* Ideally one device can be open by only one application, 
        but sometime it quite possible same application opening it again (
        application restart)so print a warning and allow */
     printk(KERN_NOTICE "%s minor %x already open \n",__FUNCTION__, minor);

   }
   module_use[minor]++;

   pFile->private_data=(void *)minor;
   return 0;
}
/**
 * Driver Function for releasing channel. It opens corresponsing bun channel
 * @param  pInode  Inode of the file (not used)
 * @param  pFile   Pointer to file struct for minor no.
 * @retun  success or error
  */

static int LVOICE_release (struct inode *pInode, struct file *pFile)
{
   int minor=MINOR(pInode->i_rdev);

   DEBUG_MACRO("Entering %s minor %x\n",__FUNCTION__,minor);

   if ( minor >=LVOICE_MAX_DEVS )
   {
     if ( minor >=LVOICE_MINOR_MAX_DEVS)
     {
        printk(KERN_WARNING "%s minor %x out of range \n",__FUNCTION__, minor);
        return -ENOENT;
     }
   }
   module_use[minor]--;
   return 0;
}

/**
 * Driver Function for  hanling ioctl command
 * @param  inode       
 * @param  pFile      For minor number
 * @param  cmd        POTS command 
 * @param  arg        Argumnet based on command
 * @retun  success or error
  */
static int LVOICE_ioctl ( struct inode *inode, struct file *pFile, unsigned int
                          cmd, unsigned long arg)

{
   U16 minor=MINOR(inode->i_rdev);
   S16  result=0; 
   U32               lInt;
   U16  index=0;
   tLVOICE_Object *pObject = pLVOICE_Object;
   CYANBUFFER *pBuffer;


   DEBUG_MACRO("Entering %s minor %x cmd %x\n ", __FUNCTION__, minor,
                                                   cmd);
   if (lvoice_debug)
   {
     for (index=0; index < LVOICE_MAX_CMD; index++ )
     {
        if (lvoiceCmdtoName[index].cmd == cmd )
        {
          printk("\n%s: index %x recvd command %s\n", __FUNCTION__,index, lvoiceCmdtoName[index].name);
          break;

        }
     }
   }
   switch (cmd)
   {
      case POTS_InitPots_Cmd:
                   {
                    tPOTS_InitPots lPOTS_InitPots;
                    copy_from_user( &lPOTS_InitPots, (void *)arg, sizeof lPOTS_InitPots);
                    if ( (result=POTS_InitPots(COMMON_VOIP, LVOICE_statCallBack, NULL, lPOTS_InitPots.region, 0, &lPOTS_InitPots.hPots)) !=ESUCCESS )
                    {
                        printk(KERN_ERR"POTS_InitPots_Cmd failed \n");
                        return -result;
                    }
                    copy_to_user( (void *)arg,  &lPOTS_InitPots,  sizeof lPOTS_InitPots);
                    /* Flush event and audio queue */
                   for (index=0;  index< LVOICE_MAX_PORTS; index++)
                   {
                       pObject->pAudioHandler[index]=0;
                       pObject->pEventHandler[index]=0;
                       pObject->upStreamSubChannel[index]=0;
                       pObject->hPhone[index]=0;
                       while(pObject->audioQueue[index].head )
                       {
                          if ((pBuffer=cyan_qGet(&pObject->audioQueue[index]) ) !=NULL )
                              cyan_bufRelease(pBuffer);
                       }
                       while(pObject->eventQueue[index].head )
                       {
                          if ((pBuffer=cyan_qGet(&pObject->eventQueue[index]) ) !=NULL )
                              cyan_bufRelease(pBuffer);
                       }
                   }
              
                   /* Restart pots module */
                   if ( (result=POTS_RestartPots(lPOTS_InitPots.hPots)) !=ESUCCESS )
                   { 
                          printk(KERN_ERR "POTS Restart failed \n");
                          return -result;
                   }
                   pObject->initialized=true;

                   break;
                  }

      case POTS_StartSignalAnPhone_Cmd:
                  { 
                    tPOTS_StartSignalAnPhone lPOTS_StartSignalAnPhone;
                    copy_from_user( &lPOTS_StartSignalAnPhone, (void *)arg, sizeof lPOTS_StartSignalAnPhone);
                   if ( (result =  POTS_StartSignalAnPhone(lPOTS_StartSignalAnPhone.hPhone,
                                                lPOTS_StartSignalAnPhone.signalId,
                                                lPOTS_StartSignalAnPhone.direction,
                                                lPOTS_StartSignalAnPhone.customId))
                                                !=ESUCCESS) 
                   {

                      printk(KERN_ERR "POTS_StartSignalAnPhone_Cmd Failed \n"); 
                      return -result;
                   }
                     
                  }
                   break;

      case POTS_StartRingAnPhone_Cmd:
                  {
                    tPOTS_StartRingAnPhone lPOTS_StartRingAnPhone;
                    copy_from_user( &lPOTS_StartRingAnPhone, (void *)arg, sizeof lPOTS_StartRingAnPhone);
                    if ( (result=POTS_StartRingAnPhone( lPOTS_StartRingAnPhone.hPhone,
                                               lPOTS_StartRingAnPhone.ringId,
                                               lPOTS_StartRingAnPhone.numRings))
                                               !=ESUCCESS ) 
                    {

                      printk(KERN_ERR "POTS_StartRingAnPhone failed \n");
                      return -result;
                    }
                  }
                   break;
      case POTS_StopSignalAnPhone_Cmd:
                  {
                    tPOTS_StopSignalAnPhone lPOTS_StopSignalAnPhone;
                    copy_from_user( &lPOTS_StopSignalAnPhone, (void *)arg, sizeof lPOTS_StopSignalAnPhone);
                    if ( (result=POTS_StopSignalAnPhone( lPOTS_StopSignalAnPhone.hPhone,
                                               lPOTS_StopSignalAnPhone.direction))!=ESUCCESS) 
                    {

                      printk(KERN_ERR "POTS_StopSignalAnPhone failed \n");
                      return -result;
                    }
                   }
                   break;

      case POTS_CallerIdAnPhone_Cmd:
                   {
                      tPOTS_CallerIdAnPhone lPOTS_CallerIdAnPhone;
                      copy_from_user( &lPOTS_CallerIdAnPhone, (void *)arg, sizeof lPOTS_CallerIdAnPhone);
                      if ( (result=POTS_CallerIdAnPhone( lPOTS_CallerIdAnPhone.hPhone,
                                                 lPOTS_CallerIdAnPhone.name,
                                                 lPOTS_CallerIdAnPhone.number,
                                                 lPOTS_CallerIdAnPhone.dateTime))
                                                 != ESUCCESS )
                     {
                        printk(KERN_ERR "POTS_CallerIdAnPhone failed \n"); 
                        return -result;
                     }
                  
                   }
                   break;

      case POTS_ConnectPhone_Cmd:
                   copy_from_user( &lInt, (void *)arg, sizeof lInt);
                   if ( (result=POTS_ConnectPhone (lInt)) !=ESUCCESS )
                   {
                       printk(KERN_ERR "POTS_ConnectPhone failed \n"); 
                       return -result;
                   }
              
                   break;

      case POTS_AddUpstreamAudio_Cmd:
                  { 
                      tPOTS_AddUpstreamAudio lPOTS_AddUpstreamAudio;
                      copy_from_user( &lPOTS_AddUpstreamAudio, (void *)arg, sizeof lPOTS_AddUpstreamAudio);
                      if ( (result=POTS_AddUpstreamAudio( lPOTS_AddUpstreamAudio.hPhone,
                                                  lPOTS_AddUpstreamAudio.hUpperHandle,
                                                  (FILE **)&lPOTS_AddUpstreamAudio.pAudioHandler,
                                                  (U32 *) &(lPOTS_AddUpstreamAudio.subChannel ))) !=ESUCCESS )
                      {
                         printk(KERN_ERR "POTS_AddUpstreamAudio failed %x\n", result);
                         return -result;
                      }
                     
                      
                      copy_to_user( (void *)arg, &lPOTS_AddUpstreamAudio,  sizeof lPOTS_AddUpstreamAudio);
                                                       
                     /* Store audio handler */
                     index = pObject->hPhone[0]==lPOTS_AddUpstreamAudio.hPhone ? 0:1;
                     pObject->pAudioHandler[index]=lPOTS_AddUpstreamAudio.hUpperHandle;
                     pObject->upStreamSubChannel[index]= 
                                                lPOTS_AddUpstreamAudio.subChannel;
                   }
                   break;

      case POTS_AddAudioPhone_Cmd:
                  {
                    tPOTS_AddAudioPhone lPOTS_AddAudioPhone;
                    copy_from_user( &lPOTS_AddAudioPhone, (void *)arg, sizeof lPOTS_AddAudioPhone);
                    if ( (result=POTS_AddAudioPhone(lPOTS_AddAudioPhone.hPhone,
                                            (FILE **)&lPOTS_AddAudioPhone.pAudioHandler,
                                            (U32 *) & lPOTS_AddAudioPhone.subChannel)) !=ESUCCESS )
                    {
                      printk(KERN_ERR "POTS_AddAudioPhone failed \n");
                      return -result;
                    }
                    copy_to_user( (void *)arg, &lPOTS_AddAudioPhone,  sizeof lPOTS_AddAudioPhone);
                   }
                   lvoice_read_audio_jiffies=jiffies; /* audio flow shall start */
                                             
                   break;

      case POTS_DisconnectPhone_Cmd: 
                   result=copy_from_user( &lInt, (void *)arg, sizeof lInt);
                   index = pObject->hPhone[0] == lInt ? 0: 1;
                   pObject->pAudioHandler[index]=0;
                   if ( (result=POTS_DisconnectPhone(lInt)) != ESUCCESS )
                   {
                      printk(KERN_ERR "POTS_DisconnectPhone failed \n");
                      return -result;
                   }
                   /* free up the queue */
                   while(pObject->audioQueue[index].head )
                   {
                        if ((pBuffer=cyan_qGet(&pObject->audioQueue[index]) )
!=NULL )
                              cyan_bufRelease(pBuffer);
                   }

                   break;

      case POTS_RemoveAudioPhone_Cmd: 
                   result=copy_from_user( &lInt, (void *)arg, sizeof lInt);
                   if ( (result=POTS_RemoveAudioPhone(lInt)) != ESUCCESS )
                   {
                      printk(KERN_ERR "POTS_RemoveAudioPhone_Cmd failed \n");
                      return -result;
                   }
                      /* free up the queue */
                   while(pObject->audioQueue[index].head )
                   {
                        if ((pBuffer=cyan_qGet(&pObject->audioQueue[index]) )
!=NULL )
                              cyan_bufRelease(pBuffer);
                   }

                   break;

      case POTS_ModifyPhone_Cmd:
                  {
                     tPOTS_ModifyPhone lPOTS_ModifyPhone;
                     copy_from_user( &lPOTS_ModifyPhone, (void *)arg, sizeof lPOTS_ModifyPhone);
                     if ( (result=POTS_ModifyPhone (lPOTS_ModifyPhone.hPhone,
                                            lPOTS_ModifyPhone.modifications))
                                            !=ESUCCESS )
                     {
                         printk(KERN_ERR "POTS_ModifyPhone_Cmd Failed \n");
                         return -result;
                     }
                    }    
                   break;

      case POTS_SetFaxCoderName_Cmd: 
                   {
                      tPOTS_SetFaxCoderName lPOTS_SetFaxCoderName;
                      copy_from_user( &lPOTS_SetFaxCoderName, (void *)arg, sizeof lPOTS_SetFaxCoderName);
                       if ( (result=POTS_SetFaxCoderName(lPOTS_SetFaxCoderName.hPhone,
                                                 lPOTS_SetFaxCoderName.coderName))
                                                 !=ESUCCESS )
                       {
                          printk(KERN_ERR "POTS_SetFaxCoderName failed \n");
                          return -result; 
                       }
                    }
                   break;
#ifdef VOICE_T38_FAX_SUPPORT
      case POTS_SetFaxOption_Cmd:
                  {
                     tPOTS_SetFaxOption lPOTS_SetFaxOption;
                     copy_from_user( &lPOTS_SetFaxOption, (void *)arg, sizeof lPOTS_SetFaxOption);
                      if ( (result=POTS_SetFaxOption(lPOTS_SetFaxOption.hPhone,
                                                lPOTS_SetFaxOption.faxOptions))
                                                !=ESUCCESS )
                      {
                         printk(KERN_ERR "POTS_SetFaxOption failed \n");
                         return -result;
                      }
                   }
                  break;
#endif
      case POTS_EnableAudioMixing_Cmd: 
                   result=copy_from_user( &lInt, (void *)arg, sizeof lInt);
                   if ( (result= POTS_EnableAudioMixing(lInt)) !=ESUCCESS )
                   {
                      printk(KERN_ERR "POTS_EnableAudioMixing failed \n");
                      return -result;
                   }
                   break;
     
      case POTS_InitAnPhone_Cmd:
                  {
                     tPOTS_InitAnPhone lPOTS_InitAnPhone;
                     copy_from_user( &lPOTS_InitAnPhone, (void *)arg, sizeof lPOTS_InitAnPhone);
                          
                     if ( (result=POTS_InitAnPhone(lPOTS_InitAnPhone.hPots,
                                           lPOTS_InitAnPhone.logicalId,
                                           LVOICE_audioCallBack,
                                           (void *)lPOTS_InitAnPhone.hUserEventId,
                                           (U32 *) &lPOTS_InitAnPhone.hPhone,
                                           &pfTxCall)) !=ESUCCESS)
                    {
                        printk(KERN_ERR "POTS_InitAnPhone_Cmd failed \n");
                        return -result;
                    }
                    copy_to_user( (void *)arg, &lPOTS_InitAnPhone,  sizeof lPOTS_InitAnPhone);
                    /* assumption is that POTS_InitAnPhone will be called in sequence */

                    index =  pObject->hPhone[0] == 0 ? 0: 1;
                    pObject->hPhone[index]=lPOTS_InitAnPhone.hPhone;
                     /* Store event handler */
                    pObject->pEventHandler[index]=(U32 *)lPOTS_InitAnPhone.hUserEventId;
                   }
                   break;
 
     case POTS_RemoveUpstreamAudio_Cmd:
                   copy_from_user( &lInt, (void *)arg, sizeof lInt);
                    if ( (result=POTS_RemoveUpstreamAudio(lInt)) !=ESUCCESS )
                    {
                      printk(KERN_ERR "POTS_RemoveUpstreamAudio fail \n");
                      return -result;
                    }
                    index = pObject->upStreamSubChannel[0] == lInt ? 0:1;
                    pObject->pAudioHandler[index]=0;
                    pObject->upStreamSubChannel[index]=0;
                   break;
     case POTS_FailSafePots_Cmd:
                   copy_from_user( &lInt, (void *)arg, sizeof lInt);
                    if ( (result=POTS_FailSafePots(lInt)) !=ESUCCESS )
                    {
                      printk(KERN_ERR "POTS_FailSafePots_Cmd fail \n");
                      return -result;
                    }
                    /* Flush event and audio queue */
                   for (index=0; index < LVOICE_MAX_PORTS; index++)
                   {
                       pObject->pAudioHandler[index]=0;
                       pObject->pEventHandler[index]=0;
                       pObject->upStreamSubChannel[index]=0;
                       while(pObject->audioQueue[index].head )
                       {
                          if ((pBuffer=cyan_qGet(&pObject->audioQueue[index]) ) !=NULL
 )
                              cyan_bufRelease(pBuffer);
                       }
                       while(pObject->eventQueue[index].head )
                       {
                          if ((pBuffer=cyan_qGet(&pObject->eventQueue[index]) ) !=NULL
 )
                              cyan_bufRelease(pBuffer);
                       }
                       while(pObject->eventQueue[index].head )
                       {
                          if ((pBuffer=cyan_qGet(&pObject->eventQueue[index]) ) !=NULL
 )
                              cyan_bufRelease(pBuffer);
                       }
                   }

                   pObject->initialized=false;

                   break;
     case POTS_AddDelDCM_Cmd:
                   {
                      tPOTS_AddDelDCM   lPOTS_AddDelDCM;
                      copy_from_user( &lPOTS_AddDelDCM, (void *)arg, sizeof lPOTS_AddDelDCM);
                      if ( (result=POTS_AddDelDCM( lPOTS_AddDelDCM.hPots,
                                           lPOTS_AddDelDCM.prefix,
                                           lPOTS_AddDelDCM.numDigits,
                                           lPOTS_AddDelDCM.dialAction,
                                           lPOTS_AddDelDCM.add) )!=ESUCCESS) 
                     {
                        printk(KERN_ERR "POTS_AddDelDCM called fail \n");
                        return -result;
                      }
                    }

                   break;
     case POTS_GetPstnStatus_Cmd:
                   {
                      tPOTS_GetPstnStatus lPOTS_GetPstnStatus;
                      copy_from_user( &lPOTS_GetPstnStatus, (void *)arg, sizeof lPOTS_GetPstnStatus);
                      POTS_GetPstnStatus(lPOTS_GetPstnStatus.hPhone,
                                             &(lPOTS_GetPstnStatus.status));
                      copy_to_user((void *)arg,  &lPOTS_GetPstnStatus, sizeof lPOTS_GetPstnStatus);
                    }
                   break;
     case POTS_StopRingAnPhone_Cmd:
                   result=copy_from_user( &lInt, (void *)arg, sizeof lInt);
                   index = pObject->hPhone[0] == lInt ? 0: 1;
                   if ( (result=POTS_StopRingAnPhone(lInt)) != ESUCCESS )
                   {
                      printk(KERN_ERR "POTS_StopRingAnPhone failed \n");
                      return -result;
                   }

                   break;

     default:
              printk(KERN_WARNING "%s unknown cmd %x\n",__FUNCTION__, cmd);
              result = -ENOENT;
              break;
   }
   return result;
  
}


/**
 * Driver Function for  reading audio stream from driver
 * @param  pFile      For minor number
 * @param  pBuffer    user buffer
 * @param  len        length of user buffer 
 * @param  pLoff      not used
 * @retun  success or error
 * In case of audio, first four bytes (int) of uBuffer is masked value to 
 * determine whether audio or event  on specific port requied( handler) or on any (0xffffffff)
 */


static ssize_t LVOICE_read(struct file *pFile, char *uBuffer, size_t len, loff_t *pLoff)
{

   CYANBUFFER *pBuffer=NULL;
   U16        result=0;
   static unsigned char portIndex=0; 
   tLVOICE_Object *pObject = pLVOICE_Object;
   static unsigned int count;
   static unsigned char index; /* to get event on next port */
   unsigned  int  handler,i; 
   CYANQUEUE   *pQueue=NULL;

   DEBUG_MACRO("Entering %s minor %x\n",__FUNCTION__,pFile->private_data);
   __copy_from_user( &handler, (void *)uBuffer, sizeof handler);
   if ( !pFile->private_data )
   { /* event */
      pQueue=pObject->eventQueue;
   }
   else
   { /* Audio */
      pQueue=pObject->audioQueue;
      lvoice_read_audio_jiffies=jiffies;
   } 
   if (pQueue) 
   {
     if ( handler == 0xffffffff )
     { /* event on any port */
        if ( pQueue[index].head )  
        {
          pBuffer = cyan_qGet(&pQueue[index]);
        }
        else if ( pQueue[(index+ 1) % LVOICE_MAX_PORTS ].head) 
        {
          pBuffer = cyan_qGet(&(pQueue[(index+1) % LVOICE_MAX_PORTS]));
        }
        else 
        {
          return result; 
        }
        index = ++index % LVOICE_MAX_PORTS;

      }
     else /*interested on specific port only */
     { 
       
       if (pFile->private_data)
       {
          i=(pLVOICE_Object->pAudioHandler[0]==(U32 *)handler?0:1);
       }
       else
       {
          i=(pLVOICE_Object->pEventHandler[0]==(U32 *)handler?0:1);
       }
       pBuffer = cyan_qGet(&pQueue[i]);
     }

     if (pBuffer)
     {
       if (lvoice_debug && (!(count++ %100)))
       {
           printk("%s count %x \n",__FUNCTION__,count);
       }
       result=LVOICE_copyCyanToUser(pFile->private_data?POTS_Audio_Cmd:POTS_Event_Cmd, uBuffer, pBuffer);
       pBuffer->status = 0;  /* later on function requires this to be zero*/
       cyan_bufRelease(pBuffer);
     }
     else
     {
       result=0;
     }
    }
   
  return result;
}

/**
 * Driver Function for  writing audio stream
 * @param  pFile      For minor number
 * @param  pBuffer    user buffer
 * @param  len        length of user buffer
 * @param  pLoff      not used
 * @retun  length of data written
 */

static ssize_t LVOICE_write(struct file *pFile, const char *uBuffer, size_t len, loff_t *pLoff)
{

   U32            minor =(U32)(pFile->private_data)  ;
   CYANBUFFER     *pBuffer;
   S16            result=0;
   tLVOICE_Object *pObject =pLVOICE_Object;
   static   unsigned int count=0;

   DEBUG_MACRO("Entering %s minor %x\n",__FUNCTION__,minor);
   if ((pBuffer = (CYANBUFFER *)cyan_poolGet(pObject->pAudioPool)) == NULL )
   {
      printk(KERN_WARNING "%s No pool buffer \n", __FUNCTION__);
      return 0;

   }
  result=LVOICE_copyAudioUserToCyan((char *)uBuffer,pBuffer);
  /* Call Audio transfer function. Status contain pFile parameter */
  (*pfTxCall)(pBuffer,(void *)(pBuffer->status)); 
  if (lvoice_debug && (!(count++ %100)))
  {
     printk("%s count %x \n",__FUNCTION__,count);
  }
  return result;

}
/**
 * Driver Function for select and poll. Based on minor no, it sleeps on waitque
 * @param  pFile      For minor number
 * @param  poll_table
 * @retun  success or error
 * For asterisk integration two more devices are provided 
 */
static  unsigned int LVOICE_poll ( struct file * pFile, poll_table *wait )
{
   int minor= (int)pFile->private_data , audioIndex=0;
   unsigned char i=0;
   DEBUG_MACRO("Entering %s minor %x\n",__FUNCTION__,minor);
   /* First check event queue and then audio queue 
      This function can be called by two process but on different minor device
      so locking is not required */

  if ( minor == LVOICE_MINOR_SIGNALLING_DEV  )
  { 
     for (i=0; i < LVOICE_MAX_PORTS; i++ )
     {
       if ( pLVOICE_Object->eventQueue[i].head )
       {
         DEBUG_MACRO("Entering %s Event detected minor %x\n",__FUNCTION__,
                                                           minor);
         return POLLIN;
       }
     }

    poll_wait(pFile,&pLVOICE_Object->waitQueue[LVOICE_EVENT_WAIT_QUEUE_ID] , wait);
    for (i=0; i < LVOICE_MAX_PORTS; i++ )
     {
       if ( pLVOICE_Object->eventQueue[i].head )
       {
         DEBUG_MACRO("Entering %s Event detected minor %x\n",__FUNCTION__,
                                                           minor);
         return POLLIN;
       }
     }
   
  }
  else if ( minor == LVOICE_MINOR_AUDIO_DEV )
  {
     /* Interested in audio read only */
     for (i=0; i < LVOICE_MAX_PORTS; i++ ) 
     {
       if ( pLVOICE_Object->audioQueue[i].head )
       {
         DEBUG_MACRO("Entering %s Event detected minor %x\n",__FUNCTION__,
                                                           minor);
         return POLLIN;
       } 
     }
     poll_wait(pFile,&pLVOICE_Object->waitQueue[LVOICE_AUDIO_WAIT_QUEUE_ID], wait);
     for (i=0; i < LVOICE_MAX_PORTS; i++ ) 
     {
       if ( pLVOICE_Object->audioQueue[i].head )
       {
         DEBUG_MACRO("Entering %s Event detected minor %x\n",__FUNCTION__,
                                                           minor);
         return POLLIN;
       } 
     }
  }
  else if ( (minor == LVOICE_MINOR_FXS0_DEV) || (minor == LVOICE_MINOR_FXS1_DEV ))
  { /* Only report audio through this */

       if(pLVOICE_Object->audioQueue[minor-LVOICE_MINOR_FXS0_DEV].head)
       {
           DEBUG_MACRO("Entering %s Event detected minor %x\n",__FUNCTION__, minor);
           return POLLIN;
       }
       poll_wait(pFile,&pLVOICE_Object->waitQueue[minor], wait);
       if(pLVOICE_Object->audioQueue[minor-LVOICE_MINOR_FXS0_DEV].head)
       {
           DEBUG_MACRO("Entering %s Event detected minor %x\n",__FUNCTION__, minor);
           return POLLIN;
       }
  }
  return 0;
}

/**
 * Driver Function for removing. It free all the resources
 * Note: Created cyan pool can not be deleted
 */

void  LVOICE_DriverExit (void )
{
  int result;
  DEBUG_MACRO("Entering %s\n",__FUNCTION__);
   
  if ( (result=unregister_chrdev(LVOICE_MAJOR, LVOICE_CHAR_NAME)) < 0 )
  {
     printk(KERN_WARNING "%s Error unregistering device major no %x",__FUNCTION__,
                                                     LVOICE_MAJOR);
     return;
  }
  if (pLVOICE_Object)
  {
     kfree(pLVOICE_Object);
  }
  printk(KERN_INFO "Conexant Voice Driver is unregistered \n");
}
